/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans.
 *
 * CosyBeans is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.application;

import java.util.EventObject;

/**
 * <code>PlugInEvent</code> is triggered by the PlugInManager.
 * 
 * @author Igor Kriznar (igor.kriznar@cosylab.com)
 * @version @@VERSION@@
 */
/**
 * Event used in event notifications through <code>PlugInListener</code>
 * interface.
 * 
 * @version @@VERSION@@
 */
public class PlugInEvent extends EventObject {
  
	private static final long serialVersionUID = 1L;
	private PlugIn plugin;
	
	/**
	 * Creates a new instance of this event. Delegates to 
	 * parent constructor.
	 * 
	 * @param source the plugin manager that is firing this event
   * @param plugin the plugin which was installed
	 */
	public PlugInEvent(PlugInManager source, PlugIn plugin)
	{
		super(source);
		this.plugin=plugin;
	}
  
  
   /**
    * Returns plugin which generated this event.
    * @return plugin which generated this event
    */
	public PlugIn getPlugIn() {
		return plugin;
	}
}

