/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans.
 *
 * CosyBeans is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.application;

import com.cosylab.util.CommonException;

/**
 * An exception describing the problem with plugin installation.
 * These checked exceptions are thrown when the construction or
 * initialization of the plugin fail; when there are cyclic references
 * in the plugin installation; or when the plugin does not conform
 * to the plugin design contract.
 */
public class PlugInException extends CommonException {
	
	private static final long serialVersionUID = 1L;
	private Class<? extends PlugIn> plugType = null;
	
	/**
	 * Creates a new instance of this exception.
	 * 
	 * @param source the object throwing the exception
	 * @param message message to be printed by this exception
	 * @param plugType class object representing the plug the installation of
	 *        which failed (can be <code>null</code>
	 * @param t exception that caused this exception to be thrown
	 *        (can be <code>null</code>)
	 */
	public PlugInException(Object source, String message, Class<? extends PlugIn> plugType, Throwable t)
	{
		super(source, message, t);
		this.plugType = plugType;
	}

	/**
	 * Returns a string rendering of this exception.
	 * 
	 * @return String overloaded <code>toString</code> return value.
	 */	
	@Override
    public String toString()
	{
		String plugTypeName = "N/A";
		if (plugType != null) plugTypeName = plugType.getName();
		String tMessage = "";
		return getMessage() + "\nPlug type: " + plugTypeName + "\n" + "Caused by:\n" + tMessage;
	}
}

