/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of Java-Common.
 *
 * Java-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Java-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Java-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.application.state;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import java.util.Iterator;
import java.util.List;


/**
 * This interface should be implemented by the objects which will be used for
 * storing and restoring an array of the <code>State</code> objects.
 *
 * @author dvitas
 */
public interface StateStorage
{
	/**
	 * Adds new state to storage.
	 *
	 * @param state the state to be added
	 */
	public void add(State state);

	/**
	 * DOCUMENT ME!
	 *
	 * @param states DOCUMENT ME!
	 */
	public void addAll(List<State> states);

	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public List<State> getStates();

	/**
	 * DOCUMENT ME!  TODO: Add load(InputStream, String) to support loading
	 * from non-file sources!
	 *
	 * @param filePath DOCUMENT ME!
	 * @param applicationName DOCUMENT ME!
	 *
	 * @throws IOException DOCUMENT ME!
	 */
	public void load(String filePath, String applicationName)
		throws IOException;

	/**
	 * DOCUMENT ME!
	 *
	 * @param stream DOCUMENT ME!
	 *
	 * @throws IOException DOCUMENT ME!
	 */
	public void load(InputStream stream) throws IOException;

	/**
	 * DOCUMENT ME!
	 *
	 * @param filePath DOCUMENT ME!
	 * @param applicationName DOCUMENT ME!
	 *
	 * @throws IOException DOCUMENT ME!
	 */
	public void store(String filePath, String applicationName)
		throws IOException;

	/**
	 * DOCUMENT ME!
	 *
	 * @param stream
	 *
	 * @throws IOException DOCUMENT ME!
	 */
	public void store(OutputStream stream) throws IOException;

	/**
	 * DOCUMENT ME!
	 *
	 * @param st DOCUMENT ME!
	 */
	public void remove(State st);

	/**
	 * DOCUMENT ME!
	 *
	 * @param st DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public boolean contains(State st);

	/**
	 * Rerurns iterator over states.
	 *
	 * @return iterator over states
	 */
	public Iterator<State> iterator();
}

/* __oOo__ */
