/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of Java-Common.
 *
 * Java-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Java-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Java-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.application.state.impl;

import com.cosylab.application.state.*;

import java.io.EOFException;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;

/**
 * This class implements the state storage and saves states into a binary file
 * using simple JAVA serialization.
 * 
 * @author dvitas
 */
public class SerialStateStorage extends DefaultStateStorage {
	/**
	 *  
	 */
	public SerialStateStorage() {
		super();
	}

	/**
	 * Creates a new SerialStateStorage object.
	 * 
	 * @param ss
	 *            DOCUMENT ME!
	 */
	public SerialStateStorage(StateStorage ss) {
		super(ss);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.cosylab.application.state.StateStorage#load(java.io.InputStream)
	 */
	@Override
    public void load(String filePath, String applicationName)
		throws IOException {
		if (delegate != null) {
			delegate.load(filePath, applicationName);
			addAll(delegate.getStates());

			return;
		}

		// get the InputStream if file exists
		InputStream is = getInputStream(filePath, applicationName + ".bin");
		load(is);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.cosylab.application.state.StateStorage#load(java.io.InputStream)
	 */
	public void load(InputStream is) throws IOException {
		try {
			if (is == null) {
				return;
			}

			ObjectInputStream ois = new ObjectInputStream(is);

			while (true) {
				states.add((State)ois.readObject());
			}
		} catch (ClassNotFoundException e) {
			throw new IOException(e.getMessage());
		} catch (EOFException e) {
			// end of stream
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.cosylab.application.state.StateStorage#store(java.io.OutputStream,
	 *      com.cosylab.application.state.State[])
	 */
	@Override
    public void store(String filePath, String applicationName)
		throws IOException {
		OutputStream os = getOutputStream(filePath, applicationName + ".bin");
		store(os);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.cosylab.application.state.StateStorage#store(java.io.OutputStream)
	 */
	public void store(OutputStream os) throws IOException {
		if (os == null) {
			return;
		}

		ObjectOutputStream oos = new ObjectOutputStream(os);

		for (int i = 0; i < states.size(); i++) {
			oos.writeObject(states.get(i));
		}

		os.flush();
		os.close();
	}
}

/* __oOo__ */
