/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of Java-Common.
 *
 * Java-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Java-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Java-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.events;

import java.util.EventObject;

/**
 * 
 * 
 * @author gpajor
 * @version @@VERSION@@
 */
public class RangeSelectionEvent extends EventObject {
	/**
     * 
     */
    private static final long serialVersionUID = 1L;
    private int selectionStart=-1;
	private int selectionEnd=-1;
	private boolean selected=true;

	/**
	 * Constructor for RangeSelectionEvent.
	 * @param source
	 * @param start Start index, should be -1 iff nothing is selected
	 * @param end End index, should be >= start, -1 if nothing is selected.
	 * @param selecrted, <code>true</code> if interval was selected, <code>false</code> if deselected.
	 */
	public RangeSelectionEvent(RangeSelectable source, int start, int end, boolean selected) {
		super(source);
		selectionStart=start;
		selectionEnd=end;
		this.selected=selected;
	}

	/**
	 * Returns the selectionEnd.
	 * @return int
	 */
	public int getSelectionEnd() {
		return selectionEnd;
	}

	/**
	 * Returns the selectionStart.
	 * @return int
	 */
	public int getSelectionStart() {
		return selectionStart;
	}


	/**
	 * Returns <code>true</code> if specified interval was selected. 
	 * If it was deselected, it is <code>false</code>.
	 * @return boolean <code>true</code> if selected
	 */
	public boolean isSelected() {
		return selected;
	}

}
