/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans.
 *
 * CosyBeans is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.util.Map;

import javax.swing.AbstractAction;
import javax.swing.Icon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.plaf.metal.MetalLabelUI;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.TableColumn;

import com.cosylab.gui.components.customizer.AbstractCustomizerPanel;
import com.cosylab.gui.components.introspection.MapTableModel;
import com.cosylab.gui.components.introspection.PropertiesTableModel;
import com.cosylab.gui.components.util.CosyUIElements;
import com.cosylab.gui.components.util.PopupManager;
import com.cosylab.gui.displayers.AbstractDisplayerPanel;
import com.cosylab.gui.displayers.DataConsumer;
import com.cosylab.gui.displayers.DataState;
import com.cosylab.gui.displayers.ObjectConsumer;
import com.cosylab.gui.displayers.ObjectConsumerMulticaster;
import com.cosylab.gui.displayers.ObjectDisplayer;
import com.cosylab.util.ArrayHelper;
import com.cosylab.util.CommonException;


/**
 * <code>CharacteristicsMapPanel</code> ...  DOCUMENT ME!
 *
 * @author <a href="mailto:anze.zupanc@cosylab.com">Anze Zupanc</a>
 * @version $Id: CharacteristicsMapPanel.java,v 1.15 2008-04-22 12:31:02 jbobnar Exp $
 *
 * @since Mar 29, 2004.
 */
public  class CharacteristicsMapPanel extends AbstractDisplayerPanel
	implements ObjectDisplayer 
{
	private static final long serialVersionUID = 1L;
	protected PopupManager popupManager;
	private boolean initialized = false;
	public  static MapTableModel tableModel = null;
	private JLabel title = null;
	public static JTable table; 
	private AbstractCustomizerPanel customizer;
	private int suspendCount = 0;
	private Object lastValue=null;
	/**
	 * Creates a new CharacteristicsMapPanel object.
	 * 
	 */
	public CharacteristicsMapPanel()
	{
		super();
		initialize();
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getDataConsumer(java.lang.Class)
	 */
	public DataConsumer getDataConsumer(Class type)
	{
		if (type == ObjectConsumer.class) {
			return this;
		}

		return ObjectConsumerMulticaster.createDataConsumer(type, this);
	}

	@Override
    public void resume()
	{
		if (suspendCount > 0) {
			suspendCount--;
		}
		
		if (suspendCount==0) {
			setEnabled(true);
		    setValue(lastValue);
		}
	}

	@Override
    public void setEnabled(boolean enabled){
	    table.setEnabled(enabled);
	    title.setEnabled(enabled);   
	}
	
	
	@Override
    public void suspend()
	{
		suspendCount++;
		setEnabled(false);
	}


	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getDefaultDataConsumer()
	 */
	@Override
    public DataConsumer getDefaultDataConsumer()
	{
		return this;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.core.CosyComponent#getName()
	 */
	@Override
    public String getName()
	{
		return "CharacteristicsMapPanel";
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getSupportedCharacteristics()
	 */
	@Override
    public String[] getSupportedCharacteristics()
	{
		return null;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getSupportedConsumerTypes()
	 */
	public Class[] getSupportedConsumerTypes()
	{
		return ObjectConsumerMulticaster.SUPPORTED_CONSUMER_TYPES;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.Displayer#getTitle()
	 */
	@Override
    public String getTitle()
	{
		return title.getText();
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.ObjectDisplayer#getValue()
	 */
	public Object getValue()
	{
		return null;
	}
	
	/**
	 * costomizer for settings
	 * @return costomizer
	 */
	@Override
    public AbstractCustomizerPanel getCustomizer()
		{
			if (customizer == null) {
				customizer = AbstractCustomizerPanel.findCustomizer(this);
			}

			return customizer;
		}

	
	private void initialize()
	{
	  if (initialized) {
			return;
		}

		
		initialized = true;

		setLayout(new GridBagLayout());
		setBorder(CosyUIElements.getPlainBorder(true));

		GridBagConstraints c = new GridBagConstraints();
				
		getPopupManager().addAction(new AbstractAction("Preferences...") {
						/**
             * 
             */
            private static final long serialVersionUID = 1L;

                        public void actionPerformed(ActionEvent e)
						{
							getCustomizer().showDialog();
						}
			
					});
		this.addMouseListener(getPopupManager().getMouseHook());
		
		//end adding tomo

		title = new JLabel();
		title.setHorizontalAlignment(SwingConstants.CENTER);
		title.setHorizontalTextPosition(SwingConstants.CENTER);
		title.setFont(title.getFont().deriveFont(Font.BOLD));

		// Trim text on the left side
		title.setUI(new MetalLabelUI() {
				@Override
                protected String layoutCL(JLabel label,
				    FontMetrics fontMetrics, String text, Icon icon,
				    Rectangle viewR, Rectangle iconR, Rectangle textR)
				{
					char[] chars = text.toCharArray();
					ArrayHelper.flip(chars);

					String prevText = String.copyValueOf(chars);
					String modText = super.layoutCL(label, fontMetrics,
						    prevText, icon, viewR, iconR, textR);
					chars = modText.toCharArray();
					ArrayHelper.flip(chars);

					return String.copyValueOf(chars);
				}
			});

		c.anchor = GridBagConstraints.NORTH;
		c.weightx = 1.0;
		c.weighty = 0.0;
		c.fill = GridBagConstraints.BOTH;
		c.insets = new Insets(4, 4, 2, 4);
		add(title, c);

		
		tableModel = new MapTableModel(new String[]{ "property", "value"});
		tableModel.setValueEditable(false);
		tableModel.setKeyEditable(false);

		final JScrollPane scroll = new JScrollPane();
		table = new JTable(tableModel);

		
		table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		table.setSize(new Dimension(200,200));
				
		scroll.setMinimumSize(new Dimension(100,50));		
		scroll.setViewportView(table);

		table.addMouseListener(getPopupManager().getMouseHook());
		scroll.addMouseListener(getPopupManager().getMouseHook());

		c.gridy = 1;
		c.weighty = 1.0;
		c.insets = new Insets(0, 4, 4, 4);
		add(scroll, c);
		revalidate();
		
		/**
		 * we set all row width to 50% larger 
		 */
		setRowWidthMultiplayer(1.5);
		
	
	}
	
	private void setRowWidthMultiplayer(double i){
	
		DefaultTableColumnModel colModel = (DefaultTableColumnModel)table.getColumnModel();
        for(int j=0;j<colModel.getColumnCount();j++){
        	TableColumn col = colModel.getColumn(j);
        	
        	col.setPreferredWidth((int)(col.getWidth()*i));
        }
        	
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.Displayer#isEditable()
	 */
	public boolean isEditable()
	{
		return false;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#setCharacteristics(java.util.Map)
	 */
	public void setCharacteristics(Map characteristics)
	{
		tableModel.setMap(characteristics);
		Object o= characteristics.get(C_DISPLAY_NAME);
		if (o==null) {
			o="N/A";
		}
		String t= o.toString();
		setTitle(t);
		revalidate();
		repaint();
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.Displayer#setTitle(java.lang.String)
	 */
	@Override
    public void setTitle(String label)
	{
		//String old= title.getText();
		title.setText(label);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.ObjectDisplayer#setValue(java.lang.Object)
	 */
	public void setValue(Object value)
	{
		tableModel.put("VALUE",value);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.ObjectConsumer#updateValue(long, java.lang.Object)
	 */
	public void updateValue(long timestamp, Object value)
		throws CommonException
	{
	    lastValue=value;
	    if(!isSuspended()){
	        setValue(value);
	    }
	}
	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#updateDataState(com.cosylab.gui.displayers.DataState)
	 */
	@Override
    public void updateDataState(DataState state) {
		super.updateDataState(state);
		tableModel.put("DATA_STATE",state);
	}
	
	/**
	 * added by tomo
	 * @param args
	 */
	public static void main(String[] arg0) {
		try {
			JFrame bla = new JFrame("CharacteristicsMapPanel");
			CharacteristicsMapPanel cmp = new CharacteristicsMapPanel();
			bla.getContentPane().setLayout(new BorderLayout());
			bla.getContentPane().add(cmp);
			bla.setSize(350,150);
			cmp.setTitle("aha");
			
			bla.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE );
			
			bla.setVisible(true);

			/**
			 * we create two rows in table,... just to see if suspend&resume works...
			 */
			
			String[] names = new String[2];
			Class[] types = new Class[2];
			Object[] values = new Object[2];

			names[0] = C_DISPLAY_NAME;
			types[0] = String.class;
			values[0] = "SOME/DUMMY/DATA/SOURCE";

			names[1] = C_DESCRIPTION;
			types[1] = String.class;
			values[1] = "This is simple DataSource implementation.";

			

			PropertiesTableModel model = null;
			model = new PropertiesTableModel(names, types, values);
			
			table.setModel(model);
		//TODO
			for(int i = 0 ; i<15; i++){

				values[0] = new Integer(i);
				values[1] = "This "+i;

					cmp.updateValue(1,values);
				Thread.sleep(1000);
				if(i==5){
				    cmp.suspend();
				}
				if (i==10){
				    cmp.resume();
				}
			}			
		} catch (Exception e) {
			e.printStackTrace();
		}
		
	}

	// samo za test
	private boolean resizable, titleVisible;
	private int titleMinimumFontSize, titleMaximumFontSize;
	public boolean isResizable() {
		return resizable;
	}

	public void setResizable(boolean resizable) {
		this.resizable = resizable;
	}

	@Override
    public int getTitleMaximumFontSize() {
		return titleMaximumFontSize;
	}

	@Override
    public void setTitleMaximumFontSize(int titleMaximumFontSize) {
		this.titleMaximumFontSize = titleMaximumFontSize;
	}

	@Override
    public int getTitleMinimumFontSize() {
		return titleMinimumFontSize;
	}

	@Override
    public void setTitleMinimumFontSize(int titleMinimumFontSize) {
		this.titleMinimumFontSize = titleMinimumFontSize;
	}

	@Override
    public boolean isTitleVisible() {
		return titleVisible;
	}

	@Override
    public void setTitleVisible(boolean titleVisible) {
		this.titleVisible = titleVisible;
	}
	// do tu je samo za test

}

/* __oOo__ */
