/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans.
 *
 * CosyBeans is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.beans.PropertyVetoException;
import java.util.Map;

import javax.swing.AbstractAction;
import javax.swing.JFrame;

import com.cosylab.events.SetEvent;
import com.cosylab.events.SetListener;
import com.cosylab.gui.adapters.Converter;
import com.cosylab.gui.adapters.FeedbackConsumerSupport;
import com.cosylab.gui.components.DialKnob;
import com.cosylab.gui.components.DialKnobRotationType;
import com.cosylab.gui.components.customizer.AbstractCustomizerPanel;
import com.cosylab.gui.components.range2.TrimValuePolicy;
import com.cosylab.gui.components.util.ColorHelper;
import com.cosylab.gui.components.util.CosyUIElements;
import com.cosylab.gui.components.util.PopupManager;
import com.cosylab.gui.displayers.ConvertibleDisplayer;
import com.cosylab.gui.displayers.DataConsumer;
import com.cosylab.gui.displayers.DataSource;
import com.cosylab.gui.displayers.DataSourceSupport;
import com.cosylab.gui.displayers.DataState;
import com.cosylab.gui.displayers.DisplayerUtilities;
import com.cosylab.gui.displayers.DoubleConsumer;
import com.cosylab.gui.displayers.DoubleConsumerMulticaster;
import com.cosylab.gui.displayers.DoubleDisplayer;
import com.cosylab.gui.displayers.NonblockingNumberConsumer;
import com.cosylab.util.CommonException;


/**
 * DialKnob implemented as <code>Displayer</code>.
 *
 * @author <a href="mailto:jernej.kamenik@cosylab.com">Jernej Kamenik</a>
 * @version $id$
 */
public class DialKnobDisplayer extends DialKnob implements DoubleDisplayer,
	DataSource, ConvertibleDisplayer
{
	private static final long serialVersionUID = 1L;

	@SuppressWarnings("unchecked")
	private DataSourceSupport support = new DataSourceSupport(new Class[]{
			    NonblockingNumberConsumer.class
		    });

	//private InitializationValuePolicy ivp = new InitializationValuePolicy();
	private DataState dataState = new DataState(DataState.UNDEFINED);
	private int suspendCount = 0;
	private DataSource dataSource;
	private boolean flag;
	private Converter converter;
	private AbstractCustomizerPanel customizer;
	private InfoDialog infoDialog;
	private PopupManager popupManager;

	//protected DisplayerSupport delegate = new DisplayerSupport();
	//private SetterSupport setter = null;
	//private int requests = 0;
	//private SetMode setMode = SetMode.SYNCHRONOUS;

	/**
						 *
						 */
	public DialKnobDisplayer()
	{
		super();
		initialize();
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#setCharacteristics(java.util.Map)
	 */
	public void setCharacteristics(Map characteristics)
	{
		if (characteristics == null) {
			throw new NullPointerException("characteristics");
		}

		DisplayerUtilities.setCharacteristics(characteristics, this);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataSource#getConsumers()
	 */
	public DataConsumer[] getConsumers()
	{
		return support.getConsumers();
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getDataConsumer(java.lang.Class)
	 */
	public DataConsumer getDataConsumer(Class type)
	{
		if (type == DoubleConsumer.class) {
			return this;
		}

		return DoubleConsumerMulticaster.createDataConsumer(type, this);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.CommonDisplayer#getDataState()
	 */
	public DataState getDataState()
	{
		return dataState;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getDefaultDataConsumer()
	 */
	public DataConsumer getDefaultDataConsumer()
	{
		return this;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getSupportedCharacteristics()
	 */
	public String[] getSupportedCharacteristics()
	{
		return DisplayerUtilities.COMMON_NUMERIC_DISPLAYER_CHARACTERISTICS;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataSource#getSupportedConsumersTypes()
	 */
	public Class[] getAcceptableConsumerTypes()
	{
		return support.getAcceptableConsumerTypes();
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.CommonDisplayer#isSuspended()
	 */
	public boolean isSuspended()
	{
		return isEnabled();
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataSource#addConsumer(com.cosylab.gui.displayers.DataConsumer)
	 */
	public void addConsumer(DataConsumer consumer) throws PropertyVetoException
	{
		if (consumer == null) {
			throw new NullPointerException("consumer");
		}

		NonblockingNumberConsumer c = consumer
			.getDataConsumer(NonblockingNumberConsumer.class);

		if (c == null) {
			throw new PropertyVetoException("Consumer '" + consumer
			    + "' must support NonblockingNumberConsumer.", null);
		}

		support.addConsumer(c);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.CommonDisplayer#cleanup()
	 */
	public void cleanup()
	{
		internalCleanup();
		updateDataState(new DataState(DataState.NOT_INITIALIZED));
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.core.CosyComponent#destroy()
	 */
	public void destroy()
	{
		cleanup();
		support.removeAllConsumers();
		support = null;
		dataState = null;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataSource#removeConsumer(com.cosylab.gui.displayers.DataConsumer)
	 */
	public void removeConsumer(DataConsumer consumer)
	{
		support.removeConsumer(consumer);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.CommonDisplayer#resume()
	 */
	public void resume()
	{
		if (suspendCount > 0) {
			suspendCount--;
		}

		if (suspendCount == 0) {
			setEnabled(true);
		}
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.CommonDisplayer#suspend()
	 */
	public void suspend()
	{
		suspendCount++;
		setEnabled(false);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#updateDataState(com.cosylab.gui.displayers.DataState)
	 */
	public void updateDataState(DataState state)
	{
		DataState old = dataState;
		dataState = state;
		firePropertyChange(DATA_STATE, old, dataState);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DoubleConsumer#updateValue(long, double)
	 */
	public void updateValue(long timestamp, double value)
		throws CommonException
	{
		setValue(value);
		/*
		 * Bug fix for RT 5053
		 */
	
		if (flag) {
			synchronize();
			flag = false;
		} 
	}

	private void initialize()
	{
		setBorder(new CosyUIElements.PanelFlushBorder());
		setBackground(ColorHelper.getCosyControl());
		addSetListener(new SetListener() {
				public void setPerformed(SetEvent e)
				{
					try {
						Number value = e.getValue();

						DataConsumer[] d = getConsumers();

						for (int i = 0; i < d.length; i++) {
							NonblockingNumberConsumer dd = (NonblockingNumberConsumer)d[i];

							if (dd != null) {
								dd.updateNonblocking(value);
							}
						}
					} catch (Exception ex) {
						ex.printStackTrace();
					}
				}
			});
		setPopupEnabled(true);
		internalCleanup();
	}

	private void internalCleanup()
	{
		setTitle("Dialknob");
		setTitleVisible(true);
		setUnits(null);
		setUnitsVisible(true);
		getRangedValue().setValue(0, 100, 0);
		getUserRangedValue().setValue(0, 100, 0);
		setValuePolicy(new TrimValuePolicy());
		setEditable(true);
		setFormat("%3.2f");
		setEnhanced(true);
		setResizable(true);
		setTiltingEnabled(true);
		flag = true;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getSupportedConsumerTypes()
	 */
	public Class[] getSupportedConsumerTypes()
	{
		return DoubleConsumerMulticaster.PREFERED_CONSUMER_TYPES;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.Displayer#setDataSource(com.cosylab.gui.displayers.DataSource)
	 */
	public void setDataSource(DataSource dataSource)
		throws PropertyVetoException
	{
		DisplayerUtilities.prepareNewDataSource(dataSource,this);
		
		DataSource old= this.dataSource;
		this.dataSource = dataSource;
		
		firePropertyChange(DATA_SOURCE,old,dataSource);
	}

	
	/**
	 * Returns the converter.
	 * @return Returns the converter.
	 */
	public Converter getConverter() {
		return converter;
	}

	
	/**
	 * Sets new converter.
	 * @param converter The converter to set.
	 * @throws PropertyVetoException if set fails
	 */
	public void setConverter(Converter converter) throws PropertyVetoException {
		if (this.converter != null && this.converter.equals(converter) ||
				(this.converter == null && converter == null)) return;
		DisplayerUtilities.prepareNewConverter(converter,this);
		
		Converter old= this.converter;
		this.converter = converter;
		
		firePropertyChange(CONVERTER_PROPERTY,old,this.converter);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataSource#removeAllConsumers()
	 */
	public void removeAllConsumers()
	{
		support.removeAllConsumers();
	}
	
	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.Displayer#getDataSource()
	 */
	public DataSource getDataSource() {
		return dataSource;
	}
	

	/**
	 * Loads the default
	 * <code>com.cosylab.gui.components.customizer.Customizer</code> for this
	 * displayer and returns it.
	 *
	 * @return the Customizer intance for this displayer
	 */
	@Override
    public AbstractCustomizerPanel getCustomizer()
	{
		if (customizer == null) {
			customizer = AbstractCustomizerPanel.findCustomizer(this);
		}

		return customizer;
	}
	
	/**
	 * Returns the info dialog that belongs to this displayer.
	 * 
	 * @return
	 */
	public InfoDialog getInfoDialog() {
		if (infoDialog == null) {
			infoDialog = new InfoDialog(this);
		}
		return infoDialog;
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.cosylab.gui.components.DialKnob#getPopupManager()
	 */
	@Override
    public PopupManager getPopupManager()
	{
		if (popupManager == null) {
			popupManager = super.getPopupManager();
			popupManager.addAction(new AbstractAction("Info...") {
	
				private static final long serialVersionUID = 1L;

					public void actionPerformed(ActionEvent e)
					{
						getInfoDialog().setVisible(true);
					}
				});
		}
		
		return popupManager;
	}
	

	/**
	 * @param arg0
	 * @throws  
	 */
	public static void main(String[] arg0) {
		JFrame f = new JFrame();
		f.getContentPane().setLayout(new BorderLayout());
		DialKnobDisplayer disp = new DialKnobDisplayer();
		try {
			disp.setDataSource(new DataSourceSupport(new Class[]{DoubleConsumer.class}){
				/**
                 * 
                 */
                private static final long serialVersionUID = 1L;

                @Override
				public void addConsumer(DataConsumer consumer) throws PropertyVetoException {
					super.addConsumer(consumer);
					
					((DataSource)consumer).addConsumer(getConsumer());
				}
				
				private NonblockingNumberConsumer getConsumer() {
					return new FeedbackConsumerSupport(){
						@Override
						protected void updateRequest(long timestamp, Number value) {
							DataConsumer[] cc = getConsumers();
							try {
							for (DataConsumer c : cc) {
								((DoubleConsumer)c).updateValue(timestamp,value.doubleValue());
							}
							} catch (Exception e) {
								
							}
						}
					};
				}
			});
		}
		catch (PropertyVetoException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 
		disp.setRotationType(DialKnobRotationType.ENDLESS);
		disp.setPopupEnabled(true);
		f.getContentPane().add(disp);
		f.setSize(300,300);
		f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		f.setVisible(true);
		
	}
}

/* __oOo__ */
