/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans.
 *
 * CosyBeans is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.beans.Customizer;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;

import javax.swing.DefaultListCellRenderer;
import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import com.cosylab.gui.components.NumberField;
import com.cosylab.gui.components.util.RunnerHelper;
import com.cosylab.gui.util.GUIUtilities;

/**
 * <code>IconCustomizer</code> is a customizer for IconDisplayer. It enables customizing 
 * icons for different values. User is allowed to choose among the icons in the
 * classpath.
 * 
 * @author Jaka Bobnar, Cosylab
 *
 */
public class IconCustomizer extends JPanel implements Customizer {
	
	private static final long serialVersionUID = -5846378735807305307L;
 
	private class ValueHolder {
		private ValueIconPair pair;
		private String path;
		public ValueHolder(ValueIconPair pair, String path) {
			this.pair = pair;
			this.path = path;
		}
		@Override
		public boolean equals(Object o) {
			if (!(o instanceof ValueHolder)) 
				return false;
			else return (((ValueHolder)o).pair.equals(pair));
		}
		
		@Override
		public int hashCode() {
		    return pair.hashCode();
		}
	}
	
	private class CellRenderer extends DefaultListCellRenderer {
		
		private static final long serialVersionUID = 1L;
		
		@Override
        public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
			Component c = super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
			if (! (c instanceof JLabel)) return c;
			
			if (value instanceof ValueHolder) {
				JLabel component = (JLabel) c;
				ValueHolder vh = (ValueHolder) value;
				component.setToolTipText(vh.path);
				if (vh.pair.getIcon() instanceof ImageIcon)
					if (vh.pair.getIcon().getIconHeight() > 16 || vh.pair.getIcon().getIconWidth() > 16)
					component.setIcon(getScaledIcon((ImageIcon)vh.pair.getIcon(), 16, 16));
				else 
					component.setIcon(vh.pair.getIcon());
				component.setText(" Value: " + vh.pair.getValue() + "; Icon: " + vh.path);
			}
			return c;
		}
		
	}
	
//	private class LabelAccessory extends JLabel implements PropertyChangeListener {
//
//		private static final long serialVersionUID = -8578024556019972274L;
//
//		private static final int PREFERRED_WIDTH = 125;
//
//		private static final int PREFERRED_HEIGHT = 125;
//
//		public LabelAccessory(JFileChooser chooser) {
//			setVerticalAlignment(JLabel.CENTER);
//			setHorizontalAlignment(JLabel.CENTER);
//			chooser.addPropertyChangeListener(this);
//			setPreferredSize(new Dimension(PREFERRED_WIDTH, PREFERRED_HEIGHT));
//		}
//
//		public void propertyChange(PropertyChangeEvent changeEvent) {
//			String changeName = changeEvent.getPropertyName();
//			if (changeName.equals(JFileChooser.SELECTED_FILE_CHANGED_PROPERTY)) {
//				File file = (File) changeEvent.getNewValue();
//				if (file != null) {
//					setIcon(getScaledIcon(new ImageIcon(file.getPath()), PREFERRED_HEIGHT, PREFERRED_WIDTH));
//				}
//			}
//		}
//	}
//	
//	private class ThumbnailFileView extends FileView {
//		
//		private final Icon fileIcon = MetalIconFactory.getTreeLeafIcon();
//
//		private final Icon folderIcon = MetalIconFactory.getTreeFolderIcon();
//		
//		public String getName(File f) {
//			String name = f.getName();
//		    return name.equals("") ? f.getPath() : name;
//		}
//		
//		public String getDescription(File f) {
//			return getTypeDescription(f);
//		}
//		
//		public String getTypeDescription(File f) {
//			String name = f.getName().toLowerCase();
//		    if (f.isDirectory()) {
//		        return "Folder";
//		    }
//		    if (name.endsWith(".jpg")) {
//		        return "JPEG Image";
//		    } else if (name.endsWith(".gif")) {
//		        return "GIF Image";
//		    } else if (name.endsWith(".png")) {
//		        return "PNG Image";
//		    }
//		    return "Generic File";
//		}
//		
//		public Icon getIcon(File f) {
//			if (f.isDirectory()) {
//		        return folderIcon;
//		    }
//
//		    String name = f.getName().toLowerCase();
//		    if (name.endsWith(".jpg") || name.endsWith(".gif") || name.endsWith(".png")) {
//		    	ImageIcon icon = new ImageIcon(f.getAbsolutePath());
//		    	icon = new ImageIcon(icon.getImage().getScaledInstance(16,16,Image.SCALE_DEFAULT));
//		        return icon;
//		    }
//		    
//		    return fileIcon;
//		}
//		
//		public Boolean isTraversable(File f) {
//			 return f.isDirectory() ? Boolean.TRUE : Boolean.FALSE;
//		}
//		
//	}
	
	private IconDisplayer displayer;
	private NumberField numberField;
//	private JFileChooser fileChooser;
	private IconEditor iconField;
	private JList valueList;
	private JScrollPane listScroll;
	private JButton addButton;
	private JButton removeButton;
	private JPanel buttonPanel;
//	private JButton chooserButton;
	private JLabel valueLabel;
	private JLabel iconLabel;
	private DefaultListModel model;
	private ArrayList<ValueHolder> icons = new ArrayList<>();
	private IconEditor defaultIconPanel;
	private boolean defaultIconMode = true;
	private boolean valueIconMode = true;
//	private Icon defaultIcon;
	private String defaultIconPath;
	
	
	/**
	 * Constructs a new IconCustomizer.
	 *
	 */
	public IconCustomizer() {
		super();
		initialize();
	}
	
	private void initialize() {
		this.setLayout(new GridBagLayout());
		valueLabel = new JLabel("Value");
		GUIUtilities.setFixedSize(valueLabel,new Dimension(50,21));
		valueLabel.setVisible(valueIconMode);
		this.add(valueLabel, new GridBagConstraints(0,0,1,1,0,0,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(11,11,1,1),0,0));
		this.add(getNumberField(), new GridBagConstraints(1,0,2,1,1,0,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(11,1,1,11),0,0));
		
		iconLabel = new JLabel("Icon");
		GUIUtilities.setFixedSize(iconLabel,new Dimension(50,21));
		iconLabel.setVisible(valueIconMode);
		this.add(iconLabel, new GridBagConstraints(0,1,1,1,0,0,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(1,11,1,1),0,0));
		this.add(getIconField(), new GridBagConstraints(1,1,1,1,1,0,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(-10,-10,-5,0),0,0));
//		this.add(getChooserButton(), new GridBagConstraints(2,1,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,1,1,11),0,0));
		this.add(getButtonPanel(), new GridBagConstraints(0,2,3,1,1,0,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5,11,5,11),0,0));
		this.add(getListScroll(), new GridBagConstraints(0,3,3,1,1,1,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0,11,5,11),0,0));
		this.add(getDefaultIconPanel(), new GridBagConstraints(0,4,3,1,1,0,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(-6,0,0,0),0,0));
	}
	
	private IconEditor getDefaultIconPanel() {
		if (defaultIconPanel == null) {
			defaultIconPanel = new IconEditor("Default icon");
			defaultIconPanel.addPropertyChangeListener(IconEditor.ICON, new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent evt) {
//					setDefaultIcon((Icon)evt.getNewValue());					
					setDefaultIcon(defaultIconPanel.getIconPath());
				}
			});
		}
		
		return defaultIconPanel;
	}
	
	private JPanel getButtonPanel() {
		if(buttonPanel == null) {
			buttonPanel = new JPanel(new GridBagLayout());
			buttonPanel.add(getAddButton(), new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0,0,0,5),0,0));
			buttonPanel.add(getRemoveButton(), new GridBagConstraints(1,0,1,1,1,1,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,5,0,0),0,0));
			buttonPanel.setVisible(valueIconMode);
		}
		return buttonPanel;
	}
	
	private void createAndAddIcon() {
//		Icon icon = getIconField().getIcon();
		String path = getIconField().getIconPath();
		long value = getNumberField().getLongValue();
		ValueHolder vh = new ValueHolder(new ValueIconPair(value, path), path);
		addIcon(vh);
	}
	
	private void addIcon(ValueHolder vh) {
		getListModel().addElement(vh);
		if (displayer != null) {
		    displayer.addIcon(vh.pair);
	    }
		
		firePropertyChange(IconDisplayer.ICONS_PROPERTY,null,getIcons());
	}
	
	private void removeIcon(ValueHolder agp) {
		if (agp == null || !icons.contains(agp)) return;
		getListModel().removeElement(agp);
		icons.remove(agp);
		if (displayer != null) {
		    displayer.removeIcon(agp.pair);
	    }
		firePropertyChange(IconDisplayer.ICONS_PROPERTY,null,getIcons());
	}

	private JButton getAddButton() {
		if (addButton == null) {
			addButton = new JButton("Add");
			addButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					createAndAddIcon();
				}
			});
			addButton.setEnabled(false);
		}
		return addButton;
	}
	
	private JButton getRemoveButton() {
		if (removeButton == null) {
			removeButton = new JButton("Remove");
			removeButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					Object[] selected = getValueList().getSelectedValues();
					if (selected != null) {
						for (Object o : selected) {
							removeIcon((ValueHolder) o);
						}
					}
				}
			});
		}
		return removeButton;
	}
	
	private NumberField getNumberField() {
		if (numberField == null) {
			numberField = new NumberField();
			numberField.setNumberType(Long.class);
			numberField.setVisible(valueIconMode);
		}
		return numberField;
	}
	
	private IconEditor getIconField() {
		if (iconField == null) {
			iconField = new IconEditor(null);
			iconField.setVisible(valueIconMode);
			iconField.addPropertyChangeListener(IconEditor.ICON, new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent evt) {
					if (evt.getNewValue() != null)
						getAddButton().setEnabled(true);
					else
						getAddButton().setEnabled(false);
				}
			});
		}
		return iconField;
	}
	
//	private JButton getChooserButton() {
//		if (chooserButton == null) {
//			chooserButton = new JButton("...");
//	        GUIUtilities.setFixedSize(chooserButton, new Dimension(20,19));
//			chooserButton.addActionListener(new ActionListener() {
//				public void actionPerformed(ActionEvent e) {
//					getIconField().setText(chooseIcon());					
//				}
//			});
//			chooserButton.setVisible(valueIconMode);
//		}
//		return chooserButton;
//	}
	
	private JScrollPane getListScroll() {
		if (listScroll == null) {
			listScroll = new JScrollPane(getValueList());
			listScroll.setVisible(valueIconMode);
			GUIUtilities.setFixedSize(listScroll,new Dimension(250, 190));
		}
		return listScroll;
	}
	
	private JList getValueList() {
		if (valueList == null) {
			valueList = new JList();
			valueList.setCellRenderer(new CellRenderer());
			valueList.setModel(getListModel());
		}
		return valueList;
	}
	
	private DefaultListModel getListModel() {
		if (model == null) {
			model = new DefaultListModel() {
				private static final long serialVersionUID = -3602336759931247665L;

				@Override
				public void addElement(Object o) {
					long value = ((ValueHolder)o).pair.getValue();
					ValueHolder old;
					if (getSize() != 0) {
						for (int i = 0; i < getSize(); i++) {
							old = (ValueHolder) getElementAt(i);
							if (value < old.pair.getValue()) {
								super.add(i, o);
								return;
							} else if (value == old.pair.getValue()) {
								super.add(i, o);
								removeElement(old);
								return;
							} 
							
						}
						super.addElement(o);
					} else {
						super.add(0, o);
					}
					
					
					
				}
			};
		}
		return model;
	}
	
//	private String chooseIcon() {
//		if (fileChooser == null) {
//			fileChooser = new JFileChooser();
//			fileChooser.setAccessory(new LabelAccessory(fileChooser));
//			fileChooser.setFileView(new ThumbnailFileView());
//			fileChooser.setFileFilter(new FileFilter(){
//				@Override
//				public boolean accept(File f) {
//					if (f.isDirectory()) {
//						return true;
//					}
//					
//					String name = f.getName();
//					if (name.endsWith(".gif") || name.endsWith(".png") || name.endsWith(".jpg")) {
//						return true;
//					}
//					return false;
//				}
//
//				@Override
//				public String getDescription() {
//					return "Image files (gif, png, jpg)";
//				}
//				
//			});
//		}
//
//		int selection = fileChooser.showDialog(this, "Open");
//		if (selection == JFileChooser.CANCEL_OPTION) return "";
//		return fileChooser.getSelectedFile().getAbsolutePath();
//	}
	
	/**
	 * Sets default icon.
	 * 
	 * @param icon the path to the icon
	 */
	public void setDefaultIcon(String icon) {
		String old = this.defaultIconPath;
		if ((old != null && old.equals(icon)) || (old == null && icon == null)) return;
		
		this.defaultIconPath = icon;
		getDefaultIconPanel().setIcon(icon);
		if (displayer != null)
			displayer.setDefaultIconName(icon);
		firePropertyChange(IconDisplayer.DEFAULT_ICON_NAME_PROPERTY, old, defaultIconPath);
	}
	
	/**
	 * Returns the default icon
	 * 
	 * @return path to the icon
	 */
	public String getDefaultIcon() {
		return defaultIconPath;
	}
	 
	/*
	 * (non-Javadoc)
	 * @see java.beans.Customizer#setObject(java.lang.Object)
	 */
	public void setObject(Object bean) {
		if (!(bean instanceof IconDisplayer)) return;
		this.displayer = (IconDisplayer) bean;
		this.displayer.addPropertyChangeListener(IconDisplayer.ICONS_PROPERTY, new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
			    if (!IconDisplayer.ICONS_PROPERTY.equals(evt.getPropertyName())) return;
			    setIcons(displayer.getIcons());
			}
		});
		this.displayer.addPropertyChangeListener(IconDisplayer.DEFAULT_ICON_NAME_PROPERTY, new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
			    if (!IconDisplayer.DEFAULT_ICON_NAME_PROPERTY.equals(evt.getPropertyName())) return;
//			    setDefaultIcon(displayer.getDefaultIcon());
			    setDefaultIcon(displayer.getDefaultIconName());
			}
		});
		setDefaultIcon(displayer.getDefaultIconName());
		setIcons(displayer.getIcons());
	}
	
	/**
	 * Sets the icons to the customizer chooser.
	 * 
	 * @param pairs
	 */
	public void setIcons(ValueIconPair[] pairs) {
		if (pairs == null) return;
		
		ArrayList<ValueHolder> toBeRemoved = new ArrayList<>();
		
		if (icons != null) {
			for(ValueHolder cp : icons) {
				toBeRemoved.add(cp);
			}
		} else {
			icons = new ArrayList<>();
		}
		
		ArrayList<ValueHolder> toBeAdded = new ArrayList<>();
		
		
		ValueHolder vh;
		for (ValueIconPair pair : pairs) {
			vh = new ValueHolder(pair, pair.getIconName());
			if (toBeRemoved.contains(vh)) {
				toBeRemoved.remove(vh);
			} else {
				toBeAdded.add(vh);
			}
		}
		
		for (ValueHolder p : toBeRemoved) {
			getListModel().removeElement(p);
			icons.remove(p);
		}
		
		for (ValueHolder p : toBeAdded) {
			getListModel().addElement(p);
			icons.add(p);
		}
		
		firePropertyChange(IconDisplayer.ICONS_PROPERTY,null,getIcons());
		
	}
	
	/**
	 * Returns the icons selected in the customizer.
	 * 
	 * @return
	 */
	public ValueIconPair[] getIcons() {
		ValueIconPair[] icons = new ValueIconPair[getListModel().size()];
		for (int i = 0; i< getListModel().size(); i++) {
			icons[i] = ((ValueHolder) getListModel().get(i)).pair;
		}
		return icons;
	}
	
	/**
	 * Enables/disables default icon selection. If disabled, user cannot
	 * set default icon.
	 * 
	 * @param defaultIconMode
	 */
	public void setDefaultIconMode(boolean defaultIconMode) {
		if (this.defaultIconMode == defaultIconMode) return;
		this.defaultIconMode = defaultIconMode;
		getDefaultIconPanel().setVisible(this.defaultIconMode);
	}
	
	/**
	 * Enables/disables selection of icons. If disabled user cannot 
	 * set any icons except default one if defaultIconMode is true.
	 * 
	 * @param valueIconMode
	 */
	public void setValueIconMode(boolean valueIconMode) {
		if (this.valueIconMode == valueIconMode) return;
		this.valueIconMode = valueIconMode;
		iconLabel.setVisible(this.valueIconMode);
		valueLabel.setVisible(this.valueIconMode);
		getNumberField().setVisible(this.valueIconMode);
		getIconField().setVisible(this.valueIconMode);
//		getChooserButton().setVisible(this.valueIconMode);
		getButtonPanel().setVisible(this.valueIconMode);
		getListScroll().setVisible(this.valueIconMode);
	}
	
	/**
	 * Returns true if value icon mode is enabled.
	 * 
	 * @return
	 */
	public boolean getValueIconMode() {
		return valueIconMode;
	}
	
	/**
	 * Returns true if default icon mode is enabled.
	 * 
	 * @return
	 */
	public boolean getDefaultIconMode() {
		return defaultIconMode;
	}
	
	/**
	 * Scales icon to fit the constraints preserving the aspect ration of the icon.
	 * 
	 * @param icon icon to be scaled
	 * @param maxHeight maximum allowed hight
	 * @param maxWidth maximum allowed width
	 * @return scaled icon
	 */
	public static ImageIcon getScaledIcon(ImageIcon icon, int maxHeight, int maxWidth) {
		
		int height = icon.getIconHeight();
		int width = icon.getIconWidth();
		double aspect = (double)height/width;

		width = (int)(maxHeight/aspect);
		if (width > maxWidth) {
			height =(int) (maxWidth*aspect);
			width = maxWidth;
		} else{
			height = maxHeight;
		}

		return new ImageIcon(icon.getImage().getScaledInstance(
				width, height, Image.SCALE_FAST));
	}
	
	/**
	 * Scales image to fit the constraints preserving the aspect ratio of the image.
	 * 
	 * @param image image to be scaled
	 * @param maxHeight maximum allowed hight
	 * @param maxWidth maximum allowed width
	 * @return scaled icon
	 */
	public static ImageIcon getScaledImage(BufferedImage image, int maxHeight, int maxWidth) {
		
		int height = image.getHeight();
		int width = image.getWidth();
		double aspect = (double)height/width;

		width = (int)(maxHeight/aspect);
		if (width > maxWidth) {
			height =(int) (maxWidth*aspect);
			width = maxWidth;
		} else{
			height = maxHeight;
		}

		return new ImageIcon(image.getScaledInstance(
				width, height, Image.SCALE_DEFAULT));
	}
	
	public static void main(String[] args) {
		IconCustomizer c = new IconCustomizer();
		c.setDefaultIconMode(true);
		c.setValueIconMode(false);
		RunnerHelper.runComponent(c, 500, 400);
//		Icon icon = new ImageIcon("java/chart/ChartAutoResize16.gif");
//		System.out.println(icon);
//		icon = IconHelper.createIcon("../chart/ChartAutoResize16.gif");
//		System.out.println(icon);
//		Icon icon = new ImageIcon(IconCustomizer.class.getClassLoader().getResource("C:/java/chart/ChartAutoResize16.gif"));
//		System.out.println(icon);
	}

}
