/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans.
 *
 * CosyBeans is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui;

import java.awt.Component;
import java.awt.Container;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyEditor;
import java.beans.PropertyEditorSupport;

import com.cosylab.gui.components.util.RunnerHelper;

/**
 * <code>ValueIconPropertyEditor</code> is an implementation of the 
 * <code>java.beans.PropertyEditor</code> used for creating ValueIconPairs
 * and setting the value as a bean property.
 *  
 * @author Jaka Bobnar, Cosylab
 *
 */
public class ValueIconPropertyEditor extends PropertyEditorSupport implements PropertyEditor {

	private IconCustomizer editor;
	 
	/**
	 * Creates new instance of SelectorPropertyEditor.
	 */
	public ValueIconPropertyEditor() {
		super();
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#getCustomEditor()
	 */
	@Override
    public Component getCustomEditor() {
		if (editor == null) {
			editor = new IconCustomizer();
			editor.setIcons((ValueIconPair[]) getValue());
			editor.addPropertyChangeListener(IconDisplayer.ICONS_PROPERTY,new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent evt) {
					setValueSilently(editor.getIcons());
				}
			});
			editor.setValueIconMode(true);
			editor.setDefaultIconMode(false);
			editor.setSize(400,400);
		}

		return editor;
    }

	/*
	 * (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#getJavaInitializationString()
	 */
	@Override
    public String getJavaInitializationString() {
		StringBuffer str = new StringBuffer("new com.cosylab.gui.ValueIconPair[]{");
		if (getValue() == null) return str.toString()+"}";
		ValueIconPair[] pairs = (ValueIconPair[]) getValue();
		String name;
		int index;
		for (ValueIconPair pair : pairs) {
			name = pair.getIconName(); 
			name = name.replace('\\','/');
			index = name.indexOf("!/");
			if (index > 0) {
				name = name.substring(index+2);
			}
			str.append("new com.cosylab.gui.ValueIconPair(new Long(" +pair.getValue() + "),\"" + name+ "\"),");
		}
		str.append("}");
		
		return str.substring(0);
    }

	/*
	 * (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#supportsCustomEditor()
	 */
	@Override
    public boolean supportsCustomEditor() {
	    return true;
    }
	
//	/*
//	 * (non-Javadoc)
//	 * @see java.beans.PropertyEditorSupport#setAsText(java.lang.String)
//	 */
//	public void setAsText(String text) {
//		String[] values = text.split(";");
//		ValueIconPair[] pairs;
//		Icon icon;
//		Long value;
//		if (values != null && values.length > 0) {
//			String[] d;
//			pairs = new ValueIconPair[values.length];
//			for (int i = 0; i < values.length; i++) {
//				d = values[i].split("=");
//				value = Long.parseLong(d[0].trim());
//				icon = IconHelper.createIcon(d[1].trim());
//				pairs[i] = new ValueIconPair(value,icon);
//			}
//		} else {
//			values = text.split("=");
//			value = Long.parseLong(values[0].trim());
//			icon = IconHelper.createIcon(values[1].trim());
//			pairs = new ValueIconPair[]{new ValueIconPair(value,icon)};
//		}
//		setValue(pairs);
//    }
	
	@Override
    public String getAsText() {
		ValueIconPair[] pairs = (ValueIconPair[]) getValue();	
	    if (pairs == null || pairs.length == 0) return "";
	    StringBuffer buffer = new StringBuffer();
	    for (ValueIconPair cp : pairs) {
	    	buffer.append(cp.toString() + ";");
	    }
	    return buffer.toString();
	}
	
	private void setValueSilently(Object value) {
		super.setValue(value);
	}
	
	/* (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#setValue(java.lang.Object)
	 */
	@Override
    public void setValue(Object value) {
		super.setValue(value);
		if (editor!=null) {
			editor.setIcons((ValueIconPair[]) value);
		}
	}
	
	public static void main(String[] args) {
		ValueIconPropertyEditor ed = new ValueIconPropertyEditor();
		ed.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				System.out.println(evt);
				
			}
		});
		RunnerHelper.runComponent((Container) ed.getCustomEditor(), 500,500);
	}
}


/* __oOo__ */
