/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans.
 *
 * CosyBeans is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.adapters;

import com.cosylab.gui.displayers.AsynchronousNumberConsumer;
import com.cosylab.gui.displayers.DataConsumer;
import com.cosylab.gui.displayers.UpdateRequest;
import com.cosylab.util.CommonException;


/**
 * <code>AsynchronousDispatcher</code> is implementation of
 * <code>AsynchronousNumberConsumer</code> which delegates value updates to
 * registered consumers.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $Id: AsynchronousDispatcher.java,v 1.5 2008-04-22 12:31:02 jbobnar Exp $
 *
 * @since Feb 14, 2004.
 */
public class AsynchronousDispatcher extends DataConsumerDispatcher
	implements AsynchronousNumberConsumer
{
	private static final long serialVersionUID = 1L;

	/**
	 * Creates new dispatcher.
	 */
	public AsynchronousDispatcher()
	{
		super(new Class[]{ AsynchronousNumberConsumer.class });
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.NonblockingNumberConsumer#updateNonblocking(java.lang.Number)
	 */
	 /* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.AsynchronousNumberConsumer#updateAcynchronously(com.cosylab.gui.displayers.UpdateRequest)
	 */
	public void updateAcynchronously(UpdateRequest request)
		throws CommonException {
		DataConsumer[] delegates = getConsumers();

		for (int i = 0; i < delegates.length; i++) {
			try {
				((AsynchronousNumberConsumer)delegates[i]).updateAcynchronously(request);
			} catch (CommonException e) {
				throw e;
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

}

/* __oOo__ */
