/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans.
 *
 * CosyBeans is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.adapters;

import java.util.Map;

import com.cosylab.gui.displayers.DataConsumer;
import com.cosylab.gui.displayers.DataSourceSupport;
import com.cosylab.gui.displayers.DataState;


/**
 * <code>DataConsumerDispatcher</code> is support class for data consumer
 * dispatcher implementators.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $Id: DataConsumerDispatcher.java,v 1.7 2008-04-22 12:31:02 jbobnar Exp $
 *
 * @since Feb 14, 2004.
 */
public abstract class DataConsumerDispatcher extends DataSourceSupport
	implements DataConsumer
{
	/**
     * 
     */
    private static final long serialVersionUID = 1L;
    protected String name = "DataConsumerDisplatcher";

	/**
	 * Constructs a new DataConsumerDispatcher with the given supported types.
	 *
	 * @param supportedTypes
	 *
	 * @throws IllegalArgumentException
	 * @see {@link DataSourceSupport#DataSourceSupport(Class[])}
	 */
	public DataConsumerDispatcher(Class[] supportedTypes)
	{
		super(supportedTypes);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getDefaultDataConsumer()
	 */
	public DataConsumer getDefaultDataConsumer()
	{
		return this;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getName()
	 */
	public String getName()
	{
		return name;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getSupportedCharacteristics()
	 */
	public String[] getSupportedCharacteristics()
	{
		return extractSupportedCharacteristics();
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getSupportedConsumerTypes()
	 */
	public Class[] getSupportedConsumerTypes()
	{
		return getAcceptableConsumerTypes();
	}

	public DataConsumer getDataConsumer(Class type) {
		if (type==null) {
			throw new NullPointerException("type");
		}
		if (type.isAssignableFrom(getClass())) {
			return this;
		}
		return null;
	}

	public void updateDataState(DataState state) {
		DataConsumer[] delegates = getConsumers();
	
		for (int i = 0; i < delegates.length; i++) {
			try {
				delegates[i].updateDataState(state);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	public void setCharacteristics(Map characteristics) {
		DataConsumer[] delegates = getConsumers();
	
		for (int i = 0; i < delegates.length; i++) {
			try {
				delegates[i].setCharacteristics(characteristics);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
	
}

/* __oOo__ */
