/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans.
 *
 * CosyBeans is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.adapters;

import com.cosylab.gui.displayers.DataConsumer;
import com.cosylab.gui.displayers.DoubleSeqConsumer;
import com.cosylab.util.CommonException;


/**
 * <code>SimpleConverterSupport</code> is implementation of
 * <code>Converter</code> interface intended for simple implementation, which
 * use transfromation function on double data flow.
 * 
 * <p>
 * <b>Note!</b> This implementation works only with data sources and consumers,
 * whcih support <code>double[]</code> as value type.
 * </p>
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $Id: SimpleArrayConverterSupport.java,v 1.4 2008-04-22 12:31:02 jbobnar Exp $
 *
 * @see com.cosylab.gui.adapters.Converter
 * @since Feb 14, 2004.
 */
public abstract class SimpleArrayConverterSupport extends AbstractConverter
	implements Converter, DoubleSeqConsumer
{

	/**
     * 
     */
    private static final long serialVersionUID = 1L;

    /**
	 * Creates new converter.
	 */
	public SimpleArrayConverterSupport()
	{
		// we dalagate to super suppport class only the consumer type/types we want to suppport
		super(new Class[]{ DoubleSeqConsumer.class });

		// the name of this converter when it acts as a consumer
		name = "SimpleConverter";
	}

	/**
	 * This is called by peer data source. Call is delegated to contained
	 * consumers.
	 *
	 * @see com.cosylab.gui.displayers.DoubleConsumer#updateValue(long, double)
	 */
	public void updateValue(long timestamp, double[] value)
		throws CommonException
	{

		DataConsumer[] delegates = getConsumers();

		// transform value
		final double[] newValue = transform(value);

		for (int i = 0; i < delegates.length; i++) {
			try {
				((DoubleSeqConsumer)delegates[i]).updateValue(timestamp, newValue);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * User must implement this method and from prvided parameter calculate
	 * with own transfromatin function new value and return it.
	 *
	 * @param value the value to be transfirmed
	 *
	 * @return the transformed value
	 */
	abstract protected double[] transform(double[] value);

}

/* __oOo__ */
