/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.Customizer;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.ButtonGroup;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;

import com.cosylab.gui.components.customizer.AbstractCustomizerPanel;
import com.cosylab.gui.components.range2.RangedValuePolicy;
import com.cosylab.gui.components.range2.RescalingValuePolicy;
import com.cosylab.gui.components.range2.ShiftValuePolicy;
import com.cosylab.gui.components.range2.TrimValuePolicy;

public class GaugerCustomizer extends AbstractCustomizerPanel {
	private static final long serialVersionUID = 1L;
	/** Component name */
	public static final String GAUGER = "Gauger";
	/** ASPECTS */
	public static final String[] ASPECTS = { GAUGER };
	
	/** Tags for properties */
	public static final String FORMAT = "format", UNITS = "units", MINIMUM = "minimum", MAXIMUM = "maximum";
	
	/** Value display properties */
	public static final String[] GAUGER_PROPERTIES = {FORMAT, UNITS, MINIMUM, MAXIMUM};
		
	private class WPanel extends JPanel implements Customizer
	{
		private static final long serialVersionUID = 1L;
		private boolean initialized = false;
		private Gauger disp;

		// GUI elements
		private NumberField minField;
		private NumberField maxField;
		private JTextField formatField;
		private JTextField unitsField;
		private JCheckBox logarithmic;
		private JPanel behaviourPanel;
		private JPanel displayPanel;
		private JRadioButton fixedButton;
		private JRadioButton stretchButton;
		private JRadioButton slideButton;
		private ButtonGroup scaleGroup;
		private JTextField fontSize;

		private void initialize()
		{
			if (initialized) {
				return;
			}

			initialized = true;

			setLayout(new GridBagLayout());
			setSize(300, 450);

			add(getDisplayPanel(),
			    new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0,
			        GridBagConstraints.CENTER, GridBagConstraints.BOTH,
			        new Insets(11, 11, 4, 11), 0, 0));
			add(getBehaviourPanel(),
			    new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0,
			        GridBagConstraints.CENTER, GridBagConstraints.BOTH,
			        new Insets(11, 11, 4, 11), 0, 0));
		}

		private JPanel getDisplayPanel()
		{
			if (displayPanel == null) {
				displayPanel = new JPanel();
				displayPanel.setBorder(new GroupBoxBorder("Value Display"));
				displayPanel.setLayout(new GridBagLayout());

				displayPanel.add(new JLabel("Format"),
				    new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
				        GridBagConstraints.WEST, GridBagConstraints.NONE,
				        new Insets(4, 11, 4, 4), 0, 0));
				displayPanel.add(getFormatField(),
				    new GridBagConstraints(1, 0, 1, 1, 1.0, 1.0,
				        GridBagConstraints.WEST, GridBagConstraints.NONE,
				        new Insets(4, 4, 4, 4), 0, 0));

				displayPanel.add(new JLabel("Units"),
				    new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
				        GridBagConstraints.WEST, GridBagConstraints.NONE,
				        new Insets(4, 11, 4, 4), 0, 0));
				displayPanel.add(getUnitsField(),
				    new GridBagConstraints(1, 1, 1, 1, 1.0, 1.0,
				        GridBagConstraints.WEST, GridBagConstraints.NONE,
				        new Insets(4, 4, 4, 4), 0, 0));

				displayPanel.add(new JLabel("Minimum"),
				    new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0,
				        GridBagConstraints.WEST, GridBagConstraints.NONE,
				        new Insets(4, 11, 4, 4), 0, 0));
				displayPanel.add(getMinField(),
				    new GridBagConstraints(1, 2, 1, 1, 1.0, 1.0,
				        GridBagConstraints.WEST, GridBagConstraints.NONE,
				        new Insets(4, 4, 4, 4), 0, 0));

				displayPanel.add(new JLabel("Maximum"),
				    new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0,
				        GridBagConstraints.WEST, GridBagConstraints.NONE,
				        new Insets(4, 11, 4, 4), 0, 0));
				displayPanel.add(getMaxField(),
				    new GridBagConstraints(1, 3, 1, 1, 1.0, 1.0,
				        GridBagConstraints.WEST, GridBagConstraints.NONE,
				        new Insets(4, 4, 4, 4), 0, 0));

				displayPanel.add(new JLabel("Value label size"),
				    new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0,
				        GridBagConstraints.WEST, GridBagConstraints.NONE,
				        new Insets(4, 11, 4, 4), 0, 0));
				displayPanel.add(getFontSize(),
				    new GridBagConstraints(1, 4, 1, 1, 1.0, 1.0,
				        GridBagConstraints.WEST, GridBagConstraints.NONE,
				        new Insets(4, 4, 4, 4), 0, 0));
			}

			return displayPanel;
		}

		private JPanel getBehaviourPanel()
		{
			if (behaviourPanel == null) {
				Insets insets = new Insets(2, 2, 2, 2);
				behaviourPanel = new JPanel();
				behaviourPanel.setBorder(new GroupBoxBorder("Scale preferences"));
				behaviourPanel.setLayout(new GridBagLayout());
				behaviourPanel.add(getLogarithmic(),
				    new GridBagConstraints(0, 1, 3, 1, 1.0, 1.0,
				        GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
				        insets, 0, 0));

				behaviourPanel.add(getFixedButton(),
				    new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0,
				        GridBagConstraints.WEST, GridBagConstraints.NONE,
				        insets, 0, 0));

				behaviourPanel.add(getStretchButton(),
				    new GridBagConstraints(1, 0, 1, 1, 1.0, 1.0,
				        GridBagConstraints.WEST, GridBagConstraints.NONE,
				        insets, 0, 0));

				behaviourPanel.add(getSlideButton(),
				    new GridBagConstraints(2, 0, 1, 1, 1.0, 1.0,
				        GridBagConstraints.WEST, GridBagConstraints.NONE,
				        insets, 0, 0));
			}

			return behaviourPanel;
		}

		private JTextField getFontSize()
		{
			if (fontSize == null) {
				fontSize = new JTextField();
				fontSize.setPreferredSize(new Dimension(75, 21));
				fontSize.setMinimumSize(new Dimension(75, 21));
				fontSize.setToolTipText("Font size of value label");
				fontSize.addActionListener(applySettingsListener);
			}

			return fontSize;
		}

		private JCheckBox getLogarithmic()
		{
			if (logarithmic == null) {
				logarithmic = new JCheckBox("Logarithmic");
				logarithmic.setPreferredSize(new Dimension(150, 21));
				logarithmic.setMinimumSize(new Dimension(75, 21));
				logarithmic.setToolTipText("Use logarithmic scale.");
				logarithmic.addActionListener(applySettingsListener);
			}

			return logarithmic;
		}

		/**
		 * Returns editor for units.
		 *
		 * @return Instance of JTextField.
		 */
		private JTextField getUnitsField()
		{
			if (unitsField == null) {
				unitsField = new JTextField();
				unitsField.setPreferredSize(new Dimension(75, 21));
				unitsField.setMinimumSize(new Dimension(75, 21));
				unitsField.setToolTipText("Physical units of displayed value");
				unitsField.addActionListener(applySettingsListener);
			}

			return unitsField;
		}

		/**
		 * Returns editor for format.
		 *
		 * @return Instance of JTextField.
		 */
		private JTextField getFormatField()
		{
			if (formatField == null) {
				formatField = new JTextField();
				formatField.setPreferredSize(new Dimension(75, 21));
				formatField.setMinimumSize(new Dimension(75, 21));
				formatField.setToolTipText("C Style Format");
				formatField.addActionListener(applySettingsListener);
			}

			return formatField;
		}

		/**
		 * Returns editor for max value.
		 *
		 * @return Instance of NumberField.
		 */
		private NumberField getMaxField()
		{
			if (maxField == null) {
				maxField = new NumberField();
				maxField.setNumberType(Double.class);
				maxField.setPreferredSize(new Dimension(100, 21));
				maxField.setToolTipText("Maximal allowed value");
				maxField.addActionListener(applySettingsListener);
			}

			return maxField;
		}

		private NumberField getMinField()
		{
			if (minField == null) {
				minField = new NumberField();
				minField.setNumberType(Double.class);
				minField.setPreferredSize(new Dimension(100, 21));
				minField.setToolTipText("Minimal allowed value");
				minField.addActionListener(applySettingsListener);
			}

			return minField;
		}

		private ButtonGroup getScaleGroup()
		{
			if (scaleGroup == null) {
				scaleGroup = new ButtonGroup();
			}

			return scaleGroup;
		}

		private JRadioButton getFixedButton()
		{
			if (fixedButton == null) {
				fixedButton = new JRadioButton("Fixed");
				getScaleGroup().add(fixedButton);
				fixedButton.addActionListener(applySettingsListener);
			}

			return fixedButton;
		}

		private JRadioButton getStretchButton()
		{
			if (stretchButton == null) {
				stretchButton = new JRadioButton("Stretch");
				getScaleGroup().add(stretchButton);
				stretchButton.addActionListener(applySettingsListener);
			}

			return stretchButton;
		}

		private JRadioButton getSlideButton()
		{
			if (slideButton == null) {
				slideButton = new JRadioButton("Slide");
				getScaleGroup().add(slideButton);
				slideButton.addActionListener(applySettingsListener);
			}

			return slideButton;
		}

		/*
		 * Updates editor with values from diplayer.
		 */
		private void update()
		{
			if (disp == null) {
				getMinField().setDoubleValue(0.0);
				getMaxField().setDoubleValue(0.0);
				getFormatField().setText("N/A");
				getUnitsField().setText("N/A");
			} else {
				getMinField().setDoubleValue(disp.getMinimum());
				getMaxField().setDoubleValue(disp.getMaximum());
				getFormatField().setText(disp.getFormat());
				getUnitsField().setText(disp.getUnits());
				getLogarithmic().setSelected(disp.isLogarithmicScale());

				getFontSize().setText(String.valueOf(disp
				        .getValueLabelFont().getSize()));

				RangedValuePolicy policy = disp.getValuePolicy();
				if (policy instanceof TrimValuePolicy) {
					getFixedButton().setSelected(true);
				} else if (policy instanceof ShiftValuePolicy) {
					getSlideButton().setSelected(true);
				} else if (policy instanceof RescalingValuePolicy) {
					getStretchButton().setSelected(true);
				}
			}
		}

		/*
		 * Sets editor values to displayer.
		 */
		private void applyDefaultProperties()
		{
			if (disp == null) {
				return;
			}
			
			disp.setValue(getMinField().getDoubleValue(),
			    getMaxField().getDoubleValue(), disp.getValue());
			disp.setFormat(getFormatField().getText());
			disp.setUnits(getUnitsField().getText());
			
			
			if (getLogarithmic().isSelected()) {
				disp.setLogarithmicScale();
			} else {
				disp.setLinearScale();
			}

			RangedValuePolicy policy = new TrimValuePolicy();

			if (getFixedButton().isSelected()) {
				policy = new TrimValuePolicy();
			}

			if (getSlideButton().isSelected()) {
				policy = new ShiftValuePolicy();
			}

			if (getStretchButton().isSelected()) {
				policy = new RescalingValuePolicy();
			}

			disp.setValuePolicy(policy);

			setFontSize(getFontSize().getText());

			disp.invalidate();
		}

		public void setFontSize(String s){
			Font f = disp.getValueLabelFont();
			try {
				disp.setValueLabelFont(f.deriveFont(
				        (float)Integer.parseInt(s)));
			} catch (NumberFormatException e) {
			}
		}

		/* (non-Javadoc)
		 * @see com.cosylab.gui.components.customizer.Editor#applySetting()
		 */
		public void applySettings()
		{
			applyDefaultProperties();
			update();
		}

		public void setObject(Object arg0) {
			disp = (Gauger)arg0;
			disp.addPropertyChangeListener("Gauger", new PropertyChangeListener(){

				public void propertyChange(PropertyChangeEvent evt) {
					if (!"Gauger".equals(evt.getPropertyName())) return;
					update();
				}
				
			});
			update();
		}
		
		ActionListener applySettingsListener= new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				applySettings();
			}
		};
		public WPanel() {
			initialize();
			update();
		}
	}
	
	public GaugerCustomizer() {
		addCustomizer(GAUGER, new WPanel());
		setSize(349, 347);
	}
	
}
