/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components;

import java.util.EventObject;


/**
 * Event for describing task progress change.
 *
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public class ProgressEvent extends EventObject
{
	private static final long serialVersionUID = 1L;
	private String status;
	private int current;
	private int total;

	/**
	 * Creates new instance of progress change event.
	 *
	 * @param source ProgressMonitor that has triggered this event.
	 * @param status Text description of progress.
	 * @param current New progress value.
	 * @param total Progress value when completed.
	 */
	public ProgressEvent(Object source, String status, int current, int total)
	{
		super(source);
		this.status = status;
		this.current = current;
		this.total = total;
	}

	/**
	 * Returns current value of progress.
	 *
	 * @return current value of progress.
	 */
	public int getCurrent()
	{
		return current;
	}

	/**
	 * Returns status description.
	 *
	 * @return Textual description of current status.
	 */
	public String getStatus()
	{
		return status;
	}

	/**
	 * Returns total ammount.
	 *
	 * @return total ammount for progress.
	 */
	public int getTotal()
	{
		return total;
	}
}

/* __oOo__ */
