/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components;

/**
 * Interface defining progress monitoring for long running tasks. This
 * interface defines listeners to be notified on progress of long running tasks.<br>
 * Progress is defined as an integer value without any predefined scale.
 * Implementor will provide maximum ammount (<code>getTotal()</code>) and
 * periodically update current value. When <code>getCurrent()</code> equals
 * <code>getTotal()</code> the task is expected to end.
 *
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $Revision: 1.3 $
 */
public interface ProgressMonitor
{
	/**
	 * Add progress listener to this object. Listener will be notified about
	 * task start and end and also itermediate progress or interruptions.
	 *
	 * @param listener to add.
	 */
	void addProgressListener(ProgressListener listener);

	/**
	 * Returns current progress, value will be between 0 and
	 * <code>getStatus()</code>.
	 *
	 * @return current value of progress.
	 */
	int getCurrent();

	/**
	 * Returns total progress, that is value when this task will be complete.
	 *
	 * @return Total progress.
	 */
	int getTotal();

	/**
	 * Returns description of current task. This text may change at any time to
	 * indicate different phases.
	 *
	 * @return Textual description.
	 */
	String getStatus();

	/**
	 * Removes listener.
	 *
	 * @param listener to remove.
	 */
	void removeProgressListener(ProgressListener listener);
}

/* __oOo__ */
