/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components;

import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;

/**
 * An extension of the <code>JTextArea</code> providing features commonly needed
 * when using a text area to report large amounts of information to the screen.
 * 
 * @author <a href="mailto:miha.kadunc@cosylab.com">Miha Kadunc</a>
 * @version $id$
 */
public class ReportTextArea extends javax.swing.JTextArea {

	private static final long serialVersionUID = 1L;
	private int maxLines = 500;
    private boolean autoScroll = true;
    private boolean autoCut = true;
    private javax.swing.JFileChooser fileChooser = null;
    EventProcessor ivjEventHandler = new EventProcessor();
    private javax.swing.JMenuItem ivjJMenuItem1 = null;
    private javax.swing.JPopupMenu ivjJPopupMenu1 = null;

    class EventProcessor
        implements java.awt.event.ActionListener, java.awt.event.MouseListener {
        public void actionPerformed(java.awt.event.ActionEvent e) {
            if (e.getSource() == ReportTextArea.this.getJMenuItem1())
                connEtoC2(e);
        }
        public void mouseClicked(java.awt.event.MouseEvent e) {
            if (e.getSource() == ReportTextArea.this)
                connEtoC1(e);
        }
        public void mouseEntered(java.awt.event.MouseEvent e) {
        }
        public void mouseExited(java.awt.event.MouseEvent e) {
        }
        public void mousePressed(java.awt.event.MouseEvent e) {
        }
        public void mouseReleased(java.awt.event.MouseEvent e) {
        }
    }
    
    /**
     * Constructs an empty ReportTextArea.
     */
    public ReportTextArea() {
        super();
        initialize();
    }
    /**
     * Constructs a text area with the specified number of rows and columns.
     * 
     * @param rows int
     * @param columns int
     */
    public ReportTextArea(int rows, int columns) {
        super(rows, columns);
    }
    /**
     * Constructs a ReportTextArea containing the specified text.
     * 
     * @param text java.lang.String
     */
    public ReportTextArea(String text) {
        super(text);
    }
    /**
     * Constructs a ReportTextArea containing the specified text and number of
     * rows and columns.
     * 
     * @param text java.lang.String
     * @param rows int
     * @param columns int
     */
    public ReportTextArea(String text, int rows, int columns) {
        super(text, rows, columns);
    }
    /**
     * Constructs a new <code>ReportTextArea</code> with the specified document.
     * 
     * @param doc javax.swing.text.Document
     */
    public ReportTextArea(javax.swing.text.Document doc) {
        super(doc);
    }
    /**
     * Constructs a new <code>ReportTextArea</code> with the specified document,
     * text, and number of rows and columns. 
     * 
     * @param doc javax.swing.text.Document
     * @param text java.lang.String
     * @param rows int
     * @param columns int
     */
    public ReportTextArea(
        javax.swing.text.Document doc,
        String text,
        int rows,
        int columns) {
        super(doc, text, rows, columns);
    }
    /**
     * Appends a string to the end of the text area.
     * 
     * @param text java.lang.String
     */
    @Override
    public void append(String text) {
        super.append(text);
        textInserted();
    }
    /**
     * Appends a string to the end of the text area with ginven text style.
     * @param text appended text
     * @param style a style of appended text
     */
    public void append(String text,AttributeSet style) {
        try  {
            getDocument().insertString(getDocument().getLength(), text, style);
        } catch (BadLocationException e) {
            handleException(e);
        }
        textInserted();
    }
    /**
     * connEtoC1:  (ReportTextArea.mouse.mouseClicked(java.awt.event.MouseEvent) --> ReportTextArea.showPopup(Ljava.awt.event.MouseEvent;)V)
     * @param arg1 java.awt.event.MouseEvent
     */
    /* WARNING: THIS METHOD WILL BE REGENERATED. */
    private void connEtoC1(java.awt.event.MouseEvent arg1) {
        try {
            // user code begin {1}
            // user code end
            this.showPopup(arg1);
            // user code begin {2}
            // user code end
        } catch (java.lang.Throwable ivjExc) {
            // user code begin {3}
            // user code end
            handleException(ivjExc);
        }
    }
    /**
     * connEtoC2:  (JMenuItem1.action.actionPerformed(java.awt.event.ActionEvent) --> ReportTextArea.saveTextToFile()V)
     * @param arg1 java.awt.event.ActionEvent
     */
    /* WARNING: THIS METHOD WILL BE REGENERATED. */
    private void connEtoC2(java.awt.event.ActionEvent arg1) {
        try {
            // user code begin {1}
            // user code end
            this.saveTextToFile();
            // user code begin {2}
            // user code end
        } catch (java.lang.Throwable ivjExc) {
            // user code begin {3}
            // user code end
            handleException(ivjExc);
        }
    }
    /**
     * connEtoM1:  (ReportTextArea.initialize() --> ReportTextArea.registerActionsForKeyStroke()V)
     */
    /* WARNING: THIS METHOD WILL BE REGENERATED. */
    private void connEtoM1() {
        try {
            // user code begin {1}
            // user code end
            this.registerActionsForKeyStroke();
            // user code begin {2}
            // user code end
        } catch (java.lang.Throwable ivjExc) {
            // user code begin {3}
            // user code end
            handleException(ivjExc);
        }
    }
    /**
     * Returns the <code>JFileChooser</code> that this component uses to save
     * the text to a file.
     * 
     * @return javax.swing.JFileChooser
     * @see #saveTextToFile()
     */
    public javax.swing.JFileChooser getFileChooser() {
        if (fileChooser == null)
            fileChooser = new javax.swing.JFileChooser();
        return fileChooser;
    }
    /**
     * Return the JMenuItem1 property value.
     * @return javax.swing.JMenuItem
     */
    /* WARNING: THIS METHOD WILL BE REGENERATED. */
    private javax.swing.JMenuItem getJMenuItem1() {
        if (ivjJMenuItem1 == null) {
            try {
                ivjJMenuItem1 = new javax.swing.JMenuItem();
                ivjJMenuItem1.setName("JMenuItem1");
                ivjJMenuItem1.setText("Save...");
                // user code begin {1}
                // user code end
            } catch (java.lang.Throwable ivjExc) {
                // user code begin {2}
                // user code end
                handleException(ivjExc);
            }
        }
        return ivjJMenuItem1;
    }
    /**
     * Return the JPopupMenu1 property value.
     * @return javax.swing.JPopupMenu
     */
    /* WARNING: THIS METHOD WILL BE REGENERATED. */
    javax.swing.JPopupMenu getJPopupMenu1() {
        if (ivjJPopupMenu1 == null) {
            try {
                ivjJPopupMenu1 = new javax.swing.JPopupMenu();
                ivjJPopupMenu1.setName("JPopupMenu1");
                ivjJPopupMenu1.add(getJMenuItem1());
                // user code begin {1}
                // user code end
            } catch (java.lang.Throwable ivjExc) {
                // user code begin {2}
                // user code end
                handleException(ivjExc);
            }
        }
        return ivjJPopupMenu1;
    }
    /**
     * Returns the maximum number of lines that this text area displays when
     * <code>autoCut</code> property is set to <code>true</code>
     * 
     * @return int
     */
    public int getMaxLines() {
        return maxLines;
    }
    /**
     * Called whenever the part throws an exception.
     * @param exception java.lang.Throwable
     */
    private void handleException(java.lang.Throwable exception) {
        if (exception instanceof RuntimeException) {
            throw (RuntimeException)exception;
        }
        else throw new RuntimeException(exception);
    }
    /**
     * Initializes connections
     * @exception java.lang.Exception The exception description.
     */
    /* WARNING: THIS METHOD WILL BE REGENERATED. */
    private void initConnections() throws java.lang.Exception {
        // user code begin {1}
        // user code end
        this.addMouseListener(ivjEventHandler);
        getJMenuItem1().addActionListener(ivjEventHandler);
    }
    /**
     * Initialize the class.
     */
    /* WARNING: THIS METHOD WILL BE REGENERATED. */
    private void initialize() {
        try {
            // user code begin {1}
            // user code end
            setName("ReportTextArea");
            setSize(160, 120);
            setEditable(true);
            initConnections();
            connEtoM1();
        } catch (java.lang.Throwable ivjExc) {
            handleException(ivjExc);
        }
        // user code begin {2}
        // user code end
    }
    /**
     * Inserts text to this text area at the specified index.
     * 
     * @param text java.lang.String
     * @param pos int
     */
    @Override
    public void insert(String text, int pos) {
        super.insert(text, pos);
        textInserted();
    }
    /**
     * When <code>autoCut</code> property is set to true, the component
     * automatically cuts the beginning of text when it contains more lines than
     * specified by the <code>maxLines</code> property.
     * 
     * @return boolean
     */
    public boolean isAutoCut() {
        return autoCut;
    }
    /**
     * Returns the value of the <code>autoScroll</code> property. This property
     * determines whether the caret position should be moved to the end of the
     * text when a string is appended to the text.  
     * 
     * @return boolean
     */
    public boolean isAutoScroll() {
        return autoScroll;
    }
    /**
     * main entrypoint - starts the part when it is run as an application
     * @param args java.lang.String[]
     */
    public static void main(java.lang.String[] args) {
        try {
            javax.swing.JFrame frame = new javax.swing.JFrame();
            ReportTextArea aSmartTextArea;
            aSmartTextArea = new ReportTextArea();
            frame.setContentPane(aSmartTextArea);
            frame.setSize(aSmartTextArea.getSize());
            frame.addWindowListener(new java.awt.event.WindowAdapter() {
                @Override
                public void windowClosing(java.awt.event.WindowEvent e) {
                    System.exit(0);
                }
            });
            frame.setVisible(true);
            java.awt.Insets insets = frame.getInsets();
            frame.setSize(
                frame.getWidth() + insets.left + insets.right,
                frame.getHeight() + insets.top + insets.bottom);
            frame.setVisible(true);
            aSmartTextArea.append("TOLE PROBAVAMO\n");
            aSmartTextArea.append("TOLE PROBAVAMO\n");
            aSmartTextArea.append("TOLE PROBAVAMO\n");
        } catch (Throwable exception) {
            System.err.println(
                "Exception occurred in main() of si.ijs.acs.objectexplorer.ReportTextArea");
            exception.printStackTrace(System.out);
        }
    }
    private void registerActionsForKeyStroke() {
        try {
            javax.swing.text.Keymap kmap = getKeymap();

            javax.swing.Action cpy =
                kmap.getAction(javax.swing.KeyStroke.getKeyStroke("control C"));
            javax.swing.Action other =
                kmap.getAction(javax.swing.KeyStroke.getKeyStroke("control C"));
            kmap.removeBindings();
            kmap.setDefaultAction(other);
            if (cpy != null)
                kmap.addActionForKeyStroke(
                    javax.swing.KeyStroke.getKeyStroke("control C"),
                    cpy);
            if (cpy != null)
                kmap.addActionForKeyStroke(
                    javax.swing.KeyStroke.getKeyStroke("control INSERT"),
                    cpy);
        } catch (Throwable exception) {
            handleException(exception);
        }
    }
    /**
     * Shows a dialog to prompt the user for a fileName and saves the text to
     * a file specified by the user.
     */
    public void saveTextToFile() {
        getFileChooser().showSaveDialog(this);
        java.io.File file = getFileChooser().getSelectedFile();
        if (file == null) return;
        try {
            if (!file.getParentFile().exists()) {
                file.getParentFile().mkdirs();
            }
            try (java.io.BufferedWriter bw =
                new java.io.BufferedWriter(new java.io.FileWriter(file))) {
                write(bw);
            }
            //System.out.println("written to:" + file + " text:" + getText());
        } catch (java.io.IOException e) {
            System.out.println(
                "exception while writing " + e + " " + file + " " + getText());
            e.printStackTrace();
        }
        return;
    }
    /**
     * Sets the <code>autoCut</code> property.
     * @param newAutocut boolean
     * @see #isAutoCut()
     */
    public void setAutoCut(boolean newAutocut) {
        autoCut = newAutocut;
    }
    /**
     * Sets the value of the <code>autoScroll</code> property.
     *  
     * @param newAutoScroll boolean
     * @see #isAutoScroll()
     */
    public void setAutoScroll(boolean newAutoScroll) {
        autoScroll = newAutoScroll;
    }
    /**
     * Sets the <code>JFileChooser</code> that this component should use when
     * saving text to a file.
     * 
     * @param newFileChooser javax.swing.JFileChooser
     */
    public void setFileChooser(javax.swing.JFileChooser newFileChooser) {
        fileChooser = newFileChooser;
    }
    /**
     * Sets the <code>maxLines</code> property.
     * 
     * @param newMaxLines int
     */
    public void setMaxLines(int newMaxLines) {
        maxLines = newMaxLines;
    }
    
    /* (non-Javadoc)
	 * @see javax.swing.text.JTextComponent#setText(java.lang.String)
	 */
	@Override
    public void setText(String t) {
		super.setText(t);
		textInserted();
	}
    
    /**
     * Comment
     */
    private synchronized void showPopup(java.awt.event.MouseEvent mouseEvent) {
        if (mouseEvent.getModifiers() == java.awt.event.MouseEvent.META_MASK) {
            if (getJPopupMenu1().getComponents().length < 2) {
                getParent().getParent().getParent();
            }
            getJPopupMenu1().show(this, mouseEvent.getX(), mouseEvent.getY());
        }
    }
    /**
     * Method should be called when text was inserted
     */
    private void textInserted() {
        try {
            if (autoCut && (getLineCount() > maxLines))
                replaceRange(
                    "",
                    0,
                    getLineStartOffset(getLineCount() - maxLines + 1));
            if (autoScroll)
                setCaretPosition(getDocument().getLength());
        } catch (Exception e) {
            handleException(e);
        }
    }
}
