/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components;

import java.beans.PropertyEditorSupport;

/**
 * <code>SliderSetModePropertyEditor</code> is an implementation
 * of the <code>PropertyEditor</code> which enables setting
 * of the {@link Slider#setSetMode(SliderSetMode)} property.
 * 
 * @see Slider
 * @see Slider#setSetMode(SliderSetMode)
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 * @version $Id: SliderSetModePropertyEditor.java,v 1.4 2008-04-22 12:28:40 jbobnar Exp $
 *
 */
public class SliderSetModePropertyEditor extends PropertyEditorSupport {

	@Override
	public String getAsText() {
		return ((SliderSetMode)getValue()).name();
	}
		
	/*
	 * (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#setAsText(java.lang.String)
	 */
	@Override
    public void setAsText(String text) {
		setValue(SliderSetMode.valueOf(text));
    }

	/*
	 * (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#getJavaInitializationString()
	 */
	@Override
    public String getJavaInitializationString() {
		return "com.cosylab.gui.components.SliderSetMode."+getAsText();
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#getTags()
	 */
	@Override
	public String[] getTags() {
		SliderSetMode[] modes =  SliderSetMode.values();
		
		String[] tags = new String[modes.length];
		for (int i = 0; i < tags.length; i++) {
			tags[i] = modes[i].name();
		}
		return tags;
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#supportsCustomEditor()
	 */
	@Override
    public boolean supportsCustomEditor() {
		return false;
	}
}
