/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components;

import java.awt.Component;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyEditorSupport;

import com.cosylab.gui.components.range2.RangedValuePolicy;

/**
 * RangedValuePolicy property editor for gui builders.
 * 
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 * @version $Id: ValuePolicyPropertyEditor.java,v 1.7 2008-04-22 12:28:40 jbobnar Exp $
 *
 */
public class ValuePolicyPropertyEditor extends PropertyEditorSupport {
	
	private ValuePolicyCustomizer editor;
	 
	/*
	 * (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#getCustomEditor()
	 */
	@Override
	public Component getCustomEditor() {
		if (editor == null) {
			editor = new ValuePolicyCustomizer();
			editor.setValuePolicy((RangedValuePolicy)getValue());
			editor.addPropertyChangeListener("valuePolicy", new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent evt) {
					setValueSilently(editor.getValuePolicy());
				}
			});
		}

		return editor;
    }

	/*
	 * (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#getJavaInitializationString()
	 */
	@Override
	public String getJavaInitializationString() {
		return "new " + getValue().getClass().getName()+"()";
    }

	/*
	 * (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#supportsCustomEditor()
	 */
	@Override
	public boolean supportsCustomEditor() {
	    return true;
    }
	
	private void setValueSilently(Object value) {
		super.setValue(value);
	}
	
	/* (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#setValue(java.lang.Object)
	 */
	@Override
	public void setValue(Object value) {
		super.setValue(value);
		if (editor!=null) {
			editor.setValuePolicy((RangedValuePolicy)value);
		}
	}
}
