/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.customizer;

import javax.swing.JComponent;


/**
 * <p>
 * <code>Editor</code> controls editor panel, which prowides GUI editor for
 * some aspect of displayer.
 * </p>
 * 
 * <p>
 * Implementator of this interface must provide defult constructor for
 * implementation class.
 * </p>
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @author <a href="mailto:jernej.kamenik@cosylab.com">Jernej Kamenik</a>
 * @version $id$
 */
public interface Editor
{
	/** List of names of editable aspects with this editor.*/
	public static String[] ASPECTS = {  };

	/**
	 * Starts editing <b>session</b> and returns conponent, which will provide
	 * GUI for editing some <code>aspect</code>, given with  parameter,  of
	 * the given displayer. IF displayer is not design to edit given component
	 * in  given aspect, <code>IllegalArgumentException</code> is thrown.
	 *
	 * @param aspect the aspect in which displayer will be edited
	 *
	 * @return editor editor component
	 */
	JComponent getEditorComponent(Object object, String aspect)
		throws IllegalArgumentException;

	/**
	 * Applies current settings to the edited displayer.  This does not stop
	 * current editing session.
	 */
	void applySettings();

	/**
	 * Test if this editor can edit given displayer with given aspect.
	 *
	 * @param aspect the aspect in which displayer should be edited
	 *
	 * @return true if this editor can edit given displayer
	 */
	boolean canEdit(Object object, String aspect);

	/**
	 * Reverts editor values to currently used settings. This does not stop
	 * current editing session.
	 */
	void revertSettings();

	/**
	 * Stop current editing session and release edited displayer.
	 */
	void stopEditing();
}

/* __oOo__ */
