/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.gauger;

import java.awt.Graphics2D;
import java.awt.Rectangle;


/**
 * Interface specifies a graphical element, that needs is relatively small in
 * size and  needs to be drawn very often over the prerendered scale. The
 * methods specified are sufficied for the Gauger to perform optimized
 * rendering when the location of the element changes.
 *
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public interface ActiveElement
{
	/**
	 * Returns the extents of this object. This method must return the bounding
	 * box of the element. The area can be larger, but it must not be smaller,
	 * or visual appearance of the gauger will be corrupted.
	 *
	 * @return Bounding rectangle.
	 */
	Rectangle getBounds();

	/**
	 * Performs the rendering of the element. The element should perform the
	 * neccesary space transform to obtain the coordinates to draw to.
	 *
	 * @param g Graphics object to draw to.
	 * @param transform transformation defining the scale.
	 */
	void render(Graphics2D g);

	/**
	 * Returns whether the element has changed significantly since it was last
	 * redrawn. If this element has the same position as the last time it was
	 * redrawn and its visual appearance has not changed, then this method
	 * should return false. If the result is true, the component will be
	 * redrawn. This method is used to  minimize the number of redraws.
	 *
	 * @return True if change is signifficant.
	 */
	boolean isChangeSignificant();

	/**
	 * Notifies the element that the transformation used to draw this scale has
	 * changed. The element must recalculate its position and bounds as it
	 * will be redrawn.
	 */
	public void setTransform(ScaleTransform transform);
}

/* __oOo__ */
