/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.gauger;

import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.geom.Point2D;


/**
 * Defines vertical linear transform.
 *
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public class LinearVerticalTransform extends ScaleTransform
{
	private static final int MAXIMUM_WIDTH = 80;
	private static final int MINIMUM_WIDTH = 25;
	private static final double RELATIVE_X_OFFSET = 0.1;
	private static final double RELATIVE_Y_OFFSET = 0.05;
	private int width = 0;
	private int height = 0;
	private int xOffset = 0;
	private int yOffset = 0;
	private int tickOffset = 0;

	/**
	 * @see ScaleTransform#setParameters(int, int, int, int, int)
	 */
	@Override
	public void setParameters(int w, int h, int marginX, int marginY,
	    int tickOffset)
	{
		this.tickOffset = tickOffset;

		width = w - (int)(2 * w * RELATIVE_X_OFFSET) - 5;
		height = h - (int)(2 * h * RELATIVE_Y_OFFSET) - 5;
		width = Math.min(width, MAXIMUM_WIDTH);
		width = Math.max(width, MINIMUM_WIDTH);

		xOffset = (int)(0.5 * (w - width));
		yOffset = (int)(0.5 * (h - height));

		Rectangle outline = new Rectangle(xOffset, yOffset, width, height);
		addSegment(new RectangleSegment(outline));

		setLabelPosition(w / 2, (int)(0.5 * (h - height - yOffset)));
	}

	/**
	 * @see ScaleTransform#mapUVtoXY(PointDouble, Point)
	 */
	@Override
	public void mapUVtoXY(Point2D scaleSpace, Point2D cartesianSpace)
	{
		double x = (xOffset + width * scaleSpace.getY());
		double y = (yOffset + scaleWidth(0.0) * (scaleSpace.getX())
			+ tickOffset);
		cartesianSpace.setLocation(x, y);
	}

	/**
	 * @see ScaleTransform#maxXYtoUV(Point, PointDouble)
	 */
	public void mapXYtoUV(Point2D cartesianSpace, Point2D scaleScape)
	{
	}

	/**
	 * @see ScaleTransform#scaleWidth(double)
	 */
	@Override
	public double scaleWidth(double v)
	{
		return Math.abs(height - 2 * tickOffset);
	}

	/**
	 * @see ScaleTransform#scaleHeight(double)
	 */
	@Override
	public double scaleHeight(double u)
	{
		return width;
	}

	/**
	 * @see com.cosylab.gui.components.gauger.ScaleTransform#measureTick(Graphics,
	 *      double, String)
	 */
	@Override
	public int measureTick(Graphics g, double x, String text)
	{
		return g.getFontMetrics().getHeight();
	}
}

/* __oOo__ */
