/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.gauger;

import java.awt.geom.Point2D;


/**
 * Defines horizontal polar transform. This transform will render the scale
 * as arc of less the 180 degrees with minimum at left and maximum at right.
 *
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public class PolarHorizontalTransform extends PolarTransform
{
	private static final int MAXIMUM_HEIGHT = 90;
	private static final int MINIMUM_HEIGHT = 35;
	private static final int MAXIMUM_HEIGHT_2 = MAXIMUM_HEIGHT / 2;
	private static final int MAXIMUM_HEIGHT_6 = MAXIMUM_HEIGHT / 6;
	private static final double RELATIVE_X_OFFSET = 0.05;
	private static final double RELATIVE_Y_OFFSET = 0.1;
	private static final double RELATIVE_SCALE_SIZE_COMPARED_TO_HEIGHT = 0.3;

	/*
	 * @see ScaleTransform#setParameters(int, int, int, int, int)
	 */
	@Override
	public void setParameters(int w, int h, int marginX, int marginY,
	    int tickOffset)
	{
		super.setParameters(w, h, marginX, marginY, tickOffset);

		height = h;

		double widthDouble = w * (1 - RELATIVE_X_OFFSET);
		double heightDouble = (h - MAXIMUM_HEIGHT_2) * (1 - RELATIVE_Y_OFFSET);

		height = (int)(heightDouble * RELATIVE_SCALE_SIZE_COMPARED_TO_HEIGHT);
		height = Math.min(height, MAXIMUM_HEIGHT);
		height = Math.max(height, MINIMUM_HEIGHT);

		// imas height in width;
		double RDouble = (0.5 * heightDouble * (1
			+ Math.pow(widthDouble / (2 * heightDouble), 2)));

		R = RDouble;
		r = R - height;

		int offset = (int)(RELATIVE_Y_OFFSET * heightDouble / 2
			+ MAXIMUM_HEIGHT_6);

		centre = new Point2D.Double(w / 2, R + offset);

		double phi = Math.asin(widthDouble / (2 * RDouble)) * 180 / Math.PI;

		alpha = (90.0 + phi);
		span = (2.0 * phi);

		setLabelPosition((int)centre.getX(), (int)(h * 0.75));

		this.tickOffset = (tickOffset * 180.0 / Math.PI) / r;

		ArcSegment segment = new ArcSegment(centre, r, R, alpha, span);
		addSegment(segment);
	}
}

/* __oOo__ */
