/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.introspection;

import com.cosylab.util.StringUtilities;

/**
 * A JPanel which allows user to enter class name, which is then
 * used to create a <code>java.lang.Class</code> instance which is
 * this bean's property and can be used by application programmers
 *
 * Creation date: (9.1.2002 22:04:46)
 * @author <a href="mailto:miha.kadunc@cosylab.com">Miha Kadunc</a>
 * @version $id$
 * 
 */
public class ClassEntryPanel extends javax.swing.JPanel {

	private static final long serialVersionUID = 1L;
	private javax.swing.JTextField ivjClassNameField = null;
	private javax.swing.JLabel ivjClassTextFieldLabel = null;
	private java.lang.Class fieldUserClass = null;
	private java.lang.Class fieldSuperClass = null;
	private boolean fieldAllowInterface = true;
	private javax.swing.JLabel ivjErrorLabel = null;
	/**
	 */
	IvjEventHandler ivjEventHandler = new IvjEventHandler();

/**
 * 
 * @author  Miha Kadunc (miha.kadunc@cosylab.com)
 */
class IvjEventHandler implements java.awt.event.KeyListener, java.beans.PropertyChangeListener {
		/**
		 * @param e java.awt.event.KeyEvent
		 * @see java.awt.event.KeyListener#keyPressed(KeyEvent)
		 */
		public void keyPressed(java.awt.event.KeyEvent e) {}
		/**
		 * @param e java.awt.event.KeyEvent
		 * @see java.awt.event.KeyListener#keyReleased(KeyEvent)
		 */
		public void keyReleased(java.awt.event.KeyEvent e) {
			if (e.getSource() == ClassEntryPanel.this.getClassNameField()) 
				connEtoC1(e);
		}
		/**
		 * @param e java.awt.event.KeyEvent
		 * @see java.awt.event.KeyListener#keyTyped(KeyEvent)
		 */
		public void keyTyped(java.awt.event.KeyEvent e) {}
		/**
		 * @param evt java.beans.PropertyChangeEvent
		 * @see java.beans.PropertyChangeListener#propertyChange(PropertyChangeEvent)
		 */
		public void propertyChange(java.beans.PropertyChangeEvent evt) {
			if (evt.getSource() == ClassEntryPanel.this.getErrorLabel() && (evt.getPropertyName().equals("background"))) 
				connPtoP1SetTarget();
			if (evt.getSource() == ClassEntryPanel.this && (evt.getPropertyName().equals("background"))) 
				connPtoP2SetTarget();
			if (evt.getSource() == ClassEntryPanel.this && (evt.getPropertyName().equals("foreground"))) 
				connPtoP3SetTarget();
		}
	}
/**
 * ClassTextField constructor comment.
 */
public ClassEntryPanel() {
	super();
	initialize();
}
/**
 * ClassTextField constructor comment.
 * @param superClass Class
 */
public ClassEntryPanel(Class superClass) {
	this();
	setSuperClass(superClass);
}
/**
 * Creation date: (9.1.2002 23:37:31)
 */
private boolean checkClass(Class c) {
	if (c == null)
		return true;
	if ((fieldSuperClass != null) && (!(fieldSuperClass.isAssignableFrom(c)))) {
		getErrorLabel().setText("Class is not a " + fieldSuperClass.getName());
		return false;
	}
	if (!(fieldAllowInterface) && (c.isInterface())) {
		getErrorLabel().setText("Interfaces are not supported");
		return false;
	}
	getErrorLabel().setText(" ");
	getErrorLabel().repaint();
	return true;
}

private void checkText() throws Exception{
    this.setUserClass(StringUtilities.classFromString(getClassNameField().getText()));
}
/**
 * connEtoC1:  (ClassNameField.key.keyReleased(java.awt.event.KeyEvent) --> ClassTextField.setUserClass(Ljava.lang.Class;)V)
 * @param arg1 java.awt.event.KeyEvent
 */
/* WARNING: THIS METHOD WILL BE REGENERATED. */
private void connEtoC1(java.awt.event.KeyEvent arg1) {
	try {
		// user code begin {1}
		// user code end
		checkText();
		// user code begin {2}
		// user code end
	} catch (java.lang.Throwable ivjExc) {
		// user code begin {3}
		// user code end
		handleException(ivjExc);
	}
}
/**
 * connPtoP1SetTarget:  (ErrorLabel.background <--> ClassTextField.background)
 */
/* WARNING: THIS METHOD WILL BE REGENERATED. */
private void connPtoP1SetTarget() {
	/* Set the target from the source */
	try {
		this.setBackground(getErrorLabel().getBackground());
		// user code begin {1}
		// user code end
	} catch (java.lang.Throwable ivjExc) {
		// user code begin {3}
		// user code end
		handleException(ivjExc);
	}
}
/**
 * connPtoP2SetTarget:  (ClassTextField.background <--> ClassTextFieldLabel.background)
 */
/* WARNING: THIS METHOD WILL BE REGENERATED. */
private void connPtoP2SetTarget() {
	/* Set the target from the source */
	try {
		getClassTextFieldLabel().setBackground(this.getBackground());
		// user code begin {1}
		// user code end
	} catch (java.lang.Throwable ivjExc) {
		// user code begin {3}
		// user code end
		handleException(ivjExc);
	}
}
/**
 * connPtoP3SetTarget:  (ClassTextField.foreground <--> ClassTextFieldLabel.foreground)
 */
/* WARNING: THIS METHOD WILL BE REGENERATED. */
private void connPtoP3SetTarget() {
	/* Set the target from the source */
	try {
		getClassTextFieldLabel().setForeground(this.getForeground());
		// user code begin {1}
		// user code end
	} catch (java.lang.Throwable ivjExc) {
		// user code begin {3}
		// user code end
		handleException(ivjExc);
	}
}
/**
 * Gets the allowInterface property (boolean) value.
 * @return The allowInterface property value.
 * @see #setAllowInterface
 */
public boolean getAllowInterface() {
	return fieldAllowInterface;
}
/**
 * 
 */
/* WARNING: THIS METHOD WILL BE REGENERATED. */
/**
 * Method generated to support the promotion of the caption attribute.
 * 
 * @return java.lang.String
 */
public java.lang.String getCaption() {
	return getClassTextFieldLabel().getText();
}
/**
 * Return the ClassNameField property value.
 * @return javax.swing.JTextField
 */
private javax.swing.JTextField getClassNameField() {
	if (ivjClassNameField == null) {
		try {
			ivjClassNameField = new javax.swing.JTextField();
			ivjClassNameField.setName("ClassNameField");
			ivjClassNameField.setPreferredSize(new java.awt.Dimension(20, 20));
			ivjClassNameField.setText("");
			ivjClassNameField.setMinimumSize(new java.awt.Dimension(100, 20));
		} catch (java.lang.Throwable ivjExc) {
			handleException(ivjExc);
		}
	}
	return ivjClassNameField;
}
/**
 * Return the ClassTextFieldLabel property value.
 * @return javax.swing.JLabel
 */
private javax.swing.JLabel getClassTextFieldLabel() {
	if (ivjClassTextFieldLabel == null) {
		try {
			ivjClassTextFieldLabel = new javax.swing.JLabel();
			ivjClassTextFieldLabel.setName("ClassTextFieldLabel");
			ivjClassTextFieldLabel.setText("Enter class name");
			ivjClassTextFieldLabel.setForeground(java.awt.Color.black);
		} catch (java.lang.Throwable ivjExc) {
			handleException(ivjExc);
		}
	}
	return ivjClassTextFieldLabel;
}
/**
 * Method generated to support the promotion of the errorColor attribute.
 * 
 * @return java.awt.Color
 */
public java.awt.Color getErrorColor() {
	return getErrorLabel().getForeground();
}
/**
 * Return the ErrorLabel property value.
 * @return javax.swing.JLabel
 */
/* WARNING: THIS METHOD WILL BE REGENERATED. */
private javax.swing.JLabel getErrorLabel() {
	if (ivjErrorLabel == null) {
		try {
			ivjErrorLabel = new javax.swing.JLabel();
			ivjErrorLabel.setName("ErrorLabel");
			ivjErrorLabel.setFont(new java.awt.Font("dialog", 0, getFont().getSize()));
			ivjErrorLabel.setText("Class not found");
			ivjErrorLabel.setForeground(java.awt.Color.red);
			// user code begin {1}
			// user code end
		} catch (java.lang.Throwable ivjExc) {
			// user code begin {2}
			// user code end
			handleException(ivjExc);
		}
	}
	return ivjErrorLabel;
}
/**
 * Method generated to support the promotion of the showCaption attribute.
 * 
 * @return boolean
 */
public boolean getShowCaption() {
	return getClassTextFieldLabel().isVisible();
}
/**
 * Gets the showErrorLabel property (boolean) value.
 * 
 * @return The showErrorLabel property value.
 * @see #setShowErrorLabel
 */
public boolean getShowErrorLabel() {
	return getErrorLabel().isVisible();
}
/**
 * Gets the superClass property (java.lang.Class) value.
 * 
 * @return The superClass property value.
 * @see #setSuperClass
 */
public java.lang.Class getSuperClass() {
	return fieldSuperClass;
}
/**
 * Gets the userClass property (java.lang.Class) value.
 * 
 * @return The userClass property value.
 * @see #setUserClass
 */
public java.lang.Class getUserClass() {
	return fieldUserClass;
}
/**
 * Called whenever the part throws an exception.
 * 
 * @param exception java.lang.Throwable
 */
private void handleException(java.lang.Throwable exception) {

	if ((exception instanceof ClassNotFoundException) || (exception instanceof NoClassDefFoundError)) {
		paintField(true);
		if (fieldUserClass!=null) setUserClass(null);
		getErrorLabel().setText("Class not found");
	}
	else if (exception instanceof NoClassDefFoundError) {
		paintField(true);
		if (fieldUserClass!=null) setUserClass(null);
		getErrorLabel().setText("Class definition not found");
	}
	else if (exception instanceof ClassFormatError) {
		paintField(true);
		if (fieldUserClass!=null) setUserClass(null);
		getErrorLabel().setText("Class format error");
	}
	else {
		paintField(true);
		if (fieldUserClass!=null) setUserClass(null);
        if (exception.getCause()!=null) {
            handleException(exception.getCause());
        } 
		else {
            getErrorLabel().setText("Exception: "+exception.toString());
        } 
	}
}
/**
 * Initializes connections
 * @exception java.lang.Exception The exception description.
 */
/* WARNING: THIS METHOD WILL BE REGENERATED. */
private void initConnections() throws java.lang.Exception {
	// user code begin {1}
	// user code end
	getClassNameField().addKeyListener(ivjEventHandler);
	getErrorLabel().addPropertyChangeListener(ivjEventHandler);
	this.addPropertyChangeListener(ivjEventHandler);
	connPtoP1SetTarget();
	connPtoP2SetTarget();
	connPtoP3SetTarget();
}
/**
 * Initialize the class.
 */
private void initialize() {
	try {
		setName("ClassEntryPanel");
		setLayout(new java.awt.GridBagLayout());
		setSize(326, 57);

		java.awt.GridBagConstraints constraintsClassNameField = new java.awt.GridBagConstraints();
		constraintsClassNameField.gridx = 0; constraintsClassNameField.gridy = 1;
		constraintsClassNameField.fill = java.awt.GridBagConstraints.HORIZONTAL;
		constraintsClassNameField.weightx = 1.0;
		constraintsClassNameField.insets = new java.awt.Insets(2, 4, 2, 4);
		add(getClassNameField(), constraintsClassNameField);

		java.awt.GridBagConstraints constraintsClassTextFieldLabel = new java.awt.GridBagConstraints();
		constraintsClassTextFieldLabel.gridx = 0; constraintsClassTextFieldLabel.gridy = 0;
		constraintsClassTextFieldLabel.fill = java.awt.GridBagConstraints.BOTH;
		constraintsClassTextFieldLabel.insets = new java.awt.Insets(0, 4, 0, 4);
		add(getClassTextFieldLabel(), constraintsClassTextFieldLabel);

		java.awt.GridBagConstraints constraintsErrorLabel = new java.awt.GridBagConstraints();
		constraintsErrorLabel.gridx = 0; constraintsErrorLabel.gridy = 2;
		constraintsErrorLabel.fill = java.awt.GridBagConstraints.BOTH;
		constraintsErrorLabel.insets = new java.awt.Insets(0, 4, 0, 4);
		add(getErrorLabel(), constraintsErrorLabel);
		initConnections();
	} catch (java.lang.Throwable ivjExc) {
		handleException(ivjExc);
	}
}

/**
 * Starts the application.
 * 
 * @param args an array of command-line arguments
 */
public static void main(java.lang.String[] args) {
	try {
		javax.swing.JFrame frame = new javax.swing.JFrame();
		ClassEntryPanel aCalculationSettingsEditor;
		aCalculationSettingsEditor = new ClassEntryPanel(javax.swing.table.TableModel.class);
		aCalculationSettingsEditor.setAllowInterface(false);
		aCalculationSettingsEditor.setCaption("Enter a name of TableModel implementation");
		aCalculationSettingsEditor.setShowErrorLabel(false);
		frame.setContentPane(aCalculationSettingsEditor);
		frame.setSize(aCalculationSettingsEditor.getSize());
		frame.addWindowListener(new java.awt.event.WindowAdapter() {
		    @Override
			public void windowClosing(java.awt.event.WindowEvent e) {
				System.exit(0);
			}
		});
		frame.setVisible(true);
		java.awt.Insets insets = frame.getInsets();
		frame.setSize(frame.getWidth() + insets.left + insets.right, frame.getHeight() + insets.top + insets.bottom);
		frame.setVisible(true);
	} catch (Throwable exception) {
		System.err.println("Exception occurred in main() of javax.swing.JPanel");
		exception.printStackTrace(System.out);
	}
}
/**
 * Method generated to support the promotion of the errorColor attribute.
 * 
 * @param arg1 java.awt.Color
 */
private void paintField(boolean isError) {
    if (isError && !getShowErrorLabel()) getClassNameField().setForeground(getErrorColor());
    else getClassNameField().setForeground(getForeground());
}
/**
 * Sets the allowInterface property (boolean) value.
 * 
 * @param allowInterface The new value for the property.
 * @see #getAllowInterface
 */
public void setAllowInterface(boolean allowInterface) {
	fieldAllowInterface = allowInterface;
}
/**
 * Method generated to support the promotion of the caption attribute.
 * 
 * @param arg1 java.lang.String
 */
public void setCaption(java.lang.String arg1) {
	getClassTextFieldLabel().setText(arg1);
}
/**
 * Method generated to support the promotion of the errorColor attribute.
 * 
 * @param arg1 java.awt.Color
 */
public void setErrorColor(java.awt.Color arg1) {
	getErrorLabel().setForeground(arg1);
}
/**
 * Method generated to support the promotion of the showCaption attribute.
 * 
 * @param arg1 boolean
 */
public void setShowCaption(boolean arg1) {
	getClassTextFieldLabel().setVisible(arg1);
}
/**
 * Sets the showErrorLabel property (boolean) value.
 * 
 * @param showErrorLabel The new value for the property.
 * @see #getShowErrorLabel
 */
public void setShowErrorLabel(boolean showErrorLabel) {
	getErrorLabel().setVisible(showErrorLabel);
}
/**
 * Sets the superClass property (java.lang.Class) value.
 * 
 * @param superClass The new value for the property.
 * @see #getSuperClass
 */
public void setSuperClass(java.lang.Class superClass) {
	try{
		fieldSuperClass = superClass;
	    checkText();
	}
	catch(Exception e){
		handleException(e);
	}
}
/**
 * Sets the userClass property (java.lang.Class) value.
 * @param userClass The new value for the property.
 * @see #getUserClass
 */
private void setUserClass(java.lang.Class userClass) {
	if (checkClass(userClass)){
		Class oldValue = fieldUserClass;
		if (userClass!=null){
			paintField(false);
		}
		if (!((""+oldValue).equals(""+userClass))) {
			fieldUserClass = userClass;
			firePropertyChange("userClass", oldValue, fieldUserClass);
		}
	}
	else{
		paintField(true);
		if (fieldUserClass!=null) setUserClass(null);
	}
}
}
