/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.introspection;

import java.lang.reflect.Method;


/**
 * A class that holds the necessary information for invoking 
 * a method on a class (if the method is static) or an object.
 * 
 * @author <a href="mailto:miha.kadunc@cosylab.com">Miha Kadunc</a>
 * @version $id$
 */
public class MethodInvocationSettings {
	private Method method=null;
	private Object[] parameters=null;
	private Object object=null;
	private boolean staticMethod=false;
	
	/**
	 * @param method Method
	 */
	public MethodInvocationSettings(Method method){
		super();
		setMethod(method);
	}
	/**
	 * Gets the method
	 * @return Returns a Method
	 */
	public Method getMethod() {
		return method;
	}
	/**
	 * Gets the object
	 * @return Returns a Object
	 */
	public Object getObject() {
		return object;
	}
	/**
	 * Gets the parameters
	 * @return Returns a String[]
	 */
	public Object[] getParameters() {
		return parameters;
	}
	/**
	 * Gets the staticMethod
	 * @return Returns a boolean
	 */
	public boolean isStaticMethod() {
		return staticMethod;
	}
	/**
	 * Sets the method
	 * @param method The method to set
	 */
	
	public void setMethod(Method method){
		this.method=method;
		staticMethod=java.lang.reflect.Modifier.isStatic(method.getModifiers());
	}
	/**
	 * Sets the object
	 * @param object The object to set
	 */
	public void setObject(Object object) {
		this.object = object;
	}
	/**
	 * Sets the parameters
	 * @param parameters The parameters to set
	 */
	public void setParameters(Object[] parameters) {
		this.parameters = parameters;
	}
	/**
	 * @return String
	 */
	@Override
    public String toString(){
		String className=method.getClass().getName();
		StringBuffer b=new StringBuffer(className.substring(className.lastIndexOf(".")));
		b.append(".");
		b.append(method.getName());
		b.append("(");
		b.append(")");
		if (staticMethod) b.append("<S>");
		return b.toString();
	}
}
