/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.introspection;

import java.awt.CardLayout;
import java.lang.reflect.Method;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import com.cosylab.introspection.ClassIntrospector;

/**
 * Creation date: (18.1.2002 18:29:16)
 *  
 * @author <a href="mailto:miha.kadunc@cosylab.com">Miha Kadunc</a>
 * @version $id$
 */
public class ParametersTable extends DefaultPropertiesTable
{
	private static final long serialVersionUID = 1L;
	private java.lang.reflect.Method fieldMethod = null;
	
/**
 * ParametersTable constructor comment.
 */
public ParametersTable() {
	super();
}
/**
 * Gets the method property (java.lang.reflect.Method) value.
 * @return The method property value.
 * @see #setMethod
 */
public java.lang.reflect.Method getMethod() {
	return fieldMethod;
}


/**
 * main entrypoint - starts the part when it is run as an application
 * @param args java.lang.String[]
 */
public static void main(java.lang.String[] args) {
	try {
		JFrame frame = new JFrame();
		JPanel panel=new JPanel();
		JScrollPane sp=new JScrollPane();
		ParametersTable aParametersTable;
		aParametersTable = new ParametersTable();

		sp.add(aParametersTable);
		sp.setViewportView(aParametersTable);
		panel.setLayout(new CardLayout());
		panel.add(sp,"ScrollPane");
		frame.setContentPane(panel);
		frame.setSize(200,100);
		frame.addWindowListener(new java.awt.event.WindowAdapter() {
			@Override
            public void windowClosing(java.awt.event.WindowEvent e) {
				System.exit(0);
			}
		});
		PropertiesTableModel ptm = new PropertiesTableModel(
				new String[] {"name","class","value","shown"},
				new Class[] {String.class,Class.class,Integer.TYPE,Boolean.TYPE},
				new Object[]{"krneki",int.class,new Integer(8),new Boolean(false)
		});
		aParametersTable.setModel(ptm);
		java.awt.Insets insets = frame.getInsets();
		frame.setSize(frame.getWidth() + insets.left + insets.right, frame.getHeight() + insets.top + insets.bottom);
		frame.setVisible(true);
	} catch (Throwable exception) {
		System.err.println("Exception occurred in main() of com.cosylab.introspection.ParametersTable");
		exception.printStackTrace(System.out);
	}
}
/**
 * Sets the method property (java.lang.reflect.Method) value.
 * @param method The new value for the property.
 * @see #getMethod
 */
public void setMethod(java.lang.reflect.Method method) {
  Method oldValue = fieldMethod;
  fieldMethod = method;
  PropertiesTableModel m =
    new PropertiesTableModel(
      ClassIntrospector.getParameterNames(method),
      method.getParameterTypes(),
      new Object[method.getParameterTypes().length]);  
  setModel(m);
  firePropertyChange("method", oldValue, method);
  revalidate();
}

}
