/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.ledder;
import java.awt.Paint;
import java.awt.PaintContext;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.ColorModel;

/**
 * This is the implementation of Paint interface tha provides access to 
 * LedPaintContext and is used to render LED icons.
 * 
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public class LedPaint implements Paint {
	
	// Parameters for LedPaintContext.
	private LedPaintParameters params;
	private int centerX;
	private int centerY;

	/**
	 * Default constructor for LedPaint.
	 * 
	 * @param parameters LedPaintParameters
	 */
	public LedPaint(LedPaintParameters parameters) {
		this(parameters,parameters.radius+1,parameters.radius+1);
	}

	/**
	 * Default constructor for LedPaint.
	 * 
	 * @param parameters LedPaintParameters
	 */
	public LedPaint(LedPaintParameters parameters, int centerX, int centerY ) {
		if (parameters == null) {
			throw new NullPointerException("Cannot create LedPaint with null parameters");
		}
			
		params = parameters;
		this.centerX=centerX;
		this.centerY=centerY;
	}

	/**
	 * @param cm ColorModel
	 * @param deviceBounds Rectangle
	 * @param userBounds Rectangle2D
	 * @param xform AffineTransform
	 * @param hints RenderingHints
	 * @return PaintContext
	 * @see Paint#createContext(ColorModel, Rectangle, Rectangle2D, AffineTransform, RenderingHints)
	 */
	public PaintContext createContext(
		ColorModel cm,
		Rectangle deviceBounds,
		Rectangle2D userBounds,
		AffineTransform xform,
		RenderingHints hints) {

		Point2D center = new Point2D.Double(centerX, centerY);

		Point2D p = xform.transform(center, null);

		return new LedPaintContext((int) p.getX(), (int) p.getY(), params);
	}

	/**
	 * Always returns OPAQUE.
	 * 
	 * @return int
	 * @see java.awt.Transparency#getTransparency()
	 */
	public int getTransparency() {
		return OPAQUE;
	}
	
	public void setCenter(int x, int y){
		centerX = x;
		centerY = y;
	}
}
