/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.ledder;
import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.geom.Rectangle2D;

import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JTextPane;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;

import com.cosylab.gui.util.GUIUtilities;

/**
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public class LedPaintDemo extends JPanel {

	/**
     * 
     */
    private static final long serialVersionUID = -3738027874678756401L;
    private JSlider hlFactorSlider;
	private JSlider hlFocusSlider;

	private JSlider shadowSlider;
	private JSlider borderSlider;

	private JSlider radiusSlider;
	private JSlider aliasSlider;

	private JPanel controlPanel;

	private JTextPane sourcePane;

	private JButton colorButton;

	private LightPosition lightPosition;

	private LedPainter painter;

	private SliderMouseListener sml = new SliderMouseListener();

	private Color currentColor = Color.RED;

	private class LedPainter extends JComponent {

		/**
         * 
         */
        private static final long serialVersionUID = -2270869689579808808L;
        private LedPaintParameters params = new LedPaintParameters();

		public void setParameters(LedPaintParameters p) {
			params = p;
		}

		@Override
        protected void paintComponent(Graphics g) {
			super.paintComponent(g);
			Graphics2D g2 = (Graphics2D) g;
			Rectangle2D r =
				new Rectangle2D.Double(0, 0, getWidth(), getHeight());

			long time = System.currentTimeMillis();
			
			LedPaint rgp = new LedPaint(params,getWidth() / 2,getHeight() / 2);

			g2.setPaint(rgp);
			g2.fill(r);

			time = System.currentTimeMillis() - time;
			System.out.println(time);
		}
	}

	private class LightPosition extends JComponent {
		/**
         * 
         */
        private static final long serialVersionUID = 5569854810051127650L;
        public double xPos = 0.0;
		public double yPos = 0.0;
		private class PointListener extends MouseAdapter {
			@Override
            public void mouseClicked(MouseEvent e) {
				int w = LightPosition.this.getWidth();
				int h = LightPosition.this.getHeight();
				int r = (w < h) ? w : h;

				xPos = (1.0 * e.getX() - (r / 2)) / (r / 2);
				yPos = (1.0 * e.getY() - (r / 2)) / (r / 2);

				getPainter().setParameters(getParameters());
				getPainter().repaint();
				displayParameters();
			}
		}
		public LightPosition() {
			addMouseListener(new PointListener());
		}
		@Override
        protected void paintComponent(Graphics g) {
			super.paintComponent(g);
			int w = LightPosition.this.getWidth();
			int h = LightPosition.this.getHeight();
			int r = (w < h) ? w : h;

			g.drawOval(0, 0, r - 1, r - 1);
		}

	}

	private class SliderMouseListener extends MouseAdapter {
		@Override
        public void mouseReleased(MouseEvent arg0) {
			getPainter().setParameters(getParameters());
			getPainter().repaint();
			displayParameters();
		}
	}

	private class ColorButtonListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			Color newColor =
				JColorChooser.showDialog(
					LedPaintDemo.this,
					"Select led color",
					currentColor);

			if (newColor == null) {
				return;
			}

			currentColor = newColor;

			getPainter().setParameters(getParameters());
			getPainter().repaint();
		}
	}

	public LedPaintDemo() {
		initialize();
	}

	private void initialize() {
		setLayout(new BorderLayout());

		add(getControlPanel(), BorderLayout.EAST);

		JPanel panel = new JPanel();
		panel.setLayout(new GridBagLayout());

		getPainter().setMinimumSize(new Dimension(10, 10));
		getPainter().setMaximumSize(new Dimension(200, 200));
		getPainter().setPreferredSize(new Dimension(200, 220));
		GridBagConstraints gbc = new GridBagConstraints();

		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.anchor = GridBagConstraints.CENTER;
		gbc.fill = GridBagConstraints.NONE;

		panel.add(getPainter(), gbc);

		add(panel, BorderLayout.CENTER);

		add(new JScrollPane(getSourcePane()), BorderLayout.SOUTH);

		resetParameters();
		getPainter().setParameters(getParameters());
		getPainter().repaint();
		displayParameters();
	}

	private LedPainter getPainter() {
		if (painter == null) {
			painter = new LedPainter();
			painter.setBorder(new TitledBorder("Preview"));
		}
		return painter;
	}

	private JPanel getControlPanel() {
		if (controlPanel == null) {
			controlPanel = new JPanel();

			controlPanel.setLayout(new GridLayout(4, 2));

			controlPanel.add(getHlFactorSlider());
			controlPanel.add(getHlFocusSlider());

			controlPanel.add(getShadowSlider());
			controlPanel.add(getBorderSlider());

			controlPanel.add(getRadiusSlider());
			controlPanel.add(getAliasSlider());

			controlPanel.add(getColorButton());

			controlPanel.add(getLightPosition());

		}
		return controlPanel;
	}

	private JSlider createSlider(String title) {
		JSlider slider = new JSlider();

		slider.setMinimum(0);
		slider.setMaximum(100);
		slider.setMajorTickSpacing(20);
		slider.setMinorTickSpacing(5);
		slider.setPaintLabels(true);
		slider.setPaintTicks(true);
		slider.setBorder(new TitledBorder(new EtchedBorder(), title));
		//		slider.addChangeListener(scl);
		slider.addMouseListener(sml);

		return slider;
	}

	private JSlider getHlFactorSlider() {
		if (hlFactorSlider == null) {
			hlFactorSlider = createSlider("Highlight factor");
		}
		return hlFactorSlider;
	}

	private JSlider getHlFocusSlider() {
		if (hlFocusSlider == null) {
			hlFocusSlider = createSlider("Highlight focus");
		}
		return hlFocusSlider;
	}

	private JSlider getShadowSlider() {
		if (shadowSlider == null) {
			shadowSlider = createSlider("Shadow intensity");
		}
		return shadowSlider;
	}

	private JSlider getBorderSlider() {
		if (borderSlider == null) {
			borderSlider = createSlider("Border size");
		}
		return borderSlider;
	}

	private JSlider getRadiusSlider() {
		if (radiusSlider == null) {
			radiusSlider = createSlider("Radius");
			radiusSlider.setMinimum(0);
		}
		return radiusSlider;
	}

	private JSlider getAliasSlider() {
		if (aliasSlider == null) {
			aliasSlider = createSlider("Antialiasing level");
			aliasSlider.setMinimum(1);
			aliasSlider.setMaximum(6);
			aliasSlider.setMinorTickSpacing(1);
			aliasSlider.setMajorTickSpacing(2);
		}
		return aliasSlider;
	}

	private JTextPane getSourcePane() {
		if (sourcePane == null) {
			sourcePane = new JTextPane();

		}
		return sourcePane;
	}

	private LedPaintParameters getParameters() {
		LedPaintParameters lpp = new LedPaintParameters();

		lpp.radius = getRadiusSlider().getValue();

		lpp.highlightFactor = getHlFactorSlider().getValue() / 100.0;
		lpp.highlightFocus = getHlFocusSlider().getValue() / 20.0;

		lpp.shadowIntensity = getShadowSlider().getValue() / 100.0;
		lpp.borderSize = getBorderSlider().getValue() / 100.0;

		lpp.color = currentColor;

		lpp.antialiasing = getAliasSlider().getValue();

		lpp.lightX = lightPosition.xPos;
		lpp.lightY = lightPosition.yPos;

		return lpp;
	}

	private LightPosition getLightPosition() {
		if (lightPosition == null) {
			lightPosition = new LightPosition();
		}
		return lightPosition;
	}

	private void displayParameters() {
		LedPaintParameters lpp = getParameters();
		StringBuffer sb = new StringBuffer();

		sb.append("LedPaintParameters lpp = new LedPaintParameters();\n");
		sb.append("lpp.radius = ");
		sb.append(lpp.radius);
		sb.append(";\n");
		sb.append("lpp.highlightFactor = ");
		sb.append(lpp.highlightFactor);
		sb.append(";\n");
		sb.append("lpp.highlightFocus = ");
		sb.append(lpp.highlightFocus);
		sb.append(";\n");
		sb.append("lpp.shadowIntensity = ");
		sb.append(lpp.shadowIntensity);
		sb.append(";\n");
		sb.append("lpp.borderSize = ");
		sb.append(lpp.borderSize);
		sb.append(";\n");
		sb.append("lpp.lightX = ");
		sb.append(lpp.lightX);
		sb.append(";\n");
		sb.append("lpp.lightY = ");
		sb.append(lpp.lightY);
		sb.append(";\n");
		sb.append("lpp.antialiasing = ");
		sb.append(lpp.antialiasing);
		sb.append(";\n");
		sb.append("lpp.color = new java.awt.Color(");
		sb.append(lpp.color.getRed());
		sb.append(", ");
		sb.append(lpp.color.getGreen());
		sb.append(", ");
		sb.append(lpp.color.getBlue());
		sb.append(");\n");

		getSourcePane().setText(sb.toString());
	}

	private void resetParameters() {
		getHlFactorSlider().setValue(85);
		getHlFocusSlider().setValue(60);
		getShadowSlider().setValue(100);
		getBorderSlider().setValue(5);
		getAliasSlider().setValue(1);
	}

	private JButton getColorButton() {
		if (colorButton == null) {
			colorButton = new JButton("Select color");
			colorButton.addActionListener(new ColorButtonListener());
		}
		return colorButton;
	}

	/**
	 * @see javax.swing.JComponent#paintComponent(Graphics)
	 */
	@Override
    protected void paintComponent(Graphics g) {
		super.paintComponent(g);
	}

	public static void main(String[] args) {
		JDialog dialog = new JDialog();

		dialog.setSize(750, 550);
		dialog.setModal(true);

		dialog.getContentPane().setLayout(new CardLayout());
		LedPaintDemo lc = new LedPaintDemo();
		dialog.getContentPane().add(lc, "");

		dialog.setVisible(true);

		System.exit(0);
	}

}
