package com.cosylab.gui.components.numberfield;

import java.text.DecimalFormatSymbols;

/**
 * 
 * Implementation of the <code>AbstractNumberDocument</code> class
 * implementing the floating point parser.
 *
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 *
 */
public class DoubleDocument extends AbstractNumberDocument
{
	private static final long serialVersionUID = 1L;

	/**
	 * Constructor for DoubleDocument.
	 */
	public DoubleDocument()
	{
		super();
	}

	/*
	 * (non-Javadoc)
	 * @see com.cosylab.iba.grail.gui.components.AbstractNumberDocument#parseNumber(java.lang.String)
	 */
	@Override
	protected Number parseNumber(String s)
	{
        if (s==null || s.length()<1) {
            return null;
        }

        double val;
        try {
        	s = s.replace(DecimalFormatSymbols.getInstance().getDecimalSeparator(),'.');
        	s = s.replace(',','.');
        	s = s.trim();
        	String lower = s.toLowerCase();
        	if (lower.endsWith("e")) {
        	    s = s.substring(0,lower.length()-1);
        	} else if (lower.endsWith("e-")) {
        	    s = s.substring(0,lower.length()-2);
        	}
        	
        	val = new Double(s);
        	if (Double.isInfinite(Math.abs(val))) {
        		return Double.MAX_VALUE;
        	}
        	if (Double.isNaN(val)) {
        		return Double.valueOf(0);
        	}
        	return val;
        } catch (NumberFormatException e) {
			return null;
		}


		}

	/*
	 * (non-Javadoc)
	 * @see com.cosylab.iba.grail.gui.components.AbstractNumberDocument#getValue()
	 */
	@Override
	public Double getValue() {
		return (Double)super.getValue();
	}
	
	/**
	 * Returns the double value parsed by this document.
	 * 
	 * @return the value
	 */
	public double getDoubleValue() {
		Number val = super.getValue();
		return val != null ? val.doubleValue() : 0;
	}
}

/* __oOo__ */
