/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.numberfield;

/**
 * Visualize double value in radians by converting it to degrees and displays
 * it in format XX\u00B0XX'XX\".
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 */
public class Rad2DegNumberDescriptor extends AngleNumberDescriptor
{
	/**
	 * @see NumberDescriptor
	 */
	public final static String NAME = "Radians as Degrees Visualization";

	/**
	 * @see NumberDescriptor
	 */
	public final static String DESCRIPTION = "Converts double value from radians to degrees and diplays in degree format: XX\u00B0XX'XX\".";
	/**
		 *
		 */
	public Rad2DegNumberDescriptor()
	{
		super();
	}

	/**
	 * Constructs a new Rad2DegNumberDescriptor.
	 *
	 * @param full should full format be used
	 * @see #setUseFullFormat(boolean)
	 */
	public Rad2DegNumberDescriptor(boolean full)
	{
		super(full);
	}

	/**
	 * Constructs a new Rad2DegNumberDescriptor.
	 *
	 * @param degreeSeparator
	 * @param minuteSeparator
	 * @param secondSeparator
	 */
	public Rad2DegNumberDescriptor(char degreeSeparator, char minuteSeparator,
	    char secondSeparator)
	{
		super(degreeSeparator, minuteSeparator, secondSeparator);
	}

	/**
	 * Constructs a new Rad2DegNumberDescriptor.
	 *
	 * @param degreeSeparator
	 * @param minuteSeparator
	 * @param secondSeparator
	 * @param full should full format be used
	 * @see #setUseFullFormat(boolean)
	 */
	public Rad2DegNumberDescriptor(char degreeSeparator, char minuteSeparator,
	    char secondSeparator, boolean full)
	{
		super(degreeSeparator, minuteSeparator, secondSeparator, full);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.numberfield.AngleNumberDescriptor#printString(java.lang.Number)
	 */
	@Override
	public String printString(Number number)
	{
		return super.printString(new Double(Math.toDegrees(number.doubleValue())));
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.numberfield.AngleNumberDescriptor#parseNumber(java.lang.String)
	 */
	@Override
	public Number parseNumber(String number)
	{
		Number n = super.parseNumber(number);

		if (n == null) {
			return n;
		}

		return new Double(Math.toRadians(n.doubleValue()));
	}
}

/* __oOo__ */
