/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.numberfield;

/**
 * Visualize double value in radians by converting it to hours and displays it
 * in format XX:XX:XX.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 */
public class Rad2HMSNumberDescriptor extends HMSNumberDescriptor
{
	/**
	 * Converts angle in radians to angle in hours. 24 hours is 2PI radians.
	 *
	 * @param rad angle in radians
	 *
	 * @return angle in hours
	 */
	public final static double toHours(double rad)
	{
		return rad * 12.0 / Math.PI;
	}

	/**
	 * Converts angle in hours to angle in radians. 24 hours is 2PI radians.
	 *
	 * @param hour angle in hours
	 *
	 * @return angle in radians
	 */
	public static double toRadians(double hour)
	{
		return hour / 12.0 * Math.PI;
	}

	/**
	 * @see NumberDescriptor
	 */
	public final static String NAME = "Radians as hours Visualization";

	/**
	 * @see NumberDescriptor
	 */
	public final static String DESCRIPTION = "Converts double value from radians to hours and diplays in degree format: XX:XX:XX.";

	/**
	 * Constructs a new Rad2HMSNNumberDescritptor.
	 *
	 */
	public Rad2HMSNumberDescriptor()
	{
		super();
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.numberfield.AngleNumberDescriptor#printString(java.lang.Number)
	 */
	@Override
	public String printString(Number number)
	{
		return super.printString(new Double(toHours(number.doubleValue())));
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.numberfield.AngleNumberDescriptor#parseNumber(java.lang.String)
	 */
	@Override
	public Number parseNumber(String number)
	{
		Number n = super.parseNumber(number);

		if (n == null) {
			return n;
		}

		return new Double(toRadians(n.doubleValue()));
	}
}

/* __oOo__ */
