/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.range2;

/**
 * This policy enforces the proper order of values. For a triplet of numbers
 * specified in <code>RangedValueHolder</code> it will rearange them (if
 * neccessary) so that minimum will be less or equal to value and value will
 * be less or equal to maximum.
 * 
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public class ProperOrderPolicy extends RangedValuePolicy {

	/**
	 * Swaps the parameters as neccessary to establis proper ordering of values.
	 * 
	 * @see com.cosylab.gui.components.range.RangedValuePolicy#validate(RangedValueHolder)
	 */
	@Override
	public RangedValue validate(RangedValue params) {
		double min = params.getMinimum();
		double max = params.getMaximum();
		double val = params.getValue();
		
		double temp;

		if (min > val) {
			temp = min;
			min = val;
			val = temp;
		}
		if (min > max) {
			temp = min;
			min = max;
			max = temp;
		}
		if (val > max) {
			temp = val;
			val = max;
			max = temp;
		}
		
		params = new RangedValue(min, max, val);
	
		return validatePeerPolicy(params);
	}
}
