/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.range2;

/**
 * TickCalculator calculates ticks for the displayer. The returned Ticks 
 * should be ready to be drawn on the displayer.
 *   
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 * @version $Id: TickCalculator.java,v 1.3 2008-04-22 12:28:40 jbobnar Exp $
 *
 */
public interface TickCalculator {

	/**
	 * Calculates ticks using some default TickParameters.
	 * 
	 * @param length the available space for ticks in the displayer
	 * @param range range which provides transformation to absolute/relative values
	 * @param rangedValue the value bounds
	 * @return
	 */
	Tick[] calculateTicks(int length, Range range, RangedValue rangedValue);
	
	/**
	 * Calculates ticks using the given TickParameters.
	 * 
	 * @param length the available space for ticks in the displayer
	 * @param measurer TickParameters used to measure ticks
	 * @param range range which provides transformation to absolute/relative values
	 * @param rangedValue the value bounds
	 * @return
	 */
	Tick[] calculateTicks(int length, TickParameters measurer, Range range, RangedValue rangedValue);
}
