/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.range2;

/**
 * Defines callback interface that is called when determining optimal tick
 * position.
 *
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public interface TickParameters
{
	/**
	 * Returns size of tick label in pixels. Size is determined as distance
	 * between maximum extent of the label along the tick axis.
	 *
	 * @param position relative position of the tick on 0..1.
	 * @param text label to display, may be empty string or null.
	 *
	 * @return size of tick in pixels.
	 */
	int measureTick(double position, String text);
	
	
	/**
	 * Formats the input value to a specific print format. If return value
	 * is null use internal format specified by the caller of this method.
	 * 
	 * @param value double to be formatted
	 * @return formatted value
	 */
	String formatNumber(double value);
}

/* __oOo__ */
