/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.table;

import java.util.Comparator;

/**
 * <code>MutableTableModelRows</code> is implementation of
 * <code>TableModelRows</code> which acts as mutable
 * <code>TabmeRowModel</code> in the same time. User can set this model to
 * <code>JTable</code> and then add or remove rows directly from this model.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $Id: MutableTableModelRows.java,v 1.5 2008-04-22 12:28:40 jbobnar Exp $
 *
 * @since May 4, 2004.
 */
public class MutableTableModelRows extends TableModelRows
{
	private static final long serialVersionUID = 1L;
	private TableRowModelSupport support;
	private Comparator comparator;

	/**
	 * Creates new instance.
	 */
	public MutableTableModelRows()
	{
		super(new TableRowModelSupport());
		support = (TableRowModelSupport)getRowModel();
	}

	/**
	 * Inserts row at the specified index.
	 *
	 * @param index the index where to insert
	 * @param row the row to be added
	 */
	public synchronized void addRow(int index, TableRow row)
	{
		support.addRow(index, row);
	}

	/**
	 * Adds row to the end of the row list.
	 *
	 * @param row the row to be added
	 */
	public synchronized void addRow(TableRow row)
	{
		support.addRow(row);
	}

	/**
	 * Returns row at given index,
	 *
	 * @param index the index of  the row
	 *
	 * @return the row at the specified index
	 */
	public synchronized TableRow getRow(int index)
	{
		return support.getRow(index);
	}

	/**
	 * Removes all rows from this model.
	 */
	public synchronized void removeAllRows()
	{
		support.removeAllRows();
	}

	/**
	 * Removes row from this model.
	 *
	 * @param index rowindex to be roemoved
	 *
	 * @return the removed row
	 */
	public synchronized TableRow removeRow(int index)
	{
		return support.removeRow(index);
	}

	/**
	 * Removes row from this model.
	 *
	 * @param row the row to be removed
	 */
	public synchronized void removeRow(TableRow row)
	{
		support.removeRow(row);
	}

	/**
	 * Sets the number of columns this model presents.
	 *
	 * @param count the number of columns
	 */
	public void setColumnCount(int count)
	{
		support.setColumnCount(count);
	}

	/**
	 * Sets column names.
	 *
	 * @param names the column names
	 */
	public void setColumnNames(String[] names)
	{
		support.setColumnNames(names);
	}

	/**
	 * @return the comparator
	 */
	public Comparator getComparator() {
		return comparator;
	}

	/**
	 * @param comparator the comparator to set
	 */
	public void setComparator(Comparator comparator) {
		this.comparator = comparator;
		support.setComparator(comparator);
	}
}

/* __oOo__ */
