/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.table;

/**
 * <code>ObjectTableEngine</code> is intermediator betwean arbitrary objects
 * and <code>ObjectTableModel</code> structure. It converts objects to
 * <code>TableRow</code>s and executes operations on rows.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 */
public interface ObjectTableEngine
{
	/**
	 * Creates new row for key object.
	 *
	 * @param key key for which row is created
	 *
	 * @return new or recycled table row
	 */
	public TableRow getTableRow(Object key);

	/**
	 * Signals to factory that row with associated key is no longer in use.
	 *
	 * @param key kay object for row
	 * @param row row to be released
	 */
	public void releaseTableRow(Object key, TableRow row);

	/**
	 * Sets new value which resulted from user interaction with table to the
	 * context underlayng kay object.
	 *
	 * @param value new value as user has set with table editors
	 * @param key the key object
	 * @param row row associated with kay object
	 * @param columnIndex DOCUMENT ME!
	 */
	public void setValue(Object value, Object key, TableRow row, int columnIndex);

	/**
	 * Returns name of column. Same order  of columns must be used
	 * when construction rows.
	 *
	 * @return name of column, or <code>null</code>
	 */
	public String getColumnName(int index);
}

/* __oOo__ */
