/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.table;

/**
 * <code>TableRowModel</code> is table model which natural organization of data
 * is grouped by rows. This model can not be used directly in
 * <code>JTable</code>, it must be used in collaboration with
 * <code>TableModelRows</code> or similar implementation of
 * <code>TableModel</code>.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $Id: TableRowModel.java,v 1.4 2008-04-22 12:28:40 jbobnar Exp $
 *
 * @since May 3, 2004.
 */
public interface TableRowModel
{
	/**
	 * Number of columns inside this model.
	 *
	 * @return number of columns inside this model
	 */
	public int getColumnCount();

	/**
	 * Returns number of rows contained in this model.
	 *
	 * @return number of  rows
	 */
	public int getRowCount();

	/**
	 * Returns a row object at row index.
	 *
	 * @param rowIndex the row index
	 *
	 * @return a row at the row index
	 */
	public TableRow getRowAt(int rowIndex);

	/**
	 * Returns index of a row.
	 *
	 * @param row a row form this model
	 *
	 * @return the index ot the row
	 */
	public int getRowIndex(TableRow row);

	/**
	 * Add listener, which receives notification of changes in the model.
	 *
	 * @param l a <code>TableRowModelListener</code> listener
	 */
	public void addTableRowModelListener(TableRowModelListener l);

	/**
	 * Removes listener.
	 *
	 * @param l a <code>TableRowModelListener</code> listener
	 */
	public void removeTableRowModelListener(TableRowModelListener l);

	/**
	 * Returns name of column.
	 *
	 * @param index the index of  the colummn
	 *
	 * @return the name of the column under provided index
	 *
	 * @see javax.swing.table.TableModel#getColumnName(int)
	 */
	public String getColumnName(int index);

	/**
	 * This method is called from table row, when wants to notify its parent
	 * model about value change.
	 *
	 * @param row the row which value has changes
	 * @param valueIndex the index of the value, if <code>-1</code> all values
	 *        has been changed
	 */
	public void fireRowUpdate(TableRow row, int valueIndex);

	/**
	 * Sort rows in ascending or descengin order according to the natural order
	 * of cells in specified column. Natural order is determined by cells,
	 * which implements <code>Comparable</code>.
	 *
	 * @param column the column index, according to which rows are sorted
	 * @param descending sorted rows are ordered ascending if
	 *        <code>false</code>, or descending if <code>true</code>.
	 */
	public void sortRows(int column, boolean descending);
}

/* __oOo__ */
