/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.table.cells;

import com.cosylab.util.CommonException;


/**
 * Common exception that occurs during command execution.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $id$
 */
public class CommandException extends CommonException
{
	private static final long serialVersionUID = 1L;
	private Command command;

	/**
	 * CommandException constructor comment.
	 *
	 * @param instance cell which generated exception
	 * @param c command
	 * @param message exception message
	 * @param cause cause
	 */
	public CommandException(TableCell instance, Command c, String message,
	    Throwable cause)
	{
		super(instance, message, cause);
		command = c;
	}

	/**
	 * Return command.
	 *
	 * @return command
	 */
	public Command getCommand()
	{
		return command;
	}

	/**
	 * Returns the cell that command was executed on.
	 *
	 * @return the comman owner
	 */
	public TableCell getTableCell()
	{
		return (TableCell)getInstance();
	}
}

/* __oOo__ */
