/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.table.cells;

import com.cosylab.util.NumericStringComparator;

/**
 * DeviceCell holds place for device: an entitiy providion identifier and
 * possibly commands.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 */
public class DeviceCell extends TableCell
{
	
	private boolean highlightCell;
	
	/**
	 * Constructs a new DeviceCell.
	 *
	 */
	public DeviceCell()
	{
		super();
	}

	/**
	 * Constructs a new DeviceCell.
	 *
	 * @param identifier
	 * @param dataSource
	 * @param value
	 * @param commands
	 */
	public DeviceCell(String identifier, Object dataSource, Object value,
	    Command[] commands)
	{
		super(identifier, dataSource, value, commands);
	}

	/**
	 * Returns identifier.
	 *
	 * @see com.cosylab.gui.components.table.cells.TableCell#toString()
	 */
	@Override
    public String toString()
	{
		return getIdentifier();
	}

	NumericStringComparator numericStringComparator = new NumericStringComparator();

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.table.cells.TableCell#compareTo(java.lang.Object)
	 */
	@Override
    public int compareTo(Object o)
	{
		if (o instanceof DeviceCell) {
			return numericStringComparator.compare(getIdentifier(),((DeviceCell)o).getIdentifier());
		}

		return super.compareTo(o);
	}

	/**
	 * Returnst true if this cell is highlighted (selected).
	 * 
	 * @return true if highlighted
	 */
	public boolean isHighlightCell() {
		return highlightCell;
	}

	/**
	 * Highlights this cell. This method does not fire any events. Upon changing the 
	 * highlight property the cell should be repainted.
	 * 
	 * @param cellSelected
	 */
	public void setHighlightCell(boolean cellSelected) {
		this.highlightCell = cellSelected;
	}
}

/* __oOo__ */
