/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.table.cells;

/**
 * Numeric cell holds numeric value: double or long.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 */
public abstract class NumericCell extends TableCell
{
	/** Characteristic name for string describing ANSI type of format. */
	public static final String FORMAT = "format";

	/** Characteristic name for string describing physical units of value. */
	public static final String UNITS = "units";

	/** Characteristic name for string describing minimal allowed value. */
	public static final String MINIMUM = "minimum";

	/** Characteristic name for string describing maximal allowed value. */
	public static final String MAXIMUM = "maximum";

	/** Characteristic name for value defining if units should be visible. */
	public static final String UNITS_VISIBLE = "unitsVisible";
	
	/**
	 * Constructs a new NumericCell.
	 *
	 */
	public NumericCell()
	{
		super();
	}

	/**
	 * Constructs a new NumericCell.
	 *
	 * @param identifier
	 * @param dataSource
	 * @param value
	 * @param commands
	 */
	public NumericCell(String identifier, Object dataSource, Object value,
	    Command[] commands)
	{
		super(identifier, dataSource, value, commands);
	}

	/**
	 * Returns value of FORMAT characteristic.
	 *
	 * @return value of FORMAT characteristic
	 */
	public String getFormat()
	{
		return (String)getCharacteristic(FORMAT);
	}

	/**
	 * Returns value of UNITS characteristic.
	 *
	 * @return value of UNITS characteristic
	 */
	public String getUnits()
	{
		return (String)getCharacteristic(UNITS);
	}
	
	/**
	 * Returns a flag if units are visible.
	 *  
	 * @return
	 */
	public boolean isUnitsVisible() {
		Boolean b = (Boolean) getCharacteristic(UNITS_VISIBLE);
		if (b == null) return false;
		else return b;
	}

	/**
	 * Returns value of MINIMUM characteristic.
	 *
	 * @return value of MINIMUM characteristic
	 */
	public Number getMinimum()
	{
		return (Number)getCharacteristic(MINIMUM);
	}

	/**
	 * Returns value of MINIMUM characteristic.
	 *
	 * @param defaultValue DOCUMENT ME!
	 *
	 * @return value of MINIMUM characteristic
	 */
	public Number getMinimum(Number defaultValue)
	{
		return (Number)getCharacteristic(MINIMUM, defaultValue);
	}

	/**
	 * Returns value of MAXIMUM characteristic.
	 *
	 * @return value of MAXIMUM characteristic
	 */
	public Number getMaximum()
	{
		return (Number)getCharacteristic(MAXIMUM);
	}

	/**
	 * Returns value of MAXIMUM characteristic.
	 *
	 * @param defaultValue DOCUMENT ME!
	 *
	 * @return value of MAXIMUM characteristic
	 */
	public Number getMaximum(Number defaultValue)
	{
		return (Number)getCharacteristic(MAXIMUM, defaultValue);
	}
}

/* __oOo__ */
