/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.table.cells;

import java.awt.Color;


/**
 * PatternCell describes pattern of bits.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $id$
 */
public class PatternCell extends LongCell
{
	/**
	 * Constructs a new PatternCell.
	 *
	 */
	public PatternCell()
	{
		super();
	}

	/**
	 * Constructs a new PatternCell.
	 *
	 * @param identifier
	 * @param dataSource
	 * @param value
	 * @param commands
	 */
	public PatternCell(String identifier, Object dataSource, long value,
	    Command[] commands)
	{
		super(identifier, dataSource, value, commands);
	}

	/**
	 * Characteristic name for array with colors which are used when bit in
	 * value is 1.
	 */
	public static final String COLORS_WHEN_ON = "colorsWhenOn";

	/**
	 * Characteristic name for array with colors which are used when bit in
	 * value is 0.
	 */
	public static final String COLORS_WHEN_OFF = "colorsWhenOff";

	/**
	 * Characteristic  name for array with description strings for each pattern
	 * bit.
	 */
	public static final String DESCRIPTIONS = "descriptions";

	/**
	 * Returns value of COLORS_WHEN_ON characteristic.
	 *
	 * @return value of COLORS_WHEN_ON characteristic
	 */
	public Color[] getColorsWhenOn()
	{
		return (Color[])getCharacteristic(COLORS_WHEN_ON);
	}

	/**
	 * Returns value of COLORS_WHEN_OFF characteristic.
	 *
	 * @return value of COLORS_WHEN_OFF characteristic
	 */
	public Color[] getColorsWhenOff()
	{
		return (Color[])getCharacteristic(COLORS_WHEN_OFF);
	}

	/**
	 * Returns value of DESCRIPTIONS characteristic.
	 *
	 * @return value of DESCRIPTIONS characteristic
	 */
	public String[] getDescriptions()
	{
		return (String[])getCharacteristic(DESCRIPTIONS);
	}
}

/* __oOo__ */
