/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.table.renderers;

import java.awt.Component;

import javax.swing.JCheckBox;
import javax.swing.JTable;
import javax.swing.table.TableCellRenderer;

import com.cosylab.gui.components.table.cells.BooleanCell;

/**
 * This is the default renderer for the ObjectTable cells implementing 
 * BooleanCell. Is is rendered as a checkbox that can be in two states.
 * Since it is a renderer, the event handling for changing the states must
 * be provided at ObjectTable level.
 * 
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>  
 * @version $id$
 */
public class BooleanCellRenderer
	extends JCheckBox
	implements TableCellRenderer {

	private static final long serialVersionUID = 1L;

	/**
	 * Default constructor for BooleanTableCellRenderer.
	 */
	public BooleanCellRenderer() {
		super();
		setOpaque(true);
	}

	/*
	 * (non-Javadoc)
	 * @see javax.swing.table.TableCellRenderer#getTableCellRendererComponent(javax.swing.JTable, java.lang.Object, boolean, boolean, int, int)
	 */
	public Component getTableCellRendererComponent(
		JTable table,
		Object booleanTableCell,
		boolean isSelected,
		boolean hasFocus,
		int row,
		int column) {

		BooleanCell cellProperty = (BooleanCell) booleanTableCell;
		setForeground(table.getForeground());
		setBackground(table.getBackground());

		setSelected(cellProperty.getBooleanValue());
		return this;
	}
}
