/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.table.renderers;

import com.cosylab.gui.components.numberfield.DoubleDocument;
import com.cosylab.gui.components.table.cells.DoubleCell;

import com.cosylab.util.FormatCache;
import com.cosylab.util.PrintfFormat;
import com.cosylab.util.StringUtilities;

import java.awt.Component;

import java.text.DecimalFormat;

import java.util.EventObject;

import javax.swing.JTable;


/**
 * Edits DoubleCell value.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $id$
 */
public class DoubleCellEditor extends JTextFieldCellEditor
{
	private static final long serialVersionUID = 1L;
	private DoubleDocument document;
	private boolean active;
	private DecimalFormat f = new DecimalFormat("#.####################");

	/**
	 * Creates a new DoubleCellEditor object.
	 */
	public DoubleCellEditor()
	{
		setDocument(document = new DoubleDocument());
	}

	/*
	 * (non-Javadoc)
	 * @see com.cosylab.gui.components.table.renderers.JTextFieldCellEditor#getTableCellEditorComponent(javax.swing.JTable, java.lang.Object, boolean, int, int)
	 */
	@Override
    public Component getTableCellEditorComponent(JTable table, Object cell,
	    boolean isSelected, int row, int column)
	{
		if (cell instanceof DoubleCell) {
			active = !((DoubleCell)cell).isSuspended();

			document.setMaxValue(((DoubleCell)cell).getMaximum());
			document.setMinValue(((DoubleCell)cell).getMinimum());

			double d = ((DoubleCell)cell).getDoubleValue();

			if (d < 1e20 && d > 1e-20) {
				if (active) {
					setText(f.format(d));
				} else {
					setText("");
				}
			} else {
				PrintfFormat formatter = FormatCache.getFormatter(((DoubleCell)cell)
					    .getFormat());

				if (formatter == null) {
					formatter = FormatCache.getDefaultFormatter();
				}

				if (active) {
					setText(formatter.sprintf(d));
				} else {
					setText("");
				}
			}

			selectAll();
		}

		return this;
	}

	/* (non-Javadoc)
	 * @see javax.swing.DefaultCellEditor#getCellEditorValue()
	 */
	public Object getCellEditorValue()
	{
		return document.getValue();
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.table.renderers.JTextFieldCellEditor#isValueValid()
	 */
	@Override
    protected boolean isValueValid()
	{
		return StringUtilities.isTrueNumber(getText());
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.table.renderers.JTextFieldCellEditor#isCellEditable(java.util.EventObject)
	 */
	@Override
    public boolean isCellEditable(EventObject anEvent)
	{
		//	    if (!active) {
		//	        return false;
		//		}
		return super.isCellEditable(anEvent);
	}

	public static void main(String[] args)
	{
		PrintfFormat formatter = FormatCache.getFormatter("%.f");

		System.out.println(formatter.sprintf(1e-20));

		DecimalFormat f = new DecimalFormat("#.####################");

		System.out.println(f.format(1e20));
	}
}

/* __oOo__ */
