/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.table.renderers;

import com.cosylab.gui.components.table.cells.DoubleCell;

import com.cosylab.util.FormatCache;
import com.cosylab.util.PrintfFormat;

import java.awt.Component;

import javax.swing.JTable;


/**
 * This is the default renderer for ObjectTable cells implementing DoubleCell
 * interface.
 *
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public class DoubleCellRenderer extends DefaultTableCellRenderer
{
	
	private static final long serialVersionUID = 1L;

	/**
	 * Constructs a new DoubleCellRenderer. 
	 */
	public DoubleCellRenderer() {
		this(false,true);
	}
	
	/**
	 * Constructs a new DoubleCellRenderer.
	 * 
	 * @param decorateBorder if true decorated border will be used
	 * @param decorateBackground if true decorated background will be used
	 */
	public DoubleCellRenderer(boolean decorateBorder, boolean decorateBackground)
	{
		super(decorateBorder,decorateBackground);
		setOpaque(true);
	}

	/*
	 * (non-Javadoc)
	 * @see com.cosylab.gui.components.table.renderers.DefaultTableCellRenderer#getTableCellRendererComponent(javax.swing.JTable, java.lang.Object, boolean, boolean, int, int)
	 */
	@Override
    public Component getTableCellRendererComponent(JTable table, Object cell,
	    boolean isSelected, boolean hasFocus, int row, int column)
	{
		super.getTableCellRendererComponent(table, cell, isSelected, hasFocus,
		    row, column);

		if (cell instanceof DoubleCell) {
			if (((DoubleCell)cell).isSuspended()) {
				setText("");
			} else {
				PrintfFormat formatter = FormatCache.getFormatter(((DoubleCell)cell)
					    .getFormat());

				if (formatter == null) {
					formatter = FormatCache.getDefaultFormatter();
				}
				
				String value = formatter.sprintf(((DoubleCell)cell).getDoubleValue());
				if (((DoubleCell)cell).isUnitsVisible()) {
					value += " " + ((DoubleCell)cell).getUnits();
				}
				setText(value);
			}
		}

		return this;
	}
}

/* __oOo__ */
