/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.table.renderers;

import java.awt.Component;

import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.table.TableCellRenderer;

/**
 * This renderer is used to display cells whose contents are null. It accepts no
 * parameters an is rendered as a blank cell. Purpose of this class is to 
 * simplify drawing of cells when there could be null contents in the 
 * TableModel.
 * 
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>  
 * @version $id$
 */
public class EmptyCellRenderer extends JLabel implements TableCellRenderer {

	private static final long serialVersionUID = 1L;
	/**
	 * Default constructor for EmptyCellRenderer.
	 */
	public EmptyCellRenderer() {
		setOpaque(true);
	}

	/*
	 * (non-Javadoc)
	 * @see javax.swing.table.TableCellRenderer#getTableCellRendererComponent(javax.swing.JTable, java.lang.Object, boolean, boolean, int, int)
	 */
	public Component getTableCellRendererComponent(
		JTable table,
		Object cell,
		boolean isSelected,
		boolean hasFocus,
		int row,
		int column) {

		if (isSelected) {
			setBackground(table.getSelectionBackground());
			setForeground(table.getSelectionForeground());
		} else {
			setBackground(table.getBackground());
			setForeground(table.getForeground());
		}


		return this;			
	}

}
