/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.table.renderers;

import com.cosylab.gui.components.table.cells.LongEnumCell;

import javax.swing.AbstractListModel;
import javax.swing.ComboBoxModel;


class EnumComboBoxModel extends AbstractListModel implements ComboBoxModel
{
	private static final long serialVersionUID = 1L;
	LongEnumCell cell;
	int min = -1;
	int max = 0;
	String selected;

	public EnumComboBoxModel()
	{
		super();
	}

	/* (non-Javadoc)
	 * @see javax.swing.ListModel#getElementAt(int)
	 */
	public Object getElementAt(int index)
	{
		if (cell == null) {
			return null;
		}

		return cell.getStateLabel(index - min);
	}

	/* (non-Javadoc)
	 * @see javax.swing.ListModel#getSize()
	 */
	public int getSize()
	{
		return max - min + 1;
	}

	/* (non-Javadoc)
	 * @see javax.swing.ComboBoxModel#getSelectedItem()
	 */
	public Object getSelectedItem()
	{
		return selected;
	}

	/* (non-Javadoc)
	 * @see javax.swing.ComboBoxModel#setSelectedItem(java.lang.Object)
	 */
	public void setSelectedItem(Object anItem)
	{
		selected = (String)anItem;
	}

	public void setCell(LongEnumCell c)
	{
		cell = c;
		selected = null;

		int size = getSize();
		max = 0;
		min = -1;

		if (size > 0) {
			fireIntervalRemoved(this, 0, size);
		}

		if (cell == null) {
			return;
		}

		min = 15;
		max = 0;

		for (int i = 0; i < 16; i++) {
			Object o = cell.getCharacteristic(LongEnumCell.STATE_DESCRIPTION
				    + i);

			if (o !=null ) {
				if (i < min) {
					min = i;
				}

				if (i > max) {
					max = i;
				}
			}
		}

		fireIntervalAdded(this, 0, getSize());
	}

	public Long getLongVale()
	{
		if (selected == null || cell == null) {
			return null;
		}

		return new Long(cell.toLongValue(selected));
	}
}

/* __oOo__ */
