/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.table.renderers;

import com.cosylab.gui.components.table.cells.LongEnumCell;

import java.awt.Component;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.DefaultCellEditor;
import javax.swing.JComboBox;
import javax.swing.JTable;


/**
 * <code>LongEnumCellRenderer</code> is the editor for the LongEnumCell.
 *
 * @author <a href="mailto:jernej.kamenik@cosylab.com">Jernej Kamenik</a>
 * @version $Id: LongEnumCellEditor.java,v 1.6 2008-04-22 12:28:41 jbobnar Exp $
 *
 * @since May 8, 2004.
 */
public class LongEnumCellEditor extends DefaultCellEditor
{
	private static final long serialVersionUID = 1L;
	EnumComboBoxModel comboModel;

	/**
	 * Creates a new DiamondStatusCellEditor object.
	 */
	public LongEnumCellEditor()
	{
		super(new JComboBox(new EnumComboBoxModel()));
		
		getComponent().addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent e) {
			}

			public void mouseEntered(MouseEvent e) {
			}

			public void mouseExited(MouseEvent e) {
				cancelCellEditing();
			}

			public void mousePressed(MouseEvent e) {
			}

			public void mouseReleased(MouseEvent e) {
			}
		});
	}

	/* (non-Javadoc)
	 * @see javax.swing.DefaultCellEditor#getTableCellEditorComponent(javax.swing.JTable, java.lang.Object, boolean, int, int)
	 */
	@Override
    public Component getTableCellEditorComponent(JTable table, Object value,
	    boolean isSelected, int row, int column)
	{
		JComboBox box = (JComboBox)super.getTableCellEditorComponent(table,
			    value, isSelected, row, column);

		if (comboModel == null) {
			comboModel = (EnumComboBoxModel)box.getModel();
		}

		if (value instanceof LongEnumCell) {
			LongEnumCell cell = (LongEnumCell)value;
			comboModel.setCell(cell);
		} else {
			comboModel.setCell(null);
		}

		return box;
	}

	/* (non-Javadoc)
	 * @see javax.swing.DefaultCellEditor#getCellEditorValue()
	 */
	@Override
    public Object getCellEditorValue()
	{
		return comboModel.getLongVale();
	}
}

/* __oOo__ */
