/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.table.renderers;

import com.cosylab.gui.components.table.cells.PatternCell;
import com.cosylab.gui.components.util.PaintHelper;
import com.cosylab.util.BitSetUtilities;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.event.MouseEvent;

import javax.swing.JTable;
import javax.swing.ToolTipManager;


/**
 * Renderer for <code>com.cosylab.gui.components.table.cells.PatternCell</code>
 * cells. Divides available area into horizontal regions and displays values
 * as rectangular areas matching the color specified in <code>PatternCell
 * </code>.
 *
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public class PatternCellRenderer extends DefaultTableCellRenderer
{
	private static final long serialVersionUID = 1L;
	protected TimeoutBorder border = new TimeoutBorder();
	private Point position = null;
	private int size;
	private int h;
	private int w;
	private long value;
	private Color[] colorsOn;
	private Color[] colorsOff;
	private String[] descriptions;
	public static final Color DEFAULT_ON = Color.GREEN;
	public static final Color DEFAULT_OFF = Color.GRAY;

	/**
	 * Creates a new PatternCellRenderer object.
	 */
	public PatternCellRenderer()
	{
		super(true, false);
		setToolTipText("");
		w = getWidth();
		ToolTipManager.sharedInstance().setInitialDelay(0);
		ToolTipManager.sharedInstance().setDismissDelay(1000);

		//setBorder(border);
	}

	/*
	 * (non-Javadoc)
	 * @see com.cosylab.gui.components.table.renderers.DefaultTableCellRenderer#getTableCellRendererComponent(javax.swing.JTable, java.lang.Object, boolean, boolean, int, int)
	 */
	@Override
    public Component getTableCellRendererComponent(JTable table,
	    Object patternCell, boolean isSelected, boolean hasFocus, int row,
	    int column)
	{
		super.getTableCellRendererComponent(table, patternCell, isSelected,
		    hasFocus, row, column);

		if (patternCell instanceof PatternCell) {
			PatternCell cell = (PatternCell)patternCell;
			descriptions = cell.getDescriptions();
			value = cell.getLongValue();
			
			if (descriptions == null) {
				size = BitSetUtilities.forLong(value).length();
				if (size == 0) size = 1;
			} else {
				size = descriptions.length;
			}
			
			colorsOn = cell.getColorsWhenOn();
			colorsOff = cell.getColorsWhenOff();
	
			//border.setEnabled(cell.isTimeout());
				
			setText("");
		}
		return this;
	}

	/**
	 * Returns tool tip text based on area the mouse is over.
	 *
	 * @param e event
	 *
	 * @return description of selected led.
	 */
	@Override
    public String getToolTipText(MouseEvent e)
	{
		position = e.getPoint();

		for (int i = 0; i < size; i++) {
			if (position.getX() > (i * w) / size
			    && position.getX() < ((i + 1) * w) / size) {
				if (descriptions != null) {
					return descriptions[i];
				} else {
					return String.valueOf(i);
				}
			}
		}

		return null;
	}

	/**
	 * Paints the component.
	 *
	 * @param g graphics
	 */
	@Override
    public void paintComponent(Graphics g)
	{
		paintSuperComponent(g);

		if (active) {
			w = getWidth() - 1;
			h = getHeight() - 1;

			for (int i = 0; i < size; i++) {
				if ((value & 1) == 1) {
					if (colorsOn != null && colorsOn.length > i) {
						g.setColor(colorsOn[i]);
					} else {
						g.setColor(DEFAULT_ON);
					}					
				} else {
					if (colorsOn != null && colorsOn.length > i) {
						g.setColor(colorsOff[i]);
					} else {
						g.setColor(DEFAULT_OFF);
					}
				}

				value >>= 1;
				int bin= w/size+1;
				g.fillRect((i * w) / size, 0, bin, h);
				g.setColor(Color.WHITE);
				g.drawRect((i * w) / size, 0, bin, h);
			}
		}

		paintDecorations(g);

		if (alarm) {
				PaintHelper.paintRectangle(g, 2, 2, getWidth()-5, getHeight()-5,
				    Color.WHITE, 1);
		}
}
}

/* __oOo__ */
