/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.wheelswitch;



/**
 * Formats <code>java.lang.String</code>s to be displayed as digits in  the
 * <code>Wheelswitch</code>. The format is specified in a format
 * <code>java.lang.String</code>. <code>WheelswitchFormatter</code> also
 * stores the value, its bounds (minimum and maximum) and also the unit
 * displayed by the <code>Wheelswitch</code>.
 *
 * @author <a href="mailto:jernej.kamenik@cosylab.com">Jernej Kamenik</a>
 * @version $id$
 */
public class WheelswitchFormatter extends AbstractWheelswitchFormatter
{
	/**
	 * Constructs the PlainWheelswitchFormatter and sets the format  string.
	 *
	 * @param newFormatString
	 *
	 * @see #setFormat(String)
	 */
	public WheelswitchFormatter(String newFormatString)
	{
		super(newFormatString);
	}

	/**
	 * Constructs the PlainWheelswitchFormatter with no format string.
	 *
	 * @see #setFormat(String)
	 */
	public WheelswitchFormatter()
	{
		super();
	}

	@Override
	protected void internalSetString(String newValueString) {
		if (debug) System.out.println("WheelswitchFormatter:SET STRING VALUE: " + newValueString);		
		
		double newValue = Double.parseDouble(newValueString);
		String tempFormatString = formatString;

		if (formatString == null) {
			tempFormatString = generateFormat();
		}

		int newExpIndex = Math.max(newValueString.indexOf("E"),
				newValueString.indexOf("e"));

		if (newExpIndex < 0) {
			newExpIndex = newValueString.length();
		}

		int formatExpIndex = tempFormatString.indexOf("E");

		if (formatExpIndex < 0) {
			formatExpIndex = tempFormatString.length();
		}

		int newDotIndex = Math.max(newValueString.indexOf("."),
				newValueString.indexOf(","));

		if (newDotIndex < 0) {
			newDotIndex = newExpIndex;
		}

		int formatDotIndex = tempFormatString.indexOf(".");

		if (formatDotIndex < 0) {
			formatDotIndex = formatExpIndex;
		}

		int newSignTag = 0;

		if (newValueString.charAt(0) == '+') {
			newSignTag++;
		} else if (newValueString.charAt(0) == '-') {
			newSignTag--;
		}

		int formatSignTag = 0;

		if (tempFormatString.charAt(0) == '+') {
			formatSignTag++;
		}
		
		int formatExpSignTag = 0;

		if (formatExpIndex + 1 < tempFormatString.length()
			&& tempFormatString.charAt(formatExpIndex + 1) == '+') {
			formatExpSignTag++;
		}

		int newExpValue = 0;

		if (newExpIndex < newValueString.length() - 1) {
			newExpValue += (int)Double.parseDouble(newValueString.substring(newExpIndex
					+ 1));
		}
		
		//sync the exp value with format
		newExpValue += newDotIndex - formatDotIndex;

		if (newSignTag != 0) {
			newExpValue--;
		}

		if (formatSignTag != 0) {
			newExpValue++;
		}

		//display mantissa as integer
		newValueString = newValueString.substring(0, newDotIndex)
			+ ((newDotIndex < newExpIndex - 1)
			? (newValueString.substring(newDotIndex + 1, newExpIndex)) : (""));

		if (newSignTag != 0) {
			newValueString = newValueString.substring(1);
		}

		boolean substract = true;
		while (newValueString.charAt(0) == '0' && newValueString.length() > 1) {
			newValueString = newValueString.substring(1);
			//bugfix by jbobnar: If the value is 0 don't do any manipulation, since 
			//this would change what the value that is being set
			if (newValue != 0.) {
				newExpValue--;
			}
			substract = false;
		}

		if (newValue == 0. && substract) {
			newExpValue+=newExpIndex-3;
		}
		
		// Ike: Check if eponent over double range
		// TODO: this does not eliminates problem if decimal in whole value over range, even if exp is in range.
		if (newExpValue<-323) {
			newExpValue=-323;
		} else {
			if (newExpValue>307) {
				newExpValue=307;
			}
		}

		//sync mantissa with format
		while (newValueString.length() < formatExpIndex - formatSignTag
			- ((formatDotIndex < formatExpIndex) ? (1) : (0))) {
			newValueString += "0";
		}

		//if exponent should be displayed
		if (formatExpIndex + formatExpSignTag + 1 < tempFormatString.length()) {
			//parse exponent
			String exponent = String.valueOf(newExpValue);

			int newExpSignTag = 1;

			if (newExpValue < 0) {
				newExpSignTag = -1;
				exponent = exponent.substring(1);
			}

			//parse mantissa
			newValueString = newValueString.substring(0,
					formatDotIndex - formatSignTag) + "."
				+ newValueString.substring(formatDotIndex - formatSignTag);

			if (newValue != 0.) {
				newValueString = formatter.sprintf(Double.parseDouble(
							newValueString)).replaceAll(",", ".");

				if (newValueString.charAt(0) == '+') {
					newValueString = newValueString.substring(1);
				}
			}

			if (newSignTag == -1) {
				newValueString = "-" + newValueString;
			} else if (formatSignTag == 1) {
				newValueString = "+" + newValueString;
			}

			newValueString += "E";

			//add exponent
			if (newExpSignTag == -1) {
				newValueString += "-";
			} else if (formatExpSignTag == 1) {
				newValueString += "+";
			}

			int exponentLength = tempFormatString.length() - formatExpIndex
				- formatExpSignTag - 1;

			while (exponent.length() < exponentLength) {
				exponent = "0" + exponent;
			}

			newValueString += exponent;
		} else {
			//parse mantissa only
			while (newExpValue > 0) {
				newValueString += "0";
				formatDotIndex++;
				newExpValue--;
			}

			while (newExpValue < 0) {
				newValueString = "0" + newValueString;
				newExpValue++;
			}

			newValueString = newValueString.substring(0,
					formatDotIndex - formatSignTag) + "."
				+ newValueString.substring(formatDotIndex - formatSignTag);

			if (newValueString.charAt(0) != '0') {
				newValueString = formatter.sprintf(Double.parseDouble(
							newValueString)).replaceAll(",", ".");

				if (newValueString.charAt(0) == '+') {
					newValueString = newValueString.substring(1);
				}
			} else {
				newValueString = "1" + newValueString.substring(1);
				newValueString = formatter.sprintf(Double.parseDouble(
							newValueString)).replaceAll(",", ".");

				if (newValueString.charAt(0) == '+') {
					newValueString = newValueString.substring(1);
				}

				newValueString = "0" + newValueString.substring(1);
			}

			if (newSignTag == -1) {
				newValueString = "-" + newValueString;
			} else if (formatSignTag == 1) {
				newValueString = "+" + newValueString;
			}
		}

		value = newValue;
		valueString = newValueString;	
	}
}

/* __oOo__ */
