/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans.
 *
 * CosyBeans is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.displayers;

import java.beans.PropertyChangeListener;


/**
 * <code>DataStateProvider</code> interface marks those components, which can
 * provide <code>DataState</code> Java property to describe theier data
 * quality state.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 *
 * @see com.cosylab.gui.displayers.DataState
 */
public interface DataStateProvider
{
	/** Property name for data state. */
	public String DATA_STATE = "dataState";

	/**
	 * Adds property change listener, which receives notifications when data
	 * state changes.
	 *
	 * @param l new listener
	 */
	public void addPropertyChangeListener(PropertyChangeListener l);

	/**
	 * Adds property change listener, which receives notifications when data
	 * state changes.
	 *
	 * @param propertyName the property name for which events will be received
	 * @param l new listener
	 */
	public void addPropertyChangeListener(String propertyName,
	    PropertyChangeListener l);

	/**
	 * Returns data quality state. This property is changed by data consumer
	 * interface. In case of multiple displayer, this state indicates integral
	 * state for all contained data consumers.
	 *
	 * @return dynamic data quality state
	 */
	public DataState getDataState();

	/**
	 * Removes property change listener.
	 *
	 * @param l removed listener
	 */
	public void removePropertyChangeListener(PropertyChangeListener l);

	/**
	 * Removes property change listener.
	 *
	 * @param propertyName the property name for which listener is removed
	 * @param l removed listener
	 */
	public void removePropertyChangeListener(String propertyName,
	    PropertyChangeListener l);
}

/* __oOo__ */
