/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans.
 *
 * CosyBeans is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.infopanel;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.DefaultListModel;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;

import com.cosylab.gui.adapters.Converter;
import com.cosylab.gui.adapters.ConverterChain;
import com.cosylab.gui.adapters.ConverterUtilities;
import com.cosylab.gui.displayers.DataSource;

/**
 * <code>ConverterPanel</code> shows information about the converters and  
 *  data sources.
 * 
 * @author Jaka Bobnar, Cosylab
 *
 */
public class ConverterPanel extends JPanel {
	private static final long serialVersionUID = -7953031747105924874L;
	private Converter converter;
	private DataSource dataSource;
	private JLabel functionLabel;
	private JLabel dataSourceLabel;
	private JLabel dataSourceL;
	private JList converterList;
	private DefaultListModel model;
	
	/**
	 * Constructs a new ConverterPanel.
	 *
	 */
	public ConverterPanel() {
		super();
		initialize();
	}
	
	private void initialize() {
		this.setLayout(new BorderLayout());
	
		dataSourceLabel = new JLabel("Data Source:");
		dataSourceL = new JLabel("N/A");
		JPanel northPanel = new JPanel(new GridBagLayout());
		northPanel.add(dataSourceLabel, new GridBagConstraints(0,0,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1,1,1,1),0,0));
		northPanel.add(dataSourceL, new GridBagConstraints(1,0,1,1,1,0,GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, new Insets(1,1,1,1),0,0));
		this.add(northPanel, BorderLayout.NORTH);
		
		converterList = new JList();
		model = new DefaultListModel();
		converterList.setModel(model);
		this.add(converterList, BorderLayout.CENTER);
		functionLabel = new JLabel("id");
		this.add(functionLabel, BorderLayout.SOUTH);
	}
	
	/**
	 * Sets the converter, which data should be presented in the info panel.
	 * 
	 * @param c
	 */
	public void setConverter(Converter c) {
		this.converter = c;
		if (converter != null) {
			functionLabel.setText("f(x) = " + ConverterUtilities.composeFunctionString(c, "x"));
			model.clear();
			if (converter instanceof ConverterChain) {
				Converter[] converters = ((ConverterChain)converter).getConverters();
				for (Converter co : converters) {
					model.addElement(co);
				}
			} else {
				model.addElement(converter);
			}
		} else {
			model.clear();
			functionLabel.setText("id");
		}
	}
	
	/**
	 * Returns the converter, which the info belongs to.
	 * 
	 * @return
	 */
	public Converter getConverter() {
		return converter;
	}
	
	/**
	 * Sets the DataSource, which info is presented in the panel.
	 * @param dataSource
	 */
	public void setDataSource(DataSource dataSource) {
		this.dataSource = dataSource;
		if (this.dataSource != null) {
			dataSourceL.setText(dataSource.toString());
		} else {
			dataSourceL.setText("N/A");
		}
	}
	
	/**
	 * Returns the DataSource, which the info belongs to.
	 * 
	 * @return
	 */
	public DataSource getDataSource() {
		return dataSource;
	}
	
}
