/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans.
 *
 * CosyBeans is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.infopanel;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.util.Date;

import javax.swing.JLabel;
import javax.swing.JPanel;

import com.cosylab.gui.displayers.DataState;
import com.cosylab.gui.util.GUIUtilities;

/**
 * <code>DebugPanel</code> displays the current state that was initiated on
 * a displayer. The panel shows data that belongs to the set DataState.
 * 
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 * @version $Id: DebugPanel.java,v 1.4 2008-04-22 12:31:02 jbobnar Exp $
 *
 */
public class DebugPanel extends JPanel {
		
	private static final long serialVersionUID = -1976156943972439878L;
	private DataState dataState;
	
	private JLabel stateLabel;
	private JLabel errorLabel;
	private JLabel timeLabel;
	
	private JLabel state;
	private JLabel error;
	private JLabel time;
	
	public DebugPanel() {
		super();
		initialize();
	}
	
	private void initialize() {
		this.setLayout(new GridBagLayout());
		stateLabel = new JLabel("State: ");
		GUIUtilities.setFixedSize(stateLabel,new Dimension(100,21));
		this.add(stateLabel, new GridBagConstraints(0,0,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(11,11,1,1),0,0));
		state = new JLabel();
		this.add(state, new GridBagConstraints(1,0,1,1,1,0,GridBagConstraints.WEST, GridBagConstraints.BOTH, new Insets(11,1,1,11),0,0));
		
		errorLabel = new JLabel("Description: ");
		GUIUtilities.setFixedSize(errorLabel,new Dimension(100,21));
		this.add(errorLabel, new GridBagConstraints(0,1,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1,11,1,1),0,0));
		error = new JLabel();
		this.add(error, new GridBagConstraints(1,1,1,1,1,0,GridBagConstraints.WEST, GridBagConstraints.BOTH, new Insets(1,1,1,11),0,0));

		timeLabel = new JLabel("Time: ");
		GUIUtilities.setFixedSize(timeLabel,new Dimension(100,21));
		this.add(timeLabel, new GridBagConstraints(0,2,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1,11,11,1),0,0));
		time = new JLabel();
		this.add(time, new GridBagConstraints(1,2,1,1,1,0,GridBagConstraints.WEST, GridBagConstraints.BOTH, new Insets(1,1,11,11),0,0));
	}
	
	/**
	 * Sets the DataState, which provides data for this panel.
	 * 
	 * @param dataState
	 */
	public void setDataState(DataState dataState) {
		this.dataState = dataState;
		if (this.dataState != null) {
			state.setText(dataState.type);
			error.setText(dataState.description);
			time.setText(String.valueOf(new Date(dataState.timestamp)));
		} else {
			state.setText("< NONE >");
			error.setText("< NONE >");
			time.setText(String.valueOf(new Date(System.currentTimeMillis())));
		}
		
	}
	
	/**
	 * Returns the DataState, that the info belongs to.
	 * @return
	 */
	public DataState getDataState() {
		return dataState;
	}
	
}
