/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans.
 *
 * CosyBeans is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.infopanel;

import java.awt.BorderLayout;
import java.beans.IntrospectionException;
import java.lang.reflect.InvocationTargetException;

import javax.swing.JPanel;
import javax.swing.JScrollPane;

import com.cosylab.gui.components.introspection.PropertiesTable;

/**
 * <code>PropertiesPanel</code> shows a table with all properties and their values
 * of the set bean.
 * 
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 * @version $Id: PropertiesPanel.java,v 1.4 2008-04-22 12:31:02 jbobnar Exp $
 *
 */
public class PropertiesPanel extends JPanel {

	private static final long serialVersionUID = 1523647646017553287L;
	private PropertiesTable table;
	private JScrollPane tableScroll;
	private Object bean;
	
	/**
	 * Constructs a new PropertiesPanel.
	 *
	 */
	public PropertiesPanel() {
		super();
		initialize();
	}
	
	private void initialize() {
		this.setLayout(new BorderLayout());
		table = new PropertiesTable();
		table.setEnabled(false);
		tableScroll = new JScrollPane(table);
		this.add(tableScroll, BorderLayout.CENTER);
	}
	
	/**
	 * Sets the bean, which provides the data.
	 * 
	 * @param bean
	 */
	public void setBean(Object bean) {
		this.bean = bean;
		try {
			table.setBean(bean);
		} catch (IntrospectionException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		}			
	}
	
	/**
	 * Returns the bean, which the data belongs to.
	 * 
	 * @return
	 */
	public Object getBean() {
		return bean;
	}
}
