/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.property.editors;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.Map;

import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JColorChooser;
import javax.swing.JList;

import com.cosylab.gui.components.SimpleButton;
import com.cosylab.gui.components.SimpleComboBox;
import com.cosylab.gui.components.util.ColorHelper;
import com.cosylab.gui.components.util.CosyUIElements;
import com.cosylab.gui.components.util.FontHelper;


/**
 * @author <a href="mailto:miha.kadunc@cosylab.com">Miha Kadunc</a>
 * @version $id$
 */
public class ColorEditor extends SimpleComboBox implements PropertyEditor {
    /**
     * 
     */
    private static final long serialVersionUID = 4783089461008709678L;

    /**
     * Both drop-down selection and button for the dialog.
     */
    public static final int MODE_BOTH = 0;

    /**
     * Only the drop-down selection
     */
    public static final int MODE_COMBO = 1;
    private int mode = -1;
    private Map map = null;
    private Color value = null;
    private SimpleButton extendButton = null;

    /**
     */
    public ColorEditor() {
        this(null);
    }

    /**
     * @param defaultColor Color
     */
    public ColorEditor(Color defaultColor) {
        super();
        initialize();
        setPropertyValue(defaultColor);
    }

    /**
     * @return Object
     * @see com.cosylab.gui.property.editors.PropertyEditor#getPropertyValue()
     */
    public Object getPropertyValue() {
        return value;
    }

    /**
     * @param value Object
     * @return boolean
     * @see com.cosylab.gui.property.editors.PropertyEditor#setPropertyValue(Object)
     */
    public boolean setPropertyValue(Object value) {
        if (value instanceof Color) {
            Color daNew = (Color) value;
            Color old = this.value;
            this.value = daNew;
            
            try {
                if (map.containsValue(daNew)) {
                    setSelectedItem(getKeyForValue(map, daNew));
                } else {
                    setSelectedItem(null);
                }
            } catch (NullPointerException e) {
                //ignore could happened if called at the wrong time
            }

            if ((daNew != null) && (daNew.getBlue() < 130) &&
                    (daNew.getRed() < 130) && (daNew.getGreen() < 130)) {
                setForeground(Color.WHITE);
            } else {
                setForeground(Color.BLACK);
            }

            extendButton.repaint();
            firePropertyChange(PROPERTY_VALUE_NAME, old, daNew);

            return true;
        }

        return false;
    }

    public static Object getKeyForValue(Map m, Object value) {
        Object[] keys = m.keySet().toArray();
        Object val;
        int i = 0;

        while (i < keys.length) {
            val = m.get(keys[i]);

            if (val.equals(value)) {
                return keys[i];
            }

            i++;
        }

        return null;
    }

    /**
     * @return String
     * @see com.cosylab.gui.property.editors.PropertyEditor#getDescription()
     */
    public String getDescription() {
        return null;
    }

    /**
     * @param description String
     * @see com.cosylab.gui.property.editors.PropertyEditor#setDescription(String)
     */
    public void setDescription(String description) {
    }

    /**
     * @return PropertyEditor
     * @see com.cosylab.gui.property.editors.PropertyEditor#getEditor()
     */
    public PropertyEditor getPropertyEditor() {
        return null;
    }

    private void initialize() {
        setBorder(CosyUIElements.getPlainBorder(false));
        addActionListener(new ActionListener() {
                /**
                 * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
                 */
                public void actionPerformed(ActionEvent e) {
                    Color daNew = (Color) map.get(getSelectedItem());
                    setPropertyValue(daNew);
                }
            });
        initializeComboListeners();
        extendButton = new SimpleButton("...");
        extendButton.setBorder(CosyUIElements.getPlainBorder(true));
        extendButton.setPressedBorder(null);
        extendButton.setActionMode(SimpleButton.FAST_ACTION_MODE);
        extendButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    Color c = JColorChooser.showDialog(ColorEditor.this,
                            "Choose the desired color", value);
                    setPropertyValue(c);
                }
            });
        setMode(MODE_BOTH);
        setColorMap(getJavaColorMap());
    }

    private void initializeComboListeners() {
    }

    public void setMode(int mode) {
        if (mode != this.mode) {
            int old = this.mode;
            this.mode = mode;

            if (mode == MODE_BOTH) {
                setUI(new ColorEditorUI());
            } else if (mode == MODE_COMBO) {
                setStyle(Style.DROP_DOWN_STYLE);
            }

            firePropertyChange("mode", old, mode);
        }
    }

    public void setColorMap(Map colorMap) {
        map = colorMap;
        setModel(new DefaultComboBoxModel(colorMap.keySet().toArray()));
        setRenderer(new ColorCellRenderer(colorMap));
        setSelectedIndex(0);
    }

    /**
     * @return Map
     */
    public static Map getCosyColorMap() {
        HashMap ret = new HashMap();
        String[] names = ColorHelper.COLOR_NAMES;

        for (int i = 0; i < names.length; i++) {
            try {
                ret.put(names[i],
                    ColorHelper.class.getMethod("get" + names[i], new Class[0])
                                     .invoke(null, new Object[0]));
            } catch (IllegalAccessException e) {
            } catch (InvocationTargetException e) {
            } catch (NoSuchMethodException e) {
            }
        }

        return ret;
    }

    /**
     * @return Map
     */
    public static Map getJavaColorMap() {
        HashMap ret = new HashMap();
        String[] names = ColorHelper.JAVA_COLOR_NAMES;

        for (int i = 0; i < names.length; i++) {
            try {
                ret.put(names[i], Color.class.getField(names[i]).get(null));
            } catch (IllegalAccessException e) {
            } catch (NoSuchFieldException e) {
            }
        }

        return ret;
    }

    /**
     * @see javax.swing.JComponent#paintComponent(Graphics)
     */
    @Override
    protected void paintComponent(Graphics g) {
        g.setColor(getBackground());
        g.fillRect(0, 0, getWidth(), getHeight());
        super.paintComponent(g);
    }

    public class ColorEditorUI extends SimpleComboBox.DropComboBoxUI {
        /**
         * @see com.cosylab.gui.components.SimpleComboBox.DropComboBoxUI#getDropButton()
         */
        @Override
        protected SimpleButton getDropButton() {
            return extendButton;
        }
    }

    /**
    *
    *
    * @author  Miha Kadunc (miha.kadunc@cosylab.com)
    * @version @@VERSION@@
    */
    public class ColorCellRenderer extends DefaultListCellRenderer {
        /**
         * 
         */
        private static final long serialVersionUID = 6828213764864288040L;
        private Map colorMap = null;

        /**
         * @param valueToColorMap Map
         */
        public ColorCellRenderer(Map valueToColorMap) {
            colorMap = valueToColorMap;
            getInsets().left = 10;
            setFont(FontHelper.getDefaultFont());
        }

        /**
         * @param list JList
         * @param value Object
         * @param index int
         * @param isSelected boolean
         * @param cellHasFocus boolean
         * @return Component
         * @see javax.swing.DefaultListCellRenderer#getListCellRendererComponent(JList, Object, int, boolean, boolean)
         */
        @Override
        public Component getListCellRendererComponent(JList list, Object value,
            int index, boolean isSelected, boolean cellHasFocus) {
            super.getListCellRendererComponent(list, value, index, isSelected,
                cellHasFocus);

            Color bkg = (Color)colorMap.get(value);
            if (bkg==null) bkg=ColorEditor.this.value; 
            setBackground(bkg);

            if ((bkg != null) && (bkg.getBlue() < 130) && (bkg.getRed() < 130) &&
                    (bkg.getGreen() < 130)) {
                setForeground(Color.WHITE);
            } else {
                setForeground(Color.BLACK);
            }

            return this;
        }
    }
}
