/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.property.editors;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.util.HashMap;
import java.util.Map;

import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListCellRenderer;
import javax.swing.Icon;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JList;

import com.cosylab.gui.components.ledder.CustomLedIcon;

/**
 * @author <a href="mailto:miha.kadunc@cosylab.com">Miha Kadunc</a>
 */
public class EnumEditor extends JComboBox implements PropertyEditor {
    /**
     * 
     */
    private static final long serialVersionUID = -8381165276842613111L;
    private static Object NULL_VALUE=new Object() {
        @Override
        public String toString() {
            return "";
        }
    };
    private static String NULL_NAME="  "; 
    private class EnumRenderer extends DefaultListCellRenderer {
        /**
         * 
         */
        private static final long serialVersionUID = 6349607036048670520L;
        private Map icons;
        
        private Icon getIcon(Color c) {
            if (icons==null) {
                icons=new HashMap();
            }
            if (!icons.containsKey(c)) {
                CustomLedIcon ci=new CustomLedIcon(c,18,2);
                icons.put(c,ci);
            }
            return (Icon)icons.get(c);
        }
        
        /* (non-Javadoc)
         * @see javax.swing.DefaultListCellRenderer#getListCellRendererComponent(javax.swing.JList, java.lang.Object, int, boolean, boolean)
         */
        @Override
        public Component getListCellRendererComponent(JList list, Object value,
                int index, boolean isSelected, boolean cellHasFocus) {
            String strVal=(String)names.get(value);
            if (strVal==null) {
                if (value!=null) strVal=String.valueOf(value);
                else strVal="  ";
            }
            JComponent comp=(JComponent)super.getListCellRendererComponent(list, strVal, index, isSelected, cellHasFocus);
            if (colors.containsKey(value) && colors.get(value)!=null) {
                setIcon(getIcon((Color)colors.get(value)));
            } else {
                setIcon(null);
            }
            return comp;
        }
        
    }    
    private Map names=new HashMap();
    private Map colors=new HashMap();
    private boolean nullAllowed=false;
    /**
     * @param items
     */
    public EnumEditor(Object[] items) {
        this(items, null, null);
    }
    public EnumEditor(Object[] items, String[] names, Color[] colors) {
        super();
        setItems(items, names, colors);
        initGUI();
    }
    
    public void setItems(Object[] newItems, String[] names, Color[] colors) {
        Object[] items=new Object[newItems.length];
        System.arraycopy(newItems, 0, items, 0, newItems.length);
        this.names.clear();
        this.colors.clear();
        for (int i = 0; i < items.length; i++) {
            if (items[i]==null) items[i]=NULL_VALUE;
            
            String curName=(names==null)?null:names[i];
            if ((curName==null || curName.length()<1) && items[i]!=null) curName=items[i].toString();
            if (curName==null || curName.length()<1) curName=NULL_NAME;
            this.names.put(items[i],curName);
            
            Color curCol=colors==null?null:colors[i];
            this.colors.put(items[i],curCol);
        }
        setModel(new DefaultComboBoxModel(items));
    }

    private void initGUI() {
        setFont(getFont().deriveFont(Font.PLAIN));
        setMaximumRowCount(20);
        setRenderer(new EnumRenderer());
    }

    /* (non-Javadoc)
     * @see com.cosylab.gui.property.PropertyEditor#getPropertyValue()
     */
    public Object getPropertyValue() {
        Object ret=getSelectedItem();
        if (ret==NULL_VALUE) {
            return null;
        }
        if ("".equals(ret) && nullAllowed) {
            return null;
        }
        return ret; 
    }

    /* (non-Javadoc)
     * @see com.cosylab.gui.property.PropertyEditor#setPropertyValue(java.lang.Object)
     */
    public boolean setPropertyValue(Object value) {
        setSelectedItem(value);
        if (getSelectedItem()!=value && !isEditable()) {
            setEditable(true);
            setSelectedItem(value);
            setEditable(false);
            return getSelectedItem()==value;
        }
        return false;
    }

    /* (non-Javadoc)
     * @see com.cosylab.gui.property.PropertyEditor#getDescription()
     */
    public String getDescription() {
        return null;
    }

    /* (non-Javadoc)
     * @see com.cosylab.gui.property.PropertyEditor#setDescription(java.lang.String)
     */
    public void setDescription(String description) {
    }

    public boolean isNullAllowed() {
        return nullAllowed;
    }
    public void setNullAllowed(boolean nullAllowed) {
        this.nullAllowed = nullAllowed;
        DefaultComboBoxModel mdl=(DefaultComboBoxModel)getModel();
        if (mdl.getIndexOf(NULL_VALUE)<0 && nullAllowed) {
            mdl.insertElementAt(NULL_VALUE,0);
            names.put(NULL_VALUE, NULL_NAME);
        }
        if (!nullAllowed){
            if (mdl.getIndexOf(NULL_VALUE)>=0) {
                mdl.removeElement(NULL_VALUE);
                names.remove(NULL_VALUE);
            }
        }
    }
}
