/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.property.editors;

import java.awt.Component;
import java.awt.Font;
import java.awt.GraphicsEnvironment;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JComboBox;
import javax.swing.JList;


/**
 * 
 * To change the template for this generated type comment go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 * 
 * @author <a href="mailto:jernej.kamenik@cosylab.com">Jernej Kamenik</a>
 * @version $id$
 */
public class SimpleFontEditor extends JComboBox implements PropertyEditor {
	/**
     * 
     */
    private static final long serialVersionUID = 778998936245897928L;
    String[] fonts;

	private class FontRenderer extends DefaultListCellRenderer {

		/**
         * 
         */
        private static final long serialVersionUID = 6856815835178506253L;

        @Override
        public Component getListCellRendererComponent(JList list, Object value,
				int index, boolean isSelected, boolean cellHasFocus) {
			String name = (String) value;
			Component c = super.getListCellRendererComponent(list, value,
					index, isSelected, cellHasFocus);
			if (isSelected || !SimpleFontEditor.this.hasFocus())
				SimpleFontEditor.this.setFont(Font.decode(name));
			c.setFont(Font.decode(name));
			return c;

		}

	}

	public SimpleFontEditor() {
		super();
		fonts = GraphicsEnvironment.getLocalGraphicsEnvironment()
				.getAvailableFontFamilyNames();
		setModel(new DefaultComboBoxModel(fonts));
		setRenderer(new FontRenderer());
		setEditable(false);

		addItemListener(new ItemListener() {
			Object old = null;

			public void itemStateChanged(ItemEvent e) {
				if (e.getStateChange() == ItemEvent.DESELECTED) {
					old = e.getItem();
				} else {
					firePropertyChange(PROPERTY_VALUE_NAME, old, e.getItem());
				}
			}
		});
	}

	public Object getPropertyValue() {
		String name = (String) getSelectedItem();
		return Font.decode(name);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.cosylab.gui.property.PropertyEditor#setPropertyValue(java.lang.Object)
	 */
	public boolean setPropertyValue(Object value) {
		setSelectedItem(((Font) value).getFamily());
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.cosylab.gui.property.PropertyEditor#getDescription()
	 */
	public String getDescription() {
		return "Font";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.cosylab.gui.property.PropertyEditor#setDescription(java.lang.String)
	 */
	public void setDescription(String description) {
	}

}
