/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans.
 *
 * CosyBeans is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.util;

import java.awt.Component;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyVetoException;
import java.io.IOException;

import javax.swing.JComponent;

import com.cosylab.gui.components.util.CosyTransferHandler;
import com.cosylab.gui.displayers.Displayer;
import com.cosylab.gui.displayers.DisplayerParameters;
import com.cosylab.gui.displayers.DoubleDisplayer;
import com.cosylab.gui.displayers.MultipleDisplayer;
import com.cosylab.introspection.BeanIntrospector;

/**
 * <code>DisplayerParametersTransferHandler</code> is a TransferHandler for
 * DisplayerParametersAcceptors and MultipleParametersAcceptors.
 * 
 * @author tkusterle
 * @depreacated use {@link DefaultDisplayerParametersTransferHandler} instead
 *
 */
public class DisplayerParametersTransferHandler extends CosyTransferHandler {
	
	private static final long serialVersionUID = -4223198013311762602L;
	protected DisplayerParametersSelectorDialog dialog;

	protected Displayer displayer;
	protected MultipleDisplayer mDisplayer;

	public DisplayerParametersTransferHandler(Displayer comp, boolean reciveEnabled, boolean exportEnabled) {
		super(reciveEnabled, exportEnabled);
		displayer= comp;
		hookTransferHandler((JComponent)comp);
	}

	public DisplayerParametersTransferHandler(MultipleDisplayer comp, boolean reciveEnabled, boolean exportEnabled) {
		super(reciveEnabled, exportEnabled);
		mDisplayer= comp;
		hookTransferHandler((JComponent)comp);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.util.CosyTransferHandler#getTransferData(java.awt.datatransfer.DataFlavor)
	 */
	@Override
	public Object getTransferData(DataFlavor flavor)
			throws UnsupportedFlavorException, IOException {
		if (flavor instanceof MultipleDisplayerParametersFlavor) {
			DisplayerParameters[] p= exportMultipleDisplayerParameters();
			if (p!=null) {
				return p;
			}
		}
		if (flavor instanceof DisplayerParametersFlavor) {
			DisplayerParameters p= exportDisplayerParameters();
			if (p!=null) {
				return p;
			}
		}
		
		throw new UnsupportedFlavorException(flavor);
	}

	protected DisplayerParameters exportDisplayerParameters() {
		if (isMultiDisplayer()) {
			DisplayerParameters[] d= exportMultipleDisplayerParameters();
			int i = selectParameters((Component)mDisplayer,d);
			if (i<0) {
				return null;
			}
			return d[i];
 		} 
 		if (displayer instanceof DoubleDisplayer) {
			return new DisplayerParameters(displayer.getTitle(),(DoubleDisplayer)displayer);
		} 
 		return new DisplayerParameters(displayer.getTitle());
	}

	protected DisplayerParameters[] exportMultipleDisplayerParameters() {
		throw new UnsupportedOperationException();
	}
	
	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.util.CosyTransferHandler#getTransferDataFlavors()
	 */
	@Override
	public DataFlavor[] getTransferDataFlavors() {
		if (isMultiDisplayer()) {
			return new DataFlavor[] {MultipleDisplayerParametersFlavor.FLAVOR, DisplayerParametersFlavor.FLAVOR};
		}
		return new DataFlavor[] {DisplayerParametersFlavor.FLAVOR};
	}

	/* (non-Javadoc)
	 * @see javax.swing.TransferHandler#importData(javax.swing.JComponent, java.awt.datatransfer.Transferable)
	 */
	@Override
	public boolean importData(JComponent comp, Transferable t) {
		try {
			Object drop=null;
			if (isMultiDisplayer()) {
				if (t.isDataFlavorSupported(MultipleDisplayerParametersFlavor.FLAVOR)) {
					drop= t.getTransferData(MultipleDisplayerParametersFlavor.FLAVOR);
				}
			} else {
				if (t.isDataFlavorSupported(DisplayerParametersFlavor.FLAVOR)) {
					drop = t.getTransferData(DisplayerParametersFlavor.FLAVOR);
				}
			}
			DisplayerParameters[] dp = null;

			dp = castParameters(drop);

			if (dp!=null) {
				return importDisplayerParameters(dp);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		return false;
	}
	
	protected DisplayerParameters[] castParameters(Object... parameters) {
		if (parameters instanceof DisplayerParameters[]) {
			return (DisplayerParameters[])parameters.getClass().cast(parameters);
		} else if (parameters.length == 1 && parameters[0] instanceof DisplayerParameters){
			return new DisplayerParameters[] {(DisplayerParameters)parameters[0]};
		}
		return null;
	}
	
	/**
	 * @param dp
	 * @throws PropertyVetoException 
	 */
	protected boolean importDisplayerParameters(DisplayerParameters[] dp) throws PropertyVetoException {
		if (!isMultiDisplayer()) {
			int i= selectParameters((Component)displayer,dp);
			if (i<0) {
				return false;
			}
			setParametersToDisplayer(displayer,dp[i]);
			return true;
		}
		return false;
		
	}

	protected DisplayerParametersSelectorDialog getParameterSelectionDialog() {
		if (dialog == null) {
			dialog = new DisplayerParametersSelectorDialog((Component)(isMultiDisplayer() ? mDisplayer : displayer), "Adding Source", "Select Source:");
		}
		return dialog;
	}
	
	protected int selectParameters(Component c,DisplayerParameters[] dp) {
		return getParameterSelectionDialog().showSelectionDialog(c,dp);
	}

	protected int selectParameters(Component c,String[] dp) {
		return getParameterSelectionDialog().showSelectionDialog(c,dp);
	}
	
	public void setParametersToDisplayer(Object disp, DisplayerParameters parameters) throws PropertyVetoException {
		String[] n= parameters.getNames();
		Object[] o= parameters.getValues();
		
		for (int i = 0; i < n.length; i++) {
			if (o[i]!=null && (!(o[i] instanceof Double) || !((Double)o[i]).isNaN())) {
				try {
					BeanIntrospector.setPropertyValue(disp,n[i],o[i]);
				} catch(NoSuchMethodException e) {
					//do nothing
				} catch (Exception e) {
					throw new PropertyVetoException("Property '"+n[i]+"' not set: "+e.getMessage(),new PropertyChangeEvent(disp,n[i],null,o[i]));
				}
			}
		}
	}
	
	public int displayerParametersDimension() {
		if (displayer != null) {
			return 1;
		}
		return mDisplayer.getConsumers().length;
	}

	public boolean isMultiDisplayer() {
		return mDisplayer !=null;
	}
	/**
	 * Returns the displayer.
	 * @return Returns the displayer.
	 */
	public Displayer getDisplayer() {
		return displayer;
	}
	
	/**
	 * Returns the displayer.
	 * @return Returns the displayer.
	 */
	public MultipleDisplayer getMultipleDisplayer() {
		return mDisplayer;
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.cosylab.gui.components.util.CosyTransferHandler#canImport(javax.swing.JComponent, java.awt.datatransfer.DataFlavor[])
	 */
	@Override
	public boolean canImport(JComponent comp, DataFlavor[] transferFlavors) {
		return super.canImport(comp, transferFlavors) & !isDragging;
	}
}
