/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans.
 *
 * CosyBeans is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.util;

import java.beans.PropertyDescriptor;
import java.util.ArrayList;

import com.cosylab.gui.displayers.Displayer;
import com.cosylab.introspection.BeanIntrospector;

/**
 * <code>GuiIntrospector</code> offers utility methods for inspecting beans 
 * properties.
 * 
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 * @version $Id: GuiIntrospector.java,v 1.4 2008-04-22 12:31:02 jbobnar Exp $
 *
 */
public class GuiIntrospector {
	
	/**
	 * Returns all static writable properties of the specified class.
	 * 
	 * @param displayerClass
	 * @return property descriptors for all writable properties
	 * @throws java.beans.IntrospectionException
	 */
	public static PropertyDescriptor[] getDisplayerWritableStaticProperties(Class displayerClass)
			throws java.beans.IntrospectionException {
		PropertyDescriptor[] desc = BeanIntrospector.getWritable(BeanIntrospector.getInterfacePropertyDescriptors(
				displayerClass, Displayer.class));
		ArrayList<PropertyDescriptor> list = new ArrayList<>();
		for (int i = 0; i < desc.length; i++) {
			if (!desc[i].getName().equals("value"))
				list.add(desc[i]);
		}
		PropertyDescriptor[] ret = new PropertyDescriptor[list.size()];
		list.toArray(ret);
		return ret;
	}

	/**
	 * Returns dynamic properties of the displayer.
	 * 
	 * @param displayerClass
	 * @return
	 * @throws java.beans.IntrospectionException
	 */
	public static PropertyDescriptor getDisplayerDynamicProperty(Class displayerClass)
			throws java.beans.IntrospectionException {
		return BeanIntrospector.getPropertyDescriptor(displayerClass, "value");
	}

	/**
	 * Returns all static readable properties of the specified class.
	 * 
	 * @param displayerClass
	 * @return property descriptors for all writable properties
	 * @throws java.beans.IntrospectionException
	 */
	public static PropertyDescriptor[] getDisplayerReadableStaticProperties(Class displayerClass)
			throws java.beans.IntrospectionException {
		PropertyDescriptor[] desc = BeanIntrospector.getReadable(BeanIntrospector.getInterfacePropertyDescriptors(
				displayerClass, Displayer.class));
		ArrayList<PropertyDescriptor> list = new ArrayList<>();
		for (int i = 0; i < desc.length; i++) {
			if (!desc[i].getName().equals("value"))
				list.add(desc[i]);
		}
		PropertyDescriptor[] ret = new PropertyDescriptor[list.size()];
		list.toArray(ret);
		return ret;
	}
}
