/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans.
 *
 * CosyBeans is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.util;

import java.awt.Component;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.IOException;

import javax.swing.JComponent;

import com.cosylab.gui.displayers.DisplayerParameters;
import com.cosylab.gui.displayers.MultipleDisplayer;

/**
 * <code>MultipleDisplayerParametersTransferHandler</code> can handle
 * MultipleDisplayerParameters.
 * 
 * @author tkusterle
 *
 */
public class MultipleDisplayerParametersTransferHandler extends AbstractDisplayerParametersTransferHandler {
	
	private static final long serialVersionUID = -4223198013311762602L;

	protected MultipleDisplayer mDisplayer;

	public MultipleDisplayerParametersTransferHandler(MultipleDisplayer comp, boolean reciveEnabled, boolean exportEnabled) {
		super((JComponent)comp,reciveEnabled, exportEnabled);
		mDisplayer= comp;
	}
	
	public MultipleDisplayerParametersTransferHandler(MultipleDisplayer comp, boolean reciveEnabled, boolean exportEnabled, DataFlavor[] f, boolean resetFlavors) {
		super((JComponent)comp, reciveEnabled, exportEnabled, f, resetFlavors);
		mDisplayer= comp;
	}



	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.util.CosyTransferHandler#getTransferData(java.awt.datatransfer.DataFlavor)
	 */
	@Override
	public Object getTransferData(DataFlavor flavor)
			throws UnsupportedFlavorException, IOException {
		if (flavor instanceof MultipleDisplayerParametersFlavor) {
			DisplayerParameters[] p= exportMultipleDisplayerParameters();
			if (p!=null) {
				return p;
			}
		}
		return super.getTransferData(flavor);
	}

	@Override
	protected DisplayerParameters exportDisplayerParameters() {
		DisplayerParameters[] d= exportMultipleDisplayerParameters();
		int i = selectParameters((Component)mDisplayer,d);
		if (i<0) {
			return null;
		}
		return d[i];
	}

	protected DisplayerParameters[] exportMultipleDisplayerParameters() {
		throw new UnsupportedOperationException();
	}
	
	/* (non-Javadoc)
	 * @see javax.swing.TransferHandler#importData(javax.swing.JComponent, java.awt.datatransfer.Transferable)
	 */
	@Override
	public boolean importData(JComponent comp, Transferable t) {
		try {
			Object drop=null;
			if (t.isDataFlavorSupported(MultipleDisplayerParametersFlavor.FLAVOR)) {
				drop= t.getTransferData(MultipleDisplayerParametersFlavor.FLAVOR);
				DisplayerParameters[] dp = null;
				dp = castParameters(drop);
	
				if (dp!=null) {
					return importDisplayerParameters(dp);
				}
			} else {
				super.importData(comp, t);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		return false;
	}
	
	public int displayerParametersDimension() {
		return mDisplayer.getConsumers().length;
	}

	/**
	 * Returns the displayer.
	 * @return Returns the displayer.
	 */
	public MultipleDisplayer getMultipleDisplayer() {
		return mDisplayer;
	}
}
