/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of Java-Common.
 *
 * Java-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Java-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Java-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.introspection;

import java.lang.reflect.Member;
/**
 * This class serves as a filter for selecting members in
 * a <code>Class</code> or from an array.
 * 
 * @author Miha Kadunc (miha.kadunc@cosylab.com)
 * @version @@VERSION@@
 */
public interface MemberFilter {
	/**
   * Should return true iff the filter allows
   * constructors to be contained in the output.
   * Should only some of the constructors be filtered,
   * this method should return true and specify a
   * custom filter in the <code>isAllowed(Member)</code> method.
   * 
	 * @return boolean
	 */
	public boolean areConstructorsAllowed();
  
	/**
   * If filter is used to retrieve members from a class,
   * this method should return true iff only members
   * that are declared in the given class are to be
   * returned, discarding members that are declared in
   * superclasses or superinterfaces.
   * 
	 * @return boolean
	 */
	public boolean areDeclaredOnly();
  
	/**
   * Should return true iff the filter allows fields.
   * 
	 * @return boolean
	 */
	public boolean areFieldsAllowed();
  
	/**
   * Should return true iff the filter allows methods.
   * 
	 * @return boolean
	 */
	public boolean areMethodsAllowed();
  
	/**
   * Should return true iff the filter discards all
   * the non-public members.
   * 
	 * @return boolean
	 */
	public boolean arePublicOnly();
  
	/**
   * When other methods in this class are not sufficient for
   * the filter implementation, this method should be used.
   * Members, that comply with all other requirements by this
   * class's methods, will be passed as a parameter to this method,
   * which decides whether a given member should be contained in the
   * output or not.
   * 
	 * @param member Member
	 * @return boolean
	 */
	public boolean isAllowed(Member member);
}
