/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of Java-Common.
 *
 * Java-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Java-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Java-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.logging;

import java.io.PrintWriter;
import java.io.StringWriter;

import java.text.SimpleDateFormat;

import java.util.Date;
import java.util.logging.Formatter;
import java.util.logging.LogRecord;


/**
 * An implementation of <code>java.util.logging.Formatter</code>. Produces
 * simple single line log reports meant to go to the console.
 *
 * @author Matej Sekoranja (matej.sekoranjaATcosylab.com)
 */
public class ConsoleLogFormatter extends Formatter
{
	/** System property key to enable trace messages. */
	public static String KEY_TRACE = "TRACE";

	/** Line separator string. */
	private static boolean showTrace = System.getProperties().containsKey(KEY_TRACE);

	/** Line separator string. */
	private static String lineSeparator = System.getProperty("line.separator");

	/** ISO 8601 date formatter. */
	private SimpleDateFormat timeFormatter = new SimpleDateFormat(
		    "yyyy-MM-dd'T'HH:mm:ss.SSS");

	/** Date object (used not to recreate it every time). */
	private Date date = new Date();

	/**
	 * Format the given LogRecord.
	 *
	 * @param record the log record to be formatted.
	 *
	 * @return a formatted log record
	 */
	@Override
    public String format(LogRecord record)
	{
		StringBuffer sb = new StringBuffer(128);

		synchronized (date) {
			date.setTime(record.getMillis());

			//sb.append('(');
			sb.append(timeFormatter.format(date));

			//sb.append(')');
		}

		/*
		if (record.getLoggerName() != null)
		{
		    sb.append(' ');
		    sb.append('[');
		    sb.append(record.getLoggerName());
		    sb.append(']');
		}
		*/
		if (record.getLoggerName() != null
		    || record.getSourceClassName() != null
		    || record.getSourceMethodName() != null) {
			sb.append(' ');
			sb.append('[');

			if (record.getLoggerName() != null) {
				sb.append(record.getLoggerName());

				if (record.getSourceClassName() != null) {
					sb.append(':');
				}
			}

			if (record.getSourceClassName() != null) {
				sb.append(record.getSourceClassName().substring(record.getSourceClassName()
				        .lastIndexOf('.') + 1));
			}

			sb.append('#');

			if (record.getSourceMethodName() != null) {
				sb.append(record.getSourceMethodName());
			}

			sb.append(']');
		}

		sb.append(' ');
		sb.append(record.getMessage());

		//sb.append(' ');
		//sb.append(record.getLevel().getLocalizedName());
		// trace
		/*if (showTrace)
		{
		    // source
		    sb.append('[');
		    if (record.getSourceClassName() != null)
		        sb.append(record.getSourceClassName());

		    // method name
		    if (record.getSourceMethodName() != null)
		    {
		        sb.append('#');
		        sb.append(record.getSourceMethodName());
		    }
		    sb.append(']');
		}*/
		sb.append(lineSeparator);

		// exceptions
		if (showTrace) {
			if (record.getThrown() != null) {
				try {
					StringWriter sw = new StringWriter();
					try (PrintWriter pw = new PrintWriter(sw)) {
					    record.getThrown().printStackTrace(pw);
					}
					sb.append(sw.toString());
				} catch (Exception ex) {
				}

				//record.getThrown().printStackTrace();
			}
		}

		return new String(sb);
	}
}

/* __oOo__ */
