/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of Java-Common.
 *
 * Java-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Java-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Java-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.util;

import java.util.Iterator;

/**
 * An iterator for iterating arrays.
 * 
 * @author Klemen Zagar, Cosylab
 * 
 * @param <T>
 *            Type of the array.
 */
public class ArrayIterator<T> implements Iterator<T> {
	private int index = 0;
	private T[] array;
	private int length;

	public ArrayIterator(T[] array, int length) {
		if(array == null && length > 0) {
			throw new IllegalArgumentException("Array iterator's length is non-zero, but no array is given.");
		}
		if(length < 0) {
			throw new IllegalArgumentException("Array iterator's length can not be negative.");
		}
		this.array = array;
		this.length = length;
	}

	public boolean hasNext() {
		return this.index < this.length;
	}

	public T next() {
		return this.array[this.index++];
	}

	@Override
    public void remove() {
		throw new UnsupportedOperationException(
				"Removing of elements with array iterator is not possible.");
	}
}
