/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of Java-Common.
 *
 * Java-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Java-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Java-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.util;

import java.io.PrintStream;

import java.util.ArrayList;


/**
 * <p>
 * <code>CombinedException</code> holds together association of exception,
 * which was for some reason thrown together. For example when a operation is
 * made on set of  object in same hirarchical level and some of them throw
 * exception. These  exceptions should be collected and associated and
 * reethrown as single  <code>CombinedException</code>.
 * </p>
 * 
 * <p>
 * <b>NOTE!</b> instances of this class are not synchronized. In case more
 * threads is addind exceptions, htan programmer must provide own
 * synchronization.
 * </p>
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $id$
 *
 * @see com.cosylab.util.CommonException
 */
public class CombinedException extends CommonException
{
	/**
     * 
     */
    private static final long serialVersionUID = -1450047804664314521L;
    /*
	 * For storing collected exceptions.
	 */
	protected ArrayList collected = new ArrayList();

	/**
	 * Create a new instance of <code>CombinedException</code>. This
	 * constructor should normally not be used since it carries no additional
	 * exception data.
	 */
	protected CombinedException()
	{
		super();
	}

	/**
	 * Create an instance of <code>CombinedException</code> with a with a
	 * specified message string. The construction is delegated to
	 * <code>super</code>.
	 *
	 * @param instance identifier of the instance throwing this exception
	 * @param s message to be printed together with the exception type when the
	 *        <code>toString()</code> is called.
	 */
	public CombinedException(Object instance, String s)
	{
		super(instance, s);
	}

	/**
	 * Create an instance of <code>CombinedException</code> by specifying both
	 * the string message and a <code>Throwable</code> object that represents
	 * the nested exception.
	 *
	 * @param instance identifier of the instance throwing this exception
	 * @param message a string message passed to <code>super</code>
	 * @param t an instance of <code>Throwable</code> that caused this to be
	 *        thrown, can be <code>null</code>
	 */
	public CombinedException(Object instance, String message, Throwable t)
	{
		super(instance, message, t);
	}

	/**
	 * Associates parameter <code>throwable</code> to this exception.
	 *
	 * @param throwble exception to be associated
	 */
	public void associateException(Throwable throwble)
	{
		collected.add(throwble);
	}

	/**
	 * Returns exceptions associated with this object.
	 *
	 * @return exceptions associated with this object
	 */
	public Throwable[] getAssociatedExceptions()
	{
		Throwable[] t = new Throwable[collected.size()];
		collected.toArray(t);

		return t;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param s DOCUMENT ME!
	 */
	@Override
    public void printStackTrace(PrintStream s)
	{
		super.printStackTrace(s);
		s.println("Associated exceptions (" + collected.size() + " items):");

		Throwable[] t = getAssociatedExceptions();

		for (int i = 0; i < t.length; i++) {
			s.print("[" + i + "]: ");
			t[i].printStackTrace(s);
		}
	}
}

/* __oOo__ */
