/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of Java-Common.
 *
 * Java-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Java-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Java-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.util;

import java.util.Date;
import java.util.Objects;

/**
 * 
 * @author <a href="mailto:jernej.kamenik@cosylab.com">Jernej Kamenik</a>
 * @version $id$
 */
public class DateSpan {
	private Date startingDate;
	private Date finishingDate;
	

	/**
	 * Constructor for DateSpan.
	 */
	public DateSpan(Date startingDate, Date finishingDate) {
		super();
		this.startingDate = startingDate;
		this.finishingDate = finishingDate;
	}

  /**
   * Constructor for DateSpan.
   */
  public DateSpan(Date startingDate, long duration) {
    super();
    this.startingDate = startingDate;
    this.finishingDate = new Date(startingDate.getTime()+duration);
  }

	
	public Date getStartingDate() {
		return startingDate;
	}

	public Date getFinishingDate() {
		return finishingDate;
	}
	public long getTimeSpan() {
		return finishingDate.getTime()-startingDate.getTime();
	}

  @Override
public String toString() {
	
    return "from: "+ startingDate.toString()+" to: "+finishingDate.toString()+" spans: "+getTimeSpan();
  }
  
    @Override
	public boolean equals(Object obj) {
    	if (!(obj instanceof DateSpan)) return false;
    	DateSpan d = (DateSpan)obj;
    	return (d.startingDate.equals(startingDate) && d.finishingDate.equals(finishingDate));
  	}

    @Override
    public int hashCode() {
        return Objects.hash(startingDate, finishingDate);
    }
}
