/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of Java-Common.
 *
 * Java-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Java-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Java-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.util;

/**
 * <code>DefaultObjectFactory</code> is default <code>ObjectFactory</code> implementation.
 * Tries to create a new instance of certain <code>Class</code> objects with default constructor.
 * If it fails creating new instance with default constructor it will returned <code>null</code>
 *
 * @author <a href="mailto:andrej.kosmrlj@cosylab.com">Andrej Kosmrlj</a>
 * @version $Id: DefaultObjectFactory.java,v 1.3 2008-04-22 12:26:29 jbobnar Exp $
 * 
 * @since VERSION
 */
public class DefaultObjectFactory implements ObjectFactory {
	
	Class objectType;
	
	public DefaultObjectFactory(Class objectType){
		if (objectType == null) throw new NullPointerException("objectType");
		if (!PoolableObject.class.isAssignableFrom(objectType)) throw new IllegalArgumentException("objectType !instanceof PoolableObject");
		this.objectType=objectType;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.util.ObjectFactory#newInstance()
	 */
	public PoolableObject newInstance() {
		PoolableObject o=null;
		try{
			o=(PoolableObject)objectType.newInstance();
		} catch (Exception e){
			e.printStackTrace();
		}
		return o;
	}

}
