/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of Java-Common.
 *
 * Java-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Java-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Java-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.util;

import java.util.GregorianCalendar;

/**
 * Mock implementation of a {@link TimeProvider}, which returns pre-set time.
 * Especially useful in unit tests. Also supports auto-advance of time (1 second
 * by default).
 * 
 * @author Klemen Zagar, Cosylab
 */
public class MockTimeProvider implements TimeProvider {
	private long millis;
	private long advance = 1000;

	/**
	 * Constructor sets the initial mock time.
	 */
	public MockTimeProvider() {
		this.millis = new GregorianCalendar(2008, 1, 1, 0, 0, 0)
				.getTimeInMillis();
	}

	/**
	 * Set the amount of time (in milliseconds) by which return value of
	 * {@linkplain #getTimeInMillis()} will advance every time it is called.
	 * 
	 * @param advance
	 *            Mock time provider's auto advance (in milliseconds).
	 */
	public void setAutoAdvance(long advance) {
		this.advance = advance;
	}

	/**
	 * Set the current time, which will be returned the next time when
	 * {@link #getTimeInMillis()} is called.
	 * 
	 * @param millis
	 *            Current time.
	 */
	public void setTimeInMillis(long millis) {
		this.millis = millis;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.cosylab.util.TimeProvider#getTimeInMillis()
	 */
	public long getTimeInMillis() {
		// Time to return.
		long ret = this.millis;
		// Auto-advance for the next time we are called.
		this.millis += this.advance;
		return ret;
	}
}
