/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of Java-Common.
 *
 * Java-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Java-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Java-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.util;

/**
 * Interface that defines a task that can be run by Scheduler.
 *
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 * @deprecated Use concurency from Java 1.5 instead.
 */
@Deprecated
public interface SchedulerTask extends Runnable
{
	/**
	 * This task should be canceled at first available oportunity.
	 */
	void cancel();

	/**
	 * Returns true if this task is scheduled for canceling. One canceled
	 * returns true, this task will never be executed again. If task is
	 * running currently, it will finish, then be destroyed.
	 *
	 * @return True if the task should be destroyed.
	 */
	boolean isCanceled();

	/**
	 * Returns name of this task. This name is for informative purposes only.
	 * Can be null. If specified it can aid in debugging, since the thread
	 * executing this task can be identified.
	 *
	 * @return Name Name of this task or null.
	 */
	String getTaskName();

	/**
	 * Interval between two executions. This value indicates the desired time
	 * between to calls to run() method. If value is negative, the task will
	 * be executed at next available oportunity.
	 * 
	 * <p>
	 * If this value is changed while task is already scheduled, changes will
	 * take effect after this task is run next time.
	 * </p>
	 *
	 * @return Time in miliseconds.
	 */
	long getInterval();

	/**
	 * Method that runs this task. Deliberately made to clash with Thread.run()
	 * to prevent this class to be implemented as a Thread or Runnable.
	 */
	void run();
}

/* __oOo__ */
