/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of Java-Common.
 *
 * Java-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Java-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Java-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.util;

import java.lang.reflect.Array;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import com.cosylab.introspection.ClassIntrospector;

/**
 * 
 * @author <a href="mailto:miha.kadunc@cosylab.com">Miha Kadunc</a>
 * @version $id$
 * 
 * @deprecated renamed to StringUtilities
 */

@Deprecated
public final class StringHelper {

    /**
     * Constructor for StringHelper.
     */
    private StringHelper() {

    }
    public static Object fromString(String sValue, Class targetClass) throws CommonException {
        try {
            if ((sValue == null) || (sValue.equals("")))
                return targetClass.newInstance();
            if (targetClass.isArray())
                return arrayFromString(sValue, targetClass);
            if (targetClass.equals(String.class))
                return sValue;
            else if (
                Number.class.isAssignableFrom(targetClass)
                    || Boolean.class.isAssignableFrom(targetClass)) {
                return targetClass.getMethod("valueOf", new Class[] { String.class }).invoke(
                    null,
                    new Object[] { sValue });
            } else if (targetClass.isPrimitive()) {
                // we are not able to return Object of a primitive type so use its class implementation
                if (Boolean.TYPE.equals(targetClass))
                    return Boolean.valueOf(sValue);
                if (Short.TYPE.equals(targetClass))
                    return Short.valueOf(sValue);
                if (Byte.TYPE.equals(targetClass))
                    return Byte.valueOf(sValue);
                if (Character.TYPE.equals(targetClass))
                    return new Character(sValue.charAt(0));
                if (Integer.TYPE.equals(targetClass))
                    return Integer.valueOf(sValue);
                if (Long.TYPE.equals(targetClass))
                    return Long.valueOf(sValue);
                if (Float.TYPE.equals(targetClass))
                    return Float.valueOf(sValue);
                if (Double.TYPE.equals(targetClass))
                    return Double.valueOf(sValue);
                return null;
            } else if (Class.class == targetClass) {
                // in case of primitive types return its class
                return classFromString(sValue);
            }
            // check if the class has the valueOf(String) method and call it with given parameter if so
            // the example is in case of DataProducer which is instantiated by the parameters from the xml file
            else {
                Method met =
                    ClassIntrospector.getMethod(
                        targetClass,
                        "valueOf",
                        new Class[] { String.class });
                if (met != null) {
                    return met.invoke(null, sValue);
                }
                return targetClass.newInstance();
            }
        } catch (NoSuchMethodException e) {
            throw new CommonException(
                ArrayHelper.class,
                "Objects could not be instantiated from strings",
                e);
        } catch (IllegalAccessException e) {
            throw new CommonException(
                ArrayHelper.class,
                "Objects could not be instantiated from strings",
                e);
        } catch (InvocationTargetException e) {
            throw new CommonException(
                ArrayHelper.class,
                "Objects could not be instantiated from strings",
                e);
        } catch (InstantiationException e) {
            throw new CommonException(
                ArrayHelper.class,
                "Objects could not be instantiated from strings",
                e);
        }

    }

    public static Class classFromString(String sValue) throws CommonException {
        try {
            if (sValue.equals(Boolean.TYPE.getName()))
                return Boolean.TYPE;
            if (sValue.equals(Short.TYPE.getName()))
                return Short.TYPE;
            if (sValue.equals(Byte.TYPE.getName()))
                return Byte.TYPE;
            if (sValue.equals(Character.TYPE.getName()))
                return Character.TYPE;
            if (sValue.equals(Integer.TYPE.getName()))
                return Integer.TYPE;
            if (sValue.equals(Long.TYPE.getName()))
                return Long.TYPE;
            if (sValue.equals(Float.TYPE.getName()))
                return Float.TYPE;
            if (sValue.equals(Double.TYPE.getName()))
                return Double.TYPE;
            if (sValue.equals(Void.TYPE.getName()))
                return Void.TYPE;
            return Class.forName(sValue);
        } catch (ClassNotFoundException e) {
            throw new CommonException(
                ArrayHelper.class,
                "Class could not be instantiated from a String",
                e);
        }
    }
    public static Object arrayFromString(String arrayString, Class arrayClass) throws CommonException {
            return arrayFromString(arrayString, arrayClass, ",", true);
        }

        public static Object arrayFromString(String arrayString, Class arrayClass,String separator,boolean parentheses) throws CommonException {
            String[] pStrings = arrayString.split(separator);
            if (parentheses) {
                pStrings[0] = pStrings[0].substring(1);
                pStrings[pStrings.length-1] = pStrings[pStrings.length-1].substring(0, pStrings[pStrings.length-1].length() - 1);
            }

            Object retArray = Array.newInstance(arrayClass.getComponentType(), pStrings.length);
            for (int i = 0; i < pStrings.length; i++) {
                Array.set(retArray,i,StringHelper.fromString(pStrings[i], arrayClass.getComponentType()));
            }
            return retArray;
        }
    

}
