/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of Java-Common.
 *
 * Java-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Java-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Java-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.util;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.util.Locale;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;


/**
 * This class provides converter from any character to unicode escape code. The
 * GUI for the encoder is added as inner class.
 *
 * @author <a href="mailto:gasper.pajor@cosylab.com">Gasper Pajor</a>
 * @version $id$
 */
public class StringUnicoder
{
	private class StringUnicoderGUI extends JFrame implements ActionListener
	{
		/**
         * 
         */
        private static final long serialVersionUID = 2375092085826069339L;
        private boolean convertAscii = true;
		private JScrollPane scrollGarble = new JScrollPane();
		private JScrollPane scrollUnicode = new JScrollPane();
		private JTextArea textGarble = new JTextArea();
		private JTextArea textUnicode = new JTextArea();
		private JButton doItButton = new JButton("Transform!");
		private JCheckBox refactorAsciiCheckBox = new JCheckBox("Convert ASCII",
			    convertAscii);
		private JPanel commandPanel = new JPanel(new GridBagLayout());

		/**
		 * Creates a new StringUnicoderGUI object.
		 */
		public StringUnicoderGUI()
		{
			super();
			initialize();
		}

		private void initialize()
		{
			getContentPane().setLayout(new GridBagLayout());
			constructCommandPanel();

			GridBagConstraints c = new GridBagConstraints(0, 0, 1, 1, 1, 0,
				    GridBagConstraints.WEST, GridBagConstraints.NONE,
				    new Insets(4, 4, 4, 4), 0, 0);
			getContentPane().add(new JLabel("Normal Text:"), c);

			c.gridy = 1;
			c.weighty = 1;
			c.fill = GridBagConstraints.BOTH;
			scrollGarble.setViewportView(textGarble);
			getContentPane().add(scrollGarble, c);

			c.gridy = 2;
			c.weighty = 0;
			getContentPane().add(commandPanel, c);

			c.gridy = 3;
			getContentPane().add(new JLabel("Unicode Escape Format:"), c);

			c.gridy = 4;
			c.weighty = 1;
			textUnicode.setEditable(false);
			scrollUnicode.setViewportView(textUnicode);
			getContentPane().add(scrollUnicode, c);

			setSize(600, 300);
			setLocation(100, 100);
			setTitle("StringUnicoder -- Cosylab");
		}


		private void constructCommandPanel()
		{
			GridBagConstraints c = new GridBagConstraints(0, 0, 1, 1, 0, 0,
				    GridBagConstraints.CENTER, GridBagConstraints.NONE,
				    new Insets(4, 4, 4, 4), 0, 0);

			doItButton.addActionListener(this);
			commandPanel.add(refactorAsciiCheckBox, c);

			c.gridy = 1;
			refactorAsciiCheckBox.addActionListener(this);
			commandPanel.add(doItButton, c);
		}

		/**
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e)
		{
			if (e.getSource() == doItButton) {
				textUnicode.setText(convertUnicodeString(textGarble.getText(),
				        convertAscii));
			}

			if (e.getSource() == refactorAsciiCheckBox) {
				convertAscii = refactorAsciiCheckBox.isSelected();
			}
		}
	}
	

	/**
	 * Run test applet.
	 *
	 * @param args command line parameters
	 */
	public static void main(String[] args)
	{
		StringUnicoder s = new StringUnicoder();
		StringUnicoderGUI sugui = s.new StringUnicoderGUI();
		sugui.setVisible(true);
	}

	/**
	 * Converts input string into unicode escape code.
	 *
	 * @param str the string to be converted
	 * @param convertAscii wheather to also convert ascii charcters
	 *
	 * @return converted String
	 */
	public static String convertUnicodeString(String str, boolean convertAscii)
	{
		String ostr = new String();

		for (int i = 0; i < str.length(); i++) {
			char ch = str.charAt(i);

			if (!convertAscii && ((ch >= 0x0020) && (ch <= 0x007e))) {
				ostr += ch;

				//System.out.println("character "+ch+" was not transformed, it is ASCII char");
			} else {
				ostr += "\\u";

				String hex = Integer.toHexString(str.charAt(i) & 0xFFFF);

				if (hex.length() == 2) {
					ostr += "00";
				}

				ostr += hex.toUpperCase(Locale.ENGLISH);
			}
		}

		return ostr;
	}
}

/* __oOo__ */
