/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of Java-Common.
 *
 * Java-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Java-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Java-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.util;

import java.io.File;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;


/**
 * A simple runnable class that transforms a stylesheet and an XML document
 * into a HTML document. It works by using default Java 1.4 XML transformer.
 */
public class XMLTransformer
{
	/**
	 * Starts the transformer and transforms XML document. As an input, the
	 * main method takes two string arguments:
	 * 
	 * <ul>
	 * <li>
	 * <b>The first argument </b> specifies the full name (filename and path)
	 * of the XSL transformation stylesheet.
	 * </li>
	 * <li>
	 * <b>The second argument </b> specifies the full name (filename and path)
	 * of the XML file that is to be transformed into HTML.
	 * </li>
	 * </ul>
	 * 
	 * As a result, the program places the HTML file into the directory, where
	 * the source XML was found. The resulting filename is the same as the
	 * filename of the input XML file, except that the resulting file has HTML
	 * extension instead  of XML.
	 *
	 * @param args DOCUMENT ME!
	 */
	public static void main(String[] args)
	{
		try {
			TransformerFactory tf = TransformerFactory.newInstance();

			// Transformer t = tf.newTransformer(new StreamSource(new File(args[0])));
			// String result = args[1].substring(0, args[1].lastIndexOf('.')) + ".html";
			// t.transform(new StreamSource(new File(args[1])), new StreamResult(new File(result)));
			Transformer t = tf.newTransformer();
			String result = args[0].substring(0, args[0].lastIndexOf('.'))
				+ ".html";
			t.transform(new StreamSource(new File(args[0])),
			    new StreamResult(new File(result)));
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}

/* __oOo__ */
