package de.desy.acop.about;

import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import com.cosylab.application.AutoScalable;

/**
 * 
 * <code>AboutDialog</code> is a dialog which presents information about the 
 * software. It can be used in combination with any application.
 *
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 *
 */
@AutoScalable
public class AboutDialog extends JDialog {

	private static final long serialVersionUID = 1L;
	private JPanel contentPanel;
	private JButton closeButton;
	private JTabbedPane tabPane;
	
	private ProgramTabPanel programTabPanel;
	private SystemTabPanel systemTabPanel;
	
	private ApplicationInfo info;
	
	/**
	 * Constructs a new AboutDialog.
	 * @param owner the parent frame
	 * @param info the application info to be presented in the dialog
	 */
	public AboutDialog(Frame owner, ApplicationInfo info) {
		super(owner);
		this.info = info;
		initialize();
	}
	
	/**
	 * Initializes the layout.
	 *
	 */
	private void initialize() {
		setSize(500,400);
		setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		setModal(true);
		setTitle("About");
		setContentPane(getContentPanel());
	}
	
	/**
	 * Returns the content panel of this dialog.
	 * 
	 * @return the content panel
	 */
	private JPanel getContentPanel() {
		if (contentPanel == null) {
			contentPanel = new JPanel(new GridBagLayout());
			contentPanel.add(getTabPanel(), new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.CENTER,GridBagConstraints.BOTH, new Insets(5,5,5,5),0,0));
			contentPanel.add(getCloseButton(), new GridBagConstraints(0,1,1,1,0,0,GridBagConstraints.EAST,GridBagConstraints.NONE, new Insets(0,5,5,5),0,0));
		}
		return contentPanel;
	}

	/**
	 * Returns the tab pane with all the about tabs.
	 * 
	 * @return the tab pane
	 */
	private JTabbedPane getTabPanel() {
		if (tabPane == null) {
			tabPane = new JTabbedPane();
			tabPane.add("Program", getProgramTabPanel());
			tabPane.add("System", getSystemTabPanel());
		}
		return tabPane;
	}
	
	/**
	 * Returns the program tab panel.
	 * 
	 * @return the program tab panel
	 */
	private ProgramTabPanel getProgramTabPanel() {
		if (programTabPanel == null) {
			programTabPanel = new ProgramTabPanel(info);
		}
		return programTabPanel;
	}
	
	/**
	 * Returns the system tab panel.
	 * 
	 * @return the system tab panel
	 */
	private SystemTabPanel getSystemTabPanel() {
		if (systemTabPanel == null) {
			systemTabPanel = new SystemTabPanel();
		}
		return systemTabPanel;
	}
	
	/**
	 * Returns the close button. On action this button closes the about dialog.
	 * 
	 * @return the close button
	 */
	private JButton getCloseButton() {
		if (closeButton == null) {
			closeButton = new JButton("Close");
			closeButton.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					dispose();
				}
			});
		}
		return closeButton;		
	}
	
	/**
	 * Sets the title fot his dialog. This should be the name of the application.
	 * @param applicationName the title
	 */
	public void setApplicationTitle(String applicationName) {
		getProgramTabPanel().setTitle(applicationName);
	}
	
	/**
	 * Sets the name, email and phone of the responsible person for this application.
	 * 
	 * @param responsibleName the responsible person
	 * @param email the email of the responsible person
	 * @param phone the phone number of the responsible person
	 */
	public void setResponsible(String responsibleName, String email, String phone) {
		getProgramTabPanel().setResponsible(responsibleName, email, phone);
	}
	
	/**
	 * Sets the url where data about the application can be found.
	 * @param url
	 */
	public void setApplicationURL(String url) {
		getProgramTabPanel().setApplicationURL(url);
	}
	
	public static void main(String[] args) {
		new AboutDialog(null,new ApplicationInfo(new java.util.HashMap<String,String>())).setVisible(true);
	}
}
