package de.desy.acop.about;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Desktop;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.Insets;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;

import com.cosylab.gui.components.HyperLinkLabel;
import com.cosylab.gui.components.util.IconHelper;

/**
 * 
 * <code>ProgramTabPanel</code> is the main tab for the about dialog. It provides
 * information about the application.
 *
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 *
 */
public class ProgramTabPanel extends JPanel {

	private static final long serialVersionUID = 1L;
	private JPanel authorPanel;
	private JPanel mainPanel;
	private JLabel responsibleLabel;
	private JLabel phoneLabel;
	private JPanel contactPanel;
	private JLabel titleLabel;
	private JPanel releasePanel;
	private HyperLinkLabel urlLabel;
	private URI mailURI;
	
	private ApplicationInfo applicationInfo;
	
	
	/**
	 * Constructs a new ProgramTabPanel.
	 *
	 * @param info the application info which is the source of data for this panel
	 */
	public ProgramTabPanel(ApplicationInfo info) {
		super();
		this.applicationInfo = info;
		initialize();
	}
	
	/**
	 * Initializes the layout.
	 *
	 */
	private void initialize() {
		this.setLayout(new GridBagLayout());
		add(getTitleLabel(), new GridBagConstraints(0,0,2,1,0,0,GridBagConstraints.CENTER,GridBagConstraints.NONE,new Insets(11,11,5,11),0,0));
		add(getReleasePanel(), new GridBagConstraints(0,1,2,1,0,0,GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(5,11,5,11),0,0));
		add(getContactPanel(), new GridBagConstraints(0,2,2,1,1,0,GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(5,11,11,11),0,0));
		add(getMainPanel(), new GridBagConstraints(0,3,1,1,1,0,GridBagConstraints.SOUTH,GridBagConstraints.BOTH,new Insets(5,11,5,11),0,0));
		add(getAuthorPanel(), new GridBagConstraints(1,3,1,1,1,0,GridBagConstraints.SOUTH,GridBagConstraints.BOTH,new Insets(5,11,11,11),0,0));
	}
	
	private JPanel getReleasePanel() {
		if (releasePanel == null) {
			releasePanel = new JPanel(new GridBagLayout());			
			releasePanel.add(new JLabel("Release Version: "), new GridBagConstraints(0,1,1,1,0,1,GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(5,11,5,5),0,0));
			releasePanel.add(new JLabel(applicationInfo.getVersion()), new GridBagConstraints(1,1,1,1,1,1,GridBagConstraints.WEST,GridBagConstraints.NONE, new Insets(5,5,5,11),0,0));
			releasePanel.add(new JLabel("Release Date: "), new GridBagConstraints(0,2,1,1,0,1,GridBagConstraints.WEST,GridBagConstraints.NONE, new Insets(5,11,5,5),0,0));
			releasePanel.add(new JLabel(applicationInfo.getDate()), new GridBagConstraints(1,2,1,1,1,1,GridBagConstraints.WEST,GridBagConstraints.NONE, new Insets(5,5,5,11),0,0));
			
			
		}
		return releasePanel;
	}
	
	private JPanel getContactPanel() {
		if (contactPanel == null) {
			contactPanel = new JPanel(new GridBagLayout());
			contactPanel.add(getURLLabel(), new GridBagConstraints(0,0,2,1,1,1,GridBagConstraints.WEST,GridBagConstraints.NONE, new Insets(5,11,5,11),0,0));
			contactPanel.add(new JLabel("Responsible: "), new GridBagConstraints(0,1,1,1,0,1,GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(5,11,5,5),0,0));
			contactPanel.add(getResponsibleLabel(), new GridBagConstraints(1,1,1,1,1,1,GridBagConstraints.WEST,GridBagConstraints.NONE, new Insets(5,5,5,11),0,0));
			contactPanel.add(new JLabel("Phone: "), new GridBagConstraints(0,2,1,1,0,1,GridBagConstraints.WEST,GridBagConstraints.NONE, new Insets(5,11,5,5),0,0));
			contactPanel.add(getPhoneLabel(), new GridBagConstraints(1,2,1,1,1,1,GridBagConstraints.WEST,GridBagConstraints.NONE, new Insets(5,5,5,11),0,0));
		}
		return contactPanel;
	}
	
	private JLabel getResponsibleLabel() {
		if (responsibleLabel == null) {
			responsibleLabel = new JLabel();
			responsibleLabel.setForeground(Color.BLUE);
			responsibleLabel.addMouseListener(new MouseAdapter(){
				@Override
				public void mouseClicked(MouseEvent e) {
					if (mailURI == null) return;
					try {
						Desktop.getDesktop().mail(mailURI);
					} catch (IOException e1) {
						e1.printStackTrace();
					}
				}
				@Override
				public void mouseEntered(MouseEvent e){
					e.getComponent().setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
			    }
			});
		}
		return responsibleLabel;
	}
	
	private JLabel getPhoneLabel() {
		if (phoneLabel == null) {
			phoneLabel = new JLabel();
		}
		return phoneLabel;
	}
	
	private JLabel getTitleLabel() {
		if (titleLabel == null) {
			titleLabel = new JLabel();
			titleLabel.setFont(titleLabel.getFont().deriveFont(titleLabel.getFont().getSize() * 1.8f));
		}
		return titleLabel;
	}
	
	private HyperLinkLabel getURLLabel() {
		if (urlLabel == null) {
			urlLabel = new HyperLinkLabel();
			
		}
		return urlLabel;
	}
	
	/**
	 * Sets the title of this program tab.
	 * 
	 * @param title
	 */
	public void setTitle(String title) {
		getTitleLabel().setText(title);
	}
	
	/**
	 * Sets the application url. The url where the help for the application 
	 * is located.
	 * @param url
	 */
	public void setApplicationURL(String url) {
		urlLabel.setText(url);
		try {
			urlLabel.setHyperLink(url);
		} catch (URISyntaxException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Sets the properties of the responsible person.
	 * 
	 * @param name the name of the person
	 * @param email the email of the person
	 * @param phone the phone number of the person
	 */
	public void setResponsible(String name, String email, String phone) {
		if (name == null) {
			name = "Unknown";
		}
		String mail;
		if (email == null) {
			mail = "Unknown e-mail";
		} else {
			mail = email;
		}
		getResponsibleLabel().setText(name + " (" + mail +")");
		getPhoneLabel().setText(phone);
		try {
			if (email != null) {
				StringBuilder sb = new StringBuilder("mailto:");
				sb.append(email);
				sb.append("?subject=");
				sb.append(getTitleLabel().getText().replace(" ", "%20"));
				getResponsibleLabel().setToolTipText(sb.toString());
				mailURI = new URI(sb.toString());
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Returns the panel which describes this software and its owner.
	 * 
	 * @return the main panel
	 */
	private JPanel getMainPanel() {
		if (mainPanel == null) {
			try {
				mainPanel = new JPanel(new GridBagLayout());
				Icon icon = IconHelper.createIcon("desy_logo.gif");
				int width = 80, height = 80;
				icon = new ImageIcon(((ImageIcon)icon).getImage().getScaledInstance(
						width, -1, Image.SCALE_SMOOTH));
				if (icon.getIconHeight() > height) {
					if (height <= 0) height = 1;
					icon = new ImageIcon(((ImageIcon)icon).getImage().getScaledInstance(
							-1, height, Image.SCALE_SMOOTH));
				}
				JLabel desyLogo = new JLabel(icon);
				
				HyperLinkLabel linkLabel = new HyperLinkLabel("www.desy.de", "http://www.desy.de");
				linkLabel.setFont(linkLabel.getFont().deriveFont(linkLabel.getFont().getSize() * 0.8f).deriveFont(Font.PLAIN));
				mainPanel.add(desyLogo, new GridBagConstraints(0,0,1,1,0,0,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0,0,1,0),0,0));
				mainPanel.add(linkLabel, new GridBagConstraints(0,1,1,1,0,0,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0,0,0,0),0,0));
			} catch(URISyntaxException e) {
				e.printStackTrace();
			}
		}
		return mainPanel;		
	}
	
	/**
	 * Returns the panel which describes the author of the software.
	 * 
	 * @return the author panel
	 */
	private JPanel getAuthorPanel() {
		if (authorPanel == null) {
			authorPanel = new JPanel(new GridBagLayout());
			try {
				JLabel cosyLogo = new JLabel(IconHelper.createIcon("logo_cosylab.gif"));
				authorPanel.add(cosyLogo, new GridBagConstraints(0,1,1,1,0,1,GridBagConstraints.CENTER,GridBagConstraints.NONE, new Insets(5,5,1,5),0,0));
				
				Font f = cosyLogo.getFont().deriveFont(cosyLogo.getFont().getSize() * 0.8f).deriveFont(Font.PLAIN);
//				JLabel l1 = new JLabel("(c) Copyright Cosylab, 2007-2011. All Right Reserved.");
//				l1.setFont(f);
				JLabel l2 = new HyperLinkLabel("www.cosylab.com", "http://www.cosylab.com");
				l2.setFont(f);
				
				authorPanel.add(l2, new GridBagConstraints(0,3,1,1,0,0,GridBagConstraints.CENTER,GridBagConstraints.NONE, new Insets(0,0,0,0),0,0));
			} catch (URISyntaxException e) {
				e.printStackTrace();
			}
			
		}
		return authorPanel;
	}
}
