package de.desy.acop.about;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.net.URISyntaxException;
import java.util.Properties;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import com.cosylab.gui.components.HyperLinkLabel;


/**
 * 
 * <code>SystemTabPanel</code> about dialog panel with system information.
 *
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 *
 */
public class SystemTabPanel extends JPanel {
	
	private static final long serialVersionUID = 1L;

	private Properties properties;
	
	private JLabel systemOSPropertyLabel;
	private HyperLinkLabel systemJavaPropertyLabel;
	private JTextArea systemJavaClasspathPropertyArea;
	private JTextArea systemJavaDirPropertyArea;
	
	private JScrollPane classpathScroll;
	private JScrollPane dirScroll;
	
	/**
	 * Constructs a new panel with system information.
	 *
	 */
	public SystemTabPanel() {
		super();
		properties = System.getProperties();
		initialize();
		initializeData();
	}
	
	/**
	 * Initializes the layout.
	 *
	 */
	private void initialize() {
		setLayout(new GridBagLayout());
		JLabel titleLabel = new JLabel("System Information");
		titleLabel.setFont(titleLabel.getFont().deriveFont(18f));
		add(titleLabel, new GridBagConstraints(0,0,2,1,0,0,GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(11,11,5,11),0,0));
		add(new JLabel("Operating System:"), new GridBagConstraints(0,1,1,1,0,0,GridBagConstraints.EAST,GridBagConstraints.NONE, new Insets(5,11,5,5),0,0));
		add(new JLabel("Java Version:"), new GridBagConstraints(0,2,1,1,0,0,GridBagConstraints.EAST,GridBagConstraints.NONE, new Insets(5,11,5,5),0,0));
		add(new JLabel("Java Directory:"), new GridBagConstraints(0,3,1,1,0,0,GridBagConstraints.EAST,GridBagConstraints.NONE, new Insets(5,11,5,5),0,0));
		add(new JLabel("Java Classpath:"), new GridBagConstraints(0,4,1,1,0,1,GridBagConstraints.EAST,GridBagConstraints.NONE, new Insets(5,11,5,5),0,0));
		add(getSystemOSPropertyLabel(), new GridBagConstraints(1,1,1,1,1,0,GridBagConstraints.WEST,GridBagConstraints.NONE, new Insets(5,5,5,11),0,0));		
		add(getSystemJavaPropertyLabel(), new GridBagConstraints(1,2,1,1,1,0,GridBagConstraints.WEST,GridBagConstraints.NONE, new Insets(5,5,5,11),0,0));
		add(getDirScroll(), new GridBagConstraints(1,3,1,1,1,0,GridBagConstraints.WEST,GridBagConstraints.BOTH, new Insets(5,5,5,11),0,0));
		add(getClasspathScroll(), new GridBagConstraints(1,4,1,1,1,1,GridBagConstraints.WEST,GridBagConstraints.BOTH, new Insets(5,5,11,11),0,0));

	}
	
	/**
	 * Returns the scroll pane for the java directory.
	 * 
	 * @return the java directory scroll
	 */
	private JScrollPane getDirScroll() {
		if (dirScroll == null) {
			dirScroll = new JScrollPane(getSystemJavaDirPropertyArea());
		}
		return dirScroll;
	}
	
	/**
	 * Returns the scroll pane for the java class path area.
	 * 
	 * @return the java class path scroll
	 */
	private JScrollPane getClasspathScroll() {
		if (classpathScroll == null) {
			classpathScroll = new JScrollPane(getSystemJavaClasspathPropertyArea());
		}
		return classpathScroll;
	}
	
	/**
	 * Returns the label with OS info.
	 * 
	 * @return the os info label
	 */
	private JLabel getSystemOSPropertyLabel()
	{
		if (systemOSPropertyLabel == null) {
			systemOSPropertyLabel = new JLabel();
			systemOSPropertyLabel.setText("os");
		}						
		return systemOSPropertyLabel;
	}
	
	/**
	 * Returns the label with java info.
	 * 
	 * @return the java info label
	 */
	private HyperLinkLabel getSystemJavaPropertyLabel()
	{
		if (systemJavaPropertyLabel == null) {
			systemJavaPropertyLabel = new HyperLinkLabel();
			try {
				systemJavaPropertyLabel.setHyperLink(getVendorURI());
			} catch (URISyntaxException e) {
				e.printStackTrace();
			}
			systemJavaPropertyLabel.setText("Java Version");
		}

		return systemJavaPropertyLabel;
	}
	
	/**
	 * Returns the text area with class path info.
	 * 
	 * @return the class path info area
	 */
	private JTextArea getSystemJavaClasspathPropertyArea()
	{
		if (systemJavaClasspathPropertyArea == null) {
			systemJavaClasspathPropertyArea = new JTextArea();
			systemJavaClasspathPropertyArea.setBounds(0, 0, 160, 120);
			systemJavaClasspathPropertyArea.setMargin(new java.awt.Insets(
			        1, 3, 1, 3));
			systemJavaClasspathPropertyArea.setEditable(false);
		
		}

		return systemJavaClasspathPropertyArea;
	}
	
	/**
	 * Returns the area with java direcory info.
	 * 
	 * @return java directory area
	 */
	private JTextArea getSystemJavaDirPropertyArea()
	{
		if (systemJavaDirPropertyArea == null) {
			systemJavaDirPropertyArea = new JTextArea();
			systemJavaDirPropertyArea.setBounds(0, 0, 160, 120);
			systemJavaDirPropertyArea.setMargin(new java.awt.Insets(
			        1, 3, 1, 3));
			systemJavaDirPropertyArea.setEditable(false);
		}

		return systemJavaDirPropertyArea;
	}
	
	/**
	 * Returns the URL of the java vednor's web site.
	 * 
	 * @return the vednor's url
	 */
	private String getVendorURI() {
		return properties.getProperty("java.vendor.url");
	}
	
	/**
	 * Returns the name of the operating system.
	 * 
	 * @return operating system
	 */
	private String getOSName() {
		return properties.getProperty("os.name");
	}
	
	/**
	 * Returns the version of the operating system.
	 *  
	 * @return operating system version
	 */
	private String getOSVersion() {
		return properties.getProperty("os.version");
	}
	
	private String getOSArch() {
		return properties.getProperty("os.arch");
	}
	
	/**
	 * Returns the version of java.
	 * 
	 * @return java version
	 */
	private String getJavaVersion() {
		return properties.getProperty("java.version");
	}
	
	/**
	 * Returns the vednor of the java JVM.
	 * 
	 * @return the java vendor
	 */
	private String getJavaVendor() {
		return properties.getProperty("java.vendor");
	}
	
	/**
	 * Returns the java class path.
	 * 
	 * @return the java class path
	 */
	private String getJavaClasspath() {
		return properties.getProperty("java.class.path");
	}
	
	/**
	 * Returns the java home directory.
	 * 
	 * @return java home directory
	 */
	private String getJavaDir() {
		return properties.getProperty("java.home");
	}
	
	/**
	 * Sets the values to the labels and areas.
	 *
	 */
	private void initializeData() {
		getSystemOSPropertyLabel().setText(getOSName() + " " + getOSVersion() + " (" + getOSArch() + ")");
		getSystemJavaPropertyLabel().setText("Java " + getJavaVersion() + " by " + getJavaVendor());
		getSystemJavaClasspathPropertyArea().setText(getJavaClasspath().replaceAll(System.getProperty("path.separator"),"\n"));
		getSystemJavaDirPropertyArea().setText(getJavaDir());
		
		getClasspathScroll().getHorizontalScrollBar().setValue(0);
		getClasspathScroll().getVerticalScrollBar().setValue(0);
		
		getDirScroll().getHorizontalScrollBar().setValue(0);
		getDirScroll().getVerticalScrollBar().setValue(0);
	}
	
}
