/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.chart;
import java.awt.BasicStroke;
import java.awt.Color;

public class AcopAxis
{   
  protected  int 	axisIndex;       	// own axis index number
  protected  int 	npoints;    		// display points
  protected  boolean	axisON;			// on or off
  
  protected	double[] Position;          // input display data
  protected	Color[]  penColor;
  protected int[]	 penWidth, penStyle;
  protected BasicStroke axisStroke;
  private int strokeWidth, strokeStyle;
  
  private Acop acop;

  public AcopAxis(Acop acop, int index)
  {
    this.acop = acop;
    npoints = 0;
    axisON = false;
    axisIndex = index;
  }

  protected int loadData(int size, Object PositionArray, Color[] ColorArray, int[] StyleArray, int[] WidthArray)
  { int i, ndata, dataChanged, penChange;
    
    dataChanged = 0;
    penChange = 0;
    
    if ( npoints < size )
    {
      dataChanged++;
      npoints = size;
      Position = new double[npoints];
      penColor = new Color[npoints];
      penWidth = new int[npoints];
      penStyle = new int[npoints];
      for (i=0; i<npoints; i++)
      {
        penColor[i] = acop.sca[0].gridLineColor;
        Position[i] = 0.0;
        penWidth[i] = 1;
        penStyle[i] = AcopConst.PS_DOT;
      }
      penChange++;
    }
    ndata = AcopHisto.getArrayLength(PositionArray);
    if ( ndata > 0 )
    {
      dataChanged++;
      if ( npoints < ndata ) ndata = npoints;
      AcopHisto.copyArray(Position, PositionArray, 0, ndata, ndata, 1);
    }
    ndata = AcopHisto.getArrayLength(ColorArray);
    if ( ndata > 0 )
    {
      dataChanged++;
      if ( npoints < ndata ) ndata = npoints;
      for(i=0;i<ndata;i++) penColor[i] = ColorArray[i];
    }
    ndata = AcopHisto.getArrayLength(StyleArray);
    if ( ndata > 0 )
    {
      dataChanged++;
      if ( penStyle[0] != StyleArray[0] ) penChange++;      
      if ( npoints < ndata ) ndata = npoints;
      for(i=0;i<ndata;i++)penStyle[i] = StyleArray[i];
    }
    ndata = AcopHisto.getArrayLength(WidthArray);
    if ( ndata > 0 )
    {
      dataChanged++;
      if ( penWidth[0] != WidthArray[0] ) penChange++;      
      if ( npoints < ndata ) ndata = npoints;
      for(i=0;i<ndata;i++)penWidth[i] = WidthArray[i];
    }
    if ( penChange != 0 ) getStroke(0);
    return dataChanged;
  }

  private void getStroke(int i)
  {
    strokeWidth = penWidth[i];
   	strokeStyle = penStyle[i];
    axisStroke = AcopConst.createStroke(strokeWidth,strokeStyle);
  }

  protected void drawAxis(java.awt.Graphics2D a)
  { int xtemp, ytemp;
    double tmp;
    Color lastColor;
     
    if ( axisON == false || npoints <= 0 ) return;
	  
	a.setColor(penColor[0]);
	lastColor = penColor[0];
	a.setStroke(axisStroke);

    for ( int i=0; i<npoints; i++ )
	{
      tmp = acop.sca[axisIndex].log != 0 ? Math.log( AcopConst.log_check(Position[i]) )/Math.log(10) : Position[i];
	  if ( tmp < acop.sca[axisIndex].min || tmp > acop.sca[axisIndex].max ) continue;

      if ( penColor[i] != lastColor )
      {
		a.setColor(penColor[i]);
		lastColor = penColor[i];
      }
      if ( strokeWidth != penWidth[i] || strokeStyle != penStyle[i] )
      {
        getStroke(i);
    	a.setStroke(axisStroke);
      }
      if ( axisIndex == 0 )	// xaxis
      {
        xtemp = (int)((tmp - acop.sca[axisIndex].min) / acop.sca[axisIndex].dispSize * acop.aFrame.dxwidth + acop.aFrame.xleft + 0.01);
        a.drawLine(xtemp, acop.aFrame.histRect.height, xtemp, 0);
	  }
	  else
	  {
	    ytemp = (int)(acop.aFrame.histRect.height - ( (tmp - acop.sca[axisIndex].min) / acop.sca[axisIndex].dispSize * acop.aFrame.histRect.height ));
        a.drawLine(acop.aFrame.xleft, ytemp, acop.aFrame.dxwidth, ytemp);
	  }
	}           
  }

}
