package de.desy.acop.chart;

import java.beans.BeanDescriptor;
import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.MethodDescriptor;
import java.beans.PropertyDescriptor;
//import java.beans.SimpleBeanInfo;

public class AcopBeanInfo extends java.beans.SimpleBeanInfo
{
  private final String ICON_FILENAME = "AcopIcon.gif";
  public AcopBeanInfo()
  {
  }

  /**
   * Gets a BeanInfo for the superclass of this bean.
   * @return BeanInfo[] containing this bean's superclass BeanInfo
   */
  public BeanInfo[] getAdditionalBeanInfo()
  {
    try
    {
    	BeanInfo[] bi = new BeanInfo[1];
    	bi[0] = Introspector.getBeanInfo(beanClass.getSuperclass());
    	return bi;
    }
    catch (IntrospectionException e)
    {
    	throw new Error(e.toString());
    }
  }

	/**
	* Gets the BeanDescriptor for this bean.
	* @return an object of type BeanDescriptor
	* @see java.beans.BeanDescriptor
	*/
	public BeanDescriptor getBeanDescriptor()
	{
		BeanDescriptor bd = new BeanDescriptor(beanClass);
    bd.setDisplayName("dispNameAcop");
    bd.setName("NameAcop");
    bd.setShortDescription("descriptionAcop");
		return bd;
	}

	/**
	 * Gets an image that may be used to visually represent this bean
	 * (in the toolbar, on a form, etc).
	 * @param iconKind the type of icon desired, one of: BeanInfo.ICON_MONO_16x16,
	 * BeanInfo.ICON_COLOR_16x16, BeanInfo.ICON_MONO_32x32, or BeanInfo.ICON_COLOR_32x32.
	 * @return an image for this bean
	 * @see BeanInfo#ICON_MONO_16x16
	 * @see BeanInfo#ICON_COLOR_16x16
	 * @see BeanInfo#ICON_MONO_32x32
	 * @see BeanInfo#ICON_COLOR_32x32
	 */
	public java.awt.Image getIcon(int nIconKind)
	{
    if ( nIconKind == BeanInfo.ICON_COLOR_16x16 ) 
    {
    	java.awt.Image img = loadImage(ICON_FILENAME);
      return img;
    } 
    else 
    {
  		System.out.println("AcopIcon only has a 16x16 icon.");
  		return null;
    }
//		java.awt.Image img = null;
//		return img;
	}
  public String getIconFilename()
  {
    return ICON_FILENAME;
  }
  /**
   * Return the method descriptors for this bean.
   * @return java.beans.MethodDescriptor[]
   */
  public MethodDescriptor[] getMethodDescriptors()
  {
    try {
      Class args[] = {Object.class, Object.class, Object.class, String[].class, int.class, int.class};
      MethodDescriptor Draw = new MethodDescriptor(
                 beanClass.getMethod("Draw",args));
      MethodDescriptor methods[] = {Draw};
      return methods;
    }
    catch (Exception e) {
         throw new Error(e.toString());
    }
  }
  /**
	 * Returns descriptions of this bean's properties.
	 */
	public PropertyDescriptor[] getPropertyDescriptors()
	{
		try{
      PropertyDescriptor caption = new PropertyDescriptor("caption",beanClass, "getCaption","setCaption");
      PropertyDescriptor captionLocation = new PropertyDescriptor("captionLocation",beanClass, "getCaptionLocation","setCaptionLocation");
			PropertyDescriptor frameWidth = new PropertyDescriptor("frameWidth",beanClass, "getFrameWidth","setFrameWidth");
			PropertyDescriptor frameLeftOffset = new PropertyDescriptor("frameLeftOffset",beanClass, "getFrameLeftOffset","setFrameLeftOffset");
			PropertyDescriptor frameRightOffset = new PropertyDescriptor("frameRightOffset",beanClass, "getFrameRightOffset","setFrameRightOffset");
			PropertyDescriptor frameTopOffset = new PropertyDescriptor("frameTopOffset",beanClass, "getFrameTopOffset","setFrameTopOffset");
			PropertyDescriptor frameBottomOffset = new PropertyDescriptor("frameBottomOffset",beanClass, "getFrameBottomOffset","setFrameBottomOffset");
      PropertyDescriptor frameForeColor = new PropertyDescriptor("frameForeColor",beanClass, "getFrameForeColor","setFrameForeColor");
      PropertyDescriptor xTickLabelColor = new PropertyDescriptor("xTickLabelColor",beanClass, "getXTickLabelColor","setXTickLabelColor");
      PropertyDescriptor yTickLabelColor = new PropertyDescriptor("yTickLabelColor",beanClass, "getYTickLabelColor","setYTickLabelColor");
      PropertyDescriptor secondaryYTickLabelColor = new PropertyDescriptor("secondaryYTickLabelColor",beanClass, "getSecondaryYTickLabelColor","setSecondaryYTickLabelColor");
      PropertyDescriptor secondaryYAxisOn = new PropertyDescriptor("secondaryYAxisOn",beanClass, "getSecondaryYAxisOn","setSecondaryYAxisOn");
      PropertyDescriptor secondaryYLogOn = new PropertyDescriptor("secondaryYLogOn",beanClass, "getSecondaryYLogOn","setSecondaryYLogOn");
			PropertyDescriptor displayMode = new PropertyDescriptor("displayMode",beanClass, "getDisplayMode","setDisplayMode");
//			displayMode.setConstrained(true);
//			displayMode.setBound(true);
			PropertyDescriptor xMin = new PropertyDescriptor("xMin",beanClass, "getXMin","setXMin");
			PropertyDescriptor xMax = new PropertyDescriptor("xMax",beanClass, "getXMax","setXMax");
			PropertyDescriptor yMin = new PropertyDescriptor("yMin",beanClass, "getYMin","setYMin");
			PropertyDescriptor yMax = new PropertyDescriptor("yMax",beanClass, "getYMax","setYMax");
      PropertyDescriptor secondaryYMin = new PropertyDescriptor("SecondaryYMin",beanClass, "getSecondaryYMin","setSecondaryYMin");
      PropertyDescriptor secondaryYMax = new PropertyDescriptor("SecondaryYMax",beanClass, "getSecondaryYMax","setSecondaryYMax");
			PropertyDescriptor drawWidth = new PropertyDescriptor("drawWidth",beanClass, "getDrawWidth","setDrawWidth");
			PropertyDescriptor frameFont = new PropertyDescriptor("frameFont",beanClass, "getFrameFont","setFrameFont");
			
			PropertyDescriptor drawStyle = new PropertyDescriptor("drawStyle",beanClass, "getDrawStyle","setDrawStyle");
//			drawStyle.setConstrained(true);
//			drawStyle.setBound(true);
      PropertyDescriptor averagingMode = new PropertyDescriptor("averagingMode",beanClass, "getAveragingMode","setAveragingMode");
      PropertyDescriptor averagingDepth = new PropertyDescriptor("averagingDepth",beanClass, "getAveragingDepth","setAveragingDepth");
			PropertyDescriptor FFT = new PropertyDescriptor("FFT",beanClass, "getFFT","setFFT");
			PropertyDescriptor graphStyle = new PropertyDescriptor("graphStyle",beanClass, "getGraphStyle","setGraphStyle");
			PropertyDescriptor gridStyle = new PropertyDescriptor("gridStyle",beanClass, "getGridStyle","setGridStyle");
			PropertyDescriptor gridColor = new PropertyDescriptor("gridColor",beanClass, "getGridColor","setGridColor");
			PropertyDescriptor gridWidth = new PropertyDescriptor("gridWidth",beanClass, "getGridWidth","setGridWidth");
			PropertyDescriptor keepStyle = new PropertyDescriptor("keepStyle",beanClass, "getKeepStyle","setKeepStyle");
			PropertyDescriptor keepColor = new PropertyDescriptor("keepColor",beanClass, "getKeepColor","setKeepColor");
			PropertyDescriptor keepWidth = new PropertyDescriptor("keepWidth",beanClass, "getKeepWidth","setKeepWidth");
			PropertyDescriptor histogramPosition = new PropertyDescriptor("histogramPosition",beanClass, "getHistogramPosition","setHistogramPosition");
			PropertyDescriptor invert = new PropertyDescriptor("invert",beanClass, "isInvert","setInvert");
      PropertyDescriptor randomData = new PropertyDescriptor("randomData",beanClass, "isRandomData","setRandomData");
      PropertyDescriptor randomText = new PropertyDescriptor("randomText",beanClass, "isRandomText","setRandomText");
			PropertyDescriptor reuseScreen = new PropertyDescriptor("reuseScreen",beanClass, "isReuseScreen","setReuseScreen");
			PropertyDescriptor reuseText = new PropertyDescriptor("reuseText",beanClass, "isReuseText","setReuseText");
			PropertyDescriptor screenDepth = new PropertyDescriptor("screenDepth",beanClass, "getScreenDepth","setScreenDepth");
			PropertyDescriptor textDepth = new PropertyDescriptor("textDepth",beanClass, "getTextDepth","setTextDepth");
			PropertyDescriptor wrapAround = new PropertyDescriptor("wrapAround",beanClass, "isWrapAround","setWrapAround");
			PropertyDescriptor zooming = new PropertyDescriptor("zooming",beanClass, "getZooming","setZooming");
			PropertyDescriptor XAxisLabel = new PropertyDescriptor("XAxisLabel",beanClass, "getXAxisLabel","setXAxisLabel");
			PropertyDescriptor YAxisLabel = new PropertyDescriptor("YAxisLabel",beanClass, "getYAxisLabel","setYAxisLabel");
			PropertyDescriptor SecondaryYAxisLabel = new PropertyDescriptor("SecondaryYAxisLabel",beanClass, "getSecondaryYAxisLabel","setSecondaryYAxisLabel");
			PropertyDescriptor XBestScale = new PropertyDescriptor("XBestScale",beanClass, "isXBestScale","setXBestScale");
      PropertyDescriptor YBestScale = new PropertyDescriptor("YBestScale",beanClass, "isYBestScale","setYBestScale");
      PropertyDescriptor SecondaryYBestScale = new PropertyDescriptor("SecondaryYBestScale",beanClass, "isSecondaryYBestScale","setSecondaryYBestScale");
			PropertyDescriptor XExtent = new PropertyDescriptor("XExtent",beanClass, "getXExtent","setXExtent");
			PropertyDescriptor YExtent = new PropertyDescriptor("YExtent",beanClass, "getYExtent","setYExtent");
			PropertyDescriptor XFineTicks = new PropertyDescriptor("XFineTicks",beanClass, "getXFineTicks","setXFineTicks");
			PropertyDescriptor YFineTicks = new PropertyDescriptor("YFineTicks",beanClass, "getYFineTicks","setYFineTicks");
      PropertyDescriptor SecondaryYFineTicks = new PropertyDescriptor("SecondaryYFineTicks",beanClass, "getSecondaryYFineTicks","setSecondaryYFineTicks");
			PropertyDescriptor XGrid = new PropertyDescriptor("XGrid",beanClass, "isXGrid","setXGrid");
			PropertyDescriptor YGrid = new PropertyDescriptor("YGrid",beanClass, "isYGrid","setYGrid");
			PropertyDescriptor XScale = new PropertyDescriptor("XScale",beanClass, "getXScale","setXScale");
			PropertyDescriptor YScale = new PropertyDescriptor("YScale",beanClass, "getYScale","setYScale");
			PropertyDescriptor XShift = new PropertyDescriptor("XShift",beanClass, "getXShift","setXShift");
			PropertyDescriptor YShift = new PropertyDescriptor("YShift",beanClass, "getYShift","setYShift");
			PropertyDescriptor XTicks = new PropertyDescriptor("XTicks",beanClass, "getXTicks","setXTicks");
      PropertyDescriptor YTicks = new PropertyDescriptor("YTicks",beanClass, "getYTicks","setYTicks");
      PropertyDescriptor SecondaryYTicks = new PropertyDescriptor("SecondaryYTicks",beanClass, "getSecondaryYTicks","setSecondaryYTicks");
			PropertyDescriptor XTickText = new PropertyDescriptor("XTickText",beanClass, "isXTickText","setXTickText");
			PropertyDescriptor YTickText = new PropertyDescriptor("YTickText",beanClass, "isYTickText","setYTickText");
			PropertyDescriptor SecondaryYTickText = new PropertyDescriptor("SecondaryYTickText",beanClass, "isSecondaryYTickText","setSecondaryYTickText");
			PropertyDescriptor colorFilled = new PropertyDescriptor("colorFilled",beanClass, "iscolorFilled","setcolorFilled");
			PropertyDescriptor leftTimeLabel = new PropertyDescriptor("leftTimeLabel",beanClass, "isleftTimeLabel","setleftTimeLabel");
      PropertyDescriptor leftYLabel = new PropertyDescriptor("leftYLabel",beanClass, "isleftYLabel","setleftYLabel");
      PropertyDescriptor bottomXLabel = new PropertyDescriptor("bottomXLabel",beanClass, "isbottomXLabel","setbottomXLabel");
			PropertyDescriptor XTicksBottom = new PropertyDescriptor("XTicksBottom",beanClass, "getXTicksBottom","setXTicksBottom");
			PropertyDescriptor YTicksLeft = new PropertyDescriptor("YTicksLeft",beanClass, "getYTicksLeft","setYTicksLeft");
      PropertyDescriptor xlabelRotation = new PropertyDescriptor("xlabelRotation",beanClass, "getxlabelRotation","setxlabelRotation");
			PropertyDescriptor printTextStickyAttribute = new PropertyDescriptor("printTextStickyAttribute",beanClass, "getprintTextStickyAttribute","setprintTextStickyAttribute");
      PropertyDescriptor printTextLocation = new PropertyDescriptor("printTextLocation",beanClass, "getprintTextLocation","setprintTextLocation");
      PropertyDescriptor printTextRotation = new PropertyDescriptor("printTextRotation",beanClass, "getprintTextRotation","setprintTextRotation");
      PropertyDescriptor TagColor = new PropertyDescriptor("TagColor",beanClass, "getTagColor","setTagColor");
      PropertyDescriptor Tagging = new PropertyDescriptor("Tagging",beanClass, "isTagging","setTagging");
      
      PropertyDescriptor ErrorColor = new PropertyDescriptor("ErrorColor",beanClass, "getErrorColor","setErrorColor");

			PropertyDescriptor LeadingEdgeColor = new PropertyDescriptor("LeadingEdgeColor",beanClass, "getLeadingEdgeColor","setLeadingEdgeColor");
			PropertyDescriptor LeadingEdgeGapSize = new PropertyDescriptor("LeadingEdgeGapSize",beanClass, "getLeadingEdgeGapSize","setLeadingEdgeGapSize");
			PropertyDescriptor LeadingEdgeMotion = new PropertyDescriptor("LeadingEdgeMotion",beanClass, "getLeadingEdgeMotion","setLeadingEdgeMotion");
			PropertyDescriptor LeadingEdgeMode = new PropertyDescriptor("LeadingEdgeMode",beanClass, "getLeadingEdgeMode","setLeadingEdgeMode");
			PropertyDescriptor LeadingEdgeWidth = new PropertyDescriptor("LeadingEdgeWidth",beanClass, "getLeadingEdgeWidth","setLeadingEdgeWidth");
			PropertyDescriptor LeadingEdgeStyle = new PropertyDescriptor("LeadingEdgeStyle",beanClass, "getLeadingEdgeStyle","setLeadingEdgeStyle");
			PropertyDescriptor LeadingEdgeXExtent = new PropertyDescriptor("LeadingEdgeXExtent",beanClass, "getLeadingEdgeXExtent","setLeadingEdgeXExtent");
			PropertyDescriptor LeadingEdgeYExtent = new PropertyDescriptor("LeadingEdgeYExtent",beanClass, "getLeadingEdgeYExtent","setLeadingEdgeYExtent");
      PropertyDescriptor markerColor = new PropertyDescriptor("markerColor",beanClass, "getMarkerColor","setMarkerColor");
      PropertyDescriptor markerMode = new PropertyDescriptor("markerMode",beanClass, "getMarkerMode","setMarkerMode");
      PropertyDescriptor markerStyle = new PropertyDescriptor("markerStyle",beanClass, "getMarkerStyle","setMarkerStyle");
      PropertyDescriptor markerWidth = new PropertyDescriptor("markerWidth",beanClass, "getMarkerWidth","setMarkerWidth");
      PropertyDescriptor markerXExtent = new PropertyDescriptor("markerXExtent",beanClass, "getMarkerXExtent","setMarkerXExtent");
      PropertyDescriptor markerYExtent = new PropertyDescriptor("markerYExtent",beanClass, "getMarkerYExtent","setMarkerYExtent");

      PropertyDescriptor cursorMarkerColor = new PropertyDescriptor("cursorMarkerColor",beanClass, "getCursorMarkerColor","setCursorMarkerColor");
      PropertyDescriptor cursorMarkerMode = new PropertyDescriptor("cursorMarkerMode",beanClass, "getCursorMarkerMode","setCursorMarkerMode");
      PropertyDescriptor cursorMarkerStyle = new PropertyDescriptor("cursorMarkerStyle",beanClass, "getCursorMarkerStyle","setCursorMarkerStyle");
      PropertyDescriptor cursorMarkerWidth = new PropertyDescriptor("cursorMarkerWidth",beanClass, "getCursorMarkerWidth","setCursorMarkerWidth");
      PropertyDescriptor cursorMarkerXExtent = new PropertyDescriptor("cursorMarkerXExtent",beanClass, "getCursorMarkerXExtent","setCursorMarkerXExtent");
      PropertyDescriptor cursorMarkerYExtent = new PropertyDescriptor("cursorMarkerYExtent",beanClass, "getCursorMarkerYExtent","setCursorMarkerYExtent");

			PropertyDescriptor ReceiveQueueDepth = new PropertyDescriptor("ReceiveQueueDepth",beanClass, "getReceiveQueueDepth","setReceiveQueueDepth");
			PropertyDescriptor TimeStamp = new PropertyDescriptor("ReceiveTimeStamp",beanClass, "getTimeStamp","setTimeStamp");
			PropertyDescriptor Status = new PropertyDescriptor("Status",beanClass, "getStatus",null);
			PropertyDescriptor GroupIndex = new PropertyDescriptor("GroupIndex",beanClass, "getGroupIndex",null);
			PropertyDescriptor Grouped = new PropertyDescriptor("isGrouped",beanClass, "getGrouped","setGrouped");
			PropertyDescriptor DeviceContext = new PropertyDescriptor("DeviceContext",beanClass, "getDeviceContext","setDeviceContext");
			PropertyDescriptor DeviceGroup = new PropertyDescriptor("DeviceGroup",beanClass, "getDeviceGroup","setDeviceGroup");
			PropertyDescriptor DeviceName = new PropertyDescriptor("DeviceName",beanClass, "getDeviceName","setDeviceName");
			PropertyDescriptor DeviceProperty = new PropertyDescriptor("DeviceProperty",beanClass, "getDeviceProperty","setDeviceProperty");
			PropertyDescriptor AccessMode = new PropertyDescriptor("AccessMode",beanClass, "getAccessMode","setAccessMode");
			PropertyDescriptor AccessProtocol = new PropertyDescriptor("AccessProtocol",beanClass, "getAccessProtocol","setAccessProtocol");
			PropertyDescriptor AccessRate = new PropertyDescriptor("AccessRate",beanClass, "getAccessRate","setAccessRate");
			PropertyDescriptor AcopConfig = new PropertyDescriptor("AcopConfig",beanClass, "getAcopConfig","setAcopConfig");
			
			PropertyDescriptor Layout = new PropertyDescriptor("Layout",beanClass, "getLayout","setLayout");
			Layout.setHidden(true);
			PropertyDescriptor[] rv = {
				caption
        ,captionLocation
				,frameWidth
				,frameLeftOffset
				,frameRightOffset
				,frameTopOffset
				,frameBottomOffset
				,frameForeColor
        ,xTickLabelColor
        ,yTickLabelColor
        ,secondaryYTickLabelColor
        ,secondaryYAxisOn
        ,secondaryYLogOn
				,displayMode
				,xMin
				,xMax
        ,yMin
        ,yMax
        ,secondaryYMin
        ,secondaryYMax
				,drawWidth
				,frameFont
				,drawStyle
        ,averagingMode
        ,averagingDepth
        ,FFT
				,graphStyle
				,gridStyle
				,gridColor
				,gridWidth
				,keepStyle
				,keepColor
				,keepWidth
				,histogramPosition
				,invert
        ,randomData
        ,randomText
				,reuseScreen
				,reuseText
				,screenDepth
				,textDepth
				,wrapAround
				,zooming
				,XAxisLabel
				,YAxisLabel
				,SecondaryYAxisLabel
				,XBestScale
        ,YBestScale
        ,SecondaryYBestScale
				,XExtent
				,YExtent
				,XFineTicks
				,YFineTicks
        ,SecondaryYFineTicks
				,XGrid
				,YGrid
				,XScale
				,YScale
				,XShift
				,YShift
				,XTicks
        ,YTicks
        ,SecondaryYTicks
				,XTickText
				,YTickText
				,SecondaryYTickText
				,colorFilled
        ,leftTimeLabel
        ,leftYLabel
        ,bottomXLabel
				,XTicksBottom
				,YTicksLeft
        ,xlabelRotation
				,printTextStickyAttribute
        ,printTextLocation
        ,printTextRotation
        ,TagColor
        ,Tagging
        ,ErrorColor
				,LeadingEdgeColor
				,LeadingEdgeGapSize
				,LeadingEdgeMotion
				,LeadingEdgeMode
				,LeadingEdgeWidth
				,LeadingEdgeStyle
				,LeadingEdgeXExtent
				,LeadingEdgeYExtent
        ,markerColor
        ,markerMode
        ,markerStyle
        ,markerWidth
        ,markerXExtent
        ,markerYExtent
        ,cursorMarkerColor
        ,cursorMarkerMode
        ,cursorMarkerStyle
        ,cursorMarkerWidth
        ,cursorMarkerXExtent
        ,cursorMarkerYExtent

        ,ReceiveQueueDepth
        ,TimeStamp
		    ,Status
		    ,GroupIndex
		    ,Grouped
		    ,DeviceContext
		    ,DeviceGroup
		    ,DeviceName
		    ,DeviceProperty
		    ,AccessMode
		    ,AccessProtocol
		    ,AccessRate
		    ,AcopConfig
				,Layout
			};
			return rv;
		}
		catch (IntrospectionException e)
		{
			throw new Error(e.toString());
		}
	}

	private final Class beanClass = Acop.class;
}