package de.desy.acop.chart;
import java.awt.Point;

public class AcopEvent extends java.util.EventObject
{
  protected int event_type;             // event type
  protected int link_id, link_error;    // receive event
  protected int histo_index;            // histo index
  protected Point mouse_point;          // mouse position for mouse event
  protected double mouseUserX, mouseUserY; // converted user mouse position for mouse event
  protected double zoomX1, zoomY1, zoomX2, zoomY2;        // converted zoom rectangle

  // total AcopEventType
  public final static int ACOP_MOUSE_MOVE_EVENT = 1;
  public final static int ACOP_MOUSE_CLICK_EVENT = 2;
  public final static int ACOP_MOUSE_ZOOM_EVENT = 3;
  public final static int ACOP_MOUSE_ENTER_EVENT = 4;
  public final static int ACOP_MOUSE_EXIT_EVENT = 5;
  public final static int ACOP_MOUSE_DCLICK_EVENT = 6;
  public final static int ACOP_RECEIVE_EVENT = 10;
  
  // the constructor
  public AcopEvent(Object source, int eventId, int linkId, int error)
  {
    // pass the source object to the superclass
    super(source);
    
    // save data
    event_type = eventId;
    link_id = linkId;
    link_error = error;
  }

  // the constructor
  public AcopEvent(Object source, int eventId, int histIndex, int x, int y, double userx, double usery)
  {
    // pass the source object to the superclass
    super(source);
    
    event_type = eventId;

    // save data
    histo_index = histIndex;
    mouse_point = new Point(x,y);
    mouseUserX = userx;
    mouseUserY = usery;
  }

  // the constructor
  public AcopEvent(Object source, int eventId, double x1, double y1, double x2, double y2)
  {
    // pass the source object to the superclass
    super(source);
    
    event_type = eventId;

    // save data
    zoomX1 = x1;
    zoomY1 = y1;
    zoomX2 = x2;
    zoomY2 = y2;
  }

  public int getEventType()
  {
    return event_type;
  }
  
  public int getLinkId()
  {
    return link_id;
  }
  
  public int getLinkError()
  {
    return link_error;
  }
  
  public int getHistoIndex()
  {
    return histo_index;
  }
  
  public Point getMousePoint()
  {
    return mouse_point;
  }
  
  public double getMousePositionX()
  {
    return mouseUserX;
  }
  
  public double getMousePositionY()
  {
    return mouseUserY;
  }
  
  public double getZoomX1()
  {
    return zoomX1;
  }  
  
  public double getZoomY1()
  {
    return zoomY1;
  }  
  
  public double getZoomX2()
  {
    return zoomX2;
  }  
  
  public double getZoomY2()
  {
    return zoomY2;
  }  
}