package de.desy.acop.chart;

public class AcopPesist
{
  protected int size; // persistent depth
  protected int[] ids; // id array of size
  protected int hisIndex; // primary id
  protected int position, numCurves;
  protected boolean copyMemory;
  protected boolean endlessFlag;
  protected java.awt.Color vorColor, pesisColor;
  private Acop acop;
  private int drawStyle = AcopDrawStyleEnum.PS_SOLID.ordinal();

  public AcopPesist(Acop acop, int index)
  {
    this.acop = acop;
    resetPesist(index);
  }
  private void resetPesist(int index)
  {
    size = 0;
    hisIndex = index;
    position = -1;    
    numCurves = 0;
    copyMemory = false;
    endlessFlag = false;
    pesisColor = null;    
    drawStyle = AcopDrawStyleEnum.PS_SOLID.ordinal();
  }
  private void clearPesistScreen()
  {
    if ( size > 1 ) for (int i=1; i<size; i++)
    {
      if ( ids[i] < 0 ) break;
      acop.clearScreen(ids[i]);
    }
  }
  public int initPesist(int depth, boolean memCopy, java.awt.Color c)
  {
    return initPesist(depth, memCopy, true, c);
  }
  public int initPesist(int depth, boolean memCopy, boolean endless, java.awt.Color c)
  { int ret = 0;

    clearPesistScreen();   
    pesisColor = c;
    size = depth;
    copyMemory = memCopy;
    endlessFlag = endless;
    if ( size < 1 ) size = 1;
    ids = new int[size];
    ids[0] = hisIndex;
    for(int i=1; i<size; i++) ids[i] = -1;
    vorColor = acop.histo[hisIndex].penColor;
    position = 0;
    numCurves = 1;
    drawStyle = acop.getDrawStyle();
    return ret;    
  }

  public int refreshPesist(Object Yarray)
  { 
    acop.setDrawStyle(drawStyle);
    // refresh last curve with persistent color
    if (numCurves < size || (size == 1 && endlessFlag) || (size > 1 && copyMemory == false))
    {
      acop.histo[ ids[position] ].penColor = pesisColor;
      acop.repainHisto = true;
      acop.histo[ids[position]].updateScreen(false, acop.histo[ids[position]].y, -1, -1, null);
      acop.repainHisto = false;
    }
    // draw new curve
    if ( numCurves < size )
    {
      java.awt.Color tmpForColor = acop.getForeground();
      // draw current curve
      acop.setForeground(vorColor);
      ids[position + 1] = acop.draw(Yarray, acop.histo[hisIndex].x);
      acop.setForeground(tmpForColor);
      numCurves++;
      position++;
      if ( ids[position ] >= 0 ) return position;
      // memory full, adjust the size
      size = position;
      numCurves = size;
      position--;
    }
    // copy curves and refreshScreen, or move the position
    if ( copyMemory ) for (int i=1; i<size; i++)
    {
      int points = acop.histo[ids[i]].npoints;
      if (acop.histo[ids[i]].y.length != acop.histo[ids[i-1]].y.length) {
          acop.histo[ids[i-1]].y = new double[points];
          acop.histo[ids[i-1]].npoints = points;
      }
      System.arraycopy(acop.histo[ids[i]].y, 0, acop.histo[ids[i-1]].y, 0, points);
      acop.repainHisto = true;
      acop.histo[ids[i-1]].updateScreen(false, acop.histo[ids[i-1]].y, -1, -1, null);
      acop.repainHisto = false;
    }
    else if ( size > 1 ) if ( ++position >= size ) position = 0;

    // refresh new curve
    if ( endlessFlag ) acop.repainHisto = true;
    acop.histo[ ids[position] ].penColor = vorColor;
    acop.histo[ids[position]].updateScreen(false, Yarray, -1, -1, null);
    if ( endlessFlag ) acop.repainHisto = false;
    
    return position;
  }
  
  public void clearPesist()
  {
    if ( size <= 0 || hisIndex < 0 ) return;
    clearPesistScreen();   
    if ( ids[0] >= 0 ) acop.histo[ ids[0] ].penColor = vorColor;
    resetPesist(-1);
  }
}
