package de.desy.acop.chart;

import java.awt.*;
import java.awt.geom.*;
import java.text.*;
import java.util.*;

public class AcopScale
{
  private static int longTick = 4, shortTick = 2;
  protected static int gridLineMode = AcopConst.PS_DOT;
  protected static int gridLineWidth = 1;
  protected static java.awt.Color gridLineColor = Color.cyan;
  protected static BasicStroke gridStroke = AcopConst.createStroke(gridLineWidth, gridLineMode);
  private double Rfirst, Rlast, Rinc;
  private short Nteile, Ntick;
  private double Rtick;
  private double Rmin, Rmax;
  // flags set by property
  protected boolean bestScale, grid, ticktext;
  protected int ticks, fineTicks, xtime, log;
  protected java.lang.String AxisLabel = "";
  protected java.awt.Color tickLabelForeColor;
  private boolean xAxis; // x or y axis
  protected int time_scale;
  protected boolean leftBottom; // where to put ticks, or right_top?
  // for x-axis: Bottom ot Top side of Histogram
  // for y-axis: Left ot Right side of Histogram
  protected boolean leftBottLabel; // where to put Label
  protected Rectangle labelRect; // where to put label
  protected Rectangle ticksRect; // tick drawing rectangle
  protected double tick_min, tick_max; // ticks area
  protected double min, max, dispSize; // data (user) display area after
                                        // scaling
  protected double usermin, usermax, userSize; // user input value(property min
                                                // and max)
  private DecimalFormat tickFormat = new DecimalFormat("#####.###");
  private FontMetrics fMetrix;
  private Acop acop;
  private AcopFrame aFrame;
  protected boolean OnFlag;
  public AcopScale(Acop acop, boolean xax)
  {
    this.acop = acop;
    this.aFrame = acop.aFrame;
    xAxis = xax;
  }
  protected int getScreenPosition(double userPosition)
  {
    double tmp;
    if (xAxis)
    {
      if (xtime != 0)
        tmp = (userPosition - min) / dispSize;
      else
        tmp = ((log != 0 ? (Math.log(userPosition) / Math.log(10) + 0.01) : userPosition) - tick_min)
            / (tick_max - tick_min);
      tmp = tmp * aFrame.dxwidth + aFrame.histRect.x; // aFrame.xleft ;
    }
    else
      tmp = aFrame.histRect.height
          * (1 - ((log != 0 ? (Math.log(userPosition) / Math.log(10) + 0.01) : userPosition) - min)
              / dispSize) + aFrame.histRect.y;
    return (int) tmp;
  }
  protected double getUserPosition(int screenPosition)
  {
    double tmp;
    if (xAxis)
    {
      tmp = (screenPosition - aFrame.xleft) / aFrame.dxwidth;
      if (xtime != 0)
        tmp = tmp * dispSize + min;
      else
        tmp = tmp * (tick_max - tick_min) + tick_min;
    }
    else
      tmp = (screenPosition - aFrame.histRect.y) / aFrame.histRect.width * dispSize + min;
    if (log != 0) tmp = Math.pow(10.0, tmp);
    return tmp;
  }
  protected double getMin()
  {
    return log != 0 ? Math.pow(10.0, min) : min;
  }
  protected double getMax()
  {
    return log != 0 ? Math.pow(10.0, max) : max;
  }
  protected void setDispscale(double low, double high) // set scaled size
  {
    min = low;
    max = high;
    dispSize = max - min;
  }
  protected void setscale()
  {
    setDispscale(usermin, usermax);
  }
  protected void setUserscale(double low, double high) // set scaled size
  {
    usermin = low;
    usermax = high;
    userSize = usermax - usermin;
  }
  protected boolean isOriginMaxMin()
  {
    if (Math.abs(min - usermin) < AcopConst.zero_check && Math.abs(max - usermax) < AcopConst.zero_check)
      return true;
    else
      return false;
  }
  private void logRound()
  {
    int i1, i2;
    int zero_flag = 0;
    double tmpmin, tmpmax;
    tmpmin = Rmin;
    tmpmax = Rmax;
    if (Rmin < AcopConst.zero_check)
    { /* check zero */
      Rmin = 0.1; // 10 to -1
      zero_flag = 1;
    }
    if (Rmax < AcopConst.zero_check)
    {
      Rmax = 10.0; // 10 to 1
      zero_flag += 2;
    }
    if (zero_flag == 1)
    { /* make new boundary */
      if (Rmax < Rmin * 5.0) if (Rmax * 0.01 > tmpmin) Rmin = Rmax * 0.01;
    }
    else if ((zero_flag == 0 && Rmax < Rmin * 5.0) || zero_flag == 2)
      if (Rmin * 100.0 < tmpmax) Rmax = Rmin * 100.0;
    if (Rmin < tmpmin && (zero_flag & 0x1) == 0) Rmin = tmpmin; /* check limit */
    if (Rmax > tmpmax && (zero_flag & 0x2) == 0) Rmax = tmpmax;
    i1 = Rmin >= 1.0 ? (int) (Math.log(Rmin) / Math.log(10) + 0.01)
        : (int) (Math.log(Rmin) / Math.log(10) - 0.99);
    i2 = Rmax > 1.0 ? (int) (Math.log(Rmax) / Math.log(10) + 0.99)
        : (int) (Math.log(Rmax) / Math.log(10) - 0.01);
    if (i1 == i2) i1--;
    Rfirst = i1;
    Rlast = i2;
    Rinc = 1;
    Nteile = (short) (i2 - i1 + 1);
    Rtick = 2; // take log10
    Ntick = 5;
  }
  private void lineRound()
  {
    double del, dl, f, dlm, fdl;
    int ideca;
    del = Rmax - Rmin;
    if (Math.abs(del) < AcopConst.zero_check)
    { /* ??????? */
      Rfirst = Rmin;
      Rinc = 1.0;
      Rlast = Rmax;
      Nteile = 2;
      Ntick = 1;
      Rtick = 0.5;
    }
    else
    {
      if (del < 0.0)
      { /* revers it */
        dlm = Rmin;
        Rmin = Rmax;
        Rmax = dlm;
        del = Math.abs(del);
      }
      dl = Math.log(del) / Math.log(10);
      dlm = Math.abs(Rmin) > Math.abs(Rmax) ? Math.log(Math.abs(Rmin)) / Math.log(10) : Math.log(Math
          .abs(Rmax))
          / Math.log(10);
      if (Rmin < 0.0 && Rmax > 0.0) dl = dlm;
      dlm = (double) ((int) (dlm));
      fdl = (double) ((int) (dl));
      f = Math.abs(dl - fdl);
      if (fdl > 0.0)
      {
        if (fdl > dlm + 2.0)
        {
          fdl = dlm + 1.0;
          f = 0.0;
        }
      }
      else
      {
        if (fdl < dlm - 2.0)
        {
          fdl = dlm - 3.0;
          f = 0.0;
        }
      }
      Rinc = 1.0;
      if (del > 1.0)
      {
        Rinc = Math.pow(10.0, fdl);
        if (fdl < 10) Rinc = (double) Math.round(Rinc);
        // sca[index].Rinc = (double) round ( pow (10.0, fdl) );
        if (f <= 0.31)
          Rinc /= 5.0;
        else if (f <= 0.71) Rinc /= 2.0;
      }
      else
      {
        fdl -= 1.0;
        Rinc = Math.pow(10.0, fdl);
        if (f > 0.69)
          Rinc /= 5.0;
        else if (f > 0.29) Rinc /= 2.0;
      }
      Rfirst = Rmin >= 0.0 ? ((int) (Rmin / Rinc)) * Rinc : ((int) (Rmin / Rinc - 0.99)) * Rinc;
      Rlast = Rmax >= 0.0 ? ((int) (Rmax / Rinc + 0.99)) * Rinc : ((int) (Rmax / Rinc)) * Rinc;
      Nteile = (short) (Math.round((Rlast - Rfirst) / Rinc + 1.0));
      ideca = (int) (Math.log(Math.abs(Rinc)) / Math.log(10));
      if (ideca < 0) ideca--;
      Rtick = Math.pow(10.0, (double) ideca);
      Ntick = (short) (Math.round(Rinc / Rtick) - 1);
      if (Ntick < 1) Ntick = 1; // for loop to wotk!!!???
    }
    ideca = ticks; // user defines ticks!!
    if (ideca < 0)
      return;
    else if (ideca == 0) ideca = 1;
    del = bestScale ? (Rlast - Rfirst) : (Rmax - Rmin);
    Nteile = (short) (ideca + 1);
    Rinc = del / ideca;
    ideca = fineTicks;
    Ntick = (short) (ideca <= 1 ? 1 : ideca);
    Rtick = Rinc / Ntick;
  }
  protected void ScaleToGrid()
  {
    int xstep, i;
    if (xtime != 0) // convert to time tick
    {
      if (min < 0) min = 0; // time cannt be < 0!
      if (max - min < 1) max = min + 1; // one second is minimal
      dispSize = max - min;
      xstep = (int) dispSize;
      for (i = 0; i < AcopConst.N_TIME; i++)
      {
        if (xstep > AcopConst.timeMax[i] * AcopConst.timeScale[i]) continue;
        time_scale = i;
        break;
      }
      if (aFrame.leftTimeLabel == true)
      {
        tick_min = 0;
        tick_max = dispSize / AcopConst.timeScale[i];
      }
      else
      {
        tick_min = -dispSize / AcopConst.timeScale[i];
        tick_max = 0;
      }
    }
    else
    {
      tick_min = min;
      tick_max = max;
    }
  }
  protected void xyscale(boolean setYaxis)
  {
    if (OnFlag == false) return;
    if (xAxis == false)
    {
      tick_min = min;
      tick_max = max;
    }
    Rmin = tick_min;
    Rmax = tick_max;
    if (log != 0)
      logRound();
    else
      lineRound();
    if (bestScale || log != 0)
    {
      tick_min = Rfirst;
      tick_max = Rlast;
      if (xtime != 0)
      {
        if (aFrame.leftTimeLabel)
        {
          min = min + (log != 0 ? Math.pow(10.0, tick_min) : tick_min) * AcopConst.timeScale[time_scale]; // back
                                                                                                          // to
                                                                                                          // normal
                                                                                                          // units
          max = ((log != 0 ? Math.pow(10.0, tick_max) : tick_max) - (log != 0 ? Math.pow(10.0, tick_min)
              : tick_min))
              * AcopConst.timeScale[time_scale] + min;
        }
        else
        {
          max = max + (log != 0 ? Math.pow(10.0, tick_max) : tick_max) * AcopConst.timeScale[time_scale]; // back
                                                                                                          // to
                                                                                                          // normal
                                                                                                          // units
          min = max
              - ((log != 0 ? Math.pow(10.0, tick_max) : tick_max) - (log != 0 ? Math.pow(10.0, tick_min)
                  : tick_min)) * AcopConst.timeScale[time_scale];
        }
        if (log != 0)
        {
          min = Math.log(AcopConst.log_check(min)) / Math.log(10);
          max = Math.log(AcopConst.log_check(max)) / Math.log(10);
        }
      }
      else
      {
        min = tick_min;
        max = tick_max;
      }
      dispSize = max - min;
      if (xAxis == false && setYaxis && log == 0)
      {
        usermin = min;
        usermax = max;
        userSize = usermax - usermin;
      }
    }
  }
  protected void drawGrid(java.awt.Graphics2D a)
  {
    boolean tickflag;
    int wx, wy, k, m;
    double tmp;
    AcopHisto hist;
    if (!grid) return;
    a.setColor(gridLineColor);
    a.setStroke(gridStroke);
    tmp = tick_max - tick_min;
    tickflag = (Nteile <= 2 && log != 0) ? true : false;
    wy = xAxis ? aFrame.dxwidth : aFrame.histRect.height;
    for (int i = (tickflag ? 1 : 0); i < (tickflag ? Ntick : Nteile); i++)
    {
      wx = (int) (wy * (tickflag ? Math.log(Rtick * i) / Math.log(10) : (Rinc * i)) / tmp + 0.01);
      if (wx > wy || wx < 0) continue;
      wx += aFrame.xleft;
      // if ( wx >= wy || wx == 0 ) continue;
      if (xAxis)
      {
        hist = acop.histo[acop.m_histstart];
        if (aFrame.drawFlag == false || hist.xlabON == 0 || log != 0 || xtime != 0)
          a.drawLine(wx, 0, wx, aFrame.histRect.height);
        else
        {
          for (k = 0, m = hist.wrapindex; k < hist.npoints; k++, m++)
          {
            if (m >= hist.npoints) m = 0;
            if (wx <= hist.ix[m]) break;
          }
          if (k == hist.npoints) break;
          if (hist.ix[m] <= aFrame.xright && hist.ix[m] > 1) // within
                                                              // histogram
          {
            wx = hist.ix[m];
            a.drawLine(wx, 0, wx, aFrame.histRect.height);
          }
        }
      }
      else
        a.drawLine(0, wx, aFrame.histRect.width, wx);
    }
  }
  private void convert(java.lang.String string, double w, int[] format)
  {
    /*
     * char s_format [10] ; int i, j, k ; s_format [0] ='%' ; if ( format[2] )
     * sprintf ( &s_format[1], ".%de", format[1]) ; else sprintf (&s_format[1],
     * ".%df", format[1]) ; sprintf ( string, s_format, w) ; if ( format[2] ) //
     * delete unneccesary + & 0 { for ( i = 0; string [i++] != 'e'; ) ; k = i-1 ;
     * j = string [i++] == '+' ? i : i+1 ; // store copy position while ( string
     * [i] == '0' && string [i] != 0 ) i++ ; if ( string [i] == 0 ) { string
     * [j=k] = 0 ; // no exp format[2] = 0 ; } else { // copy rest significant
     * digits do { string[j++] = string[i++] ; } while ( string[i-1] != 0 ) ;
     * j-- ; } } else for (j = 0 ; string[j] ; j++ ); format[3] = j; // length
     * of the string
     */
  }
  private void g_stellen(double wert, int[] format)
  {
    double z;
    int stellen = 5; // default effective # of digits
    format[1] = 1; // hinter der komma
    format[2] = 0; // exp format
    format[4] = 0; // exp format digits after e;
    z = Math.abs(wert) > AcopConst.zero_check ? // make z = etwas * 1.0e3
        wert / Math.pow(10., (double) (int) (Math.log(Math.abs(wert)) / Math.log(10.) * 1.001)) * 1.0e3 : 0.0;
    z = Math.abs(z);
    wert = Math.abs(wert);
    z = wert < 1.0 ? z * 10.0 + 0.05 : z + 0.05; // make etwas=x.xxx
    while (z - (int) z < 0.1 && stellen > 1) // get effective digits
    {
      stellen--;
      z /= 10.0;
    }
    if (wert >= 100000.0 || (wert > AcopConst.zero_check && wert < 0.001))
    {
      format[2] = 2; // { mit Exponent }
      if (wert >= 1.0E10)
        format[2] = 3;
      else if (wert <= 1.0E-10) format[2] = 4;
      z = wert * 1.0001; // avoid 10e+-xx problems 2005-Feb
      format[4] = (int) (Math.log(z) / Math.log(10.));
      // format[4] = (int)(Math.log(wert)/Math.log(10.));
      if (format[4] < 0) format[4]--; // digits after exponential
    }
    format[0] = 1; // vor der komma
    if (wert >= 0.001 && wert < .999)
    {
      format[0] = 0;
      if (wert < 0.00999)
      {
        if (stellen > 3)
          stellen = 3;
        else
          stellen += 2;
      }
      else if (wert < 0.0999)
      {
        if (stellen > 4)
          stellen = 2;
        else
          stellen++;
      }
    }
    if (wert >= 10.0 && wert < 100.0)
      format[0] = 2;
    else if (wert >= 100.0 && wert < 1000.0)
      format[0] = 3;
    else if (wert >= 1000.0 && wert < 10000.0)
      format[0] = 4;
    else if (wert >= 10000.0 && wert < 100000.0) format[0] = 5;
    format[1] = stellen - format[0];
    if (format[1] < 0) format[1] = 0;
  }
  protected void drawLabel(java.awt.Graphics2D a)
  {
    int wz, hei;
    if (AxisLabel.length() == 0) return;
    Rectangle Cliprect = a.getClipBounds();
    if (Cliprect != null && Cliprect.intersects(labelRect) == false) return;
    fMetrix = a.getFontMetrics();
    a.clipRect(labelRect.x, labelRect.y, labelRect.width, labelRect.height);
    AffineTransform oldTransform = a.getTransform();
    if (xAxis)
    {
      a.translate(labelRect.x, labelRect.y);
      hei = fMetrix.getAscent();
      wz = leftBottLabel ? (5 + hei * 2) : (labelRect.height - 4 - hei);
      a.drawString(AxisLabel, (labelRect.width - fMetrix.stringWidth(AxisLabel)) / 2, wz);
    }
    else
    {
      hei = (fMetrix.getDescent() + fMetrix.getAscent()) / 2;
      if (leftBottLabel == true)
        a.translate(hei + 5, (labelRect.height + fMetrix.stringWidth(AxisLabel)) / 2);
      else
        a.translate(labelRect.x + labelRect.width - hei,
            (labelRect.height + fMetrix.stringWidth(AxisLabel)) / 2);
      a.rotate(-Math.PI / 2);
      a.drawString(AxisLabel, 0, 0);
    }
    a.setTransform(oldTransform);
    a.setClip(Cliprect);
  }
  protected void drawTicks(java.awt.Graphics2D a, AcopHisto pHisto)
  {
    int wx, wy, wz, lastright, k, m, xpos = 0, xtp, ytp;
    double tmp;
    int strformat[] = {0, 0, 0, 0, 0}; // vor, hinter, texp, lang, exp-digits;
    java.lang.String str = "erty";
    int siz, hei, tlen, strmaxlen;
    Rectangle txtRect = new Rectangle();
    Date date = new Date();
    if ((xAxis ? ticksRect.height : ticksRect.width) < 1) return;
    Rectangle Cliprect = a.getClipBounds();
    if (Cliprect != null && Cliprect.intersects(ticksRect) == false) return;
    a.setFont(aFrame.frameFont);
    a.setColor(aFrame.frameForeColor);
    fMetrix = a.getFontMetrics();
    strmaxlen = 0;
    if (xAxis)
    {
      a.clipRect(ticksRect.x, ticksRect.y - 1, ticksRect.width + 1, ticksRect.height + 4);
      a.translate(aFrame.histRect.x + aFrame.xleft, ticksRect.y - 1);
      hei = fMetrix.getAscent();
      txtRect.y = leftBottom ? 4 : 0;
      txtRect.height = ticksRect.height - 4;
      wy = aFrame.dxwidth;
      lastright = -100000;
    }
    else
    {
      a.clipRect(ticksRect.x - 2, ticksRect.y, ticksRect.width + 3, ticksRect.height);
      a.translate(ticksRect.x, aFrame.histRect.y);
      txtRect.x = 0;
      txtRect.width = ticksRect.width - 4;
      hei = (fMetrix.getDescent() + fMetrix.getAscent()) / 2;
      wy = aFrame.histRect.height;
      lastright = 100000;
    }
    if (ticks != 0)
      for (int i = 0; i < Nteile; i++)
        for (int j = 0; j < Ntick; j++)
        {
          tmp = Rtick * j; // linear scale
          if (j > 0 && log != 0) tmp = Math.log(tmp) / Math.log(10.); // log
                                                                      // scale
          tmp += Rinc * i;
          if (xAxis)
          {
            wx = (int) (wy * tmp / (tick_max - tick_min));
            if (wx > wy) continue;
          }
          else
          {
            wx = (int) (wy * (1 - tmp / (tick_max - tick_min)));
            if (wx < 0) continue;
          }
          if (j == 0) // convert to string
          {
            tmp += tick_min;
            if (log != 0)
              tmp = Math.pow(10., tmp);
            else if (Math.abs(tmp) < (Rlast - Rfirst) * 1.0e-7) tmp = 0.0; // avoid
                                                                            // 0.1e-9
            str = tickFormat.format(tmp);
            g_stellen(tmp, strformat);
            str = strformat[4] == 0 ? tickFormat.format(tmp) : tickFormat.format(tmp
                / Math.pow(10., (double) strformat[4]))
                + "e" + strformat[4];
            // convert (str, tmp, strformat) ;
            // texp = g_stellen (tmp, &vor, &hinter);
            // lang = convert (str, tmp, hinter, &texp ) ;
          }
          if (xAxis) // output tick and text for x
          {
            if (aFrame.drawFlag == false || pHisto.xlabON == 0 || log != 0 || xtime != 0)
            {
              wz = leftBottom ? 0 : ticksRect.height - 1;
              a.drawLine(wx, wz, wx, wz + (j == 0 ? longTick : shortTick));
              if (ticktext && j == 0)
              {
                siz = fMetrix.stringWidth(str);
                if ((txtRect.x = wx - siz / 2) >= lastright)
                {
                  wz = leftBottom ? (4 + hei) : (txtRect.height);
                  txtRect.width = siz;
                  if (tickLabelForeColor != aFrame.frameForeColor) a.setColor(tickLabelForeColor);
                  a.drawString(str, txtRect.x, wz);
                  lastright = wx + siz;
                  if (tickLabelForeColor != aFrame.frameForeColor) a.setColor(aFrame.frameForeColor);
                }
              }
            }
            // else if ( j == 0 )
            else
            {
              for (k = 0, m = pHisto.wrapindex; k < pHisto.npoints; k++, m++)
              {
                if (m >= pHisto.npoints) m = 0;
                xpos = pHisto.ix[m] - aFrame.xleft;
                if (wx <= xpos) break;
              }
              if (xpos <= aFrame.xright) // within histogram
              {
                if (m == pHisto.npoints) m--;
                wz = leftBottom ? 0 : ticksRect.height - 1;
                a.drawLine(xpos, wz, xpos, wz + (j == 0 ? longTick : shortTick));
                if (tickLabelForeColor != aFrame.frameForeColor) a.setColor(tickLabelForeColor);
                if (ticktext && m < pHisto.xlabLength)
                {
                  siz = fMetrix.stringWidth(pHisto.xlab[m]);
                  if (acop.xlabelRotation == 0)
                  {
                    if (j == 0 && (txtRect.x = xpos - siz / 2) >= lastright)
                    {
                      wz = leftBottom ? (4 + hei) : (txtRect.height);
                      txtRect.width = siz;
                      a.drawString(pHisto.xlab[m], txtRect.x, wz);
                      lastright = xpos + siz;
                    }
                  }
                  else
                  {
                    AffineTransform oldTransform = a.getTransform();
                    xtp = xpos;
                    ytp = 4;
                    if (acop.xlabelAxis == 1) // 90 dwegree
                    {
                      xtp += hei / 2;
                      ytp += siz;
                    }
                    else if (acop.xlabelAxis == 2) // 180 dwegree
                    {
                      xtp += siz / 2;
                      ytp += hei / 2;
                    }
                    else if (acop.xlabelAxis == 3) // -90 dwegree
                    {
                      // ytp += 4 ;
                    }
                    else if (acop.xlabelQudrant == 4)
                    {
                      xtp += (int) (hei / 2 * acop.xlabelsin);
                      ytp += (int) (hei * acop.xlabelcos);
                    }
                    else if (acop.xlabelQudrant == 2)
                    {
                      xtp -= (int) (siz * acop.xlabelcos);
                      ytp += (int) (siz * acop.xlabelsin);
                    }
                    else if (acop.xlabelQudrant == 1)
                    {
                      xtp -= (int) (siz * acop.xlabelcos - hei / 2 * acop.xlabelsin);
                      ytp += (int) (siz * acop.xlabelsin + hei * acop.xlabelcos);
                      // xtp -= (int)(hei * de.desy.acop.chart.xlabelsin - siz *
                      // de.desy.acop.chart.xlabelcos) ;
                      // ytp += (int)(hei * de.desy.acop.chart.xlabelcos + siz *
                      // de.desy.acop.chart.xlabelsin) ;
                    }
                    a.translate(xtp, ytp);
                    a.rotate(-acop.xlabelRotateGrad);
                    a.drawString(pHisto.xlab[m], 0, 0);
                    a.setTransform(oldTransform);
                  }
                }
                if (tickLabelForeColor != aFrame.frameForeColor) a.setColor(aFrame.frameForeColor);
              }
            }
          }
          else if (ticksRect.width > 3) // y axis
          {
            wz = leftBottom ? ticksRect.width + 1 : 2;
            a.drawLine(wz, wx, wz - (j == 0 ? longTick : shortTick), wx);
            if (ticktext && j == 0 && ticksRect.width > 4)
            {
              siz = fMetrix.stringWidth(str);
              if (siz > strmaxlen) strmaxlen = siz;
              if ((txtRect.y = wx + hei / 2) <= lastright)
              {
                wz = leftBottom ? ticksRect.width - 5 - siz : 5;
                txtRect.height = hei;
                if (tickLabelForeColor != aFrame.frameForeColor) a.setColor(tickLabelForeColor);
                a.drawString(str, wz, txtRect.y);
                lastright = txtRect.y - hei * 3 / 2;
                if (tickLabelForeColor != aFrame.frameForeColor) a.setColor(aFrame.frameForeColor);
              }
            }
          }
        }
    if (tickLabelForeColor != aFrame.frameForeColor) a.setColor(tickLabelForeColor);
    if (xAxis) // x label and time label
    {
      wz = leftBottom ? (5 + hei * 2) : (ticksRect.height - 4 - hei);
      // if ( AxisLabel.length() != 0 && leftBottom == leftBottLabel )
      // a.drawString(AxisLabel, (aFrame.dxwidth -
      // fMetrix.stringWidth(AxisLabel))/2, wz );
      if (ticktext && xtime != 0 && ticks != 0)
      {
        if (aFrame.leftTimeLabel)
          date.setTime((long) getMin() * 1000);
        else
          date.setTime((long) getMax() * 1000);
        siz = fMetrix.stringWidth(date.toString());
        tlen = fMetrix.stringWidth(AcopConst.timeStr[time_scale]);
        if (aFrame.leftTimeLabel)
        {
          // a.drawString(date.toString(), -siz/4, 5+hei*2);
          a.drawString(date.toString(), -siz / 4, wz);
          a.drawString(AcopConst.timeStr[time_scale], aFrame.dxwidth - tlen / 2, wz);
        }
        else
        {
          a.drawString(date.toString(), aFrame.dxwidth - siz * 3 / 4, wz);
          a.drawString(AcopConst.timeStr[time_scale], -tlen / 2, wz);
        }
      }
    }
    // else if ( AxisLabel.length() != 0 && leftBottom == leftBottLabel ) // y
    // label
    else if (false) // y label
    {
      AffineTransform oldTransform = a.getTransform();
      if (leftBottLabel == true)
      {
        if (ticksRect.width >= hei + 4)
        {
          // a.translate(hei+5,
          // ticksRect.height/2+fMetrix.stringWidth(AxisLabel)/2);
          a.translate(hei + 5, ticksRect.height / 2);
          a.rotate(-Math.PI / 2);
          a.drawString(AxisLabel, 0, 0);
        }
      }
      else if (acop.yAxisRect.width != 0)
      {
        a.translate(acop.yAxisRect.width - hei, acop.yAxisRect.height / 2);
        // a.translate(de.desy.acop.chart.yAxisRect.x+de.desy.acop.chart.yAxisRect.width/2,
        // de.desy.acop.chart.yAxisRect.height/2);
        a.rotate(-Math.PI / 2);
        a.drawString(AxisLabel, 0, 0);
      }
      a.transform(oldTransform);
      /*
       * pdc->SelectObject(&y_font); pdc->SetTextAlign( TA_CENTER | TA_TOP); if (
       * !m_leftYLabel ) { if ( fRight > 0 ) { pdc->SetWindowExt (fRight, -dy);
       * pdc->SetViewportExt (yAxisRect.right - yAxisRect.left, dy);
       * pdc->SetViewportOrg (yAxisRect.left, histRect.bottom-1); //
       * pdc->GetTextMetrics(&tm); // pdc->ExtTextOut((100-tm.tmHeight)/2,
       * 0,ETO_CLIPPED, &txtRect, m_yAxisLabel,NULL); pdc->TextOut(2, -dy/2,
       * m_yAxisLabel); } } else if ( fLeft >= strmaxlen + 4 + tm.tmHeight) { //
       * pdc->GetTextMetrics(&tm); // pdc->ExtTextOut((100-tm.tmHeight)/2,
       * 0,ETO_CLIPPED, &txtRect, m_yAxisLabel,NULL); strmaxlen =
       * fLeft-strmaxlen-4-tm.tmHeight-2; pdc->TextOut( strmaxlen < 0 ? 0 :
       * strmaxlen, -dy/2, m_yAxisLabel); }
       */
    }
    if (xAxis)
      a.translate(-aFrame.histRect.x - aFrame.xleft, -ticksRect.y + 1);
    else
      a.translate(-ticksRect.x, -aFrame.histRect.y);
    a.setClip(Cliprect);
    drawLabel(a);
    if (tickLabelForeColor != aFrame.frameForeColor) a.setColor(aFrame.frameForeColor);
  }
}