/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.chart.customizer;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.LayoutManager;
import java.beans.Customizer;

import javax.swing.JLabel;
import javax.swing.JPanel;
import de.desy.acop.chart.Acop;
import de.desy.acop.chart.AcopAveragingMode;
import de.desy.acop.chart.AcopConst;
import de.desy.acop.chart.AcopDisplayMode;
import de.desy.acop.chart.AcopFFTEnum;

public class DisplayCustomizerPanel extends JPanel implements Customizer
{
	
    private static final long serialVersionUID = 4847680179665327083L;
    public static final String[] DISPLAY_PROPERTIES = new String[]{
		"displayMode", "invert", "FFT", "wrapAround", "tagging", "XExtent", "YExtent",
		"colorFilled", "randomData", "averagingMode", "averagingDepth", "screenDepth",
		"reuseScreen"
	};

  private JLabel displayLabel = null;
  private JLabel modeLabel = null;
  private AcopJComBox<AcopDisplayMode> displayModeBox;
  private JLabel fftLabel = null;
  private AcopJComBox<AcopFFTEnum> fftBox;
  private AcopJRadioButton acopReverseY, Wraparound, Tagging, colorFill, randomData, reuseScreen;
  private JLabel extendXLabel, extendYLabel;
  private AcopJFormattedTextField extXText, extYText;
  private JLabel averageLabel = null;
  private JLabel averModeLabel = null;
  private AcopJComBox<AcopAveragingMode> averModeBox;
  private JLabel averageDepthLabel;
  private AcopJFormattedTextField averDepthText;
  private JLabel screenLabel = null;
  private JLabel screenDepthLabel;
  private AcopJFormattedTextField screenDepthText;
  public DisplayCustomizerPanel(LayoutManager layout, boolean isDoubleBuffered)
  {
    super(layout, isDoubleBuffered);
    initialize();
  }
  public DisplayCustomizerPanel(LayoutManager layout)
  {
    super(layout);
    initialize();
  }
  public DisplayCustomizerPanel(boolean isDoubleBuffered)
  {
    super(isDoubleBuffered);
    initialize();
  }
  public DisplayCustomizerPanel()
  {
    super();
    initialize();
  }
  /**
   * This method initializes this
   * 
   * @return void
   */
  private void initialize()
  {
    // GridBagLayout gbl = new GridBagLayout();
    // this.setLayout(gbl);
    int offX;
    GridBagConstraints gCon;
    setLayout(new GridBagLayout());
    if (displayLabel == null)
    {
      displayLabel = new JLabel("-- Display Parameters --");
    }
    if (modeLabel == null)
    {
      modeLabel = new JLabel("Mode:");
    }
    if (fftLabel == null)
    {
      fftLabel = new JLabel("FFT:");
    }
    if (extendXLabel == null)
    {
      extendXLabel = new JLabel("Extent-X:");
    }
    if (extendYLabel == null)
    {
      extendYLabel = new JLabel("Extent-Y:");
    }
    if (averageLabel == null)
    {
      averageLabel = new JLabel("-- Averagin Parameters --");
    }
    if (averModeLabel == null)
    {
      averModeLabel = new JLabel("AverMode:");
    }
    if (averageDepthLabel == null)
    {
      averageDepthLabel = new JLabel("AverDepth:");
    }
    if (screenLabel == null)
    {
      screenLabel = new JLabel("-- Screen Parameters --");
    }
    if (screenDepthLabel == null)
    {
      screenDepthLabel = new JLabel("#Screen Buffers:");
    }
    add(displayLabel, new GridBagConstraints(0, 0, 4, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.CENTER, new Insets(4, 4, 4, 4), 0, 0));
    // GridBagConstraints clearConstraints = new
    // GridBagConstraints(0,1,1,1,1.0,1.0,GridBagConstraints.EAST,GridBagConstraints.NONE,
    // new Insets(4,4,4,4),0,0);
    // gbl.setConstraints(modeLabel, clearConstraints);
    // this.add(modeLabel);
    add(modeLabel, new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    displayModeBox = new AcopJComBox<>("DisplayMode", AcopDisplayMode.values());
    add(displayModeBox, new GridBagConstraints(1, 1, 3, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
    add(fftLabel, new GridBagConstraints(0, 2, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    fftBox = new AcopJComBox<>("FFT", AcopFFTEnum.values());
    add(fftBox, new GridBagConstraints(1, 2, 3, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
    gCon = new GridBagConstraints(0, 3, 1, 1, 1.0, 1.0, GridBagConstraints.EAST, GridBagConstraints.NONE,
        new Insets(4, 4, 4, 4), 0, 0);
    acopReverseY = new AcopJRadioButton("1/y", "Invert", "invert");
    add(acopReverseY, gCon);
    gCon.gridx = 1;
    Wraparound = new AcopJRadioButton("wrapAround", "WrapAround", "wrapAround");
    add(Wraparound, gCon);
    gCon.gridx = 2;
    gCon.gridwidth = 1;
    Tagging = new AcopJRadioButton("tagging", "Tagging", "tagging");
    add(Tagging, gCon);
    add(extendXLabel, new GridBagConstraints(0, 4, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    extXText = new AcopJFormattedTextField("1", "XExtent", double.class);
    add(extXText, new GridBagConstraints(1, 4, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
    add(extendYLabel, new GridBagConstraints(2, 4, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    extYText = new AcopJFormattedTextField("1", "YExtent", double.class);
    add(extYText, new GridBagConstraints(3, 4, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
    gCon = new GridBagConstraints(1, 5, 1, 1, 1.0, 1.0, GridBagConstraints.EAST, GridBagConstraints.NONE,
        new Insets(4, 4, 4, 4), 0, 0);
    colorFill = new AcopJRadioButton("Color Fill", "ColorFilled", "colorFilled");
    add(colorFill, gCon);
    gCon.gridx = 3;
    randomData = new AcopJRadioButton("Random Data", "RandomData", "randomData");
    add(randomData, gCon);
    // averagging:
    offX = 4;
    add(averageLabel, new GridBagConstraints(offX + 0, 0, 4, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.CENTER, new Insets(4, 4, 4, 4), 0, 0));
    add(averModeLabel, new GridBagConstraints(offX + 0, 1, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    averModeBox = new AcopJComBox<>("AveragingMode", AcopAveragingMode.values());
    add(averModeBox, new GridBagConstraints(offX + 1, 1, 3, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
    add(averageDepthLabel, new GridBagConstraints(offX + 0, 2, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    averDepthText = new AcopJFormattedTextField("1", "AveragingDepth", int.class, true, 0,
        AcopConst.N_DEF_DISPLAY);
    add(averDepthText, new GridBagConstraints(offX + 1, 2, 3, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
    add(screenLabel, new GridBagConstraints(offX + 0, 3, 4, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.CENTER, new Insets(4, 4, 4, 4), 0, 0));
    add(screenDepthLabel, new GridBagConstraints(offX + 0, 4, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    screenDepthText = new AcopJFormattedTextField("1", "ScreenDepth", int.class, true, 1,
        AcopConst.MAX_DISPLAY);
    add(screenDepthText, new GridBagConstraints(offX + 1, 4, 3, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
    reuseScreen = new AcopJRadioButton("Reuse Screen", "ReuseScreen", "reuseScreen");
    add(reuseScreen, new GridBagConstraints(offX + 0, 5, 3, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    this.setSize(478, 297);
  }
  public void setAcopBean(Acop acopBean)
  {
    displayModeBox.setAcopBean(acopBean);
    fftBox.setAcopBean(acopBean);
    acopReverseY.setAcopBean(acopBean);
    Wraparound.setAcopBean(acopBean);
    Tagging.setAcopBean(acopBean);
    extXText.setAcopBean(acopBean);
    extYText.setAcopBean(acopBean);
    colorFill.setAcopBean(acopBean);
    randomData.setAcopBean(acopBean);
    averModeBox.setAcopBean(acopBean);
    averDepthText.setAcopBean(acopBean);
    screenDepthText.setAcopBean(acopBean);
    reuseScreen.setAcopBean(acopBean);
  }
	public void setObject(Object bean) {
		setAcopBean((Acop)bean);
		
	}
} // @jve:decl-index=0:visual-constraint="10,10"
