/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.chart.customizer;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.LayoutManager;
import java.beans.Customizer;

import javax.swing.JLabel;
import javax.swing.JPanel;
import de.desy.acop.chart.Acop;
import de.desy.acop.chart.AcopCursorMarkerModeEnum;
import de.desy.acop.chart.AcopMarkerModeEnum;

public class MarkersCustomizerPanel extends JPanel implements Customizer
{

  private static final long serialVersionUID = 8529414138275519256L;
  public static final String[] MARKERS_PROPERTIES = {
		"leadingEdgeYExtent","leadingEdgeXExtent","leadingEdgeWidth","leadingEdgeStyle",
		"leadingEdgeMode", "markerYExtent","markerXExtent","markerWidth",
		"markerStyle","markerMode"
	};
  private JLabel markerLabel = null;
  private JLabel modeLabel = null;
  private AcopJComBox<AcopMarkerModeEnum> modeBox;
  private JLabel styleLabel = null;
  private AcopJComBox<AcopCursorMarkerModeEnum> styleBox;
  private JLabel widthLabel = null;
  private AcopJFormattedTextField widthText;
  private JLabel extendXLabel, extendYLabel;
  private AcopJFormattedTextField extXText, extYText;
  
  private JLabel cursorLabel = null;
  private JLabel cursormodeLabel = null;
  private AcopJComBox<AcopMarkerModeEnum> cursormodeBox;
  private JLabel cursorstyleLabel = null;
  private AcopJComBox<AcopCursorMarkerModeEnum> cursorstyleBox;
  private JLabel cursorwidthLabel = null;
  private AcopJFormattedTextField cursorwidthText;
  private JLabel cursorextendXLabel, cursorextendYLabel;
  private AcopJFormattedTextField cursorextXText, cursorextYText;
  
  private JLabel leadingLabel = null;
  private JLabel leadingmodeLabel = null;
  private AcopJComBox<AcopMarkerModeEnum> leadingmodeBox;
  private JLabel leadingstyleLabel = null;
  private AcopJComBox<AcopCursorMarkerModeEnum> leadingstyleBox;
  private JLabel leadingwidthLabel = null;
  private AcopJFormattedTextField leadingwidthText;
  private JLabel leadingextendXLabel, leadingextendYLabel;
  private AcopJFormattedTextField leadingextXText, leadingextYText;
  
  public MarkersCustomizerPanel(LayoutManager layout, boolean isDoubleBuffered)
  {
    super(layout, isDoubleBuffered);
    initialize();
  }
  public MarkersCustomizerPanel(LayoutManager layout)
  {
    super(layout);
    initialize();
  }
  public MarkersCustomizerPanel(boolean isDoubleBuffered)
  {
    super(isDoubleBuffered);
    initialize();
  }
  public MarkersCustomizerPanel()
  {
    super();
    initialize();
  }
  /**
   * This method initializes this
   * 
   * @return void
   */
  private void initialize()
  {
   setLayout(new GridBagLayout());
   if (markerLabel == null)
   {
     markerLabel = new JLabel("-- Display Marker --");
   }
   if (modeLabel == null)
   {
     modeLabel = new JLabel("Mode:");
   }
   if (styleLabel == null)
   {
     styleLabel = new JLabel("Style:");
   }
   if (widthLabel == null)
   {
     widthLabel = new JLabel("Width:");
   }
   if (extendXLabel == null)
   {
     extendXLabel = new JLabel("X-Extent:");
   }
   if (extendYLabel == null)
   {
     extendYLabel = new JLabel("Y-Exttnt:");
   }

   if (cursorLabel == null)
   {
     cursorLabel = new JLabel("-- Cursor Marker --");
   }
   if (cursormodeLabel == null)
   {
     cursormodeLabel = new JLabel("Mode:");
   }
   if (cursorstyleLabel == null)
   {
     cursorstyleLabel = new JLabel("Style:");
   }
   if (cursorwidthLabel == null)
   {
     cursorwidthLabel = new JLabel("Width:");
   }
   if (cursorextendXLabel == null)
   {
     cursorextendXLabel = new JLabel("X-Extent:");
   }
   if (cursorextendYLabel == null)
   {
     cursorextendYLabel = new JLabel("Y-Exttnt:");
   }

   if (leadingLabel == null)
   {
     leadingLabel = new JLabel("-- Leading Edge --");
   }
   if (leadingmodeLabel == null)
   {
     leadingmodeLabel = new JLabel("Mode:");
   }
   if (leadingstyleLabel == null)
   {
     leadingstyleLabel = new JLabel("Style:");
   }
   if (leadingwidthLabel == null)
   {
     leadingwidthLabel = new JLabel("Width:");
   }
   if (leadingextendXLabel == null)
   {
     leadingextendXLabel = new JLabel("X-Extent:");
   }
   if (leadingextendYLabel == null)
   {
     leadingextendYLabel = new JLabel("Y-Exttnt:");
   }

   add(markerLabel, new GridBagConstraints(0, 0, 2, 1, 1.0, 1.0, GridBagConstraints.CENTER,
       GridBagConstraints.CENTER, new Insets(4, 4, 4, 4), 0, 0));
   add(modeLabel, new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
       GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
   modeBox = new AcopJComBox<>("MarkerMode", AcopMarkerModeEnum.values());
   add(modeBox, new GridBagConstraints(1, 1, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER,
       GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
   
   add(styleLabel, new GridBagConstraints(0, 2, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
       GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
   styleBox = new AcopJComBox<>("MarkerStyle", AcopCursorMarkerModeEnum.values());
   add(styleBox, new GridBagConstraints(1, 2, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER,
       GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));

   add(widthLabel, new GridBagConstraints(0, 3, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
       GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
   widthText = new AcopJFormattedTextField("10", "MarkerWidth", int.class);
   add(widthText, new GridBagConstraints(1, 3, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER,
       GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
   
   add(extendXLabel, new GridBagConstraints(0, 4, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
       GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
   extXText = new AcopJFormattedTextField("10", "MarkerXExtent", double.class);
   add(extXText, new GridBagConstraints(1, 4, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER,
       GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));

   add(extendYLabel, new GridBagConstraints(0, 5, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
       GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
   extYText = new AcopJFormattedTextField("10", "MarkerYExtent", double.class);
   add(extYText, new GridBagConstraints(1, 5, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER,
       GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
  

   add(cursorLabel, new GridBagConstraints(2, 0, 2, 1, 1.0, 1.0, GridBagConstraints.CENTER,
       GridBagConstraints.CENTER, new Insets(4, 4, 4, 4), 0, 0));

   add(cursormodeLabel, new GridBagConstraints(2, 1, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
       GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
   cursormodeBox = new AcopJComBox<>("MarkerMode", AcopMarkerModeEnum.values());
   add(cursormodeBox, new GridBagConstraints(3, 1, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER,
       GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
   
   add(cursorstyleLabel, new GridBagConstraints(2, 2, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
       GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
   cursorstyleBox = new AcopJComBox<>("MarkerStyle", AcopCursorMarkerModeEnum.values());
   add(cursorstyleBox, new GridBagConstraints(3, 2, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER,
       GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));

   add(cursorwidthLabel, new GridBagConstraints(2, 3, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
       GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
   cursorwidthText = new AcopJFormattedTextField("10", "MarkerWidth", int.class);
   add(cursorwidthText, new GridBagConstraints(3, 3, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER,
       GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
   
   add(cursorextendXLabel, new GridBagConstraints(2, 4, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
       GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
   cursorextXText = new AcopJFormattedTextField("10", "MarkerXExtent", double.class);
   add(cursorextXText, new GridBagConstraints(3, 4, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER,
       GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));

   add(cursorextendYLabel, new GridBagConstraints(2, 5, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
       GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
   cursorextYText = new AcopJFormattedTextField("10", "MarkerYExtent", double.class);
   add(cursorextYText, new GridBagConstraints(3, 5, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER,
       GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
   
   
   add(leadingLabel, new GridBagConstraints(4, 0, 2, 1, 1.0, 1.0, GridBagConstraints.CENTER,
       GridBagConstraints.CENTER, new Insets(4, 4, 4, 4), 0, 0));
   add(leadingmodeLabel, new GridBagConstraints(4, 1, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
       GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
   leadingmodeBox = new AcopJComBox<>("LeadingEdgeMode", AcopMarkerModeEnum.values());
   add(leadingmodeBox, new GridBagConstraints(5, 1, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER,
       GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
   
   add(leadingstyleLabel, new GridBagConstraints(4, 2, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
       GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
   leadingstyleBox = new AcopJComBox<>("LeadingEdgeStyle", AcopCursorMarkerModeEnum.values());
   add(leadingstyleBox, new GridBagConstraints(5, 2, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER,
       GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));

   add(leadingwidthLabel, new GridBagConstraints(4, 3, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
       GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
   leadingwidthText = new AcopJFormattedTextField("10", "LeadingEdgeWidth", int.class);
   add(leadingwidthText, new GridBagConstraints(5, 3, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER,
       GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
   
   add(leadingextendXLabel, new GridBagConstraints(4, 4, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
       GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
   leadingextXText = new AcopJFormattedTextField("10", "LeadingEdgeXExtent", double.class);
   add(leadingextXText, new GridBagConstraints(5, 4, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER,
       GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));

   add(leadingextendYLabel, new GridBagConstraints(4, 5, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
       GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
   leadingextYText = new AcopJFormattedTextField("10", "LeadingEdgeYExtent", double.class);
   add(leadingextYText, new GridBagConstraints(5, 5, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER,
       GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
   
   this.setSize(300, 200);
  }
  public void setAcopBean(Acop acopBean)
  {
    modeBox.setAcopBean(acopBean);
    styleBox.setAcopBean(acopBean);
    widthText.setAcopBean(acopBean);
    extXText.setAcopBean(acopBean);
    extYText.setAcopBean(acopBean);
    
    cursormodeBox.setAcopBean(acopBean);
    cursorstyleBox.setAcopBean(acopBean);
    cursorwidthText.setAcopBean(acopBean);
    cursorextXText.setAcopBean(acopBean);
    cursorextYText.setAcopBean(acopBean);

    leadingmodeBox.setAcopBean(acopBean);
    leadingstyleBox.setAcopBean(acopBean);
    leadingwidthText.setAcopBean(acopBean);
    leadingextXText.setAcopBean(acopBean);
    leadingextYText.setAcopBean(acopBean);
  }
  
  public void setObject(Object bean) {
	setAcopBean((Acop)bean);
		
  }
}
