/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.displayers.selector;

import java.awt.Component;

import de.desy.acop.chart.AcopDisplayMode;
import de.desy.acop.chart.AcopDrawStyleEnum;
import de.desy.acop.chart.AcopFFTEnum;
import de.desy.acop.displayers.tools.AcopDisplayerParameters;
import de.desy.acop.displayers.tools.AcopGraphParameters;
import de.desy.acop.transport.ConnectionParameters;

/**
 * <code>AcopGraphParametersDialog</code> enables generation of 
 * <code>AcopGraphParameters</code>. It uses the <code>GraphCustomizerPanel</code>
 * to customize the parameters.
 * 
 * @author Jaka Bobnar, Cosylab
 * @see AcopGraphParameters
 * @see GraphCustomizerPanel
 */
public class AcopGraphParametersDialog extends AcopDisplayerParametersDialog<AcopGraphParameters>{

	private static final long serialVersionUID = 4031445693248397678L;
	private GraphCustomizerPanel graphCustomizerPanel;
	
	/**
	 * Constructs a new AcopGraphParametersDialog.
	 * 
	 * @param comp the parent of the dialog
	 */
	public AcopGraphParametersDialog(Component comp) {
		super(comp);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.AcopDisplayerParametersDialog#generateParameters()
	 */
	@Override
	protected AcopGraphParameters generateParameters() {
	    return  new AcopGraphParameters(
		getDisplayerParameters(),
		getGraphCustomizerPanel().getColor(),
		getGraphCustomizerPanel().getGraphFFT(),
		getGraphCustomizerPanel().getGraphStyle(),
		getGraphCustomizerPanel().getGraphMode(),
		getGraphCustomizerPanel().getGraphWidth(),
		getGraphCustomizerPanel().isTrend(),
		getGraphCustomizerPanel().getTrendLength(),
		getConverterPanel().getConverter());
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.AcopDisplayerParametersDialog#getCustomizerPanel()
	 */
	@Override
	protected MultipleDisplayerAbstractSettingsPanel<AcopGraphParameters> getCustomizerPanel() {
		return getGraphCustomizerPanel();
	}
	
	private GraphCustomizerPanel getGraphCustomizerPanel()
	{
		if (graphCustomizerPanel == null) {
			graphCustomizerPanel = new GraphCustomizerPanel();
			graphCustomizerPanel.setParameters(null);
		}
		return graphCustomizerPanel;
	}
	
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.AcopDisplayerParametersDialog#showChartDialog(java.awt.Component, de.desy.acop.displayers.tools.AcopDisplayerParameters)
	 */
	@Override
	public AcopGraphParameters showDialog(Component component, AcopDisplayerParameters adp) {
		this.getGraphCustomizerPanel().setColor(null);
		this.getGraphCustomizerPanel().setGraphFFT(AcopFFTEnum.NoFFT.ordinal());
		this.getGraphCustomizerPanel().setGraphWidth(1);
		this.getGraphCustomizerPanel().setDrawStyle(AcopDrawStyleEnum.PS_SOLID.ordinal());
		if (SelectorUtilities.isChannel(adp.getConnectionParameters())) {
			getGraphCustomizerPanel().setGraphMode(AcopDisplayMode.SimpleHistogram.ordinal());
		} else {
			getGraphCustomizerPanel().setGraphMode(AcopDisplayMode.PolyLine.ordinal());
		}
		return super.showDialog(component, adp);
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.AcopDisplayerParametersDialog#showChartDialog(java.awt.Component, de.desy.acop.transport.ConnectionParameters)
	 */
	@Override
	public AcopGraphParameters showDialog(Component component, ConnectionParameters cp) {
		this.getGraphCustomizerPanel().setColor(null);
		this.getGraphCustomizerPanel().setGraphFFT(AcopFFTEnum.NoFFT.ordinal());
		this.getGraphCustomizerPanel().setGraphWidth(1);
		this.getGraphCustomizerPanel().setDrawStyle(AcopDrawStyleEnum.PS_SOLID.ordinal());
		if (SelectorUtilities.isChannel(cp)) {
			getGraphCustomizerPanel().setGraphMode(AcopDisplayMode.SimpleHistogram.ordinal());
		} else {
			getGraphCustomizerPanel().setGraphMode(AcopDisplayMode.PolyLine.ordinal());
		}
		return super.showDialog(component, cp);
	}

}
