/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.displayers.selector;

import java.awt.Component;

import javax.swing.JOptionPane;

/**
 * This class offers a method that opens an input dialog with a list of 
 * Integers where an index can be selected and returned.
 * 
 * @author tkusterle
 *
 */
public class ChannelSelector {

	/**
	 * Displays a JDialog, which allows user to choose a specific index from the list.
	 * 
	 * @param c the owner of the dialog
	 * @param length the highest index number displayed (excluded)
	 * @param current default selected index
	 * @return the selected index
	 */
	public static int selectChannelDialog(Component c, int length, int current) {
		if (length == 1) return 0;
		Object[] possibilities = new Object[length];
		for (int i = 0; i < length; i++) {
			possibilities[i] = Integer.valueOf(i);
		}
		Object selected = JOptionPane.showInputDialog(
                c,
                "Select Channel:",
                "Select Index",
                JOptionPane.PLAIN_MESSAGE,
                null,
                possibilities,
                Integer.valueOf(current));
		if (selected == null) {
			return -1;
		} else {
			return ((Integer) selected).intValue();
		}
	}
	
}
