/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.displayers.selector;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.Beans;
import java.beans.Customizer;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyVetoException;

import javax.swing.JButton;
import javax.swing.JPanel;

import com.cosylab.gui.components.util.RunnerHelper;
import com.cosylab.util.CommonException;

import de.desy.acop.displayers.tools.ConnectionParametersReceiver;
import de.desy.acop.displayers.tools.MultipleAcopDisplayer;
import de.desy.acop.transport.ConnectionParameters;

/**
 * <code>ConnectionCustomizer</code> enables device and property selection
 * through the custumizer of the displayer. The Customizer supports Drag and Drop
 * of the selected property.
 * <p>
 * This customizer allow selection of a single <code>ConnectionParameters</code>
 * </p>
 * 
 * @author Jaka Bobnar, Cosylab
 * @see ConnectionParameters
 */
public class ConnectionCustomizer extends JPanel implements Customizer {
	
	private static final long serialVersionUID = -4511139029213172995L;

	private Object bean;
	
	private JPanel commandPanel;
	private SelectorGUI selectorGUI;
	
	private JButton connectButton;
	private boolean connectButtonVisible = true;
	
	private ConnectionParameters connectionParameters;
	
	/**
	 * Constructs a new ConnectionCustomizer.
	 *
	 */
	public ConnectionCustomizer() {
		super();
		initialize();
	}
	
	private void initialize() {
		
		this.setLayout(new GridBagLayout());
		this.add(getCommandPanel(), new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(11,11,11,11),0,0));
	}
	
	private JButton getConnectButton() {
		if (connectButton == null) {
			connectButton = new JButton("Connect");
			connectButton.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent e) {
					try {
						if (bean instanceof ConnectionParametersReceiver) {
							((ConnectionParametersReceiver)bean).setConnectionParameters(getSelectorGUI().getConnectionParameters());
						}
					} catch (CommonException e1) {
						e1.printStackTrace();
					} catch (PropertyVetoException e1) {
						e1.printStackTrace();
					}
                }
				
			});
			connectButton.setVisible(connectButtonVisible);
			connectButton.setEnabled(false);
		}
		return connectButton;
	}
	
	private SelectorGUI getSelectorGUI() {
		if (selectorGUI == null) {
			selectorGUI = new SelectorGUI();
			selectorGUI.addSelectorGUIListener(new SelectorGUIListener() {
				public void selectionChanged(SelectorGUIEvent e) {
					setButtonsEnabled(e.isSelectionComplete());
				}
			});
			selectorGUI.addPropertyChangeListener(ConnectionParametersReceiver.CONNECTION_PARAMETERS_PROPERTY, new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent evt) {
				    if (!ConnectionParametersReceiver.CONNECTION_PARAMETERS_PROPERTY.equals(evt.getPropertyName())) return;
				    setConnectionParameters((ConnectionParameters) evt.getNewValue());
				}
			});
			if (bean != null)
				selectorGUI.setObject(bean);
		}
		return selectorGUI;
	}
	
	private JPanel getCommandPanel() {
		if (commandPanel == null) {
			commandPanel = new JPanel(new GridBagLayout());
			commandPanel.add(getSelectorGUI(), new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0,0,0,0),0,0));
			
			if (!Beans.isDesignTime()) {
				commandPanel.add(getConnectButton(), new GridBagConstraints(0,1,2,1,1,0,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0,0,0,0),0,0));
			}			
		}
		return commandPanel;
	}
	
	
	/*
	 * (non-Javadoc)
	 * @see java.beans.Customizer#setObject(java.lang.Object)
	 */
	public void setObject(Object bean) {
		if (bean instanceof MultipleAcopDisplayer) {
			this.bean = bean;
		} else if (bean instanceof ConnectionParametersReceiver) {
			this.bean = bean;
			((ConnectionParametersReceiver)this.bean).addPropertyChangeListener(ConnectionParametersReceiver.CONNECTION_PARAMETERS_PROPERTY,
					new PropertyChangeListener(){
				public void propertyChange(PropertyChangeEvent evt) {
					setConnectionParameters(((ConnectionParametersReceiver)ConnectionCustomizer.this.bean).getConnectionParameters());				
				}
			});
		} else {
			throw new IllegalArgumentException("Only ConnectionParametersReceiver and" +
					" MultipleAcopDisplayer can use this Customizer.");
		}
	
		getSelectorGUI().setObject(bean);
//		setConnectionParameters(getSelectorGUI().getConnectionParameters());
    }

	/**
	 * Returns the selected ConnectionParameters.
	 * 
     * @return the parameters
     */
    public ConnectionParameters getConnectionParameters() {
    	return connectionParameters;
    }

	/**
	 * Sets the ConnectionParameters and applies values to the selector.
	 *  
     * @param parameters the parameters to set
     */
    public void setConnectionParameters(ConnectionParameters parameters) {
    	if (parameters != null && parameters.equals(connectionParameters)) return;
    	ConnectionParameters old = connectionParameters;
    	connectionParameters = parameters;
    	if (Beans.isDesignTime() && bean != null && bean instanceof ConnectionParametersReceiver && SelectorUtilities.isConnectionParametersValid(connectionParameters)) {
			try {
				((ConnectionParametersReceiver)bean).setConnectionParameters(parameters);
            } catch (CommonException e) {
                 e.printStackTrace();
            } catch (PropertyVetoException e) {
                 e.printStackTrace();
            }
		}
    	getSelectorGUI().setConnectionParameters(parameters);
    	if (SelectorUtilities.isConnectionParametersValid(connectionParameters)) 
    		setButtonsEnabled(true);
    	firePropertyChange(ConnectionParametersReceiver.CONNECTION_PARAMETERS_PROPERTY, old, parameters);
    }
    
	/**
	 * Returns true if connect button is visible.
	 * @return true if connect button is visible
	 */
	public boolean isConnectButtonVisible() {
		return connectButtonVisible;
	}

	
	/**
	 * Set the connect button visible/invisible.
	 * .
	 * @param connectButtonVisible true if connect button should be visible
	 */
	public void setConnectButtonVisible(boolean connectButtonVisible) {
		this.connectButtonVisible = connectButtonVisible;
		if (connectButton!=null) {
			connectButton.setVisible(connectButtonVisible);
		}
	}
	
	private void setButtonsEnabled(boolean enabled) {
		getConnectButton().setEnabled(enabled);
	}
	
	/**
	 * Flag indicating whether array size choosing should also be available or only
	 * the remote name and access paramters should be available.
	 * 
	 * @param onlyCP if true only the remote name and access parameters can be selected
	 */
	public void setOnlyConnectionParametersMode(boolean onlyCP) {
		getSelectorGUI().setOnlyConnectionParametersMode(onlyCP);
	}
	
	/**
	 * Returns true if only remote name and access parameters can be selected.
	 * @return true if only remote name and access parameters can be selected
	 */
	public boolean getOnlyConnectionParametersMode() {
		return getSelectorGUI().getOnlyConnectionParametersMode();
	}
		
	public static void main(String[] args) {
		try {
			
			ConnectionCustomizer c= new ConnectionCustomizer();
			RunnerHelper.runComponent(c,300,300);
			
//			c.addPropertyChangeListener(new PropertyChangeListener() {
//			
//				public void propertyChange(PropertyChangeEvent evt) {
//					System.out.println("E: " + evt);
//				}
//			
//			});
			
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	

}
/* __oOo__ */
