/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.displayers.selector;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JColorChooser;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import com.cosylab.gui.components.NumberField;
import com.cosylab.gui.components.util.RunnerHelper;
import com.cosylab.gui.util.GUIUtilities;
import com.cosylab.gui.components.util.ColorManager;

import de.desy.acop.chart.AcopDisplayMode;
import de.desy.acop.chart.AcopDrawStyleEnum;
import de.desy.acop.chart.AcopFFTEnum;
import de.desy.acop.displayers.tools.AcopGraphParameters;

/**
 * <code>GraphCustomizerPanel</code> is a customization panel for 
 * <code>AcopGraphParameters</code>. Panel has user interface which enable
 * selecting various properties which are part of the graph parameters.
 * 
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 * @see AcopGraphParameters
 *
 */
public class GraphCustomizerPanel extends MultipleDisplayerAbstractSettingsPanel<AcopGraphParameters> {

		private static final long serialVersionUID = -2986099881478526811L;
		private JLabel graphStyleLabel;
		private JLabel graphModeLabel;
		private JLabel graphFFTLabel;
		private JLabel graphColorLabel;
		private JLabel trendLengthLabel;
		private JComboBox<AcopDrawStyleEnum> drawStyleCombo;
		private JComboBox<AcopDisplayMode> graphModeCombo;
		private JComboBox<AcopFFTEnum> graphFFTCombo;
		private JLabel graphColorSelector;
		private Color selectedColor;
		private JPanel propertiesPanel;
		private JButton colorPicker;
		private JCheckBox trendCheckBox;
		private NumberField trendLength;
		private JLabel graphWidthLabel;
		private NumberField graphWidthField;
		private JCheckBox useDefaultColor;
		
		/**
		 * Constructs new GraphCustomizerPanel.
		 *
		 */
		public GraphCustomizerPanel() {
			setLayout(new GridBagLayout());
			add(getPropertiesPanel(), new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.NORTH, GridBagConstraints.NONE, new Insets(0,0,0,0),0,0));
		}
		
		private JPanel getPropertiesPanel() {
			if (propertiesPanel == null) {
		        propertiesPanel = new JPanel(new GridBagLayout());
		        
		        graphStyleLabel = new JLabel("Style");
		        GUIUtilities.setFixedSize(graphStyleLabel,new Dimension(50,21));
		        drawStyleCombo = new JComboBox<>(AcopDrawStyleEnum.values());
		        GUIUtilities.setFixedSize(drawStyleCombo,new Dimension(120,21));
		        propertiesPanel.add(graphStyleLabel, new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1,1,1,1),0,0));
		        propertiesPanel.add(drawStyleCombo, new GridBagConstraints(1,0,3,1,1,1,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(1,1,1,1),0,0));
		        
		        graphModeLabel = new JLabel("Mode");
		        GUIUtilities.setFixedSize(graphModeLabel,new Dimension(50,21));
		        graphModeCombo = new JComboBox<>(AcopDisplayMode.values());
		        GUIUtilities.setFixedSize(graphModeCombo,new Dimension(120,21));
		        propertiesPanel.add(graphModeLabel, new GridBagConstraints(0,1,1,1,1,1,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1,1,1,1),0,0));
		        propertiesPanel.add(graphModeCombo, new GridBagConstraints(1,1,3,1,1,1,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(1,1,1,1),0,0));
		        
		        graphFFTLabel = new JLabel("FFT");
		        GUIUtilities.setFixedSize(graphFFTLabel,new Dimension(50,21));
		        graphFFTCombo = new JComboBox<>(AcopFFTEnum.values());
		        GUIUtilities.setFixedSize(graphFFTCombo,new Dimension(120,21));
		        propertiesPanel.add(graphFFTLabel, new GridBagConstraints(0,2,1,1,1,1,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1,1,1,1),0,0));
		        propertiesPanel.add(graphFFTCombo, new GridBagConstraints(1,2,3,1,1,1,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(1,1,1,1),0,0));
		        
		        graphColorLabel = new JLabel("Color");
		        GUIUtilities.setFixedSize(graphColorLabel,new Dimension(50,21));
		        graphColorSelector = new JLabel();
		        graphColorSelector.setOpaque(true);
		        GUIUtilities.setFixedSize(graphColorSelector,new Dimension(88,21));

		        selectedColor = ColorManager.DEFAULT_COLORS[0];
		        graphColorSelector.setBackground(selectedColor);
		        propertiesPanel.add(graphColorLabel, new GridBagConstraints(0,3,1,1,1,1,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1,1,1,1),0,0));
		        propertiesPanel.add(graphColorSelector, new GridBagConstraints(1,3,2,1,1,1,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(1,1,1,1),0,0));
		        
		        colorPicker = new JButton("...");
		        GUIUtilities.setFixedSize(colorPicker,new Dimension(28,21));
		        colorPicker.addActionListener(new ActionListener() {
		        	public void actionPerformed(ActionEvent e) {
		        		selectColor();
		        		if (selectedColor != null)
		        			graphColorSelector.setBackground(selectedColor);	
		        	}
		        });
		        propertiesPanel.add(colorPicker, new GridBagConstraints(3,3,1,1,0.1,1,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(1,1,1,1),0,0));
		        
		        useDefaultColor = new JCheckBox("Automatic color");
		        useDefaultColor.addItemListener(new ItemListener() {
		        	public void itemStateChanged(ItemEvent e) {
		        		colorPicker.setEnabled(!useDefaultColor.isSelected());
		        		graphColorSelector.setEnabled(!useDefaultColor.isSelected());
		        		graphColorLabel.setEnabled(!useDefaultColor.isSelected());
		        	}
		        });
		        propertiesPanel.add(useDefaultColor, new GridBagConstraints(0,4,4,1,1,1,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1,1,1,1),0,0));
		        useDefaultColor.setSelected(true);
		        
		        
		        graphWidthLabel = new JLabel("Width");
		        GUIUtilities.setFixedSize(graphWidthLabel,new Dimension(50,21));
		        graphWidthField = new NumberField();
		        GUIUtilities.setFixedSize(graphWidthField,new Dimension(120,21));
		        graphWidthField.setNumberType(Integer.class);
		        graphWidthField.setValue(Integer.valueOf(1));
		        propertiesPanel.add(graphWidthLabel, new GridBagConstraints(0,5,1,1,1,1,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1,1,1,1),0,0));
		        propertiesPanel.add(graphWidthField, new GridBagConstraints(1,5,3,1,1,1,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(1,1,1,1),0,0));
		        
		        trendCheckBox = new JCheckBox("Draw as trend");
		        trendCheckBox.addActionListener(new ActionListener() {
		        	public void actionPerformed(ActionEvent e) {
		        		if (trendCheckBox.isSelected()) {
		        			setGraphMode(AcopDisplayMode.PolyLine.ordinal());
		        		}
		        	}
		        });
		        propertiesPanel.add(trendCheckBox, new GridBagConstraints(0,6,4,1,0.1,1,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1,1,1,1),0,0));
		        
		        trendLengthLabel = new JLabel("Trend length");
		        GUIUtilities.setFixedSize(trendLengthLabel,new Dimension(80,21));
		        
		        trendLength = new NumberField();
		        GUIUtilities.setFixedSize(trendLength,new Dimension(90,21));
		        trendLength.setNumberType(Integer.class);
		        trendLength.setValue(Integer.valueOf(100));
		        propertiesPanel.add(trendLengthLabel, new GridBagConstraints(0,7,3,1,1,1,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1,1,1,1),0,0));
		        propertiesPanel.add(trendLength, new GridBagConstraints(2,7,2,1,1,1,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(1,1,1,1),0,0));
		        
	        }
			
	        return propertiesPanel;
		}
		
		private void selectColor() {
			selectedColor = JColorChooser.showDialog(this, "Graph color", selectedColor);
		}
		
		/**
		 * Returns the selected color.
		 * 
		 * @return selected color
		 */
		public Color getColor() {
			return useDefaultColor.isSelected() ? null : selectedColor;
		}
		
		/**
		 * Returns the selected graph style.
		 * 
		 * @return the graph style
		 * @see AcopDrawStyleEnum
		 */
		public int getGraphStyle() {
			return ((AcopDrawStyleEnum)drawStyleCombo.getSelectedItem()).ordinal();
		}
		
		/**
		 * Returns the selected graph mode.
		 * 
		 * @return the graph mode
		 * @see AcopDisplayMode
		 */
		public int getGraphMode() {
			return ((AcopDisplayMode)graphModeCombo.getSelectedItem()).ordinal();
		}
		
		/**
		 * Returns the selected fft mode.
		 * 
		 * @return the fft
		 * @see AcopFFTEnum
		 */
		public int getGraphFFT() {
			return ((AcopFFTEnum)graphFFTCombo.getSelectedItem()).ordinal();
		}
		
		/**
		 * Sets the color to the UI.
		 * 
		 * @param color new color
		 */
		public void setColor(Color color) {
			if (color==null) {
				color=Color.BLACK;
			}
			selectedColor = color;
			graphColorSelector.setBackground(selectedColor);
		}
		
		/**
		 * Sets the draw style to the UI.
		 * @param drawStyle new draw style
		 * @see AcopDrawStyleEnum
		 */
		public void setDrawStyle(int drawStyle) {
			drawStyleCombo.setSelectedIndex(drawStyle);
		}
		
		/**
		 * Sets the graph mode to the UI.
		 * 
		 * @param graphMode new graph mode
		 * @see AcopDisplayMode 
		 */
		public void setGraphMode(int graphMode) {
			graphModeCombo.setSelectedIndex(graphMode);
		}
		
		/**
		 * Sets the graph fft to the UI.
		 * 
		 * @param graphFFT new graph FFT
		 * @see AcopFFTEnum
		 */
		public void setGraphFFT(int graphFFT) {
			graphFFTCombo.setSelectedIndex(graphFFT);
		}
		
		/**
		 * Sets the trend mode.
		 * 
		 * @param isTrend true if in trend mode
		 */
		public void setTrend(boolean isTrend) {
			trendCheckBox.setSelected(isTrend);
		}
		
		/**
		 * Returns true if trend mode is selected.
		 * 
		 * @return true if trend selected
		 */
		public boolean isTrend() {
			return trendCheckBox.isSelected();
		}
		
		/**
		 * Sets the length of the trend line.
		 * 
		 * @param length new trend length
		 * 
		 */
		public void setTrendLength(int length) {
			trendLength.setValue(length);
		}
		
		/**
		 * Returns the length of the trend
		 * 
		 * @return trend length
		 */
		public int getTrendLength() {
			return trendLength.getValue().intValue();
		}
		
		/**
		 * Sets the graph width to the UI.
		 * 
		 * @param width the width of the graph lines
		 */
		public void setGraphWidth(int width) {
			graphWidthField.setValue(Integer.valueOf(width));
		}
		
		/**
		 * Returns the graph width.
		 * 
		 * @return the graph width
		 */
		public int getGraphWidth() {
			return graphWidthField.getValue().intValue();
		}
		
		/*
		 * (non-Javadoc)
		 * @see de.desy.acop.displayers.selector.MultipleDisplayerAbstractSettingsPanel#setParameters(de.desy.acop.displayers.tools.AcopDisplayerParameters)
		 */
		@Override
		public void setParameters(AcopGraphParameters param) {
			if (param == null) {
				setColor(null);
				setGraphFFT(0);
				setGraphMode(0);
				setGraphFFT(0);
				setTrend(false);
				setGraphWidth(1);
				setTrendLength(100);
			} else {
				setColor(param.getColor());
				setDrawStyle(param.getDrawStyle());
				setGraphMode(param.getMode());
				setGraphFFT(param.getFFT());
				setTrend(param.isTrend());
				setGraphWidth(param.getWidth());
				setTrendLength(param.getTrendLength());
			}
		}
		
		public static void main(String[] args) {
			RunnerHelper.runComponent(new GraphCustomizerPanel(), 600,600);
		}
		
		/**
		 * Enables/disables the trend controls. If true trend will be enabled.
		 * 
		 * @param enable
		 */
		public void toggleTrendControls(boolean enable) {
			trendCheckBox.setVisible(enable);
			trendLength.setVisible(enable);
			trendLengthLabel.setVisible(enable);
		}
		
	}
		
