/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.displayers.selector;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import com.cosylab.gui.util.GUIUtilities;

import de.desy.acop.displayers.table.AcopTableParameters;
import de.desy.acop.displayers.table.TableColumnType;

/**
 * <code>TableCustomizerPanel</code> is a customization panel for 
 * <code>AcopTableParameters</code>. Panel has user interface which enable
 * selecting various properties which are part of the table parameters.
 * 
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 * @see AcopTableParameters
 *
 */
public class TableCustomizerPanel extends MultipleDisplayerAbstractSettingsPanel<AcopTableParameters>{

	private static final long serialVersionUID = -4547732237651600499L;
	private JComboBox<TableColumnType> tableCustomizerCombo;
	private JLabel tableCustomizerLabel;
	
	private JTextField tableColumnNameField;
	private JLabel tableColumnNameLabel;
	
	private JPanel propertiesPanel;
	
	/**
	 * Constructs new TableCustomizerPanel.
	 *
	 */
	public TableCustomizerPanel() {
		super();
		initialize();
	}
	
	private void initialize() {
		setLayout(new GridBagLayout());
		add(getPropertiesPanel(), new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.NORTH, GridBagConstraints.NONE, new Insets(0,0,0,0),0,0));
	}
	
	private JPanel getPropertiesPanel() {
		if (propertiesPanel == null) {
			propertiesPanel = new JPanel(new GridBagLayout());
			tableCustomizerLabel = new JLabel("Type of data");
			GUIUtilities.setFixedSize(tableCustomizerLabel,new Dimension(100,21));
			propertiesPanel.add(tableCustomizerLabel, new GridBagConstraints(0,0,1,1,1,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1,1,1,1),0,0));
			propertiesPanel.add(getTableCustomizerCombo(), new GridBagConstraints(1,0,2,1,1,0,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(1,1,1,1),0,0));
			
			tableColumnNameLabel = new JLabel("Name of column");
			GUIUtilities.setFixedSize(tableColumnNameLabel,new Dimension(100,21));
			propertiesPanel.add(tableColumnNameLabel, new GridBagConstraints(0,1,1,1,1,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1,1,1,1),0,0));
			propertiesPanel.add(getTableColumnNameField(), new GridBagConstraints(1,1,2,1,1,0,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(1,1,1,1),0,0));
		
		}
		return propertiesPanel;
	}
	
	private JTextField getTableColumnNameField() {
		if (tableColumnNameField == null) {
			tableColumnNameField = new JTextField();
			GUIUtilities.setFixedSize(tableColumnNameField,new Dimension(150,21));
		}
		return tableColumnNameField;
	}
	
	private JComboBox<TableColumnType> getTableCustomizerCombo() {
		if (tableCustomizerCombo == null) {
			tableCustomizerCombo = new JComboBox<>(TableColumnType.values());
			GUIUtilities.setFixedSize(tableCustomizerCombo,new Dimension(150,21));
			tableCustomizerCombo.setSelectedItem(null);
		}
		return tableCustomizerCombo;
	}
	
	/**
	 * Returns the selected type.
	 * @return the type
	 */
	public TableColumnType getType() {
		if (getTableCustomizerCombo().getSelectedItem() != null) {
			return (TableColumnType) getTableCustomizerCombo().getSelectedItem();
		} else {
			return TableColumnType.VALUE;
		}
	}
	
	/**
	 * Sets the selected type. 
	 * @param type
	 */
	public void setType(TableColumnType type) {
		getTableCustomizerCombo().setSelectedItem(type);
	}
	
	/**
	 * Sets the name for the column.
	 * @param name column name
	 */
	@Override
	public void setName(String name) {
		if (name == null) {
			name = "";
		}
		getTableColumnNameField().setText(name);
		getTableColumnNameField().setToolTipText(name);
	}

	/**
	 * Returns the name for the column.
	 * 
	 * @return column name
	 */
	@Override
	public String getName() {
		return getTableColumnNameField().getText();
	}
		
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.MultipleDisplayerAbstractSettingsPanel#setParameters(de.desy.acop.displayers.tools.AcopDisplayerParameters)
	 */
	@Override
	public void setParameters(AcopTableParameters parameters) {
		if (parameters == null) {
			getTableCustomizerCombo().setSelectedItem(null);
			setName("");
		} else {
			getTableCustomizerCombo().setSelectedItem(parameters.getColumnType());
			setName(parameters.getName());
		}
	}

	
}
