/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.displayers.selector;

import java.awt.Component;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyEditorSupport;

import com.cosylab.gui.adapters.ConverterUtilities;

import de.desy.acop.displayers.table.AcopTableParameters;
import de.desy.acop.displayers.tools.MultipleAcopDisplayer;
import de.desy.acop.transport.ConnectionParameters;

/**
 * <code>TableSelectorPropertyEditor</code> is a <code>PropertyEditor</code>
 * for <code>AcopTableParameters</code>.
 * 
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 * @see AcopTableParameters
 */
public class TableSelectorPropertyEditor extends PropertyEditorSupport {

	private TableConnectionCustomizer editor;

	/**
     * Creates new instance of SelectorPropertyEditor.
     */
	public TableSelectorPropertyEditor() {
		super();
	}

	/*
	 * (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#getCustomEditor()
	 */
	@Override
	public Component getCustomEditor() {
		if (editor == null) {
			editor = new TableConnectionCustomizer();
			editor.setDisplayerParameters((AcopTableParameters[]) getValue());
			editor.addPropertyChangeListener(MultipleAcopDisplayer.DISPLAYER_PARAMETERS_PROPERTY,
			        new PropertyChangeListener() {
				        public void propertyChange(PropertyChangeEvent evt) {
					        setValueSilently(editor.getDisplayerParameters());
				        }
			        });
		}
		return editor;
	}

	/* 
	 * (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#getJavaInitializationString()
	 */
	@Override
	public String getJavaInitializationString() {
		AcopTableParameters[] params = (AcopTableParameters[]) getValue();
		StringBuffer buffer = new StringBuffer();
		buffer.append("new de.desy.acop.displayers.table.AcopTableParameters[]{");
		ConnectionParameters cp; 
		if (params != null) {
			for (AcopTableParameters p : params) {
				cp = p.getConnectionParameters();
				buffer.append("new de.desy.acop.displayers.table.AcopTableParameters(");
				buffer.append("new de.desy.acop.transport.ConnectionParameters(\""
				        + cp.getRemoteName()
				        + "\",de.desy.acop.transport.AccessMode."
				        + cp.getAccessMode() + ", " + cp.getAccessRate() + "," + cp.getPropertySize() + "," + SelectorUtilities.getExtendedDataString(cp) + "),");
				buffer.append("de.desy.acop.displayers.table.TableDisplayType."+p.getColumnType() + 
						(p.getConverter() == null ? ", null" : ", " + 
						ConverterUtilities.getInitializationString(p.getConverter())) + "),");
			}
		}
		return buffer.toString() + "}";
	}

	/*
	 * (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#supportsCustomEditor()
	 */
	@Override
	public boolean supportsCustomEditor() {
		return true;
	}
	

	/*
	 * (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#getAsText()
	 */
	@Override
	public String getAsText() {
		AcopTableParameters[] params = (AcopTableParameters[]) getValue();	
	    if (params == null || params.length == 0) return "";
	    StringBuffer buffer = new StringBuffer();
	    for (AcopTableParameters cp : params) {
	    	buffer.append(cp.toString() + ";");
	    }
	    return buffer.toString();
	}

	
	private void setValueSilently(Object value) {
		super.setValue(value);
	}
	/*
     * (non-Javadoc)
     * 
     * @see java.beans.PropertyEditorSupport#setValue(java.lang.Object)
     */
	@Override
	public void setValue(Object value) {
		super.setValue(value);
		if (editor != null) {
			editor.setDisplayerParameters((AcopTableParameters[]) value);
		}
	}
}

/* __oOo__ */
