/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.displayers.selector;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.cosylab.gui.components.util.RunnerHelper;

import de.desy.acop.displayers.AcopTrendChart;
import de.desy.acop.displayers.chart.HistoryParameters;
import de.desy.acop.displayers.chart.TrendSelector;
import de.desy.acop.displayers.tools.AcopDisplayer;
import de.desy.acop.displayers.tools.AcopGraphHistoryParameters;
import de.desy.acop.displayers.tools.AcopGraphParameters;
import de.desy.acop.transport.ConnectionParameters;

/**
 * 
 * <code>TrendChartConnectionCustomizer</code> is a connection customizer
 * which handles <code>AcopGraphHistoryParameters</code>.
 * 
 * @author Tilen Kusterle, Cosylab
 * @see AcopGraphHistoryParameters
 */
public class TrendChartConnectionCustomizer extends ChartConnectionCustomizer {
	
	public static void main(String[] args) {
		RunnerHelper.runComponent(new TrendChartConnectionCustomizer(), 800,400);
	}
	
	private static final long serialVersionUID = 1L;
	
	private TrendSelector trendSelector;
	private JPanel trendPanel;
	private JButton suggestButton;
	private JButton applyButton;
	private SelectorGUI selector;
	
	/**
	 * Constructs a new TrendChartConnectionCustomizer.
	 *
	 */
	public TrendChartConnectionCustomizer() {
		super();
		JPanel p = new JPanel(new GridBagLayout());
		p.add(getTrendPanel(), new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, new Insets(0,0,0,0),0,0));
		getCustomizerTabbedPane().add(p, "History", 0);
		
		getSelectorGUI().addSelectorGUIListener(new SelectorGUIListener() {

			public void selectionChanged(SelectorGUIEvent e) {
				setToTrendSelector(getSelectorGUI().getConnectionParameters(), null);
			}});
		getSelectorGUI().addPropertyChangeListener(AcopDisplayer.CONNECTION_PARAMETERS_PROPERTY,new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				setToTrendSelector(getSelectorGUI().getConnectionParameters(), null);
			}
		});
		getSelectionList().getSelectionModel().addListSelectionListener(new ListSelectionListener() {

			public void valueChanged(ListSelectionEvent e) {
				int sel = getSelectionList().getSelectedIndex();
        		if (sel >= 0) {
        			AcopGraphHistoryParameters parameters = (AcopGraphHistoryParameters) getSelectionList().getModel().getElementAt(sel);
        			HistoryParameters hp = parameters.getHistoryParameters();
        			getTrendSelector().setHistoryParameters(hp);
        			if (hp != null) getTrendSelector().setHistoryEnabled(true);
        			else getTrendSelector().setHistoryEnabled(false);
        		} else {
        			getTrendSelector().setHistoryParameters(null);
        			getTrendSelector().setHistoryEnabled(false);
        		}
				
			}});
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.MultipleConnectionCustomizer#getSelectorGUI()
	 */
	@Override
	protected SelectorGUI getSelectorGUI() {
		if (selector == null) {
			selector = new SelectorGUI();
			selector.addSelectorGUIListener(new SelectorGUIListener() {
				public void selectionChanged(SelectorGUIEvent e) {
					getAddButton().setEnabled(SelectorUtilities.isConnectionParametersValid(selector.getConnectionParameters()));
				}
			});
			selector.addPropertyChangeListener(AcopDisplayer.CONNECTION_PARAMETERS_PROPERTY,new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent e) {
					getAddButton().setEnabled(SelectorUtilities.isConnectionParametersValid(selector.getConnectionParameters()));
				}
			});
		}
		
	    return selector;
    }
	
	private JPanel getTrendPanel() {
		if (trendPanel == null) {
			trendPanel = new JPanel(new GridBagLayout());
			trendPanel.add(getTrendSelector(), new GridBagConstraints(0,0,2,1,1,1,GridBagConstraints.CENTER,GridBagConstraints.BOTH,new Insets(1,1,1,1),0,0));
			trendPanel.add(getSuggestButton(), new GridBagConstraints(0,1,1,1,1,0,GridBagConstraints.EAST,GridBagConstraints.NONE,new Insets(1,1,1,3),0,0));
			trendPanel.add(getApplyButton(), new GridBagConstraints(1,1,1,1,1,0,GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,3,1,1),0,0));
		}

		return trendPanel;
	}
	
	private JButton getSuggestButton() {
		if (suggestButton == null) {
			suggestButton = new JButton("Suggest Default");
			suggestButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					int sel = getSelectionList().getSelectedIndex();
	        		if (sel >= 0) {
	        			AcopGraphHistoryParameters parameters = (AcopGraphHistoryParameters) getSelectionList().getModel().getElementAt(sel);
	        			HistoryParameters hp = parameters.getHistoryParameters();
	        			getTrendSelector().setHistoryParameters(hp);
	        			setToTrendSelector(parameters.getConnectionParameters(), hp);
	        		} else {
	        			setToTrendSelector(getSelectorGUI().getConnectionParameters(), null);
	        		}
				}});
		}

		return suggestButton;
	}
	
	private TrendSelector getTrendSelector() {
    	if (trendSelector == null) {
			trendSelector = new TrendSelector();
		}
		return trendSelector;
    }

	/* (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.ChartConnectionCustomizer#getSettingsPanel()
	 */
	@Override
	protected MultipleDisplayerAbstractSettingsPanel<AcopGraphParameters> getSettingsPanel() {
		GraphCustomizerPanel gcp = (GraphCustomizerPanel) super.getSettingsPanel();
		gcp.toggleTrendControls(false);
		return gcp;
	}

	/* (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.ChartConnectionCustomizer#generateParameters()
	 */
	@Override
	protected AcopGraphParameters generateParameters() {
		AcopGraphParameters agp = super.generateParameters();
		agp.deriveWith(getTrendSelector().getSelectedIndex());
		return new AcopGraphHistoryParameters(agp, getTrendSelector().getHistoryParameters());
	}
	
	private void setToTrendSelector(ConnectionParameters cp, HistoryParameters hp) {
		if (hp == null) {
			getTrendSelector().setHistoryParameters(
					cp.getDeviceContext(),
					"HISTORY",
					cp.getDeviceName(),
					cp.getDeviceProperty());
		}
		else {
			getTrendSelector().setHistoryParameters(hp);
		}
		if (SelectorUtilities.isConnectionParametersValid(cp)) {
			getTrendSelector().setIndexMax(SelectorUtilities.getSequenceLength(cp)-1);
		}
		if (hp != null) getTrendSelector().setSelectedIndex(hp.getHistoryIndex());
		getApplyButton().setEnabled(selectedParameters != null);
	}
	
	@Override
	protected void setSelectedParameters(AcopGraphParameters selected) {
		super.setSelectedParameters(selected);
		if (selected instanceof AcopGraphHistoryParameters) {
			setToTrendSelector(((AcopGraphHistoryParameters)selected).getConnectionParameters(),
					((AcopGraphHistoryParameters)selected).getHistoryParameters());
		}
	}
	
	private JButton getApplyButton() {
		if (applyButton == null) {
			applyButton = new JButton("Apply");
			applyButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (selectedParameters != null) {
						if (selectedParameters instanceof AcopGraphHistoryParameters) {
							((AcopGraphHistoryParameters)selectedParameters).setHistoryParameters(getTrendSelector().getHistoryParameters());
						} else {
							selectedParameters.putParameter("historyParameters",getTrendSelector().getHistoryParameters());
						}
						if (getDisplayer() instanceof AcopTrendChart) {
							((AcopTrendChart)getDisplayer()).reloadHistory(selectedParameters);
						}
					}
				}
			});
		}
		return applyButton;
	}
}
