/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.displayers.selector;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.RejectedExecutionHandler;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import javax.swing.DefaultListCellRenderer;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import com.cosylab.gui.components.util.RunnerHelper;
import com.cosylab.gui.util.GUIUtilities;

import de.desy.acop.displayers.chart.HistoryParameters;
import de.desy.acop.displayers.tools.ConnectionParametersReceiver;

/**
 * 
 * <code>TrendHistorySelectorGUI</code> is a selector for 
 * history parameters.
 *
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 * @depreacated TINE doesn't allow duplicated TQuery use
 */
@Deprecated
public class TrendHistorySelectorGUI extends JPanel {
		
	private class ComboRenderer extends DefaultListCellRenderer {
		
		private static final long serialVersionUID = 1L;

		@Override
		public Component getListCellRendererComponent(JList<?> list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
		
			Component c = super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
			if (value != null && c instanceof JLabel) {
				((JLabel)c).setToolTipText(String.valueOf(value));
			}
			return c;
		}
	}
		
	private class IndexModel extends AcopComboBoxModel<Integer> {

		private static final long serialVersionUID = 1L;
		public IndexModel() {
			super(Integer.class);
		}
		
		public void setMaxIndex(int maxIndex) {
			removeAllElements();
			Integer[] values = new Integer[maxIndex+1];
			for (int i = 0; i <= maxIndex; i++) {
				values[i] = i;
			}
			setElements(values);
			if (values.length > 0) {
				setSelectedItem(values[0]);
			}
		}
	}
	
	
	private static final long serialVersionUID = 1647591422586736408L;
		
	private JPanel commandPanel;
	private JComboBox<String> deviceContextCombo;
	private JComboBox<String> subSystemCombo;
	private JComboBox<String> deviceServerCombo;
	private JComboBox<String> deviceNameCombo;
	private JComboBox<String> propertyNameCombo;
	private AcopComboBoxModel<String> deviceContextModel;
	private AcopComboBoxModel<String> subSystemModel;
	private AcopComboBoxModel<String> deviceServerModel;
	private AcopComboBoxModel<String> deviceNameModel;
	private AcopComboBoxModel<String> propertyNameModel;
	private JLabel deviceContextLabel;
	private JLabel subSystemLabel;
	private JLabel deviceServerLabel;
	private JLabel deviceNameLabel;
	private JLabel propertyNameLabel;
	private JLabel indexLabel;
	private JComboBox<Integer> indexCombo;
	private JCheckBox enableHistoryBox;
	private IndexModel indexModel;
	
	private HistoryParameters historyParameters;
	private volatile Selector selector; 
	
	private PropertyChangeListener selectorPropertyListener;
	private QueryListener selectorQueryListener;
	private ExceptionListener exceptionListener;
	private boolean settingFromSelector = false;
	private Dimension comboDim = new Dimension(170,21);
	
	private boolean historyEnabled = false;
	
	/**
	 * Constructs a new SelectorGUI.
	 *
	 */
	public TrendHistorySelectorGUI() {
		super();
		initialize();
	}
	
	private void initialize() {
		deviceContextModel= new AcopComboBoxModel<>(String.class);
		subSystemModel = new AcopComboBoxModel<>(String.class);
		deviceServerModel= new AcopComboBoxModel<>(String.class);
		deviceNameModel= new AcopComboBoxModel<>(String.class);
		propertyNameModel= new AcopComboBoxModel<>(String.class);
		setSelector(new Selector());
		
		this.setLayout(new GridBagLayout());
		this.add(getCommandPanel(), new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0,0,0,0),0,0));
	}
	
	private void applySettings() {
		
		try {
			int idx = 0;
			HistoryParameters old = historyParameters;
			historyParameters = new HistoryParameters(getSelector().getSelectedContext(), getSelector().getSelectedServer(), getSelector().getSelectedDevice(), getSelector().getSelectedProperty(), idx);
			firePropertyChange(ConnectionParametersReceiver.CONNECTION_PARAMETERS_PROPERTY,old,getHistoryParameters());
        } catch (Exception e) {
            e.printStackTrace();
        } 
	}
	
	private JPanel getCommandPanel() {
		if (commandPanel == null) {
			commandPanel = new JPanel(new GridBagLayout());
			
			commandPanel.add(getEnableHistoryBox(), new GridBagConstraints(0,0,2,1,1,1,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,1,1,1), 0,0));
			deviceContextLabel = new JLabel("Device context");
			GUIUtilities.setFixedSize(deviceContextLabel,new Dimension(120,21));
			commandPanel.add(deviceContextLabel, new GridBagConstraints(0,1,1,1,1,1,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0,1,1,1), 0,0));
			commandPanel.add(getDeviceContextCombo(), new GridBagConstraints(1,1,2,1,1,1,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,1,1,1), 0,0));
			
			subSystemLabel = new JLabel("Device subsystem");
			GUIUtilities.setFixedSize(subSystemLabel,new Dimension(120,21));
			commandPanel.add(subSystemLabel, new GridBagConstraints(0,2,1,1,1,1,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0,1,1,1), 0,0));
			commandPanel.add(getSubSystemCombo(), new GridBagConstraints(1,2,2,1,1,1,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,1,1,1), 0,0));
						
			deviceServerLabel = new JLabel("Device server");
			GUIUtilities.setFixedSize(deviceServerLabel,new Dimension(120,21));
			commandPanel.add(deviceServerLabel, new GridBagConstraints(0,3,1,1,1,1,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0,1,1,1), 0,0));
			commandPanel.add(getDeviceServerCombo(), new GridBagConstraints(1,3,2,1,1,1,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,1,1,1), 0,0));
			
			deviceNameLabel = new JLabel("Device name");
			GUIUtilities.setFixedSize(deviceNameLabel,new Dimension(120,21));
			commandPanel.add(deviceNameLabel, new GridBagConstraints(0,4,1,1,1,1,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0,1,1,1), 0,0));
			commandPanel.add(getDeviceNameCombo(), new GridBagConstraints(1,4,2,1,1,1,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,1,1,1), 0,0));
			
			propertyNameLabel = new JLabel("Property name");
			GUIUtilities.setFixedSize(propertyNameLabel,new Dimension(120,21));
			commandPanel.add(propertyNameLabel, new GridBagConstraints(0,5,1,1,1,1,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0,1,1,1), 0,0));
			commandPanel.add(getPropertyNameCombo(), new GridBagConstraints(1,5,2,1,1,1,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,1,1,1), 0,0));
									
			indexLabel = new JLabel("Index");
			GUIUtilities.setFixedSize(indexLabel,new Dimension(120,21));
			commandPanel.add(indexLabel, new GridBagConstraints(0,6,1,1,1,1,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0,1,1,1), 0,0));
			commandPanel.add(getIndexCombo(), new GridBagConstraints(1,6,2,1,1,1,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,1,1,1), 0,0));
			
			setCombosEnabled(true);
		}
		return commandPanel;
	}
	
	private JCheckBox getEnableHistoryBox() {
		if (enableHistoryBox == null) {
			enableHistoryBox = new JCheckBox("History Enabled");
			enableHistoryBox.addItemListener(new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					setHistoryEnabled(enableHistoryBox.isSelected());
				}
			});
			enableHistoryBox.setSelected(true);
		}

		return enableHistoryBox;
	}
		
	/**
	 * Returns true if history selection is enabled.
	 * 
	 * @return the historyEnabled
	 */
	public boolean isHistoryEnabled() {
		return historyEnabled;
	}

	/**
	 * Enables/disables history selection.
	 * 
	 * @param historyEnabled the historyEnabled to set
	 */
	public void setHistoryEnabled(boolean historyEnabled) {
		this.historyEnabled = historyEnabled;
		getEnableHistoryBox().setSelected(historyEnabled);
		setCombosEnabled(combosEnabled);
	}
		
	private JComboBox<Integer> getIndexCombo() {
		if (indexCombo == null) {
			indexCombo = new JComboBox<>();
			indexCombo.setModel(getIndexModel());
			GUIUtilities.setFixedSize(indexCombo,new Dimension(120,21));
		}

		return indexCombo;
	}
	
	private IndexModel getIndexModel() {
		if (indexModel == null) {
			indexModel = new IndexModel();
			indexModel.setMaxIndex(0);
		}

		return indexModel;
	}
		
	private JComboBox<String> getSubSystemCombo() {
		if (subSystemCombo == null) {
			subSystemCombo = new JComboBox<>();
//			Set<String> subSysName = Selector.SUB_SYSTEM.keySet();
//			String[] subSys = new String[subSysName.size()];
//			subSys = subSysName.toArray(subSys);
//			subSystemModel.setElements(subSys);
			subSystemCombo.setModel(subSystemModel);
			GUIUtilities.setFixedSize(subSystemCombo,comboDim);
			subSystemCombo.setRenderer(new ComboRenderer());
			subSystemCombo.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (settingFromSelector) return;
					getExecutorService().execute(new Runnable() {
						public void run() {
							getSelector().setSelectedSubSystem(subSystemModel.getSelectedItem());							
						}
					});
                }
			});
		}
		return subSystemCombo;
	}
	
	private JComboBox<String> getDeviceContextCombo() {
		if (deviceContextCombo == null) {
			deviceContextCombo = new JComboBox<>();
			deviceContextCombo.setModel(deviceContextModel);
			GUIUtilities.setFixedSize(deviceContextCombo,comboDim);
			deviceContextCombo.setRenderer(new ComboRenderer());
			deviceContextCombo.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (settingFromSelector) return;
					getExecutorService().execute(new Runnable() {
						public void run() {
							getSelector().setSelectedContext(deviceContextModel.getSelectedItem());
						}
					});
					
                }
			});
		}
		return deviceContextCombo;
	}
	
	private JComboBox<String> getDeviceServerCombo() {
		if (deviceServerCombo == null) {
			deviceServerCombo = new JComboBox<>();
			deviceServerCombo.setModel(deviceServerModel);
			GUIUtilities.setFixedSize(deviceServerCombo,comboDim);
			deviceServerCombo.setRenderer(new ComboRenderer());
			deviceServerCombo.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (settingFromSelector) return;
					getExecutorService().execute(new Runnable() {
						public void run() {
							getSelector().setSelectedServer(deviceServerModel.getSelectedItem());
						}
					});
                }
			});
		}
		return deviceServerCombo;
	}
	
	private JComboBox<String> getDeviceNameCombo() {
		if (deviceNameCombo == null) {
			deviceNameCombo = new JComboBox<>();
			deviceNameCombo.setModel(deviceNameModel);
			GUIUtilities.setFixedSize(deviceNameCombo,comboDim);
			deviceNameCombo.setRenderer(new ComboRenderer());
			deviceNameCombo.setEditable(true);
			deviceNameCombo.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (settingFromSelector) return;
					getExecutorService().execute(new Runnable(){
						public void run() {
							getSelector().setSelectedDevice(deviceNameModel.getSelectedItem());
						}
					});
                }
			});
		}
		return deviceNameCombo;
	}
	
	private JComboBox<String> getPropertyNameCombo() {
		if (propertyNameCombo == null) {
			propertyNameCombo = new JComboBox<>();
			propertyNameCombo.setModel(propertyNameModel);
			GUIUtilities.setFixedSize(propertyNameCombo,comboDim);
			propertyNameCombo.setRenderer(new ComboRenderer());
			propertyNameCombo.setEditable(true);
			propertyNameCombo.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (settingFromSelector) return;
					getExecutorService().execute(new Runnable() {
						public void run() {
							String obj = ((String)propertyNameCombo.getEditor().getItem());
							if (obj != null) {
								getSelector().setSelectedProperty(obj);
							} else if (propertyNameCombo.getSelectedIndex() < 0 || propertyNameCombo.getSelectedItem() == null) {
								getSelector().setSelectedProperty(null);
							} else {
								getSelector().setSelectedProperty(propertyNameModel.getSelectedItem());
							}
						}
					});
                }
			});
		}
		return propertyNameCombo;
	}
	
	private void update(final HistoryParameters params) {
		if (params == null || params.equals(historyParameters)) return;
		HistoryParameters old = historyParameters;
		historyParameters = params;
		getExecutorService().execute(new Runnable() {
			public void run() {
				getSelector().setSelectedProtocol("TINE");
				getSelector().setSelectedContext(params.getHistoryContext());
				getSelector().setSelectedServer(params.getHistoryServer());
				getSelector().setSelectedDevice(params.getHistoryDevice());
				getSelector().setSelectedProperty(params.getHistoryProperty());
                // due to device query precedence device has to be reselected
				getSelector().setSelectedDevice(params.getHistoryDevice());
				getIndexCombo().setSelectedItem(new Integer(params.getHistoryIndex()));
			}
		});
		
        firePropertyChange(ConnectionParametersReceiver.CONNECTION_PARAMETERS_PROPERTY,old,historyParameters);
	}

	/**
	 * Returns the selected HistoryParameters.
	 * 
     * @return the parameters
     */
    public HistoryParameters getHistoryParameters() {
    	if (!enableHistoryBox.isSelected()) return null;
    	return historyParameters;
    }

	/**
	 * Sets the HistoryParameters and applies the values to the UI.
	 * 
     * @param parameters the parameters to set
     */
    public void setHistoryParameters(HistoryParameters parameters) {
    	if (this.historyParameters != null && this.historyParameters.equals(parameters)) return;
		update(parameters);
    }
    
	public void setHistoryParameters(String context, String server, String device, String property) {
		HistoryParameters hp = new HistoryParameters(context,server,device,property,0);
		update(hp);
	}
    
	/**
	 * Sets the selected index.
	 * 
	 * @param index
	 */
	public void setSelectedIndex(int index) {
		getIndexCombo().setSelectedItem(new Integer(index));
	}
	
	/**
	 * Sets the maximum value for the index selector.
	 * 
	 * @param max new maximum value
	 */
	public void setIndexMax(int max) {
		getIndexModel().setMaxIndex(max);
	}
	
	/**
	 * Returns the selected index.
	 * 
	 * @return the index
	 */
	public int getSelectedIndex() {
		return getIndexCombo().getSelectedIndex();
	}
    
    private void setSelector(Selector selector) {
    	if (this.selector != null && this.selector.equals(selector)) return;
    	if (this.selector != null) {
	    	this.selector.removePropertyChangeListener(getSelectorPropertyListener());
	    	this.selector.removeQueryListener(getSelectorQueryListener());
	    	this.selector.removeExceptionListener(getExceptionListener());
    	}
    	Selector old = this.selector;
    	this.selector = selector;
    	if (this.selector != null) {
	    	this.selector.addPropertyChangeListener(getSelectorPropertyListener());
	    	this.selector.addQueryListener(getSelectorQueryListener());
	    	this.selector.addExceptionListener(getExceptionListener());
    	}
    	
    	getExecutorService().execute(new Runnable() {
			public void run() {
		    	TrendHistorySelectorGUI.this.selector.setSelectedProtocol(Selector.PROTOCOL_TINE);
		    	fillAndSelectValue();
		    	applySettings();
			}
    	});
    	firePropertyChange("selector", old, this.selector);
    }
    
	protected Selector getSelector() {
        return selector;
    }
	
	private ExceptionListener getExceptionListener() {
		if (exceptionListener == null) {
			exceptionListener = new ExceptionListener() {
				public void exceptionReceived(ExceptionEvent e) {
					e.getThrowable().printStackTrace();
				}
			};
		}
		return exceptionListener;
	}
	
	private PropertyChangeListener getSelectorPropertyListener() {
    	if (selectorPropertyListener == null) {
    		selectorPropertyListener = new PropertyChangeListener(){
	        	public void propertyChange(PropertyChangeEvent evt) {
	        		if (getSelector() == null) return;
	        		if (!getPropertyNameCombo().isEnabled()) return;
	        		final String name = evt.getPropertyName();	        		
            		if ("selectedContext".equals(name)) {
            			deviceContextModel.setSelectedItem(getSelector().getSelectedContext());
            		} else if ("selectedSubSystem".equals(name)) {
            			subSystemModel.setSelectedItem(getSelector().getSelectedSubSystem());
            		} else if ("selectedServer".equals(name)) {
            			deviceServerModel.setSelectedItem(getSelector().getSelectedServer());
            		} else if ("selectedDevice".equals(name)) {
            			deviceNameModel.setSelectedItem(getSelector().getSelectedDevice());
            		} else if ("selectedProperty".equals(name)) {
            			propertyNameModel.setSelectedItem(getSelector().getSelectedProperty());
            		} else if ("availableContexts".equals(name)) {
            			deviceContextModel.setElements(getSelector().getAvailableContexts());
            		} else if ("availableSubsystems".equals(name)) {
            			subSystemModel.setElements(getSelector().getAvailableSubsystems());
            		} else if ("availableServers".equals(name)) {
            			deviceServerModel.setElements(getSelector().getAvailableServers());
            		} else if ("availableDevices".equals(name)) {
            			deviceNameModel.setElements(getSelector().getAvailableDevices());
            		} else if ("availableProperties".equals(name)) {
            			propertyNameModel.setElements(getSelector().getAvailableProperties());
            		}   
            		applySettings();
	        	}
	        };
    	} 
    	return selectorPropertyListener;
    }
	
	private void fillAndSelectValue() {
		settingFromSelector = true;
		deviceContextModel.setElements(getSelector().getAvailableContexts());
		deviceContextModel.setSelectedItem(getSelector().getSelectedContext());
		subSystemModel.setElements(getSelector().getAvailableSubsystems());
		subSystemModel.setSelectedItem(getSelector().getSelectedSubSystem());
		deviceServerModel.setElements(getSelector().getAvailableServers());
		deviceServerModel.setSelectedItem(getSelector().getSelectedServer());
		deviceNameModel.setElements(getSelector().getAvailableDevices());
		deviceNameModel.setSelectedItem(getSelector().getSelectedDevice());
		propertyNameModel.setElements(getSelector().getAvailableProperties());
		propertyNameModel.setSelectedItem(getSelector().getSelectedProperty());
		settingFromSelector = false;
	}
    
    private QueryListener getSelectorQueryListener() {
    	if (selectorQueryListener == null) {
    		selectorQueryListener = new QueryListener() {
				public void queryFinished(QueryEvent e) {
					if (getSelector() == null) return;
					synchronized(selector) {
						fillAndSelectValue();
						applySettings();
						setCombosEnabled(true);
					}
				}
				public void queryStarted(QueryEvent e) {
					if (getSelector() == null) return;
					synchronized (selector) {
						setCombosEnabled(false);
					}
				    
				}
    		};
    	}
    	return selectorQueryListener;
    }
	
    private boolean combosEnabled = true; 
	private void setCombosEnabled(boolean enabled) {
		
		if (enabled && getSelector().isBusy()) {
			enabled = false;
		}
//		if (this.combosEnabled == enabled) return;
		this.combosEnabled = enabled;
		SwingUtilities.invokeLater(new Runnable() {
			
			@Override
			public void run() {
				boolean b = combosEnabled && historyEnabled;
			    getDeviceContextCombo().setEnabled(b);
			    getSubSystemCombo().setEnabled(b);
			    getDeviceServerCombo().setEnabled(b);
			    getDeviceNameCombo().setEnabled(b);
			    getPropertyNameCombo().setEnabled(b);
			    getIndexCombo().setEnabled(b);
			
			}
		});
		
	}
	
	public static void main(String[] args) {
		try {
			
			TrendHistorySelectorGUI c= new TrendHistorySelectorGUI();
			RunnerHelper.runComponent(c,300,370);
		} catch (Exception e) {
			e.printStackTrace();
		}
		
	}
	
	private ThreadPoolExecutor executorService;
	private ThreadPoolExecutor getExecutorService() {
		if (executorService == null) {
			executorService = new  ThreadPoolExecutor(1, 1, 0, TimeUnit.NANOSECONDS,
		              new LinkedBlockingQueue<Runnable>()){
				@Override
				protected void afterExecute(Runnable r, Throwable t) {
					super.afterExecute(r, t);
					if (t != null) {
						t.printStackTrace();
					}
				}
			};
			executorService.setRejectedExecutionHandler(new RejectedExecutionHandler(){
				public void rejectedExecution(Runnable r, ThreadPoolExecutor executor) {
					//ignore
				}
			});			
        }
        return executorService;
	}

}
/* __oOo__ */
