/**
 * File: ENodeCondition.java
 * Date: Sep 30, 2010
 */
package de.desy.acop.displayers.tarantula;

/**
 * Node conditions. Includes depending on the descending nodes condition
 * for the current one and reflects node condition with this enumeration.
 * 
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: ENodeCondition.java 40186 2010-11-03 14:12:06Z walla $
 */
enum ENodeCondition {
	/** Normal and ordinary node condition. Is realized with a normal green 
	 * selected-hacks. */
	ORDINARY_NODE,
	
	/** Time is exceeded and therefore no more nodes are request from TINE. */
	TIME_EXCEEDED,
	
	/** The node contains more descending nodes but the search for more 
	 * is permitted by the depth of the structure. This node is realized with a 
	 * normal green selected-hacks with a question mark for the descending node 
	 * connections like '-> ?' */
	DEPTH_EXCEEDED,
	
	/** An ordinary node condition with no children. This node is realized with
	 * a normal green selected-hack with a finish symbol like '-> -|.' */
	STOP_NODE,
	
	/** Node contains an address which is present in one of the descending nodes
	 * and therefore results in a cycle loop. This node is realized with a normal
	 * green selected-hacks with a circle to demonstrate the cyclic behavior of 
	 * this node. The symbol is like '-> O' */
	CYCLE_LOOP_NODE,
	
	/** There exists a list of servers which are excepted by appending it to
	 * the model. Nodes which have such child links are marked with this 
	 * condition. */
	IGNORE_SERVER,
	
	/** Failure on the connection occurred. */
	FAILURE_CONNECTION,
	
	/** Internal failure occurred. */
	FAILURE_INTERNAL
}
/* end of file */