package de.desy.acop.displayers.tarantula;

import java.awt.Component;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.logging.Logger;

import javax.swing.GroupLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import com.cosylab.application.AutoScalable;

@AutoScalable
final class HelpDialog extends JDialog {
	
	// --- CLASS FIELDS ----------------------------------------------------
	
	@SuppressWarnings("unused")
	private static final Logger log = Utility.getLogger(HelpDialog.class);
	
	private static final long serialVersionUID = 1127806564591171384L;
	
	// --- INSTANCE FIELDS -------------------------------------------------
	
	private JButton btnExit;

	// --- CONSTRUCTOR -----------------------------------------------------

	/**
	 * Constructor.
	 * 
	 * @param parent Parent component.
	 */
	HelpDialog(final Component parent) {
		super(null, "Tarantula Help", Dialog.ModalityType.MODELESS);
		this.setContentPane(createHeader());
		this.setResizable(true);
		this.setAlwaysOnTop(true);
		initialize();
		this.pack();
		if (parent != null) {
			@SuppressWarnings("unused")
			final Dimension screenSize = this.getToolkit().getScreenSize();
			
			final int x = Math.max(0, parent.getLocationOnScreen().x + (parent.getBounds().width - this.getBounds().width)/2);
			final int y = Math.max(0, parent.getLocationOnScreen().y + (parent.getBounds().height - this.getBounds().height)/2);
			this.setLocation(x, y);
		}
		else {
			final Dimension screenSize = this.getToolkit().getScreenSize();
			final int x = Math.max(0, (screenSize.width - this.getBounds().width)/2);
			final int y = Math.max(0, (screenSize.height - this.getBounds().height)/2);
			this.setLocation(x, y);
		}
	}

	// --- INSTANCE METHODS ------------------------------------------------
	
	private void initialize() {
		getExit().addActionListener(new ActionListener() {
			
			@Override public void actionPerformed(final ActionEvent event) {
				HelpDialog.this.setVisible(false);
			}
		});
	}

	private JComponent createHeader() {
		final JLabel lblHeader = new JLabel("<html>" +
				"<center><h1>Help about the Tarantula Icons</h1>" +
				"The following content includes help information and describtion about the used icons and their<br>" +
				"meanings.</center>" +
				"</html>");
		
		final JLabel lblVersionHeader = new JLabel("Version:");
		final JLabel lblContactHeader = new JLabel("Contact:");
		final Version version = TarantulaPanel.getVersion();
		final JLabel lblVersion;
		final JLabel lblContact;
		if (version != null) {
			lblVersion = new JLabel("" + version.major() + '.' + version.minor() + '.' + version.build() + " from " + version.releaseDate());
			lblContact = new JLabel("<html>" + version.organization() + " - " + version.developers()[0] + ", e-mail: " + version.emails()[0] + "<br>(phone " + version.phones()[0] + ", bldg " + version.building() + '/' + version.rooms()[0] + ')');
		}
		else {
			lblVersion = new JLabel("currently not accredited");
			lblContact = new JLabel("currently not accredited");
		}
		

		final JComponent context = new JScrollPane(createContent());
		
		final JPanel header = new JPanel();
		final GroupLayout layout = new GroupLayout(header);
		layout.setAutoCreateContainerGaps(true);
		layout.setAutoCreateGaps(true);
		header.setLayout(layout);

		layout.setHorizontalGroup(layout.createParallelGroup(GroupLayout.Alignment.CENTER)
				.addComponent(lblHeader)
				.addComponent(context)
				
				.addGroup(layout.createSequentialGroup()
						.addGroup(layout.createParallelGroup(GroupLayout.Alignment.CENTER)
								.addComponent(lblVersionHeader)
								.addComponent(lblContactHeader)
						)
						.addGroup(layout.createParallelGroup()
								.addComponent(lblVersion)
								.addComponent(lblContact)
						)
				)
				.addComponent(getExit())
		);
		
		layout.setVerticalGroup(layout.createSequentialGroup()
				.addComponent(lblHeader)
				.addGap(10)
				.addComponent(context)
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(lblVersionHeader)
						.addComponent(lblVersion)
				)
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(lblContactHeader)
						.addComponent(lblContact)
				)
				.addGap(10)
				.addComponent(getExit())
		);
				
		return header;		
	}
	
	private JComponent createContent() {
		
		final JLabel lblSuccessIcon = new JLabel(Utility.getStatusIcon(EStatusLink.SUCCESS, ENodeCondition.ORDINARY_NODE, true));
		final JLabel lblSuccess = new JLabel("<html>" +
				"Connection executes with success. There are more connections underneath this<br>" +
				"link.</html>");

		final JLabel lblSuccessButStopNodeIcon = new JLabel(Utility.getStatusIcon(EStatusLink.SUCCESS, ENodeCondition.STOP_NODE, true));
		final JLabel lblSuccessButStopNode = new JLabel("<html>" +
				"Connection executes with success and reached a stop node. There are no more<br>" +
				"connections underneath this one.</html>");

		final JLabel lblSuccessButDepthExceededIcon = new JLabel(Utility.getStatusIcon(EStatusLink.SUCCESS, ENodeCondition.DEPTH_EXCEEDED, true));
		final JLabel lblSuccessButDepthExceeded = new JLabel("<html>" +
				"Connection executes with success but reached the structure depth. There are more<br>" +
				"connection below this entry. You can provide a deeper depth by setting the<br>" +
				"structure depth to a new limit (can be found in the left bottom corner) or by<br>" +
				"selecting one of two possibilities in the context menu for plunging deeper into<br>" +
				" the structure (for example \"" +  TarantulaPanel.STR_NEXT_CONNECTION + "\").</html>");

		final JLabel lblSuccessButCycleLoopIcon = new JLabel(Utility.getStatusIcon(EStatusLink.SUCCESS, ENodeCondition.CYCLE_LOOP_NODE, true));
		final JLabel lblSuccessButCycleLoop = new JLabel("<html>" +
				"Connection executes with success but detects a cycle loop. One of the descending<br>" +
				"entries already includes the current TINE server address and therefore a cycle<br>" +
				"loop is completed. Currently the user has to reflect on his own which of the<br>" +
				" descending connection causes the cycle loop.</html>");

		final JLabel lblSuccessButMismatchTINEPropertyIcon = new JLabel(Utility.getStatusIcon(EStatusLink.FAILURE_ILLEGAL_PROPERTY, ENodeCondition.FAILURE_CONNECTION, true));
		final JLabel lblSuccessButMismatchTINEProperty = new JLabel("<html>" +
				"Connection executes with success but detects on the ascending connection a TINE<br>" +
				"version mismatch. This can happen if the application tries to fetch information<br>" +
				"about further connections but TINE doesn't delivers them because the server don't<br>" + 
				"provide a special property which is needed. Currently this occurs on TINE version<br>" +
				"3.x, 4.0.x., or 4.1.4</html>");

		final StringBuilder successButIgnoreServer = new StringBuilder();
		final String[] ignoreServers = TineConnection.IGNORE_SERVER.toArray(new String[] {});
		int counter = 0;
		for(int i = 0; i < ignoreServers.length; i++) {
			if (i == (ignoreServers.length - 1)) 
				successButIgnoreServer.append(ignoreServers[i]);
			else if (counter > 60) {
				successButIgnoreServer.append(ignoreServers[i]).append(",<br>");
				counter = 0;
			}
			else {
				successButIgnoreServer.append(ignoreServers[i]).append(", ");
				counter += ignoreServers[i].length() + 2;
			}
		}
		final JLabel lblSuccessButIgnoreServerIcon = new JLabel(Utility.getStatusIcon(EStatusLink.SUCCESS, ENodeCondition.IGNORE_SERVER, true));
		final JLabel lblSuccessButIgnoreServer = new JLabel("<html>" +
				"Connection executes with success but ascending servers are ignored. The<br>" +
				"following servers are ignored:<br>" +
				successButIgnoreServer + "</html>");

		final JLabel lblSuccessButTimeExceededIcon = new JLabel(Utility.getStatusIcon(EStatusLink.SUCCESS, ENodeCondition.TIME_EXCEEDED, true));
		final JLabel lblSuccessButTimeExceeded = new JLabel("<html>" +
				"Connection executes with success but time exceeded. The application begins<br>" +
				"with a refreshing time of " + RefreshConnections.MIN_LOOP_TIME + " seconds. You can also set the maximum refreshing<br>" +
				"time by selecting a different one in the left upper corner up to " + RefreshConnections.MAX_LOOP_TIME + " seconds.</html>");

		final JLabel lblFailedIcon = new JLabel(Utility.getStatusIcon(EStatusLink.FAILURE, ENodeCondition.FAILURE_CONNECTION, true));
		final JLabel lblFailed = new JLabel("<html>" +
				"Connection fails because of an error returned by TINE. The concret error message<br>" +
				"and the corresponding number is provided at the end of the entry. In most cases<br>" +
				"the TINE address is also published. Because the user effects refreshing the structure<br>" +
				"(under some circumstances this can take a while) you can explicitly test the failed<br>" +
				"connection with the context menu \"" + TarantulaPanel.STR_VERIFY_FAILED_ADDRESS + "\"</html>");

		final JLabel lblAscendingWarningIcon = new JLabel(Utility.getStatusIcon(EStatusLink.ASCENDING_WARNING, true));
		final JLabel lblAscendingWarning = new JLabel("<html>" +
				"One of the ascending entries includes a connection failure.</html>");
		
		final JLabel lblFailureIcon = new JLabel(Utility.getStatusIcon(EStatusLink.FAILURE, ENodeCondition.FAILURE_INTERNAL, true));
		final JLabel lblFailure = new JLabel("<html>" +
				"The connection fails. In most cases this error occurs when the time exceeds<br>" +
				"without requesting an answer from the server. The delay time for this periode<br>" +
				"is 1 second.</html>");

		final JLabel lblFailedConnectionIcon = new JLabel(Utility.getStatusIcon(EStatusLink.SUCCESS, ENodeCondition.FAILURE_CONNECTION, true));
		final JLabel lblFailedConnection = new JLabel("<html>" +
				"The state of the connection cannot be mirrored on the running loop. Currently<br>" +
				"this icon only occurs during the refreshing loop of all links but will disappear<br>" + 
				"when all links are prospected. So, this is only mentioned for completeness.</html>");

		// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
		
		final JPanel content = new JPanel();
		final GroupLayout layout = new GroupLayout(content);
		layout.setAutoCreateContainerGaps(true);
		layout.setAutoCreateGaps(true);
		content.setLayout(layout);

		layout.setHorizontalGroup(layout.createParallelGroup(GroupLayout.Alignment.CENTER)
				.addGroup(layout.createSequentialGroup()
						.addGroup(layout.createParallelGroup(GroupLayout.Alignment.CENTER)
								.addComponent(lblSuccessIcon)
								.addComponent(lblSuccessButStopNodeIcon)
								.addComponent(lblSuccessButDepthExceededIcon)
								.addComponent(lblSuccessButCycleLoopIcon)
								.addComponent(lblSuccessButIgnoreServerIcon)
								.addComponent(lblSuccessButMismatchTINEPropertyIcon)
								.addComponent(lblSuccessButTimeExceededIcon)
								.addComponent(lblFailedIcon)
								.addComponent(lblAscendingWarningIcon)
								.addComponent(lblFailureIcon)
								.addComponent(lblFailedConnectionIcon)
						)
						.addGroup(layout.createParallelGroup()
								.addComponent(lblSuccess)
								.addComponent(lblSuccessButStopNode)
								.addComponent(lblSuccessButDepthExceeded)
								.addComponent(lblSuccessButCycleLoop)
								.addComponent(lblSuccessButIgnoreServer)
								.addComponent(lblSuccessButMismatchTINEProperty)
								.addComponent(lblSuccessButTimeExceeded)
								.addComponent(lblFailed)
								.addComponent(lblAscendingWarning)
								.addComponent(lblFailure)
								.addComponent(lblFailedConnection)
						)
				)
		);
		
		layout.setVerticalGroup(layout.createSequentialGroup()
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(lblSuccessIcon)
						.addComponent(lblSuccess)
				)
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(lblSuccessButStopNodeIcon)
						.addComponent(lblSuccessButStopNode)
				)
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(lblSuccessButDepthExceededIcon)
						.addComponent(lblSuccessButDepthExceeded)
				)
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(lblSuccessButCycleLoopIcon)
						.addComponent(lblSuccessButCycleLoop)
				)
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(lblSuccessButIgnoreServerIcon)
						.addComponent(lblSuccessButIgnoreServer)
				)
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(lblSuccessButMismatchTINEPropertyIcon)
						.addComponent(lblSuccessButMismatchTINEProperty)
				)
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(lblSuccessButTimeExceededIcon)
						.addComponent(lblSuccessButTimeExceeded)
				)
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(lblFailedIcon)
						.addComponent(lblFailed)
				)
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(lblAscendingWarningIcon)
						.addComponent(lblAscendingWarning)
				)
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(lblFailureIcon)
						.addComponent(lblFailure)
				)
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(lblFailedConnectionIcon)
						.addComponent(lblFailedConnection)
				)
		);
		
		// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
		
		return content;
	}
	
	private JButton getExit() {
		if (this.btnExit == null) {
			this.btnExit = new JButton("Exit");
		}
		return this.btnExit;
	}
}
/* end of file */