/**
 * File: TarantulaPanel.java
 * Date: Aug 24, 2010
 */
package de.desy.acop.displayers.tarantula;

import java.awt.Color;
import java.awt.Container;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Point;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.HierarchyEvent;
import java.awt.event.HierarchyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import javax.swing.BorderFactory;
import javax.swing.GroupLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JToolTip;
import javax.swing.JTree;
import javax.swing.LayoutStyle;
import javax.swing.SpinnerModel;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.Timer;
import javax.swing.ToolTipManager;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.EventListenerList;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import com.cosylab.application.AutoScalable;

import de.desy.tine.client.TLink;
import de.desy.tine.definitions.TErrorList;
import de.desy.tine.exceptions.UnresolvedAddressException;
import de.desy.tine.queryUtils.TQuery;

/**
 * <p>The tarantula panel. Most commonly you can create a new panel by calling
 * the parameterless constructor.</p> 
 * <p><b>IS CURRENTLY NOT USED</b>: You can also specify a parent window which,
 * when it is set and the OS reflects the iconify event on the window, allows to
 * install a listener to the window. This listener activates the updating 
 * sequence for the TINE connection when the window is deiconified or, if 
 * iconfied, stops updating.</p>
 * 
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: TarantulaPanel.java 85628 2019-05-31 06:40:26Z bobnar $
 */
@Version(
		application = "Tarantula",
		
		developers  = { "Marcus Walla" }, 
		emails      = { "marcus.walla(at)desy.de" }, 
		phones      = { "+49-40-8998-2628" },
		rooms       = { "517"},
		
		group = "MCS1", building = "30",

		/* ONLY CHANGE THE FOLLOWING LINE TO UPDATE THE VERSION AND RELEASE DATE */
		major = 0, minor = 7, build = 2, releaseDate = "02/12/2010")
public final class TarantulaPanel extends JPanel {
	
	// --- ENUMERATIONS --------------------------------------------------------
	
	/**
	 * Refresh status.
	 * 
	 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
	 * @version $Id: TarantulaPanel.java 85628 2019-05-31 06:40:26Z bobnar $
	 */
	enum ERefresh { RUNNING, FINISHED }
	
	/**
	 * A filter to specify special node behavior, for example all nodes which
	 * include an error, or all nodes including all successful node and
	 * errors.
	 * 
	 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
	 * @version $Id: TarantulaPanel.java 85628 2019-05-31 06:40:26Z bobnar $
	 */
	enum EFilter {
		/** All nodes including all successful node and errors. */
		ALL_CONNECTIONS("All Connections"),
		/** Only the failed connections are shown. */
		ERROR_CONNECTIONS("Only Failed Connections");
		
		private final String title;
		
		EFilter(final String title) { this.title = title; }
		
		String getTitle() { return this.title; }
	}
	
	// --- CLASS FIELDS --------------------------------------------------------

	private static Logger log = Utility.getLogger(TarantulaPanel.class);
	private static final long serialVersionUID = 2580577295976976183L;
	
	static final String STR_REFRESH_THREAD_NAME = "Thread:" + RefreshConnections.class.getSimpleName();
	static final String STR_TINE_CONNECTION = "Network Connectivity Structure (NCS)";
	static final String STR_STRUCTURE_DEPTH = "Structure Depth";
	static final String STR_MAX_LOOP_TIME = "Max Loop Time";
	static final String STR_FILTER = "Filter";
	static final String STR_CANCEL_REFRESH = "Cancel Refreshing the Structure!";
	static final String STR_START_REFRESH = "Start Refreshing the Structure!";
	
	static final String STR_INFORMATION = "Get information...";
	static final String STR_VERIFY_FAILED_ADDRESS = "Verify the failed address...";
	static final String STR_NEXT_CONNECTION = "Fetch next connection";
	static final String STR_MORE_CONNECTIONS = "Fetch more connections of a specified level...";
	static final String STR_EXPAND_ALL_ASCENDING_CONNECTIONS = "Expand all ascending connections";
	static final String STR_COLLAPSE_ALL_ASCENDING_CONNECTIONS = "Collapse all ascending connections";
	
	// --- INSTANT FIELDS ------------------------------------------------------
	
	private boolean isResetModel = false;
	private Map<EFilter, ConnectionModel> connectionModel = new HashMap<>();
	private final ConnectionRenderer connectionRenderer;
	private final RefreshConnections refreshConnections;
	private JTree tineStructuredTree = null;
	private JProgressBar updateBusy = null;
	private JButton btnSwitchUpdate = null;
	private JSpinner spinnerDepthOfTree = null;
	private JSpinner maxLoopTime = null;
	private JComboBox<String> cbFilter = null;
	private JButton btnExpandAllNodes = null;
	private JButton btnCollapseAllNodes = null;
	private JButton btnHelp = null;
	private TreePopupMenu treePopupMenu = null;
	private JMenuItem itemGetInfo = null;
	private JMenuItem itemTestFailedAddress = null;
	private JMenuItem itemCollapseNodes = null;
	private JMenuItem itemExpandNodes = null;
	private JMenuItem itemFetchMoreNodes = null;
	private JMenuItem itemFetchNextNodes = null;

	// --- CLASS METHODS -------------------------------------------------------

//	/**
//	 * Creates a new tarantula panel.
//	 * @return Tarantula panel.
//	 */
//	public static TarantulaPanel createNewTarantula() { return new TarantulaPanel(); }

//	/**
//	 * Creates a new tarantula panel.
//	 * @param parent The parent window. This can also be {@code null} in which
//	 * 		case no window events are reflected by this instance.
//	 * @return Tarantula panel.
//	 */
//	public static TarantulaPanel createNewTarantula(final Window parent) { return new TarantulaPanel(parent); }

	/**
	 * Returns the version annotation of the tarantula feature.
	 * @return Version instance.
	 */
	public static final Version getVersion() { return TarantulaPanel.class.getAnnotation(Version.class); }
	
	// --- CONSTRUCTOR ---------------------------------------------------------

	/**
	 * <p>Constructor.</p>
	 * <p>The reflection API allows to create an instance of this class
	 * by an introspection of the parameterless constructor. For example MstApp
	 * framework does this.</p>
	 */
	public TarantulaPanel() {
		this(null);
	}
	
	/**
	 * Constructor.
	 * 
	 * @param parent The parent window. This can also be {@code null} in which
	 * 		case no window events are reflected by this instance.
	 */
	public TarantulaPanel(final Window parent) {
		this.connectionRenderer = new ConnectionRenderer();
		this.refreshConnections = new RefreshConnections(this, RefreshConnections.MIN_TREE_DEPTH, RefreshConnections.INIT_LOOP_TIME);
		setConnectionModel(EFilter.ALL_CONNECTIONS, createNewConnectionModel());
		setConnectionModel(EFilter.ERROR_CONNECTIONS, createNewConnectionModel());
		
		if (parent != null && parent.getToolkit().isFrameStateSupported(Frame.ICONIFIED))
			parent.addWindowListener(new WindowAdapter() {
				@Override public void windowIconified(final WindowEvent event) {
					log.fine("tarantula is iconified");
				}
				@Override public void windowDeiconified(final WindowEvent event) {
					log.fine("tarantula is deiconified");
				}
			});
    	this.addHierarchyListener(new HierarchyListener() {
			
			/** Currently I'm only interested in the {@link HierarchyEvent#SHOWING_CHANGED}
			 * event. */
			@Override public void hierarchyChanged(final HierarchyEvent event) {
				final long flags = event.getChangeFlags();
				
				if ((flags & HierarchyEvent.SHOWING_CHANGED) != 0) {
					log.fine("tantantula hierarchy changed: SHOWING CHANGED");
				}
			}
		});
		
		/* ATTENTION: sets the UI redrawing of the tree to be more efficient then
		 * without activating it (i.e. setting the value to false). This is a UI
		 * dependent property of a tree and hence will react for diverse OS with 
		 * different realization. So I don't know if this will also work on the 
		 * MacOS operating system! -> TEST IT UNDER MAC OS! */
		getTree().setLargeModel(true);
		
		createContentPanel();
		this.getSwitchRefreshButton().setText(TarantulaPanel.STR_START_REFRESH);
		this.addListeners();
		
		// ATTENTION: must be called at end of the constructor!
		//this.startUpdate();
		
//		new Thread(new Runnable() { 
//			@Override public void run() {
//				try { Thread.sleep(1000); } catch(InterruptedException exception) {}
//				TarantulaPanel.this.startUpdate();
//			}
//		}).start();
		
		log.fine("successfully create the tarantula panel.");
	}
	
	// --- INSTANCE METHODS ----------------------------------------------------
	
	private void createContentPanel() {
		final JScrollPane scroll = new JScrollPane(getTree());
		Utility.addBorder(TarantulaPanel.STR_TINE_CONNECTION, scroll);
		
		final JLabel headerTreeDepth = new JLabel(TarantulaPanel.STR_STRUCTURE_DEPTH + ":");
		final JLabel headerMaxLoopTime = new JLabel(TarantulaPanel.STR_MAX_LOOP_TIME + ":");
		final JLabel headerFilter = new JLabel(TarantulaPanel.STR_FILTER + ":");

		final GroupLayout layout = new GroupLayout(this);
		this.setLayout(layout);

		layout.setAutoCreateContainerGaps(true);
		layout.setAutoCreateGaps(true);
		
		layout.setHorizontalGroup(layout.createParallelGroup(GroupLayout.Alignment.CENTER)
				.addGroup(layout.createSequentialGroup()
						.addComponent(headerMaxLoopTime)
						.addComponent(this.getMaxLoopTime(), 60, 60, 60)
						.addGap(10)
						.addComponent(headerFilter)
						.addComponent(this.getConnectionFilter(), 160, 160, 160)
						.addGap(10)
						.addComponent(this.getUpdateIsBusyPanel(), 200, 200, 200)
						.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED, 0, Short.MAX_VALUE)
						.addComponent(this.getHelp())
				)
				.addComponent(scroll)
				.addGroup(layout.createSequentialGroup()
						.addComponent(headerTreeDepth)
						.addComponent(this.getDepthOfTree(), 40, 40, 40)
						.addComponent(this.getExpandAllNodes())
						.addComponent(this.getCollapseAllNodes())
						.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
						.addComponent(this.getSwitchRefreshButton(), 260, 260, 260)
				)
		);
		layout.setVerticalGroup(layout.createSequentialGroup()
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.CENTER)
						.addComponent(headerMaxLoopTime)
						.addComponent(this.getMaxLoopTime(), 20, 20, 20)
						.addComponent(headerFilter)
						.addComponent(this.getConnectionFilter(), 20, 20, 20)
						.addComponent(this.getUpdateIsBusyPanel(), 20, 20, 20)
						.addComponent(this.getHelp())
				)
				.addComponent(scroll)
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.CENTER)
						.addComponent(headerTreeDepth)
						.addComponent(this.getDepthOfTree(), 20, 20, 20)
						//.addComponent(this.getTINELinkMode(), 20, 20, 20)
						.addComponent(this.getExpandAllNodes())
						.addComponent(this.getCollapseAllNodes())
						.addComponent(this.getSwitchRefreshButton())
				)
		);
	}

	private void addListeners() {
		this.getSwitchRefreshButton().addActionListener(new ActionListener() {
			@Override public void actionPerformed(ActionEvent e) {
				TarantulaPanel.this.getConnectionFilter().setSelectedItem(EFilter.ALL_CONNECTIONS.getTitle());
				startUpdate();
			}
		});
		
		this.addTarantulaUpdateListener(new ITarantulaUpdateListener() {
			@Override public void refreshTarantula(final TarantulaEvent event) {
				if (event != null && event.getUpdateState() == TarantulaPanel.ERefresh.RUNNING)
					TarantulaPanel.this.setUIEnabled(false);
				else 
					TarantulaPanel.this.setUIEnabled(true);
			}
		});
	}
	
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    /**
     * Is called to update all connections.
     */
	synchronized void startUpdate() {
    	if (isResetConnectionModel()) {
    		setResetConnectionModel(false);
    		setConnectionModel(EFilter.ALL_CONNECTIONS, createNewConnectionModel());
    	}
		final SpinnerModel spinnerModel = getDepthOfTree().getModel();
		final int treeDepth;
		if (spinnerModel.getValue() instanceof Integer)
			 treeDepth = ((Integer) spinnerModel.getValue()).intValue();
		else {
			treeDepth = RefreshConnections.MIN_TREE_DEPTH;
			log.warning("unknown value from the tree depth spinner model: " + spinnerModel.getValue().getClass().getSimpleName() + ", Integer expected -> set to " + treeDepth);
		}
		getRefreshConnections().setStructureDepth(treeDepth);

		final ConnectionModel model = this.getConnectionModel(EFilter.ALL_CONNECTIONS);
		
		new Thread(new Runnable() {
			public void run() {
				// update the connection model
				TarantulaPanel.this.getRefreshConnections().update(model);
			}
		}, TarantulaPanel.STR_REFRESH_THREAD_NAME).start();
	}
	
	/**
	 * Finish the update sequence. Currently this does nothing.
	 */
	void finishUpdate() {
		// noop
	}

	/**
	 * Updates the tree structure. The refreshing is depends on the {@link TarantulaPanel#ERefresh}
	 * enumeration.
	 */
    final void update(final ERefresh refresh) { fireTarantulaRefresh(refresh); }
    
    /**
     * Notifies all listeners that have a registered interest for
     * notification.  The event instance is created every time you call this 
     * method.
     * @see EventListenerList
     */
    private void fireTarantulaRefresh(final ERefresh refresh) {
        // Guaranteed to return a non-null array
        final Object[] listeners = this.listenerList.getListenerList();
        // Process the listeners last to first, notifying
        // those that are interested in this event
        for (int i = listeners.length - 2; i >= 0; i -= 2) {
            if (listeners[i] == ITarantulaUpdateListener.class) {
                // Lazily create the event:
                ((ITarantulaUpdateListener)listeners[i+1]).refreshTarantula(new TarantulaEvent(this, refresh));
            }
        }
    }

    // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	private static final class CustomToolTip extends JToolTip {
		private static final long serialVersionUID = 7272727926331662661L;

		CustomToolTip(final JComponent component) {
			super();
			setComponent(component);
			setBackground(Color.LIGHT_GRAY);
		}
	}
	
    /**
     * Fetches the UI tree element. If it is not initialized it will create on
     * the fly.
     * @return Tree element.
     */
	JTree getTree() {
		if (this.tineStructuredTree == null) {
			this.tineStructuredTree = new JTree(getConnectionModel(EFilter.ALL_CONNECTIONS)) {
				private static final long serialVersionUID = 7206558923517980300L;
				@Override
				public JToolTip createToolTip() { return new CustomToolTip(this); }
			};
			this.tineStructuredTree.setCellRenderer(getCellRenderer());
			this.tineStructuredTree.addMouseListener(new MouseAdapter() {
				@Override public void mousePressed(final MouseEvent event) {
					showPopupMenu(event);
				}
				
				@Override public void mouseReleased(final MouseEvent event) {
					showPopupMenu(event);
				}
				
				private void showPopupMenu(final MouseEvent event) {
					if (event.isPopupTrigger()) {
						final TreePath path = TarantulaPanel.this.fetchCurrentTreePath(new Point(event.getX(), event.getY()));
						final ANode node = TarantulaPanel.this.fetchLastNode(path);
						if (node != null && node instanceof NodeConnection) {
							final NodeConnection nodeConnection = (NodeConnection) node;
							TarantulaPanel.this.getMenuItemConnectionInfo().setEnabled(true);
							TarantulaPanel.this.getMenuItemTestFailedAddress().setEnabled(
									nodeConnection.getStatus() != EStatusLink.SUCCESS 
									&& nodeConnection.getStatus() != EStatusLink.ASCENDING_WARNING
									&& !(nodeConnection.getStatusLinkCode() == TErrorList.illegal_property || nodeConnection.getStatusLinkCode() == TErrorList.link_blacklisted));
							
							if (nodeConnection.isLeaf()) {
								TarantulaPanel.this.getMenuItemExpandNodes().setEnabled(false);
								TarantulaPanel.this.getMenuItemCollapseNodes().setEnabled(false);

								final boolean isFetchMoreNodeEnabled = isFetchMoreNodeEnabled(
										nodeConnection.getStatus() == EStatusLink.SUCCESS 
										&& nodeConnection.getCondition() == ENodeCondition.DEPTH_EXCEEDED
										&& nodeConnection.getLevel() >= Math.max(1, RefreshConnections.MIN_TREE_DEPTH - 1)
										&& nodeConnection.getLevel() < RefreshConnections.MAX_TREE_DEPTH);
								
								TarantulaPanel.this.getMenuItemFetchMoreNodes().setEnabled(isFetchMoreNodeEnabled);
								TarantulaPanel.this.getMenuItemFetchNextNodes().setEnabled(isFetchMoreNodeEnabled);
							}
							else {
								TarantulaPanel.this.getMenuItemExpandNodes().setEnabled(!TarantulaPanel.this.getTree().isExpanded(path));
								TarantulaPanel.this.getMenuItemCollapseNodes().setEnabled(TarantulaPanel.this.getTree().isExpanded(path));

								final boolean isFetchMoreNodeEnabled = isFetchMoreNodeEnabled(
										nodeConnection.getLevel() >= Math.max(1, RefreshConnections.MIN_TREE_DEPTH - 1)
										&& nodeConnection.getLevel() < RefreshConnections.MAX_TREE_DEPTH);
								
								TarantulaPanel.this.getMenuItemFetchMoreNodes().setEnabled(isFetchMoreNodeEnabled);
								TarantulaPanel.this.getMenuItemFetchNextNodes().setEnabled(false);
							}
						}
						else {
							TarantulaPanel.this.getMenuItemConnectionInfo().setEnabled(false);
							TarantulaPanel.this.getMenuItemTestFailedAddress().setEnabled(false);
							TarantulaPanel.this.getMenuItemFetchMoreNodes().setEnabled(false);
							TarantulaPanel.this.getMenuItemFetchNextNodes().setEnabled(false);
							TarantulaPanel.this.getMenuItemExpandNodes().setEnabled(false);
							TarantulaPanel.this.getMenuItemCollapseNodes().setEnabled(false);
						}
						TarantulaPanel.this.getTreePopupMenu().setActiveLocation(new Point(event.getX(), event.getY()));
						TarantulaPanel.this.getTreePopupMenu().show(event.getComponent(), event.getX(), event.getY());
					}
					else
					{
						TarantulaPanel.this.getTreePopupMenu().setActiveLocation(null);
						TarantulaPanel.this.getMenuItemConnectionInfo().setEnabled(false);
						TarantulaPanel.this.getMenuItemTestFailedAddress().setEnabled(false);
						TarantulaPanel.this.getMenuItemFetchNextNodes().setEnabled(false);
						TarantulaPanel.this.getMenuItemFetchMoreNodes().setEnabled(false);
						TarantulaPanel.this.getMenuItemExpandNodes().setEnabled(false);
						TarantulaPanel.this.getMenuItemCollapseNodes().setEnabled(false);
					}
				}
				
				private boolean isFetchMoreNodeEnabled(final boolean isFetchMoreNodesEnabled) {
					final TreePath[] selectedPaths = TarantulaPanel.this.getTree().getSelectionPaths();
					boolean tmpIsFetchMoreNodesEnabled = isFetchMoreNodesEnabled;

					if (selectedPaths != null && selectedPaths.length >= 2)
					for(final TreePath selectedPath: selectedPaths)
						if (TarantulaPanel.this.fetchLastNode(selectedPath).getCondition() != ENodeCondition.DEPTH_EXCEEDED) {
							tmpIsFetchMoreNodesEnabled = false;
							break;
						}
					return tmpIsFetchMoreNodesEnabled;
				}
			});
			getMenuItemConnectionInfo().addActionListener(new ActionListener() {
				
				@Override public void actionPerformed(final ActionEvent event) {
					final Point location = TarantulaPanel.this.getTreePopupMenu().getActiveLocation();
					if (location != null) {
						final TreePath path = TarantulaPanel.this.fetchCurrentTreePath(location);
						if (path != null && path.getLastPathComponent() instanceof NodeConnection) {
							JOptionPane.showMessageDialog(TarantulaPanel.this, new ConnectionInfoComponent((NodeConnection) path.getLastPathComponent()), "Connection information...", JOptionPane.INFORMATION_MESSAGE);
						}
					}
				}
			});
			getMenuItemTestFailedAddress().addActionListener(new ActionListener() {
				
				@Override public void actionPerformed(final ActionEvent event) {
					final Point location = TarantulaPanel.this.getTreePopupMenu().getActiveLocation();
					if (location != null) {
						final ANode node = TarantulaPanel.this.fetchLastNode(fetchCurrentTreePath(location));
						if (node instanceof NodeConnection)
							new ConnectionScreeningDialog(Window.getWindows()[0], (NodeConnection) node).setVisible(true);
					}
				}
			});
			getMenuItemFetchNextNodes().addActionListener(new ActionListener() {
				
				@Override public void actionPerformed(ActionEvent e) {
					final Point location = TarantulaPanel.this.getTreePopupMenu().getActiveLocation();
					if (location != null) {
						final ANode node = TarantulaPanel.this.fetchLastNode(fetchCurrentTreePath(location));
						if (node instanceof NodeConnection) {
							TarantulaPanel.this.getRefreshConnections().setStructureDepth(node.getLevel() + 1);
							startFetchNodeThread(node, node.getLevel() + 1);
						}
					}
				}
			});
			getMenuItemFetchMoreNodes().addActionListener(new ActionListener() {
				
				@Override public void actionPerformed(final ActionEvent event) {
					final Point location = TarantulaPanel.this.getTreePopupMenu().getActiveLocation();
					if (location != null) {
						final ANode node = TarantulaPanel.this.fetchLastNode(fetchCurrentTreePath(location));
						if (node instanceof NodeConnection) {
							final JLabel header = new JLabel("<html><center>Please specify the structure depth of<br>the new extended connections.</center></html>");
							final int currentStructureDepth = node.getLevel();
							final int numberOfStructureDepth = Math.max(1, RefreshConnections.MAX_TREE_DEPTH - currentStructureDepth);
							final Object[] selections = new Object[numberOfStructureDepth];
							for(int i = 1; i <= selections.length; i++)
								selections[i-1] = currentStructureDepth + i;
							
							final Object result = JOptionPane.showInputDialog(
									TarantulaPanel.this, 
									header, 
									"Select the structure depth level...", 
									JOptionPane.QUESTION_MESSAGE, 
									null, 
									selections , selections[0]);
							if (result != null && result instanceof Integer) {
								TarantulaPanel.this.getRefreshConnections().setStructureDepth(((Integer) result).intValue());
								startFetchNodeThread(node, ((Integer) result).intValue());
							}
						}
					}
				}
			});
			getMenuItemExpandNodes().addActionListener(new ActionListener() {
				
				@Override public void actionPerformed(final ActionEvent event) {
					final Point location = TarantulaPanel.this.getTreePopupMenu().getActiveLocation();
					if (location != null) {
						TarantulaPanel.this.expandAllAscendingNodes(TarantulaPanel.this.fetchCurrentTreePath(location));
					}
				}
			});
			getMenuItemCollapseNodes().addActionListener(new ActionListener() {
				
				@Override public void actionPerformed(final ActionEvent event) {
					final Point location = TarantulaPanel.this.getTreePopupMenu().getActiveLocation();
					if (location != null) {
						TarantulaPanel.this.collapseAllAscendingNodes(TarantulaPanel.this.fetchCurrentTreePath(location));
					}
				}
			});
			
			// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
			
			// registers the component directly to the shared tool tip manager.
			ToolTipManager.sharedInstance().registerComponent(this.tineStructuredTree);
		}
		
		return this.tineStructuredTree;
	}

	/**
	 * Starts refreshing on the current mouse-over node or, if they are more then
	 * one selected brunches, of the selected tree paths with the invoked current
	 * structure depth.
	 * 
	 * @param node Is the selected node.
	 * @param currentStructureDepth The current selected structure depth.
	 */
	private void startFetchNodeThread(final ANode node, final int currentStructureDepth) {
		final ConnectionModel model = this.getConnectionModel(EFilter.ALL_CONNECTIONS);
		final TreePath[] selectedPaths = this.getTree().getSelectionPaths();
		new Thread(new Runnable() {
			public void run() {
				if (selectedPaths != null && selectedPaths.length >= 2) {
					for(final TreePath path: selectedPaths) {
						final ANode node = TarantulaPanel.this.fetchLastNode(path);
						if (node != null && node instanceof NodeConnection)
							TarantulaPanel.this.getRefreshConnections().update(
									model, (NodeConnection) node, currentStructureDepth);
					}
				}
				else
					TarantulaPanel.this.getRefreshConnections().update(model, (NodeConnection) node, currentStructureDepth);
			}
		}, TarantulaPanel.STR_REFRESH_THREAD_NAME).start();
	}

	/**
	 * Expands all ascending nodes below the tree path.
	 * @param path Tree path of the current node.
	 */
	private void expandAllAscendingNodes(final TreePath path) {
		this.getTree().expandPath(path);
		final ANode expandNode = this.fetchLastNode(path);
		
		if (expandNode == null) {
			return;
		}
		else {
			for(final NodeConnection node: expandNode.getChildNodes()) {
				if (!node.isLeaf()) {
					expandAllAscendingNodes(new TreePath(node.getPath()));
				}
			}
		}
	}

	/**
	 * Collapse all ascending nodes below the tree path. 
	 * @param path Tree path of the current node.
	 */
	private void collapseAllAscendingNodes(final TreePath path) {
		final ANode expandNode = TarantulaPanel.this.fetchLastNode(path);
		
		if (expandNode == null) {
			return;
		}
		else {
			for(final NodeConnection node: expandNode.getChildNodes()) {
				collapseAllAscendingNodes(new TreePath(node.getPath()));
			}
		}
		this.getTree().collapsePath(path);
	}

	/**
	 * Fetch the current tree path of the location of the container UI, which is
	 * mostly the UI tree component.
	 * @param location Location.
	 * @return Returns the found tree path or null, depending if the location
	 * 		has a node below it.
	 */
	private TreePath fetchCurrentTreePath(final Point location) {
		return TarantulaPanel.this.getTree().getPathForLocation(location.x, location.y);
	}
	
	/**
	 * Fetches the last node of the tree path and transforms it, when necessary
	 * to a {@link ANode}.
	 * 
	 * @param path Tree path.
	 * @return The last {@link ANode} or {@code null} if the last element is not
	 * 		an instance of {@link ANode}. 
	 */
	private ANode fetchLastNode(final TreePath path) {
		if (path != null && path.getLastPathComponent() instanceof ANode)
			return (ANode) path.getLastPathComponent();
		else
			return null;
	}

	/**
	 * Returns the tree pop-up menu.
	 * @return Pop-up Menu.
	 */
	private TreePopupMenu getTreePopupMenu() {
		if (this.treePopupMenu == null) {
			this.treePopupMenu  = new TreePopupMenu();
			this.treePopupMenu.add(getMenuItemConnectionInfo());
			this.treePopupMenu.add(getMenuItemTestFailedAddress());
			this.treePopupMenu.add(getMenuItemFetchNextNodes());
			this.treePopupMenu.add(getMenuItemFetchMoreNodes());
			this.treePopupMenu.addSeparator();
			this.treePopupMenu.add(getMenuItemExpandNodes());
			this.treePopupMenu.add(getMenuItemCollapseNodes());
		}
		
		return this.treePopupMenu;
	}

	/**
     * Fetches the connection information menu item. If it is not initialized
     * this method will create on the fly.
	 * 
	 * @return Connection information menu item.
	 */
	private JMenuItem getMenuItemConnectionInfo() {
		if (this.itemGetInfo == null) {
			this.itemGetInfo = new JMenuItem(STR_INFORMATION);
		}
		return this.itemGetInfo ;
	}
	
	/**
     * Fetches the "test failed address" menu item. If it is not initialized
     * this method will create on the fly.
	 * 
	 * @return "Test failed address" menu item.
	 */
	private JMenuItem getMenuItemTestFailedAddress() {
		if (this.itemTestFailedAddress  == null) {
			this.itemTestFailedAddress = new JMenuItem(STR_VERIFY_FAILED_ADDRESS);
		}
		return this.itemTestFailedAddress;
	}
	
	/**
     * "Fetches the next node" menu item. If it is not initialized
     * this method will create on the fly.
	 * 
	 * @return "Fetches the next node" menu item.
	 */
	private JMenuItem getMenuItemFetchNextNodes() {
		if (this.itemFetchNextNodes  == null) {
			this.itemFetchNextNodes = new JMenuItem(STR_NEXT_CONNECTION);
		}
		return this.itemFetchNextNodes;
	}

	/**
     * "Fetches more nodes" menu item. If it is not initialized
     * this method will create on the fly.
	 * 
	 * @return "Fetches more nodes" menu item.
	 */
	private JMenuItem getMenuItemFetchMoreNodes() {
		if (this.itemFetchMoreNodes  == null) {
			this.itemFetchMoreNodes = new JMenuItem(STR_MORE_CONNECTIONS);
		}
		return this.itemFetchMoreNodes;
	}

	/**
     * "Expand nodes" menu item. If it is not initialized
     * this method will create on the fly.
	 * 
	 * @return "Expand nodes" menu item.
	 */
	private JMenuItem getMenuItemExpandNodes() {
		if (this.itemExpandNodes == null) {
			this.itemExpandNodes = new JMenuItem(STR_EXPAND_ALL_ASCENDING_CONNECTIONS); 
		}
		return this.itemExpandNodes;
	}
	
	/**
     * "Collapse nodes" menu item. If it is not initialized
     * this method will create on the fly.
	 * 
	 * @return "Collapse nodes" menu item.
	 */
	private JMenuItem getMenuItemCollapseNodes() {
		if (this.itemCollapseNodes == null) {
			this.itemCollapseNodes = new JMenuItem(STR_COLLAPSE_ALL_ASCENDING_CONNECTIONS); 
		}
		return this.itemCollapseNodes;
	}
	
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	
	/**
	 * Gets the tree model depending on the used filter.
	 * @param filter Filter.
	 * @return Connection model.
	 */
	ConnectionModel getConnectionModel(final EFilter filter) { return this.connectionModel.get(filter); }
    /**
     * Sets the tree model depending on the used filter.
     * @param filter Filter
     * @param connectionModel Tree model
     */
    void setConnectionModel(final EFilter filter, final ConnectionModel connectionModel) { this.connectionModel.put(filter, connectionModel); }
	/**
	 * Fetches the tree cell render.
	 * @return Tree cell render.
	 */
    ConnectionRenderer getCellRenderer() { return this.connectionRenderer; }
	/**
	 * Returns the instance for refreshing the TINE connections.
	 * @return Refresh connection instance.
	 */
    RefreshConnections getRefreshConnections() { return this.refreshConnections; }

	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	private JProgressBar getUpdateIsBusyPanel() {
		if (this.updateBusy == null) {
			this.updateBusy = new JProgressBar();
		}
		return this.updateBusy;
	}
	
	private JButton getSwitchRefreshButton() {
		if (this.btnSwitchUpdate == null) {
			this.btnSwitchUpdate = new JButton(TarantulaPanel.STR_CANCEL_REFRESH);
		}
		return this.btnSwitchUpdate;
	}

	/**
	 * <p>Fetches the depth of the tree as a spinner UI element.</p> 
	 * <p><b>ATTENTION</b>: don't need to create a change listener on the 
	 * spinner because the current tree depth is set before updating the hole
	 * structure! See {@link RefreshConnections#update(ConnectionModel)} or 
	 * {@link RefreshConnections#update(ConnectionModel, NodeConnection, int)}
	 * for more information.</p>
	 * @return Returns the UI component of the tree depth spinner.
	 */
	private JSpinner getDepthOfTree() {
		if (this.spinnerDepthOfTree == null) {
			final SpinnerNumberModel model = new SpinnerNumberModel(
					Integer.valueOf(RefreshConnections.MIN_TREE_DEPTH), 
					Integer.valueOf(RefreshConnections.MIN_TREE_DEPTH), 
					Integer.valueOf(RefreshConnections.MAX_TREE_DEPTH), 
					Integer.valueOf(1));
			this.spinnerDepthOfTree = new JSpinner(model);
 			final JSpinner.NumberEditor spinnerEditorMin =
 				new JSpinner.NumberEditor(this.spinnerDepthOfTree, "##0");
 			this.spinnerDepthOfTree.setEditor(spinnerEditorMin);
		}
		return this.spinnerDepthOfTree;
	}
	
	private JSpinner getMaxLoopTime() {
		if (this.maxLoopTime == null) {
			final SpinnerNumberModel model = new SpinnerNumberModel(
					Integer.valueOf(RefreshConnections.INIT_LOOP_TIME), 
					Integer.valueOf(RefreshConnections.MIN_LOOP_TIME), 
					Integer.valueOf(RefreshConnections.MAX_LOOP_TIME), 
					Integer.valueOf(1));
			this.maxLoopTime = new JSpinner(model);
 			final JSpinner.NumberEditor editor = new JSpinner.NumberEditor(this.maxLoopTime, "###0 s");
 			this.maxLoopTime.setEditor(editor);
 			model.addChangeListener(new ChangeListener() {
				@Override public void stateChanged(final ChangeEvent event) {
					if (event.getSource() instanceof SpinnerModel) {
						final SpinnerModel model = ((SpinnerModel) event.getSource());
	 					final int maxLoopTimeValue;
	 					if (model.getValue() instanceof Integer)
	 						maxLoopTimeValue = ((Integer) model.getValue()).intValue();
	 					else {
	 						maxLoopTimeValue = RefreshConnections.MIN_LOOP_TIME;
	 						log.warning("unknown value from the max loop time spinner model: " + model.getValue().getClass().getSimpleName() + ", Integer expected -> set to " + maxLoopTimeValue);
	 					}
	 					TarantulaPanel.this.getRefreshConnections().setMaxLoopTime(maxLoopTimeValue);					
					}
				}
			});
		}
		return this.maxLoopTime;
	}
	
	private JButton getExpandAllNodes() {
		if (this.btnExpandAllNodes == null) {
			this.btnExpandAllNodes = new JButton("Expand All");
			this.btnExpandAllNodes.addActionListener(new ActionListener() {

				@Override public void actionPerformed(ActionEvent e) {
					try {
						int row = 0;
						while (row < TarantulaPanel.this.getTree().getRowCount()) {
							TarantulaPanel.this.getTree().expandRow(row);
							row++;
					    }
					} catch(ArrayIndexOutOfBoundsException exception) {
						TarantulaPanel.log.warning("Array in the tree model is invalid -> reseting the connection model to new one!");
						TarantulaPanel.this.setResetConnectionModel(true);
					}
				}
			});
		}
		return this.btnExpandAllNodes;
	}

	private JButton getCollapseAllNodes() {
		if (this.btnCollapseAllNodes == null) {
			this.btnCollapseAllNodes = new JButton("Collapse All");
			this.btnCollapseAllNodes.addActionListener(new ActionListener() {

				@Override public void actionPerformed(ActionEvent e) {
					try {
						int row = TarantulaPanel.this.getTree().getRowCount();
						while (row >= 0) {
							if (TarantulaPanel.this.getTree().getPathForRow(row) != null 
									&& TarantulaPanel.this.getTree().getPathForRow(row).getPath().length >= 2)
								TarantulaPanel.this.getTree().collapseRow(row);
							row--;
					    }
					} catch(ArrayIndexOutOfBoundsException exception) {
						log.warning("Array in the tree model is invalid -> reseting the connection model to new one!");
						TarantulaPanel.this.setResetConnectionModel(true);
					}
				}
			});
		}
		return this.btnCollapseAllNodes;
	}
	
	private JButton getHelp() {
		if (this.btnHelp == null) {
			this.btnHelp = new JButton("Help");
			this.btnHelp.addActionListener(new ActionListener() {
				
				@Override public void actionPerformed(final ActionEvent event) {
					new HelpDialog(TarantulaPanel.this).setVisible(true);
				}
			});
		}
		return this.btnHelp;
	}
	
	private JComboBox<String> getConnectionFilter() {
		if (this.cbFilter == null) {
			final Map<Integer, EFilter> mapMode = new HashMap<>();
			mapMode.put(0, EFilter.ALL_CONNECTIONS);
			mapMode.put(1, EFilter.ERROR_CONNECTIONS);
			
			this.cbFilter = new JComboBox<>(new String[] {
					mapMode.get(0).getTitle(), 
					mapMode.get(1).getTitle()});
			this.cbFilter.setSelectedItem(mapMode.get(0).getTitle());
			
			// only add a new action listener to the JComboBox after 
			// initializing all selectable elements
			this.cbFilter.addActionListener(new ActionListener() {
				@Override public void actionPerformed(final ActionEvent event) {
					if (event.getSource() instanceof JComboBox) {
						final int index = ((JComboBox<?>) event.getSource()).getSelectedIndex();
						
						if (mapMode.containsKey(index)) {
							switch(mapMode.get(index)) {
							case ERROR_CONNECTIONS: {
								TarantulaPanel.this.setUIEnabled(false);
								new Thread(new Runnable() {
									@Override public void run() {
										final ConnectionModel model = 
											TarantulaPanel.this.getConnectionModel(EFilter.ERROR_CONNECTIONS);
										model.getRoot().removeAllChildren();
										model.getRoot().setStatus(EStatusRoot.ROOT_REFRESH_WAIT_FOR_REFRESH);
										model.nodeStructureChanged(model.getRoot());
										final List<NodeConnection> nodes = 
											TarantulaPanel.this.getConnectionModel(EFilter.ALL_CONNECTIONS).getRoot().getChildNodes();
										
										if (changeFilterToError(nodes, model, 0) != 0)
											model.getRoot().refreshTree(EStatusRoot.ROOT_ONLY_FAILED_CONNECTIONS);
										else
											model.getRoot().refreshTree(EStatusRoot.ROOT_NON_CONNECTION_FAILED);
										
										SwingUtilities.invokeLater(new Runnable() { 
											@Override public void run() {
												TarantulaPanel.this.getTree().setModel(model);
												model.nodeStructureChanged(model.getRoot());
												TarantulaPanel.this.setUIEnabled(true);
											}});
									}
									private int changeFilterToError(
											final List<NodeConnection> nodes, 
											final ConnectionModel model, 
											final int failedConnections) {
										int tmpFailedConnections = 0;
										for(final NodeConnection node: nodes) {
											if (!node.isLeaf())
												tmpFailedConnections += changeFilterToError(node.getChildNodes(), model, failedConnections);
											else if (node.getStatus() == EStatusLink.FAILURE && node.getParent() != null) {
												tmpFailedConnections += 1;
												insertNewNode(model, node);
											}
										}
										return failedConnections + tmpFailedConnections;
									}
									private void insertNewNode(final ConnectionModel model, final ANode newNode) {
										ANode root = model.getRoot();
										for(final TreeNode node: newNode.getPath()) {
											if (node instanceof NodeConnection) {
												if (model.findNode(((NodeConnection) node).getFullAddress(), root) == null) {
													final NodeConnection copyNode = new NodeConnection((NodeConnection) node);
													model.insertNode(copyNode, root);
													if (!TineConnection.isNodeFailure((NodeConnection) node))
														copyNode.setAscendingWarning();
												}
												root = model.findNode(((NodeConnection) node).getFullAddress(), root);
											}
										}
									}
								}).start();
								break;
							}
							default: {
								final ConnectionModel model =
									TarantulaPanel.this.getConnectionModel(EFilter.ALL_CONNECTIONS);
								TarantulaPanel.this.getTree().setModel(model);
								model.nodeStructureChanged(model.getRoot());
								break;
							}
							}
						}
					}
				}
			});
		}
		return this.cbFilter;
	}
	
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    /**
     * Adds a {@link ITarantulaUpdateListener} to the tarantula timer panel.
     * @param listener the listener to be added
     */
    void addTarantulaUpdateListener(ITarantulaUpdateListener listener) { this.listenerList.add(ITarantulaUpdateListener.class, listener); }
    
    /**
     * Removes a {@link ITarantulaUpdateListener} from the tarantula time panel.
     * @param listener the listener to be removed
     */
    void removeTarantulaUpdateListener(final ITarantulaUpdateListener listener) { this.listenerList.remove(ITarantulaUpdateListener.class, listener); }

    /**
     * Gets the {@link ITarantulaUpdateListener}.
     * @return Gets the {@link ITarantulaUpdateListener}.
     */
    ITarantulaUpdateListener[] getRefreshListeners() { return this.listenerList.getListeners(ITarantulaUpdateListener.class); }
    
    @SuppressWarnings("unused")
	private Collection<TreePath> fetchExpandedTreeBrunches() {
    	final Collection<TreePath> expandedBrunches = new ArrayList<TreePath>();
    	final JTree tree = getTree();
    	final Enumeration<TreePath> expanded = getTree().getExpandedDescendants(new TreePath(getTree().getModel().getRoot()));
    	
    	if (expanded != null) {
	    	while(expanded.hasMoreElements()) {
	    		final TreePath path = expanded.nextElement();
	    		if (tree.isExpanded(path))
	    			expandedBrunches.add(path);
	    	}
    	}
    	
    	return Collections.unmodifiableCollection(expandedBrunches);
    }

    /**
     * Implies by {@code true} that a new tree model should be created before 
     * performing the refresh. Is invoked in the {@link #startUpdate()} 
     * sequence.
     * @param isResetModel Boolean value. 
     */
    void setResetConnectionModel(final boolean isResetModel) { this.isResetModel = isResetModel; }
    /**
     * Gets the boolean value for creating a new model. For more information see
     * at {@link TarantulaPanel#setResetConnectionModel(boolean)}.
     * @return Boolean value.
     */
    private boolean isResetConnectionModel() { return this.isResetModel; }
    /**
     * Returns a new create tree model.
     * @return Tree model.
     */
    ConnectionModel createNewConnectionModel() {
   		return new ConnectionModel(EStatusRoot.ROOT_REFRESH_PRESS_BUTTON_TO_REFRESH);
    }
    
    // --- STATIC CLASS DEFINITION ---------------------------------------------

	/**
	 * Tree pop-up menu.
	 * 
	 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
	 * @version $Id: TarantulaPanel.java 85628 2019-05-31 06:40:26Z bobnar $
	 */
	private static final class TreePopupMenu extends JPopupMenu {
		private static final long serialVersionUID = 7276564626053098640L;
		
		private Point location = null;
		
		public TreePopupMenu() {
			// TODO Auto-generated constructor stub
		}
		void setActiveLocation(final Point location) { this.location = location; }
		Point getActiveLocation() { return this.location; }
	}

	/**
	 * Connection information panel. Informs the user about the connection issues,
	 * for example the FEC name, who is responsible, etc.
	 * 
	 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
	 * @version $Id: TarantulaPanel.java 85628 2019-05-31 06:40:26Z bobnar $
	 */
	private static final class ConnectionInfoComponent extends JComponent {
		
		private static final long serialVersionUID = 2239664724944331827L;

		ConnectionInfoComponent(final NodeConnection node) {
			initialize(node);
		}
		
		private void initialize(final NodeConnection node) {
			final JLabel header = new JLabel("Information about server address /" + node.getContext() + '/' + node.getServer());
			header.setHorizontalAlignment(SwingConstants.CENTER);
			final JLabel info = new JLabel(Utility.getInfo(node));
			info.setHorizontalAlignment(SwingConstants.CENTER);
			
			final GroupLayout layout = new GroupLayout(this);
			layout.setAutoCreateContainerGaps(true);
			layout.setAutoCreateGaps(true);
			
			layout.setHorizontalGroup(layout.createParallelGroup(GroupLayout.Alignment.CENTER)
					.addComponent(header)
					.addGap(10)
					.addComponent(info)
			);
			
			
			layout.setVerticalGroup(layout.createSequentialGroup()
					.addComponent(header)
					.addComponent(info)
			);
			
			this.setLayout(layout);
		}
	}
	
	@AutoScalable
	private static final class ConnectionScreeningDialog extends JDialog {

		private static final long serialVersionUID = -8991334780819800471L;
		@SuppressWarnings("unused")
		private static final Logger log = Utility.getLogger(ConnectionScreeningDialog.class);

		private JButton btnExit;
		private JLabel lblConnectionStateServer;
		private JLabel lblConnectionStateProperty;
		
		ConnectionScreeningDialog(final Window parent, final NodeConnection node) {
			super(null, "Connection Screening...", Dialog.ModalityType.TOOLKIT_MODAL);
			this.setContentPane(createContent(node));
			this.setResizable(false);
			this.setAlwaysOnTop(true);
			initialize(node);
			this.pack();
			if (parent != null)
				this.setLocation(
						parent.getLocation().x + (parent.getBounds().width - this.getBounds().width)/2,
						parent.getLocation().y + (parent.getBounds().height - this.getBounds().height)/2);
			else {
				final Dimension screenSize = this.getToolkit().getScreenSize();
				this.setLocation(
						(screenSize.width - this.getBounds().width)/2,
						(screenSize.height - this.getBounds().height)/2);
			}
		}
		
		private void initialize(final NodeConnection node) {
			final ConnectionScreeningTask refreshTask = new ConnectionScreeningTask(node);
			final java.util.Timer refreshLink = new java.util.Timer("Thread:ConnectionScreeningTimer");
			refreshLink.schedule(refreshTask, 0, 1000);
			final Timer refreshLinkUI = new Timer(0, new ActionListener() {
				
				@Override public void actionPerformed(final ActionEvent event) {
					final int resultServer = refreshTask.getResultServer();
					try {
						switch(resultServer) {
						case TErrorList.success:
							ConnectionScreeningDialog.this.getConnectionStateServer().setText("SUCCESS");
							ConnectionScreeningDialog.this.getConnectionStateServer().setBackground(Color.GREEN);
							ConnectionScreeningDialog.this.getConnectionStateServer().setForeground(Color.BLACK);
							
							final int resultProperty = refreshTask.getResultProperty();
							try {
								switch(resultProperty) {
								case TErrorList.success:
									ConnectionScreeningDialog.this.getConnectionStateProperty().setText("SUCCESS");
									ConnectionScreeningDialog.this.getConnectionStateProperty().setBackground(Color.GREEN);
									ConnectionScreeningDialog.this.getConnectionStateProperty().setForeground(Color.BLACK);
									break;
								case 512: // currently return after initializing this component
									ConnectionScreeningDialog.this.getConnectionStateProperty().setText("Please wait while refreshing the connection!");
									ConnectionScreeningDialog.this.getConnectionStateProperty().setBackground(Color.YELLOW);
									ConnectionScreeningDialog.this.getConnectionStateProperty().setForeground(Color.BLACK);
									break;
								default:
									ConnectionScreeningDialog.this.getConnectionStateProperty().setText("FAILURE: " + TErrorList.getErrorString(resultProperty));
									ConnectionScreeningDialog.this.getConnectionStateProperty().setBackground(Color.RED);
									ConnectionScreeningDialog.this.getConnectionStateProperty().setForeground(Color.WHITE);
									break;
								}
							}
							catch(UnresolvedAddressException exception) {
								ConnectionScreeningDialog.this.getConnectionStateProperty().setText("UNRESOLVED ADDRESS");
								ConnectionScreeningDialog.this.getConnectionStateProperty().setBackground(Color.RED);
								ConnectionScreeningDialog.this.getConnectionStateProperty().setForeground(Color.WHITE);
							}
							catch(RuntimeException exception) {
								ConnectionScreeningDialog.this.getConnectionStateProperty().setText("INVALID EXCEPTION");
								ConnectionScreeningDialog.this.getConnectionStateProperty().setBackground(Color.RED);
								ConnectionScreeningDialog.this.getConnectionStateProperty().setForeground(Color.WHITE);
							}
							
							break;
						case 512: // currently return after initializing this component
							ConnectionScreeningDialog.this.getConnectionStateServer().setText("Please wait while refreshing the connection!");
							ConnectionScreeningDialog.this.getConnectionStateServer().setBackground(Color.YELLOW);
							ConnectionScreeningDialog.this.getConnectionStateServer().setForeground(Color.BLACK);
							break;
						default:
							ConnectionScreeningDialog.this.getConnectionStateServer().setText("FAILURE: " + TErrorList.getErrorString(resultServer));
							ConnectionScreeningDialog.this.getConnectionStateServer().setBackground(Color.RED);
							ConnectionScreeningDialog.this.getConnectionStateServer().setForeground(Color.WHITE);
							break;
						}
					}
					catch(UnresolvedAddressException exception) {
						ConnectionScreeningDialog.this.getConnectionStateServer().setText("UNRESOLVED ADDRESS");
						ConnectionScreeningDialog.this.getConnectionStateServer().setBackground(Color.RED);
						ConnectionScreeningDialog.this.getConnectionStateServer().setForeground(Color.WHITE);
					}
					catch(RuntimeException exception) {
						ConnectionScreeningDialog.this.getConnectionStateServer().setText("INVALID EXCEPTION");
						ConnectionScreeningDialog.this.getConnectionStateServer().setBackground(Color.RED);
						ConnectionScreeningDialog.this.getConnectionStateServer().setForeground(Color.WHITE);
					}
				}
			});
			getExit().addActionListener(new ActionListener() {
				
				@Override public void actionPerformed(final ActionEvent event) {
					refreshLinkUI.stop();
					refreshLink.cancel();
					ConnectionScreeningDialog.this.setVisible(false);
					ConnectionScreeningDialog.this.dispose();
				}
			});
			refreshLinkUI.setDelay(1000);
			refreshLinkUI.setInitialDelay(0);
			refreshLinkUI.start();
		}
		
		private Container createContent(final NodeConnection node) {
			final JLabel lblHeader = new JLabel("<html><center>The current sample address is:<br><br>" + node.getFullAddress() + " (FEC name: " +  node.getInfo().getFecName() + ")<br><br>" +"(Hostname: " + node.getInfo().getHostName() + ", location: " + node.getInfo().getLocation() + ")<br></center></html>");
			final JLabel lblStateServerHeader = new JLabel("Status (Server):");
			final JLabel lblStatePropertyHeader = new JLabel("Status (Property):");
			
			// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

			final JPanel pnlContent = new JPanel();
			
			final GroupLayout layout = new GroupLayout(pnlContent);
			layout.setAutoCreateContainerGaps(true);
			layout.setAutoCreateGaps(true);
			
			layout.setHorizontalGroup(layout.createParallelGroup(GroupLayout.Alignment.CENTER)
					.addComponent(lblHeader)
					.addGroup(layout.createSequentialGroup()
							.addGroup(layout.createParallelGroup()
									.addComponent(lblStateServerHeader)
									.addComponent(lblStatePropertyHeader)
							)
							.addGroup(layout.createParallelGroup()
									.addComponent(getConnectionStateServer(), 200, 200, 300)
									.addComponent(getConnectionStateProperty(), 200, 200, 300)
							)
					)
					.addComponent(getExit())
			);
			layout.setVerticalGroup(layout.createSequentialGroup()
					.addComponent(lblHeader)
					.addGroup(layout.createParallelGroup(GroupLayout.Alignment.CENTER)
							.addGroup(layout.createSequentialGroup()
									.addComponent(lblStateServerHeader)
									.addComponent(lblStatePropertyHeader)
							)
							.addGroup(layout.createSequentialGroup()
									.addComponent(getConnectionStateServer(), 20, 20, 20)
									.addComponent(getConnectionStateProperty(), 20, 20, 20)
							)
					)
					.addGap(20)
					.addComponent(getExit())
			);

			pnlContent.setLayout(layout);
			
			return pnlContent;
		}
		
		private JButton getExit() {
			if (this.btnExit == null) {
				this.btnExit = new JButton("Exit");
			}
			return this.btnExit;
		}
		
		JLabel getConnectionStateServer() {
			if (this.lblConnectionStateServer == null) {
				this.lblConnectionStateServer = new JLabel();
				this.lblConnectionStateServer.setOpaque(true);
				this.lblConnectionStateServer.setHorizontalAlignment(SwingConstants.CENTER);
				this.lblConnectionStateServer.setVerticalAlignment(SwingConstants.CENTER);
				this.lblConnectionStateServer.setBorder(BorderFactory.createLineBorder(Color.BLACK));
			}
			return this.lblConnectionStateServer;
		}
		
		JLabel getConnectionStateProperty() {
			if (this.lblConnectionStateProperty == null) {
				this.lblConnectionStateProperty = new JLabel();
				this.lblConnectionStateProperty.setOpaque(true);
				this.lblConnectionStateProperty.setHorizontalAlignment(SwingConstants.CENTER);
				this.lblConnectionStateProperty.setVerticalAlignment(SwingConstants.CENTER);
				this.lblConnectionStateProperty.setBorder(BorderFactory.createLineBorder(Color.BLACK));
			}
			return this.lblConnectionStateProperty;
		}
	}
	
	private static class ConnectionScreeningTask extends java.util.TimerTask {
		private int resultServer = 512;
		private int resultProperty = 512;
		private final NodeConnection node;
		
		ConnectionScreeningTask(final NodeConnection node) {
			this.node = node;
		}
		
		@Override public void run() {
			this.resultServer = TQuery.pingServer(this.node.getContext(), this.node.getServer(), 1000);
			this.resultProperty = new TLink(this.node.getFullAddress()).execute();
		}
		
		int getResultServer() { return this.resultServer; }
		int getResultProperty() { return this.resultProperty; }
	}

	void setUIEnabled(final boolean isEnabled) {
		// invokes disabling/enabling the whole tree structure - IS CURRENTLY NOT USED
		//this.getTree().setEnabled(isEnabled);
		
		this.getExpandAllNodes().setEnabled(isEnabled);
		this.getCollapseAllNodes().setEnabled(isEnabled);
		this.getDepthOfTree().setEnabled(isEnabled);
		this.getMaxLoopTime().setEnabled(isEnabled);
		this.getConnectionFilter().setEnabled(isEnabled);
		this.getSwitchRefreshButton().setEnabled(isEnabled);
		this.getUpdateIsBusyPanel().setIndeterminate(!isEnabled);
	}
}
/* end of file */