/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.displayers.tools;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.DefaultListCellRenderer;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import com.cosylab.application.AutoScalable;
import com.cosylab.gui.displayers.MultipleDisplayer;
import com.cosylab.gui.infopanel.ConverterPanel;
import com.cosylab.gui.infopanel.PropertiesPanel;
import com.cosylab.gui.util.GUIUtilities;

import de.desy.acop.displayers.info.AcopDebugPanel;
import de.desy.acop.displayers.info.CharacteristicsPanel;
import de.desy.acop.displayers.info.ConnectionPanel;
import de.desy.acop.transport.ConnectionFailed;
import de.desy.acop.transport.adapters.AdapterFactoryService;

/**
 * <code>AcopMultipleDisplayerInfoDialog</code> presents information
 * about the <code>AcopMultiplDisplayer</code>. All information 
 * in this window is read only and designated for debugging. This dialog
 * will show connection information and bean properties.
 * 
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 *
 */
@AutoScalable
public class AcopMultipleDisplayerInfoDialog extends JDialog {
	
	private static final long serialVersionUID = 513076582058739594L;
		
	private JPanel contentPanel;
	private JTabbedPane tabPane;
	private ConverterPanel converterPanel;
	private PropertiesPanel propertiesPanel;
	private AcopDebugPanel debugPanel;
	private CharacteristicsPanel characteristicsPanel;
	private ConnectionPanel connectionPanel;
	private MultipleAcopDisplayer<?>displayer;
	private JButton closeButton;
	private JComboBox<AcopDisplayerParameters> parametersComboBox;
	private AcopDisplayerParameters selectedParameters;
	private JLabel connectionParametersLabel;
	private JPanel selectorPanel;
	private PropertyChangeListener propertyListener;
	
	private JTabbedPane connectionTab;
	private JPanel connectionContentPanel;
	
	/**
	 * Constructs a new AcopMultipleDisplayerInfoDialog.
	 *
	 */
	public AcopMultipleDisplayerInfoDialog() {
		this(null);
	}
	
	/**
	 * Constructs a new AcopMultipleDisplayerInfoDialog associated with the given
	 * displayer.
	 * 
	 * @param displayer displayer to be associated with this dialog
	 */
	public AcopMultipleDisplayerInfoDialog(MultipleAcopDisplayer<?> displayer) {
		initialize();
		setDisplayer(displayer);
		addPanel(getConnectionContentPanel(), "Connection", 0, true);
		addPanel(getPropertiesPanel(), "Bean properties", 1, false);
		addPanel(getAcopDebugPanel(), "Debug", 2, false);
		
	}
	
	private void initialize() {
		this.setSize(550,350);
		this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		this.setContentPane(getContentPanel());
	}
	
	private JPanel getConnectionContentPanel() {
		if (connectionContentPanel == null) {
			connectionContentPanel = new JPanel(new BorderLayout());
			connectionContentPanel.add(getSelectorPanel(), BorderLayout.NORTH);
			connectionContentPanel.add(getConnectionTab(), BorderLayout.CENTER);
		}

		return connectionContentPanel;
	}
	
	
	private JTabbedPane getConnectionTab() {
		if (connectionTab == null) {
			connectionTab = new JTabbedPane();
			connectionTab.add(getConnectionPanel(), "Connection", 0);
			connectionTab.add(getCharacteristicsPanel(), "Prop Info", 1);
			connectionTab.add(getConverterPanel(),"Converter", 2);
			connectionTab.setSelectedComponent(getConnectionPanel());
			
		}
		return connectionTab;
	}
	
	private AcopDebugPanel getAcopDebugPanel() {
		if (debugPanel == null) {
			debugPanel = new AcopDebugPanel();
		}
		return debugPanel;
	}
	
	private ConnectionPanel getConnectionPanel() {
		if (connectionPanel == null) {
			connectionPanel = new ConnectionPanel();
		}
		return connectionPanel;
	}
	
	private CharacteristicsPanel getCharacteristicsPanel() {
		if (characteristicsPanel == null) {
			characteristicsPanel = new CharacteristicsPanel();
		}
		return characteristicsPanel;
	}
	
	private JPanel getSelectorPanel() {
		if (selectorPanel == null) {
			selectorPanel = new JPanel(new GridBagLayout());
			connectionParametersLabel = new JLabel("Select parameters: ");
			connectionParametersLabel.setMaximumSize(new Dimension(70,21));
			connectionParametersLabel.setMaximumSize(new Dimension(70,21));
			connectionParametersLabel.setMaximumSize(new Dimension(70,21));
			selectorPanel.add(connectionParametersLabel, new GridBagConstraints(0,0,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(2,2,2,2), 0,0));
			selectorPanel.add(getParametersCombo(), new GridBagConstraints(1,0,1,1,1,0,GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, new Insets(2,2,2,2), 0,0));
		}
		return selectorPanel;
	}
	
	private JPanel getContentPanel() {
		if (contentPanel == null) {
			contentPanel = new JPanel(new BorderLayout());
			contentPanel.add(getTabPane(), BorderLayout.CENTER);
			JPanel buttonPanel = new JPanel(new GridBagLayout());
			buttonPanel.add(getCloseButton(), new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(1,1,1,1),0,0));
			contentPanel.add(buttonPanel, BorderLayout.SOUTH);
			
		}
		return contentPanel;
	}
	
	private JComboBox<AcopDisplayerParameters> getParametersCombo() {
		if (parametersComboBox == null) {
			parametersComboBox = new JComboBox<>();
			GUIUtilities.setFixedSize(parametersComboBox,new Dimension(100, 21));
			parametersComboBox.setRenderer(new ComboRenderer());
			parametersComboBox.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					setSelectedParameters((AcopDisplayerParameters)parametersComboBox.getSelectedItem());					
				}
			});
			
		}
		return parametersComboBox;
	}
	
	private JButton getCloseButton() {
		if (closeButton == null) {
			closeButton = new JButton("Close");
			closeButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					dispose();					
				}
			});
		}
		return closeButton;
	}
	
	private JTabbedPane getTabPane() {
		if (tabPane == null) {
			tabPane = new JTabbedPane();
		}
		return tabPane;
	}
	
	private ConverterPanel getConverterPanel() {
		if (converterPanel == null) {
			converterPanel = new ConverterPanel();
		}
		return converterPanel;
	}
	
	private PropertiesPanel getPropertiesPanel() {
		if (propertiesPanel == null) {
			propertiesPanel = new PropertiesPanel();
		}

		return propertiesPanel;
	}
	
	protected void addPanel(Component panel, String name, int position, boolean select) {
		for (int i = 0; i < getTabPane().getTabCount(); i++) {
			if (getTabPane().getTitleAt(i).equals(name)) {
				getTabPane().removeTabAt(i);
				break;
			}
		}
		getTabPane().add(panel, name, position);
		if (select) {
			getTabPane().setSelectedComponent(panel);
		}
	}
	
	protected void addPanel(Component panel, String name) {
		for (int i = 0; i < getTabPane().getTabCount(); i++) {
			if (getTabPane().getTitleAt(i).equals(name)) {
				getTabPane().removeTabAt(i);
				break;
			}
		}
		getTabPane().add(panel, name);
	}
	
	/**
	 * Sets the displayer to this info dialog. This component will listen
	 * to certain properties of the displayer and present their values.
	 * 
	 * @param displayer
	 */
	public void setDisplayer(MultipleAcopDisplayer<?> displayer) {
		if (this.displayer != null && !this.displayer.equals(displayer)) {
			this.displayer.removePropertyChangeListener(getPropertyListener());
			if (displayer != null) {
				displayer.addPropertyChangeListener(getPropertyListener());
			}
				
		} else if (this.displayer == null && displayer != null) {
			displayer.addPropertyChangeListener(getPropertyListener());
		}
		this.displayer = displayer;
		getParametersCombo().removeAllItems();
		if (this.displayer != null) {
			this.setTitle("Info dialog for " + this.displayer.getClass().getSimpleName() + ".");
			getPropertiesPanel().setBean(displayer);
			AcopDisplayerParameters[] parameters = this.displayer.getDisplayerParameters();
			
			if (parameters != null && parameters.length > 0) {
				for (AcopDisplayerParameters p :parameters) {
					getParametersCombo().addItem(p);
				}
				getParametersCombo().setSelectedIndex(0);
			}
		} else {
			this.setTitle("Info");
			getPropertiesPanel().setBean(new Object());
			setSelectedParameters(null);
		}
		
	}
	
	private void setSelectedParameters(AcopDisplayerParameters parameters) {
		this.selectedParameters = parameters;
		if (selectedParameters != null) {
			getConverterPanel().setConverter(parameters.getConverter());
			getConnectionPanel().setConnectionParameters(parameters.getConnectionParameters());
			getAcopDebugPanel().setConnectionParameters(parameters.getConnectionParameters());
			try {
				getCharacteristicsPanel().setCharacteristics(AdapterFactoryService.getInstance().getAdapterFactory().getCharacteristics(parameters.getConnectionParameters()));
			} catch (ConnectionFailed e) {
				e.printStackTrace();
			}
		} else {
			getConverterPanel().setConverter(null);
			getConnectionPanel().setConnectionParameters(null);
			getAcopDebugPanel().setConnectionParameters(null);
			getCharacteristicsPanel().setCharacteristics(null);
		}
		
	}
	
	private PropertyChangeListener getPropertyListener() {
		if (propertyListener == null) {
			propertyListener = new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent evt) {
					if (MultipleAcopDisplayer.DISPLAYER_PARAMETERS_PROPERTY.equals(evt.getPropertyName())) {
						Object item = getParametersCombo().getSelectedItem();
						getParametersCombo().removeAllItems();
						AcopDisplayerParameters[] parameters = displayer.getDisplayerParameters();
						if (parameters != null && parameters.length > 0) {
							for (AcopDisplayerParameters p :parameters) {
								getParametersCombo().addItem(p);
							}
							getParametersCombo().setSelectedIndex(0);
						}
						getParametersCombo().setSelectedItem(item);
					}
				}
			};
		}
		return propertyListener;
	}
	
	public MultipleDisplayer getDisplayer() {
		return this.displayer;
	}
	
	private static class ComboRenderer extends DefaultListCellRenderer {
		
		private static final long serialVersionUID = 1L;

		@Override
		public Component getListCellRendererComponent(JList<?> list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
			String text;
			if (value instanceof AcopDisplayerParameters) {
				text = ((AcopDisplayerParameters)value).getConnectionParameters().toString();
				if (((AcopDisplayerParameters)value).getConverter() != null) {
					text += ", Converter";
				}
				
				
			} else {
				text = "";
			}
			return super.getListCellRendererComponent(list, text, index, isSelected,
					cellHasFocus);
		}
		
	}
	
}
