package de.desy.acop.print.logbook;

import java.awt.print.PrinterJob;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import javax.print.PrintService;

//import de.desy.mst.libs.framework.mstapp.logging.Logger;
//import de.desy.mst.libs.framework.mstapp.logging.Logger;

class LogbookFinder {
	private static final Map<Maschine, LogbookFinder> map 
		= new ConcurrentHashMap<Maschine, LogbookFinder>();
	static{sInitialize();}
	
	static LogbookFinder findLogbook(final Maschine maschine, final PrintService[] services) {

        final String expectedLogbookName = maschine.getLogBuchName().toLowerCase() + "log";
        // Der Text wird wie folgt erwartet \\adprint(optionaleLaufendeNummer)(Optional
        // .desy.de)\Druckername
        final java.util.regex.Pattern expectedPrinter = java.util.regex.Pattern
                .compile("\\\\\\\\adprint\\d*(\\.desy\\.de)*\\\\" + expectedLogbookName);
        for (final PrintService service : services) {
            final String name = service.getName().toLowerCase();
            // Windows7 // adprint7 ist seit der abgeschlossenen Migration auf Windows 10 obsolete
            if (name.contains("adprint10")) {// Windows 10
                if (name.endsWith(("log"))
                        && (name.equals("\\\\" + "adprint10" + "\\" + maschine.getLogBuchName().toLowerCase() + "log")
                                || name.equals("\\\\" + "adprint10.desy.de" + "\\"
                                        + maschine.getLogBuchName().toLowerCase() + "log"))) {
                    return new LogbookFinder(maschine,service);
                }
            } else if (expectedPrinter.matcher(name).matches()) {
                // AL. 2022-12-20
                // Dieser Else Zweig ist neu und fuer die Windows Printer Server ab adprint11 vorgesehen.
                // Hier werden die drucker am Server adprint11 und an weiteren Druckern verarbeitet, die dieser
                // Konvention folgen: \\adprint(optionaleLaufendeNummer)(Optional .desy.de)\Druckername
                return new LogbookFinder(maschine,service);
            } else {// andere
                if (name.endsWith(("log")) && name.equals(maschine.getLogBuchName().toLowerCase() + "log")) {
                    return new LogbookFinder(maschine,service);
                }
            }
        }
        return new LogbookFinder(maschine,(PrintService) null);
    }
	
	/*private static LogbookFinder findLogbook(final Maschine maschine, final PrintService[] services) {
		for (final PrintService service : services) {
			final String name = service.getName().toLowerCase();
			if(name.contains("adprint7")){//Winows7
				if(name.endsWith(("log"))  && name.equals("\\\\" + "adprint7" + "\\" + maschine.getLogBuchName().toLowerCase() + "log")){
					return new LogbookFinder(maschine, service);
				}
			}else if(name.contains("adprint10")){//Windows 10
				if(name.endsWith(("log"))  && (name.equals("\\\\" + "adprint10" + "\\" + maschine.getLogBuchName().toLowerCase() + "log")|| name.equals("\\\\" + "adprint10.desy.de" + "\\" + maschine.getLogBuchName().toLowerCase() + "log"))){
					return new LogbookFinder(maschine, service);
				}
      }else if(name.contains("adprint11")){//Windows 11
        if(name.endsWith(("log"))  && (name.equals("\\\\" + "adprint11" + "\\" + maschine.getLogBuchName().toLowerCase() + "log")|| name.equals("\\\\" + "adprint11.desy.de" + "\\" + maschine.getLogBuchName().toLowerCase() + "log"))){
          return new LogbookFinder(maschine, service);
        }
      }else if(name.contains("adprint12")){//Windows 12
        if(name.endsWith(("log"))  && (name.equals("\\\\" + "adprint12" + "\\" + maschine.getLogBuchName().toLowerCase() + "log")|| name.equals("\\\\" + "adprint12.desy.de" + "\\" + maschine.getLogBuchName().toLowerCase() + "log"))){
          return new LogbookFinder(maschine, service);
        }
      }else if(name.contains("adprint13")){//Windows 13
        if(name.endsWith(("log"))  && (name.equals("\\\\" + "adprint13" + "\\" + maschine.getLogBuchName().toLowerCase() + "log")|| name.equals("\\\\" + "adprint13.desy.de" + "\\" + maschine.getLogBuchName().toLowerCase() + "log"))){
          return new LogbookFinder(maschine, service);
        }
			}else{// andere
				if(name.endsWith(("log"))  && name.equals(maschine.getLogBuchName().toLowerCase() + "log")){
					return new LogbookFinder(maschine, service);
				}
			}
		}
		return new LogbookFinder(maschine, null);
	}*/
	
	public static void sInitialize(){
		LogbookFinder.map.clear();
		PrintService[] services = tryToGetprinterList();
		if(services.length == 0)
			System.out.println(" Druckerliste konnte nicht geladen werden!! ");
		for (final Maschine maschine : Maschine.values()) {
			LogbookFinder.map.put(maschine, findLogbook(maschine, services));
		}
	}
	private static PrintService[] tryToGetprinterList() {
		final int no_of_retries = 2;
		int retryCounter = 0;		
		do {
//			final PrintService[] services = PrintServiceLookup.lookupPrintServices(null, null);
			final PrintService[] services = PrinterJob.lookupPrintServices();
			if(services.length == 0) {
				retryCounter++;
			}else {
				return services;
			}						
		} while (retryCounter <= no_of_retries);
		return PrinterJob.lookupPrintServices();
	}
	
	public static LogbookFinder getLogbook(final Maschine maschine){
		return LogbookFinder.map.get(maschine);
	}
	
	public static void main(final String ...strings){
		for (final Maschine maschine : map.keySet()) {
			System.out.println(maschine.toString() + "\t" + map.get(maschine).getService());
		}
	}
//=============================================================================
	private final Maschine maschine;
	private final PrintService service;
	public LogbookFinder(final Maschine maschine, final PrintService service) {
		this.maschine = maschine;
		this.service = service;
	}
	public final Maschine getMaschine() {return this.maschine;}
	public final PrintService   getService()     {return this.service;}
	public final boolean  isEnabled()   {return this.service != null && !this.service.getName().isEmpty();}
} 
