package de.desy.acop.print.logbook;

import de.desy.acop.about.ApplicationInfo;

/**
 * 
 * <code>PrintCallback</code> provides extra information to the printing facilities
 * or provides a dump for the errors and logging. 
 *
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 *
 */
public interface PrintCallback {
	
	/**
	 * Notifies the callback that printing has finished. The implementor
	 * might need to reenable certain features when printing is completed.
	 */
	void finished();
	
	/**
	 * Reports the exception in the standard format common to the application
	 * @param e the exception 
	 * @param message the message explaining the exception in more details
	 */
	void reportException(Throwable e, String message);
	
	/**
	 * Logs the message in the common way.
	 * @param message the message
	 * @param level the log level, smaller number means higher severity
	 */
	void log(String message, int level);
	
	/**
	 * @param args the extra arguments that might contain additional application info, which
	 * 			is used to construct the returned object
	 * @return the application info to be appended to the printed document
	 */
	ApplicationInfo getApplicationInfo(String... data);
}
