package de.desy.acop.transport;
import java.util.Date;
import java.util.Vector;
import de.desy.acop.chart.Acop;
import de.desy.acop.chart.AcopEvent;

public class AcopTransport
{
  protected String 
    DeviceContext,
    DeviceGroup,
    DeviceName,
    DeviceProperty,
    AccessRate,
    AccessMode,
    AccessProtocol,
    AcopConfig,
    Status;
  public boolean 
    isGrouped;
  protected int 
    GroupIndex,
    ReceiveQueueDepth;
  public int getReceiveQueueDepth() { return ReceiveQueueDepth; }
  protected double
    TimeStamp;
  protected Date
    DateStamp;
  public Date getDateStamp() { return DateStamp; }
  public Date getDateStamp(int hLink) 
  { 
    if (hLink >= 0)
    {
      AcopTransportRequest a;
      for (int i=0; i<acopLinkTable.size(); i++)
      {
        a = (AcopTransportRequest)acopLinkTable.elementAt(i);
        if (a.getLinkIdentifier() == hLink)
        {
          return a.getDataTimeStamp();
        }
      }
    }
    return DateStamp;
  }
  private Acop parentAcop;
  protected AcopTransportFactory atf;
  //protected AcopTransportRequest atr = new AcopTransportRequest();
  protected final Vector acopLinkTable = new Vector();
  public Vector getAcopLinkTable() { return acopLinkTable; }
  protected int 
    lastUpdatedLink = -1, 
    lastUpdatedDataSize = 0,
    lastUpdatedDataSizeEx = 0;
  
  public class AcopCallback implements AcopTransportCallback
  {
    public void callback(int linkHandle, int statusCode)
    {
      int linkTableId, identifier, i;
      AcopTransportRequest a;
      if ((linkTableId=AcopLinkTableIdFromCallbackId(linkHandle)) < 0) 
      {
        System.out.println("Link " + linkHandle + " not allocated");
        return;
      }
      if ((a=(AcopTransportRequest)acopLinkTable.elementAt(linkTableId)) == null)
      {
        System.out.println("Link " + linkHandle + " no entry in link table!");
        return;
      }      
      boolean pending = false;
      GetData(a.getData(),a.getDataEx(),a.getLinkIdentifier());
      identifier = a.getLinkIdentifier();
      if (isGrouped)
      {
        for (i=0; i<acopLinkTable.size(); i++)
        {
          if (i == linkTableId)
            ((AcopTransportRequest)acopLinkTable.elementAt(i)).pending = false;              
          if (((AcopTransportRequest)acopLinkTable.elementAt(i)).pending) pending = true;
        }
        if (pending) return;
      }
      else
      {
        a.pending = false;
      }
      parentAcop.notifyAcop(AcopEvent.ACOP_RECEIVE_EVENT,identifier,statusCode);
      if (isGrouped)
      {
        for (i=0; i<acopLinkTable.size(); i++)
         ((AcopTransportRequest)acopLinkTable.elementAt(i)).pending = true;              
      }
      else
      {
        a.pending = true;
      }
    }
  }
  protected AcopTransportCallback atc = new AcopCallback();
  public int AcopTransportAddProtocol(AcopTransportPlug plug)
  {
    return atf.addProtocol(plug);
  }
  public AcopTransport(Acop acop)
  {
    //{{INIT_CONTROLS
    parentAcop = acop;
    DeviceContext = "";
    DeviceGroup = "";
    DeviceName = "";
    DeviceProperty = "";
    AccessRate = "1000";
    AccessMode = "";
    AccessProtocol = "";
    AcopConfig = "STANDARD";
    Status = "";
    ReceiveQueueDepth = 100;
    TimeStamp = 0;
    isGrouped = false;
    GroupIndex = 0;
    atf = AcopTransportFactory.getInstance();
	//}}
  }
  //{{DECLARE_CONTROLS
  protected AcopTransportRequest getAcopTransportRequest(AcopTransportRequest atr)
  {
    AcopTransportRequest a;
    for (int i=0; i<acopLinkTable.size(); i++)
    {
      a = (AcopTransportRequest)acopLinkTable.elementAt(i);
      if (a.compareTo(atr)) return a; 
    }
    a = atr.cloneTransportRequest();
    acopLinkTable.addElement(a);
    return a;
  }
  public int AcopLinkTableId(AcopTransportRequest atr)
  {
    AcopTransportRequest a;
    for (int i=0; i<acopLinkTable.size(); i++)
    {
      a = (AcopTransportRequest)acopLinkTable.elementAt(i);
      if (a.compareTo(atr)) 
      {
        return i;
       } 
    }
    return -1;
  }
  public int AcopLinkTableId(int hLink)
  {
    AcopTransportRequest a;
    for (int i=0; i<acopLinkTable.size(); i++)
    {
      a = (AcopTransportRequest)acopLinkTable.elementAt(i);
      if (a.getLinkIdentifier() == hLink) return i;
    }
    return -1;
  }
  public int AcopLinkTableIdFromCallbackId(int hLinkCallback)
  {
    AcopTransportRequest a;
    for (int i=0; i<acopLinkTable.size(); i++)
    {
      a = (AcopTransportRequest)acopLinkTable.elementAt(i);
      if (a.getLinkHandle() == hLinkCallback) return i;
    }
    return -1;
  }
  protected int getFreeIdentifier()
  {
    AcopTransportRequest a;
    int identifier = 0;
    for (int i=0; i<acopLinkTable.size(); i++)
    {
      a = (AcopTransportRequest)acopLinkTable.elementAt(i);
      if (a.getLinkIdentifier() == identifier) identifier++;
    }
    return identifier;
  }
  public void halt()
  {
    atf.halt();
  }
  public int Execute(Object Data, int ArraySize, Object DataEx, int ArraySizeEx)
  {
    if (getObjectSize(Data) < ArraySize || getObjectSize(DataEx) < ArraySizeEx)
    {
      Status = "Data object smaller than size given";
      return -1;      
    }
    AcopTransportRequest atr = new AcopTransportRequest();
    AcopTransportPlug atp;
    atr.accessMethod = AcopTransportAccessMethods.EXECUTE;
    atr.accessMode = AccessMode;
    atr.accessRate = AccessRate;
    atr.accessProtocol = AccessProtocol;
    atr.setData(Data);
    atr.setArraySize(ArraySize);
    atr.setDataEx(DataEx);
    atr.setArraySizeEx(ArraySizeEx);
    atr.deviceContext = DeviceContext;
    atr.deviceGroup = DeviceGroup;
    atr.deviceName = DeviceName;
    atr.deviceProperty = DeviceProperty;
    atr.isUnboundToData = false;
    if ((atp=atf.getProtocol(AccessProtocol)) == null)
    {
      Status = "Protocol not available";
      return -1;
    }
    int rc = atp.handleRequest(atr);
    lastUpdatedDataSize = atr.getDataSize();
    lastUpdatedDataSizeEx = atr.getDataSizeEx();
    setTimeStamp(atr.getDataTimeStamp());
    Status = atr.status;
    return rc;
  }
  // TODO: the only difference between AttachLink() and OpenLink() seems to be atr.isUnboundToData
  // Execute() is also of the same ilk.  Maybe put the meat into one method only?
  public int AttachLink(Object Data, int ArraySize, Object DataEx, int ArraySizeEx,int identifier)
  {
    if (getObjectSize(Data) < ArraySize || getObjectSize(DataEx) < ArraySizeEx)
    {
      Status = "Data object smaller than size given";
      return -1;      
    }
    AcopTransportRequest atr = new AcopTransportRequest();
    AcopTransportPlug atp;
    atr.accessMethod = AcopTransportAccessMethods.ATTACH;
    atr.accessMode = AccessMode;
    atr.accessRate = AccessRate;
    atr.accessProtocol = AccessProtocol;
    atr.setData(Data);
    atr.setArraySize(ArraySize);
    atr.setDataEx(DataEx);
    atr.setArraySizeEx(ArraySizeEx);
    atr.deviceContext = DeviceContext;
    atr.deviceGroup = DeviceGroup;
    atr.deviceName = DeviceName;
    atr.deviceProperty = DeviceProperty;
    atr.atc = atc;
    atr.isUnboundToData = false;
    if ((atp=atf.getProtocol(AccessProtocol)) == null)
    {
      Status = "Protocol not available";
      return -1;
    }
    if (AcopLinkTableId(atr) >= 0) return atr.getLinkIdentifier(); 
    if (atp.handleRequest(atr) < 0) 
    {
      Status = atr.status;
      return -1;
    }
    if (identifier == -1) identifier = getFreeIdentifier();
    atr.setLinkIdentifier(identifier);
//    getAcopTransportRequest(atr);
    atr = getAcopTransportRequest(atr);
    atr.pending = true;
    Status = atr.status;
//    AcopLinkTableId(atr.getLinkIdentifier());
    return identifier;
  }
  public int OpenLink(Object Data, int ArraySize, Object DataEx, int ArraySizeEx,int identifier)
  {
    if (getObjectSize(Data) < ArraySize || getObjectSize(DataEx) < ArraySizeEx)
    {
      Status = "Data object smaller than size given";
      return -1;      
    }
    AcopTransportRequest atr = new AcopTransportRequest();
    AcopTransportPlug atp;
    atr.accessMethod = AcopTransportAccessMethods.ATTACH;
    atr.accessMode = AccessMode;
    atr.accessRate = AccessRate;
    atr.accessProtocol = AccessProtocol;
    atr.setData(Data);
    atr.setArraySize(ArraySize);
    atr.setDataEx(DataEx);
    atr.setArraySizeEx(ArraySizeEx);
    atr.deviceContext = DeviceContext;
    atr.deviceGroup = DeviceGroup;
    atr.deviceName = DeviceName;
    atr.deviceProperty = DeviceProperty;
    atr.atc = atc;
    atr.isUnboundToData = true;
    if ((atp=atf.getProtocol(AccessProtocol)) == null)
    {
      Status = "Protocol not available";
      return -1;
    }
    if (AcopLinkTableId(atr) >= 0) return atr.getLinkIdentifier();
    if (atp.handleRequest(atr) < 0) 
    {
      Status = atr.status;
      return -1;
    }
    if (identifier == -1) identifier = getFreeIdentifier();
    atr.setLinkIdentifier(identifier);
    getAcopTransportRequest(atr);
    atr.pending = true;
    Status = atr.status;
    AcopLinkTableId(atr.getLinkIdentifier());
    return identifier;
  }
  public int CloseLink(int hLink)  // hLink is the identifier !
  {
    AcopTransportRequest atr = new AcopTransportRequest();
    AcopTransportPlug atp;
    AcopTransportRequest a;
    int i;
    atr.accessMethod = AcopTransportAccessMethods.CLOSE;
    if (hLink >= 0)
    {
      for (i=0; i<acopLinkTable.size(); i++)
      {
        a = (AcopTransportRequest)acopLinkTable.elementAt(i);
        if (a.getLinkIdentifier() == hLink)
        {
          atr.setLinkHandle(a.getLinkHandle());     
          atr.accessProtocol = a.accessProtocol;
          if ((atp=atf.getProtocol(atr.accessProtocol)) == null)
          {
            Status = "Protocol not available";
            return -1;
          }
          atp.handleRequest(atr);
          acopLinkTable.remove(i--); //backup !
        }
      }
    }
    else
    {
      for (i=0; i<acopLinkTable.size(); i++)
      {
        a = (AcopTransportRequest)acopLinkTable.elementAt(i);
        atr.setLinkHandle(a.getLinkHandle());
        atr.accessProtocol = a.accessProtocol;
        if ((atp=atf.getProtocol(atr.accessProtocol)) == null)
        {
          Status = "Protocol not available";
          continue;
        }
        atp.handleRequest(atr);
      }
      acopLinkTable.removeAllElements();
    }
    Status = atr.status;
    return 0;
  }
  public int GetData(Object Data, Object DataEx, int hLink) 
  {
    AcopTransportRequest atr = new AcopTransportRequest();
    AcopTransportPlug atp;
    AcopTransportRequest a;
    int i;
    if (hLink < 0) hLink = 0;
    if ((i=AcopLinkTableId(hLink)) < 0) 
    {
      Status = "Link not allocated";
      return -1;
    }
    a = (AcopTransportRequest)acopLinkTable.elementAt(i);
    atr.accessMethod = AcopTransportAccessMethods.GETDATA;
    atr.setData(Data);
    atr.setDataEx(DataEx);
    atr.setLinkHandle(a.getLinkHandle());
    atr.accessProtocol = a.accessProtocol;
    atr.isUnboundToData = a.isUnboundToData;
    if ((atp=atf.getProtocol(atr.accessProtocol)) == null)
    {
      Status = "Protocol not available";
      return -1;
    }
    int rc =  atp.handleRequest(atr);
    lastUpdatedLink = hLink;
    lastUpdatedDataSize = atr.getDataSize();
    lastUpdatedDataSizeEx = atr.getDataSizeEx();
    setTimeStamp(atr.getDataTimeStamp());
    a.setDataTimeStamp(atr.getDataTimeStamp());
    Status = atr.status;
    return rc;
  }
  public int acquireDisplayProperties(Acop a,String deviceContext,String deviceGroup,String deviceName,String deviceProperty)
  {
    AcopTransportRequest atr = new AcopTransportRequest();
    AcopTransportPlug atp;
    if ((atp=atf.getProtocol(atr.accessProtocol)) == null)
    {
      Status = "Protocol not available";
      return -1;
    }
    return atp.acquireDisplayProperties(a, deviceContext, deviceGroup, deviceName, deviceProperty);
  }
  public double[] getXAxis(Object data)
  {
    AcopTransportRequest atr = new AcopTransportRequest();
    AcopTransportPlug atp;
    if ((atp=atf.getProtocol(atr.accessProtocol)) == null)
    {
      Status = "Protocol not available";
      return null;
    }
    return atp.getXAxis(data);
  }
  public double[] getYAxis(Object data)
  {
    AcopTransportRequest atr = new AcopTransportRequest();
    AcopTransportPlug atp;
    if ((atp=atf.getProtocol(atr.accessProtocol)) == null)
    {
      Status = "Protocol not available";
      return null;
    }
    return atp.getYAxis(data);
  }
  public int getObjectSize(Object data)
  {
    AcopTransportRequest atr = new AcopTransportRequest();
    AcopTransportPlug atp;
    if ((atp=atf.getProtocol(atr.accessProtocol)) == null)
    {
      Status = "Protocol not available";
      return -1;
    }
    return atp.getObjectSize(data);
  }
  public int GetDataSize(int[] DataSize, int[] DataSizeEx, int hLink) 
  {
    AcopTransportRequest a;
    int i;
    if (hLink < 0) 
    {
      if (DataSize != null) DataSize[0] = lastUpdatedDataSize;
      if (DataSizeEx != null) DataSizeEx[0] = lastUpdatedDataSizeEx;
      return 0;
    }
    if ((i=AcopLinkTableId(hLink)) < 0) 
    {
      Status = "Link not allocated";
      return -1;
    }
    a = (AcopTransportRequest)acopLinkTable.elementAt(i);
    if (DataSize != null) DataSize[0] = a.getDataSize();
    if (DataSizeEx != null) DataSizeEx[0] = a.getDataSizeEx();
    return 0;
  }
  public String[] getDeviceContexts() 
  {
    AcopTransportRequest atr = new AcopTransportRequest();
    AcopTransportPlug atp;
    atr.accessMethod = AcopTransportAccessMethods.EXECUTE;
    atr.accessMode = AccessMode;
    atr.accessRate = AccessRate;
    atr.accessProtocol = AccessProtocol;
    atr.deviceContext = DeviceContext;
    atr.deviceGroup = DeviceGroup;
    atr.deviceName = DeviceName;
    atr.deviceProperty = DeviceProperty;
    if ((atp=atf.getProtocol(atr.accessProtocol)) == null)
    {
      Status = "Protocol not available";
      return null;
    }
    return atp.getDeviceContexts();
  }
  public String[] getDeviceGroups(String deviceContext,String deviceGroup,String deviceName,String deviceProperty)
  {
    AcopTransportRequest atr = new AcopTransportRequest();
    AcopTransportPlug atp;
    atr.accessMethod = AcopTransportAccessMethods.EXECUTE;
    atr.accessMode = AccessMode;
    atr.accessRate = AccessRate;
    atr.accessProtocol = AccessProtocol;
    atr.deviceContext = DeviceContext;
    atr.deviceGroup = DeviceGroup;
    atr.deviceName = DeviceName;
    atr.deviceProperty = DeviceProperty;
    if ((atp=atf.getProtocol(atr.accessProtocol)) == null)
    {
      Status = "Protocol not available";
      return null;
    }
    return atp.getDeviceGroups(deviceContext,deviceGroup,deviceName,deviceProperty);
  }
  public String[] getDeviceNames(String deviceContext,String deviceGroup,String deviceName,String deviceProperty)
  {
    AcopTransportRequest atr = new AcopTransportRequest();
    AcopTransportPlug atp;
    atr.accessMethod = AcopTransportAccessMethods.EXECUTE;
    atr.accessMode = AccessMode;
    atr.accessRate = AccessRate;
    atr.accessProtocol = AccessProtocol;
    atr.deviceContext = DeviceContext;
    atr.deviceGroup = DeviceGroup;
    atr.deviceName = DeviceName;
    atr.deviceProperty = DeviceProperty;
    if ((atp=atf.getProtocol(atr.accessProtocol)) == null)
    {
      Status = "Protocol not available";
      return null;
    }
    return atp.getDeviceNames(deviceContext,deviceGroup,deviceName,deviceProperty);
  }
  public String[] getDeviceProperties(String deviceContext,String deviceGroup,String deviceName,String deviceProperty)
  {
    AcopTransportRequest atr = new AcopTransportRequest();
    AcopTransportPlug atp;
    atr.accessMethod = AcopTransportAccessMethods.EXECUTE;
    atr.accessMode = AccessMode;
    atr.accessRate = AccessRate;
    atr.accessProtocol = AccessProtocol;
    atr.deviceContext = DeviceContext;
    atr.deviceGroup = DeviceGroup;
    atr.deviceName = DeviceName;
    atr.deviceProperty = DeviceProperty;
    if ((atp=atf.getProtocol(atr.accessProtocol)) == null)
    {
      Status = "Protocol not available";
      return null;
    }
    return atp.getDeviceProperties(deviceContext,deviceGroup,deviceName,deviceProperty);
  }  
  public java.lang.String CTime(int TimeStamp) 
  {
    Date d = new Date((long)(TimeStamp * 1000));
    return d.toString();
  }
  public java.lang.String CTime(double TimeStamp) 
  {
    Date d = new Date((long)(TimeStamp * 1000));
    return d.toString();
  }
  public java.lang.String CTime(Date TimeStamp) 
  {
    return TimeStamp.toString();
  }
  public void setReceiveQueueDepth(int ReceiveQueueDepth)
  {
    this.ReceiveQueueDepth = ReceiveQueueDepth;
  }
  public void setTimeStamp(int TimeStamp)
  {
    this.TimeStamp = (double)TimeStamp;
    this.DateStamp = new Date(((long)TimeStamp)*1000);
  }
  public void setTimeStamp(Date TimeStamp)
  {
    this.DateStamp = TimeStamp;
    if (TimeStamp != null)
      this.TimeStamp = (double)(TimeStamp.getTime())/1000;
  }
  public void setTimeStamp(double TimeStamp)
  {
    this.TimeStamp = TimeStamp;
    this.DateStamp = new Date((long)(TimeStamp*1000));
  }
  public java.lang.String getStatus()
  {
    return Status;
  }
  public int getGroupIndex()
  {
    return isGrouped ? GroupIndex : -1;
  }
  public void setGrouped(boolean Grouped)
  {
    this.isGrouped = Grouped;
  }
  public void setDeviceContext(java.lang.String DeviceContext)
  {
    this.DeviceContext = DeviceContext;
  }
  public String getDeviceContext() { return DeviceContext; }
  public void setDeviceGroup(java.lang.String DeviceGroup)
  {
    this.DeviceGroup = DeviceGroup;
  }
  public String getDeviceGroup() { return DeviceGroup; }
  public void setDeviceName(java.lang.String DeviceName)
  {
    this.DeviceName = DeviceName;
  }
  public String getDeviceName() { return DeviceName; }
  public void setDeviceProperty(java.lang.String DeviceProperty)
  {
    this.DeviceProperty = DeviceProperty;
  }
  public String getDeviceProperty() { return DeviceProperty; }
  public void setAccessMode(java.lang.String AccessMode)
  {
    this.AccessMode = AccessMode;
  }
  public String getAccessMode() { return AccessMode; }
  public void setAccessProtocol(java.lang.String AccessProtocol)
  {
    this.AccessProtocol = AccessProtocol;
  }
  public String getAccessProtocol() { return AccessProtocol; }
  public void setAccessRate(java.lang.String AccessRate)
  {
    this.AccessRate = AccessRate;
  }
  public String getAccessRate() { return AccessRate; }
  public void setAcopConfig(java.lang.String AcopConfig)
  {
    this.AcopConfig = AcopConfig;
  } 	
  public String getAcopConfig() { return AcopConfig; }
}