/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.transport;

import java.util.Vector;
import de.desy.acop.transport.simulator.AcopTransportSimulator;
import de.desy.acop.transport.tine.AcopTransportTine;

/**
 * @version 1.0
 * @author
 */
public class AcopTransportFactory
{
  private static AcopTransportFactory instance;
  protected String defaultProtocol;
  protected Vector<AcopTransportPlug> atps = new Vector<>();
  protected String[] acopProtocolList;
  public synchronized static AcopTransportFactory getInstance()
  {
    if (instance == null) instance = new AcopTransportFactory();
    return instance;
  }
  public int addProtocol(AcopTransportPlug atp)
  {
    for (int i = 0; i < atps.size(); i++)
    {
      if (atps.elementAt(i) == atp) return 0;
    }
    atps.addElement(atp);
    System.out.println("add protocol " + atp.getPlugName());
    if (defaultProtocol == null) setDefaultProtocol(atp.getPlugName());
    return 0;
  }
  AcopTransportFactory()
  {
    // TODO: read the plugs from the manifest ?
    addProtocol(new AcopTransportTine());
    addProtocol(new AcopTransportSimulator());
  }
  int setDefaultProtocol(String protocolName)
  {
    for (int i = 0; i < atps.size(); i++)
    {
      if (atps.elementAt(i).getPlugName() == protocolName)
      {
        defaultProtocol = new String(protocolName);
        System.out.println("Use protocol " + protocolName + " as default");
        return 0;
      }
    }
    return -1;
  }
  public String getDefaultProtocol()
  {
    return defaultProtocol;
  }
  public String[] getProtocolList()
  {
    AcopTransportPlug atp;
    if (acopProtocolList == null)
    {
      acopProtocolList = new String[atps.size()];
      for (int i = 0; i < atps.size(); i++)
      {
        atp = atps.elementAt(i);
        acopProtocolList[i] = atp.getPlugName();
      }
    }
    return acopProtocolList;
  }
  public AcopTransportPlug getProtocol(String protocolName)
  {
    AcopTransportPlug atp;
    String protocol = protocolName;
    if (protocol == null || protocol.length() == 0) protocol = getDefaultProtocol();
    for (int i = 0; i < atps.size(); i++)
    {
      atp = atps.elementAt(i);
      if (atp.getPlugName().compareTo(protocol) == 0) return atp;
    }
    return null;
  }
  protected void halt()
  {
    AcopTransportPlug atp;
    AcopTransportRequest atr = new AcopTransportRequest();
    for (int i = 0; i < atps.size(); i++)
    {
      atp = atps.elementAt(i);
      atr.accessMethod = AcopTransportAccessMethods.SHUTDOWN;
      atp.handleRequest(atr);
    }
  }
}
