/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.transport.simulator;

import java.util.Date;
import java.util.Random;

import de.desy.acop.chart.Acop;
import de.desy.acop.transport.AcopTransportAccessMethods;
import de.desy.acop.transport.AcopTransportCallback;
import de.desy.acop.transport.AcopTransportPlug;
import de.desy.acop.transport.AcopTransportRequest;

/**
 * @version 1.0
 * @author
 */
public class AcopTransportSimulator implements AcopTransportPlug
{
  static private class SimulatorLink
  {
    int linkHandle = 0;
    int pollingInterval = 1000;
    long lastnotified = 0;
  }
  private final static int MAX_LINKS = 1000;
  private SimulatorLink[] links = new SimulatorLink[MAX_LINKS];
  private AcopTransportCallback atc;
  private String[] AcopSimulatorString = new String[1];
  /**
   * @see AcopPack.acopTransportPlug#handleRequest(acopTransportRequest)
   */
  public int handleRequest(AcopTransportRequest atr)
  {
    switch (atr.accessMethod)
    {
      case AcopTransportAccessMethods.EXECUTE:
        if (!atr.accessMode.isReadMode() 
            && !atr.accessMode.isWriteMode())
        {
          atr.status = "illegal mode (READ or WRITE required)";
          return -1;
        }
        if (atr.getData() != null)
        {
          int len = atr.getArraySize();
          atr.statusCode = simulateData(atr.getData(), len);
          atr.setDataSize(len); // just set it to the request
          atr.status = "Success";
          return 0;
        }
        break;
      case AcopTransportAccessMethods.ATTACH:
        int i = getLinkHandle();
        atr.setLinkHandle(i);
        links[i].pollingInterval = Integer.parseInt(atr.accessRate);
        atc = atr.atc;
        return 0;
      case AcopTransportAccessMethods.CLOSE:
        return rmvLinkHandle(atr.getLinkHandle());
      case AcopTransportAccessMethods.GETDATA:
        if (atr.getData() != null)
        {
          int len = atr.getArraySize();
          atr.statusCode = simulateData(atr.getData(), len);
          atr.setDataTimeStamp(new Date(ast.interim));
          atr.setSystemStamp(0);
          atr.setUserStamp(0);
          atr.setDataSize(len); // just set it to the request
          atr.status = "Success";
          return 0;
        }
        break;
      case AcopTransportAccessMethods.SHUTDOWN:
        active = false;
        terminate = true;
        break;
      default:
        return -1;
    }
    return 0;
  }
  /**
   * @see AcopPack.acopTransportPlug#getPlugName()
   */
  public String getPlugName()
  {
    return "Simulate";
  }
  public int getObjectSize(Object data)
  {
    if (data == null) return 0;
    if (data instanceof double[]) return ((double[]) data).length;
    if (data instanceof float[]) return ((float[]) data).length;
    if (data instanceof byte[]) return ((byte[]) data).length;
    if (data instanceof short[]) return ((short[]) data).length;
    if (data instanceof int[]) return ((int[]) data).length;
    if (data instanceof String) return ((String) data).length();
    if (data instanceof StringBuffer) return ((StringBuffer) data).length();
    if (data instanceof char[]) return ((char[]) data).length;
    return 0;
  }
  public double[] getYAxis(Object data)
  {
    int siz = getObjectSize(data);
    if (siz > 0)
    {
      if (data instanceof double[]) return (double[]) data;
      double[] y;
      y = new double[siz];
      if (data instanceof float[])
        for (int i = 0; i < siz; i++)
          y[i] = ((float[]) data)[i];
      else if (data instanceof byte[])
        for (int i = 0; i < siz; i++)
          y[i] = ((byte[]) data)[i];
      else if (data instanceof short[])
        for (int i = 0; i < siz; i++)
          y[i] = ((short[]) data)[i];
      else if (data instanceof int[]) for (int i = 0; i < siz; i++)
        y[i] = ((int[]) data)[i];
      return y;
    }
    return null;
  }
  public String[] getXAxisChannelNames(Object data)
  {
    return null;
  }
  public double[] getXAxis(Object data)
  {
    int siz = getObjectSize(data);
    if (siz > 0)
    {
      if (data instanceof double[]) return (double[]) data;
      double[] x;
      x = new double[siz];
      if (data instanceof float[])
        for (int i = 0; i < siz; i++)
          x[i] = ((float[]) data)[i];
      else if (data instanceof byte[])
        for (int i = 0; i < siz; i++)
          x[i] = ((byte[]) data)[i];
      else if (data instanceof short[])
        for (int i = 0; i < siz; i++)
          x[i] = ((short[]) data)[i];
      else if (data instanceof int[]) for (int i = 0; i < siz; i++)
        x[i] = ((int[]) data)[i];
      return x;
    }
    return null;
  }
  private int numberOfLinks = 0;
  private int getLinkHandle()
  {
    int i;
    for (i=0; i<numberOfLinks; i++) if (links[i].linkHandle < 0) break;
    if (i >= MAX_LINKS) return -1;
    if (i == numberOfLinks) 
    {
      numberOfLinks++;
      links[i] = new SimulatorLink();
      links[i].linkHandle = i;
    }
    return i;
  }
  private int rmvLinkHandle(int hLink)
  {
    if (hLink < 0 || hLink >= numberOfLinks) return -1;
    links[hLink].linkHandle = -1;
    if (hLink == numberOfLinks-1) numberOfLinks--;
    return numberOfLinks;
  }
  private static final int simulatedDataArraySize = 512;
  private float[] simulatedDataArray = new float[simulatedDataArraySize];
  private int simulateData(Object data, int datalength)
  {
    int i, len;
    if (data instanceof double[])
    {
      len = ((double[]) data).length;
      if (datalength > 0 && datalength < len) len = datalength;
      datalength = len;
      for (i = 0; i < len; i++)
        ((double[]) data)[i] =  simulatedDataArray[i % simulatedDataArraySize];
      return 0;
    }
    else if (data instanceof float[])
    {
      len = ((float[]) data).length;
      if (datalength > 0 && datalength < len) len = datalength;
      datalength = len;
      for (i = 0; i < len; i++)
        ((float[]) data)[i] = simulatedDataArray[i % simulatedDataArraySize];
      return 0;
    }
    else if (data instanceof int[])
    {
      len = ((int[]) data).length;
      if (datalength > 0 && datalength < len) len = datalength;
      datalength = len;
      for (i = 0; i < len; i++)
        ((int[]) data)[i] = (int) simulatedDataArray[i % simulatedDataArraySize];
      return 0;
    }
    else if (data instanceof short[])
    {
      len = ((short[]) data).length;
      if (datalength > 0 && datalength < len) len = datalength;
      datalength = len;
      for (i = 0; i < len; i++)
        ((short[]) data)[i] = (short) simulatedDataArray[i % simulatedDataArraySize];
      return 0;
    }
    return -1;
  }
  private Random acopSimulatorRandom = new Random();
  private void updateSimulatedData()
  {
    for (int i = 0; i < simulatedDataArraySize; i++)
    {
      simulatedDataArray[i] = 
        (float) (100 * Math.sin(i * 6.2832 / simulatedDataArraySize) + acopSimulatorRandom.nextFloat() * 10);
    }
  }
  private boolean active = false;
  private boolean terminate = false;
  private acopSimulatorThread ast;
  public AcopTransportSimulator()
  {
    active = true;
    ast = new acopSimulatorThread();
    ast.start();
    updateSimulatedData();
  }
  private class acopSimulatorThread extends Thread
  {
    boolean isWaiting = false;
    private long interim = 0;    
    public acopSimulatorThread() {
        super("AcopSimulator");
        setDaemon(true);
    }
    @Override
    public synchronized void run()
    {
      while (active)
      {
        if (terminate) break;
        try
        {
          isWaiting = true;
          sleep(100);
          interim = System.currentTimeMillis();
          isWaiting = false;
          updateSimulatedData();
          if (numberOfLinks > 0) // don't worry about link tables in the simulator
          {
            for (int i = 0; i < numberOfLinks; i++)
            {
              if (links[i].linkHandle < 0) continue;
              if (interim < links[i].lastnotified + links[i].pollingInterval) continue;
              if (atc != null)
              {
                atc.callback(i, 0);
              }
              links[i].lastnotified = interim;
            }
          }
        }
        catch (InterruptedException e)
        {
          System.out.println(" InterruptedException : " + e);
        }
      }
    }
  }
  public int acquireDisplayProperties(Acop a, String deviceContext, String deviceGroup, String deviceName,
      String deviceProperty)
  {
    a.setYMax(100);
    a.setYMin(-100);
    a.setXMax(512);
    a.setXMin(0);
    a.setYAxisLabel("Amplitude");
    a.setXAxisLabel("Simulated data");
    a.setGraphStyle(0);
    return 0;
  }
  public String[] getDeviceContexts()
  {
    if (AcopSimulatorString[0] == null)
    {
      AcopSimulatorString[0] = new String("TEST");
    }
    return AcopSimulatorString;
  }
  public String[] getDeviceGroups(String deviceContext, String deviceGroup, String deviceName,
      String deviceProperty)
  {
    if (AcopSimulatorString[0] == null)
    {
      AcopSimulatorString[0] = new String("TEST");
    }
    return AcopSimulatorString;
  }
  public String[] getDeviceNames(String deviceContext, String deviceGroup, String deviceName,
      String deviceProperty)
  {
    if (AcopSimulatorString[0] == null)
    {
      AcopSimulatorString[0] = new String("TEST");
    }
    return AcopSimulatorString;
  }
  public String[] getDeviceProperties(String deviceContext, String deviceGroup, String deviceName,
      String deviceProperty)
  {
    if (AcopSimulatorString[0] == null)
    {
      AcopSimulatorString[0] = new String("TEST");
    }
    return AcopSimulatorString;
  }
  public String getTargetInformation(String deviceContext, String deviceGroup, String deviceName,
      String deviceProperty)
  {
    return "simulated target";
  }
}
