package de.desy.acop.transport.tine;

import java.util.Date;
import java.util.Vector;
import de.desy.acop.chart.Acop;
import de.desy.acop.transport.AcopTransportAccessMethods;
import de.desy.acop.transport.AcopTransportPlug;
import de.desy.acop.transport.AcopTransportRequest;
import de.desy.tine.queryUtils.*;
import de.desy.tine.client.*;
import de.desy.tine.dataUtils.*;
import de.desy.tine.definitions.*;
import de.desy.tine.structUtils.*;
import de.desy.tine.types.*;

/**
 * @version 	1.0
 * @author
 */
public class AcopTransportTine implements AcopTransportPlug
{
  private AcopTransportTineCallback tcb;
  private Vector linkTable = new Vector();
  private int addLink(TLink tl)
  {
    linkTable.addElement(tl);
    return linkTable.indexOf(tl);
  }
  private int getTableId(TLink tl)
  {
    return linkTable.indexOf(tl);
  }  
  private int getTableId(int linkId)
  {
    for (int i=0; i<linkTable.size(); i++)
    {
      if (((TLink)linkTable.elementAt(i)).linkId == linkId) return i;
    }
    return -1;
  }  
  private int rmvLink(TLink tl)
  {
    linkTable.removeElement(tl);
    return 0;
  }
  private String deviceName;
  private String makeDeviceName(AcopTransportRequest atr)
  {
    if (atr.deviceContext != null && atr.deviceContext.length() > 0)
    {
      return new String("/" + atr.deviceContext + 
                           "/" + atr.deviceGroup + 
                           "/" + atr.deviceName);
    }
    else
    {
      return new String(atr.deviceGroup + "/" + atr.deviceName);
    }
  }
  private int makeAccessRate(AcopTransportRequest atr)
  {
    int rate = (new Integer(atr.accessRate)).intValue();
    if (rate == 0) rate = 1000;
    return rate;
  }
  private short makeAccessMode(AcopTransportRequest atr)
  {
    short m = 0;
    String tmp = (new String(atr.accessMode)).toUpperCase();
    if (tmp.indexOf("READ") >= 0) m |= TMode.CM_SINGLE;
    if (tmp.indexOf("WRITE") >= 0) m |= TMode.CM_SINGLE;
    if (tmp.indexOf("POLL") >= 0) m |= TMode.CM_POLL;
    if (tmp.indexOf("REFRESH") >= 0) m |= TMode.CM_REFRESH;
    if (tmp.indexOf("CONNECT") >= 0) m |= TMode.CM_CONNECT;
    if (tmp.indexOf("NETWORK") >= 0) m |= TMode.CM_NETWORK;
    if (tmp.indexOf("BROADCAST") >= 0) m |= TMode.CM_BCAST;
    if (tmp.indexOf("MULTICAST") >= 0) m |= TMode.CM_MCAST;
    if (tmp.indexOf("RECEIVE") >= 0) m |= TMode.CM_GLOBAL;
    if (tmp.indexOf("GLOBAL") >= 0) m |= TMode.CM_GLOBAL;
    if (tmp.indexOf("WAIT") >= 0) m |= TMode.CM_WAIT;
    if (tmp.indexOf("USE_ON_ERROR") >= 0) m |= TMode.CM_WAIT;
    return m;
  }
  private boolean isWriteCommand(String mode)
  {
    String tmp = (new String(mode)).toUpperCase();
    if (tmp.indexOf("WRITE") >= 0) return true;
    return false;
  }
  public int acquireDisplayProperties(Acop a,String deviceContext,String deviceGroup,String deviceName,String deviceProperty)
  {
    PropertyQueryEx[] p = TQuery.getDevicePropertyInformation(deviceContext, deviceGroup, deviceName, deviceProperty);
    if (p != null)
    {
      a.setYMax(p[0].prpMaxValue);
      a.setYMin(p[0].prpMinValue);
      a.setXMax(p[0].prpSize);
      a.setXMin(0);
      a.setYAxisLabel(p[0].prpUnits);
      a.setXAxisLabel(p[0].prpDescription);
      switch (p[0].prpGraphType) 
      {
        case 0: a.setGraphStyle(0); break;// lin-lin
        case 1: a.setGraphStyle(1); break;// log-lin
      }
      return 0;
    }
    return -1;
  }
  /**
   * @see AcopPack.acopTransportPlug#handleRequest(acopTransportRequest)
   */
  public int handleRequest(AcopTransportRequest atr)
  {
    TDataType dout, din;
    short access;
    int id;
    TLink tl = null;

    switch (atr.accessMethod)
    {
      case AcopTransportAccessMethods.EXECUTE:
        if (atr.accessMode.indexOf("READ") < 0 &&
            atr.accessMode.indexOf("WRITE") < 0)
        {
          atr.status = "illegal mode (READ or WRITE required)";
          atr.setDataTimeStamp(new Date());
          return -1;
        }
        deviceName = makeDeviceName(atr);
        dout = toTDataType(atr.getData(),atr.getArraySize(),!atr.isUnboundToData);
        if (dout == null) dout = new TDataType();
        din = toTDataType(atr.getDataEx(),atr.getArraySizeEx(),!atr.isUnboundToData);
        if (din == null) din = new TDataType();
        access = TAccess.CA_READ;
        if (isWriteCommand(atr.accessMode)) access |= TAccess.CA_WRITE;
        tl = new TLink(deviceName,atr.deviceProperty,dout,din,access);
        try
        {
          atr.statusCode = tl.execute(makeAccessRate(atr));
          atr.setDataSize(dout.getLastDataSize());
          if (atr.statusCode > 0) atr.status = tl.getLastError();
          else atr.status = "Success";
          tl.cancel();
          if (atr.getDataTimeStamp() == null) atr.setDataTimeStamp(new Date());
          atr.getDataTimeStamp().setTime(dout.getTimeStamp());
          return atr.statusCode;
        }
        catch (Exception e)
        {
          atr.status = e.getMessage();
          return -1;
        }
      case AcopTransportAccessMethods.ATTACH:
        deviceName = makeDeviceName(atr);
        dout = toTDataType(atr.getData(),atr.getArraySize(),!atr.isUnboundToData);
        din = toTDataType(atr.getDataEx(),atr.getArraySizeEx(),!atr.isUnboundToData);
        access = TAccess.CA_READ;
        if (isWriteCommand(atr.accessMode)) access |= TAccess.CA_WRITE;
        tl = new TLink(deviceName,atr.deviceProperty,dout,din,access);
        tcb = new AcopTransportTineCallback(atr.atc);
        addLink(tl);
        short mode = makeAccessMode(atr);
        try
        {
          if ((mode & TMode.CM_GLOBAL) == TMode.CM_GLOBAL)
          {
            atr.setLinkHandle(tl.receive(tcb));
          }
          else
          {
            atr.setLinkHandle(tl.attach(mode,tcb,makeAccessRate(atr)));
          }
          if (atr.getLinkHandle() < 0)
          {
            atr.statusCode = -atr.getLinkHandle();
            atr.status = tl.getError(atr.statusCode);
            return -1;
          }
          return atr.getLinkHandle();
        }
        catch (Exception e)
        {
          atr.statusCode = -1;
          atr.status = e.getMessage();
          return -1;
        }
      case AcopTransportAccessMethods.CLOSE:
        id = getTableId(atr.getLinkHandle());
        if (id >= 0) tl = (TLink)linkTable.elementAt(id);
        if (tl != null) 
        {
          tl.cancel();
          linkTable.remove(id);
        }
        break;
      case AcopTransportAccessMethods.GETDATA: 
        id = getTableId(atr.getLinkHandle());
        if (id >= 0) tl = (TLink)linkTable.elementAt(id);
        if (tl == null)
        {
          atr.statusCode = -1;  
          atr.status = "Invalid Link Index";
          return -1;
        }
        atr.statusCode = tl.getLinkStatus();  
        if (atr.statusCode > 0) atr.status = tl.getLastError();
        else atr.status = "Success";
        if (atr.getDataTimeStamp() == null) atr.setDataTimeStamp(new Date());        
        atr.getDataTimeStamp().setTime(tl.getLastTimeStamp());
        if (atr.isUnboundToData)
        {
          if (atr.getData() != null)
          {
            dout = tl.getOutputDataObject();
            dout.getData(atr.getData());
          }
          if (atr.getDataEx() != null)
          {
            din = tl.getInputDataObject();
            din.getData(atr.getDataEx());
          }
        }
        break;
      case AcopTransportAccessMethods.SHUTDOWN:
        for (id=0; id<linkTable.size(); id++)
        {
          ((TLink)linkTable.elementAt(id)).cancel();
        }
        break;
      default: 
        return -1;
    }
		return 0;
  }
  /**
   * @see AcopPack.acopTransportPlug#getPlugName()
   */
  public String getPlugName()
  {
    return "TINE";
  }
  public double[] getYAxis(Object data)
  {
    int siz = getObjectSize(data);
    if (siz > 0)
    {
      if (data instanceof double[]) return (double[])data;
      double[] y;
      if (data instanceof SPECTRUM)
      { 
        siz = ((SPECTRUM)data).length;
        y = new double[siz];
        for (int i=0; i<siz; i++) y[i] = ((SPECTRUM)data).d_spect_array[i];
        return y;
      }
      y = new double[siz];
      if (data instanceof float[]) for (int i=0; i<siz; i++) y[i] = ((float[])data)[i];
      else if (data instanceof byte[]) for (int i=0; i<siz; i++) y[i] = ((byte[])data)[i];
      else if (data instanceof short[]) for (int i=0; i<siz; i++) y[i] = ((short[])data)[i];
      else if (data instanceof int[]) for (int i=0; i<siz; i++) y[i] = ((int[])data)[i];
      else if (data instanceof FLTINT[]) for (int i=0; i<siz; i++) y[i] = ((FLTINT[])data)[i] != null ? ((FLTINT[])data)[i].fval : 0;
      else if (data instanceof POINT[]) for (int i=0; i<siz; i++) y[i] = ((POINT[])data)[i] != null ? ((POINT[])data)[i].f2val : 0;
      else if (data instanceof FLTFLT[]) for (int i=0; i<siz; i++) y[i] = ((FLTFLT[])data)[i] != null ? ((FLTFLT[])data)[i].f1val : 0;
      else if (data instanceof DBLINT[]) for (int i=0; i<siz; i++) y[i] = ((DBLINT[])data)[i] != null ? ((DBLINT[])data)[i].dval : 0;
      else if (data instanceof INTINT[]) for (int i=0; i<siz; i++) y[i] = ((INTINT[])data)[i] != null ? ((INTINT[])data)[i].i1val : 0;
      else if (data instanceof DBLDBL[]) for (int i=0; i<siz; i++) y[i] = ((DBLDBL[])data)[i] != null ? ((DBLDBL[])data)[i].d1val : 0;
      else if (data instanceof TDS[]) for (int i=0; i<siz; i++) y[i] = ((TDS[])data)[i] != null ? ((TDS[])data)[i].dval : 0;
      else if (data instanceof INTFLTINT[]) for (int i=0; i<siz; i++) y[i] = ((INTFLTINT[])data)[i] != null ? ((INTFLTINT[])data)[i].fval : 0;
      else if (data instanceof FLTFLTINT[]) for (int i=0; i<siz; i++) y[i] = ((FLTFLTINT[])data)[i] != null ? ((FLTFLTINT[])data)[i].f1val : 0;
      else if (data instanceof FLTINTINT[]) for (int i=0; i<siz; i++) y[i] = ((FLTINTINT[])data)[i] != null ? ((FLTINTINT[])data)[i].f1val : 0;
      else if (data instanceof Object[]) 
      {
        if ((((Object[])data)[0]) instanceof Double) for (int i=0; i<siz; i++) y[i] =  ((Double)(((Object[])data)[i])).doubleValue();
        else if ((((Object[])data)[0]) instanceof Float) for (int i=0; i<siz; i++) y[i] =  ((Float)(((Object[])data)[i])).floatValue();
        else if ((((Object[])data)[0]) instanceof Short) for (int i=0; i<siz; i++) y[i] =  ((Short)(((Object[])data)[i])).shortValue();
        else if ((((Object[])data)[0]) instanceof Integer) for (int i=0; i<siz; i++) y[i] =  ((Integer)(((Object[])data)[i])).intValue();
        else if ((((Object[])data)[0]) instanceof Byte) for (int i=0; i<siz; i++) y[i] =  ((Byte)(((Object[])data)[i])).intValue();
        else if ((((Object[])data)[0]) instanceof FLTINT) for (int i=0; i<siz; i++) y[i] =  ((FLTINT)(((Object[])data)[i])).fval;
        else if ((((Object[])data)[0]) instanceof POINT) for (int i=0; i<siz; i++) y[i] =  ((POINT)(((Object[])data)[i])).f2val;
        else if ((((Object[])data)[0]) instanceof FLTFLT) for (int i=0; i<siz; i++) y[i] =  ((FLTFLT)(((Object[])data)[i])).f1val;
        else if ((((Object[])data)[0]) instanceof INTINT) for (int i=0; i<siz; i++) y[i] =  ((INTINT)(((Object[])data)[i])).i1val;
        else if ((((Object[])data)[0]) instanceof DBLDBL) for (int i=0; i<siz; i++) y[i] =  ((DBLDBL)(((Object[])data)[i])).d1val;
        else if ((((Object[])data)[0]) instanceof TDS) for (int i=0; i<siz; i++) y[i] =  ((TDS)(((Object[])data)[i])).dval;
        else if ((((Object[])data)[0]) instanceof INTFLTINT) for (int i=0; i<siz; i++) y[i] =  ((INTFLTINT)(((Object[])data)[i])).fval;
        else if ((((Object[])data)[0]) instanceof FLTFLTINT) for (int i=0; i<siz; i++) y[i] =  ((FLTFLTINT)(((Object[])data)[i])).f1val;
        else if ((((Object[])data)[0]) instanceof FLTINTINT) for (int i=0; i<siz; i++) y[i] =  ((FLTINTINT)(((Object[])data)[i])).f1val;
      }
      return y;
    }
    return null;    
  }
  public double[] getXAxis(Object data)
  {
    int siz = getObjectSize(data);
    if (siz > 0)
    {
      if (data instanceof double[]) return (double[])data;
      double[] x;
      if (data instanceof SPECTRUM)
      { 
        siz = ((SPECTRUM)data).length;
        x = new double[siz];
        for (int i=0; i<siz; i++) 
          x[i] = ((SPECTRUM)data).s_start + i * ((SPECTRUM)data).s_inc;
        return x;
      }
      x = new double[siz];
      if (data instanceof float[]) for (int i=0; i<siz; i++) x[i] = ((float[])data)[i];
      else if (data instanceof byte[]) for (int i=0; i<siz; i++) x[i] = ((byte[])data)[i];
      else if (data instanceof short[]) for (int i=0; i<siz; i++) x[i] = ((short[])data)[i];
      else if (data instanceof int[]) for (int i=0; i<siz; i++) x[i] = ((int[])data)[i];
      else if (data instanceof FLTINT[]) for (int i=0; i<siz; i++) x[i] = ((FLTINT[])data)[i] != null ? ((FLTINT[])data)[i].ival : 0;
      else if (data instanceof POINT[]) for (int i=0; i<siz; i++) x[i] = ((POINT[])data)[i] != null ? ((POINT[])data)[i].f1val :  0;
      else if (data instanceof FLTFLT[]) for (int i=0; i<siz; i++) x[i] = ((FLTFLT[])data)[i] != null ? ((FLTFLT[])data)[i].f2val : 0;
      else if (data instanceof DBLINT[]) for (int i=0; i<siz; i++) x[i] = ((DBLINT[])data)[i] != null ? ((DBLINT[])data)[i].ival : 0;
      else if (data instanceof INTINT[]) for (int i=0; i<siz; i++) x[i] = ((INTINT[])data)[i] != null ? ((INTINT[])data)[i].i2val : 0;
      else if (data instanceof DBLDBL[]) for (int i=0; i<siz; i++) x[i] = ((DBLDBL[])data)[i] != null ? ((DBLDBL[])data)[i].d2val : 0;
      else if (data instanceof TDS[]) for (int i=0; i<siz; i++) x[i] = ((TDS[])data)[i] != null ? ((TDS[])data)[i].tval : 0;
      else if (data instanceof INTFLTINT[]) for (int i=0; i<siz; i++) x[i] = ((INTFLTINT[])data)[i] != null ? ((INTFLTINT[])data)[i].i1val : 0;
      else if (data instanceof FLTFLTINT[]) for (int i=0; i<siz; i++) x[i] = ((FLTFLTINT[])data)[i] != null ? ((FLTFLTINT[])data)[i].f2val : 0;
      else if (data instanceof FLTINTINT[]) for (int i=0; i<siz; i++) x[i] = ((FLTINTINT[])data)[i] != null ? ((FLTINTINT[])data)[i].i1val : 0;
      else if (data instanceof Object[]) 
      {
        if ((((Object[])data)[0]) instanceof Double) for (int i=0; i<siz; i++) x[i] =  ((Double)(((Object[])data)[i])).doubleValue();
        else if ((((Object[])data)[0]) instanceof Float) for (int i=0; i<siz; i++) x[i] =  ((Float)(((Object[])data)[i])).floatValue();
        else if ((((Object[])data)[0]) instanceof Short) for (int i=0; i<siz; i++) x[i] =  ((Short)(((Object[])data)[i])).shortValue();
        else if ((((Object[])data)[0]) instanceof Integer) for (int i=0; i<siz; i++) x[i] =  ((Integer)(((Object[])data)[i])).intValue();
        else if ((((Object[])data)[0]) instanceof Byte) for (int i=0; i<siz; i++) x[i] =  ((Byte)(((Object[])data)[i])).intValue();
        else if ((((Object[])data)[0]) instanceof FLTINT) for (int i=0; i<siz; i++) x[i] =  ((FLTINT)(((Object[])data)[i])).ival;
        else if ((((Object[])data)[0]) instanceof POINT) for (int i=0; i<siz; i++) x[i] =  ((POINT)(((Object[])data)[i])).f1val;
        else if ((((Object[])data)[0]) instanceof FLTFLT) for (int i=0; i<siz; i++) x[i] =  ((FLTFLT)(((Object[])data)[i])).f2val;
        else if ((((Object[])data)[0]) instanceof INTINT) for (int i=0; i<siz; i++) x[i] =  ((INTINT)(((Object[])data)[i])).i2val;
        else if ((((Object[])data)[0]) instanceof DBLDBL) for (int i=0; i<siz; i++) x[i] =  ((DBLDBL)(((Object[])data)[i])).d2val;
        else if ((((Object[])data)[0]) instanceof TDS) for (int i=0; i<siz; i++) x[i] =  ((TDS)(((Object[])data)[i])).tval;
        else if ((((Object[])data)[0]) instanceof INTFLTINT) for (int i=0; i<siz; i++) x[i] =  ((INTFLTINT)(((Object[])data)[i])).i1val;
        else if ((((Object[])data)[0]) instanceof FLTFLTINT) for (int i=0; i<siz; i++) x[i] =  ((FLTFLTINT)(((Object[])data)[i])).f2val;
        else if ((((Object[])data)[0]) instanceof FLTINTINT) for (int i=0; i<siz; i++) x[i] =  ((FLTINTINT)(((Object[])data)[i])).i1val;
      }
      return x;
    }
    return null;    
  }
  public int getObjectSize(Object data)
  {
    if (data == null) return 0;
    if (data instanceof char[]) return ((char[])data).length;
    if (data instanceof TTaggedStructure[]) return ((TTaggedStructure[])data).length;
    if (data instanceof TTaggedStructure) return 1;
    return TDataType.getObjectSize(data);
  }
  private TDataType toTDataType(Object data, int length, boolean isBound)
  {
    if (data == null) return null;  // return null if Object is null !
    if (data instanceof double[])
    {
      return isBound ? new TDataType((double[])data, length) : new TDataType(((double[])data).length,TFormat.CF_DOUBLE);
    }
    if (data instanceof float[])
    {
      return isBound ? new TDataType((float[])data, length) : new TDataType(length,TFormat.CF_FLOAT);
    }
    if (data instanceof int[])
    {
      return isBound ? new TDataType((int[])data, length) : new TDataType(((int[])data).length,TFormat.CF_LONG);
    }
    if (data instanceof short[])
    {
      return isBound ? new TDataType((short[])data, length) : new TDataType(((short[])data).length,TFormat.CF_SHORT);
    }
    if (data instanceof byte[])
    {
      return isBound ? new TDataType((byte[])data, length) : new TDataType(((byte[])data).length,TFormat.CF_BYTE);
    }
    if (data instanceof DBLINT[])
    {
      return isBound ? new TDataType((DBLINT[])data, length) : new TDataType(((DBLINT[])data).length,TFormat.CF_DBLINT);
    }
    if (data instanceof DBLDBL[])
    {
      return isBound ? new TDataType((DBLDBL[])data, length) : new TDataType(((DBLDBL[])data).length,TFormat.CF_DBLDBL);
    }
    if (data instanceof FLTINT[])
    {
      return isBound ? new TDataType((FLTINT[])data, length) : new TDataType(((FLTINT[])data).length,TFormat.CF_FLTINT);
    }
    if (data instanceof FLTFLT[])
    {
      return isBound ? new TDataType((FLTFLT[])data, length) : new TDataType(((FLTFLT[])data).length,TFormat.CF_FLTFLT);
    }
    if (data instanceof POINT[])
    {
      return isBound ? new TDataType((POINT[])data, length) : new TDataType(((POINT[])data).length,TFormat.CF_POINT);
    }
    if (data instanceof INTINT[])
    {
      return isBound ? new TDataType((INTINT[])data, length) : new TDataType(((INTINT[])data).length,TFormat.CF_LNGINT);
    }
    if (data instanceof NAME8[])
    {
      return isBound ? new TDataType((NAME8[])data, length) : new TDataType(((NAME8[])data).length,TFormat.CF_NAME8);
    }
    if (data instanceof NAME16[])
    {
      return isBound ? new TDataType((NAME16[])data, length) : new TDataType(((NAME16[])data).length,TFormat.CF_NAME16);
    }
    if (data instanceof NAME32[])
    {
      return isBound ? new TDataType((NAME32[])data, length) : new TDataType(((NAME32[])data).length,TFormat.CF_NAME32);
    }
    if (data instanceof NAME48[])
    {
      return isBound ? new TDataType((NAME48[])data, length) : new TDataType(((NAME48[])data).length,TFormat.CF_NAME48);
    }
    if (data instanceof NAME64[])
    {
      return isBound ? new TDataType((NAME64[])data, length) : new TDataType(((NAME64[])data).length,TFormat.CF_NAME64);
    }
    if (data instanceof NAME16I[])
    {
      return isBound ? new TDataType((NAME16I[])data, length) : new TDataType(((NAME16I[])data).length,TFormat.CF_NAME16I);
    }
    if (data instanceof NAME16FI[])
    {
      return isBound ? new TDataType((NAME16FI[])data, length) : new TDataType(((NAME16FI[])data).length,TFormat.CF_NAME16FI);
    }
    if (data instanceof NAME16II[])
    {
      return isBound ? new TDataType((NAME16II[])data, length) : new TDataType(((NAME16II[])data).length,TFormat.CF_NAME16II);
    }
    if (data instanceof NAME16DBLDBL[])
    {
      return isBound ? new TDataType((NAME16DBLDBL[])data, length) : new TDataType(((NAME16DBLDBL[])data).length,TFormat.CF_NAME16DBLDBL);
    }
    if (data instanceof NAME32DBLDBL[])
    {
      return isBound ? new TDataType((NAME32DBLDBL[])data, length) : new TDataType(((NAME32DBLDBL[])data).length,TFormat.CF_NAME32DBLDBL);
    }
    if (data instanceof NAME64DBLDBL[])
    {
      return isBound ? new TDataType((NAME64DBLDBL[])data, length) : new TDataType(((NAME64DBLDBL[])data).length,TFormat.CF_NAME64DBLDBL);
    }
    if (data instanceof NAME32I[])
    {
      return isBound ? new TDataType((NAME32I[])data, length) : new TDataType(((NAME32I[])data).length,TFormat.CF_NAME32I);
    }
    if (data instanceof NAME48I[])
    {
      return isBound ? new TDataType((NAME48I[])data, length) : new TDataType(((NAME48I[])data).length,TFormat.CF_NAME48I);
    }
    if (data instanceof NAME64I[])
    {
      return isBound ? new TDataType((NAME64I[])data, length) : new TDataType(((NAME64I[])data).length,TFormat.CF_NAME64I);
    }
    if (data instanceof TDS[])
    {
      return isBound ? new TDataType((TDS[])data, length) : new TDataType(((TDS[])data).length,TFormat.CF_TDS);
    }
    if (data instanceof INTFLTINT[])
    {
      return isBound ? new TDataType((INTFLTINT[])data, length) : new TDataType(((INTFLTINT[])data).length,TFormat.CF_INTFLTINT);
    }
    if (data instanceof FLTFLTINT[])
    {
      return isBound ? new TDataType((FLTFLTINT[])data, length) : new TDataType(((FLTFLTINT[])data).length,TFormat.CF_FLTFLTINT);
    }
    if (data instanceof FLTINTINT[])
    {
      return isBound ? new TDataType((FLTINTINT[])data, length) : new TDataType(((FLTINTINT[])data).length,TFormat.CF_FLTINTINT);
    }
    if (data instanceof USTRING[])
    {
      return isBound ? new TDataType((USTRING[])data, length) : new TDataType(((USTRING[])data).length,TFormat.CF_USTRING);
    }
    if (data instanceof UNAME[])
    {
      return isBound ? new TDataType((UNAME[])data, length) : new TDataType(((UNAME[])data).length,TFormat.CF_UNAME);
    }
    if (data instanceof ADDRESS[])
    {
      return isBound ? new TDataType((ADDRESS[])data, length) : new TDataType(((ADDRESS[])data).length,TFormat.CF_ADDRESS);
    }
    if (data instanceof INTINTINTINT[])
    {
      return isBound ? new TDataType((INTINTINTINT[])data, length) : new TDataType(((INTINTINTINT[])data).length,TFormat.CF_INTINTINTINT);
    }
    if (data instanceof FILTER[])
    {
      return isBound ? new TDataType((FILTER[])data, length) : new TDataType(((FILTER[])data).length,TFormat.CF_FILTER);
    }
    if (data instanceof INTFLTFLTFLT[])
    {
      return isBound ? new TDataType((INTFLTFLTFLT[])data, length) : new TDataType(((INTFLTFLTFLT[])data).length,TFormat.CF_INTFLTFLTFLT);
    }
    if (data instanceof FWINDOW[])
    {
      return isBound ? new TDataType((FWINDOW[])data, length) : new TDataType(((FWINDOW[])data).length,TFormat.CF_FWINDOW);
    }
    if (data instanceof INTINTFLTFLT[])
    {
      return isBound ? new TDataType((INTINTFLTFLT[])data, length) : new TDataType(((INTINTFLTFLT[])data).length,TFormat.CF_INTINTFLTFLT);
    }
    if (data instanceof SPECTRUM)
    {
      return isBound ? new TDataType((SPECTRUM)data) : new TDataType(((SPECTRUM)data).length,TFormat.CF_SPECTRUM);
    }
    if (data instanceof String)
    {
      return isBound ? new TDataType((String)data, length) : new TDataType(((String[])data).length,TFormat.CF_TEXT);
    }
    if (data instanceof StringBuffer)
    {
      return isBound ? new TDataType((StringBuffer)data, length) : new TDataType(((StringBuffer)data).length(),TFormat.CF_TEXT);
    }
    if (data instanceof char[])
    {
      return isBound ? new TDataType((char[])data, length) : new TDataType(((char[])data).length,TFormat.CF_TEXT);
    }
    if (data instanceof TTaggedStructure)
    {
      return isBound ? new TDataType((TTaggedStructure)data) : new TDataType(((TTaggedStructure)data).getSizeInBytes(),TFormat.CF_STRUCT);
    }
    if (data instanceof TTaggedStructure[])
    {
      return isBound ? new TDataType((TTaggedStructure[])data) : new TDataType(((TTaggedStructure[])data)[0].getSizeInBytes()*((TTaggedStructure[])data).length,TFormat.CF_STRUCT);
    }
    return null;
  }
  public String[] getDeviceContexts() 
  {
    return TQuery.getContexts();
  }
  public String[] getDeviceGroups(String deviceContext,String deviceGroup,String deviceName,String deviceProperty)
  {
    return TQuery.getDeviceServers(deviceContext);
  }
  public String[] getDeviceNames(String deviceContext,String deviceGroup,String deviceName,String deviceProperty)
  {
    return TQuery.getDeviceNames(deviceContext,deviceGroup,deviceProperty);
  }
  public String[] getDeviceProperties(String deviceContext,String deviceGroup,String deviceName,String deviceProperty)
  {
    return TQuery.getDeviceProperties(deviceContext,deviceGroup,deviceName);
  }
}
