package de.desy.acop.video;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.BevelBorder;
import javax.swing.filechooser.FileNameExtensionFilter;

import com.cosylab.gui.components.util.RunnerHelper;


import de.desy.acop.transport.AccessMode;
import de.desy.acop.transport.ConnectionParameters;
import de.desy.video.sw.ImageDisplayer.ColorMap;


public class AcopVideoApplication extends JPanel {

	private static final long serialVersionUID = 1L;
	
	private JPanel controlPanel;
	private JLabel colorMapLabel;
	private JComboBox colorMapBox;
	private JCheckBox overlayCheckBox;
	private JCheckBox aoiZoomCheckBox;
	private JCheckBox aspectRatioCheckBox;
	private JLabel addressLabel;
	private JTextField addressField;
	private JButton addressButton;
	private JButton startButton;
	private JButton stopButton;
	private JButton saveAsPNGButton;
	
	private AcopVideo acopVideo;
	
	private JFileChooser chooser;
		
	public AcopVideoApplication() {
		super();
		initialize();
	}
	
	private void initialize() {
		this.setLayout(new BorderLayout());
		this.add(getImageDisplayer(), BorderLayout.CENTER);
		this.add(getControlPanel(), BorderLayout.SOUTH);
		getControlPanel().addMouseListener(getImageDisplayer().getPopupManager().getMouseHook());
	}
	
	private AcopVideo getImageDisplayer() {
		if (acopVideo == null) {
			acopVideo = new AcopVideo();
			acopVideo.addPropertyChangeListener(new PropertyChangeListener(){
				public void propertyChange(PropertyChangeEvent evt) {
					String propertyName = evt.getPropertyName();
					if ("AOIZoom".equals(propertyName)) {
						aoiZoomCheckBox.setSelected(getImageDisplayer().isAOIZoom());
					} else if ("keepAspectRatio".equals(propertyName)) {
						aspectRatioCheckBox.setSelected(getImageDisplayer().isKeepAspectRatio());
					} else if ("overlayEnabled".equals(propertyName)) {
						overlayCheckBox.setSelected(getImageDisplayer().isOverlayEnabled());
					} else if ("falseColorSwitchingEnabled".equals(propertyName)) {
						colorMapBox.setEnabled(getImageDisplayer().isFalseColorSwitchingEnabled());
					} else if ("colorMap".equals(propertyName)) {
						colorMapBox.setSelectedItem(getImageDisplayer().getColorMap());
					} else if ("connectionParameters".equals(propertyName)) {
						ConnectionParameters p = getImageDisplayer().getConnectionParameters();
						if (p == null) {
							addressField.setText("");
						} else {
							addressField.setText("/" + p.getDeviceContext() + "/" + p.getDeviceGroup() + "/" + p.getDeviceName());
						}
					}
				}
			});
		}
		return acopVideo;
	}
				
	private JPanel getControlPanel() {
		if (controlPanel == null) {
			controlPanel = new JPanel(new GridBagLayout());
			controlPanel.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
			
			colorMapLabel = new JLabel("Color:");
			colorMapBox = new JComboBox(ColorMap.values());
			colorMapBox.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					getImageDisplayer().setColorMap((ColorMap)colorMapBox.getSelectedItem());
				}
			});
			colorMapBox.setSelectedItem(getImageDisplayer().getColorMap());
			colorMapBox.setEnabled(getImageDisplayer().isFalseColorSwitchingEnabled());
			
			overlayCheckBox = new JCheckBox("Overlay ON");
			overlayCheckBox.addItemListener(new ItemListener(){
				public void itemStateChanged(ItemEvent e) {
					getImageDisplayer().setOverlayEnabled(overlayCheckBox.isSelected());
				}
			});
			overlayCheckBox.setSelected(getImageDisplayer().isOverlayEnabled());
			
			aoiZoomCheckBox = new JCheckBox("AOI Zoom");
			aoiZoomCheckBox.addItemListener(new ItemListener(){
				public void itemStateChanged(ItemEvent e) {
					getImageDisplayer().setAOIZoom(aoiZoomCheckBox.isSelected());
				}
			});
			aoiZoomCheckBox.setSelected(getImageDisplayer().isAOIZoom());
			
			aspectRatioCheckBox = new JCheckBox("Keep Aspect Ratio");
			aspectRatioCheckBox.addItemListener(new ItemListener(){
				public void itemStateChanged(ItemEvent e) {
					getImageDisplayer().setKeepAspectRatio(aspectRatioCheckBox.isSelected());
				}
			});
			aspectRatioCheckBox.setSelected(getImageDisplayer().isKeepAspectRatio());
			
			addressLabel = new JLabel("Address:");
			addressField = new JTextField();
			addressButton = new JButton("...");
			addressButton.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					getImageDisplayer().getCustomizer().selectAspect("Connections");
					getImageDisplayer().getCustomizer().showDialog();
				}
			});
			addressButton.setMinimumSize(new Dimension(19,19));
			addressButton.setMaximumSize(new Dimension(19,19));
			addressButton.setPreferredSize(new Dimension(19,19));
			
			startButton = new JButton("Start");
			startButton.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					start();
				}
			});
			
			stopButton = new JButton("Stop");
			stopButton.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					getImageDisplayer().stop();
				}
			});
			
			saveAsPNGButton = new JButton("Save As PNG");
			saveAsPNGButton.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					saveAsPNG();
				}
			});
			

			JPanel colorMapPanel = new JPanel(new GridBagLayout());
			colorMapPanel.add(colorMapLabel, new GridBagConstraints(0,0,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(2,2,2,2),0,0));
			colorMapPanel.add(colorMapBox, new GridBagConstraints(0,1,1,1,1,0,GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, new Insets(0,2,2,2),0,0));
			controlPanel.add(colorMapPanel, new GridBagConstraints(0,0,1,1,0,1,GridBagConstraints.NORTH, GridBagConstraints.NONE, new Insets(5,5,5,5),0,0));
			
			JPanel checkBoxPanel = new JPanel(new GridBagLayout());
			checkBoxPanel.add(overlayCheckBox, new GridBagConstraints(0,0,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(2,2,2,2),0,0));
			checkBoxPanel.add(aspectRatioCheckBox, new GridBagConstraints(0,1,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,2,2,2),0,0));
			checkBoxPanel.add(aoiZoomCheckBox, new GridBagConstraints(0,2,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,2,2,2),0,0));
			controlPanel.add(checkBoxPanel, new GridBagConstraints(1,0,1,1,0,1,GridBagConstraints.NORTH, GridBagConstraints.NONE, new Insets(5,0,5,5),0,0));
			
			JPanel addressPanel = new JPanel(new GridBagLayout());
			addressPanel.add(addressLabel, new GridBagConstraints(0,0,1,1,0,0,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(2,2,2,2),0,0));
			addressPanel.add(addressField, new GridBagConstraints(1,0,1,1,1,1,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2,2,2,2),0,0));
			addressPanel.add(addressButton, new GridBagConstraints(2,0,1,1,0,0,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(2,2,2,2),0,0));
			
			JPanel buttonPanel = new JPanel(new GridBagLayout());
			buttonPanel.add(startButton, new GridBagConstraints(0,0,1,1,0,0,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(2,2,2,2),0,0));
			buttonPanel.add(stopButton, new GridBagConstraints(1,0,1,1,0,0,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(2,2,2,2),0,0));
			buttonPanel.add(saveAsPNGButton, new GridBagConstraints(0,1,2,1,0,0,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(2,2,2,2),0,0));
			addressPanel.add(buttonPanel, new GridBagConstraints(0,1,3,1,0,0,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0,0,0,0),0,0));
			
			controlPanel.add(addressPanel, new GridBagConstraints(2,0,1,1,1,1,GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, new Insets(5,0,5,5),0,0));
			
		}
		return controlPanel;
	}
	
	
	private void saveAsPNG() {
		if (chooser == null) {
			chooser = new JFileChooser(new File("."));
		    
			String[] fileType= {"png"};
			
			FileNameExtensionFilter PNGfilter = new FileNameExtensionFilter(
		        "PNG files", fileType);
		    
		    chooser.addChoosableFileFilter(PNGfilter);
		}
		
	    
	    int returnVal = chooser.showSaveDialog(this);
	    
	    if (returnVal == JFileChooser.APPROVE_OPTION) 
	    {
			String fileNamePath = chooser.getSelectedFile().getAbsolutePath();

			boolean ret = getImageDisplayer().saveAsPNGImage(fileNamePath);
			if (ret == false)
			{
				JOptionPane.showMessageDialog(this, 
						"Error saving image.","Error", JOptionPane.ERROR_MESSAGE );
			}
		}
	}
	
	private void start() {
		ConnectionParameters p = getImageDisplayer().getConnectionParameters();
		if (p == null) {
			String remoteName = addressField.getText();
			if (remoteName == null || remoteName.trim().length() == 0) return;
			try {
				if (remoteName.startsWith("/")) remoteName = remoteName.substring(1);
				int index = remoteName.indexOf('/');
				String context = remoteName.substring(0, index);
				int index2 = remoteName.indexOf('/', index + 1);
				String server;
				if (index2 < 0) {
					server = remoteName.substring(index + 1);
				} else {
					server = remoteName.substring(index + 1, index2);
				}
				String device = null;
				if (remoteName.length() > index2 && index2 > 0) {
					device = remoteName.substring(index2+1); 
				}
				p = new ConnectionParameters("TINE", context, server, device, null, AccessMode.POLL, 1000);
				getImageDisplayer().setConnectionParameters(p);
			} catch (Exception e) {
				JOptionPane.showMessageDialog(this, "You have entered an invalid remote address.\nThe address should have the form /context/server(/device).", "Invalid address", JOptionPane.ERROR_MESSAGE);
				return;
			}
		}
		getImageDisplayer().start();
	}
	
	@Override
	public void setEnabled(boolean enabled) {
		super.setEnabled(enabled);
		getImageDisplayer().setEnabled(enabled);
		startButton.setEnabled(enabled);
		stopButton.setEnabled(enabled);
		saveAsPNGButton.setEnabled(enabled);
		overlayCheckBox.setEnabled(enabled);
		colorMapBox.setEnabled(enabled);
		colorMapLabel.setEnabled(enabled);
		aspectRatioCheckBox.setEnabled(enabled);
		aoiZoomCheckBox.setEnabled(enabled);
		addressButton.setEnabled(enabled);
		addressField.setEnabled(enabled);
		addressLabel.setEnabled(enabled);
	}
	
	
	public static void main(String[] args) {
		RunnerHelper.runComponent(new AcopVideoApplication(), 500,500);
	}
	
}
