package de.desy.acop.video;

import com.cosylab.gui.components.customizer.AbstractCustomizerPanel;

import de.desy.acop.displayers.selector.ConnectionCustomizer;
import de.desy.acop.video.displayer.ImageDisplayer;

/**
 * 
 * <code>AcopVideoCustomizer</code> is the customizer for the AcopVideo.
 * <p>
 * It includes ConnectionCustomizer and the basic AcopVideo properties:
 * <ul>
 * <li> {@link AcopVideo#setAOIZoom(boolean)}
 * <li> {@link AcopVideo#setColorMap(de.desy.video.displayer.ColorMap)}
 * <li> {@link AcopVideo#setKeepAspectRatio(boolean)}
 * <li> {@link AcopVideo#setHistogramEqualisation(boolean)}
 * <li> {@link AcopVideo#setOverlayState(de.desy.video.displayer.OverlayState)}
 * <li> {@link AcopVideo#setImageZoom(de.desy.video.displayer.ImageZoom)}
 * <li> {@link AcopVideo#setImageOrientation(de.desy.acop.video.displayer.ImageOrientation)}
 * <li> {@link AcopVideo#setGridColor(java.awt.Color)}
 * <li> {@link AcopVideo#setGridMode(de.desy.acop.video.displayer.GridMode)}
 * <li> {@link AcopVideo#setGridSize(double)}
 * <li> {@link AcopVideo#setShowDroppedFramesNumber(boolean)} 
 * <li> {@link AcopVideo#setGridVisible(boolean)}
 * </ul>
 * 
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 * @author <a href="mailto:david.melkumyan@desy.de">David Melkumyan</a>
 * 
 */
public class AcopVideoCustomizer extends AbstractCustomizerPanel {

	private static final long serialVersionUID = -7682488932083846573L;

	/** AcopVideo Properties setable by this customizer */
	public static final String[] SETTINGS = { ImageDisplayer.PROPERTY_COLOR_MAP, //
			ImageDisplayer.PROPERTY_AOI_ZOOM, //
			ImageDisplayer.PROPERTY_KEEP_ASPECT_RATIO, //
			ImageDisplayer.PROPERTY_IMAGE_ZOOM, //
			ImageDisplayer.PROPERTY_IMAGE_ORIENTATION, //
			ImageDisplayer.PROPERTY_OVERLAY_STATE, //
			ImageDisplayer.PROPERTY_HISTOGRAM_EQUALISATION, //
			ImageDisplayer.PROPERTY_HISTOGRAM_MIN, //
			ImageDisplayer.PROPERTY_HISTOGRAM_MAX, //
			ImageDisplayer.PROPERTY_GRID_COLOR, //
			ImageDisplayer.PROPERTY_GRID_MODE, //
			ImageDisplayer.PROPERTY_GRID_SIZE, //
			ImageDisplayer.PROPERTY_GRID_VISIBLE, //
			ImageDisplayer.PROPERTY_SHOW_DROPPED_FRAMES_NUMBER, //
			AcopVideo.PROPERTY_SHOW_ROI,//
			AcopVideo.PROPERTY_SHOW_THRESHOLD_ROI,//
			AcopVideo.PROPERTY_DISPLAY_CLUT_HEADER };

	/**
	 * Constructs a new AcopTableCustomizer.
	 * 
	 */
	public AcopVideoCustomizer() {
		addCustomizer("Connections", new ConnectionCustomizer());
		addCustomizerTable("Video properties", SETTINGS);
		addCustomizer("Analysis", new AnalysisCustomizer());
		setSize(600, 545);
	}
}
