/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans.
 *
 * CosyBeans is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.desy.acop.video;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.LinkedHashMap;
import java.util.Map;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import com.cosylab.application.AutoScalable;
import com.cosylab.gui.infopanel.PropertiesPanel;

import de.desy.acop.displayers.info.AcopDebugPanel;
import de.desy.acop.displayers.info.CharacteristicsPanel;
import de.desy.acop.displayers.info.ConnectionPanel;
import de.desy.acop.displayers.tools.AcopDisplayer;
import de.desy.acop.displayers.tools.AcopInfoDialog;
import de.desy.acop.displayers.tools.ConnectionParametersReceiver;
import de.desy.acop.transport.ConnectionFailed;
import de.desy.acop.transport.ConnectionParameters;
import de.desy.acop.transport.adapters.AcopTransportDataSource;
import de.desy.acop.transport.adapters.AdapterFactoryService;
import de.desy.tine.client.TLinkFactory;

/**
 * 
 * <code>AcopVideoInfoDialog</code> is the info dialog for the AcopVideo. It offers
 * the same functionality as ordinary {@link AcopInfoDialog}, however it does not
 * extend it because the AcopVideo does not implement the {@link AcopDisplayer}
 * and does not work with the {@link AcopTransportDataSource}.
 *
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 *
 */
@AutoScalable
public class AcopVideoInfoDialog extends JDialog {
	
	private static final long serialVersionUID = 513076582058739594L;
	
	private JPanel contentPanel;
	private JTabbedPane tabPane;
	//java beans properties displayer
	private PropertiesPanel propertiesPanel;
	//acop connection information displayer
	private ConnectionPanel connectionPanel;
	//acop connection characteristics displayer
	private CharacteristicsPanel characteristicsPanel;
	//acop debug information
	private AcopDebugPanel acopDebugPanel;
	//the owner of this dialog
	private AcopVideo displayer;
	private JButton closeButton;
	
	private PropertyChangeListener propertyListener;
	
	/**
	 * Constructs a new AcopVideoInfoDialog which connects to the supplied displayer.
	 * 
	 * @param displayer
	 */
	public AcopVideoInfoDialog(AcopVideo displayer) {
		initialize();
		setDisplayer(displayer);		
	}
	
	private void initialize() {
		setSize(500,400);
		this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		this.setContentPane(getContentPanel());
		this.addWindowListener(new WindowAdapter(){
			@Override
			public void windowActivated(WindowEvent e) {
				getAcopDebugPanel().setDebug(TLinkFactory.getInstance().getDebugLevel());
			}
		});
		this.addPanel(getConnectionPanel(), "Connection", 0, true);
		this.addPanel(getCharacteristicsPanel(), "Prop Info", 1, false);
		this.addPanel(getAcopDebugPanel(), "Debug", 2, false);
		this.addPanel(getPropertiesPanel(), "Bean properties", 3, false);
	}
	
	private ConnectionPanel getConnectionPanel() {
		if (connectionPanel == null) {
			connectionPanel = new ConnectionPanel();
		}
		return connectionPanel;
	}
	
	private CharacteristicsPanel getCharacteristicsPanel() {
		if (characteristicsPanel == null) {
			characteristicsPanel = new CharacteristicsPanel();
		}
		return characteristicsPanel;
	}
	
	private AcopDebugPanel getAcopDebugPanel() {
		if (acopDebugPanel == null) {
			acopDebugPanel = new AcopDebugPanel();
		}
		return acopDebugPanel;
	}
	
	private JPanel getContentPanel() {
		if (contentPanel == null) {
			contentPanel = new JPanel(new BorderLayout());
			contentPanel.add(getTabPane(), BorderLayout.CENTER);
			JPanel buttonPanel = new JPanel(new GridBagLayout());
			buttonPanel.add(getCloseButton(), new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(1,1,1,1),0,0));
			contentPanel.add(buttonPanel, BorderLayout.SOUTH);
			
		}
		return contentPanel;
	}
	
	private JButton getCloseButton() {
		if (closeButton == null) {
			closeButton = new JButton("Close");
			closeButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					dispose();					
				}
			});
		}
		return closeButton;
	}
	
	private JTabbedPane getTabPane() {
		if (tabPane == null) {
			tabPane = new JTabbedPane();
		}
		return tabPane;
	}
	
	protected PropertiesPanel getPropertiesPanel() {
		if (propertiesPanel == null) {
			propertiesPanel = new PropertiesPanel();
		}

		return propertiesPanel;
	}
	
	protected void addPanel(Component panel, String name, int position, boolean select) {
		for (int i = 0; i < getTabPane().getTabCount(); i++) {
			if (getTabPane().getTitleAt(i).equals(name)) {
				getTabPane().removeTabAt(i);
				break;
			}
		}
		getTabPane().add(panel, name, position);
		if (select) {
			getTabPane().setSelectedComponent(panel);
		}
	}
	
	protected void addPanel(Component panel, String name) {
		for (int i = 0; i < getTabPane().getTabCount(); i++) {
			if (getTabPane().getTitleAt(i).equals(name)) {
				getTabPane().removeTabAt(i);
				break;
			}
		}
		getTabPane().add(panel, name);
	}
	
	/**
	 * Hooks this panel to the specified displayer.
	 * @modified mdavid:
	 * <li> added version number characteristic
	 * <li> setTittle improved
	 *  
	 * @param displayer - AcopVideo component
	 */
	public void setDisplayer(AcopVideo displayer) {
		if (this.displayer != null && !this.displayer.equals(displayer) && propertyListener != null) {
			displayer.removePropertyChangeListener(propertyListener);
		}
		if (displayer != null && !displayer.equals(this.displayer)) {
			displayer.addPropertyChangeListener(getPropertyListener());
		}
		this.displayer = displayer;
		String title = "Info";
		Object bean = null;
		ConnectionParameters connParams = null;
		Map<String, Object> characteristics = createCharacteristics();
		if (displayer != null) {
			title += (" for " + (displayer.getName() == null ? getDisplayer().getClass().getSimpleName() : displayer.getName()));
			bean = displayer;
			connParams = displayer.getConnectionParameters();
		}
		setTitle(title);
		getPropertiesPanel().setBean(bean);
		getConnectionPanel().setConnectionParameters(connParams);
		getAcopDebugPanel().setConnectionParameters(connParams);
		getCharacteristicsPanel().setCharacteristics(characteristics);
		getAcopDebugPanel().setDebug(TLinkFactory.getInstance().getDebugLevel());
	}
	
//  commented by mdavid	
//	/**
//	 * Hooks this panel to the specified displayer.
//	 * 
//	 * @param displayer
//	 */
//	public void setDisplayer(AcopVideo displayer) {	
//		if (this.displayer != null && !this.displayer.equals(displayer)) {
//			this.displayer.removePropertyChangeListener(getPropertyListener());
//			if (displayer != null) {
//				displayer.addPropertyChangeListener(getPropertyListener());
//			}
//		} else if (this.displayer == null && displayer != null) {
//			displayer.addPropertyChangeListener(getPropertyListener());
//		}
//		this.displayer = displayer;
//		if (this.displayer != null) {
//			this.setTitle(this.displayer.getName());
//			getPropertiesPanel().setBean(displayer);
//			setTitle("Info for "+getDisplayer().getClass().getSimpleName());
//			getConnectionPanel().setConnectionParameters(getDisplayer().getConnectionParameters());
//			getAcopDebugPanel().setConnectionParameters(getDisplayer().getConnectionParameters());
//			try {
//				getCharacteristicsPanel().setCharacteristics(AdapterFactoryService.getInstance().getAdapterFactory().getCharacteristics(getDisplayer().getConnectionParameters()));
//			} catch (ConnectionFailed e) {
//				e.printStackTrace();
//			}
//		} else {
//			setTitle("Info dialog");
//			getPropertiesPanel().setBean(new Object());
//			getConnectionPanel().setConnectionParameters(null);
//			getAcopDebugPanel().setConnectionParameters(null);
//			getCharacteristicsPanel().setCharacteristics(null);
//		}
//		getAcopDebugPanel().setDebug(TLinkFactory.getInstance().getDebugLevel());
//	}
	
	private PropertyChangeListener getPropertyListener() {
		if (propertyListener == null) {
			propertyListener = new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent evt) {
					if (ConnectionParametersReceiver.CONNECTION_PARAMETERS_PROPERTY.equals(evt.getPropertyName())) {
						getConnectionPanel().setConnectionParameters(getDisplayer().getConnectionParameters());
						getAcopDebugPanel().setConnectionParameters(getDisplayer().getConnectionParameters());
//						try { // commented by mdavid
//							getCharacteristicsPanel().setCharacteristics(AdapterFactoryService.getInstance().getAdapterFactory().getCharacteristics(getDisplayer().getConnectionParameters()));
							getCharacteristicsPanel().setCharacteristics(createCharacteristics());
//							
//						} catch (ConnectionFailed e) { 
//							e.printStackTrace();
//						}
					}
				}
			};
		}
		return propertyListener;
	}
	
	/**
	 * Returns the displayer connected with this InfoPanel.
	 * 
	 * @return
	 */
	public AcopVideo getDisplayer() {
		return this.displayer;
	}	
	
	/**
	 * Creates the characteristics map for the AcopVideo component.
	 * @author mdavid
	 * @return characteristics map
	 */
	private Map<String, Object> createCharacteristics() {
		Map<String, Object> characteristics = new LinkedHashMap<>();
		characteristics.put("version", "1.1.01 / 26.03.2012");
		if (displayer != null) {
			try {
				characteristics.putAll(AdapterFactoryService.getInstance().getAdapterFactory().getCharacteristics(displayer.getConnectionParameters()));
			} catch (ConnectionFailed e) {
				e.printStackTrace();
			}
		}
		return characteristics;
	}
}
