/**
 *
 */
package de.desy.acop.video;

import static java.awt.GridBagConstraints.BOTH;
import static java.awt.GridBagConstraints.CENTER;
import static java.awt.GridBagConstraints.EAST;
import static java.awt.GridBagConstraints.HORIZONTAL;
import static java.awt.GridBagConstraints.NONE;
import static java.awt.GridBagConstraints.NORTH;
import static java.awt.GridBagConstraints.NORTHEAST;
import static java.awt.GridBagConstraints.NORTHWEST;
import static java.awt.GridBagConstraints.WEST;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.event.ItemEvent;
import java.awt.image.BufferedImage;
import java.beans.Customizer;
import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;
import java.util.function.IntConsumer;
import java.util.function.IntSupplier;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.border.LineBorder;
import javax.swing.border.TitledBorder;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.DefaultTableModel;

import com.cosylab.gui.IconCustomizer;
import com.cosylab.gui.components.NumberField;
import com.cosylab.gui.components.util.RunnerHelper;
import com.cosylab.gui.util.GUIUtilities;

import de.desy.acop.video.analysis.AnalysisImage;
import de.desy.acop.video.analysis.AnalysisMode;
import de.desy.acop.video.analysis.AnalysisType;
import de.desy.acop.video.analysis.ImageAnalysisEngine;
import de.desy.acop.video.displayer.ColorMap;
import de.desy.acop.video.timageio.TImageIO;
import de.desy.tine.types.IMAGE;
import de.desy.tine.types.IMAGE.FrameHeader;

/**
 * </code>AnalysisCustomizer</code> allows customization of analysis parameters for an </code>AcopVideo</code>.
 *
 * @author Tilen Kusterle, Cosylab
 */
public class AnalysisCustomizer extends JPanel implements Customizer {

    private class ServerSettingsModel extends DefaultTableModel {

        private static final long serialVersionUID = 1L;
        private AnalysisImage image;

        public void setAImage(AnalysisImage image) {
            this.image = image;
            fireTableDataChanged();
        }

        @Override
        public Class<?> getColumnClass(int columnIndex) {
            return String.class;
        }

        @Override
        public int getColumnCount() {
            return 2;
        }

        @Override
        public int getRowCount() {
            return 11;
        }

        @Override
        public boolean isCellEditable(int row, int column) {
            return false;
        }

        @Override
        public String getColumnName(int column) {
            switch (column) {
            case 0:
                return "Parameter";
            case 1:
                return "Value";
            default:
                return null;
            }
        }

        private final String[] ROW_NAMES = new String[] { "ROI X", "ROI Y", "ROI W", "ROI H", "Threshold ROI X",
                "Threshold ROI Y", "Threshold ROI W", "Threshold ROI H", "Threshold", "Calculate Threshold",
                "Perform Fit" };

        @Override
        public Object getValueAt(int row, int column) {
            if (column == 0) {
                return row >= 0 && row < ROW_NAMES.length ? ROW_NAMES[row] : "";
            } else if (column == 1 && image != null) {
                switch (row) {
                case 0:
                    return image.getRoiX();
                case 1:
                    return image.getRoiY();
                case 2:
                    return image.getRoiW();
                case 3:
                    return image.getRoiH();
                case 4:
                    return image.getRoi2X();
                case 5:
                    return image.getRoi2Y();
                case 6:
                    return image.getRoi2W();
                case 7:
                    return image.getRoi2H();
                case 8:
                    return image.getThreshold();
                case 9:
                    return image.isCalculateThreshold();
                case 10:
                    return image.isPerformFit();
                default:
                        return "";
                }
            }
            return null;
        }
    }

    private static final long serialVersionUID = 1L;
    private static final String NO_BACKGROUND = "No Background";
    private static final String INVALID_BACKGROUND = "Invalid Background";
    private static final String VIDEO_BACKGROUND = "from video";
    private static final int PREVIEW_DIMENSION = 100;
    private static final Dimension FIELD_DIMENSION = new Dimension(80,21);
    private static final Dimension LABEL_DIMENSION = new Dimension(200,25);
    private static final String[] FILE_TYPES = new String[] { "jpg", "jpeg", "png" };
    private Executor executor;
    private AcopVideo acopVideo;
    private JComboBox<AnalysisMode> analysisModeCombo;
    private JComboBox<AnalysisType> analysisImageTypeCombo;
    private JButton applyToServerButton;
    private JButton synchronizeButton;
    private NumberField thresholdField;
    private NumberField roiXField;
    private NumberField roiYField;
    private NumberField roiWField;
    private NumberField roiHField;
    private NumberField roi2XField;
    private NumberField roi2YField;
    private NumberField roi2WField;
    private NumberField roi2HField;
    private NumberField interferometerNumOfImagesField;
    private JCheckBox calculateThresholdCheckBox;
    private JCheckBox useThresholdCheckBox;
    private JLabel numberOfPixelsLabel;
    private JButton allButton;
    private JButton allThresholdButton;
    private JButton duplicateThresholdButton;
    private JButton currentButton;
    private JButton saveButton;
    private JButton noneButton;
    private JButton browseButton;
    private JLabel backgroundLabel;
    private JLabel fileNameLabel;
    private JFileChooser fileChooser;
    private JCheckBox displayNumericalValuesBox;
    private JCheckBox performFitCheckBox;
    private JCheckBox performSmoothCheckBox;
    private JCheckBox displayStatisticsCheckBox;
    private JTabbedPane tabPane;
    private JPanel analysisPanel;
    private JPanel backgroundPanel;
    private JPanel thresholdPanel;
    private JPanel serverSettingsPanel;
    private ServerSettingsModel serverModel;
    private File backgroundFile;
    private double[] preciseBackground;
    private int preciseBackgroundWidth;
    private IMAGE backgroundImage;
    private boolean localChanges = false;
    private String fileName;
    private int roiX;
    private int roiY;
    private int roiW;
    private int roiH;
    private int roi2X;
    private int roi2Y;
    private int roi2W;
    private int roi2H;
    private int imageW;
    private int imageH;
    private int numberOfImages;
    private double threshold;
    private int thresholdPoints;
    private boolean calculateThreshold;
    private boolean performFit;
    private boolean performSmoothing;
    private boolean appliedToServer = true;
    private boolean suppressChanges = false;
    private double lastThreshold = 0;
    private Font plainFont;
    private Font boldFont;
    private Font labelPlainFont;
    private Font labelBoldFont;
    private static final String PIXELS_USED = "Pixels Used: ";
    // TODO: mdavid - can be instantiated as local variable, instead of using as class member variable (?)
    // private TImageConverter imageConverter = new TImageConverter(); // mdavid: added

    /*
     * (non-Javadoc)
     *
     * @see java.beans.Customizer#setObject(java.lang.Object)
     */
    @Override
    public void setObject(Object bean) {
        if (!(bean instanceof AcopVideo)) {
            throw new IllegalArgumentException("Only AcopVideo can use this Customizer.");
        }
        acopVideo = (AcopVideo) bean;
        AnalysisMode am = acopVideo.getAnalysisMode();
        if (am == AnalysisMode.REMOTE_ANALYSIS) fileName = VIDEO_BACKGROUND;
        else fileName = NO_BACKGROUND;
        getAnalysisModeCombo().setSelectedItem(am);
        AnalysisType type = acopVideo.getAnalysisType();
        getAnalysisImageTypeCombo().setSelectedItem(type);
        getAnalysisImageTypeCombo().setEnabled(am != AnalysisMode.NO_ANALYSIS);
        getDisplayStatisticsCheckBox().setEnabled(type == AnalysisType.GAUSSIAN);
        getPerformFitCheckBox()
                .setSelected(type == AnalysisType.INTERFEROMETER_V || type == AnalysisType.INTERFEROMETER_H);
        getDisplayNumericalValuesBox().setSelected(acopVideo.isDisplayNumericalValues());
        AnalysisImage ai = acopVideo.getAImage();
        if (ai != null) {
            updateAnalysisData(ai);
        } else {
            resetData();
        }
        updateBackground();
        acopVideo.addPropertyChangeListener(evt -> SwingUtilities.invokeLater(() -> {
            String name = evt.getPropertyName();
            if (AcopVideo.PROPERTY_AIMAGE.equals(name)) {
                AnalysisImage aimage = (AnalysisImage) evt.getNewValue();
                if (aimage == null) {
                    resetData();
                } else {
                    updateAnalysisData(aimage);
                }
            } else if (AcopVideo.DISPLAY_STATISTISC.equals(name)) {
                getDisplayStatisticsCheckBox().setSelected(acopVideo.isDisplayStatistics());
            }
        }));
        getDisplayStatisticsCheckBox().setSelected(acopVideo.isDisplayStatistics());
        acopVideo.getImageAnalysisEngine().addPropertyChangeListener(ImageAnalysisEngine.PROPERTY_PRECISE_BACKGROUND,
                e -> updateBackground());
        acopVideo.getDisplayerManager().addPropertyChangeListener(DisplayerManager.PROPERTY_ON_SCREEN_ROI,e -> {
            Rectangle r = (Rectangle) e.getNewValue();
            if (r != null) {
                updateData(true,imageW,imageH,r.x,r.y,r.width,r.height,roi2X,roi2Y,roi2W,roi2H,threshold,
                        thresholdPoints,calculateThreshold,performFit,performSmoothing);
            }
        });
    }

    public AnalysisCustomizer() {
        initialize();
    }

    private void initialize() {
        setLayout(new GridBagLayout());
        add(getAnalysisModeCombo(),gbc(0,0,1,1,1,0,EAST,NONE,11,11,2,11));
        add(getAnalysisImageTypeCombo(),gbc(1,0,1,1,1,0,WEST,NONE,11,11,2,11));
        JPanel parametersPanel = new JPanel(new GridBagLayout());
        parametersPanel.setBorder(new TitledBorder("Analysis Parameters"));
        parametersPanel.add(getTabPane(),gbc(0,0,2,1,1,1,CENTER,BOTH,2,2,2,2));
        parametersPanel.add(new JLabel("Synchronize With Server:"),gbc(0,14,2,1,0,0,CENTER,NONE,2,2,2,2));
        JPanel p = new JPanel(new GridBagLayout());
        p.add(getApplyToServerButton(),gbc(0,15,1,1,0,0,CENTER,NONE,2,2,2,2));
        p.add(getSynchronizeButton(),gbc(1,15,1,1,0,0,CENTER,NONE,2,2,2,2));
        parametersPanel.add(p,gbc(0,15,2,1,0,0,CENTER,NONE,0,0,0,0));
        add(parametersPanel,gbc(0,3,2,1,1,1,CENTER,BOTH,4,11,4,11));
        add(getDisplayNumericalValuesBox(),gbc(0,4,2,1,0,0,CENTER,HORIZONTAL,2,11,11,11));
    }

    private JTabbedPane getTabPane() {
        if (tabPane == null) {
            tabPane = new JTabbedPane();
            tabPane.add("Parameters",getAnalysisPanel());
            tabPane.add("Background",getBackgroundPanel());
            tabPane.add("Threshold",getThresholdPanel());
            tabPane.add("Applied Settings",getServerSettingsPanel());
        }
        return tabPane;
    }

    private JPanel getServerSettingsPanel() {
        if (serverSettingsPanel == null) {
            serverSettingsPanel = new JPanel(new BorderLayout());
            JTable table = new JTable(getServerModel());
            serverSettingsPanel.add(table.getTableHeader(),BorderLayout.NORTH);
            serverSettingsPanel.add(table,BorderLayout.CENTER);
            table.setMinimumSize(new Dimension(0,0));
            serverSettingsPanel.setMinimumSize(new Dimension(0,0));
        }
        return serverSettingsPanel;
    }

    private ServerSettingsModel getServerModel() {
        if (serverModel == null) {
            serverModel = new ServerSettingsModel();
        }
        return serverModel;
    }

    private JPanel getAnalysisPanel() {
        if (analysisPanel == null) {
            analysisPanel = new JPanel(new GridBagLayout());
            int y = 0;
            analysisPanel.add(getDisplayStatisticsCheckBox(),gbc(0,y,1,1,1,0,EAST,NONE,5,5,2,2));
            analysisPanel.add(getPerformFitCheckBox(),gbc(1,y,1,1,1,0,WEST,NONE,5,2,2,5));
            analysisPanel.add(getPerformSmoothCheckBox(),gbc(0,++y,2,1,1,0,CENTER,NONE,5,5,2,5));
            JPanel numImagesPanel = new JPanel(new GridBagLayout());
            numImagesPanel.add(new JLabel("Num. of Images for Averaging (interferometer):"),
                    gbc(0,0,1,1,0,0,CENTER,NONE,2,5,2,2));
            numImagesPanel.add(getInterferometerNumOfImagesField(),gbc(1,0,1,1,0,0,CENTER,NONE,2,2,2,2));
            analysisPanel.add(numImagesPanel,gbc(0,++y,2,1,1,0,CENTER,NONE,0,0,0,0));
            analysisPanel.add(new JLabel("Region of Interest Parameters [pixels]:"),
                    gbc(0,++y,2,1,1,0,CENTER,NONE,2,5,2,10));
            analysisPanel.add(new JLabel("Start X:"),gbc(0,++y,1,1,1,0,EAST,NONE,2,5,2,2));
            analysisPanel.add(getRoiXField(),gbc(1,y,1,1,1,0,WEST,NONE,2,2,2,2));
            analysisPanel.add(new JLabel("Start Y:"),gbc(0,++y,1,1,1,0,EAST,NONE,2,5,2,2));
            analysisPanel.add(getRoiYField(),gbc(1,y,1,1,1,0,WEST,NONE,2,2,2,2));
            analysisPanel.add(new JLabel("Width:"),gbc(0,++y,1,1,1,0,EAST,NONE,2,5,2,2));
            analysisPanel.add(getRoiWField(),gbc(1,y,1,1,1,0,WEST,NONE,2,2,2,2));
            analysisPanel.add(new JLabel("Height:"),gbc(0,++y,1,1,1,0,EAST,NONE,2,5,2,2));
            analysisPanel.add(getRoiHField(),gbc(1,y,1,1,1,0,NORTHWEST,NONE,2,2,2,2));
            analysisPanel.add(getAllButton(),gbc(0,++y,2,1,1,1,NORTH,NONE,2,2,2,2));
        }
        return analysisPanel;
    }

    private JPanel getThresholdPanel() {
        if (thresholdPanel == null) {
            thresholdPanel = new JPanel(new GridBagLayout());
            JPanel p1 = new JPanel(new GridBagLayout());
            p1.add(new JLabel("Threshold:"),gbc(2,0,1,1,0,1,WEST,NONE,5,2,2,5));
            p1.add(getThresholdField(),gbc(3,0,1,1,0,1,WEST,NONE,5,2,2,5));
            p1.add(getUseTresholdCheckBox(),gbc(2,1,2,1,0,1,WEST,NONE,2,2,0,5));
            p1.add(getCalculateTresholdCheckBox(),gbc(2,2,2,1,0,1,WEST,NONE,2,2,0,5));
            p1.add(getNumberOfPixelsLabel(),gbc(2,3,2,1,0,1,WEST,NONE,2,23,4,5));
            p1.add(new JLabel("Threshold Calculation Region [pixels]:"),gbc(2,4,2,1,0,1,CENTER,NONE,2,2,2,5));
            p1.add(new JLabel("Start X:"),gbc(2,5,1,1,0,1,WEST,NONE,2,2,2,2));
            p1.add(getRoi2XField(),gbc(3,5,1,1,0,1,WEST,NONE,2,2,2,5));
            p1.add(new JLabel("Start Y:"),gbc(2,6,1,1,0,1,WEST,NONE,2,2,2,2));
            p1.add(getRoi2YField(),gbc(3,6,1,1,0,1,WEST,NONE,2,2,2,5));
            p1.add(new JLabel("Width:"),gbc(2,7,1,1,0,1,WEST,NONE,2,2,2,2));
            p1.add(getRoi2WField(),gbc(3,7,1,1,0,1,WEST,NONE,2,2,2,5));
            p1.add(new JLabel("Height:"),gbc(2,8,1,1,0,1,NORTHWEST,NONE,2,2,2,2));
            p1.add(getRoi2HField(),gbc(3,8,1,1,0,1,NORTHWEST,NONE,2,2,2,5));
            JPanel p = new JPanel(new GridBagLayout());
            p.add(getAllThresholdButton(),gbc(0,0,1,1,0,0,NORTHEAST,NONE,2,2,2,2));
            p.add(getDuplicateThresholdButton(),gbc(1,0,1,1,0,0,NORTHWEST,NONE,2,2,2,2));
            thresholdPanel.add(p1,gbc(0,0,1,1,0,1,NORTH,BOTH,2,2,2,2));
            thresholdPanel.add(p,gbc(0,1,1,1,0,0,NORTH,NONE,2,2,2,2));
            // thresholdPanel.add(new JLabel("Threshold:"), new
            // GridBagConstraints(0,0,1,1,0,0,WEST,NONE,new Insets(5,2,2,5),0,0));
            // thresholdPanel.add(getThresholdField(), new
            // GridBagConstraints(1,0,1,1,0,0,WEST,NONE,new Insets(5,2,2,2),0,0));
            // thresholdPanel.add(getCalculateTresholdCheckBox(), new
            // GridBagConstraints(0,1,2,1,0,0,WEST,NONE,new Insets(2,2,0,2),0,0));
            // thresholdPanel.add(getNumberOfPixelsLabel(), new
            // GridBagConstraints(0,2,2,1,0,0,WEST,NONE,new
            // Insets(2,23,4,2),0,0));
            //
            // thresholdPanel.add(new JLabel("Threshold Calculation Region [pixels]:"), new
            // GridBagConstraints(0,3,2,1,0,0,CENTER,NONE,new
            // Insets(2,2,2,2),0,0));
            //
            // thresholdPanel.add(new JLabel("Start X:"), new
            // GridBagConstraints(0,4,1,1,0,0,WEST,NONE,new Insets(2,2,2,2),0,0));
            // thresholdPanel.add(getRoi2XField(), new
            // GridBagConstraints(1,4,1,1,0,0,WEST,NONE,new Insets(2,2,2,2),0,0));
            // thresholdPanel.add(new JLabel("Start Y:"), new
            // GridBagConstraints(0,5,1,1,0,0,WEST,NONE,new Insets(2,2,2,2),0,0));
            // thresholdPanel.add(getRoi2YField(), new
            // GridBagConstraints(1,5,1,1,0,0,WEST,NONE,new Insets(2,2,2,2),0,0));
            // thresholdPanel.add(new JLabel("Width:"), new
            // GridBagConstraints(0,6,1,1,0,0,WEST,NONE,new Insets(2,2,2,2),0,0));
            // thresholdPanel.add(getRoi2WField(), new
            // GridBagConstraints(1,6,1,1,0,0,WEST,NONE,new Insets(2,2,2,2),0,0));
            // thresholdPanel.add(new JLabel("Height:"), new
            // GridBagConstraints(0,7,1,1,0,0,NORTHWEST,NONE,new
            // Insets(2,2,2,2),0,0));
            // thresholdPanel.add(getRoi2HField(), new
            // GridBagConstraints(1,7,1,1,0,1,NORTHWEST,NONE,new
            // Insets(2,2,2,2),0,0));
        }
        return thresholdPanel;
    }

    private JLabel getNumberOfPixelsLabel() {
        if (numberOfPixelsLabel == null) {
            numberOfPixelsLabel = new JLabel(PIXELS_USED + (0));
        }
        return numberOfPixelsLabel;
    }

    private JPanel getBackgroundPanel() {
        if (backgroundPanel == null) {
            backgroundPanel = new JPanel(new GridBagLayout());
            backgroundPanel.add(new JLabel("Background Image:"),gbc(0,8,2,1,0,0,CENTER,NONE,4,4,4,4));
            backgroundPanel.add(getFileNameLabel(),gbc(0,9,2,1,0,0,CENTER,HORIZONTAL,4,4,8,4));
            backgroundPanel.add(getBackgroundLabel(),gbc(0,10,1,4,1,1,CENTER,BOTH,4,4,4,4));
            backgroundPanel.add(getCurrentButton(),gbc(1,10,1,1,0,0,NORTH,HORIZONTAL,4,4,4,4));
            backgroundPanel.add(getNoneButton(),gbc(1,11,1,1,0,0,NORTH,HORIZONTAL,4,4,4,4));
            backgroundPanel.add(getBrowseButton(),gbc(1,12,1,1,0,0,NORTH,HORIZONTAL,4,4,4,4));
            backgroundPanel.add(getSaveButton(),gbc(1,13,1,1,0,1,NORTH,HORIZONTAL,4,4,4,4));
        }
        return backgroundPanel;
    }

    private JComboBox<AnalysisMode> getAnalysisModeCombo() {
        if (analysisModeCombo == null) {
            analysisModeCombo = new JComboBox<>(AnalysisMode.values());
            analysisModeCombo.addActionListener(e -> {
                AnalysisMode mode = (AnalysisMode) analysisModeCombo.getSelectedItem();
                setAnalysisMode(mode);
                if (mode == AnalysisMode.LOCAL_ANALYSIS) {
                    if (backgroundFile != null) updateFileNameLabel(backgroundFile.getAbsolutePath());
                    else if (fileName == VIDEO_BACKGROUND) updateFileNameLabel(VIDEO_BACKGROUND);
                    else updateFileNameLabel(NO_BACKGROUND);
                } else if (mode == AnalysisMode.REMOTE_ANALYSIS) updateFileNameLabel(VIDEO_BACKGROUND);
                else updateFileNameLabel(NO_BACKGROUND);
                getAnalysisImageTypeCombo().setEnabled(mode != AnalysisMode.NO_ANALYSIS);

                AnalysisType type = (AnalysisType) analysisImageTypeCombo.getSelectedItem();
                getInterferometerNumOfImagesField()
                        .setEnabled(type != AnalysisType.GAUSSIAN && mode != AnalysisMode.NO_ANALYSIS);
            });
        }
        return analysisModeCombo;
    }

    private JComboBox<AnalysisType> getAnalysisImageTypeCombo() {
        if (analysisImageTypeCombo == null) {
            analysisImageTypeCombo = new JComboBox<>(AnalysisType.values());
            analysisImageTypeCombo.addActionListener(e -> {
                AnalysisType mode = (AnalysisType) analysisImageTypeCombo.getSelectedItem();
                acopVideo.setAnalysisType(mode);
                AnalysisMode amode = (AnalysisMode) getAnalysisModeCombo().getSelectedItem();
                getDisplayStatisticsCheckBox().setEnabled(mode == AnalysisType.GAUSSIAN);
                getInterferometerNumOfImagesField()
                        .setEnabled(mode != AnalysisType.GAUSSIAN && amode != AnalysisMode.NO_ANALYSIS);
                if (mode != AnalysisType.GAUSSIAN) {
                    getPerformFitCheckBox().setSelected(true);
                }
            });
        }
        return analysisImageTypeCombo;
    }

    private NumberField getInterferometerNumOfImagesField() {
        if (interferometerNumOfImagesField == null) {
            interferometerNumOfImagesField = new NumberField(Long.valueOf(5));
            interferometerNumOfImagesField.setNumberType(Long.class);
            interferometerNumOfImagesField.setFormat("%d");
            GUIUtilities.setFixedSize(interferometerNumOfImagesField,FIELD_DIMENSION);
            interferometerNumOfImagesField.addPropertyChangeListener("value",evt -> {
                if (suppressChanges) return;
                int value = ((Number) evt.getNewValue()).intValue();
                if (numberOfImages == value) {
                    if (localChanges) interferometerNumOfImagesField.setFont(getPlainFont());
                    return;
                }
                if (localChanges) {
                    interferometerNumOfImagesField.setFont(getBoldFont());
                    appliedToServer = false;
                } else {
                    numberOfImages = value;
                    acopVideo.getImageAnalysisEngine().setInterferometerNumOfImages(numberOfImages);
                }
            });
            interferometerNumOfImagesField.setEnabled(false);
        }
        return interferometerNumOfImagesField;
    }

    private JButton getApplyToServerButton() {
        if (applyToServerButton == null) {
            applyToServerButton = new JButton("Apply");
            applyToServerButton.setToolTipText("Apply Parameters to Server");
            applyToServerButton.setEnabled(false);
            applyToServerButton.addActionListener(e -> applyToServer());
        }
        return applyToServerButton;
    }

    private JButton getSynchronizeButton() {
        if (synchronizeButton == null) {
            synchronizeButton = new JButton("Reload");
            synchronizeButton.setToolTipText("Reload Parameters from Server");
            synchronizeButton.setEnabled(false);
            synchronizeButton.addActionListener(e -> {
                appliedToServer = true;
                updateData(true,imageW,imageH,roiX,roiY,roiW,roiH,roi2X,roi2Y,roi2W,roi2H,threshold,thresholdPoints,
                        calculateThreshold,performFit,performSmoothing);
            });
        }
        return synchronizeButton;
    }

    private NumberField getThresholdField() {
        if (thresholdField == null) {
            thresholdField = new NumberField();
            thresholdField.setNumberType(Double.class);
            thresholdField.setFormat("%3.2f");
            GUIUtilities.setFixedSize(thresholdField,FIELD_DIMENSION);
            thresholdField.addPropertyChangeListener("value",evt -> {
                if (suppressChanges) return;
                final double value = (Double) evt.getNewValue();
                if (!Double.isNaN(value)) lastThreshold = value;
                if (threshold == value) {
                    if (localChanges) thresholdField.setFont(getPlainFont());
                    return;
                }
                if (localChanges) {
                    thresholdField.setFont(getBoldFont());
                    appliedToServer = false;
                } else {
                    acopVideo.getImageAnalysisEngine().setThreshold(value);
                }
            });
            suppressChanges = true;
            thresholdField.setValue(Double.NaN);
            suppressChanges = false;
        }
        return thresholdField;
    }

    private JCheckBox getCalculateTresholdCheckBox() {
        if (calculateThresholdCheckBox == null) {
            calculateThresholdCheckBox = new JCheckBox("Automatic Threshold Calculation");
            calculateThresholdCheckBox
                    .setToolTipText("Automatic Threshold Calculation (Average Pixel Value in [Threshold ROI - ROI])");
            calculateThresholdCheckBox.addItemListener(e -> {
                if (suppressChanges) return;
                boolean value = e.getStateChange() == ItemEvent.SELECTED;
                if (localChanges) {
                    calculateThresholdCheckBox.setFont(getLabelBoldFont());
                    appliedToServer = false;
                } else {
                    acopVideo.getImageAnalysisEngine().setCalculateThreshold(value);
                }
                getThresholdField().setEditable(!value && getUseTresholdCheckBox().isSelected());
                getUseTresholdCheckBox().setEnabled(!value);
            });
        }
        return calculateThresholdCheckBox;
    }

    private JCheckBox getUseTresholdCheckBox() {
        if (useThresholdCheckBox == null) {
            useThresholdCheckBox = new JCheckBox("Apply Threshold");
            useThresholdCheckBox.setToolTipText("Select Whether Set Threshold is Used in Calculation or Not");
            useThresholdCheckBox.addItemListener(e -> {
                if (suppressChanges) return;
                boolean value = e.getStateChange() == ItemEvent.SELECTED;
                if (value) {
                    acopVideo.getImageAnalysisEngine().setThreshold(lastThreshold);
                } else {
                    acopVideo.getImageAnalysisEngine().setThreshold(Double.NaN);
                }
                getThresholdField().setEditable(!value);
            });
            useThresholdCheckBox.setEnabled(getCalculateTresholdCheckBox().isSelected());
        }
        return useThresholdCheckBox;
    }

    private JCheckBox getPerformFitCheckBox() {
        if (performFitCheckBox == null) {
            performFitCheckBox = new JCheckBox("Perform Fitting");
            performFitCheckBox.setToolTipText("Perform Exponential + Linear Function Fit on Side View Data");
            performFitCheckBox.addItemListener(e -> {
                if (suppressChanges) return;
                if (localChanges) {
                    performFitCheckBox.setFont(getLabelBoldFont());
                    appliedToServer = false;
                } else {
                    acopVideo.getImageAnalysisEngine().setPerformFit(e.getStateChange() == ItemEvent.SELECTED);
                }
            });
        }
        return performFitCheckBox;
    }

    private JCheckBox getPerformSmoothCheckBox() {
        if (performSmoothCheckBox == null) {
            performSmoothCheckBox = new JCheckBox("Perform Smoothing");
            performSmoothCheckBox.setToolTipText("Smooth Side View Data by Averaging Values With Neighbours");
            performSmoothCheckBox.addItemListener(e -> {
                if (suppressChanges) return;
                if (localChanges) {
                    performSmoothCheckBox.setFont(getLabelBoldFont());
                    appliedToServer = false;
                } else {
                    acopVideo.getImageAnalysisEngine().setPerformSmoothing(e.getStateChange() == ItemEvent.SELECTED);
                }
            });
        }
        return performSmoothCheckBox;
    }

    private JCheckBox getDisplayStatisticsCheckBox() {
        if (displayStatisticsCheckBox == null) {
            displayStatisticsCheckBox = new JCheckBox("Display Statisics");
            displayStatisticsCheckBox.setToolTipText("Display Statistical Analysis Results");
            displayStatisticsCheckBox.addItemListener(e -> {
                if (suppressChanges) return;
                acopVideo.setDisplayStatistics(e.getStateChange() == ItemEvent.SELECTED);
            });
        }
        return displayStatisticsCheckBox;
    }

    private NumberField getField(boolean sendData2, IntSupplier valueSupplier, IntConsumer consumer) {
        NumberField field = new NumberField();
        field.setNumberType(Integer.class);
        field.setFormat("%d");
        GUIUtilities.setFixedSize(field,FIELD_DIMENSION);
        field.setMinimum(0);
        field.addPropertyChangeListener("value",evt -> {
            if (suppressChanges) return;
            int value = ((Number) evt.getNewValue()).intValue();
            if (valueSupplier.getAsInt() == value) {
                if (localChanges) field.setFont(getPlainFont());
                return;
            }
            if (localChanges) {
                field.setFont(getBoldFont());
                appliedToServer = false;
            } else {
                consumer.accept(value);
                if (sendData2) {
                    setData2();
                } else {
                    setData();
                }
            }
        });
        return field;
    }

    private NumberField getRoiXField() {
        if (roiXField == null) {
            roiXField = getField(false,() -> roiX,v -> roiX = v);
        }
        return roiXField;
    }

    private NumberField getRoiYField() {
        if (roiYField == null) {
            roiYField = getField(false,() -> roiY,v -> roiY = v);
        }
        return roiYField;
    }

    private NumberField getRoiWField() {
        if (roiWField == null) {
            roiWField = getField(false,() -> roiW,v -> roiW = v);
        }
        return roiWField;
    }

    private NumberField getRoiHField() {
        if (roiHField == null) {
            roiHField = getField(false,() -> roiH,v -> roiH = v);
        }
        return roiHField;
    }

    private NumberField getRoi2XField() {
        if (roi2XField == null) {
            roi2XField = getField(true,() -> roi2X,v -> roi2X = v);
        }
        return roi2XField;
    }

    private NumberField getRoi2YField() {
        if (roi2YField == null) {
            roi2YField = getField(true,() -> roi2Y,v -> roi2Y = v);
        }
        return roi2YField;
    }

    private NumberField getRoi2WField() {
        if (roi2WField == null) {
            roi2WField = getField(true,() -> roi2W,v -> roi2W = v);
        }
        return roi2WField;
    }

    private NumberField getRoi2HField() {
        if (roi2HField == null) {
            roi2HField = getField(true,() -> roi2H,v -> roi2H = v);
        }
        return roi2HField;
    }

    private JButton getAllButton() {
        if (allButton == null) {
            allButton = new JButton("Select Entire Image");
            allButton.setToolTipText("Select Entire Image for Region of Interest");
            allButton.addActionListener(e -> {
                updateData(localChanges,imageW,imageH,0,0,imageW,imageH,roi2X,roi2Y,roi2W,roi2H,threshold,
                        thresholdPoints,calculateThreshold,performFit,performSmoothing);
                if (!localChanges) setData();
            });
        }
        return allButton;
    }

    private JButton getAllThresholdButton() {
        if (allThresholdButton == null) {
            allThresholdButton = new JButton("Select Entire Image");
            allThresholdButton.setToolTipText("Threshold Region Is Whole Image");
            allThresholdButton.addActionListener(e -> {
                updateData(localChanges,imageW,imageH,roiX,roiY,roiW,roiH,0,0,imageW,imageH,threshold,thresholdPoints,
                        calculateThreshold,performFit,performSmoothing);
                if (!localChanges) setData2();
            });
        }
        return allThresholdButton;
    }

    private JButton getDuplicateThresholdButton() {
        if (duplicateThresholdButton == null) {
            duplicateThresholdButton = new JButton("Copy Values from ROI");
            duplicateThresholdButton.setToolTipText("Threshold Region Is the Same As ROI");
            duplicateThresholdButton.addActionListener(e -> {
                updateData(localChanges,imageW,imageH,roiX,roiY,roiW,roiH,roiX,roiY,roiW,roiH,threshold,thresholdPoints,
                        calculateThreshold,performFit,performSmoothing);
                if (!localChanges) setData2();
            });
        }
        return duplicateThresholdButton;
    }

    private JButton getCurrentButton() {
        if (currentButton == null) {
            currentButton = new JButton("Use Current");
            currentButton.addActionListener(e -> {
                executeTask(() -> {
                    IMAGE im = acopVideo.getImageAnalysisEngine().getImage();
                    if (im == null) return;
                    im = im.clone();
                    fileName = VIDEO_BACKGROUND;
                    setBackgroundImage(im,false);
                    backgroundFile = null;
                });
            });
        }
        return currentButton;
    }

    private JButton getBrowseButton() {
        if (browseButton == null) {
            browseButton = new JButton("Load...");
            browseButton.addActionListener(e -> {
                int option = getFileChooser().showOpenDialog(AnalysisCustomizer.this);
                if (option == JFileChooser.APPROVE_OPTION) {
                    final File f = getFileChooser().getSelectedFile();
                    executeTask(() -> {
                        try {
                            fileName = f.getAbsolutePath();
                            setBackgroundImage(TImageIO.read(f).toIMAGE(false),false);
                            backgroundFile = f;
                        } catch (IOException e1) {
                            e1.printStackTrace();
                        }
                    });
                }
            });
        }
        return browseButton;
    }

    private JButton getSaveButton() {
        if (saveButton == null) {
            saveButton = new JButton("Save...");
            saveButton.addActionListener(e -> {
                int option = getFileChooser().showSaveDialog(AnalysisCustomizer.this);
                if (option == JFileChooser.APPROVE_OPTION) {
                    final File f = getFileChooser().getSelectedFile();
                    executeTask(() -> {
                        if (backgroundImage == null) return;
                        BufferedImage bi = ImageAnalysisEngine.toBufferedImage(backgroundImage,acopVideo.getColorMap());
                        List<String> ext = Arrays.asList(FILE_TYPES);
                        int dotIndex = f.getName().lastIndexOf('.');
                        String extension = FILE_TYPES[0];
                        if (dotIndex > 0 && dotIndex < f.getName().length() - 2) {
                            String ex = f.getName().substring(dotIndex + 1);
                            if (ext.contains(ex)) extension = ex;
                        }
                        try {
                            ImageIO.write(bi,extension,f);
                        } catch (IOException e1) {
                            e1.printStackTrace();
                        }
                    });
                }
            });
            saveButton.setEnabled(false);
        }
        return saveButton;
    }

    private JButton getNoneButton() {
        if (noneButton == null) {
            noneButton = new JButton("Use None");
            noneButton.addActionListener(e -> {
                fileName = NO_BACKGROUND;
                setBackgroundImage((double[]) null,0,null,false);
                backgroundFile = null;
            });
        }
        return noneButton;
    }

    private JLabel getBackgroundLabel() {
        if (backgroundLabel == null) {
            backgroundLabel = new JLabel();
            backgroundLabel.setBorder(new LineBorder(Color.BLACK));
            backgroundLabel.setHorizontalAlignment(SwingConstants.CENTER);
            backgroundLabel.setText(NO_BACKGROUND);
            backgroundLabel.setPreferredSize(new Dimension(PREVIEW_DIMENSION,PREVIEW_DIMENSION));
        }
        return backgroundLabel;
    }

    private JLabel getFileNameLabel() {
        if (fileNameLabel == null) {
            fileNameLabel = new JLabel("File: N/A");
            fileNameLabel.setHorizontalAlignment(SwingConstants.CENTER);
            GUIUtilities.setFixedSize(fileNameLabel,LABEL_DIMENSION);
        }
        return fileNameLabel;
    }

    private JFileChooser getFileChooser() {
        if (fileChooser == null) {
            fileChooser = new JFileChooser();
            fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
            fileChooser.setMultiSelectionEnabled(false);
            fileChooser.setFileFilter(new FileFilter() {

                List<String> ext = Arrays.asList(FILE_TYPES);

                @Override
                public boolean accept(File f) {
                    if (f.isDirectory()) return true;
                    String name = f.getName();
                    int index = name.lastIndexOf('.');
                    if (index > 0 && index < name.length() - 2) {
                        name = name.substring(index + 1);
                        if (ext.contains(name)) return true;
                    }
                    return false;
                }

                @Override
                public String getDescription() {
                    String s = "Images (";
                    for (Iterator<String> iterator = ext.iterator(); iterator.hasNext();) {
                        String e = iterator.next();
                        s += e;
                        if (iterator.hasNext()) s += ", ";
                    }
                    s += ")";
                    return s;
                }
            });
        }
        return fileChooser;
    }

    private JCheckBox getDisplayNumericalValuesBox() {
        if (displayNumericalValuesBox == null) {
            displayNumericalValuesBox = new JCheckBox("Display Values");
            displayNumericalValuesBox.addActionListener(
                    e -> acopVideo.setDisplayNumericalValues(displayNumericalValuesBox.isSelected()));
        }
        return displayNumericalValuesBox;
    }

    private void setAnalysisMode(AnalysisMode mode) {
        getApplyToServerButton().setEnabled(mode == AnalysisMode.REMOTE_ANALYSIS);
        getSynchronizeButton().setEnabled(mode == AnalysisMode.REMOTE_ANALYSIS);
        boolean enable = mode != AnalysisMode.NO_ANALYSIS;
        setSettingsEnabled(enable);
        localChanges = mode == AnalysisMode.REMOTE_ANALYSIS;
        if (!localChanges) {
            Font plainFont = getPlainFont();
            Arrays.asList(getRoiXField(),getRoiYField(),getRoiWField(),getRoiHField(),getRoi2XField(),getRoi2YField(),
                    getRoi2WField(),getRoi2HField(),getThresholdField()).forEach(c -> c.setFont(plainFont));
            Arrays.asList(getCalculateTresholdCheckBox(),getPerformFitCheckBox(),getPerformSmoothCheckBox())
                    .forEach(c -> c.setFont(getLabelPlainFont()));
        }
        acopVideo.setAnalysisMode(mode);
        appliedToServer = true;
    }

    private void setSettingsEnabled(boolean enable) {
        Arrays.asList(getThresholdField(),getCalculateTresholdCheckBox(),getPerformFitCheckBox(),
                getPerformSmoothCheckBox(),getDisplayStatisticsCheckBox(),getRoiXField(),getRoiYField(),getRoiWField(),
                getRoiHField(),getRoi2XField(),getRoi2YField(),getRoi2WField(),getRoi2HField(),getAllButton(),
                getAllThresholdButton(),getDuplicateThresholdButton(),getCurrentButton(),getBrowseButton(),
                getSaveButton(),getNoneButton(),getBackgroundLabel()).forEach(c -> c.setEnabled(enable));
        acopVideo.setCanChangeSettings(enable);
    }

    private void applyToServer() {
        performFit = getPerformFitCheckBox().isSelected();
        performSmoothing = getPerformSmoothCheckBox().isSelected();
        calculateThreshold = getCalculateTresholdCheckBox().isSelected();
        threshold = getThresholdField().getValue().doubleValue();
        roiX = getRoiXField().getValue().intValue();
        roiY = getRoiYField().getValue().intValue();
        roiW = getRoiWField().getValue().intValue();
        roiH = getRoiHField().getValue().intValue();
        acopVideo.getImageAnalysisEngine().setAnalysisParameters(threshold,roiX,roiY,roiW,roiH,
                getRoi2XField().getValue().intValue(),getRoi2YField().getValue().intValue(),
                getRoi2WField().getValue().intValue(),getRoi2HField().getValue().intValue(),calculateThreshold,
                performFit,performSmoothing);
        appliedToServer = true;
    }

    private boolean updateData(boolean localChanges, int imageWidth, int imageHeight, int roiX, int roiY, int roiW,
            int roiH, int roi2X, int roi2Y, int roi2W, int roi2H, double threshold, int thresholdPoints,
            boolean calculateThreshold, boolean performFit, boolean performSmooth) {
        if (!appliedToServer && !localChanges) return false;
        suppressChanges = true;
        if (!localChanges) {
            this.roiX = roiX;
            this.roiY = roiY;
            this.roiW = roiW;
            this.roiH = roiH;
            this.roi2X = roi2X;
            this.roi2Y = roi2Y;
            this.roi2W = roi2W;
            this.roi2H = roi2H;
            this.threshold = threshold;
            this.calculateThreshold = calculateThreshold;
            this.performFit = performFit;
            this.performSmoothing = performSmooth;
        }
        this.thresholdPoints = thresholdPoints;
        this.imageW = imageWidth;
        this.imageH = imageHeight;
        getCalculateTresholdCheckBox().setSelected(calculateThreshold);
        getThresholdField().setEditable(!calculateThreshold && getUseTresholdCheckBox().isSelected());
        getPerformFitCheckBox().setSelected(performFit);
        getPerformSmoothCheckBox().setSelected(performSmooth);
        if (getUseTresholdCheckBox().isSelected()) {
            getThresholdField().setValue(threshold);
        } else {
            getThresholdField().setValue(Double.NaN);
        }
        if (!calculateThreshold) {
            getUseTresholdCheckBox().setEnabled(true);
            if (Double.isNaN(threshold)) {
                getUseTresholdCheckBox().setSelected(false);
                getThresholdField().setValue(Double.NaN);
            } else {
                getUseTresholdCheckBox().setSelected(true);
                getThresholdField().setValue(threshold);
            }
        } else {
            getUseTresholdCheckBox().setEnabled(false);
        }
        getRoiXField().setMaximum(imageWidth);
        getRoiYField().setMaximum(imageHeight);
        getRoiWField().setMaximum(imageWidth - roiX);
        getRoiHField().setMaximum(imageHeight - roiY);
        getRoi2XField().setMaximum(imageWidth);
        getRoi2YField().setMaximum(imageHeight);
        getRoi2WField().setMaximum(imageWidth - roi2X);
        getRoi2HField().setMaximum(imageHeight - roi2Y);
        getRoiXField().setValue(roiX);
        getRoiYField().setValue(roiY);
        getRoiWField().setValue(roiW);
        getRoiHField().setValue(roiH);
        getRoi2XField().setValue(roi2X);
        getRoi2YField().setValue(roi2Y);
        getRoi2WField().setValue(roi2W);
        getRoi2HField().setValue(roi2H);
        getNumberOfPixelsLabel().setText(PIXELS_USED + (thresholdPoints));
        Font font = getPlainFont();
        Arrays.asList(getRoiXField(),getRoiYField(),getRoiWField(),getRoiHField(),getRoi2XField(),getRoi2YField(),
                getRoi2WField(),getRoi2HField(),getThresholdField()).forEach(c -> c.setFont(font));
        Arrays.asList(getCalculateTresholdCheckBox(),getPerformFitCheckBox(),getPerformSmoothCheckBox())
                .forEach(c -> c.setFont(getLabelPlainFont()));
        suppressChanges = false;
        return true;
    }

    private void resetData() {
        suppressChanges = true;
        this.roiX = 0;
        this.roiY = 0;
        this.roiW = 0;
        this.roiH = 0;
        this.roi2X = 0;
        this.roi2Y = 0;
        this.roi2W = 0;
        this.roi2H = 0;
        this.imageW = 0;
        this.imageH = 0;
        this.threshold = 0;
        this.calculateThreshold = true;
        getThresholdField().setValue(Double.NaN);
        getCalculateTresholdCheckBox().setSelected(true);
        getPerformFitCheckBox().setSelected(false);
        getPerformSmoothCheckBox().setSelected(true);
        getThresholdField().setEditable(false);
        getUseTresholdCheckBox().setSelected(false);
        Arrays.asList(getRoiXField(),getRoiYField(),getRoiWField(),getRoiHField()).forEach(c -> c.setMaximum(0));
        Arrays.asList(getRoiXField(),getRoiYField(),getRoiWField(),getRoiHField()).forEach(c -> c.setValue(0));
        Arrays.asList(getRoi2XField(),getRoi2YField(),getRoi2WField(),getRoi2HField()).forEach(c -> c.setMaximum(0));
        Arrays.asList(getRoi2XField(),getRoi2YField(),getRoi2WField(),getRoi2HField()).forEach(c -> c.setValue(0));
        getNumberOfPixelsLabel().setText(PIXELS_USED + (0));
        suppressChanges = false;
        appliedToServer = true;
    }

    private synchronized void setBackgroundImage(IMAGE img, boolean local) {
        if (img != null) {
            FrameHeader hdr = img.getFrameHeader();
            int w = hdr.aoiWidth > 0 ? hdr.aoiWidth : hdr.sourceWidth;
            BufferedImage image = ImageAnalysisEngine.toBufferedImage(img,ColorMap.NONE);
            backgroundImage = img;
            preciseBackground = ImageAnalysisEngine.imageToDoubleValues(image,
                    img.getFrameHeader().effectiveBitsPerPixel,
                    acopVideo.getImageAnalysisEngine().getDecoder());
            preciseBackgroundWidth = w;
        } else {
            backgroundImage = null;
            preciseBackground = null;
            preciseBackgroundWidth = 0;
        }
        processBackground(img,local);
    }

    private synchronized void setBackgroundImage(double[] image, int width, IMAGE orgImage, boolean local) {
        preciseBackground = image;
        backgroundImage = ImageAnalysisEngine.toImage(preciseBackground,width,orgImage);
        preciseBackgroundWidth = width;
        processBackground(orgImage,local);
    }

    private void processBackground(IMAGE orgImage, boolean local) {
        updateFileNameLabel(fileName);
        if (preciseBackground == null || backgroundImage == null) {
            getBackgroundLabel().setText(NO_BACKGROUND);
            getBackgroundLabel().setIcon(null);
        } else {
            BufferedImage bi = ImageAnalysisEngine.toBufferedImage(preciseBackground,preciseBackgroundWidth,orgImage);
            if (bi.getWidth() != 0 || bi.getHeight() != 0) {
                getSaveButton().setEnabled(true);
                int w1 = getBackgroundLabel().getWidth();
                int h1 = getBackgroundLabel().getHeight();
                h1 = Math.max(h1,100);
                w1 = Math.max(w1,100);
                ImageIcon icon = IconCustomizer.getScaledImage(bi,h1,w1);
                getBackgroundLabel().setIcon(icon);
                getBackgroundLabel().setText(null);
            } else {
                getBackgroundLabel().setText(INVALID_BACKGROUND);
                getBackgroundLabel().setIcon(null);
            }
        }
        if (!local) acopVideo.getImageAnalysisEngine().setPreciseBackground(preciseBackground,preciseBackgroundWidth);
    }

    private void updateFileNameLabel(String text) {
        getFileNameLabel().setText(text);
        getFileNameLabel().setToolTipText(text);
    }

    private Font getPlainFont() {
        if (plainFont == null) {
            plainFont = getThresholdField().getFont();
        }
        return plainFont;
    }

    private Font getLabelPlainFont() {
        if (labelPlainFont == null) {
            labelPlainFont = getNumberOfPixelsLabel().getFont();
        }
        return labelPlainFont;
    }

    private Font getBoldFont() {
        if (boldFont == null) {
            boldFont = getPlainFont().deriveFont(Font.BOLD);
        }
        return boldFont;
    }

    private Font getLabelBoldFont() {
        if (labelBoldFont == null) {
            labelBoldFont = getLabelPlainFont().deriveFont(Font.BOLD);
        }
        return labelBoldFont;
    }

    private void setData() {
        acopVideo.getImageAnalysisEngine().setRoi(roiX,roiY,roiW,roiH);
    }

    private void setData2() {
        acopVideo.getImageAnalysisEngine().setRoi2(roi2X,roi2Y,roi2W,roi2H);
    }

    private void updateAnalysisData(AnalysisImage ai) {
        appliedToServer = updateData(false,ai.getImageW(),ai.getImageH(),ai.getRoiX(),ai.getRoiY(),ai.getRoiW(),
                ai.getRoiH(),ai.getRoi2X(),ai.getRoi2Y(),ai.getRoi2W(),ai.getRoi2H(),ai.getThreshold(),
                ai.getThresholdPoints(),ai.isCalculateThreshold(),ai.isPerformFit(),ai.isPerformSmoothing());
        getServerModel().setAImage(ai);
    }

    private void updateBackground() {
        executeTask(() -> {
            if (backgroundFile != null) return;
            double[] im = acopVideo.getImageAnalysisEngine().getPreciseBackground();
            int width = acopVideo.getImageAnalysisEngine().getBackgroundWidth();
            IMAGE orgImage = acopVideo.getImageAnalysisEngine().getImage();
            fileName = VIDEO_BACKGROUND;
            setBackgroundImage(im,width,orgImage,true);
        });
    }

    private void executeTask(Runnable r) {
        if (executor == null) {
            executor = Executors.newSingleThreadExecutor();
        }
        executor.execute(r);
    }

    private static GridBagConstraints gbc(int x, int y, int w, int h, int wx, int wy, int anchor, int fill,
            int insetTop, int insetLeft, int insetBottom, int insetRight) {
        return new GridBagConstraints(x,y,w,h,wx,wy,anchor,fill,new Insets(insetTop,insetLeft,insetBottom,insetRight),0,
                0);
    }

    public static void main(String[] args) {
        AnalysisCustomizer customizer = new AnalysisCustomizer();
        RunnerHelper.runComponent(customizer,500,500);
    }
}
