package de.desy.acop.video.analysis;

/**
 * </code>ColorDecoder</code> interface provides a means of transforming the integer color code into a meaningful double
 * value.
 *
 * @author Tilen Kusterle, Cosylab
 *
 */
public interface ColorDecoder {

    /**
     * Transforms the integer color code into a meaningful double value.
     *
     * @param code the integer color code
     * @return a meaningful double value
     * @deprecated use {@link #transform(int[])} to transform the whole array at once
     */
    @Deprecated
    public double transform(int code);

    /**
     * Transforms the complete array of integer color codes into meaningful double values array. Use this method with
     * large arrays in order to avoid calling {@link #transform(int)} many times, which might be costly.
     *
     * @param data the data to transform
     * @return the transformed values
     */
    public double[] transform(int[] data);

    /**
     * Transforms the complete array of </code>TYPE_INT_BGR</code> integer colors codes into meaningful double value
     * array. Use this method with large arrays in order to avoid calling {@link #transform(int)} many times, which
     * might be costly.
     *
     * @param data the data to transform
     * @return the transformed values
     */
    public double[] transformBGR(int[] data);
}
