/**
 *
 */
package de.desy.acop.video.analysis.decoders;

import java.awt.Color;

import de.desy.acop.video.analysis.ColorDecoder;

/**
 * </code>LuminosityColorDecoder</code> transforms the</code>TYPE_INT_ARGB</code>
 * integer color code into a normalized sum of red, green and blue components by
 * calculating luminosity as Y = 0.299*R + 0.587*G + 0.114*B.
 *
 * @author Tilen Kusterle, Cosylab
 *
 */
public class LuminosityColorDecoder implements ColorDecoder {

	/* (non-Javadoc)
	 * @see de.desy.acop.video.analysis.ColorDecoder#transform(int)
	 */
	@Override
	@Deprecated
	public double transform(int code) {
		Color c = new Color(code, true);
		return 0.299*c.getRed()+0.587*c.getGreen()+0.114*c.getBlue();
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.video.analysis.ColorDecoder#transform(int[])
	 */
	@Override
	public double[] transform(int[] data) {
		int length = data.length;
		double[] ret = new double[length];
		int code;
		for (int i = 0; i < length; i++) {
			code = data[i];
			ret[i] = 0.299*((code >> 16) & 0xFF) + 0.587*((code >> 8) & 0xFF)
					+ 0.114*((code >> 0) & 0xFF);
		}
		return ret;
	}

	@Override
	public double[] transformBGR(int[] data) {
	    int length = data.length;
        double[] ret = new double[length];
        int code;
        for (int i = 0; i < length; i++) {
            code = data[i];
            ret[i] = 0.114*((code >> 16) & 0xFF) + 0.587*((code >> 8) & 0xFF)
                    + 0.299*((code >> 0) & 0xFF);
        }
        return ret;
	}
}
