package de.desy.acop.video.displayer;

import java.util.HashMap;
import java.util.Map;

/**
 * Grid view mode enumeration
 * 
 * @author mdavid
 * 
 */
public enum GridMode {

	DOT_GRID_MODE(0),

	CROSS_GRID_MODE(1),

	LINE_GRID_MODE(2);

	private int id;

	private static Map<Integer, GridMode> map = new HashMap<>();
	static {
		for (GridMode e : values())
			if (map.put(e.id, e) != null)
				throw new InternalError("duplicate id: " + e.id);
	}

	private GridMode(int id) {
		this.id = id;
	}

	/**
	 * Returns integer representation of the grid mode
	 * 
	 * @return integer representation of the grid mode
	 */
	public int getId() {
		return id;
	}

	public boolean equals(int id) {
		return this.id == id;
	}

	/**
	 * Returns grid view mode by integer id
	 * 
	 * @param id
	 *            - integer representation of the grid mode
	 * @return grid view mode
	 */
	public static GridMode valueOf(int id) throws IllegalArgumentException {
		GridMode e = map.get(id);
		if (e == null)
			throw new IllegalArgumentException("non-existing id: " + id);
		return e;
	}

}
