package de.desy.acop.video.displayer;

import java.util.LinkedHashMap;
import java.util.Map;

/**
 * To adjust an image orientation
 * 
 * @author David Melkumyan, DESY
 * @version $Revision: 2001 $ : $LastChangedDate: 2017-10-08 14:18:22 +0100 $
 * 
 */
public enum ImageOrientation {
	/**
	 * No rotation
	 */
	NoChange("NONE"),

	/**
	 * Rotate image by 90 degrees clockwise
	 */
	RotationBy90CW("CW 90\u00b0"),

	/**
	 * Rotate by 180 degrees
	 */
	RotationBy180("180\u00b0"),

	/**
	 * Rotate image by 90 degrees counter-clockwise
	 */
	RotationBy90CCW("CCW 90\u00b0");

	/**
	 * Image rotation description
	 */
	private String descr;

	private static Map<String, ImageOrientation> map = new LinkedHashMap<>(values().length);
	static {
		for (ImageOrientation e : values()) {
			if (map.put(e.descr, e) != null) {
				throw new InternalError("duplicate description: " + e.descr);
			}
		}
	}

	/**
	 * Sets a description of the rotation mode
	 * 
	 * @param description
	 */
	private ImageOrientation(String description) {
		descr = description;
	}

	public static String[] descriptions() {
		return map.keySet().toArray(new String[map.size()]);
	}

	/**
	 * @return Description of the rotation mode
	 */
	public String getDescription() {
		return descr;
	}

	/**
	 * Return true if this rotation is for 90 degrees from the original or false otherwise.
	 * 
	 * @return true if rotated for 90 degrees or false otherwise
	 */
	public boolean isVertical() {
	    return this == RotationBy90CCW || this == RotationBy90CW;
	}
	
	/**
	 * @param desc
	 *            - description
	 * @return returns {@link ImageOrientation} value by description
	 * @throws IllegalArgumentException
	 */
	public static ImageOrientation getValue(String desc) throws IllegalArgumentException {
		ImageOrientation e = map.get(desc);
		if (e == null) {
			throw new IllegalArgumentException("non-existing image rotation description: " + desc);
		}
		return e;
	}
}
