package de.desy.acop.video.displayer;

import java.util.Date;

import de.desy.tine.types.IMAGE;
import de.desy.tine.types.IMAGE.FrameHeader;
import de.desy.tine.types.IMAGE.SourceHeader;

/**
 * <code>VideoHeaderV3</code> implements reference caching of a Video System v3
 * (VSv3) data array known as TINE IMAGE datatype for usage with the rest of
 * Video System v3 Java code.<br>
 * It contains useful helper functions to decode certain properties of VSv3
 * header into human-readable format.<br>
 * <br>
 * <b>Note:</b>This class is only a very basic construct and might be
 * substantially changed in future. It fits its purpose currently but will not
 * win a price for meaningful class design!
 * 
 * @deprecated functionality replaced by IMAGE class, not needed
 *             use directly {@link de.desy.tine.types.IMAGE} instead
 * @author sweisse, mdavid
 */
@Deprecated
public final class VideoHeaderV3 implements Cloneable {

	/**
	 * referenced frameHeader of private TINE IMAGE
	 */
	public FrameHeader frameHeader = null;

	/**
	 * referenced sourceHeader of private TINE IMAGE
	 */
	public SourceHeader sourceHeader = null;

	/**
	 * size in bytes of a VSv3 header on transport level
	 * 
	 * @deprecated use {@link de.desy.tine.types.IMAGE#HEADER_SIZE} instead
	 */
	@Deprecated
	public static final int HDRSIZE = 188;

	/**
	 * maximum size in bytes of a VSv3 image (without header)
	 * 
	 * @deprecated use {@link de.desy.tine.types.IMAGE#DEFAULT_DATA_SIZE}
	 *             instead
	 */
	@Deprecated
	public static final int TRANSPORT_LENGTH_V3 = 1024 * 1024 * 6;

	/**
	 * first magic id for VSv3 'VSV3'
	 * 
	 * @deprecated use {@link de.desy.tine.types.IMAGE#DEFAULT_BASE_TAG} instead
	 */
	@Deprecated
	public static final int CF_IMAGE_MAGIC_01 = 0x33565356; // VSV3

	/**
	 * cameraPortId is put to if no camera port is set there
	 * 
	 * @deprecated use {@link de.desy.tine.types.IMAGE#DEFAULT_CAMERA_PORT_ID}
	 *             instead
	 */
	@Deprecated
	public static final int CF_IMAGE_NO_CAMERA_PORT_ID = 0;

	/** second magic id for VSv3 '2008' */
	// SW! removed October 1, 2008, because magic02 is not so important
	// to have, but a cameraPortId is much more important
	// public static final long CF_IMAGE_MAGIC_02 = 0x38303032; // 2008

	/**
	 * initial IMAGE version
	 * 
	 * @deprecated use {@link de.desy.tine.types.IMAGE#IMAGE_INITIAL_VERSION}
	 *             instead
	 */
	@Deprecated
	public static final int CF_IMAGE_INITIAL_VERSION = 0x1;

	/**
	 * current IMAGE version
	 * 
	 * @deprecated use {@link de.desy.tine.types.IMAGE#IMAGE_VERSION} instead
	 */
	@Deprecated
	public static final int CF_IMAGE_VERSION = CF_IMAGE_INITIAL_VERSION;

	/**
	 * maximum IMAGE version
	 * 
	 * @deprecated use {@link de.desy.tine.types.IMAGE#IMAGE_MAX_VERSION}
	 *             instead
	 */
	@Deprecated
	public static final int CF_IMAGE_MAX_VERSION = CF_IMAGE_INITIAL_VERSION;

	/**
	 * referenced TINE IMAGE
	 */
	private IMAGE image;

	/**
	 * default constructor - deprecated
	 */
	@SuppressWarnings("unused")
	private VideoHeaderV3() {
	}

	/**
	 * The new instance of the class will be a reference to the passed image
	 * data.
	 * 
	 * @param img
	 *            image data to wrap inside this class
	 */
	public VideoHeaderV3(IMAGE img) {
		image = img;
		sourceHeader = img.getSourceHeader();
		frameHeader = img.getFrameHeader();
	}

	/**
	 * returns the image reference currently cached in class.
	 * 
	 * @return image reference currently cached in class
	 */
	public IMAGE getImage() {
		return image;
	}

	/**
	 * decodes timestamp of video frame into java Date class
	 * 
	 * @return date representation of
	 *         sourceHeader.timestamp(Seconds+Microseconds)
	 */
	public final Date getTimestampAsDate() {
		return new Date((sourceHeader.timestampSeconds * 1000L) + (sourceHeader.timestampMicroseconds / 1000L));
	}

	/**
	 * in order to get the real width of the image bits appended this function
	 * must be used. In case if the area of interest is set, one can not
	 * directly look at sourceWidth because aoiWidth depict the height of the
	 * appended image in such case
	 * 
	 * @return the width in pixels of the appended image data bits
	 */
	public int getAppendedWidth() {
		if (frameHeader.aoiWidth > 0)
			return frameHeader.aoiWidth;
		else
			return frameHeader.sourceWidth;
	}

	/**
	 * in order to get the real height of the image bits appended this function
	 * must be used. In case if the area of interest is set, one can not
	 * directly look at sourceHeight because aoiHeight depict the height of the
	 * appended image in such case
	 * 
	 * @return the height in pixels of the appended image data bits
	 */
	public int getAppendedHeight() {
		if (frameHeader.aoiHeight > 0)
			return frameHeader.aoiHeight;
		else
			return frameHeader.sourceHeight;
	}

	/**
	 * mdavid: moved from ImageDisplayer class mdavid: removed first argument
	 * (source video image)
	 * 
	 * clones (1:1 copy) a video frame header and bits. It is used to have an
	 * unchanged video header for meta information printing and a changed header
	 * and bits for displaying later on.
	 * 
	 * @return cloned VideoHeaderV3 object
	 */
	@Override
	public VideoHeaderV3 clone() {
		return new VideoHeaderV3(image.clone());
	}

}
