/*
 * Created on Nov 8, 2006
 *
 */
package de.desy.mst.auxiliary.devl.deviceserverlib.alarms;


/**
 * The severity.
 * Each alarm can have a severity which 
 * indicates its importance. The device
 * server wizard defines six alarm levels,
 * modeled after the log level scheme of 
 * log4j. Each severity is mapped to a TINE 
 * alarm level. 
 * @author jwlg
 */
public enum Severity {
    
    NONE ("NONE", 0, 0),
    DEBUG ("DEBUG", 20, 2),
    INFO ("INFO", 40, 4),
    WARNING ("WARNING", 60, 8),
    ERROR ("ERROR", 80, 12),
    FATAL ("FATAL", 100, 15);
    
    private final int value;
    private final int tineLevel;
    private String name;
    
    /**
     * Constructor.
     * @param name Name of the severity.
     * @param value Value of the severity.
     * @param tineLevel TINE alarm level.
     */
    Severity(String name, int value, int tineLevel) {
        this.name = name;
        this.value = value;
        this.tineLevel = tineLevel;
    }
    
    /**
     * Returns the name.
     */
    public String getName() {
        return name;
    }

    /**
     * Returns the TINE alarm level.
     */
    public int getTineLevel() {
        return tineLevel;
    }

    /**
     * Returns the value.
     */
    public int getValue() {
        return value;
    }

    /**
     * Returns the name.
     */
    public String toString() {
        return name;
    }

}
