/*
 * Created on Jan 16, 2007
 *
  */
package de.desy.mst.auxiliary.devl.deviceserverlib.config;

import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

/**
 * An alarm definition.
 * A device type can have an unlimited number of alarms
 * which are defined by instances of this class.
 * @author jwlg
 * @see CfgDeviceType
 */
@XmlRootElement
public class CfgAlarm {
    
    private String name = ""; 
    private String description = "";
    private String severity = "NONE";
    
    /**
     * Returns the description.
     */
    @XmlElement
    public String getDescription() {
        return description;
    }
    
    /**
     * Sets the description.
     * Each alarm can have a description which 
     * is visible in the TINE interface.
     * @param description The description to be set.
     */
    public void setDescription(String description) {
        this.description = description;
    }
    
    /**
     * Returns the name.
     */
    @XmlElement
    public String getName() {
        return name;
    }
    
    /**
     * Sets the name.
     * This is the name of the alarm constant which the
     * wizard generates, as well as the alarm name which 
     * appears in the TINE interface.
     * @param name New name.
     */
    public void setName(String name) {
        this.name = name;
    }
    
    /**
     * Returns the severity.
     */
    @XmlElement
    public String getSeverity() {
        return severity;
    }
    
    /**
     * Sets the severity.
     * The severity is a string which corresponds to 
     * an enum constant as defined by 
     * de.desy.mst.auxiliary.devl.deviceserverlib.alarms.Severity.
     * 
     *  @see de.desy.mst.auxiliary.devl.deviceserverlib.alarms.Severity
     *  
     *  @param severity The severity to be set.
     */ 
    public void setSeverity(String severity) {
        this.severity = severity;
    }
    
}
