/*
 * Created on Jun 5, 2007
 *
  */
package de.desy.mst.auxiliary.devl.deviceserverlib.config;

import java.util.LinkedList;

import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlRootElement;

/**
 * The data types section in the definition section.
 * 
 * The data types section contains a list of type definitions.
 * It leaves some redundancy for later extensions, e.g. if it is desired 
 * to use external data types which are not defined in the same project. 
 * @see CfgDefinition
 * @see CfgTypeDefinition
 * @author jwlg
 */
@XmlRootElement
public class CfgDataTypes {
    
    private LinkedList<CfgTypeDefinition> typeDefinitions = new LinkedList<CfgTypeDefinition>();

    /**
     * Returns the list of type definitions.
     */
    @XmlElementWrapper(name="typeDefinitions")
    @XmlElement(name="typeDefinition")
    public LinkedList<CfgTypeDefinition> getTypeDefinitions() {
        return typeDefinitions;
    }
    
    /**
     * Sets the list of type definitions.
     * This method is needed by JAXB. It is not indended to be used
     * by programs.
     * @param definitions The new list of definitions.
     */
    public void setTypeDefinitions(LinkedList<CfgTypeDefinition> definitions) {
        this.typeDefinitions = definitions;
    }
    
    /**
     * Finds a type definition specified by a name.
     * @param className The class name of the demanded type.
     * @return A type definition or null if none was found.
     */
    public CfgTypeDefinition findTypeDefinition(String className) {
        CfgTypeDefinition result = null;
        for (CfgTypeDefinition def : typeDefinitions) {
            if (def.getClassName().equals(className)) {
                result = def;
                break;
            }
        }
        return result;
    }
    
    /**
     * Verifies if a type definition is contained.
     * @param className The class name of the type definition.
     * @see CfgTypeDefinition#getClassName
     * @return True if the type definition was found.
     */
    public boolean containsTypeDefinition(String className) {
        boolean result = false;
        for (CfgTypeDefinition def : typeDefinitions) {
            if (def.getClassName().equals(className)) {
                result = true;
                break;
            }
        }
        return result;
    }

    /**
     * Returns a list of the class names of all contained
     * type definitions.
     */
    public LinkedList<String> getTypeDefinitionNames() {
        LinkedList<String> result = new LinkedList<String>();
        for (CfgTypeDefinition def : typeDefinitions) {
            result.add(def.getClassName());
        }
        return result;
    }
    
}
