/*
 * Created on Jan 16, 2007
 *
 */
package de.desy.mst.auxiliary.devl.deviceserverlib.config;

import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlIDREF;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlType;

/**
 * The specification of a device group.
 * A device group contains a set of devices of the same
 * device type. A device group has a name which is also
 * visible as the Export Name (also called Device Server in
 * TINE terminology) in the TINE interface.
 * The device group is connected to a device type and
 * contains an initialization section which specifies
 * how it finds initialization data.
 * 
 * @author jwlg
 */
@XmlRootElement
@XmlType(propOrder={"name","description","alarmID","deviceType","initConfig"})
public class CfgDeviceGroup {

    private int alarmID = 0;
    private String name = "";
    private String description = "";
    private CfgDeviceType deviceType = null;
    private CfgModuleInit initConfig = new CfgModuleInit();

    /**
     * Returns the name of the device group.
     */
    @XmlElement
    public String getName() {
        return name;
    }

    /**
     * Sets the name of the device group.
     * @param name The new name.
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * Returns the alarm ID of the device group.
     * In TINE, a set of alarms is required to define 
     * a number by which it is identified.
     * This should hopefully be made optional in TINE 4. 
     */
    @XmlElement
    public int getAlarmID() {
        return alarmID;
    }

    /**
     * Sets the alarm ID.
     * @param alarmID
     */
    public void setAlarmID(int alarmID) {
        this.alarmID = alarmID;
    }

    /**
     * Returns the description of the device group.
     */
    @XmlElement
    public String getDescription() {
        return description;
    }

    /**
     * Sets the description of the device group.
     * @param description The new description.
     */
    public void setDescription(String description) {
        this.description = description;
    }

    /**
     * Returns the device type.
     * In the device server specification this is
     * a reference to a previously defined device type.
     */
    @XmlElement(nillable=true)
    @XmlIDREF
    public CfgDeviceType getDeviceType() {
        return deviceType;
    }

    /**
     * Sets the device type.
     * A valid device type must be contained in the 
     * list of device types in the definition section.
     * @see CfgDefinition
     * @see CfgDeviceType
     * @param deviceType The device type.
     */
    public void setDeviceType(CfgDeviceType deviceType) {
        this.deviceType = deviceType;
    }

    /**
     * Returns the initialization configuration.
     */
    @XmlElement
    public CfgModuleInit getInitConfig() {
        return initConfig;
    }

    /**
     * Sets the initialization configuration.
     * @param initConfig
     */
    public void setInitConfig(CfgModuleInit initConfig) {
        this.initConfig = initConfig;
    }

}
