/*
 * Created on Mar 1, 2007
 *
 */
package de.desy.mst.auxiliary.devl.deviceserverlib.config;

import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

/**
 * Configuration of file logging.
 * 
 * @author jwlg
 */
@XmlRootElement
public class CfgLogFile {
    
    private boolean enabled = true;
    private CfgLogLevel threshold = CfgLogLevel.ALL;
    private String maxFileSize = "1MB";
    private int maxBackupIndex = 10;
    private CfgLogFileType type = CfgLogFileType.ROLLING;
    
    /**
     * Returns true if file logging is enabled.
     */
    @XmlElement
    public boolean isEnabled() {
        return enabled;
    }
    
    /**
     * Enables or disables file logging.
     * @param enabled New value.
     */
    public void setEnabled(boolean enabled) {
        this.enabled = enabled;
    }
    
    /**
     * Returns the maximal backup index.
     */
    @XmlElement
    public int getMaxBackupIndex() {
        return maxBackupIndex;
    }
    
    /**
     * Sets the maximum backup index.
     * A parameter for the rolling file appender.
     * Log4j maintains a set of backup files. Backup files
     * are marked with an index. If the
     * current log file changes, all indices are increased.
     * The file with the highest index is deleted if that
     * index is greater than maxBackupIndex.
     * 
     * @param maxBackupIndex New value.
     */
    public void setMaxBackupIndex(int maxBackupIndex) {
        this.maxBackupIndex = maxBackupIndex;
    }
    
    /**
     * Returns the maximal file size.
     */
    @XmlElement
    public String getMaxFileSize() {
        return maxFileSize;
    }

    /**
     * Sets the maximal file size.
     * A parameter for the rolling file appender.
     * If the maximal file size is reached, the logger
     * creates a new file. The old file is backed up.
     * @param maxFileSize New value.
     */
    public void setMaxFileSize(String maxFileSize) {
        this.maxFileSize = maxFileSize;
    }
    
    /**
     * Returns the threshold.
     */
    @XmlElement
    public CfgLogLevel getThreshold() {
        return threshold;
    }

    /**
     * Sets the threshold.
     * @param threshold New value.
     */
    public void setThreshold(CfgLogLevel threshold) {
        this.threshold = threshold;
    }
    
    /**
     * Returns the log file type.
     */
    @XmlElement
    public CfgLogFileType getType() {
        return type;
    }
    
    /**
     * Sets the log file type.
     * Two types of file logging are supported.
     * @param type
     * @see CfgLogFileType
     */
    public void setType(CfgLogFileType type) {
        this.type = type;
    }
    
}
