/*
 * Created on Mar 1, 2007
 *
  */
package de.desy.mst.auxiliary.devl.deviceserverlib.config;

import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import org.apache.log4j.Logger;

/**
 * The logging configuration.
 * 
 * The device server specification makes it possible to
 * configure log4j in a very simplified way in opposite to the
 * rather complicated log4j configuration, and it is 
 * completely integrated with the Device Server Wizard GUI.
 * 
 * The logging configuration contains a common threshold,
 * a file logging section, and a console logging section.
 * 
 * @author jwlg
 */
@XmlRootElement
public class CfgLogging {
    
    private CfgLogLevel threshold = CfgLogLevel.DEBUG;
    private CfgLogFile file = new CfgLogFile();
    private CfgLogConsole console = new CfgLogConsole();
    
    private static Logger log = Logger.getLogger(CfgLogging.class);
    
    public CfgLogging() {
        log.debug("Initializing Log Config: " + threshold);
    }
    
    /**
     * Returns the file logging section.
     */
    @XmlElement
    public CfgLogConsole getConsole() {
        return console;
    }
    
    /**
     * Sets the file logging section.
     * Required by JAXB.
     * @param consoleLog The new console logging section.
     */
    public void setConsole(CfgLogConsole consoleLog) {
        this.console = consoleLog;
    }
    
    /**
     * Returns the file logging section.
     */
    @XmlElement
    public CfgLogFile getFile() {
        return file;
    }
    
    /**
     * Sets the file logging section.
     * Required by JAXB.
     * @param fileLog
     */
    public void setFile(CfgLogFile fileLog) {
        this.file = fileLog;
    }
    
    /**
     * Returns the common threshold.
     */
    @XmlElement
    public CfgLogLevel getThreshold() {
        return threshold;
    }
    
    /**
     * Sets the common threshold.
     * @param threshold Threshold log level.
     */
    public void setThreshold(CfgLogLevel threshold) {
        log.debug("Threshold set to " + threshold);
        this.threshold = threshold;
    }

}
