/*
 * Created on Jan 16, 2007
 *
 */
package de.desy.mst.auxiliary.devl.deviceserverlib.config;

import java.util.HashMap;

import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlTransient;
import javax.xml.bind.annotation.XmlType;

/**
 * The initialization configuration for a device group.
 * 
 * The initialization configuration consists of an initialization
 * type and a hash table which holds the specific parameters for 
 * a configuration type.
 * The name "moduleInit" is kept for historical reasons.
 * @see CfgDeviceGroup
 * @author jwlg
 */
@XmlRootElement
@XmlType(propOrder= {"type", "parameters"})
public class CfgModuleInit {

    private CfgInitType type = CfgInitType.CUSTOM;
    private HashMap<String, String> parameters = new HashMap<String, String>(); 

    /**
     * Returns the parameter map.
     */ 
    @XmlElementWrapper(name="parameters")
    public HashMap<String, String> getParameters() {
        return parameters;
    }

    /**
     * Sets the parameter map.
     * Required by JAXB. 
     * @param parameters New parameter map.
     */
    public void setParameters(HashMap<String, String> parameters) {
        this.parameters = parameters;
    }

    /**
     * Returns the initialization type.
     */
    @XmlElement
    public CfgInitType getType() {
        return type;
    }

    /**
     * Sets the initialization type.
     * @param type New initialization type.
     */
    public void setType(CfgInitType type) {
        this.type = type;
    }

    /**
     * Returns true if the initialization type is "CUSTOM".
     * Convenience method.
     * @see CfgInitType
     */
    @XmlTransient
    public boolean isCustomType() {
        return (this.type == CfgInitType.CUSTOM);
    }

}
