/*
 * Created on Jan 16, 2007
 *
  */
package de.desy.mst.auxiliary.devl.deviceserverlib.config;

import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlTransient;
import javax.xml.bind.annotation.XmlType;

/**
 * 
 * The specification of a package name.
 * The device server wizard separates the java package 
 * into three parts organization, facility, and subsystem.
 * 
 * @author jwlg
 */
@XmlRootElement
@XmlType(propOrder= {"organization", "facility", "subsystem"})
public class CfgPackage {
    
    @XmlTransient
    public static final String DEFAULT_PREFIX="de.desy.mst";
    @XmlTransient
    public static final String PACKAGE_SEPARATOR = ".";
    
    private String organization = DEFAULT_PREFIX;
    private String facility = "";
    private String subsystem = "";
    
   /**
    * Default constructor.
    *
    */
    public CfgPackage() {
        initialize(DEFAULT_PREFIX, "", "");
    }
    
    /**
     * Constructor with all parts.
     * @param organization The organization.
     * @param facility The facility.
     * @param subsystem The subsystem.
     */
    public CfgPackage(String organization, String facility, String subsystem) {
        initialize(organization, facility, subsystem);
    }
    
    /**
     * Initializer. 
     * Used by constructors.
     * @param organization The organization.
     * @param facility The facility.
     * @param subsystem The subsystem.
     */
    private void initialize(String organization, String facility, String subsystem) {
        this.organization = organization;
        this.facility = facility;
        this.subsystem = subsystem;
    }
    
    /**
     * Returns the package name.
     */
    public String getPackageName() {
        return createPackageName(organization, facility, subsystem);
    }

    /**
     * Returns the facility.
     */
    @XmlElement
    public String getFacility() {
        return facility;
    }
    

    /**
     * Sets the facility.
     * @param facility New facility.
     */
    public void setFacility(String facility) {
        this.facility = facility;
    }
    
    /**
     * Returns the organization.
     */
    @XmlElement
    public String getOrganization() {
        return organization;
    }
    
    /**
     * Sets the organization.
     * @param organization New organization.
     */
    public void setOrganization(String organization) {
        this.organization = organization;
    }
    
    /** 
     * Returns the subsystem.
     */
    @XmlElement
    public String getSubsystem() {
        return subsystem;
    }

    /**
     * Sets the subsystem.
     * @param subsystem New subsystem.
     */
    public void setSubsystem(String subsystem) {
        this.subsystem = subsystem;
    }
    
    /**
     * Creates a package name.
     * Composes a package name organization.facility.subsystem.
     * Each of the components may be an empty string.
     * @param organization The organization.
     * @param facility The facility.
     * @param subsystem The subsystem.
     * @return Java package name.
     */
    public static String createPackageName(String organization, String facility, String subsystem) {
        String result = organization;
        if (!(facility == null || facility.equals(""))) {
            if (result.equals("")) {
                result = facility;
            } else {
                result = result + PACKAGE_SEPARATOR + facility;
            }
        }
        if (!(subsystem == null || subsystem.equals(""))) {
            if (result.equals("")) {
                result = subsystem;
            } else {
                result = result + PACKAGE_SEPARATOR + subsystem;
            }
        }
        return result;

    }

    /**
     * Checks for equality.
     */
    public boolean equals(Object obj) {
        boolean result = false;
        if (this == obj) {
            result = true;
        } else if (obj instanceof CfgPackage) {
            CfgPackage rhs = (CfgPackage) obj;
            result = (rhs.organization.equals(this.organization)
                    && rhs.facility.equals(this.facility)
                    &&  rhs.subsystem.equals(this.subsystem));
        } else {
           result = false;
        }
        return result;
   }
      
    
    
}
