/*
 * Created on Jun 5, 2007
 *
 */
package de.desy.mst.auxiliary.devl.deviceserverlib.config;

import java.util.LinkedList;

import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlRootElement;

import de.desy.tine.definitions.TFormat;


/**
 * Contains the definition of a user-defined type.
 * A type definition contains a list of fields,
 * a class name, and a TINE tag.
 * @author jwlg
 */
@XmlRootElement
public class CfgTypeDefinition  {
    
    String className = "";
    String tag = null;
    LinkedList<CfgTypeField> fields = new LinkedList<CfgTypeField>();
    
    /**
     * Returns the fields.
     */
    @XmlElementWrapper(name="fields")
    @XmlElement(name="field")
    public LinkedList<CfgTypeField> getFields() {
        return fields;
    }
    
    /**
     * Sets the list of fields.
     * Not for general use. Required by JAXB.
     * @param fields New list of fields.
     */
    public void setFields(LinkedList<CfgTypeField> fields) {
        this.fields = fields;
    }
    
    /**
     * Returns the class name.
     */
    @XmlElement
    public String getClassName() {
        return className;
    }

    /**
     * Sets the class name.
     * @param name New class name.
     */
    public void setClassName(String name) {
        this.className = name;
    }

    /**
     * Returns the class name.
     */
    public String getSimpleClassName() {
        return className;
    }

    /**
     * Returns the TINE format.
     * A convenience method which always returns "CF_STRUCT"
     */
    public String getTineFormat() {
        return "CF_STRUCT";
    }

    /**
     * Returns the TINE format key.
     * A convenience method which always returns TFormat.CF_STRUCT.
     */
    public int getTineFormatKey() {
        return TFormat.CF_STRUCT;
    }
    
    /**
     * Returns the tag.
     */
    @XmlElement
    public String getTag() {
        return tag;
    }

    /**
     * Sets the tag.
     * The tag is needed for TINE.
     * @param tag New tag.
     */
    public void setTag(String tag) {
        this.tag = tag;
    }
    
    /**
     * Returns true if a tag is set.
     */
    public boolean isTagged() {
        return (tag != null);
    }

    @Override
    public int hashCode() {

        final int PRIME = 31;
        int result = 1;
        result = PRIME * result + ((className == null) ? 0 : className.hashCode());
        result = PRIME * result + ((fields == null) ? 0 : fields.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {

        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        final CfgTypeDefinition other = (CfgTypeDefinition) obj;
        if (className == null) {
            if (other.className != null)
                return false;
        } else if (!className.equals(other.className))
            return false;
        if (fields == null) {
            if (other.fields != null)
                return false;
        } else if (!fields.equals(other.fields))
            return false;
        return true;
    }

    
    
}
