/*
 * Created on Mar 7, 2007
 *
 */
package de.desy.mst.auxiliary.devl.deviceserverlib.config;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;

/**
 * A generalized JAXB writer.
 * 
 * @author jwlg
 */
public class JaxbWriter<T> {

    JAXBContext context = null;
    Class documentClass = null;

    /**
     * Constructor.
     * @param clazz The class of the root node.
     */
    public JaxbWriter(Class clazz) {
        this.documentClass = clazz;
    }

    
    private JAXBContext getContext() throws JAXBException {
        if (context == null) {    
            context = JAXBContext.newInstance(documentClass);
        }
        return context;
    }

    /**
     * Writes an object hierarchy to a file.
     * @param document The root node object.
     * @param file The file.
     * @throws FileNotFoundException
     * @throws JAXBException
     * @throws IOException
     */
    public void write(T document, File file) throws FileNotFoundException, JAXBException, IOException {
        writeAndClose(document, new FileOutputStream(file));
    }

    /**
     * Writes an object hierarchy to a file.
     * @param document The root node object.
     * @param fileName The file name.
     * @throws FileNotFoundException
     * @throws JAXBException
     * @throws IOException
     */
    public void write(T document, String fileName) throws FileNotFoundException, JAXBException, IOException {
        write(document, new File(fileName));
    }

    /**
     * Writes an object hierarchy to an output stream and closes the stream.
     * @param document The root node object.
     * @param outputStream The output stream.
     * @throws JAXBException
     * @throws IOException
     */
    private void writeAndClose(T document, OutputStream outputStream) throws JAXBException, IOException {
        write(document, outputStream);
        outputStream.close();
    }

    /**
     * Writes an object hierarchy to an output stream and closes the stream.
     * @param document The root node object.
     * @param outputStream The output stream.
     * @throws JAXBException
     * @throws IOException
     */
    public void write(T document, OutputStream outputStream) throws JAXBException, IOException {
        Marshaller m = getContext().createMarshaller(); 
        m.setProperty( Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE ); 
        m.marshal(document, outputStream );
    }

}
