/*
 * Created on Sep 19, 2006
 *
  */
package de.desy.mst.auxiliary.devl.deviceserverlib.exceptions;

/**
 * Defines error keys and standard messages.
 * 
 * Error keys are integers which are sent over TINE.
 * Currently, error keys are interpreted as user-defined
 * errors in TINE. As of TINE4, predefined error keys 
 * are expected.
 * @author jwlg
 */
public enum ErrorSpecs {
    
    IO_EXCEPTION(700, "I/O Exception"),
    FILE_IO_EXCEPTION(701, "File I/O Error"),
    NETWORK_IO_EXCEPTION(711, "Network I/O Error"),
    HARDWARE_IO_EXCEPTION(721, "Hardware I/O Error"),
    INVALID_PARAMETER_EXCEPTION(730, "Invalid Parameter"),
    INVALID_NAME_EXCEPTION(731, "Invalid Name"),
    INVALID_INDEX_EXCEPTION(732, "Invalid Index"),
    INVALID_VALUE_EXCEPTION(733, "Invalid Value"),
    INVALID_SIZE_EXCEPTION(734, "Invalid Data Size"),
    DEVICE_EXCEPTION(740, "Device Exception"),
    DEVICE_ERROR_EXCEPTION(741, "Device Error"),
    DEVICE_NOT_READY_EXCEPTION(742, "Device Not Ready"),
    DEVICE_BUSY_EXCEPTION(743, "Device Busy"),
    JAVA_RUNTIME_EXCEPTION(760, "Runtime Exception"),
    SYSTEM_EXCEPTION(790, "System Error"),
    USER_EXCEPTION(799, "User-Defined Exception");
    
    private final int key;
    private final String defaultMessage;
    
    /**
     * Constructor.
     * @param errorCode
     * @param defaultErrorMessage
     */
    private ErrorSpecs(int errorCode, String defaultErrorMessage) {
        this.key = errorCode;
        this.defaultMessage = defaultErrorMessage;
    }
    
    /**
     * Returns the key.
     */
    public int getKey() {
        return key;
    }
    
    /**
     * Returns the default message.
     */
    public String getDefaultMessage() {
        return defaultMessage;
    }
    
}
