/*
 * Created on Sep 20, 2006
 *
  */
package de.desy.mst.auxiliary.devl.deviceserverlib.exceptions;


/**
 * An exception which can be thrown by a device method when an invalid
 * parameter is given as a parameter.
 * This is a superclass which can be differentiated by using its 
 * subclasses.
 * 
 * @see InvalidIndexException
 * @see InvalidNameException
 * @see InvalidValueException
 * 
 * @author jwlg
 */
public class InvalidParameterException extends PredefinedException {

    private static final long serialVersionUID = 5031977363453866870L;

    /**
     * Default constructor.
     */
    public InvalidParameterException() {
        super(ErrorSpecs.INVALID_PARAMETER_EXCEPTION);
    }
    
    /**
     * Constructor with a specific message.
     * @param message A message which overrides the default message.
     */
    public InvalidParameterException(String message) {
        super(ErrorSpecs.INVALID_PARAMETER_EXCEPTION, message);
    }
    
    /**
     * Constructor for derived classes.
     * @param errorCode
     */
    protected InvalidParameterException(ErrorSpecs errorCode) {
        super(errorCode);
    }
    
    /**
     * Constructor
     */
    protected InvalidParameterException(ErrorSpecs errorCode, String message) {
        super(errorCode, message);
    }
    

}
