/*
 * Created on Mar 7, 2007
 *
  */
package de.desy.mst.auxiliary.devl.deviceserverlib.history;

import javax.xml.bind.annotation.XmlRootElement;

/**
 * An entry of a history map.
 * A HistoryMapEntry is the "address" of an entry in the
 * history map. it consists of device group name, device name,
 * and property name. The device name may be empty.
 * @author jwlg
 */
@XmlRootElement
public class HistoryMapEntry {
    
    private String group = "";
    private String device = "";
    private String property = "";
    
    /**
     * Default constructor.
     * Builds a HistoryMapEntry with empty components. 
     */
    public HistoryMapEntry() {
    }

    /**
     * Constructor.
     * Builds a history map entry. 
     * @param group Device group name.
     * @param device Device name.
     * @param property Property.
     */
    public HistoryMapEntry(String group, String device, String property) {
        this.group = group;
        this.device = device;
        this.property = property;
    }

    /**
     * Returns the device name.
     */
    public String getDevice() {
        return device;
    }

    /**
     * Assigns a device name.
     * @param device New device name.
     */
    public void setDevice(String device) {
        this.device = device;
    }

    /**
     * Returns the device group.
     */
    public String getGroup() {
        return group;
    }

    /**
     * Assigns the device group.
     * @param group Name of the device group.
     */
    public void setGroup(String group) {
        this.group = group;
    }

    /**
     * Returns the property.
     */
    public String getProperty() {
        return property;
    }

    /**
     * Assigns a property.
     * @param property Name of the property.
     */
    public void setProperty(String property) {
        this.property = property;
    }

    /* (non-Javadoc)
     * @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode() {

        final int PRIME = 31;
        int result = 1;
        result = PRIME * result + ((device == null) ? 0 : device.hashCode());
        result = PRIME * result + ((group == null) ? 0 : group.hashCode());
        result = PRIME * result + ((property == null) ? 0 : property.hashCode());
        return result;
    }

    /* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals(Object obj) {

        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        final HistoryMapEntry other = (HistoryMapEntry) obj;
        if (device == null) {
            if (other.device != null)
                return false;
        } else if (!device.equals(other.device))
            return false;
        if (group == null) {
            if (other.group != null)
                return false;
        } else if (!group.equals(other.group))
            return false;
        if (property == null) {
            if (other.property != null)
                return false;
        } else if (!property.equals(other.property))
            return false;
        return true;
    }

}
