/*
 * Created on Jul 5, 2006
 */
package de.desy.mst.auxiliary.devl.deviceserverlib.initialization;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;

import de.desy.mst.auxiliary.devl.deviceserverlib.config.CfgInitType;

/**
 * A Class which simplifies reading database tables.
 * A DatabaseReader needs the class of objects to read
 * and the table properties and determines the underlying
 * table type automatically, using the TableRepository.
 * @author jwlg
 */
public final class DatabaseReader {
    
    private HashMap<String,String> properties;
    private IDatabaseDriver databaseDriver = null;
    
    /**
     * Constructor.
     * @param tableType The type of the table. 
     * @param properties The table properties which determine which type of table
     * is read an which parameters are passed.
     */
    public DatabaseReader(CfgInitType tableType, HashMap<String,String> properties) {
        super();
        this.properties = properties;
        this.databaseDriver = DriverRepository.getInstance().getDatabaseDriver(tableType);
    }
    
    /**
     * Reads records from a device table.
     * @return A LinkedList of records.
     * @throws SQLException 
     * @throws ClassNotFoundException 
     */
    public ResultSet getResult() throws ClassNotFoundException, SQLException {
        ResultSet result = null;
        result = databaseDriver.getResult(this.properties);
        return result;
    }
    
    /**
     * Closes the underlying connection.
     *
     */
   public void close() {
       databaseDriver.close();
   }
    
}
