/*
 * Created on Jul 5, 2006
 *
  */
package de.desy.mst.auxiliary.devl.deviceserverlib.initialization;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;

/**
 * The common interface for device table drivers.
 * 
 * A Device Table driver executes a query using a certain
 * JDBC driver. The query ant its parameters are
 * specified by a list of parameters, which can be
 * specific to each driver.
 * The device table driver is used to read initial
 * records from an input source. 
 * 
 * @author jwlg
 */
public interface IDatabaseDriver {

    /**
     * Executes a query and returns the result.
     * @param properties The properties of the device table. Any device table may
     *      have a specific set of properties. This set is specified in 
     *      getDefaultProperties();
     * @return A TineResultSet.
     * @throws Exception
     */
    ResultSet getResult(HashMap<String, String> properties) throws ClassNotFoundException, SQLException;
    
    /**
     * Returns the default parameters for a device table.
     * This method returns the parameters which a device 
     * table needs to initialize. The main purpose is to
     * provide indivitual information for each JDBC driver type.
     * The returned properties should be filled
     * with default values if possible or otherwise empty strings.
     */
    public HashMap<String, String> getDefaultProperties();
    
    public void close();
    
 
}
