/*
 * Created on Jul 31, 2006
 *
  */
package de.desy.mst.auxiliary.devl.deviceserverlib.initialization;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashMap;

/**
 * A Generic driver for JDBC data sources.
 * 
 * This driver is completely controlled by external properties
 * which are provided as a HashMap.
 * 
 * Driver properties are not supported in this version.
 * 
 * @author jwlg
 */
public class JdbcDriver implements IDatabaseDriver {

    Connection connection = null;
  
    public HashMap<String, String> getDefaultProperties() {
        HashMap<String, String> params = new HashMap<String, String>();
        params.put("driverClass", "");
        params.put("connectionUrl", "");
        params.put("sqlQuery", "");
        return params;
    }
    
    /**
     * Performs an SQL query and returns the result.
     * @param properties The list of parameters. Expects the keys "sqlQuery", connectionUrl", and "driverClass".
     */
    public ResultSet getResult(HashMap<String, String> properties) throws ClassNotFoundException, SQLException  {
        ResultSet resultSet = null;
        String sqlQuery = SelectUtil.extendSelect(properties.get("sqlQuery"));
        String connectionUrl = properties.get("connectionUrl");
        String driverClass = properties.get("driverClass");
        Class.forName(driverClass);

        connection = DriverManager.getConnection(connectionUrl);
        Statement statement = connection.createStatement();
        resultSet = statement.executeQuery(sqlQuery);
        return resultSet;
    }
    
    public void close() {
        if (connection != null) {
            try {
                connection.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }
    }

    @Override
    protected void finalize() throws Throwable {
        super.finalize();
        close();
    }


}
