/*
 * Created on Sep 3, 2007
 *
  */
package de.desy.mst.auxiliary.devl.deviceserverlib.initialization.controller;

import java.util.LinkedList;

import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlRootElement;

/**
 * Initialization of a controller device.
 * A controller device is a device which controls
 * a set of selectedDevices. All controlled selectedDevices must be
 * in the same device group.
 * @author jwlg
 */
@XmlRootElement
public class ControllerDefinition {
    
    private String deviceName = "";
    private String managedDeviceGroup = "";
    private LinkedList<String> selectedDevices = null;
    private LinkedList<DeviceRange> selectedRanges = null;
    
    /**
     * Returns the deviceName of this controller.
     */
    @XmlElement(name="deviceName")
    public String getDeviceName() {
        return deviceName;
    }
    
    /**
     * Sets the deviceName of this controller.
     * @param deviceName The device name .
     */
    public void setDeviceName(String name) {
        this.deviceName = name;
    }
 
    /**
     * Returns the name of the managed device group.
     */
    @XmlElement(name="managedDeviceGroup")
    public String getManagedDeviceGroup() {
        return managedDeviceGroup;
    }
    
    public void setManagedDeviceGroup(String group) {
        this.managedDeviceGroup = group;
    }
    
    /**
     * Returns the list of selected selectedDevices.
     */
    @XmlElementWrapper(name="selectedDevices")
    @XmlElement(name="device")
    public LinkedList<String> getSelectedDevices() {
        return selectedDevices;
    }

    /**
     * Assigns the list of selected devices.
     * @param selected devices
     */
    public void setSelectedDevices(LinkedList<String> devices) {
        this.selectedDevices = devices;
    }

    
    /**
     * Returns the list of selected device ranges.
     * @return List of device ranges.
     */
    @XmlElementWrapper(name="selectedRanges")
    @XmlElement(name="range")
    public LinkedList<DeviceRange> getSelectedRanges() {
        return selectedRanges;
    }

    /**
     * Assigns the list of device ranges.
     * @param selectedDeviceRanges
     */
    public void setSelectedRanges(
            LinkedList<DeviceRange> selectedDeviceRanges) {
        this.selectedRanges = selectedDeviceRanges;
    }

}
