/*
 * Created on Feb 8, 2007
 *
  */
package de.desy.mst.auxiliary.devl.deviceserverlib.server;

import java.util.ArrayList;

/**
 * A container for common data in device groups.
 * 
 * An object of this class is used to store common or 
 * device-independent in a device group.
 * For example, it may contain data from CDI which are
 * incompatible with an object-oriented view of devices.
 * 
 * A CommonData instance is assigned to a device group. 
 * The instance can access to all devices in the group. 
 * The devices
 * have access to the common data via the group or via
 * ABasicDevice.
 * 
 * @author jwlg
 */
public abstract class ACommonData<T extends IBasicDevice> implements ICommonData {
    
    DeviceContext deviceContext = null;
    
    public ACommonData() {
    }
    
    public ACommonData(DeviceContext deviceContext) {
        this.deviceContext = deviceContext;
    }
    
    @SuppressWarnings("unchecked")
    protected ArrayList<T> getDevices() {
        ArrayList<T> result = null;
        DeviceGroup<T> deviceGroup = getDeviceContext().getDeviceGroup();
        result = deviceGroup.getDevices();
        return result;
    }
    
    protected DeviceContext getDeviceContext() {
        return deviceContext;
    }
    
    void setDeviceContext(DeviceContext deviceContext) {
        this.deviceContext = deviceContext;
    }
    
}
