/*
 * Created on Sep 19, 2007
 *
  */
package de.desy.mst.auxiliary.devl.deviceserverlib.server;

import java.util.HashMap;

import org.apache.log4j.Logger;

import de.desy.mst.auxiliary.devl.deviceserverlib.config.CfgInitType;


public class DeviceEquipmentModuleBuilder<T extends IBasicDevice> extends AEquipmentModuleBuilder<T> {
    
    private static Logger log = Logger.getLogger(DeviceEquipmentModuleBuilder.class);
    IDeviceModuleFactory<T> deviceModuleFactory = null;


    /**
     * Assigns a device module factory.
     * Implicitly also sets the module factory of the superclass.
     * @param moduleFactory
     */
    public void setDeviceModuleFactory(IDeviceModuleFactory<T> moduleFactory) {
        this.deviceModuleFactory = moduleFactory;
        super.setModuleFactory(moduleFactory);
    }

    @Override
    public void createDeviceGroup() throws EquipmentModuleBuilderException {
        
        if (deviceModuleFactory == null) {
            log.error("Cannot create device group. Module factory is not set.");
            throw new EquipmentModuleBuilderException();
        }
        if (getDeviceContext() == null) {
            log.error("Cannot create device group. Device context is not set.");
            throw new EquipmentModuleBuilderException();
        }
        if (getCfgDeviceGroup() == null) {
            log.error("Cannot create device group. CfgDeviceGroup is not set.");
            throw new EquipmentModuleBuilderException();
        }
        
        
        DeviceGroup<T> deviceGroup = new DeviceGroup<T>();
        deviceGroup.setName(getCfgDeviceGroup().getName());

        log.info("Building device group " + getCfgDeviceGroup().getName() );
        HashMap<String,String> parameters = getCfgDeviceGroup().getInitConfig().getParameters();

        ADeviceFactory<T> deviceFactory = null;
        if (getCfgDeviceGroup().getInitConfig().getType() == CfgInitType.CUSTOM) {
            deviceFactory = deviceModuleFactory.createDeviceFactory();
        } else {
            deviceFactory = new StandardDeviceFactory<T>();
        }

        if (deviceFactory != null) {
            deviceFactory.setDirectorySet(getDeviceServer().getDirectorySet());
            deviceFactory.setParameters(parameters);
            deviceFactory.setFactoryMethod(deviceModuleFactory.createFactoryMethod());
            deviceFactory.setCfgDeviceGroup(getCfgDeviceGroup());
            deviceFactory.createDevices(deviceGroup);
            deviceGroup.setDeviceContext(getDeviceContext());
            
        } else {
            log.error("Cannot create devices. No device factory set!");
        }
        setDeviceGroup(deviceGroup);
    }

}
