/*
 * Created on Mar 19, 2007
 *
  */
package de.desy.mst.auxiliary.devl.deviceserverlib.server;

import java.io.File;

import org.apache.log4j.Logger;

/**
 * A class which contains a set of directoty names.
 * 
 * The device server needs four local directories to work:
 * <ul>
 * <li>The <b>log</b> directory. The place where log files are written.</li>
 * <li>The <b>history</b> directory. The place where TINE local history files go.</li>
 * <li>The <b>config</b>uration directory. The place where configuration files are.</li>
 * <li>The <b>work</b> directory. The place where data can be written during runtime.</li>
 * </ul>
 * 
 * @author jwlg
 */
public class DirectorySet {
    
    private Logger log = Logger.getLogger(DirectorySet.class);
     
    String baseDirectory = "";
    String logDirectory = "";
    String historyDirectory;
    String workDirectory = "";
    String configDirectory = "";
    String databaseDirectory = "";

    /**
     * Creates a directory tree in the file system.
     * The directory set knows where its directories are
     * supposed to be. It checks if all these directories 
     * exist and creates those which are missing.
     */
    public void createDirectoryTree() {
        log.info("Reviewing directory structure in " + this.baseDirectory);
        createDirectory(this.baseDirectory);
        createDirectory(this.workDirectory);
        createDirectory(this.historyDirectory);
        createDirectory(this.configDirectory);
        createDirectory(this.logDirectory);
        createDirectory(this.databaseDirectory);
    }
    
    /**
     * Creates a directory if it doesn't exist.
     * @param dirName Name of the directory.
     */
    private void createDirectory(String dirName) {
        File directory = null;
        directory = new File(dirName);
        if (!directory.exists()) {
            log.info("Creating " + directory.getPath());
            directory.mkdirs();
        }
    }
    
    /**
     * Returns the history directory.
     */
    public String getHistoryDirectory() {
        return historyDirectory;
    }
    
    /**
     * Assigns the history directory.
     * <B>Note:</b> This method is not intended to be used directly. 
     * A factory should be used to assign the directories. 
     * @param historyDirectory The full path the history directory.
     */
    public void setHistoryDirectory(String historyDirectory) {
        this.historyDirectory = historyDirectory;
    }
    
    /**
     * Returns the base directory.
     * The base directory is the parent directory on top of 
     * all other directories.
     */
    public String getBaseDirectory() {
        return baseDirectory;
    }
    
    /**
     * Assigns the base directory.
     * <B>Note:</b> This method is not intended to be used directly. 
     * A factory should be used to assign the directories. 
     * @param baseDirectory
     */
    public void setBaseDirectory(String baseDirectory) {
        this.baseDirectory = baseDirectory;
    }
    
    /**
     * Returns the configuration directory.
     */
    public String getConfigDirectory() {
        return configDirectory;
    }

    /**
     * Assigns the configuration directory.
     * <B>Note:</b> This method is not intended to be used directly. 
     * A factory should be used to assign the directories. 
     * @param configDirectory The full path of the configuration directory.
     */
    public void setConfigDirectory(String configDirectory) {
        this.configDirectory = configDirectory;
    }
    
    /**
     * Returns the work directory.
     */
    public String getWorkDirectory() {
        return workDirectory;
    }
    
    

    /**
     * Assigns the work directory.
     * <B>Note:</b> This method is not intended to be used directly. 
     * A factory should be used to assign the directories. 
     * @param workDirectory The full path of the work directory.
     */
    public void setWorkDirectory(String workDirectory) {
        this.workDirectory = workDirectory;
    }
    
    /**
     * Returns the log directory.
     */
    public String getLogDirectory() {
        return logDirectory;
    }
    
    /**
     * Assigns the log directory.
     * <B>Note:</b> This method is not intended to be used directly. 
     * A factory should be used to assign the directories. 
     * @param logDirectory The full path of the log directory.
     */
    public void setLogDirectory(String logDirectory) {
        this.logDirectory = logDirectory;
    }

    
    /**
     * Returns the database directory.
     */
    public String getDatabaseDirectory() {
        return databaseDirectory;
    }

    
    /**
     * Assigns the database directory.
     * <B>Note:</b> This method is not intended to be used directly. 
     * A factory should be used to assign the directories. 
     * @param databaseDirectory The full path of the database directory.
     */
    public void setDatabaseDirectory(String databaseDirectory) {
        this.databaseDirectory = databaseDirectory;
    }
    
    
      
}
