/*
 * Created on Sep 17, 2007
 *
 */
package de.desy.mst.auxiliary.devl.deviceserverlib.server;

import org.apache.log4j.Logger;

/**
 * The equipment module factory.
 * 
 * An Equipment module factory uses an EquipmentModuleBuilder
 * to build an equipment module. The EquipmentModule Builder
 * is provided externally.
 * 
 * @author jwlg
 */
public class EquipmentModuleFactory<T extends IBasicDevice> {

    private final static Logger log = Logger.getLogger(EquipmentModuleFactory.class);
    IEquipmentModuleBuilder<T> equipmentModuleBuilder = null;
    
    public void EquipmentModulefactory() {
    }
    
    protected void setEquipmentModuleBuilder(
            IEquipmentModuleBuilder<T> equipmentModuleBuilder) {
        this.equipmentModuleBuilder = equipmentModuleBuilder;
    }

    public EquipmentModule buildEquipmentModule() throws EquipmentModuleBuilderException {
        
        if (equipmentModuleBuilder == null) {
            log.error("Cannot create Equipment module. Equipment module builder is not set." );
            return null;
        }
 
        equipmentModuleBuilder.initializeEquipmentModule();
        equipmentModuleBuilder.createDeviceContext();
        equipmentModuleBuilder.createDeviceGroup(); 
        equipmentModuleBuilder.createCommonData(); 
        equipmentModuleBuilder.registerDevices();
        equipmentModuleBuilder.registerProperties();
        equipmentModuleBuilder.registerLocalHistories();
 
        return equipmentModuleBuilder.getEquipmentModule();
 
    }

}
