/*
 * Created on May 30, 2006
 *
  */
package de.desy.mst.auxiliary.devl.deviceserverlib.server;

import java.util.Set;

import de.desy.mst.auxiliary.devl.deviceserverlib.alarms.Alarm;


/**
 * The common interface of all devices.
 * @author jwlg
 */
public interface IBasicDevice {
    
    /**
     * Returns the device name.
     */
    public String getDeviceName();
    
    /**
     * Updates the device.
     * 
     * The update method is called regularly by TINE in a certain
     * interval. IN the update method, the device is required to refresh its 
     * status, values, and alarms.
     *
     */
    public void update();
    
    /**
     * Sets up the device.
     * 
     * The setup method is called after all devices and device groups 
     * have been created. In the setup method, the device is required
     * to establish TINE and CDI links as well as references to 
     * other devices or groups.
     *
     */
    public void setup();
    
    /**
     * Tries to acquire a lock.
     * Implemented in ABasicDevice.
     * @see ABasicDevice#tryAcquireLock()
     */
    public boolean tryAcquireLock();
    
    /**
     * Releases a lock.
     * Implemented in ABasic device.
     * @see ABasicDevice#releaseLock()
     *
     */
    public void releaseLock();
    
    /**
     * Acquires a lock.
     * Waits until the lock is available.
     * Implemented in ABasicDevice.
     * @throws InterruptedException
     * @see ABasicDevice#acquireLock()
     */
    public void acquireLock() throws InterruptedException;
    
    /**
     * Returns the set of alarms.
     * Each device sets it alarms in the update() method.
     * Implemented in ABasicDevice.
     * 
     */
    public Set<Alarm> getAlarms();
    
    /** 
     * Assigns the device context.
     * This method is called only by the framework after a device 
     * has been constructed.
     * @param context The device context.
     */
    public void setDeviceContext(DeviceContext context);
    
    /**
     * Assigns the device name.
     * @param deviceName The device name.
     */
    public void setDeviceName(String deviceName);

}
