/*
 * Created on Nov 7, 2006
 *
  */
package de.desy.mst.auxiliary.devl.deviceserverlib.server;

import java.util.Date;

/**
 * An interface for update listeners.
 * 
 * Update listeners can be added to a device server.
 * The device server notifies update listeners before
 * the update begins and after the update has finished.
 * This interface is intended to be used by programs 
 * which have device server objects and want to 
 * synchonize themselves with the device server's state.
 * 
 * @author jwlg
 */
public interface IUpdateListener {
   
    /**
     * Called before the update begins.
     * 
     * @param sender The device server which called this method.
     * @param startTime A time stamp.
     */
    public void updateStarting(Object sender, Date startTime);
    
    /**
     * Called after the update has finished.
     * @param sender The device server which called this method.
     * @param endTime A time stamp.
     */
    public void updateFinished(Object sender, Date endTime);

}
