/*
 * Created on Mar 21, 2007
 *
  */
package de.desy.mst.auxiliary.devl.deviceserverlib.server;

import java.io.File;
import java.util.prefs.Preferences;
import org.apache.log4j.Logger;

/**
 * A directory set factory which implements the
 * MstApp directory scheme.
 * Don't know if this is correct. Use at your own risk.
 * @author jwlg
 */
public class MstAppDirectorySetFactory implements IDirectorySetFactory {
    
    private static MstAppDirectorySetFactory theInstance = new MstAppDirectorySetFactory();
    private static String MSTAPP_NODE = "de/desy/mst/libs/framework/mstapp/ini";
    private Logger log = Logger.getLogger(MstAppDirectorySetFactory.class);
    
    private MstAppDirectorySetFactory() {
    }
    
    private String getDirectoryName(String baseDirectory, String subDirectory, String applicationName) {
        if (baseDirectory.endsWith(File.separator)) {  // Bei MstApp blich!
             return baseDirectory  + subDirectory + File.separator + applicationName;
        } else {
            return baseDirectory + File.separator + subDirectory + File.separator + applicationName;
        }
    }
    
    public DirectorySet createDirectorySet(String applicationName) {
        
        DirectorySet result = null;
        Preferences userPrefs = Preferences.userRoot().node(MSTAPP_NODE);
        String mstAppRoot = userPrefs.get("home_directory", "");
        
        log.info("Creating MSTApp Directory Set for " + applicationName);
        
        if (mstAppRoot.equals("")) {
            log.warn("MstApp user settings not found. Using current directory.");
            String currentDirectory = System.getProperty("user.dir");
            result = DirectorySetFactory.getInstance().createDefaultDirectorySet(currentDirectory);
        } else {

            /**
             * Die Verzeichnisnamen "Archiv", "Betrieb" und "DeviceTabellen"
             * sind meine Erfindung. In MstApp gibt es nichts entsprechendes.
             */
            result = new DirectorySet();
            result.setBaseDirectory(mstAppRoot);
            result.setHistoryDirectory(getDirectoryName(mstAppRoot, "Archiv", applicationName));
            result.setConfigDirectory(getDirectoryName(mstAppRoot, "IniDaten", applicationName));
            result.setWorkDirectory(getDirectoryName(mstAppRoot, "Betrieb", applicationName));
            result.setLogDirectory(getDirectoryName(mstAppRoot, "Protokoll", applicationName));
            result.setDatabaseDirectory(getDirectoryName(mstAppRoot, "DeviceTabellen", applicationName));
            
        }
        return result;
        
    }
    
    public static MstAppDirectorySetFactory getInstance() {
        return theInstance;
    }

}
