/*
 * Created on Aug 9, 2006
 *
  */
package de.desy.mst.auxiliary.devl.deviceserverlib.server;

import java.util.LinkedList;

/**
 * A base class for propery sets.
 * Each device type has a set of properties.
 * The code generator generates a property set
 * for each type which is derived from PropertySet.
 * PropertySet contains basic functionality. It 
 * contains a list of operations and a list of 
 * properties and provides an update methods
 * which updates all properties.
 * 
 * @author jwlg
 */
public class PropertySet<D extends IBasicDevice>{
   
    private LinkedList<APropertyHandler<D>> properties = new LinkedList<APropertyHandler<D>>();
    private LinkedList<AOperationHandler<D>> operations = new LinkedList<AOperationHandler<D>>();
   
    /**
     * Returns the list of properties.
     */
    public LinkedList<APropertyHandler<D>> getProperties() {
        return properties;
    }

    /**
     * Adds a property to the list of properties.
     * @param property New property.
     */
    protected void addProperty(APropertyHandler<D> property) {
        properties.add(property);
    }
    
    /**
     * Returns the list of operations.
     */
    public LinkedList<AOperationHandler<D>> getOperations() {
        return operations;
    }

    /**
     * Adds an operation to the list of operations.
     * @param operation New operation.
     */
    protected void addOperation(AOperationHandler<D> operation) {
        operations.add(operation);
    }
    
    /**
     * Updates all properties.
     * Calls APropertyHandler.updateBuffer() for all properties.
     */
    public void updateProperties() {
        for (APropertyHandler<D> property : properties) {
            property.updateBuffer();
        }
    }
}
