/*
 * Created on Sep 25, 2006
 *
  */
package de.desy.mst.auxiliary.devl.deviceserverlib.server;

import de.desy.mst.auxiliary.devl.deviceserverlib.exceptions.DeviceServerException;
import de.desy.mst.auxiliary.devl.deviceserverlib.exceptions.JavaRuntimeException;


/**
 * Return data for TINE calls.
 * 
 * The result set which is used by PropertyHandlerBridge.
 * @author jwlg
 */
public class TineResultSet {

    int returnCode = 0;
    String errorMessage = null;
    
    /**
     * Constructs a TineResultSet.
     * @param returnCode The return code.
     * @param errorMessage The error message.
     */
    public TineResultSet(int returnCode, String errorMessage) {
        this.returnCode = returnCode;
        this.errorMessage = errorMessage;
    }
    
    /**
     * Constructs a TineResultSet with a standard error code.
     * A particular message is not specified.
     * @param returnCode The return code.
     */
    public TineResultSet(int returnCode) {
        this.returnCode = returnCode;
    }
    
    /**
     * Constructs a TineResultSet from a DeviceServerException.
     * This constructoris used to transform DeviceServerExceptions
     * into TINE return codes.
     * @param ex The exception.
     */
    public TineResultSet(DeviceServerException ex) {
        this.returnCode = ex.getErrorKey();
        this.errorMessage = ex.getMessage();
    }
    
    /**
     * Constructs a TineResultSet from a Java Runtime Exception.
     * This constructor is used to transform a RuntimeException
     * into a TINE result. The result only informs the client
     * that a runtime exception has occured, including
     * error message. The class of the error is not transported.
     * @param ex A runtime exception.
     */
    public TineResultSet(RuntimeException ex) {
        this(new JavaRuntimeException(ex));
    }
    
    /**
     * Constructs an empty TineResultSet.
     * The returnCode is set to0. The errorMessage is set to null.
     */
    public TineResultSet() {
        this.returnCode = 0;
    }
    
    /**
     * Returns the error message.
     */
    public String getErrorMessage() {
        return errorMessage;
    }

    /**
     * Returns the return code.
     */
    public int getReturnCode() {
        return returnCode;
    }

    /**
     * Sets the error message.
     * @param errorMessage New error message.
     */
    public void setErrorMessage(String errorMessage) {
        this.errorMessage = errorMessage;
    }

    /**
     * Sets the return code.
     * @param returnCode New return code.
     */
    public void setReturnCode(int returnCode) {
        this.returnCode = returnCode;
    }
    
    
}
