/*
 * Created on Oct 5, 2006
 *
  */
package de.desy.mst.auxiliary.devl.deviceserverlib.tools;

/**
 * Increments a value over a range.
 * Starts at a start value, increments by a specified amount,
 * stops at an end value.
 * <p>
 * <B>Example</B>
 * <code>
 * StepIncrementor incrementor = new StepIncrementor(0.0,10.0,2.0);
 * System.out.println(incrementor.getCurrentValue());
 * while (!incrementor.isFinished()) {
 *     System.out.println(incrementor.nextValue());
 * }
 * </code> 
 * @author jwlg
 */
public class StepIncrementor {
    
    double startValue = 0;
    double currentValue = 0;
    double endValue = 0;
    double increment = 0;
    boolean isFinished = true;
    private boolean positiveDirection = true;
    
    /**
     * Constructor.
     * @param startValue The start value.
     * @param endValue The end value.
     * @param increment The increment.
     */
    public StepIncrementor(double startValue, double endValue, double increment) {
        reset(startValue, endValue, increment);
    }

    /**
     * Resets the incrementor.
     * @param startValue New start value.
     * @param endValue New end value.
     * @param increment New increment.
     */
    public void reset(double startValue, double endValue, double increment) {
        this.startValue = startValue;
        this.endValue = endValue;
        currentValue = startValue;
        positiveDirection = (endValue > startValue);
        if (positiveDirection) {
            this.increment = Math.abs(increment);
        } else {
            this.increment = - Math.abs(increment);
        }
        isFinished = (startValue == endValue);
    }
    
    /**
     * Increments and returns the current value.
     * If the current value exceeds the end value,
     * it is set to the end value.
     */
    public double getNextValue() {

        double nextValue;
        
        nextValue = currentValue + increment;
        
        if (positiveDirection) {
            if (nextValue >= endValue) {
                nextValue = endValue;
                isFinished = true;
            }
        } else {
            if (nextValue <= endValue) {
                nextValue = endValue;
                isFinished = true;
            }
        }
        
        currentValue = nextValue;
        return currentValue;
    }
    
    /**
     * Returns true if the end value is reached.
     */
    public boolean isFinished() {
        return isFinished;
    }
    
    public double getCurrentValue() {
        return currentValue;
    }
    
}
