/*
 * Created on Nov 15, 2007
 *
 */
package de.desy.mst.auxiliary.devl.deviceserverlib.tools.array;

import java.util.ArrayList;

/**
 * Provides generic utilities which work on array lists.
 * 
 * @author jwlg
 */
public class ArrayListUtils {

    public static <T> void moveToFirst(ArrayList<T> array, int index) {

        T buffer = array.get(index);
        array.remove(index);
        array.add(0, buffer);
    }

    public static <T> void moveToLast(ArrayList<T> array, int index) {

        T buffer = array.get(index);
        array.remove(index);
        array.add(buffer);
    }

    public static <T> void prependOrMoveToFirst(ArrayList<T> array, T value) {

        int index = array.indexOf(value);
        if (index >= 0) {
            moveToFirst(array, index);
        } else {
            prepend(array, value);
        }
    }

    public static <T> void appendOrMoveToLast(ArrayList<T> array, T value) {

        int index = array.indexOf(value);
        if (index >= 0) {
            moveToLast(array, index);
        } else {
            append(array, value);
        }
    }

    /**
     * Prepends a value to an array. Rotates an array to the right, deletes the
     * last element and adds the value at the first index.
     * 
     * @param array
     *            The array.
     * @param value
     *            The new value on position 0.
     */
    public static <T> void prepend(ArrayList<T> array, T value) {

        array.add(0, value);
    }

    /**
     * Appends a value to an array. Rotates an array to the left, deletes the
     * first element and adds the value at the last index.
     * 
     * @param array
     *            The array.
     * @param value
     *            The value to be appended.
     */
    public static <T> void append(ArrayList<T> array, T value) {

        array.add(value);
    }

    public static <T> void limitSizeDeleteFirst(ArrayList<T> array, int maxSize) {

        if (maxSize > 0) {
            while (array.size() >= maxSize) {
                array.remove(0);
            }
        }
    }

    public static <T> void limitSizeDeleteLast(ArrayList<T> array, int maxSize) {

        if (maxSize > 0) {
            while (array.size() >= maxSize) {
                array.remove(array.size() - 1);
            }
        }
    }

}
