/*
 * CAMframeControl.java Created on 09 August 2008, 16:27 20.04.2009: added synchronized
 * setters: storeX(),storeY(),storeHeight(),storeWidth()storeState(int newState) methods:
 * getX(), getY(), getHeight(), getWidth(), getState() synchronized.
 */
package de.desy.mst.libs.framework.mstapp.util.camInstrumentation;

import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.event.WindowStateListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.lang.management.ManagementFactory;
import java.net.InetAddress;
import java.net.URL;
import java.net.UnknownHostException;
import java.rmi.registry.LocateRegistry;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;
import java.util.regex.Pattern;

import javax.management.AttributeChangeNotification;
import javax.management.InstanceAlreadyExistsException;
import javax.management.InstanceNotFoundException;
import javax.management.ListenerNotFoundException;
import javax.management.MBeanNotificationInfo;
import javax.management.MBeanRegistrationException;
import javax.management.MBeanServer;
import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.NotCompliantMBeanException;
import javax.management.Notification;
import javax.management.NotificationBroadcasterSupport;
import javax.management.NotificationEmitter;
import javax.management.NotificationFilter;
import javax.management.NotificationListener;
import javax.management.ObjectName;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXConnectorServer;
import javax.management.remote.JMXConnectorServerFactory;
import javax.management.remote.JMXServiceURL;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import javax.swing.Timer;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

import de.desy.tine.client.TLink;
import de.desy.tine.dataUtils.TDataType;
import de.desy.tine.definitions.TAccess;
import de.desy.tine.definitions.TErrorList;

/**
 * Class CAMframeControl
 *
 * @author karol
 */
public class CAMframeControl implements CAMframeControlMBean, NotificationEmitter {

    public static final int CAM_MANAGER_PORT = Integer.parseInt(System.getProperty("CAMMANAGER_PORT","50000"));
    public static final String CAMPORT = "CAMPORT";
    public static final String CAMHOST = "CAMHOST";
    static CAMframeControlManager camFrameControlManager = new CAMframeControlManager();
    private final NotificationBroadcasterSupport broadcaster = new NotificationBroadcasterSupport();
    private long seqNumber;
    private Timer delayedNotificationTimer;
    private Timer delayedCloseTimer;
    //-------------------------[MBean attribute]--------------------------------
    /**
     * Attribute : X
     */
    private int x;
    /**
     * Attribute : Y
     */
    private int y;
    /**
     * Attribute : Width
     */
    private int width;
    /**
     * Attribute : Height
     */
    private int height;
    /**
     * Attribute : Stat
     */
    private int state;
    //--------------------------------------------------------------------------
    private Frame frame;
    private ComponentListener cl;
    private WindowListener wl;
    private WindowStateListener wsl;
    private MBeanServer mbs;
    private ObjectName mbeanName;
    private String identification;
    private int port;
    private boolean ownerFrameClosed;
    private boolean controllerRoot;
    private boolean connected;
    private final boolean isWebstart;
    private final Executor exec;

    public static boolean isWebstart() {
        ClassLoader cl = Thread.currentThread().getContextClassLoader();
        while (cl != null) {
            if (cl.getClass().getName().equals("com.sun.jnlp.JNLPClassLoader")) {
                return true;
            }
            cl = cl.getParent();
        }
        return false;
    }

    public CAMframeControl(final Frame frame, final String identification) {
        final String daemon = System.getProperty("jnlp.daemon");
        final String uid = System.getProperty("jnlp.daemon.id");
        final boolean registerWithDaemon = daemon != null && uid != null;
        if (!registerWithDaemon) {
            String prop = System.getProperty("jnlp." + CAMPORT);
            if (prop != null) {
                System.setProperty(CAMPORT,prop);
            }
            System.setProperty("com.sun.management.jmxremote","true");
            try {
                final String ip = InetAddress.getLocalHost().getHostAddress();
                System.setProperty("java.rmi.server.hostname",ip);
            } catch (final Exception e) {
                //shouldn't happen, unless there is no network interface
                e.printStackTrace();
            }
            prop = System.getProperty("jnlp." + CAMHOST);
            if (prop != null) {
                System.setProperty(CAMHOST,prop);
            }
            construct(frame,identification);
        }
        //workaround for java webstart http://bugs.java.com/view_bug.do?bug_id=8019274
        this.isWebstart = isWebstart();
        //create the executor on the main thread to initialize the proper class loader and other resources
        if (this.isWebstart) {
            this.exec = Executors.newSingleThreadExecutor();
            this.exec.execute(new Runnable(){
                @Override
				public void run() { 
                    //delay the handshake until the application becomes visible
                    try {
                        while (!frame.isShowing() && !frame.isVisible()) {
                            Thread.sleep(10);
                        }
                    } catch (InterruptedException e){
                        //ignore
                    }
                    doDaemonHandshake(false);
                }
            });
        } else {
            this.exec = null;
            new Thread(new Runnable(){
                @Override
				public void run() {
                    //delay the handshake until the application becomes visible
                    try {
                        while (!frame.isShowing() && !frame.isVisible()) {
                            Thread.sleep(10);
                        }
                    } catch (InterruptedException e){
                        //ignore
                    }
                    doDaemonHandshake(false);
                }
            }, "CAM Handshake").start();
        }
    }

    public CAMframeControl(final Frame frame) {
        this(frame,frame.getClass().getCanonicalName() + "_" + frame.getTitle());
    }

    private void construct(final Frame frame, final String identification) {
        this.frame = frame;
        this.identification = identification;
        try {
            Runtime.getRuntime().addShutdownHook(new Thread() {

                @Override
                public void run() {
                    windowClosingAction();
                }
            });
        } catch (final IllegalStateException e) {
            return; // if the JVM shutdown is already in progress, then forget about any frame manipulations...
        }
        this.delayedNotificationTimer = new Timer(500,new DelayedNotificationActionListener());
        this.delayedNotificationTimer.setRepeats(false);
        this.delayedCloseTimer = new Timer(100,new DelayedCloseActionListener());
        this.delayedCloseTimer.setRepeats(false);
        this.cl = new ComponentFrameListener();
        this.wl = new WindowFrameListener();
        this.wsl = new WindowFrameStateListener();
        frame.addComponentListener(this.cl);
        frame.addWindowListener(this.wl);
        frame.addWindowStateListener(this.wsl);
        try {
            this.mbeanName = new ObjectName("camAgent:type=FrameController" + camFrameControlManager.getNextId());
            // Register MBean in Platform MBeanServer (locally in this VM)
            this.mbs = ManagementFactory.getPlatformMBeanServer();
            this.mbs.registerMBean(this,this.mbeanName);
            if (camFrameControlManager.getCount() == 0) {
                if (camFrameControlManager.register()) {
                    this.connected = register();
                    this.controllerRoot = true;
                    camFrameControlManager.setNotificationsEnabled(this.connected);
                    camFrameControlManager.enableHeartBeatGenerator(this.connected);
                } else {
                    this.connected = false;
                    return;
                }
            } else
                this.connected = camFrameControlManager.getNotificationsEnabled();
        } catch (final InstanceAlreadyExistsException ex) {
            ex.printStackTrace();
        } catch (final MBeanRegistrationException ex) {
            ex.printStackTrace();
        } catch (final NotCompliantMBeanException ex) {
            ex.printStackTrace();
        } catch (final MalformedObjectNameException ex) {
            ex.printStackTrace();
        } catch (final NullPointerException ex) {
            ex.printStackTrace();
        }
        camFrameControlManager.increment();
    }

    private int getMyPortNumber() {
        //String portStr = System.getProperty("com.sun.management.jmxremote.port");
        final boolean isFrameManager = Boolean.parseBoolean(System.getProperty("isCamManager","false"));
        if (isFrameManager) return CAM_MANAGER_PORT;
        String portStr = System.getProperty(CAMPORT);
        if (portStr == null) portStr = System.getenv(CAMPORT);
        if (portStr == null) return -1;
        if (portStr.isEmpty()) return -1;
        final int port = Integer.parseInt(portStr);
        if (port == CAM_MANAGER_PORT && !isFrameManager) {
            return -1;
        }
        return port;
    }

    private String getCamHost() {
        String host = System.getProperty(CAMHOST);
        if (host == null) {
            host = System.getenv(CAMHOST);
        }
        if (host == null) {
            host = "localhost";
        }
        return host;
    }

    private boolean register() {
        final int myport = getMyPortNumber();
        // build own connector server
        try {
            if (myport < 0) {
                System.out.println("-WARNING: no valid port specified, application remains unmanaged.");
                return false;
            }
            LocateRegistry.createRegistry(myport);
            final String localhost = System.getProperty("java.rmi.server.hostname");
            final JMXServiceURL myServiceURL = new JMXServiceURL("service:jmx:rmi:///jndi/rmi://" + localhost + ":" + myport + "/jmxrmi");
            final JMXConnectorServer connectorSerever = JMXConnectorServerFactory.newJMXConnectorServer(myServiceURL,null,this.mbs);
            connectorSerever.start();
        } catch (final Exception ex) {
            System.out.println("-WARNING: exception: " + ex + ", no connection to CAM.");
            return false;
        }
        if (myport != CAM_MANAGER_PORT) {
            //if this is is cam manager, do not connect it to itself
            try {
                final String camhost = getCamHost();
                final JMXServiceURL serviceURL = new JMXServiceURL("service:jmx:rmi:///jndi/rmi://" + camhost + ":" + CAM_MANAGER_PORT + "/jmxrmi");
                final JMXConnector jmxc = JMXConnectorFactory.connect(serviceURL,null);
                final MBeanServerConnection remoteServerConnection = jmxc.getMBeanServerConnection();
                final ObjectName connectorManager = new ObjectName("cam:type=ConnectionManager");
                final Object[] params = new Object[2];
                params[0] = this.identification;
                params[1] = myport;
                this.port = (Integer)remoteServerConnection.invoke(connectorManager,"getPort",params,
                        new String[]{"java.lang.String","int"});
                if (myport != this.port) {
                    System.out.println("-WARNING: currently dynamic port setting is not handled. Application remains unmanaged.");
                    unregister();
                    return false;
                }
            } catch (final Exception ex) {
                System.out.println("-WARNING: no connection to CAM.");
                return false;
            }
        }
        return true;
    }

    private void delayedNotifyMoved() {
        this.delayedNotificationTimer.restart();
    }

    private void notifyMoved() {
        if (!this.connected) return;
        // Create a JMX Notification
        final Notification notification = new Notification(AttributeChangeNotification.ATTRIBUTE_CHANGE,this,getNextSeqNumber(),"window moved");
        notification.setUserData(this.frame.getBounds());
        // Send a JMX notification.
        this.broadcaster.sendNotification(notification);
    }

    private void notifyResized() {
        if (!this.connected) return;
        final Notification notification = new Notification(AttributeChangeNotification.ATTRIBUTE_CHANGE,this,getNextSeqNumber(),"window resized");
        notification.setUserData(this.frame.getBounds());
        this.broadcaster.sendNotification(notification);
    }

    private void notifyStateChanged() {
        if (!this.connected) return;
        final Notification notification = new Notification(AttributeChangeNotification.ATTRIBUTE_CHANGE,this,getNextSeqNumber(),
                "window state changed");
        notification.setUserData(this.frame.getExtendedState());
        this.broadcaster.sendNotification(notification);
    }

    @Override
    public void addNotificationListener(final NotificationListener listener, final NotificationFilter filter, final Object handback)
            throws IllegalArgumentException {
        this.broadcaster.addNotificationListener(listener,filter,handback);
    }

    @Override
    public MBeanNotificationInfo[] getNotificationInfo() {
        return new MBeanNotificationInfo[]{new MBeanNotificationInfo(new String[]{AttributeChangeNotification.ATTRIBUTE_CHANGE},
                javax.management.AttributeChangeNotification.class.getName(),"Window moved")};
    }

    @Override
    public void removeNotificationListener(final NotificationListener listener) throws ListenerNotFoundException {
        this.broadcaster.removeNotificationListener(listener);
    }

    @Override
    public void removeNotificationListener(final NotificationListener listener, final NotificationFilter filter, final Object handback)
            throws ListenerNotFoundException {
        this.broadcaster.removeNotificationListener(listener,filter,handback);
    }

    public synchronized long getNextSeqNumber() {
        return this.seqNumber++;
    }

    private void unregister() {
        camFrameControlManager.enableHeartBeatGenerator(false);
        if (!this.connected) return;
        //no need to unregister the CAM manager from itself
        if (Boolean.parseBoolean(System.getProperty("isCamManager","false"))) return;
        try {
            final String camhost = getCamHost();
            final JMXServiceURL serviceURL = new JMXServiceURL("service:jmx:rmi:///jndi/rmi://" + camhost + ":" + CAM_MANAGER_PORT + "/jmxrmi");
            final JMXConnector jmxc = JMXConnectorFactory.connect(serviceURL,null);
            final MBeanServerConnection remoteServerConnection = jmxc.getMBeanServerConnection();
            final ObjectName connectorManager = new ObjectName("cam:type=ConnectionManager");
            final Object[] params = new Object[2];
            params[0] = this.identification;
            params[1] = this.port;
            final int result = (Integer)remoteServerConnection.invoke(connectorManager,"disconnect",
                    params,new String[]{"java.lang.String","int"});
            if (result != 0) System.out.println("-WARNING: can not disconnect from CAM");
        } catch (final Exception ex) {
            System.out.println("-WARNING: can not unregister. CAM not present.");
        }
    }

    private synchronized void windowClosingAction() {
        if (this.ownerFrameClosed) return;
        camFrameControlManager.decrement();
        try {
            this.mbs.unregisterMBean(this.mbeanName);
        } catch (final InstanceNotFoundException ex) {
            ex.printStackTrace();
        } catch (final MBeanRegistrationException ex) {
            ex.printStackTrace();
        }
        if (this.controllerRoot) unregister();
        this.ownerFrameClosed = true;
    }

    private class WindowFrameStateListener implements WindowStateListener {

        @Override
        public void windowStateChanged(final WindowEvent e) {
            storeState(e.getNewState());
            notifyStateChanged();
        }
    }

    private class WindowFrameListener implements WindowListener {

        @Override
        public void windowOpened(final WindowEvent e) {}

        @Override
        public void windowClosing(final WindowEvent e) {
            windowClosingAction();
        }

        @Override
        public void windowClosed(final WindowEvent e) {}

        @Override
        public void windowIconified(final WindowEvent e) {}

        @Override
        public void windowDeiconified(final WindowEvent e) {}

        @Override
        public void windowActivated(final WindowEvent e) {}

        @Override
        public void windowDeactivated(final WindowEvent e) {}
    }

    private class ComponentFrameListener implements ComponentListener {

        @Override
        public void componentResized(final ComponentEvent e) {
            storeWidth();
            storeHeight();
            notifyResized();
        }

        @Override
        public void componentMoved(final ComponentEvent e) {
            storeX();
            storeY();
            //notifyMoved();
            delayedNotifyMoved();
        }

        @Override
        public void componentShown(final ComponentEvent e) {}

        @Override
        public void componentHidden(final ComponentEvent e) {}
    }

    /**
     * Get X-Position of the window
     */
    @Override
    public synchronized int getX() {
        return this.x;
    }

    private synchronized void storeX() {
        this.x = this.frame.getX();
    }

    private synchronized void storeY() {
        this.y = this.frame.getY();
    }

    private synchronized void storeHeight() {
        this.height = this.frame.getHeight();
    }

    private synchronized void storeWidth() {
        this.width = this.frame.getWidth();
    }

    private synchronized void storeState(final int newState) {
        this.state = newState;
    }

    /**
     * Set X-Position of the window
     */
    @Override
    public void setX(final int value) {
        invokeLater(new Runnable() {

            @Override
            public void run() {
                CAMframeControl.this.frame.setLocation(value,CAMframeControl.this.frame.getY());
            }
        });
    }

    /**
     * Get Y-Position of the window
     */
    @Override
    public synchronized int getY() {
        return this.y;
    }

    /**
     * Set Y-Position of the window
     */
    @Override
    public void setY(final int value) {
        invokeLater(new Runnable() {

            @Override
            public void run() {
                CAMframeControl.this.frame.setLocation(CAMframeControl.this.frame.getX(),value);
            }
        });
    }

    /**
     * Get Width of the window
     */
    @Override
    public synchronized int getWidth() {
        return this.width;
    }

    /**
     * Set Width of the window
     */
    @Override
    public void setWidth(final int value) {
        invokeLater(new Runnable() {

            @Override
            public void run() {
                CAMframeControl.this.frame.setSize(value,CAMframeControl.this.frame.getHeight());
            }
        });
    }

    /**
     * Get Height of the window
     */
    @Override
    public synchronized int getHeight() {
        return this.height;
    }

    /**
     * Set Height of the window
     */
    @Override
    public void setHeight(final int value) {
        invokeLater(new Runnable() {

            @Override
            public void run() {
                CAMframeControl.this.frame.setSize(CAMframeControl.this.frame.getWidth(),value);
            }
        });
    }

    /**
     * Get Window state (iconified,normal,maximized)
     */
    @Override
    public synchronized int getState() {
        return this.state;
    }

    /**
     * Set Window state (iconified,normal,maximized)
     */
    @Override
    public void setState(final int value) {
        invokeLater(new Runnable() {

            @Override
            public void run() {
                CAMframeControl.this.frame.setExtendedState(value);
            }
        });
    }

    /**
     * Moves window to the given position
     *
     * @param newX new X-position
     * @param newY new Y-position
     */
    @Override
    public void move(final int newX, final int newY) {
        setX(newX);
        setY(newY);
    }

    /**
     * newOperation1 Description
     *
     * @param newWidth New width of the window
     * @param newHeight New height of the window
     */
    @Override
    public void resize(final int newWidth, final int newHeight) {
        setWidth(newWidth);
        setHeight(newHeight);
    }

    /**
     * Close window
     */
    @Override
    public void close() {
        try {
            //this.mbsc.unregisterMBean(this.mbeanName);
        } catch (final Exception ex) {
            ex.printStackTrace();
        }
        invokeLater(new Runnable() {

            @Override
            public void run() {
                CAMframeControl.this.delayedCloseTimer.start();
            }
        });
    }

    private void closeFrame() {
        invokeLater(new Runnable() {

            @Override
            public void run() {
                final WindowEvent e = new WindowEvent(CAMframeControl.this.frame,WindowEvent.WINDOW_CLOSING);
                CAMframeControl.this.frame.dispatchEvent(e);
            }
        });
    }

    /**
     * returns identification string
     *
     * @return java.lang.String
     */
    @Override
    public String getId() {
        return this.identification;
    }

    private class DelayedNotificationActionListener implements ActionListener {

        @Override
        public void actionPerformed(final ActionEvent e) {
            notifyMoved();
        }
    }

    private class DelayedCloseActionListener implements ActionListener {

        @Override
        public void actionPerformed(final ActionEvent e) {
            closeFrame();
        }
    }

    /**
     * newOperation0 Description
     *
     * @param x x
     * @param y y
     * @param width width
     * @param height height
     */
    @Override
    public void setGeometry(final int x, final int y, final int width, final int height) {
        invokeLater(new Runnable() {

            @Override
            public void run() {
                CAMframeControl.this.frame.setBounds(x,y,width,height);
            }
        });
    }

    /**
     * newOperation0 Description
     *
     * @param windowState Frame.ICONIFIED, Frame.NORMAL, etc...
     */
    @Override
    public void windowState(final int windowState) {
        invokeLater(new Runnable() {

            @Override
            public void run() {
                CAMframeControl.this.frame.setExtendedState(windowState);
            }
        });
    }

    public void invokeLater(final Runnable runnable) {
        if (this.isWebstart) {
            this.exec.execute(new Runnable() {

                @Override
                public void run() {
                    SwingUtilities.invokeLater(runnable);
                }
            });
        } else {
            SwingUtilities.invokeLater(runnable);
        }
    }

    private static final Pattern JNLPPATTERN = Pattern.compile(".+-[0-9]+.jnlp");

    public static void doDaemonHandshake(final boolean lookForChildProcesses) {
        try {
            String daemon = System.getProperty("jnlp.daemon");
            final String uid = System.getProperty("jnlp.daemon.id");
            final String name = ManagementFactory.getRuntimeMXBean().getName();
            final String appName = isWebstart() ? "javaws" : "javaw";
            final int pid = Integer.parseInt(name.split("\\@")[0]);
            if (daemon == null || uid == null) {
                String path = null;
                if (isWebstart()) {
                    try {
                        final Class<?> cl = Class.forName("com.sun.jnlp.BasicServiceImpl");
                        final Object obj = cl.getMethod("getInstance").invoke(null);
                        final URL url = (URL)cl.getMethod("getCodeBase").invoke(obj);
                        path = url.toString();
                        if (path.charAt(path.length()-1) != '/') {
                            path = path + "/";
                        }
                        String href = System.getProperty("jnlp.href");
                        if (href == null) {
                            href = System.getProperty("jnlpx.origFilenameArg");
                            final File file = new File(href);
                            if (file.exists()) {
                                try (FileInputStream fis = new FileInputStream(file)) {
                                    final DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
                                    final DocumentBuilder db = dbf.newDocumentBuilder();
                                    final Document document = db.parse(fis);
                                    document.getElementsByTagName("jnlp");
                                    final Node node = document.getElementsByTagName("jnlp").item(0);
                                    final NamedNodeMap map = node.getAttributes();
                                    href = map.getNamedItem("href").getTextContent();
                                } catch (final IOException e) {
                                    //ignore
                                }
                            } else {
                                if (href != null) {
                                    if (JNLPPATTERN.matcher(href).matches()) {
                                        href = href.replace('\\','/');
                                        href = href.substring(href.lastIndexOf('/') + 1, href.length()-5);
                                        href = href.substring(0,href.lastIndexOf('-')) + ".jnlp";
                                    }
                                }
                            }
                        }

                        if (href != null && !href.isEmpty()) {
                            if (href.charAt(0) == '/') {
                                href = href.substring(1);
                            }
                            path += href;
                        }
                    } catch (final Exception e) {
                        System.out.println("Cannot perform handshake with the console manager: " + e.getMessage());
                    }
                } else {
                    //if the application was started by jaws, this parameter exists and points to the jnlp that was used
                    path = System.getProperty("jaws.href");
                    JOptionPane.showMessageDialog(null,"Path: " + path);
                    if (path == null || path.isEmpty()) {
                        //if jaws was not used use one of the standard arguments
                        path = System.getProperty("sun.java.command");
                    }
                }
                try {
                    daemon = "/SERVICE/DMN_" + InetAddress.getLocalHost().getHostName();
                } catch (final UnknownHostException e) {
                    System.out.println("Cannot perform handshake with the console manager: " + e.getMessage());
                }
                final Handshake handshake = new Handshake(appName,path,-1,pid,lookForChildProcesses);
                final TDataType din = new TDataType(handshake);
                final TLink link = new TLink(daemon+ "/#0", "Handshake", null, din,TAccess.CA_WRITE);
                final int cc = link.executeAndClose();
                if (cc == TErrorList.link_timeout) {
                    System.out.println("Daemon server " + daemon + " not running on this host.");
                } else if (cc != 0) {
                    System.out.println("Error during handshake with the daemon server: " + daemon + " UID: " + uid + " Error: " + TErrorList.toString(cc));
                }
            } else {
                //send the third parameter, which signals that this app only notifies the
                //daemon that it is a parent of another application
                final Handshake handshake = new Handshake(appName,null,Integer.parseInt(uid),pid,lookForChildProcesses);
                final TDataType din = new TDataType(handshake);
                final TLink link = new TLink(daemon+ "/#0", "Handshake", null, din,TAccess.CA_WRITE);
                final int cc = link.executeAndClose();
                if (cc == TErrorList.link_timeout) {
                    System.out.println("Daemon server " + daemon + " not running on this host.");
                } else if (cc != 0) {
                    System.out.println("Error during handshake with the daemon server: " + daemon + " UID: " + uid + " Error: " + TErrorList.toString(cc));
                }
            }
        } catch (final Exception e) {
            System.out.println("No connection to remote CAM.");
        }
    }
}
