/*
 * CAMframeControlManager.java
 *
 * Created on 28 August 2008, 14:35
 */

package de.desy.mst.libs.framework.mstapp.util.camInstrumentation;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.lang.management.ManagementFactory;

import javax.management.AttributeChangeNotification;
import javax.management.MBeanNotificationInfo;
import javax.management.NotificationBroadcasterSupport;
import javax.management.NotificationEmitter;
import javax.management.NotificationFilter;
import javax.management.NotificationListener;
import javax.management.ObjectName;
import javax.management.JMException;
import javax.management.ListenerNotFoundException;
import javax.management.Notification;
import javax.swing.Timer;

/**
 * Class CAMframeControlManager
 * 
 * @author karol
 */
public class CAMframeControlManager implements CAMframeControlManagerMBean, NotificationEmitter 
{
    private final NotificationBroadcasterSupport broadcaster = new NotificationBroadcasterSupport();
    private long seqNumber;
    
    
    /**
     * Attribute : FrameControllerCount
     */
    private int frameControllerCount;
    private int idCounter;
    private boolean notificationsEnabled;
    private final Timer heartBeatTimer;
    
    public CAMframeControlManager()
    {   
        this.heartBeatTimer = new Timer(3000, new HeartBeatTimerListener());
        this.notificationsEnabled = false;
        this.idCounter = 0;
        this.frameControllerCount = 0;
    }

    /**
     * Get Count of CAMframeControllers
     */
    @Override public int getFrameControllerCount()
    {
        return this.frameControllerCount;
    }

    /**
     * returns number of CAMframeControllers
     */
  
    @Override public void addNotificationListener(final NotificationListener listener, final NotificationFilter filter, final Object handback) throws IllegalArgumentException
    {
        this.broadcaster.addNotificationListener(listener, filter, handback);
    }

    @Override public MBeanNotificationInfo[] getNotificationInfo()
    {
        return new MBeanNotificationInfo[]{new MBeanNotificationInfo(new String[]{AttributeChangeNotification.ATTRIBUTE_CHANGE}, javax.management.AttributeChangeNotification.class.getName(), "Number of CAMframeControllers has changed")};
    }

    @Override public void removeNotificationListener(final NotificationListener listener) 
    { 
        try
        {
            this.broadcaster.removeNotificationListener(listener);
        }
        catch (final ListenerNotFoundException ex)
        {
            ex.printStackTrace();
        }
    }

    @Override public void removeNotificationListener(final NotificationListener listener, final NotificationFilter filter, final Object handback) 
    {
        try
        {
            this.broadcaster.removeNotificationListener(listener, filter, handback);
        }
        catch (final ListenerNotFoundException ex)
        {
           ex.printStackTrace();
        }
    }

    public synchronized long getNextSeqNumber()
    {
        return this.seqNumber++;
    }

    public int getCount()
    {
        return this.frameControllerCount;
    }

    public boolean register()
    {
        try 
        { // Register MBean in Platform MBeanServer
            ManagementFactory.getPlatformMBeanServer().registerMBean(this,
                    new ObjectName("camAgent:type=CAMframeControlManager"));                    
            return true;
        }
        catch(final JMException ex) 
        {
            ex.printStackTrace();
        }
        return false;
    }
    
    
    public void increment()
    {
        ++this.frameControllerCount;
        if( !this.notificationsEnabled ) return;
        System.out.println("sending notification: CAMframeControllers count changed");
        
        final AttributeChangeNotification  notification = new AttributeChangeNotification(this, getNextSeqNumber(), 0, "CAMframeControllers count incremented", "frameControllerCount", "int", this.frameControllerCount-1, this.frameControllerCount);
//      Notification notification = new Notification(AttributeChangeNotification.ATTRIBUTE_CHANGE,
//                                                     this, getNextSeqNumber(), "CAMframeControllers count changed");

        // Send a JMX notification.
        this.broadcaster.sendNotification(notification);        
    }
    
    public void decrement()
    {
        --this.frameControllerCount;
        if( !this.notificationsEnabled ) return;
        System.out.println("sending notification: CAMframeControllers count changed");
        
        final AttributeChangeNotification  notification = new AttributeChangeNotification(this, getNextSeqNumber(), 0, "CAMframeControllers count decremented", "frameControllerCount", "int", this.frameControllerCount+1, this.frameControllerCount);        
        this.broadcaster.sendNotification(notification);     
    }

    public int getNextId()
    {
       return ++this.idCounter;
    }

    void enableHeartBeatGenerator(final boolean enable)
    {
        if( enable )
        {
            this.heartBeatTimer.setRepeats(true);
            this.heartBeatTimer.start();
        }
        else this.heartBeatTimer.stop();
    }

    void setNotificationsEnabled(final boolean notificationsEnabled)
    {
        this.notificationsEnabled = notificationsEnabled;
    }
    
    boolean getNotificationsEnabled()
    {
        return this.notificationsEnabled;
    }
  
    private void sendHeartBeat()
    {
         if( !this.notificationsEnabled ) return;        
         final Notification notification = new Notification(AttributeChangeNotification.ATTRIBUTE_CHANGE,
                                                     this, getNextSeqNumber(), "heartbeat");        
         this.broadcaster.sendNotification(notification);    
    }
    
    private class HeartBeatTimerListener implements ActionListener
    {
    	@Override public void actionPerformed(final ActionEvent e)
        {
           sendHeartBeat();
        }        
    }        
}


