package de.desy.mst.libs.framework.mstapp.util.camInstrumentation;

import java.util.Arrays;

import de.desy.tine.structUtils.TTaggedStructure;

public class Handshake extends TTaggedStructure {

    private final char[] name = new char[32];
    private String nameStr;
    private final char[] path = new char[256];
    private String pathStr;
    private final short[] iamparent = new short[1];
    private final int[] uid = new int[1];
    private final int[] pid = new int[1];

    private void initStructure() {
        addField(this.name,"name");
        addField(this.path,"path");
        addField(this.iamparent,"iamparent");
        addField(this.uid,"uid");
        addField(this.pid,"pid");
        initDone();
    }

    /**
     * Construct a new Handshake.
     */
    public Handshake() {
        this(null,null,-1,-1,false);
    }

    public Handshake(final String name, final String path, final int uid, final int pid, final boolean parent) {
        super("Handshake");
        initStructure();
        setUID(uid);
        setPID(pid);
        setParent(parent);
        setProcessName(name);
        setPath(path);
    }

    private static String charToString(final char[] array) {
        final StringBuilder sb = new StringBuilder();
        for (int i = 0; i < array.length && array[i] != 0; i++) {
            sb.append(array[i]);
        }
        return sb.toString();
    }

    /**
     * @return the name
     */
    public String getProcessName() {
        if (this.nameStr == null) {
            this.nameStr = charToString(this.name).trim();
        }
        return this.nameStr;
    }

    /**
     * @return the path
     */
    public String getPath() { 
        if (this.pathStr == null) {
            this.pathStr = charToString(this.path).trim();
        }
        return this.pathStr;
    }

    /**
     * @return the uid of the application
     */
    public int getUID() {
        return this.uid[0];
    }

    /**
     * @return the pid of the application
     */
    public int getPID() {
        return this.pid[0];
    }

    /**
     * @return true if the application is parent of the monitored application or false otherwise
     */
    public boolean isParent() {
        return this.iamparent[0] != 1;
    }

    /**
     * @param processName the name of the process
     */
    public void setProcessName(final String processName) {
        if (processName == null) {
            pushChars("",this.name);
        } else {
            pushChars(processName,this.name);
        }
        this.nameStr = null;
    }

    /**
     * @param path the path to the process exe (the exe file included)
     */
    public void setPath(final String path) {
        if (path == null) {
            pushChars("",this.path);
        } else {
            pushChars(path,this.path);
        }
        this.pathStr = null;
    }

    /**
     * @param uid the application unique id
     */
    public void setUID(final int uid) {
        this.uid[0] = uid;
    }


    /**
     * @param pid the pid of the application
     */
    public void setPID(final int pid) {
        this.pid[0] = pid;
    }

    /**
     * Mark the handshake as if this application is the parent application of the monitored application. In this case
     * the given pid is the pid of the parent, and the child processes are searched for
     *
     * @param isParent true if this is a parent or false otherwise
     */
    public void setParent(final boolean isParent) {
        this.iamparent[0] = (short)(isParent ? 1 : 0);
    }

    private static void pushChars(String s, final char[] c) {
        Arrays.fill(c,(char)0);
        if (s == null) s = "";
        int len = s.length();
        Arrays.fill(c,(char)0);
        if (len > c.length) len = c.length;
        s.getChars(0,len,c,0);
    }
}
