/*
 * File ChannelCallback2.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.accesslayer;

import de.desy.tine.dataUtils.TDataType;

/**
 * <code>ChannelCallback2</code> is an extension of the channel callback which provides more bullet proof mechanism to
 * catch all updates and states. For backward compatibility the extra methods are kept in a separate interface.
 *
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 * @deprecated use {@link ChannelListener}
 */
@Deprecated
public interface ChannelCallback2 extends ChannelCallback {

    /**
     * The channel value was updated and is given as a parameter.
     *
     * @param channel the channel which value was updated
     * @param rawValue the value of the channel at the time when the event was triggered
     * @deprecated use {@link ChannelListener#updateValue(ChannelReadEvent)} instead
     */
    @Deprecated
    public void updateValue(Channel channel, TDataType rawValue);

    /**
     * The channel status was updated. The status that triggered the call is given as a parameter.
     *
     * @param channel the channel which value was updated
     * @param value the value of the channel at the time when the event was triggered
     * @param status the status that triggered the callback
     * @deprecated use {@link ChannelListener#updateError(ChannelReadEvent)} instead
     */
    @Deprecated
    public void updateErrorStatus(Channel channel, TDataType value, int status);
}
