/*
 * File FECInfo.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

/*
 * Created on Nov 20, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package de.desy.tine.addrUtils;
import java.io.*;
import de.desy.tine.definitions.TErrorList;
import de.desy.tine.definitions.TStrings;
import de.desy.tine.server.logger.*;
import de.desy.tine.server.properties.TStockProperties;

/**
 * \internal
 * 
 * @author duval
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class FECInfo
{
  private String os;
  private String desc;
  private String loc;
  private String hdw;
  private String ver;
  private String resp;
  private static int sizeInBytes = 
    TStrings.FEC_OS_SIZE + 
    TStrings.FEC_DESC_SIZE +
    TStrings.FEC_LOCATION_SIZE +
    TStrings.FEC_VERSION_SIZE +
    TStrings.FEC_HDW_SIZE +
    TStrings.FEC_RESP_SIZE;
  public static int getSizeInBytes() { return sizeInBytes; }
  public FECInfo(String fecOS,String fecDescription,String fecLocation,String fecHardware,String fecVersion,String responsiblePersons)
  { // constructor used in queries for information
    os = fecOS == null ? "unknown" : fecOS.trim();
    desc = fecDescription == null ? "unknown" : fecDescription.trim();
    loc = fecLocation == null ? "unknown" : fecLocation.trim();
    hdw = fecHardware == null ? "unknown" : fecHardware.trim();
    ver = fecVersion == null ? "unknown" : fecVersion.trim();
    resp = responsiblePersons == null ? "unknown" : responsiblePersons.trim();
  }
  public FECInfo(String fecOS,String fecDescription,String fecLocation,String fecHardware,String responsiblePersons)
  {
    os = fecOS == null ? "unknown" : fecOS.trim();
    desc = fecDescription == null ? "unknown" : fecDescription.trim();
    loc = fecLocation == null ? "unknown" : fecLocation.trim();
    hdw = fecHardware == null ? "unknown" : fecHardware.trim();
    ver = TStockProperties.getStockSrvVersion().trim();
    resp = responsiblePersons == null ? "unknown" : responsiblePersons.trim();
  }
  private void pushStringToBytes(String s,byte[] b,int maxlen)
  {
    int i,len;
    if (s != null)
    {
      if((len=s.length()) > maxlen) len = maxlen;
      System.arraycopy(s.getBytes(),0,b,0,len);
    }
    else len = 0;
    for (i=len; i<maxlen; i++) b[i] = 0;    
  }
  public byte[] toByteArray()
  {
    ByteArrayOutputStream dBuffer = new ByteArrayOutputStream(sizeInBytes);
    try
    {
      DataOutputStream ds = new DataOutputStream(dBuffer);
      byte[] bstr = new byte[64];
      pushStringToBytes(os, bstr, TStrings.FEC_OS_SIZE);
      ds.write(bstr,0,TStrings.FEC_OS_SIZE);
      pushStringToBytes(desc, bstr, TStrings.FEC_DESC_SIZE);
      ds.write(bstr,0,TStrings.FEC_DESC_SIZE);
      pushStringToBytes(loc, bstr, TStrings.FEC_LOCATION_SIZE);
      ds.write(bstr,0,TStrings.FEC_LOCATION_SIZE);
      pushStringToBytes(ver, bstr, TStrings.FEC_VERSION_SIZE);
      ds.write(bstr,0,TStrings.FEC_VERSION_SIZE);
      pushStringToBytes(hdw, bstr, TStrings.FEC_HDW_SIZE);
      ds.write(bstr,0,TStrings.FEC_HDW_SIZE);
      pushStringToBytes(resp, bstr, TStrings.FEC_RESP_SIZE);
      ds.write(bstr,0,TStrings.FEC_RESP_SIZE);
      ds.close(); 
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("FECInfo","could not prepare byte stream",TErrorList.code_failure,e,0);
      return null;
    }  
    return dBuffer.toByteArray();
  }
  public void toStruct(byte[] hByteArray)
  {
    try
    {
      ByteArrayInputStream dis = new ByteArrayInputStream(hByteArray);
      DataInputStream ds = new DataInputStream(dis);
      byte[] bstr = new byte[64];
      ds.read(bstr,0,TStrings.FEC_OS_SIZE);
      os = new String(bstr);
      ds.read(bstr,0,TStrings.FEC_DESC_SIZE);
      desc = new String(bstr);
      ds.read(bstr,0,TStrings.FEC_LOCATION_SIZE);
      loc = new String(bstr);
      ds.read(bstr,0,TStrings.FEC_VERSION_SIZE);
      ver = new String(bstr);
      ds.read(bstr,0,TStrings.FEC_HDW_SIZE);
      hdw = new String(bstr);
      ds.read(bstr,0,TStrings.FEC_RESP_SIZE);
      resp = new String(bstr);
      ds.close(); dis.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("FECInfo","could not prepare byte stream",TErrorList.code_failure,e,0);
    }  
  }
  public String toString()
  {
    String msg = new String("FEC os   : " + os);
    msg = msg.concat("\nFEC loc  : " + loc);
    msg = msg.concat("\nFEC hdw  : " + hdw);
    msg = msg.concat("\nFEC ver  : " + ver);
    msg = msg.concat("\nFEC resp : " + resp);
    msg = msg.concat("\nFEC desc : " + desc);
    return msg;  
  }
  public String getDescription()
  {
    return desc;
  }
  public String getDesc()
  {
    return desc;
  }
  public String getHardware()
  {
    return hdw;
  }
  public String getHdw()
  {
    return hdw;
  }
  public String getLocation()
  {
    return loc;
  }
  public String getLoc()
  {
    return loc;
  }
  public String getOs()
  {
    return os;
  }
  public String getResponsible()
  {
    return resp;
  }
  public String getResp()
  {
    return resp;
  }
  public String getVersion()
  {
    return ver;
  }
  public String getVer()
  {
    return ver;
  }
 }
