/*
 * File GrpMbr.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.addrUtils;

import de.desy.tine.structUtils.TTaggedStructure;

public class GrpMbr extends TTaggedStructure
{
  public void clear()
  {
    serverString = null;
    devPrefixString = null;
    devPostfixString = null;
  }
  private char[] server = new char[32];
  private String serverString;
  private char[] devPrefix = new char[8];
  private String devPrefixString;
  private char[] devPostfix = new char[8];
  private String devPostfixString;
  private int[] index = new int[1];
  private void initStructure()
  {
    addField(server,"server");
    addField(devPrefix,"devPrefix");
    addField(devPostfix,"devPostfix");
    addField(index,"index");
    initDone();    
  }
  public GrpMbr()
  {
    super();
    initStructure();
  }
  public GrpMbr(GrpMbr reference)
  {
    super();
    initStructure();
    System.arraycopy(reference.server, 0, server, 0, 32);    
    System.arraycopy(reference.devPrefix, 0, devPrefix, 0, 8);    
    System.arraycopy(reference.devPostfix, 0, devPostfix, 0, 8);
    index[0] = reference.index[0];
  }
  public String getServer()
  {
    if (serverString == null) serverString = new String(server).trim();
    return serverString;
  }
  public String getDevPrefix()
  {
    if (devPrefixString == null) devPrefixString = new String(devPrefix).trim();
    return devPrefixString;
  }
  public String getDevPostfix()
  {
    if (devPostfixString == null) devPostfixString = new String(devPostfix).trim();
    return devPostfixString;
  }
  public int getIndex()
  {
    return index[0];
  }
  public void setServer(String server) 
  {
    pushChars(server,this.server);
    this.serverString = new String(server);
  }
  public void setDevPrefix(String devPrefix) 
  {
    pushChars(devPrefix,this.devPrefix);
    this.devPrefixString = new String(devPrefix);
  }
  public void setDevPostfix(String devPostfix) 
  {
    pushChars(devPostfix,this.devPostfix);
    this.devPostfixString = new String(devPostfix);
  }
  public void setAppend(int index) 
  {
    this.index[0] = index;
  } 
}
