/*
 * File ApcBlk.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.alarmUtils;

import de.desy.tine.definitions.TStrings;
import de.desy.tine.structUtils.TTaggedStructure;

public class ApcBlk extends TTaggedStructure
{
  private int[] sessionId = new int[1];
  private int[] event = new int[1];
  private char[] ext = new char[16]; // mostly unused;
  private char[] ctx = new char[TStrings.CONTEXT_NAME_SIZE];
  private char[] exp = new char[TStrings.EXPORT_NAME_SIZE];
  private char[] dev = new char[TStrings.DEVICE_NAME_SIZE];
  private char[] prp = new char[TStrings.PROPERTY_NAME_SIZE];
  private char[] tag = new char[TStrings.TAG_NAME_SIZE];
  private int[] siz = new int[1];
  private short[] fmt = new short[1];
  private short[] reserved = new short[1];
  private void initStructure()
  {
    addField(sessionId,"sessionId");
    addField(event,"event");
    addField(ext,"ext");
    addField(ctx,"context");
    addField(exp,"export");
    addField(dev,"device");
    addField(prp,"property");
    addField(tag,"tag");
    addField(siz,"size");
    addField(fmt,"format");
    addField(reserved,"reserved");
    initDone();    
  }
  ApcBlk()
  {
    super("APCBLKr4");
    initStructure();
  }
  ApcBlk(int session,int event,String context,String server,String device,String property,int size,short format,String tag)
  {
    super("APCBLKr4");
    this.event[0] = event;
    sessionId[0] = session;
    pushChars(context,ctx);
    pushChars(server,exp);
    pushChars(device,dev);
    pushChars(property,prp);
    siz[0] = size;
    fmt[0] = format;
    pushChars(tag,this.tag);
    initStructure();
  }
}
