/*
 * File CasAlarmSys.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.alarmUtils;

import de.desy.tine.definitions.TStrings;
import de.desy.tine.structUtils.TTaggedStructure;

public class CasAlarmSys extends TTaggedStructure
{ // CasAlSys
//  int number;
//  int column;
//  int row;
//  char tag[ALARM_TAG_SIZE];
//  char subsystem[SUBSYSTEM_NAME_SIZE];
  
	public void clear() {
		subsystemString = null;
		tagString = null;
	}

	private char[] subs = new char[TStrings.SUBSYSTEM_NAME_SIZE]; /* calling subsystem */
	private String subsystemString;
	private char[] tag = new char[TStrings.ALARM_TAG_SIZE]; /* calling alarm tag */
	private String tagString;
	private int[] number = new int[1];
	private int[] column = new int[1];
	private int[] row = new int[1];
	
	private void initStructure() {
		addField(number,"number");
		addField(column,"column");
		addField(row,"row");
		addField(tag,"tag");
		addField(subs,"subSystem");
		initDone();
	}

	public CasAlarmSys() {
		super("CasAlSys");
		initStructure();
	}

	public CasAlarmSys(CasAlarmSys reference) {
		super("CasAlSys");
		initStructure();
		row[0] = reference.row[0];
		column[0] = reference.column[0];
		number[0] = reference.number[0];
		setSubsystem(reference.getSubsystem());
		setTag(reference.getTag());
	}

	
	public int getRow() {
		return row[0];
	}
	
	public void setRow(int row) {
		this.row[0] = row;
	}
	
	public int getColumn() {
		return column[0];
	}
	
	public void setColumn(int column) {
		this.column[0] = column;
	}
	
	public int getNumber() {
		return number[0];
	}
	
	public void setNumber(int number) {
		this.number[0] = number;
	}

	public String getSubsystem() {
		if (subsystemString == null)
			subsystemString = new String(subs).trim();
		return subsystemString;
	}

	public void setSubsystem(String subsystem) {
		pushChars(subsystem,subs);
		subsystemString = new String(subs);
	}

	public String getTag() {
		if (tagString == null)
			tagString = new String(tag).trim();
		return tagString;
	}

	public void setTag(String tag) {
		pushChars(tag,this.tag);
		tagString = new String(tag);
	}
	
}
