/*
 * File CasSrvInfo.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.alarmUtils;

import de.desy.tine.structUtils.TTaggedStructure;
import de.desy.tine.definitions.TStrings;

public class CasSrvInfo extends TTaggedStructure
{
  private char[] ctx = new char[TStrings.CONTEXT_NAME_SIZE];
  private String contextString = null;
  private char[] srv = new char[TStrings.EXPORT_NAME_SIZE];
  private String serverString = null;
  private char[] sub = new char[TStrings.SUBSYSTEM_NAME_SIZE];
  private String subString = null;
  private char[] ext = new char[16];
  private String extString = null;
  private int[] sevLvl = new int[1];
  private int[] arcLvl = new int[1];
  private int[] retLvl = new int[1];
  private int[] almLvl = new int[1];
  private int[] almSys = new int[1];
  public void clear()
  {
    contextString = null;
    serverString = null;
    subString = null;
    extString = null;
    sevLvl[0] = 0;
    arcLvl[0] = 0;
    almLvl[0] = 0;
    almSys[0] = 0;
  }
  private void initStructure()
  {
    addField(ctx,"devContext");
    addField(srv,"devServer");
    addField(sub,"subSystem");
    addField(ext,"arcExt");
    addField(sevLvl,"SeverityLevel");
    addField(arcLvl,"ArchiveLevel");
    addField(retLvl,"Retention");
    addField(almLvl,"AlarmLevel");
    addField(almSys,"AlarmSystem");
    initDone();    
  }
  CasSrvInfo()
  {
    super("CQS");
    initStructure();
  }
  CasSrvInfo(CasSrvInfo reference)
  {
    super("CQS");
    initStructure();
    System.arraycopy(reference.ctx, 0, ctx, 0, ctx.length);    
    System.arraycopy(reference.srv, 0, srv, 0, srv.length);    
    System.arraycopy(reference.sub, 0, sub, 0, sub.length);    
    System.arraycopy(reference.ext, 0, ext, 0, ext.length);    
    sevLvl[0] = reference.sevLvl[0];   
    arcLvl[0] = reference.arcLvl[0];   
    retLvl[0] = reference.retLvl[0];   
    almLvl[0] = reference.almLvl[0];   
    almSys[0] = reference.almSys[0];   
  }
  public String getContext()
  {
    if (contextString == null) contextString = new String(ctx).trim();
    return contextString;
  }  
  public String getServer()
  {
    if (serverString == null) serverString = new String(srv).trim();
    return serverString;
  }  
  public String getSubsystem()
  {
    if (subString == null) subString = new String(sub).trim();
    return subString;
  }  
  public String getArchiveExtension()
  {
    if (extString == null) extString = new String(ext).trim();
    return extString;
  }
  public int getSeverityLevel() { return sevLvl[0]; }
  public int getArchiveLevel() { return arcLvl[0]; }
  public int getRetention() { return retLvl[0]; }
  public int getAlarmLevel() { return almLvl[0]; }
  public int getAlarmSystem() { return almSys[0]; }
}
