/*
 * File TBitfieldRegistry.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.bitfieldUtils;

import java.util.HashMap;

public class TBitfieldRegistry
{
  private static HashMap<String, TBitfield> bfRegistry = new HashMap<String, TBitfield>();
  private static String rmvDeviceName(String fullName)
  { // might pass fullName of the form /<context>/<server>/<device>
    if (fullName == null) return "this";
    String[] parts = fullName.split("/");
    if (parts.length == 1) return parts[0];
    if (fullName.startsWith("/")) // context given 
      return "/" + parts[1] + "/" + parts[2];
    return parts[0];
  }
  public static TBitfield[] getRegisteredBitfields()
  {
    return bfRegistry.values().toArray(new TBitfield[0]);
  }
  public static void add(TBitfield bitfield)
  {
    if (bitfield == null) return; //TODO: throw exception
    String srv = rmvDeviceName(bitfield.getSrvName());
    String key = srv + "/" +  bitfield.getName();
    if (!bfRegistry.containsKey(key))
    {
      bfRegistry.put(key, bitfield);
    }
  }
  public static TBitfield getBitfield(String context, String server, String tag)
  {
    if (tag == null) return null;
    if (context == null) context = "default";
    if (server == null) server = "this"; 
    return getBitfield("/"+context+"/"+server+"/"+tag);
  }
  public static TBitfield getBitfield(String server, String tag)
  {
    if (tag == null) return null;
    if (server == null) server = "this"; 
    return getBitfield("/"+server+"/"+tag);
  }
  public static TBitfield getBitfield(String name)
  {
    if (name == null) return null;
    if (bfRegistry.isEmpty()) return null;
    if (bfRegistry.containsKey(name))
    {
      return (TBitfield) bfRegistry.get(name);
    }
    boolean useAny = false;
    if (name.length() == 0 && bfRegistry.size() == 1)
    {
      useAny = true;
    }
    if (!name.contains("/"))
    { // naked bitfield tag name given
      String keytag = "/" + name;
      String[] keys = bfRegistry.keySet().toArray(new String[0]);
      if (keys == null) return null;
      for (String s : keys)
      {
        if (s.endsWith(keytag) || useAny)
        {
          return (TBitfield) bfRegistry.get(s);
        }
      }
    }
    return null;
  }
}
