/*
 * File TCallback.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

//: TCallback.java
package de.desy.tine.client;

/**
 * Interface to a TLink callback class for use in asynchronous links
 * 
 * This class provides an inteface to a TCallback object whose callback 
 * method will be called which the link state of an asynchronous link has 
 * changed, either due to incoming data or non-zero status code.
 *  
 * \b Example:
 * 
 * \include eg_attach1.java
 */
public interface TCallback
{
  /**
   * The method which will be called when new data arrive.
   * 
   * @param LinkIndex is either the index returned when calling the 
   * TLink.attach(short mode,TCallback f) or 
   * TLink.attach(short mode,TCallback f,int pollrate) methods, or the
   * index supplied when calling the 
   * TLink.attach(short mode,TCallback f,int pollrate,int notificationId)
   * method.
   * @param LinkStatus is the current link status code.  The caller should
   * always inspect this value to determine the validity of the data.
   * 
   * @see de.desy.tine.client.TLink#attach
   */
  public void callback(int LinkIndex,int LinkStatus);    
}