/*
 * File TWildcardMembersCallback.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

/*
 * Created on Jul 12, 2006
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package de.desy.tine.client;

import de.desy.tine.dataUtils.*;
import de.desy.tine.definitions.*;
import de.desy.tine.server.logger.*;

public class TWildcardMembersCallback implements TLinkCallback
{
  public void callback(TLink link)
  {
    int lstid = link.callbackId;
    TWildcardLink wc = link.twcl;
    int wccc;
    int i, k=0, len;
    int debug = TLinkFactory.debugLevel;
    if (wc == null || wc.status == null)
    {
      MsgLog.log("TWildcardMembersCallback", "called with no wildcard link",TErrorList.code_failure,null,0);
      return;
    }
    if (lstid < 0 || lstid >= wc.status.length)
    {
      MsgLog.log("TWildcardMembersCallback", "invalid member id "+lstid,TErrorList.code_failure,null,0);
      return;    	
    }
    if (wc.status[lstid] == TErrorList.not_ready) wc.numPending--;
    wc.status[lstid] = link.linkStatus;
    if (debug > 1) 
      DbgLog.log("TWildcardMembersCallback","grp cb : (" + lstid + ") : <" + link.linkStatus + "> pending : " + wc.numPending + " from " + wc.length);    
    if (link.linkStatus == TErrorList.link_blacklisted) link.close();
    if (link.linkStatus == TErrorList.illegal_property) link.close();
    if (link.linkStatus == TErrorList.illegal_format) link.close();

    if (wc.numPending > 0) return;

    if (debug > 1) DbgLog.log("TWildcardMembersCallback","wildcard group all in !!!!!!!!");
    TDataType pd = wc.parent.getOutputDataObject();
    TDataType md = null;
    int maxlen = pd.getArrayLength();
    try 
    {
	    for (i=0, k=0; i<wc.links.length && k<maxlen; i++)
	    { // each member link has a subset of the list to return ...
	    	md = wc.links[i].getOutputDataObject();
	    	len = md.getCompletionLength();
	    	pd.putData(k,md.getDataObject(),len,0,false);
	      pd.dTimestamp = md.dTimestamp;
	      pd.sysDataStamp = md.sysDataStamp;
	      pd.usrDataStamp = md.usrDataStamp;
	    	k += len;
	    }
  	} 
    catch (Exception x)
  	{ // if anything bizarre happens we still have to notify !
  		x.printStackTrace();
  		link.linkStatus = TErrorList.code_failure;
  	}
    wc.numActive = wc.links.length;
    wccc = link.linkStatus;
    wc.numPending = wc.numActive;
    for (i=0; i<wc.length; i++)
    {
      if (wc.status[i] == TErrorList.illegal_format) continue;
      if (wc.status[i] == TErrorList.illegal_property) continue;
      if (wc.status[i] == TErrorList.link_blacklisted) continue;
      if (wccc != 0 && wc.status[i] == 0) wccc = 0; /* at least one success */
      wc.status[i] = TErrorList.not_ready;
    }
    //wc.parent.dOutput.setArrayLength(k);
    wc.parent.linkStatus = wccc;
    wc.parent.linkStatusSource = link.linkStatusSource;
    wc.parent.getOutputDataObject().dCompletionLength = k;
    wc.parent.hasObtainedStatus = true;
    if (wc.tlcb != null) wc.tlcb.callback(wc.parent);
    else if (wc.tcb != null) wc.tcb.callback(wc.parent.linkId,wccc);
    else wc.parent.wakeUpCall();
    wc.canNotify = true;
    wc.checkActive();
    return;
  }
}
