/*
 * File TCommandItem.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.console;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import de.desy.tine.definitions.TAccess;

public class TCommandItem
{
  private String cmdString = null;
  private String cmdDesc = null;
  public String getCommandString() { return cmdString; }
  public String getCommandDescription() { return cmdDesc; }
  private Method cmdR = null;
  private Method cmdW = null;
  private int access = TAccess.CA_NULL;
  public static boolean getBoolean(String value)
  {
  	if (value == null) return false;
  	boolean bvalue = false;
  	if (value.compareToIgnoreCase("true") == 0 ||
  			value.compareToIgnoreCase("yes") == 0 ||
  			value.compareToIgnoreCase("on") == 0)
  	{
  		bvalue = true;
  	}
  	else
  	{
  		try
  		{
  			int v = Integer.parseInt(value);
  			bvalue = (v != 0);
  		}
  		catch (Exception ignore) 
  		{ 
  			bvalue = false; 
  		}
  	}
  	return bvalue;
  }
  public boolean canSet() { return TAccess.isWrite((byte)access); }
  public boolean canGet() { return TAccess.isRead((byte)access); }
  public TCommandItem(String command,String description,Method method)
  {
    cmdString = command;
    cmdDesc = description;
    cmdR = method;
    cmdW = method;
    access = TAccess.CA_READ;
  }
  public TCommandItem(String command,String description,Method method,int access)
  {
    cmdString = command;
    cmdDesc = description;
    cmdR = method;
    cmdW = method;
    this.access = access;
  }
  public TCommandItem(String command,String description,Method methodRead,Method methodWrite,int access)
  {
    cmdString = command;
    cmdDesc = description;
    cmdR = methodRead;
    cmdW = methodWrite;
    this.access = access;
  }
  public void invoke()
  {
    invoke(TAccess.CA_READ);
  }
  public void invoke(short access)
  {
    try
    {
      if (TAccess.isWrite((byte)access))
        cmdW.invoke((Object)null, (Object[])null);
      else
        cmdR.invoke((Object)null, (Object[])null);
    }
    catch (Exception e)
    {
      System.out.println("cannot invoke "+cmdString+" : "+e.getMessage());
    }
  }
  public void invoke(int value) throws IllegalAccessException, IllegalArgumentException, InvocationTargetException
  {
    invoke(value,TAccess.CA_WRITE);
  }
  public void invoke(int value,short access) throws IllegalAccessException, IllegalArgumentException, InvocationTargetException
  {
//    try
    {
      if (TAccess.isWrite((byte)access))
        cmdW.invoke((Object)null, value);
      else
        cmdR.invoke((Object)null, value);
    }
//    catch (Exception e)
//    {
//      System.out.println("cannot invoke "+cmdString+" : "+e.getMessage());
//    }
  }
  public void invoke(String value)
  {
    invoke(value,TAccess.CA_WRITE);
  }
  public void invoke(String value,short access)
  {
    try
    {
      if (TAccess.isWrite((byte)access))
        cmdW.invoke((Object)null, value);
      else
        cmdR.invoke((Object)null, value);
    }
    catch (Exception e)
    {
      System.out.println("cannot invoke "+cmdString+" : "+e.getMessage());
    }
  }
}
