/*
 * File TChannelArray.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

/*
 * Created on Jan 13, 2007
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package de.desy.tine.dataUtils;

import de.desy.tine.types.TCompoundDataObject;

public final class TChannelArray
{
  public static float[] getChannelArray(float[] dataArray,int start,int length)
  {
    if (dataArray == null) return dataArray;
    if (start == 0 && length == dataArray.length) return dataArray;
    if (start >= 0 && length <= dataArray.length)
    {
      float[] subArray = new float[length];
      for (int i=0; i<length; i++)
        subArray[i] = dataArray[(start+i) % dataArray.length];
      return subArray;
    }
    return null;
  }
  public static double[] getChannelArray(double[] dataArray,int start,int length)
  {
    if (dataArray == null) return dataArray;
    if (start == 0 && length == dataArray.length) return dataArray;
    if (start >= 0 && length <= dataArray.length)
    {
      double[] subArray = new double[length];
      for (int i=0; i<length; i++)
        subArray[i] = dataArray[(start+i) % dataArray.length];
      return subArray;
    }
    return null;
  }
  public static int[] getChannelArray(int[] dataArray,int start,int length)
  {
    if (dataArray == null) return dataArray;
    if (start == 0 && length == dataArray.length) return dataArray;
    if (start >= 0 && length <= dataArray.length)
    {
      int[] subArray = new int[length];
      for (int i=0; i<length; i++)
        subArray[i] = dataArray[(start+i) % dataArray.length];
      return subArray;
    }
    return null;
  }
  public static short[] getChannelArray(short[] dataArray,int start,int length)
  {
    if (dataArray == null) return dataArray;
    if (start == 0 && length == dataArray.length) return dataArray;
    if (start >= 0 && length <= dataArray.length)
    {
      short[] subArray = new short[length];
      for (int i=0; i<length; i++)
        subArray[i] = dataArray[(start+i) % dataArray.length];
      return subArray;
    }
    return null;
  }
  public static byte[] getChannelArray(byte[] dataArray,int start,int length)
  {
    if (dataArray == null) return dataArray;
    if (start == 0 && length == dataArray.length) return dataArray;
    if (start >= 0 && length <= dataArray.length)
    {
      byte[] subArray = new byte[length];
      for (int i=0; i<length; i++)
        subArray[i] = dataArray[(start+i) % dataArray.length];
      return subArray;
    }
    return null;
  }
  public static TCompoundDataObject[] getChannelArray(TCompoundDataObject[] dataArray,int start,int length)
  {
    if (dataArray == null) return dataArray;
    if (start == 0 && length == dataArray.length) return dataArray;
    if (start >= 0 && length <= dataArray.length)
    {
      TCompoundDataObject[] subArray = new TCompoundDataObject[length];
      for (int i=0; i<length; i++)
        subArray[i] = dataArray[(start+i) % dataArray.length];
      return subArray;
    }
    return null;
  }

}
