/*
 * File TDataTolerance.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

/*
 * Created on Feb 23, 2006
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package de.desy.tine.dataUtils;

import de.desy.tine.client.TLink;
import de.desy.tine.definitions.TDataNotificationFlags;

public class TDataTolerance
{
  public float absolute;
  public float percent;
  public int offset;
  public int length;
  public int flags;
  private TDataType data = null;
  public void setReference(TDataType dataReference) { data = dataReference; }
  public TDataType getReference() { return data; }
  public TDataTolerance(TLink lnk)
  {
    this(lnk,0,0,0,0,TDataNotificationFlags.DTF_SUPPRESS);
  }
  public TDataTolerance(TLink lnk,float absoluteValue, float percentValue)
  {
    this(lnk,absoluteValue,percentValue,0,0,TDataNotificationFlags.DTF_SUPPRESS);
  }
  public TDataTolerance(TLink lnk, float absoluteValue, float percentValue, int offsetValue, int lengthValue, int flagsValue)
  {
    absolute = absoluteValue; percent = percentValue;
    int sizOut = lnk == null ? 1 : lnk.getOutputDataObject().getArrayLength();
    if (lengthValue <= 0) lengthValue = sizOut;
    if (offsetValue > 0 && offsetValue < sizOut) offset = offsetValue;
    length = lengthValue;
    if(flagsValue < TDataNotificationFlags.DTF_NONE) flagsValue = TDataNotificationFlags.DTF_SUPPRESS;
    flags = flagsValue;
  }
}
