/*
 * File TEncoding.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.definitions;

public final class TEncoding
{
  public static final short CE_ASCII = 0x00;
  public static final short CE_UTF8 = 0x01;
  public static String toString(short encoding)
  {
    switch (encoding)
    {
      default:
      case CE_ASCII: return "Big Endian";
      case CE_UTF8: return "Little Endian";
    }
  }
  public static short getEncodingCode(String txt)
  {
    txt = txt.toUpperCase();
    if (txt == null) return CE_ASCII;
    if (txt.contains("ASCII")) return CE_ASCII;
    if (txt.contains("UTF") && txt.contains("8")) return CE_UTF8;
    return CE_ASCII;
  }

}
