/*
 * File TTransferCodes.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.definitions;

public final class TTransferCodes
{
  static public final int CX_NULL = 0x00;
  static public final int CX_RESPONSE = 0x01;
  static public final int CX_STALE = 0x02;
  static public final int CX_HEARTBEAT = 0x04;
  static public final int CX_EVENT = 0x08;
  static public final int CX_TIMER = 0x10;
  static public final int CX_MESSAGE = 0x20;
  static public final int CX_ERROR = 0x40;
  static public final int CX_ATTACH = 0x80;
  static public final int CX_RETRY = 0x100;
  static public String toString(int code)
  {
    String s = "";
    String sep = "";
    if ((code & CX_RESPONSE) == CX_RESPONSE) s += "response";
    if (s.length() > 0 && sep.length() == 0) sep = "|";
    if ((code & CX_STALE) == CX_STALE) s += (sep+"stale");
    if (s.length() > 0 && sep.length() == 0) sep = "|";
    if ((code & CX_HEARTBEAT) == CX_HEARTBEAT) s += (sep+"heartbeat");
    if (s.length() > 0 && sep.length() == 0) sep = "|";
    if ((code & CX_EVENT) == CX_EVENT) s += (sep+"event");
    if (s.length() > 0 && sep.length() == 0) sep = "|";
    if ((code & CX_TIMER) == CX_TIMER) s += (sep+"timer");
    if (s.length() > 0 && sep.length() == 0) sep = "|";
    if ((code & CX_MESSAGE) == CX_MESSAGE) s += (sep+"message");
    if (s.length() > 0 && sep.length() == 0) sep = "|";
    if ((code & CX_ERROR) == CX_ERROR) s += (sep+"error");
    if (s.length() > 0 && sep.length() == 0) sep = "|";
    if ((code & CX_ATTACH) == CX_ATTACH) s += (sep+"attach");
    if (s.length() > 0 && sep.length() == 0) sep = "|";
    if ((code & CX_RETRY) == CX_RETRY) s += (sep+"retry");
    return s;
  }
}
