/*
 * File SwapUtil.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

/*
 * Created on Feb 1, 2005
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package de.desy.tine.endianUtils;

/**
 * \internal 
 * 
 * New utilities for big endian / little endian conversion. This is an
 * Extension/Replacement for the class Swap. Adds swap utilities for int,long,
 * double, and float which don't need intermediate byte arrays. These utilities
 * are packed into a separate class, because they would disturb Swap (where long
 * means int) otherwise.
 * 
 * @author MCS
 */
public class SwapUtil
{
  /**
   * Swaps a short integer byte-wise.
   * 
   * @param value
   *          Input value.
   * @return Byte-swapped value.
   */
  public static short swapShort(short value)
  {
    return (short) (((value >> 8) & 0xff) | ((value & 0xff) << 8));
  }
  /**
   * Swaps an integer byte-wise.
   * 
   * @param value
   *          Input value.
   * @return Byte-swapped value.
   */
  public static int swapInt(int value)
  {
    return Integer.reverseBytes(value);
  }
  /**
   * Swaps a long integer bytewise.
   * 
   * @param value
   *          Input value.
   * @return Byte-swapped value.
   */
  public static long swapLong(long value)
  {
    return Long.reverseBytes(value);
  }
  /**
   * Swaps a double bytewise.
   * 
   * @param value
   *          Input value.
   * @return Byte-swapped value.
   */
  public static double swapDouble(double value)
  {
    return Double.longBitsToDouble(swapLong(Double.doubleToLongBits(value)));
  }
  public static double swapDouble(long value)
  {
    return Double.longBitsToDouble(swapLong(value));
  }
  /**
   * Swaps a float bytewise.
   * 
   * @param value
   *          Input value.
   * @return Byte-swapped value.
   */
  public static float swapFloat(float value)
  {
    return Float.intBitsToFloat(swapInt(Float.floatToIntBits(value)));
  }
  public static float swapFloat(int value)
  {
    return Float.intBitsToFloat(swapInt(value));
  }
  /**
   * Unusable Constructor; pure utility class
   */
  private SwapUtil()
  {
  }
}
