package de.desy.tine.eventArchive;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import de.desy.tine.definitions.TStrings;
import de.desy.tine.endianUtils.Swap;

public class ADRB
{
  public int eventNumber;
  public int starttime;
  public int stoptime;
  public String deviceContext = new String(new StringBuffer(TStrings.CONTEXT_NAME_SIZE));
  public String deviceServerName = new String(new StringBuffer(TStrings.EXPORT_NAME_SIZE));
  public String deviceProperty = new String(new StringBuffer(TStrings.PROPERTY_NAME_SIZE));
  public String deviceName = new String(new StringBuffer(TStrings.DEVICE_NAME_SIZE));
  public String deviceTag = new String(new StringBuffer(TStrings.TAG_NAME_SIZE));
  public short dataFormat;
  public short reserved;
  public int dataIndex;
  public int dataDimension;
  public static int sizeInBytes = (3*4 + 2*32 + 2*64 + 16 + 2*2 + 2*4);
  ByteArrayOutputStream dBuffer = new ByteArrayOutputStream(sizeInBytes);
  byte[] hByteArray;
  public byte[] toByteArray()
  {
    try
    {
      dBuffer.reset();
      DataOutputStream ds = new DataOutputStream(dBuffer);
      ds.writeInt(Swap.Int(eventNumber));
      ds.writeInt(Swap.Int(starttime));
      ds.writeInt(Swap.Int(stoptime));
      byte[] bstr = new byte[64];
      byte[] tmp  = deviceContext.getBytes();
      for (int i=0; i<TStrings.CONTEXT_NAME_SIZE; i++) bstr[i] = i < tmp.length ? tmp[i] : 0;
      ds.write(bstr,0,TStrings.CONTEXT_NAME_SIZE);
      tmp  = deviceServerName.getBytes();
      for (int i=0; i<TStrings.EXPORT_NAME_SIZE; i++) bstr[i] = i < tmp.length ? tmp[i] : 0;
      ds.write(bstr,0,TStrings.EXPORT_NAME_SIZE);
      tmp  = deviceProperty.getBytes();
      for (int i=0; i<TStrings.PROPERTY_NAME_SIZE; i++) bstr[i] = i < tmp.length ? tmp[i] : 0;
      ds.write(bstr,0,TStrings.PROPERTY_NAME_SIZE);
      tmp  = deviceName.getBytes();
      for (int i=0; i<TStrings.DEVICE_NAME_SIZE; i++) bstr[i] = i < tmp.length ? tmp[i] : 0;
      ds.write(bstr,0,TStrings.DEVICE_NAME_SIZE);
      tmp  = deviceTag.getBytes();
      for (int i=0; i<TStrings.TAG_NAME_SIZE; i++) bstr[i] = i < tmp.length ? tmp[i] : 0;
      ds.write(bstr,0,TStrings.TAG_NAME_SIZE);
      ds.writeShort(Swap.Short(dataFormat+512));
      ds.writeShort(0);
      ds.writeInt(Swap.Int(dataIndex));
      ds.writeInt(Swap.Int(dataDimension));
      ds.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      System.exit(1);
    }    
    hByteArray = dBuffer.toByteArray();
    return hByteArray;
  }
  public void toStruct()
  {
    try
    {
      ByteArrayInputStream dis = new ByteArrayInputStream(hByteArray);
      DataInputStream ds = new DataInputStream(dis);
      eventNumber = Swap.Int(ds.readInt());
      starttime = Swap.Int(ds.readInt());
      stoptime  = Swap.Int(ds.readInt());
      byte[] bstr = new byte[64];
      ds.read(bstr,0,TStrings.CONTEXT_NAME_SIZE);
      deviceContext = new String(bstr);
      ds.read(bstr,0,TStrings.EXPORT_NAME_SIZE);
      deviceServerName = new String(bstr);
      ds.read(bstr,0,TStrings.PROPERTY_NAME_SIZE);
      deviceProperty = new String(bstr);
      ds.read(bstr,0,TStrings.DEVICE_NAME_SIZE);
      deviceName = new String(bstr);
      ds.read(bstr,0,TStrings.TAG_NAME_SIZE);
      deviceTag = new String(bstr);
      dataFormat = Swap.Short(ds.readShort()-512);
      reserved = Swap.Short(ds.readShort()-512);
      dataIndex = Swap.Int(ds.readInt());
      dataDimension = Swap.Int(ds.readInt());
      ds.close(); dis.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      System.exit(1);
    }    
  }
  public String toString()
  {
    String dmpstr = "Event Number : " + eventNumber;
    dmpstr += "\nStart time : " + starttime;
    dmpstr += "\nStop time : " + stoptime;
    dmpstr += "\nDevice Context: " + deviceServerName;
    dmpstr += "\nDevice Server Name: " + deviceServerName;
    dmpstr += "\nDevice Property: " + deviceProperty;
    dmpstr += "\nDevice Name: " + deviceName;
    dmpstr += "\nDevice Tag: " + deviceTag;
    dmpstr += "\nData Format: " + dataFormat;
    dmpstr += "\nData Index: " + dataIndex;
    dmpstr += "\nData Dimension: " + dataDimension;
    return dmpstr;
  }
}
