package de.desy.tine.eventArchive;
import java.io.*;
import de.desy.tine.definitions.TFormat;
import de.desy.tine.endianUtils.Swap;

public class DHS
{
  public String channelID = new String(new StringBuffer(32));
  public int eventCode;
  public short arraysize;
  public short datatype;
  public float SamplingRate;
  public int numPreTriggerValues;
  public int TimeOffset;
  public int LowCount;
  public int HighCount;
  public String egu = new String(new StringBuffer(16));
  public float LowEGU;
  public float HighEGU;  
  public int datalength;
  public String comment = new String(new StringBuffer(256));
  public static int sizeInBytes = (32 + 4 + 2*2 + 4 + 4*4 + 16 + 4*2 + 4 + 4*64);
  ByteArrayOutputStream dBuffer = new ByteArrayOutputStream(sizeInBytes);
  byte[] hByteArray;
  public String toString()
  {
    String dhs;
    
    dhs = new String(
    	"channelID    : " + channelID + "\n" +
        "eventCode    : " + eventCode + "\n" +
        "arraySize    : " + (datalength > 0 ? datalength : arraysize) + "\n" +
        "dataType     : " + TFormat.toString(datatype) + "\n" +
        "SamplingRate : " + SamplingRate + "\n" +
        "nPreTrigVals : " + numPreTriggerValues + "\n" +
        "timeOffset   : " + TimeOffset + "\n" +
        "lowCount     : " + LowCount + "\n" +
        "highCount    : " + HighCount + "\n" +
        "egu          : " + egu + "\n" + 
        "lowEGU       : " + LowEGU + "\n" +
        "highEGU      : " + HighEGU + "\n" +
        "comment      : " + comment
        );
    return dhs;
  }
  public byte[] toByteArray()
  {
    try
    {
      dBuffer.reset();
      DataOutputStream ds = new DataOutputStream(dBuffer);
      byte[] bstr = new byte[256];
      byte[] tmp  = channelID.getBytes();
      for (int i=0; i<32; i++) bstr[i] = i < tmp.length ? tmp[i] : 0;
      ds.write(bstr,0,32);
      ds.writeInt(Swap.Int(eventCode));
      ds.writeShort(Swap.Short(arraysize));
      ds.writeShort(Swap.Short(datatype)+512);
      ds.write(Swap.Float(SamplingRate),0,4);
      ds.writeInt(Swap.Int(numPreTriggerValues));
      ds.writeInt(Swap.Int(TimeOffset));
      ds.writeInt(Swap.Int(LowCount));
      ds.writeInt(Swap.Int(HighCount));
      tmp  = egu.getBytes();
      for (int i=0; i<16; i++) bstr[i] = i < tmp.length ? tmp[i] : 0;
      ds.write(bstr,0,16);
      ds.write(Swap.Float(LowEGU),0,4);
      ds.write(Swap.Float(HighEGU),0,4);
      ds.writeInt(Swap.Int(datalength));
      tmp  = comment.getBytes();
      for (int i=0; i<256; i++) bstr[i] = i < tmp.length ? tmp[i] : 0;
      ds.write(bstr,0,256);
      ds.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      System.exit(1);
    }	 
    hByteArray = dBuffer.toByteArray();
    return hByteArray;
  }
  public void toStruct()
  {
    try
    {
      byte[] bstr = new byte[256];
      ByteArrayInputStream dis = new ByteArrayInputStream(hByteArray);
      DataInputStream ds = new DataInputStream(dis);
      ds.read(bstr,0,32);
      channelID = new String(bstr).trim();
      eventCode = Swap.Int(ds.readInt());
      arraysize = Swap.Short(ds.readShort());
      datatype = Swap.Short(ds.readShort());
      if (datatype > 512) datatype -= 512;
      ds.read(bstr,0,4);
      SamplingRate = Swap.Float(bstr);
      numPreTriggerValues = Swap.Int(ds.readInt());
      TimeOffset = Swap.Int(ds.readInt());
      LowCount = Swap.Int(ds.readInt());
      HighCount = Swap.Int(ds.readInt());
      ds.read(bstr,0,16);
      int len;
      for (len=0; len < 16 && bstr[len] != 0; len++);
      egu = new String(bstr,0,len);
      ds.read(bstr,0,4);
      LowEGU = Swap.Float(bstr);
      ds.read(bstr,0,4);
      HighEGU = Swap.Float(bstr);
      datalength = Swap.Int(ds.readInt());
      ds.read(bstr,0,256);
      for (len=0; len < 256 && bstr[len] != 0; len++);
      comment = new String(bstr,0,len);
      ds.close(); dis.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      System.exit(1);
    }	 
  }
}
