package de.desy.tine.eventArchive;

import java.io.IOException;
import java.util.Arrays;

import de.desy.tine.client.TLink;
import de.desy.tine.dataUtils.TDataType;
import de.desy.tine.definitions.TAccess;
import de.desy.tine.definitions.TErrorList;
import de.desy.tine.queryUtils.TQuery;
import de.desy.tine.types.NAME16;

public class TEventArchiveDataBase
{
  private static final int nice_query_size = 256;
  private static final int trg_query_size = 64;
  private static ARBQS[] arbLst = new ARBQS[nice_query_size];
  private static DASQS[] dasLst = new DASQS[nice_query_size];
  private static TrgCmp[] tcmpLst = new TrgCmp[trg_query_size];
  private static TrgChn[] tchnLst = new TrgChn[trg_query_size];
  private static TrgView[] trgCfg = new TrgView[trg_query_size];
  public static TrgCmp[] getTrcComponents(String context, String cfgName) throws IOException
  { //TRC.COMPONENTS
    if (context == null || context.length() == 0) return null;
    if (cfgName == null || cfgName.length() == 0) return null;
    if (tcmpLst[0] == null)
    {
      for (int i = 0; i < trg_query_size; i++) tcmpLst[i] = new TrgCmp();
    }
    else
    {
      for (int i = 0; i < trg_query_size; i++) tcmpLst[i].clear();
    }
    TDataType dout = new TDataType(tcmpLst);
    TLink lnk = new TLink("/"+context+"/EVENTS/"+cfgName,"TRC.COMPONENTS",dout, null,TAccess.CA_READ);
    if (lnk.executeAndClose(500) != 0) throw new IOException("link error " + lnk.getLastError());
    int n = dout.getCompletionLength();
    if (n == 0) return null;
    return (TrgCmp[]) Arrays.copyOf(tcmpLst, n);
  }
  /**
   * @deprecated
   * This method doesn't do anything (the property doesn't take a write command)
   * 
   * 
   * @param context
   * @param cfgName
   * @param cmps
   * @return
   */
  public static int setTrcComponents(String context, String cfgName, TrgCmp[] cmps)
  {
    if (context == null || context.length() == 0) return TErrorList.invalid_parameter;
    if (cfgName == null || cfgName.length() == 0) return TErrorList.invalid_parameter;
    TDataType din;
    if (cmps == null || cmps.length == 0)
      din = new TDataType();
    else
      din = new TDataType(cmps);
    TLink lnk = new TLink("/"+context+"/EVENTS/"+cfgName,"TRC.COMPONENTS",null, din,TAccess.CA_WRITE);
    return lnk.executeAndClose(500);    
  }
  public static TrgView[] getTrcConfigs(String context) throws IOException
  { //TRC.COMPONENTS.NAM
    if (context == null || context.length() == 0) return null;
    if (trgCfg[0] == null)
    {
      for (int i = 0; i < trg_query_size; i++) trgCfg[i] = new TrgView();
    }
    else
    {
      for (int i = 0; i < trg_query_size; i++) trgCfg[i].clear();
    }
    TDataType dout = new TDataType(trgCfg);
    TLink lnk = new TLink("/"+context+"/EVENTS","TRC.COMPONENTS.NAM",dout, null,TAccess.CA_READ);
    if (lnk.executeAndClose(500) != 0) throw new IOException("link error " + lnk.getLastError());
    int n = dout.getCompletionLength();
    if (n == 0) return null;
    TrgView[] cfgs = new TrgView[n]; //Arrays.copyOf(trgCfg, n);
    for (int i=0; i<cfgs.length; i++) cfgs[i] = new TrgView(trgCfg[i]);
    return cfgs;
  }
  /**
   * This method updates the entire viewer configs files at the server 
   * 
   * @param context
   * @param configs
   * @return
   */
  public static int setTrcConfigs(String context, TrgView[] configs)
  {
    if (context == null || context.length() == 0) return TErrorList.invalid_parameter;
    TDataType din;
    if (configs == null || configs.length == 0)
      din = new TDataType();
    else
      din = new TDataType(configs);
    TLink lnk = new TLink("/"+context+"/EVENTS","TRC.COMPONENTS.NAM",null, din,TAccess.CA_WRITE);
    return lnk.executeAndClose(500);
  }
  public static TrgView[] getTrcXref(String context) throws IOException
  { //TRC.CHANNELS.NAM
    if (context == null || context.length() == 0) return null;
    if (trgCfg[0] == null)
    {
      for (int i = 0; i < trg_query_size; i++) trgCfg[i] = new TrgView();
    }
    else
    {
      for (int i = 0; i < trg_query_size; i++) trgCfg[i].clear();
    }
    TDataType dout = new TDataType(trgCfg);
    TLink lnk = new TLink("/"+context+"/EVENTS/ALL","TRC.CHANNELS.NAM",dout, null,TAccess.CA_READ);
    if (lnk.executeAndClose(500) != 0) throw new IOException("link error " + lnk.getLastError());
    int n = dout.getCompletionLength();
    if (n == 0) return null;
    TrgView[] cfgs = new TrgView[n]; //Arrays.copyOf(trgCfg, n);
    for (int i=0; i<cfgs.length; i++) cfgs[i] = new TrgView(trgCfg[i]);
    return cfgs;
  }
  /**
   * Get a list of component/file name cross-reference for the designated configuration
   * 
   * @param context
   * @param config
   * @return
   * @throws IOException
   */
  public static TrgView[] getTrcComponentViews(String context,String config) throws IOException
  { //TRC.CHANNELS.NAM
    if (context == null || context.length() == 0) return null;
    if (config == null || config.length() == 0) config = "ALL";
    if (trgCfg[0] == null)
    {
      for (int i = 0; i < trg_query_size; i++) trgCfg[i] = new TrgView();
    }
    else
    {
      for (int i = 0; i < trg_query_size; i++) trgCfg[i].clear();
    }
    TDataType dout = new TDataType(trgCfg);
    TLink lnk = new TLink("/"+context+"/EVENTS/"+config,"TRC.CHANNELS.NAM",dout, null,TAccess.CA_READ);
    if (lnk.executeAndClose(500) != 0) throw new IOException("link error " + lnk.getLastError());
    int n = dout.getCompletionLength();
    if (n == 0) return null;
    TrgView[] cfgs = new TrgView[n]; //Arrays.copyOf(trgCfg, n);
    for (int i=0; i<cfgs.length; i++) cfgs[i] = new TrgView(trgCfg[i]);
    return cfgs;
  }
  /**
   * @deprecated
   * This method will not do anything
   * 
   * @param context
   * @param xref
   * @return
   */
  public static int setTrcXref(String context, TrgView[] xref)
  {
    if (context == null || context.length() == 0) return TErrorList.invalid_parameter;
    TDataType din;
    if (xref == null || xref.length == 0)
      din = new TDataType();
    else
      din = new TDataType(xref);
    TLink lnk = new TLink("/"+context+"/EVENTS","TRC.CHANNELS.NAM",null, din,TAccess.CA_WRITE);
    return lnk.executeAndClose(500);    
  }
  /**
   * This method will assign the component/file name cross-reference list to the given config
   * 
   * @param context
   * @param config
   * @param xref
   * @return
   */
  public static int setTrcComponentViews(String context, String config, TrgView[] xref)
  {
    if (context == null || context.length() == 0) return TErrorList.invalid_parameter;
    if (config == null || config.length() == 0) return TErrorList.invalid_parameter;
    TDataType din;
    if (xref == null || xref.length == 0)
      din = new TDataType();
    else
      din = new TDataType(xref);
    TLink lnk = new TLink("/"+context+"/EVENTS/"+config,"TRC.CHANNELS.NAM",null, din,TAccess.CA_WRITE);
    return lnk.executeAndClose(500);    
  }
  public static TrgChn[] getTrcChannels(String context, String cfgName) throws IOException
  { //TRC.CHANNELS
    if (context == null || context.length() == 0) return null;
    if (cfgName == null || cfgName.length() == 0) return null;
    if (tchnLst[0] == null)
    {
      for (int i = 0; i < trg_query_size; i++) tchnLst[i] = new TrgChn();
    }
    else
    {
      for (int i = 0; i < trg_query_size; i++) tchnLst[i].clear();
    }
    TDataType dout = new TDataType(tchnLst);
    TLink lnk = new TLink("/"+context+"/EVENTS/"+cfgName,"TRC.CHANNELS",dout, null,TAccess.CA_READ);
    if (lnk.executeAndClose(500) != 0) throw new IOException("link error " + lnk.getLastError());
    int n = dout.getCompletionLength();
    if (n == 0) return null;
    TrgChn[] chns = new TrgChn[n]; //Arrays.copyOf(trgCfg, n);
    for (int i=0; i<chns.length; i++) chns[i] = new TrgChn(tchnLst[i]);
    return chns;
  }
  /**
   * This method will assign the channels to the given component
   * 
   * @param context
   * @param component
   * @param chans
   * @return
   */
  public static int setTrcChannels(String context, String component, TrgChn[] chans)
  {
    if (context == null || context.length() == 0) return TErrorList.invalid_parameter;
    if (component == null || component.length() == 0) return TErrorList.invalid_parameter;
    TDataType din;
    if (chans == null || chans.length == 0)
      din = new TDataType();
    else
      din = new TDataType(chans);
    TLink lnk = new TLink("/"+context+"/EVENTS/"+component,"TRC.CHANNELS",null, din,TAccess.CA_WRITE);
    return lnk.executeAndClose(500);    
  }
  public static String[] getTrcComponentNames(String context)
  {
    return TQuery.getDeviceNames(context, "EVENTS", "TRC.COMPONENTS");
  }
  public static ARBQS[] getTriggerTable(String context) throws IOException
  { // "TRIGGER.TBL"
    if (context == null || context.length() == 0) return null;
    if (arbLst[0] == null)
    {
      for (int i = 0; i < nice_query_size; i++) arbLst[i] = new ARBQS();
    }
    else
    {
      for (int i = 0; i < nice_query_size; i++) arbLst[i].clear();
    }
    TDataType dout = new TDataType(arbLst);
    TLink lnk = new TLink("/"+context+"/EVENTSTORE","TRIGGER.TBL",dout, null,
        TAccess.CA_READ);
    if (lnk.executeAndClose(500) != 0) throw new IOException("link error " + lnk.getLastError());
    int n = dout.getCompletionLength();
    if (n == 0) return null;
    return (ARBQS[]) Arrays.copyOf(arbLst, n);
  }
  public static int setTriggerTable(String context,ARBQS[] tableItems)
  {
    if (context == null || context.length() == 0) return TErrorList.invalid_parameter;
    TDataType din;
    if (tableItems == null || tableItems.length == 0)
      din = new TDataType();
    else
      din = new TDataType(tableItems);
    TLink lnk = new TLink("/"+context+"/EVENTSTORE","TRIGGER.TBL",null, din,
        TAccess.CA_WRITE);
    return lnk.executeAndClose(500);
  }
  public static boolean isTriggered(String context)
  {
    if (context == null || context.length() == 0) return false;
    int[] sts = new int[1];
    TDataType dout = new TDataType(sts);
    TLink lnk = new TLink("/"+context+"/EVENTSTORE/ANY","TRIGGER.STATUS",dout, null,TAccess.CA_READ);
    if (lnk.executeAndClose(500) != 0) return false;
    return !(sts[0] == 0);
  }
  public static int resetServer(String context)
  {
    if (context == null || context.length() == 0) return TErrorList.invalid_parameter;
    if (isTriggered(context)) return TErrorList.operation_busy;
    TLink lnk = new TLink("/"+context+"/EVENTSTORE","SRVRESET",null, null,TAccess.CA_WRITE);
    return lnk.executeAndClose(500);
  }
  public static DASQS[] getTriggerDataBase(String context, String trigger) throws IOException
  { // "TRIGGER.DB"
    if (context == null || context.length() == 0) return null;
    if (dasLst[0] == null)
    {
      for (int i = 0; i < nice_query_size; i++) dasLst[i] = new DASQS();
    }
    else
    {
      for (int i = 0; i < nice_query_size; i++) dasLst[i].clear();
    }
    TDataType dout = new TDataType(dasLst);
    TLink lnk = new TLink("/"+context+"/EVENTSTORE/"+trigger,"TRIGGER.DB",dout, null,
        TAccess.CA_READ);
    if (lnk.executeAndClose(500) != 0) throw new IOException("link error " + lnk.getLastError());
    int n = dout.getCompletionLength();
    if (n == 0) return null;
    DASQS[] rtLst = new DASQS[n];
    for (int i = 0; i < n; i++) rtLst[i] = dasLst[i];
    return rtLst;
  }
  public static int setTriggerDataBase(String context,String trigger, DASQS[] triggerItems)
  {
    if (context == null || context.length() == 0) return TErrorList.invalid_parameter;
    TDataType din;
    if (triggerItems == null || triggerItems.length == 0)
      din = new TDataType();
    else
      din = new TDataType(triggerItems);
    TLink lnk = new TLink("/"+context+"/EVENTSTORE/"+trigger,"TRIGGER.DB",null, din,
        TAccess.CA_WRITE);
    return lnk.executeAndClose(500);
  }
  public static Object getTriggerInputData(String context,String file,int format) throws IOException
  { // "TRIGGER.ITEM.DATA"
    if (context == null || context.length() == 0) return null;
    if (file == null || file.length() == 0) return null;
    TDataType dout = new TDataType(nice_query_size,(short)format);
    TLink lnk = new TLink("/"+context+"/EVENTSTORE/"+file,"TRIGGER.ITEM.DATA",dout, null,
        TAccess.CA_READ);
    if (lnk.executeAndClose(500) != 0) throw new IOException("link error " + lnk.getLastError());
    int n = dout.getCompletionLength();
    if (n == 0) return null;
    return dout.getDataObject();
  }
  public static int setTriggerInputData(String context,String file,Object data)
  {
    if (context == null || context.length() == 0) return TErrorList.invalid_parameter;
    if (file == null || file.length() == 0) return TErrorList.invalid_parameter;
    TDataType din = null;
    if (data == null)
      din = new TDataType();
    else if (data instanceof byte[])
      din = new TDataType((byte[])data);
    else if (data instanceof short[])
      din = new TDataType((short[])data);
    else if (data instanceof int[])
      din = new TDataType((int[])data);
    else if (data instanceof float[])
      din = new TDataType((float[])data);
    else if (data instanceof double[])
      din = new TDataType((double[])data);
    else 
      return TErrorList.illegal_format;
    TLink lnk = new TLink("/"+context+"/EVENTSTORE/"+file,"TRIGGER.ITEM.DATA",null, din,
        TAccess.CA_WRITE);
    return lnk.executeAndClose(500);
  }
  public static String[] getTriggerDataFiles(String context) throws IOException
  {
    if (context == null || context.length() == 0) return null;
    NAME16[] n16 = new NAME16[nice_query_size];
    TDataType dout = new TDataType(n16);
    TLink lnk = new TLink("/"+context+"/EVENTSTORE","TRIGGER.ITEM.DATAFILES",dout, null,
        TAccess.CA_READ);
    if (lnk.executeAndClose(500) != 0) throw new IOException("link error " + lnk.getLastError());
    int n = dout.getCompletionLength();
    if (n == 0) return null;
    String[] files = new String[n];
    for (int i=0; i<n; i++) files[i] = n16[i].getName();
    return files;
  }
}
