package de.desy.tine.eventArchive;
import de.desy.tine.client.*;
import de.desy.tine.dataUtils.*;
import de.desy.tine.definitions.*;
import de.desy.tine.types.*;
import java.io.*;

/**
 * This class can take a variable number of parameters on the command
 * line. Program execution begins with the main() method. The class
 * constructor is not invoked unless an object of type 'Trpc'
 * created in the main() method.
 */
public class TEventServer
{
  public int statusCode;
  public String status="undefined";
  public int debugLevel = 0;
  int[] timeRange = new int[2];
  int[] nevents = new int[1];
  short[] ndservers = new short[1];
  short[] naservers = new short[1];
  int[] evnlst;
  NAME16[] aservers;
  NAME16[] dservers;
  AES[] aes;
  AESnotify aesnot;
  TDataType d_null;
  TDataType d_nevent;
  TDataType d_range;
  TDataType d_aes;
  TDataType d_evnlst;
  TDataType d_aesnot;
  TDataType d_ndservers;
  TDataType d_naservers;
  TDataType d_dservers;
  TDataType d_aservers;
  ByteArrayOutputStream dBuffer;
  public TEventServer()
  {
    d_null = new TDataType();
  }
  public int getNumberOfArchiveServers()
  {
    d_naservers = new TDataType(naservers);
    TLink nas = new TLink("ARCHEVN","NASERVERS",d_naservers,d_null,(short)1);
    if ((statusCode=nas.execute(3000)) == 0) 
    {
      nas.cancel();
      return naservers[0];
    }
    else 
    {
      status = nas.getLastError();
      nas.cancel();
      return -1;
    }
  }
  public NAME16[] getArchiveServers()
  {
    int nas = 16; //getNumberOfArchiveServers();
    if (nas <= 0) return null;
    aservers = new NAME16[nas];
    for (int i=0; i<aservers.length; i++) 
    {
      aservers[i] = new NAME16();
    }
    d_aservers = new TDataType(aservers);
    TLink as = new TLink("ARCHEVN/ALL","ASERVERS",d_aservers,d_null,(short)1);
    if ((statusCode=as.execute(3000)) == 0) 
    {
      as.cancel();
      return aservers;
    }
    else 
    {
      status = as.getLastError();
      as.cancel();
      return null;
    }
  }
  public int setEventComment(int eventNumber, String comment, String mode)
  {
    aesnot = new AESnotify();
    aesnot.eventNumber = eventNumber;
    aesnot.comment = comment;
    d_aesnot = new TDataType(aesnot.toByteArray(),"AESNOT");
    TLink ec = new TLink("ARCHEVN/"+mode,"SETCOMMENT",d_null,d_aesnot,(short)TAccess.CA_WRITE);
    statusCode = ec.execute(3000);
    status = ec.getLastError();
    ec.cancel();
    return statusCode;
  }
  public int getNumberOfDeviceServers()
  {
    d_ndservers = new TDataType(ndservers);
    TLink nds = new TLink("ARCHEVN/ALL","NDSERVERS",d_ndservers,d_null,(short)1);
    if ((statusCode=nds.execute(3000)) == 0) 
    {
      nds.cancel();
      return ndservers[0];
    }
    else 
    {
      status = nds.getLastError();
      nds.cancel();
      return -1;
    }
  }
  public int getNumberOfDeviceServers(String ArchiveServer)
  {
    d_ndservers = new TDataType(ndservers);
    TLink nds = new TLink("ARCHEVN/"+ArchiveServer,"NDSERVERS",d_ndservers,d_null,(short)1);
    if ((statusCode=nds.execute(3000)) == 0) 
    {
      nds.cancel();
      return ndservers[0];
    }
    else 
    {
      status = nds.getLastError();
      nds.cancel();
      return -1;
    }
  }
  public NAME16[] getDeviceServers()
  {
    int nds = 256; //getNumberOfDeviceServers();
    if (nds <= 0) return null;
    dservers = new NAME16[nds];
    for (int i=0; i<dservers.length; i++) 
    {
      dservers[i] = new NAME16();
    }
    d_dservers = new TDataType(dservers);
    TLink ds = new TLink("ARCHEVN/ALL","DSERVERS",d_dservers,d_null,(short)1);
    if ((statusCode=ds.execute(3000)) == 0) 
    {
      ds.cancel();
      return dservers;
    }
    else 
    {
      status = ds.getLastError();
      ds.cancel();
      return null;
    }
  }
  public NAME16[] getDeviceServers(String ArchiveServer)
  {
    int nds = 16; //getNumberOfDeviceServers();
    if (nds <= 0) return null;
    dservers = new NAME16[nds];
    for (int i=0; i<dservers.length; i++) 
    {
      dservers[i] = new NAME16();
    }
    d_dservers = new TDataType(dservers);
    TLink ds = new TLink("ARCHEVN/"+ArchiveServer,"DSERVERS",d_dservers,d_null,(short)1);
    if ((statusCode=ds.execute(3000)) == 0) 
    {
      ds.cancel();
      return dservers;
    }
    else 
    {
      status = ds.getLastError();
      ds.cancel();
      return null;
    }
  }
  public int getNumberOfEvents(int start, int stop)
  {
    timeRange[0] = start; timeRange[1] = stop;
    d_nevent = new TDataType(nevents);
    d_range = new TDataType(timeRange);
    TLink nevn = new TLink("ARCHEVN","NEVENTS",d_nevent,d_range,(short)1);
    if ((statusCode=nevn.execute(3000)) == 0) 
    {
      nevn.cancel();
      return nevents[0];
    }
    else 
    {
      status = nevn.getLastError();
      nevn.cancel();
      return -1;
    }
  }
  public int getNumberOfEvents() { return nevents[0]; }
  public AES[] getArchivedEvents(int start, int stop)
  {
    int nevn = getNumberOfEvents(start,stop);
    if (nevn <= 0) return null;
    int nbytes = 0;
    aes = new AES[nevn];
    for (int i=0; i<aes.length; i++) 
    {
      aes[i] = new AES();
      nbytes += aes[i].sizeInBytes;
    }
    int siz = nbytes/nevn;
    int iptr = 0, ptr = 0;
    int blksiz = nevn;
    if (blksiz > 100) blksiz = 100;
    byte[] bBuffer = new byte[blksiz*siz]; //byte[nbytes];
    timeRange[0] = start; timeRange[1] = stop;
    d_aes = new TDataType(bBuffer,"AES");
    d_range = new TDataType(timeRange);
    TLink evn = new TLink("ARCHEVN","EVENTS",d_aes,d_range,(short)1);
    int i=0;
    
    while ((statusCode=evn.execute(1000)) == 0) 
    {
      if (debugLevel > 0)
        System.out.println("get events: nevents = " + nevn + " i = " + i + " iptr = " + iptr + " ptr = " + ptr + " numblks = " + blksiz);
      for (i=iptr,ptr=0; i<iptr+blksiz && i<nevn; i++)
      {
        byte[] b = aes[i].toByteArray();
        for (int n=0; n<b.length; n++) b[n] = bBuffer[ptr+n];
        ptr += siz;
        aes[i].toStruct();
      }
      if (i >= nevn) 
      {
        evn.cancel();
        return aes;
      }
      iptr = i;
      timeRange[0] = aes[i-1].eventNumber + 1;
      d_range.putData(timeRange);
    }
    status = evn.getLastError();
    evn.cancel();
    return null;
  }
  public int[] getArchivedEventList(int start, int stop)
  {
    int nevn = getNumberOfEvents(start,stop);
    if (nevn <= 0) return null;
    evnlst = new int[nevn];
    timeRange[0] = start; timeRange[1] = stop;
    d_evnlst = new TDataType(evnlst);
    d_range = new TDataType(timeRange);
    TLink evn = new TLink("ARCHEVN","EVENTS",d_evnlst,d_range,(short)1);    
    if ((statusCode=evn.execute(3000)) == 0)
    { 
      evn.cancel();
      return evnlst;
    }
    status = evn.getLastError();
    evn.cancel();
    return null;
  }
}
