/*
 * File ADRB5.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.eventUtils;
import java.io.*;
import de.desy.tine.endianUtils.Swap;
/**
 * @deprecated
 * @author duval
 *
 */
public class ADRB5
{
  public int eventNumber;
  public int starttime;
  public int stoptime;
  public String deviceServerName = new String(new StringBuffer(16));
  public String deviceProperty = new String(new StringBuffer(16));
  public String deviceName = new String(new StringBuffer(16));
  public short dataFormat;
  public short dataDimension;
  public static int sizeInBytes = (3*4 + 3*16 + 2*2);
  ByteArrayOutputStream dBuffer = new ByteArrayOutputStream(sizeInBytes);
  byte[] hByteArray;
  public byte[] toByteArray()
  {
    try
    {
      dBuffer.reset();
      DataOutputStream ds = new DataOutputStream(dBuffer);
      ds.writeInt(Swap.Int(eventNumber));
      ds.writeInt(Swap.Int(starttime));
      ds.writeInt(Swap.Int(stoptime));
      byte[] bstr = new byte[16];
      byte[] tmp  = deviceServerName.getBytes();
      for (int i=0; i<16; i++) bstr[i] = i < tmp.length ? tmp[i] : 0;
      ds.write(bstr,0,16);
      tmp  = deviceProperty.getBytes();
      for (int i=0; i<16; i++) bstr[i] = i < tmp.length ? tmp[i] : 0;
      ds.write(bstr,0,16);
      tmp  = deviceName.getBytes();
      for (int i=0; i<16; i++) bstr[i] = i < tmp.length ? tmp[i] : 0;
      ds.write(bstr,0,16);
      ds.writeShort(Swap.Short(dataFormat+512));
      ds.writeShort(Swap.Short(dataDimension));
      ds.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      System.exit(1);
    }	 
    hByteArray = dBuffer.toByteArray();
    return hByteArray;
  }
  public void toStruct()
  {
    try
    {
      ByteArrayInputStream dis = new ByteArrayInputStream(hByteArray);
      DataInputStream ds = new DataInputStream(dis);
      eventNumber = Swap.Int(ds.readInt());
      starttime = Swap.Int(ds.readInt());
      stoptime  = Swap.Int(ds.readInt());
      byte[] bstr = new byte[16];
      ds.read(bstr,0,16);
      deviceServerName = new String(bstr);
      ds.read(bstr,0,16);
      deviceProperty = new String(bstr);
      ds.read(bstr,0,16);
      deviceName = new String(bstr);
      dataFormat = Swap.Short(ds.readShort()-512);
      dataDimension = Swap.Short(ds.readShort());
      ds.close(); dis.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      System.exit(1);
    }	 
  }
  public void dump()
  {
    String dmpstr = "Event Number : " + eventNumber; System.out.println(dmpstr);
    dmpstr = "Start time : " + starttime; System.out.println(dmpstr);
    dmpstr = "Stop time : " + stoptime; System.out.println(dmpstr);
    dmpstr = "Device Server Name: " + deviceServerName; System.out.println(dmpstr);
    dmpstr = "Device Property: " + deviceProperty; System.out.println(dmpstr);
    dmpstr = "Device Name: " + deviceName; System.out.println(dmpstr);
    dmpstr = "Data Format: " + dataFormat; System.out.println(dmpstr);
    dmpstr = "Data Dimension: " + dataDimension; System.out.println(dmpstr);
  }
}
