/*
 * File AES.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.eventUtils;
import java.io.*;
import de.desy.tine.endianUtils.Swap;

public class AES
{
  public int eventNumber;
  public int eventCode;
  public String comment = new String(new StringBuffer(256));
  public byte[] dsMask = new byte[32];
  public static int sizeInBytes = (2*4 + 256 + 32);
  ByteArrayOutputStream dBuffer = new ByteArrayOutputStream(sizeInBytes);
  byte[] hByteArray;
  public byte[] toByteArray()
  {
    try
    {
      dBuffer.reset();
      DataOutputStream ds = new DataOutputStream(dBuffer);
      ds.writeInt(Swap.Int(eventNumber));
      ds.writeInt(Swap.Int(eventCode));
      byte[] bstr = new byte[256];
      byte[] tmp  = comment.getBytes();
      for (int i=0; i<256; i++) bstr[i] = i < tmp.length ? tmp[i] : 0;
      ds.write(bstr,0,256);
      ds.write(dsMask,0,32);
      ds.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      System.exit(1);
    }
    hByteArray = dBuffer.toByteArray();
    return hByteArray;
  }
  public void toStruct()
  {
    try
    {
    	ByteArrayInputStream dis = new ByteArrayInputStream(hByteArray);
    	DataInputStream ds = new DataInputStream(dis);
    	eventNumber = Swap.Int(ds.readInt());
    	eventCode = Swap.Int(ds.readInt());
    	byte[] bstr = new byte[256];
    	ds.read(bstr,0,256);
    	comment = new String(bstr);
    	ds.read(dsMask,0,32);
    	ds.close(); dis.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      System.exit(1);
    }	 
  }
  String Hex(byte[] b)
  {
    char[] digits = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
    String s = new String("0x");
    int v,i,nib;
    for (i=b.length; i>0; i--)
    {
      for (nib=1; nib>=0; nib--)
      {
        v = (b[i-1]>>>(nib*4)) & 0x0f;
        s += digits[v];
      }
    }
    return s;
  }
  public void dump()
  {
    String dmpstr = "Event Number : " + eventNumber; System.out.println(dmpstr);
    dmpstr = "Event Code : " + eventCode; System.out.println(dmpstr);
    dmpstr = "Comment: " + comment; System.out.println(dmpstr);    
    dmpstr = "Mask: " + Hex(dsMask); System.out.println(dmpstr); // fix later
  }
}
