/*
 * File AIB.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.eventUtils;
import java.io.*;
import de.desy.tine.endianUtils.*;
public class AIB
{
  public short numProperties;
  public short numDevices;
  public int eventCode;
  public String comment = new String(new StringBuffer(64));
  public static int sizeInBytes = (2*4 + 4 + 64);
  ByteArrayOutputStream dBuffer = new ByteArrayOutputStream(sizeInBytes);
  byte[] hByteArray;
  public byte[] toByteArray()
  {
    try
    {
    	dBuffer.reset();
    	DataOutputStream ds = new DataOutputStream(dBuffer);
    	ds.writeShort(Swap.Short(numProperties));
    	ds.writeShort(Swap.Short(numDevices));
    	ds.writeInt(Swap.Int(eventCode));
    	byte[] bstr = new byte[64];
    	byte[] tmp  = comment.getBytes();
    	for (int i=0; i<64; i++) bstr[i] = i < tmp.length ? tmp[i] : 0;
    	ds.write(bstr,0,64);
    	ds.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      System.exit(1);
    }	 
    hByteArray = dBuffer.toByteArray();
    return hByteArray;
  }
  public void toStruct()
  {
    try
    {
    	ByteArrayInputStream dis = new ByteArrayInputStream(hByteArray);
    	DataInputStream ds = new DataInputStream(dis);
    	numProperties = Swap.Short(ds.readShort());
    	numDevices = Swap.Short(ds.readShort());
    	eventCode = Swap.Int(ds.readInt());
    	byte[] bstr = new byte[64];
    	ds.read(bstr,0,64);
    	comment = new String(bstr);
    	ds.close(); dis.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      System.exit(1);
    }	 
  }
}
