/*
 * File PIB.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.eventUtils;
import java.io.*;
import de.desy.tine.endianUtils.*;

public class PIB
{
  public float SamplingRate;
  public int numPreTriggerValues;
  public int TimeOffset;
  public int LowCount;
  public int HighCount;
  public String Units = new String(new StringBuffer(16));
  public double LowEGU;
  public double HighEGU;  
  public short numDevices;
  public short reserved;
  public static int sizeInBytes = (5*4 + 16 + 2*8 + 2*2);
  ByteArrayOutputStream dBuffer = new ByteArrayOutputStream(sizeInBytes);
  byte[] hByteArray;
  public byte[] toByteArray()
  {
    try
    {
      dBuffer.reset();
      byte[] bstr = new byte[256];
      byte[] tmp;
      DataOutputStream ds = new DataOutputStream(dBuffer);
      ds.write(Swap.Float(SamplingRate),0,4);
      ds.writeInt(Swap.Int(numPreTriggerValues));
      ds.writeInt(Swap.Int(TimeOffset));
      ds.writeInt(Swap.Int(LowCount));
      ds.writeInt(Swap.Int(HighCount));
      tmp  = Units.getBytes();
      for (int i=0; i<16; i++) bstr[i] = i < tmp.length ? tmp[i] : 0;
      ds.write(bstr,0,16);
      ds.write(Swap.Double(LowEGU),0,8);
      ds.write(Swap.Double(HighEGU),0,8);
      ds.writeShort(Swap.Short(numDevices));
      ds.writeShort(Swap.Short(reserved));
      ds.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      System.exit(1);
    }	 
    hByteArray = dBuffer.toByteArray();
    return hByteArray;
  }
  public void toStruct()
  {
    try
    {
      byte[] bstr = new byte[256];
      ByteArrayInputStream dis = new ByteArrayInputStream(hByteArray);
      DataInputStream ds = new DataInputStream(dis);
      SamplingRate = Swap.Float(bstr);
      numPreTriggerValues = Swap.Int(ds.readInt());
      TimeOffset = Swap.Int(ds.readInt());
      LowCount = Swap.Int(ds.readInt());
      HighCount = Swap.Int(ds.readInt());
      ds.read(bstr,0,16);
      Units = new String(bstr);
      ds.read(bstr,0,8);
      LowEGU = Swap.Double(bstr);
      ds.read(bstr,0,8);
      HighEGU = Swap.Double(bstr);
      numDevices = Swap.Short(ds.readShort());
      reserved = Swap.Short(ds.readShort());
      ds.close(); dis.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      System.exit(1);
    }	 
  }
}
