/*
 * File TEventServer.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.eventUtils;
import de.desy.tine.client.*;
import de.desy.tine.dataUtils.*;
import de.desy.tine.definitions.*;
import de.desy.tine.types.*;
import java.io.*;
// Super-duper legacy, this is!
// can't think of anything anywhere that might be using this class ...
/**
 * This class can take a variable number of parameters on the command
 * line. Program execution begins with the main() method. The class
 * constructor is not invoked unless an object of type 'Trpc'
 * created in the main() method.
 */
public class TEventServer
{
  public static final String EventManager = "/SITE/EVENTMGR";
  public int statusCode;
  public String status="undefined";
  public int debugLevel = 0;
  int[] timeRange = new int[2];
  int[] nevents = new int[1];
  short[] ndservers = new short[1];
  short[] naservers = new short[1];
  int[] evnlst;
  NAME16[] aservers;
  NAME16[] dservers;
  AES[] aes;
  AESnotify aesnot;
  TDataType d_null;
  TDataType d_nevent;
  TDataType d_range;
  TDataType d_aes;
  TDataType d_evnlst;
  TDataType d_aesnot;
  TDataType d_ndservers;
  TDataType d_naservers;
  TDataType d_dservers;
  TDataType d_aservers;
  ByteArrayOutputStream dBuffer;
  public TEventServer()
  {
    d_null = new TDataType();
  }
  public int getNumberOfArchiveServers()
  {
    d_naservers = new TDataType(naservers);
    TLink nas = new TLink(EventManager,"NASERVERS",d_naservers,d_null,(short)1);
    if ((statusCode=nas.executeAndClose(3000)) == 0) 
    {
      return naservers[0];
    }
    else 
    {
      status = nas.getLastError();
      return -1;
    }
  }
  public NAME16[] getArchiveServers()
  {
    int nas = 16; //getNumberOfArchiveServers();
    if (nas <= 0) return null;
    aservers = new NAME16[nas];
    for (int i=0; i<aservers.length; i++) 
    {
      aservers[i] = new NAME16();
    }
    d_aservers = new TDataType(aservers);
    TLink as = new TLink("ARCHEVN/ALL","ASERVERS",d_aservers,d_null,(short)1);
    if ((statusCode=as.executeAndClose(3000)) == 0) 
    {
      return aservers;
    }
    else 
    {
      status = as.getLastError();
      return null;
    }
  }
  public int setEventComment(int eventNumber, String comment, String mode)
  {
    aesnot = new AESnotify();
    aesnot.eventNumber = eventNumber;
    aesnot.comment = comment;
    d_aesnot = new TDataType(aesnot.toByteArray(),"AESNOT");
    TLink ec = new TLink("ARCHEVN/"+mode,"SETCOMMENT",d_null,d_aesnot,(short)TAccess.CA_WRITE);
    statusCode = ec.executeAndClose(3000);
    status = ec.getLastError();
    return statusCode;
  }
  public int getNumberOfDeviceServers()
  {
    d_ndservers = new TDataType(ndservers);
    TLink nds = new TLink("ARCHEVN/ALL","NDSERVERS",d_ndservers,d_null,(short)1);
    if ((statusCode=nds.executeAndClose(3000)) == 0) 
    {
      return ndservers[0];
    }
    else 
    {
      status = nds.getLastError();
      return -1;
    }
  }
  public int getNumberOfDeviceServers(String ArchiveServer)
  {
    d_ndservers = new TDataType(ndservers);
    TLink nds = new TLink("ARCHEVN/"+ArchiveServer,"NDSERVERS",d_ndservers,d_null,(short)1);
    if ((statusCode=nds.executeAndClose(3000)) == 0) 
    {
      return ndservers[0];
    }
    else 
    {
      status = nds.getLastError();
      return -1;
    }
  }
  public NAME16[] getDeviceServers()
  {
    int nds = 256; //getNumberOfDeviceServers();
    if (nds <= 0) return null;
    dservers = new NAME16[nds];
    for (int i=0; i<dservers.length; i++) 
    {
      dservers[i] = new NAME16();
    }
    d_dservers = new TDataType(dservers);
    TLink ds = new TLink("ARCHEVN/ALL","DSERVERS",d_dservers,d_null,(short)1);
    if ((statusCode=ds.executeAndClose(3000)) == 0) 
    {
      return dservers;
    }
    else 
    {
      status = ds.getLastError();
      return null;
    }
  }
  public NAME16[] getDeviceServers(String ArchiveServer)
  {
    int nds = 16; //getNumberOfDeviceServers();
    if (nds <= 0) return null;
    dservers = new NAME16[nds];
    for (int i=0; i<dservers.length; i++) 
    {
      dservers[i] = new NAME16();
    }
    d_dservers = new TDataType(dservers);
    TLink ds = new TLink("ARCHEVN/"+ArchiveServer,"DSERVERS",d_dservers,d_null,(short)1);
    if ((statusCode=ds.executeAndClose(3000)) == 0) 
    {
      return dservers;
    }
    else 
    {
      status = ds.getLastError();
      return null;
    }
  }
  public int getNumberOfEvents(int start, int stop)
  {
    timeRange[0] = start; timeRange[1] = stop;
    d_nevent = new TDataType(nevents);
    d_range = new TDataType(timeRange);
    TLink nevn = new TLink(EventManager,"NEVENTS",d_nevent,d_range,(short)1);
    if ((statusCode=nevn.executeAndClose(3000)) == 0) 
    {
      return nevents[0];
    }
    else 
    {
      status = nevn.getLastError();
      return -1;
    }
  }
  public int getNumberOfEvents() { return nevents[0]; }
  public AES[] getArchivedEvents(int start, int stop)
  {
    int nevn = getNumberOfEvents(start,stop);
    if (nevn <= 0) return null;
    int nbytes = 0;
    aes = new AES[nevn];
    for (int i=0; i<aes.length; i++) 
    {
      aes[i] = new AES();
      nbytes += AES.sizeInBytes;
    }
    int siz = nbytes/nevn;
    int iptr = 0, ptr = 0;
    int blksiz = nevn;
    if (blksiz > 100) blksiz = 100;
    byte[] bBuffer = new byte[blksiz*siz]; //byte[nbytes];
    timeRange[0] = start; timeRange[1] = stop;
    d_aes = new TDataType(bBuffer,"AES");
    d_range = new TDataType(timeRange);
    TLink evn = new TLink(EventManager,"EVENTS",d_aes,d_range,(short)1);
    int i=0;
    
    while ((statusCode=evn.executeAndClose(1000)) == 0) 
    {
      if (debugLevel > 0)
        System.out.println("get events: nevents = " + nevn + " i = " + i + " iptr = " + iptr + " ptr = " + ptr + " numblks = " + blksiz);
      for (i=iptr,ptr=0; i<iptr+blksiz && i<nevn; i++)
      {
        byte[] b = aes[i].toByteArray();
        for (int n=0; n<b.length; n++) b[n] = bBuffer[ptr+n];
        ptr += siz;
        aes[i].toStruct();
      }
      if (i >= nevn) 
      {
        return aes;
      }
      iptr = i;
      timeRange[0] = aes[i-1].eventNumber + 1;
      d_range.putData(timeRange);
    }
    status = evn.getLastError();
    return null;
  }
  public int[] getArchivedEventList(int start, int stop)
  {
    int nevn = getNumberOfEvents(start,stop);
    if (nevn <= 0) return null;
    evnlst = new int[nevn];
    timeRange[0] = start; timeRange[1] = stop;
    d_evnlst = new TDataType(evnlst);
    d_range = new TDataType(timeRange);
    TLink evn = new TLink(EventManager,"EVENTS",d_evnlst,d_range,(short)1);    
    if ((statusCode=evn.executeAndClose(3000)) == 0)
    { 
      return evnlst;
    }
    status = evn.getLastError();
    return null;
  }
}
