/*
 * File TrgChn.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.eventUtils;

import de.desy.tine.structUtils.TTaggedStructure;

public class TrgChn extends TTaggedStructure
{
  public void clear()
  {
    contextString = null;
    serverString = null;
    propertyString = null;
    deviceString = null;
  }
  private char[] context = new char[32];
  private String contextString;
  private char[] server = new char[32];
  private String serverString;
  private char[] property = new char[64];
  private String propertyString;
  private char[] device = new char[64];
  private String deviceString;
  private void initStructure()
  {
    addField(context,"context");
    addField(server,"server");
    addField(property,"property");
    addField(device,"device");
    initDone();    
  }
  public TrgChn()
  {
    super();
    initStructure();
  }
  public TrgChn(TrgChn reference)
  {
    super();
    initStructure();
    System.arraycopy(reference.context, 0, context, 0, 32);    
    System.arraycopy(reference.server, 0, server, 0, 32);    
    System.arraycopy(reference.property, 0, property, 0, 64);    
    System.arraycopy(reference.device, 0, device, 0, 64);
  }
  public String getContext()
  {
    if (contextString == null) contextString = new String(context).trim();
    return contextString;
  }
  public String getDevice()
  {
    if (deviceString == null) deviceString = new String(device).trim();
    return deviceString;
  }
  public String getProperty()
  {
    if (propertyString == null) propertyString = new String(property).trim();
    return propertyString;
  }
  public String getServer()
  {
    if (serverString == null) serverString = new String(server).trim();
    return serverString;
  }
  public void setContext(String context) 
  {
    pushChars(context,this.context);
    this.contextString = new String(context);
  }
  public void setDevice(String device) 
  {
    pushChars(device,this.device);
    this.deviceString = new String(device);
  }
  public void setServer(String server) 
  {
    pushChars(server,this.server);
    this.serverString = new String(server);
  }
  public void setProperty(String property) 
  {
    pushChars(property,this.property);
    this.propertyString = new String(property);
  }
}
