/*
 * File TrgView.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.eventUtils;

import java.util.Arrays;

import de.desy.tine.structUtils.TTaggedStructure;

public class TrgView extends TTaggedStructure
{
  public void clear()
  {
    contextString = null;
    triggerString = null;
    descString = null;
    fileString = null;
    Arrays.fill(context, (char)0);
    Arrays.fill(trigger, (char)0);
    Arrays.fill(desc, (char)0);
    Arrays.fill(file, (char)0);
  }
  private char[] context = new char[32];
  private String contextString;
  private char[] trigger = new char[64];
  private String triggerString;
  private char[] desc = new char[64];
  private String descString;
  private char[] file = new char[64];
  private String fileString;
  private int[] components = new int[1]; // unused -> actually a pointer at the server
  private int[] nComponents = new int[1];
  private void initStructure()
  {
    addField(context,"context");
    addField(trigger,"trigger");
    addField(desc,"desc");
    addField(file,"file");
    addField(components,"reference");
    addField(nComponents,"nComponents");
    initDone();    
  }
  public TrgView()
  {
    super();
    initStructure();
  }
  public TrgView(TrgView reference)
  {
    super();
    initStructure();
    System.arraycopy(reference.context, 0, context, 0, 32);    
    System.arraycopy(reference.trigger, 0, trigger, 0, 64);    
    System.arraycopy(reference.desc, 0, desc, 0, 64);    
    System.arraycopy(reference.file, 0, file, 0, 64);
    components[0] = reference.components[0];
    nComponents[0] = reference.nComponents[0];
  }
  public String getContext()
  {
    if (contextString == null) contextString = new String(context).trim();
    return contextString;
  }
  public String getFile()
  {
    if (fileString == null) fileString = new String(file).trim();
    return fileString;
  }
  public String getDescription()
  {
    if (descString == null) descString = new String(desc).trim();
    return descString;
  }
  public String getTrigger()
  {
    if (triggerString == null) triggerString = new String(trigger).trim();
    return triggerString;
  }
  public int getNumComponents()
  { // not relevant here anyway ...
    return nComponents[0];
  }
  public void setContext(String context) 
  {
    pushChars(context,this.context);
    this.contextString = new String(context);
  }
  public void setDescription(String description) 
  {
    pushChars(description,this.desc);
    this.descString = new String(description);
  }
  public void setFile(String file) 
  {
    pushChars(file,this.file);
    this.fileString = new String(file);
  }
  public void setTrigger(String trigger) 
  {
    pushChars(trigger,this.trigger);
    this.triggerString = new String(trigger);
  }
  public void setNumComponents(int n) 
  {
    nComponents[0] = n;  
  }
}
