/*
 * File eaSwap.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.eventUtils;

public class eaSwap
{
  static byte[] barr;
  static float fltval;
  static double dblval;
  public static void Bytes(byte[] barr, String tag)
  {
    int bounds;
    byte b1, b2, b3, b4;
    if (tag.compareTo("AESNOT") == 0)
    {
      bounds = 4*4 + 4*64;  // size of AESnotify structure
      if (barr.length % bounds != 0) return;
      for (int i=0; i<barr.length; i += bounds) 
      {
        // 4 UINT32 values ...
        for (int j=0; j<16; j += 4) 
        {
          b1 = barr[i+j]; b2 = barr[i+j+1]; 
          barr[i+j] = barr[i+j+3]; barr[i+j+1] = barr[i+j+2];
          barr[i+j+2] = b2; barr[i+j+3] = b1;
        }
        // 256 8-bit characters (no swapping)
      }
      return;
    }
    if (tag.compareTo("AESCON") == 0)
    {
      bounds = 4*4 + 16;  // size of AEScontext structure
      if (barr.length % bounds != 0) return;
      for (int i=0; i<barr.length; i += bounds) 
      {
        // 4 UINT32 values ...
        for (int j=0; j<16; j += 4) 
        {
          b1 = barr[i+j]; b2 = barr[i+j+1]; 
          barr[i+j] = barr[i+j+3]; barr[i+j+1] = barr[i+j+2];
          barr[i+j+2] = b2; barr[i+j+3] = b1;
        }
        // 16 8-bit characters (no swapping)
      }
      return;
    }
    if (tag.compareTo("AES") == 0)
    {
      bounds = 2*4 + 4*64 + 32;  // size of AES structure
      if (barr.length % bounds != 0) return;
      for (int i=0; i<barr.length; i += bounds) 
      {
        // 2 UINT32 values ...
        for (int j=0; j<8; j += 4) 
        {
          b1 = barr[i+j]; b2 = barr[i+j+1]; 
          barr[i+j] = barr[i+j+3]; barr[i+j+1] = barr[i+j+2];
          barr[i+j+2] = b2; barr[i+j+3] = b1;
        }
        // 256 8-bit characters (no swapping)
        // 32 bytes (no swapping)
      }
      return;
    }
    if (tag.compareTo("AIB") == 0)
    {
      bounds = 2*2 + 4 + 64;  // size of Archive Info Block
      if (barr.length % bounds != 0) return;
      for (int i=0,j=0; i<barr.length; i += bounds) 
      {
        // 2 UINT16 values ...
        for (j=0; j<4; j += 2) 
        {
          b1 = barr[i+j]; barr[i+j] = barr[i+j+1]; barr[i+j+1] = b1;
        }
        // 1 UINT32 value ...
        for (int offset = j; j<4+offset; j += 4) 
        {
          b1 = barr[i+j]; b2 = barr[i+j+1]; 
          barr[i+j] = barr[i+j+3]; barr[i+j+1] = barr[i+j+2];
          barr[i+j+2] = b2; barr[i+j+3] = b1;
        }
        // 64 8-bit characters (no swapping)
      }
      return;
    }
    if (tag.compareTo("PIB") == 0)
    {
      bounds = 5*4 + 16 + 2*8 + 2*2;  // size of Property Info Block
      if (barr.length % bounds != 0) return;
      for (int i=0,j=0; i<barr.length; i += bounds) 
      {
        // 5 UINT32 or float values ...
        for (j=0; j<20; j += 4) 
        {
          b1 = barr[i+j]; b2 = barr[i+j+1]; 
          barr[i+j] = barr[i+j+3]; barr[i+j+1] = barr[i+j+2];
          barr[i+j+2] = b2; barr[i+j+3] = b1;
        }
        // 16 8-bit characters (no swapping)
        // 2 double values ...
        for (int offset=j+16; j<16+offset; j += 8) 
        {
          b1 = barr[i+j]; b2 = barr[i+j+1]; 
          b3 = barr[i+j+2]; b4 = barr[i+j+3]; 
          barr[i+j] = barr[i+j+7]; barr[i+j+1] = barr[i+j+6];
          barr[i+j+2] = barr[i+j+5]; barr[i+j+3] = barr[i+j+4];
          barr[i+j+4] = b4; barr[i+j+5] = b3;
          barr[i+j+6] = b2; barr[i+j+7] = b1;
        }
        // 2 UINT16 values ...
        for (int offset=j; j<4+offset; j += 2) 
        {
          b1 = barr[i+j]; barr[i+j] = barr[i+j+1]; barr[i+j+1] = b1;
        }
      }
      return;
    }
    if (tag.compareTo("DHS") == 0)
    {
      bounds = 32 + 4 + 2*2 + 4 + 4*4 + 16 + 4*2 + 4 + 4*64;  // size of Data Header Structure
      if (barr.length % bounds != 0) return;
      for (int i=0,j=0; i<barr.length; i += bounds) 
      {
        // 32 8-bit characters (no swapping)
        // 1 UINT32 or float values ...
        for (int offset=32; j<4+offset; j += 4) 
        {
          b1 = barr[i+j]; b2 = barr[i+j+1]; 
          barr[i+j] = barr[i+j+3]; barr[i+j+1] = barr[i+j+2];
          barr[i+j+2] = b2; barr[i+j+3] = b1;
        }
        // 2 UINT16 values ...
        for (int offset=j; j<4+offset; j += 2) 
        {
          b1 = barr[i+j]; barr[i+j] = barr[i+j+1]; barr[i+j+1] = b1;
        }
        // 5 UINT32 or float values ...
        for (int offset=j; j<20+offset; j += 4) 
        {
          b1 = barr[i+j]; b2 = barr[i+j+1]; 
          barr[i+j] = barr[i+j+3]; barr[i+j+1] = barr[i+j+2];
          barr[i+j+2] = b2; barr[i+j+3] = b1;
        }
        // 16 8-bit characters (no swapping)
        // 2 UINT32 or float values ...
        for (int offset=j+16; j<8+offset; j += 4) 
        {
          b1 = barr[i+j]; b2 = barr[i+j+1]; 
          barr[i+j] = barr[i+j+3]; barr[i+j+1] = barr[i+j+2];
          barr[i+j+2] = b2; barr[i+j+3] = b1;
        }
        // 4 + 256 8-bit characters (no swapping)
      }
      return;
    }
    if (tag.compareTo("ADRB") == 0)
    {
      bounds = 3*4 + 3*16 + 2*2;  // size of Archive Data Request Block
      if (barr.length % bounds != 0) return;
      for (int i=0,j=0; i<barr.length; i += bounds) 
      {
        // 3 UINT32 values ...
        for (int offset = j; j<12+offset; j += 4) 
        {
          b1 = barr[i+j]; b2 = barr[i+j+1]; 
          barr[i+j] = barr[i+j+3]; barr[i+j+1] = barr[i+j+2];
          barr[i+j+2] = b2; barr[i+j+3] = b1;
        }
        // 48 8-bit characters (no swapping)
        // 2 UINT16 values ...
        for (int offset = j+48; j<4+offset; j += 2) 
        {
          b1 = barr[i+j]; barr[i+j] = barr[i+j+1]; barr[i+j+1] = b1;
        }
      }
      return;
    }
    return;
  }
}
