package de.desy.tine.globalsViewer;

import java.awt.event.ActionListener;
import java.util.Arrays;
import java.awt.event.ActionEvent;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import java.awt.Point;
import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JMenuItem;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JFrame;
import javax.swing.JDialog;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;

import de.desy.tine.client.TLink;
import de.desy.tine.client.TLinkCallback;
import de.desy.tine.dataUtils.TDataTime;
import de.desy.tine.dataUtils.TDataType;
import de.desy.tine.definitions.*;
import de.desy.tine.queryUtils.TQuery;
import de.desy.tine.types.*;

public class GlobalsViewer implements TLinkCallback
{
  private class StatusColumnCellRenderer extends DefaultTableCellRenderer 
  {
    private static final long serialVersionUID = 2464751533436312938L;
    @Override
    public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int col) 
    {

      //Cells are by default rendered as a JLabel.
      JLabel l = (JLabel) super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, col);

      //Get the status for the current row.
      GlobalsTableModel tableModel = (GlobalsTableModel) table.getModel();
      if (tableModel.getStatus(row) == 0) 
      {
        l.setForeground(Color.BLACK);
      } 
      else 
      {
        l.setForeground(Color.RED);
      }

    //Return the JLabel which renders the cell.
    return l;

    }
  }
  private class GlobalsTableModel extends AbstractTableModel
  {
    private static final long serialVersionUID = 1L;
    private Object[][] tableData;
    private int[] statusData;
    public void setData(Object[][] data)
    {
      this.tableData = data;
      statusData = new int[tableData.length];
      fireTableDataChanged();
    }
    public String getColumnName(int column)
    {
      switch (column)
      {
        case 0:
          return "Keyword";
        case 1:
          return "Value";
      }
      return super.getColumnName(column);
    }
    public int getColumnCount()
    {
      return 2;
    }
    public int getRowCount()
    {
      if (tableData == null)
        return 0;
      else
        return tableData.length;
    }
    public Object getValueAt(int rowIndex, int columnIndex)
    {
      if (tableData == null || rowIndex >= tableData.length) return null;
      return tableData[rowIndex][columnIndex];
    }
    @Override
    public void setValueAt(Object aValue, int rowIndex, int columnIndex)
    {
      if (tableData == null || tableData.length <= rowIndex)
        return;
      else
        tableData[rowIndex][columnIndex] = aValue;
      fireTableCellUpdated(rowIndex, columnIndex);
    }
    public int getStatus(int row)
    {
      if (row < 0 || row >= statusData.length) return 0;
      return statusData[row];
    }
    public void setStatus(int row, int value)
    {
      if (row < 0 || row >= statusData.length) return;
      statusData[row] = value;
    }
  }
  private JFrame jFrame = null;
  private JPanel jContentPane = null;
  private JMenuBar jJMenuBar = null;
  private JMenu contextMenu = null;
  private JMenu helpMenu = null;
  private JMenuItem aboutMenuItem = null;
  private JRadioButtonMenuItem[] contextMenuItems = null;
  private ButtonGroup contextGroup = null;
  private JDialog aboutDialog = null;
  private JPanel aboutContentPane = null;
  private JLabel aboutVersionLabel = null;
  private JTable globalsTable = null;
  private GlobalsTableModel tableModel = null;
  private GlobalsViewer instance = null;
  private int hDim = 300;
  private int vDim = 300;
  String[] vals = new String[2];
  String[] contexts = null;
  String[] keywords = null;
  String[] keyvalues = null;
  static String selectedContext = "SITE"; // @jve:decl-index=0:
  String[] keyerrors = {"error acquiring information !"};
  TLink[] glbLinks = null;
  String[] getGlobalsForContext(String context)
  {
    NAME64DBLDBL[] keys = new NAME64DBLDBL[512];
    //NAME64[] keys = new NAME64[512];
    TDataType dout = new TDataType(keys);
    String dev = "/" + context + "/GLOBALS";
    TLink tl = new TLink(dev, "SRVGLOBALS", dout, null, TAccess.CA_READ);
    int cc = tl.executeAndClose(200);
    String[] glbs = keyerrors;
    if (cc == 0)
    {
      glbs = new String[dout.getCompletionLength()];
      for (int i = 0; i < glbs.length; i++)
        glbs[i] = keys[i].getName();
    }
    Arrays.sort(glbs);
    return glbs;
  }
  private void makeTableForContext(final String context)
  {
    keywords = getGlobalsForContext(context);
    keyvalues = new String[keywords.length];
    for (int i = 0; i < keyvalues.length; i++)
      keyvalues[i] = "-1";
    final Object[][] colData = new Object[keyvalues.length + 1][2];
    for (int i = 0; i < keyvalues.length; i++)
    {
      colData[i][0] = keywords[i];
      colData[i][1] = keyvalues[i];
    }
    colData[keyvalues.length][0] = "Globals Time";
    // changes to the table have to be made in the swing thread
    SwingUtilities.invokeLater(new Runnable()
    {
      public void run()
      {
        getTableModel().setData(colData);
        JFrame frm = instance.getJFrame();
        frm.setTitle(context + " Globals");
        vDim = (keyvalues.length + 4) * 19;
        frm.setSize(hDim, vDim);
      }
    });
  }
  synchronized void stopGloblLinks()
  {
    if (glbLinks != null)
    {
      for (TLink l : glbLinks)
        l.close();
      glbLinks = null;
    }
  }
  synchronized int startGlobalLinks()
  {
    TDataType dout = null;
    String dev = "/" + selectedContext + "/GLOBALS";
    glbLinks = new TLink[keywords.length];
    // TPropertyQuery[] pq;
    for (int i = 0; i < keywords.length; i++)
    {
      // pq = TQuery.getPropertyInformation(selectedContext, "GLOBALS",
      // "keyword", keywords[i]);
      // if (pq == null) continue;
      dout = new TDataType(1024, TFormat.CF_DEFAULT);
      glbLinks[i] = new TLink(dev, keywords[i], dout, null, TAccess.CA_READ);
      glbLinks[i].attach(TMode.CM_GLOBAL, instance, 1000);
    }
    return 0;
  }
  private GlobalsViewer()
  {
    instance = this;
    contexts = TQuery.getContexts("GLOBALS");
    // makeTableForContext(selectedContext);
    // startGlobalLinks();
  }
  /**
   * This method initializes globalsTable
   * 
   * @return javax.swing.JTable
   */
  private JTable getGlobalsTable()
  {
    if (globalsTable == null)
    {
      globalsTable = new JTable(getTableModel());
      globalsTable.getTableHeader().setFont(new Font(Font.DIALOG, Font.BOLD, 14));
      globalsTable.getTableHeader().setResizingAllowed(true);
      globalsTable.setFont(new Font(Font.DIALOG, Font.BOLD, 14));
      globalsTable.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
      globalsTable.getColumnModel().getColumn(1).setCellRenderer(new StatusColumnCellRenderer());
    }
    return globalsTable;
  }
  private GlobalsTableModel getTableModel()
  {
    if (tableModel == null)
    {
      tableModel = new GlobalsTableModel();
    }
    return tableModel;
  }
  /**
   * @param args
   */
  public static void main(String[] args)
  {
    // TODO Auto-generated method stub
    if (args != null && args.length > 0 && args[0] != null) selectedContext = args[0];
    System.out.println("selected context : "+selectedContext);
    SwingUtilities.invokeLater(new Runnable()
    {
      public void run()
      {
        GlobalsViewer application = new GlobalsViewer();
        application.getJFrame().setVisible(true);
      }
    });
  }
  /**
   * This method initializes jFrame
   * 
   * @return javax.swing.JFrame
   */
  private JFrame getJFrame()
  {
    if (jFrame == null)
    {
      jFrame = new JFrame();
      jFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      jFrame.setJMenuBar(getJJMenuBar());
      jFrame.setSize(hDim, vDim);
      jFrame.setContentPane(getJContentPane());
      jFrame.setTitle("SITE Globals");
    }
    return jFrame;
  }
  /**
   * This method initializes jContentPane
   * 
   * @return javax.swing.JPanel
   */
  private JPanel getJContentPane()
  {
    if (jContentPane == null)
    {
      jContentPane = new JPanel();
      jContentPane.setLayout(new BorderLayout());
      JScrollPane pane = new JScrollPane(getGlobalsTable());
      jContentPane.add(pane, BorderLayout.CENTER);
    }
    return jContentPane;
  }
  /**
   * This method initializes jJMenuBar
   * 
   * @return javax.swing.JMenuBar
   */
  private JMenuBar getJJMenuBar()
  {
    if (jJMenuBar == null)
    {
      jJMenuBar = new JMenuBar();
      jJMenuBar.add(getContextMenu());
      jJMenuBar.add(getHelpMenu());
    }
    return jJMenuBar;
  }
  /**
   * This method initializes jMenu
   * 
   * @return javax.swing.JMenu
   */
  private JMenu getContextMenu()
  {
    if (contextMenu == null)
    {
      contextMenu = new JMenu();
      contextMenu.setText("Context");
      contextMenuItems = new JRadioButtonMenuItem[contexts.length];
      contextGroup = new ButtonGroup();
      int idx = -1, site_idx = 0;
      for (int i = 0; i < contexts.length; i++)
      {
        if (contexts[i].compareToIgnoreCase(selectedContext) == 0) idx = i;
        if (contexts[i].compareToIgnoreCase("SITE") == 0) site_idx = i;
        contextMenuItems[i] = new JRadioButtonMenuItem(contexts[i]);
        contextGroup.add(contextMenuItems[i]);
        contextMenu.add(contextMenuItems[i]);
        contextMenuItems[i].addItemListener(new java.awt.event.ItemListener()
        {
          public void itemStateChanged(java.awt.event.ItemEvent e)
          {
            if (!((JRadioButtonMenuItem) e.getItem()).isSelected())
            {
              stopGloblLinks();
            }
            else
            {
              selectedContext = ((JRadioButtonMenuItem) e.getItem()).getText();
              makeTableForContext(selectedContext);
              startGlobalLinks();
            }
          }
        });
      }
      if (idx == -1) idx = site_idx;
      contextMenuItems[idx].setSelected(true);
    }
    return contextMenu;
  }
  /**
   * This method initializes jMenu
   * 
   * @return javax.swing.JMenu
   */
  private JMenu getHelpMenu()
  {
    if (helpMenu == null)
    {
      helpMenu = new JMenu();
      helpMenu.setText("Help");
      helpMenu.add(getAboutMenuItem());
    }
    return helpMenu;
  }
  /**
   * This method initializes jMenuItem
   * 
   * @return javax.swing.JMenuItem
   */
  private JMenuItem getAboutMenuItem()
  {
    if (aboutMenuItem == null)
    {
      aboutMenuItem = new JMenuItem();
      aboutMenuItem.setText("About");
      aboutMenuItem.addActionListener(new ActionListener()
      {
        public void actionPerformed(ActionEvent e)
        {
          JDialog aboutDialog = getAboutDialog();
          aboutDialog.pack();
          Point loc = getJFrame().getLocation();
          loc.translate(20, 20);
          aboutDialog.setLocation(loc);
          aboutDialog.setVisible(true);
        }
      });
    }
    return aboutMenuItem;
  }
  /**
   * This method initializes aboutDialog
   * 
   * @return javax.swing.JDialog
   */
  private JDialog getAboutDialog()
  {
    if (aboutDialog == null)
    {
      aboutDialog = new JDialog(getJFrame(), true);
      aboutDialog.setTitle("About");
      aboutDialog.setContentPane(getAboutContentPane());
    }
    return aboutDialog;
  }
  /**
   * This method initializes aboutContentPane
   * 
   * @return javax.swing.JPanel
   */
  private JPanel getAboutContentPane()
  {
    if (aboutContentPane == null)
    {
      aboutContentPane = new JPanel();
      aboutContentPane.setLayout(new BorderLayout());
      aboutContentPane.add(getAboutVersionLabel(), BorderLayout.CENTER);
    }
    return aboutContentPane;
  }
  /**
   * This method initializes aboutVersionLabel
   * 
   * @return javax.swing.JLabel
   */
  private JLabel getAboutVersionLabel()
  {
    if (aboutVersionLabel == null)
    {
      aboutVersionLabel = new JLabel();
      aboutVersionLabel.setText("Version 1.0");
      aboutVersionLabel.setHorizontalAlignment(SwingConstants.CENTER);
    }
    return aboutVersionLabel;
  }
  /**
   * This method initializes jMenuItem
   * 
   * @return javax.swing.JMenuItem
   */
  public void callback(final TLink link)
  {
    // changes have to be made in swing thread
    SwingUtilities.invokeLater(new Runnable()
    {
      public void run()
      {
        int i;
        for (i = 0; i < glbLinks.length; i++)
        {
          if (link == glbLinks[i])
          {
            ((GlobalsTableModel) globalsTable.getModel()).setStatus(i,link.getLinkStatus());
            keyvalues[i] = link.getOutputDataObject().toString().trim();
            if (!TErrorList.hasData(link.getLinkStatus()))
            {
              keyvalues[i] += " ("+link.getLastError()+")";
            }
            break;
          }
        }
        if (i < glbLinks.length - 1) return;
        
        for (i = 0; i < keyvalues.length; i++)
        {
          globalsTable.setValueAt(keyvalues[i], i, 1);
        }
        String gt = TDataTime.toString(glbLinks[0].getOutputDataObject().getDataTimeStamp());
        globalsTable.setValueAt(gt, keyvalues.length, 1);
      }
    });
  }
}
