/*
 * File DclProc.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.histUtils;

import de.desy.tine.structUtils.TTaggedStructure;

public class DclProc extends TTaggedStructure
{
  public void clear()
  {
    devnameString = null;
    fileString = null; // without the .csv; ususally the same name as devnameString ...
    machine[0] = 0;
    nstates[0] = 0;
  }
  private char[] devname = new char[64];
  private String devnameString;
  private char[] file = new char[64];
  private String fileString;
  private int[] machine = new int[1];
  private int[] nstates = new int[1];
  private void initStructure()
  {
    addField(devname,"name");
    addField(file,"file");
    addField(machine,"machine");
    addField(nstates,"nstates");
    initDone();    
  }
  public DclProc()
  {
    super();
    initStructure();
  }
  public DclProc(DclProc reference)
  {
    super();
    initStructure();
    System.arraycopy(reference.devname, 0, devname, 0, 64);    
    System.arraycopy(reference.file, 0, file, 0, 64);
    machine[0] = reference.machine[0];
    nstates[0] = reference.nstates[0];
  }
  public String getDeviceName()
  {
    if (devnameString == null) devnameString = new String(devname).trim();
    return devnameString;
  }
  public void setDeviceName(String name) {
      pushChars(name,this.devname);
      this.devnameString = new String(name);
  }
  public String getFile()
  {
    if (fileString == null) fileString = new String(file).trim();
    return fileString;
  }
  public void setFile(String file) {
      pushChars(file,this.file);
      this.fileString = new String(file);
  }
  public int getNumberStates()
  {
    return nstates[0];
  }
  public void setNumberStates(int number)
  {
    nstates[0] = number;
  }
  public int getMachine()
  {
    return machine[0];
  }
  public void setMachine(int value) 
  {
    machine[0] = value;
  }
  
}
