/*
 * File HstView.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.histUtils;

import de.desy.tine.structUtils.TTaggedStructure;

public class HstView extends TTaggedStructure
{
  public void clear()
  {
    fileNameString = null;
    descString = null;
  }
  private char[] fileName = new char[64];
  private String fileNameString;
  private char[] desc = new char[64];
  private String descString;
  
  private void initStructure()
  {
    addField(fileName,"context");
    addField(desc,"server");
    initDone();    
  }
  public HstView()
  {
    super();
    initStructure();
  }
  public HstView(HstView reference)
  {
    super();
    initStructure();
    System.arraycopy(reference.fileName, 0, fileName, 0, 64);    
    System.arraycopy(reference.desc, 0, desc, 0, 64);    
  }
  public String getFileName()
  {
    if (fileNameString == null) fileNameString = new String(fileName).trim();
    return fileNameString;
  }
  public String getDescription()
  {
    if (descString == null) descString = new String(desc).trim();
    return descString;
  }
  public void setFileName(String context) {
    pushChars(context,this.fileName);
    this.fileNameString = new String(context);
  }
  public void setDescription(String description) {
    pushChars(description,this.desc);
    this.descString = new String(description);
  }
}
