/*
 * File McaCfg.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.histUtils;

import de.desy.tine.structUtils.TTaggedStructure;

public class McaCfg extends TTaggedStructure
{
  public void clear()
  {
    contextString = null;
    serverString = null;
    deviceString = null;
    propertyString = null;
    tagString = null;
    shellString = null;
    aliasString = null;
    partnerString = null;
    descriptionString = null;
  }
  private final static int MCA_APPEND_ENTRY = 1;
  private final static int MCA_REVERSE_ENTRY = 2;
  public boolean appendEntry()
  {
    if ((append[0] & MCA_APPEND_ENTRY) != 0) return true;
    return false;
  }
  public boolean reverseEntry()
  {
    if ((append[0] & MCA_REVERSE_ENTRY) != 0) return true;
    return false;
  }
  private char[] context = new char[32];
  private String contextString;
  private char[] server = new char[32];
  private String serverString;
  private char[] device = new char[64];
  private String deviceString;
  private char[] property = new char[64];
  private String propertyString;
  private char[] tag = new char[16];
  private String tagString;
  private char[] description = new char[64];
  private String descriptionString;
  private char[] shell = new char[64];
  private String shellString;
  private char[] alias = new char[64];
  private String aliasString;
  private char[] partner = new char[64];
  private String partnerString;
  private char[] reserved = new char[128];
  private int append[] = new int[1];
  private void initStructure()
  {
    addField(context,"context");
    addField(server,"server");
    addField(device,"device");
    addField(property,"property");
    addField(tag,"tag");
    addField(description,"description");
    addField(shell,"shell");
    addField(alias,"alias");
    addField(partner,"partner");
    addField(reserved,"reserved");
    addField(append,"append");
    initDone();    
  }
  public McaCfg()
  {
    super();
    initStructure();
  }
  public McaCfg(McaCfg reference)
  {
    super();
    initStructure();
    System.arraycopy(reference.context, 0, context, 0, 32);    
    System.arraycopy(reference.server, 0, server, 0, 32);    
    System.arraycopy(reference.device, 0, device, 0, 64);    
    System.arraycopy(reference.property, 0, property, 0, 64);    
    System.arraycopy(reference.tag, 0, tag, 0, 16);    
    System.arraycopy(reference.description, 0, description, 0, 64);
    System.arraycopy(reference.shell, 0, shell, 0, 16);    
    System.arraycopy(reference.alias, 0, alias, 0, 64);
    System.arraycopy(reference.partner, 0, partner, 0, 64);
    append[0] = reference.append[0];   
  }
  public String getContext()
  {
    if (contextString == null) contextString = new String(context).trim();
    return contextString;
  }
  public String getDescription()
  {
    if (descriptionString == null) descriptionString = new String(description).trim();
    return descriptionString;
  }
  public String getDevice()
  {
    if (deviceString == null) deviceString = new String(device).trim();
    return deviceString;
  }
  public String getProperty()
  {
    if (propertyString == null) propertyString = new String(property).trim();
    return propertyString;
  }
  public String getServer()
  {
    if (serverString == null) serverString = new String(server).trim();
    return serverString;
  }
  public String getShell()
  {
    if (shellString == null) shellString = new String(shell).trim();
    return shellString;
  }
  public String getAlias()
  {
    if (aliasString == null) aliasString = new String(alias).trim();
    return aliasString;
  }
  public String getPartner()
  {
    if (partnerString == null) partnerString = new String(partner).trim();
    return partnerString;
  }
  public String getTag()
  {
    if (tagString == null) tagString = new String(tag).trim();
    return tagString;
  }
  
  public void setContext(String context) {
	pushChars(context,this.context);
   	this.contextString = new String(context);
  }
  
  
  public void setServer(String server) {
	pushChars(server,this.server);
	this.serverString = new String(server);
  }
  
  public void setDevice(String device) {
	pushChars(device,this.device);
	this.deviceString = new String(device);
  }
  
  public void setProperty(String property) {
	pushChars(property,this.property);
	this.propertyString = new String(property);
  }
  
  public void setDescription(String description) {
	pushChars(description,this.description);
	this.descriptionString = new String(description);
  }
  
  public void setAlias(String alias) {
    pushChars(alias,this.alias);
	this.aliasString = new String(alias);
  }
  
  
  public void setPartner(String partner) {
	pushChars(partner,this.partner);
	this.partnerString = new String(partner);
  }
  
  public void setShell(String shell) {
	pushChars(shell,this.shell);
	this.shellString = new String(shell);
  }
  
  public void setTag(String tag) {
    pushChars(tag,this.tag);
	this.tagString = new String(tag);
  }
  
  public void setAppend(int append) {
	  this.append[0] = append;
  }
}
