/*
 * File PrfCfg.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.histUtils;

import de.desy.tine.structUtils.TTaggedStructure;

public class PrfCfg extends TTaggedStructure
{
  public void clear()
  {
    nameString = null;
    typeString = null;
    statesString = null;
    colorString = null;
  }
  private char[] name = new char[64];
  private String nameString;
  private char[] type = new char[16];
  private String typeString;
  private char[] statesBuffer = new char[512];
  private String statesString;
  private char[] color = new char[16];
  private String colorString;
  private void initStructure()
  {
    addField(name,"name");
    addField(type,"type");
    addField(statesBuffer,"states");
    addField(color,"color");
    initDone();    
  }
  public PrfCfg()
  {
    super();
    initStructure();
  }
  public PrfCfg(PrfCfg reference)
  {
    super();
    initStructure();
    System.arraycopy(reference.name, 0, name, 0, 64);    
    System.arraycopy(reference.type, 0, type, 0, 16);    
    System.arraycopy(reference.statesBuffer, 0, statesBuffer, 0, 512);    
    System.arraycopy(reference.color, 0, color, 0, 16);    
  }
  public PrfCfg(PrfCfgEx reference)
  {
    super();
    initStructure();
    setProfileName(reference.getProfileName());
    setType(reference.getType());
    setStates(reference.getStates());
    setColor(reference.getColor());
  }
  public String getProfileName()
  {
    if (nameString == null) nameString = new String(name).trim();
    return nameString;
  }
  public void setProfileName(String name) {
     pushChars(name,this.name);
     this.nameString = new String(name);
  }
  public String getType()
  {
    if (typeString == null) typeString = new String(type).trim();
    return typeString;
  }
  public void setType(String type) {
      pushChars(type,this.type);
      this.typeString = new String(type);
  }
  public String getStatesString()
  {
    if (statesString == null) statesString = new String(statesBuffer).trim();
    return statesString;
  }
  public String[] getStates()
  {
    if (statesString == null) statesString = new String(statesBuffer).trim();
    return statesString.split(",");
  }
  public void setStates(String[] states) {
      StringBuilder sb = new StringBuilder(512);
      for (String s : states) {
          sb.append(s).append(',');
      }
      String st = sb.length() > 0 ? sb.substring(0,sb.length()-1) : "";
      pushChars(st,this.statesBuffer);
      this.statesString = new String(st);
  }
  public String getColor()
  {
    if (colorString == null) colorString = new String(color).trim();
    return colorString;
  }
  public void setColor(String color) {
      pushChars(color,this.color);
      this.colorString = new String(color);
  }
  
}
