/*
 * File THistorySource.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.histUtils;

import de.desy.tine.types.NAME32DBLDBL;

public class THistorySource
{
  HstSrc CentralArchive;
  HstSrc LocalHistory;
  private int numberOfPointsReturned = 0;
  private boolean extendedTimeRange = false;
  private int heartbeat = 0;
  public void copyFrom(THistorySource src)
  {
    CentralArchive = src.CentralArchive;
    LocalHistory = src.LocalHistory;
    numberOfPointsReturned = src.getNumberOfPointsReturned();
    extendedTimeRange = src.hasExtendedTimeRange();
    heartbeat = src.getHeartbeat();
    statusList = src.getStatusList();
    annotationTimes = src.getAnnotationTimes();
    annotationActive = src.getAnnotationActive();
  }
  public int getHeartbeat()
  {
    return heartbeat;
  }
  public void setHeartbeat(int value)
  {
    if (value < 0) value = 0; 
    heartbeat = value;
  }
  public boolean hasExtendedTimeRange()
  {
    return extendedTimeRange;
  }
  public void setExtendedTimeRange(boolean hasExtendedTimeRange)
  {
    extendedTimeRange = hasExtendedTimeRange;
  }
  private NAME32DBLDBL[] statusList = null;
  private double[] annotationTimes = null;
  private boolean[] annotationActive = null;
  public HstSrc getCentralArchive()
  {
    return CentralArchive;
  }
  public HstSrc getLocalHistory()
  {
    return LocalHistory;
  }
  public int getNumberOfPointsReturned()
  {
    return numberOfPointsReturned;
  }
  public void setNumberOfPointsReturned(int numberOfPointsReturned)
  {
    this.numberOfPointsReturned = numberOfPointsReturned;
  }
  public void setStatusList(NAME32DBLDBL[] lst)
  {
    statusList = lst;
  }
  public NAME32DBLDBL[] getStatusList()
  {
    return statusList;
  }
  public void setAnnotationTimes(double[] lst)
  {
    annotationTimes = lst;
  } 
  public double[] getAnnotationTimes()
  {
    return annotationTimes;
  }
  public void setAnnotationActive(boolean[] lst)
  {
    annotationActive = lst;
  } 
  public boolean[] getAnnotationActive()
  {
    return annotationActive;
  } 
}
