/*
 * File MCastAddrMap.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.io;

import java.util.ArrayList;
import de.desy.tine.client.TLinkFactory;
import de.desy.tine.startup.TInitializer;
import de.desy.tine.startup.TInitializerFactory;

public class MCastAddrMap
{
  private static TInitializer ti = TInitializerFactory.getInstance().getInitializer();
  //private static String defaultAddr = null;
  private static ArrayList<MCastAddrMap> mcastAddrLst = new ArrayList<MCastAddrMap>();
  private static void add(MCastAddrMap newmap)
  {
    if (newmap == null) return;
    for (MCastAddrMap m : mcastAddrLst)
    {
      if (m.pat.compareTo(newmap.pat) != 0) continue;
      if (m.mca.compareTo(newmap.mca) != 0) continue;
      return; // already in the list
    }
    mcastAddrLst.add(newmap);    
  }
  public static MCastAddrMap[] getRegisteredMaps()
  {
    return mcastAddrLst.toArray(new MCastAddrMap[0]);
  }
  public static void dumpAddrMap()
  {
    TLinkFactory.dbgPrint("multicast address mappings");
    TLinkFactory.dbgPrint("  default: "+ti.getMyIpAddr()+" -> "+ti.getMCastAddress());
    int n = 0;
    for (MCastAddrMap m: mcastAddrLst)
      TLinkFactory.dbgPrint("  ["+(n++)+"]      : "+m.pat+" -> "+m.mca);
  }
  public static String getMCastAddr() 
  {
    return getMCastAddr(null);
  }
  public static String getMCastAddr(String addr)
  {
    return getMCastAddr(addr,false); 
  }
  public static String getMCastAddr(String addr,boolean useLegacy)
  {
    String defaultAddr = useLegacy ? ti.getMCastLegacyAddress() : ti.getMCastAddress();
    if (addr == null || mcastAddrLst.isEmpty()) return defaultAddr;
    String sp, st=getMcastPrefix(addr);
    for (MCastAddrMap m : mcastAddrLst)
    { // run thru list
      sp = getMcastPrefix(m.pat); // pattern prefix
      if (sp.compareTo(st) == 0) return m.mca; // given addr matches pattern
    }
    return defaultAddr; // no match (no configuration?)
  }
  private String mca;
  private String pat = "1.1.1.1";
  public MCastAddrMap(String mcaddr,String pattern)
  {
    mca = mcaddr;
    if (pattern != null) pat = pattern;
    add(this);
  }
  public String getAddr() { return mca; }
  public String getPattern() { return pat; }
  private static String getIpv4McastPrefix(String addr)
  {
    char[] ipch = addr.toCharArray();
    for (int i=0, n=0; i<ipch.length; i++)
    {
      if (ipch[i] == '.' && n++ > 0) 
      {
        return addr.substring(0, i);
      }
    }
    return null;
  }
  private static String getMcastPrefix(String addr)
  {
    if (addr.contains(":")) return addr.substring(0,addr.indexOf(':'));
    return getIpv4McastPrefix(addr);
  }
}
