/*
 * File PropertyQuery.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

/**
 * Title:        Instant Client<p>
 * Description:  <p>
 * Copyright:    Copyright (c) Piotr Karol Bartkiewicz<p>
 * Company:      DESY<p>
 * @author Piotr Karol Bartkiewicz
 * @version 1.0
 */
package de.desy.tine.queryUtils;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import de.desy.tine.definitions.TErrorList;
import de.desy.tine.endianUtils.Swap;
import de.desy.tine.server.logger.MsgLog;

public class PropertyQuery
{
  //  Simple Property Query structure
  public  String    name;       // property name
  public  String    prpDesc;    //
  public  short     prpSize;    //
  public  byte      prpFormat;  //
  public  byte      prpAccess;  // ----------------------------
  
  public static int sizeInBytes = 68;
  ByteArrayOutputStream dBuffer = new ByteArrayOutputStream(sizeInBytes);
  byte[] hByteArray;
  public PropertyQuery()
  {
    name = new String(new StringBuffer(32));
    prpDesc = new String(new StringBuffer(32));
  }  
  public byte[] toByteArray()
  {
    try
    {
    	dBuffer.reset();
    	DataOutputStream ds = new DataOutputStream(dBuffer);
    	byte[] bname = name.getBytes();
    	byte[] b = new byte[sizeInBytes];
    	for (int i=0; i<32 && i<bname.length; i++) b[i] = bname[i];
    	ds.write(b,0,32);
    	bname = prpDesc.getBytes();
    	for (int i=0; i<32 && i<bname.length; i++) b[i] = bname[i];
    	ds.write(b,0,32);
    	ds.writeShort(Swap.Short(prpSize));
    	ds.writeByte( prpFormat );
    	ds.writeByte( prpAccess );
    	ds.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("PropertyQuery","could not prepare byte stream",TErrorList.code_failure,e,0);
      return null;
    }
    hByteArray = dBuffer.toByteArray();
    return hByteArray;
  }  
  public void toStruct()
  {
    int len;
    try
    {
    	ByteArrayInputStream dis = new ByteArrayInputStream(hByteArray);
    	DataInputStream ds = new DataInputStream(dis);
    	byte[] b = new byte[32];
    	ds.read(b,0,32);
    	len = 0; for (int i=0; i<32 && b[i] != 0; i++, len++);
    	name = new String(b,0,len);
			ds.read(b,0,32);
    	len = 0; for (int i=0; i<32 && b[i] != 0; i++, len++);
    	prpDesc = new String(b,0,len);
    	prpSize = Swap.Short(ds.readShort());
  		prpFormat = ds.readByte();
  		prpAccess = ds.readByte();
    	ds.close(); dis.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("PropertyQuery","could not prepare byte stream",TErrorList.code_failure,e,0);
    }
  }  
}