/*
 * File ServerQuery.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

/*
 * Created on Mar 22, 2007
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package de.desy.tine.queryUtils;

import java.net.InetAddress;
import java.util.Arrays;

public class ServerQuery
{
  private String name = null;
  private String context = null;
  private String subsystem = null;
  private String os = null;
  private String xref = null;
  private int importance = 0;
  private byte[] haddr = new byte[4];
  private InetAddress ia = null;
  private boolean xrefIsFec = false;
  public void setXRefIsFec(boolean value) { xrefIsFec = value; }
  public boolean getXRefIsFec() { return xrefIsFec; }
  public int getImportance() { return importance; }
  public void setImportance(int value) { importance = value; }
  public void setAddress(int haddr)
  {
    this.haddr[0] = (byte)(haddr & 0x0ff);
    this.haddr[1] = (byte)((haddr>>8) & 0x0ff);
    this.haddr[2] = (byte)((haddr>>16) & 0x0ff);
    this.haddr[3] = (byte)((haddr>>24) & 0x0ff);
    try { ia = InetAddress.getByAddress(this.haddr); }
    catch (Exception ignore) {}
  }
  public InetAddress getAddress()
  {
    return ia;
  }
  public String getContext()
  {
    return context;
  }
  protected void setContext(String context)
  {
    this.context = context;
  }
  public String getName()
  {
    return name;
  }
  protected void setName(String name)
  {
    this.name = name;
  }
  public String getOs()
  {
    return os;
  }
  protected void setOs(String os)
  {
    this.os = os;
  }
  public String getSubsystem()
  {
    return subsystem;
  }
  protected void setSubsystem(String subsystem)
  {
    this.subsystem = subsystem;
  }
  public String getXref()
  {
    return xref;
  }
  public String getFecName()
  {
    return xrefIsFec ? xref : name;
  }
  public String getServerName()
  {
    return xrefIsFec ? name : xref;
  }
  protected void setXref(String xref)
  {
    this.xref = xref;
  }
  public String toString()
  {
    String msg = "";
    
    if (name != null) msg += "server = " + getServerName() + "\n";
    if (context != null) msg += "context = " + context + "\n";
    if (subsystem != null) msg += "subsystem = " + subsystem + "\n";
    if (os != null) msg += "os = " + os + "\n";
    if (xref != null) msg += "FEC = " + getFecName() + "\n";
    
    return msg;
  }
    /* (non-Javadoc)
     * @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode() {
    	final int prime = 31;
    	int result = 1;
    	result = prime * result + ((context == null) ? 0 : context.hashCode());
    	result = prime * result + Arrays.hashCode(haddr);
    	result = prime * result + ((ia == null) ? 0 : ia.hashCode());
    	result = prime * result + importance;
    	result = prime * result + ((name == null) ? 0 : name.hashCode());
    	result = prime * result + ((os == null) ? 0 : os.hashCode());
    	result = prime * result + ((subsystem == null) ? 0 : subsystem.hashCode());
    	result = prime * result + ((xref == null) ? 0 : xref.hashCode());
    	return result;
    }
    /* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals(Object obj) {
    	if (this == obj)
    		return true;
    	if (obj == null)
    		return false;
    	if (getClass() != obj.getClass())
    		return false;
    	ServerQuery other = (ServerQuery)obj;
    	if (context == null) {
    		if (other.context != null)
    			return false;
    	} else if (!context.equals(other.context))
    		return false;
    	if (!Arrays.equals(haddr,other.haddr))
    		return false;
    	if (ia == null) {
    		if (other.ia != null)
    			return false;
    	} else if (!ia.equals(other.ia))
    		return false;
    	if (importance != other.importance)
    		return false;
    	if (name == null) {
    		if (other.name != null)
    			return false;
    	} else if (!name.equals(other.name))
    		return false;
    	if (os == null) {
    		if (other.os != null)
    			return false;
    	} else if (!os.equals(other.os))
    		return false;
    	if (subsystem == null) {
    		if (other.subsystem != null)
    			return false;
    	} else if (!subsystem.equals(other.subsystem))
    		return false;
    	if (xref == null) {
    		if (other.xref != null)
    			return false;
    	} else if (!xref.equals(other.xref))
    		return false;
    	return true;
    }
  
}
