/*
 * File THistoryQuery.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

/*
 * Created on Dec 14, 2006
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package de.desy.tine.queryUtils;

import de.desy.tine.client.TLink;
import de.desy.tine.dataUtils.*;
import de.desy.tine.definitions.*;
import de.desy.tine.types.*;

public final class THistoryQuery
{
  public static boolean isMetaProperty(String key)
  {
    if (key.endsWith(".HST")) return true;
    if (key.endsWith(".HIST")) return true;
    if (key.endsWith(".ARCH")) return true;
    if (key.endsWith(".ARC")) return true;
    if (key.endsWith(".AR")) return true;
    return false;    
  }
  public static int getArchivedDataAsFloat(String fullName,long starttime,long stoptime,FLTINT[] fiArray)
  {
    String[] names;
    String srv, key;
    int[] npoints = new int[1];
    int cc;
    
    // parse the fullName (what are the rules here?)
    // "/context/server/name[property]" or "/context/server/name/property" ?
    names = TLink.splitDeviceAndPropertyFromName(fullName);
    if ((srv = names[0]) == null) return TErrorList.argument_list_error;
    if ((key = names[1]) == null) return TErrorList.argument_list_error;
    
    if (!srv.contains("HISTORY") && !isMetaProperty(key))
    { // not a central server and meta tag not yet appended
      key.concat(".HIST"); // append the lowest common denominator
    }
    double[] startstop = new double[2];
    startstop[0] = (double)(starttime/1000) + ((double)(starttime%1000)/1000);
    startstop[1] = (double)(stoptime/1000) + ((double)(stoptime%1000)/1000);

    TDataType npts = new TDataType(npoints);
    TDataType din = new TDataType(startstop);
    TLink nl = new TLink(srv,key,npts,din,TAccess.CA_READ);
    cc = nl.execute(1000,true); // number of points in region
    if (cc != 0) return -cc;

    TDataType dout = new TDataType(fiArray);
    TLink tl = new TLink(srv,key,dout,din,TAccess.CA_READ);
    cc = tl.execute(1000,true); // data returned
    if (cc != 0) return -cc;
    
    return npoints[0];
  }
}
