/*
 * File TFilterStruct.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

/*
 * Created on Oct 31, 2016
 */
package de.desy.tine.server;

import de.desy.tine.server.alarms.TAlarmDefinition;
import de.desy.tine.server.histories.THistoryRecord;
import de.desy.tine.structUtils.TTaggedStructure;

/**
 *
 * @author szczesny
 */
public class TFilterStruct extends TTaggedStructure 
{    
  private int[] filterIdx = new int[1];
  private char[] filterLink = new char[256];
  private String filterLinkString;
  
  public int getCodeIdx() { return filterIdx[0]; }
  public String getFilterLink() 
  { 
    if (filterLinkString == null) filterLinkString = new String(filterLink).trim();
    return filterLinkString; 
  } 
  
  private void initStructDescription()
  {
    addField(filterIdx,"FilterIdx");
    addField(filterLink,"FilterLink");
    initDone();
  }
  
  public TFilterStruct()
  {
    super("Filt");
    initStructDescription();
  }
  
  public TFilterStruct(int filterIndex, String filterLnk)
  {
    super("Filt");
    initStructDescription();
    setFilterIdx(filterIndex);
    pushString(filterLnk, filterLink);
  }
  
  private void pushString(String s,char[] c)
  {
    if (s != null) 
    {
      int len = s.length();
      if (len > c.length) len = c.length; 
      s.getChars(0, len, c, 0);
      for (int i=len; i<c.length; i++) c[i]=0;
    }    
  }
  
  public void setAlmFields(TAlarmDefinition reference)
  {
    filterIdx[0] = reference.getAlarmCode();
    if(reference.getFilter() == null) pushString("no assigned filter", filterLink);
    else pushString(reference.getFilter().toString(), filterLink);
  }
  
  public void setHstFields(THistoryRecord reference)
  {
    filterIdx[0] = reference.getRcdIdx();
    if(reference.getFilter() == null) pushString("no assigned filter", filterLink);
    else pushString(reference.getFilter().toString(), filterLink);
  }
  
  public void setFilterIdx(int fi)
  {
    filterIdx[0] = fi;
  }
  
  public void setFilterLnk(String fl)
  {
    pushString(fl, filterLink);
  }
}
