/*
 * File TAlarmDefinitionStructure.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.server.alarms;

import de.desy.tine.structUtils.TTaggedStructure;

public class TAlarmDefinitionStructure extends TTaggedStructure
{
  private char[] alarmTag = new char[32];
  private String alarmTagString;
  private int[] alarmCode = new int[1];
  private int[] alarmMask = new int[1];
  private short[] alarmSystem = new short[1];
  private short[] alarmSeverity = new short[1];
  private byte[] alarmDataFormat = new byte[1];
  private byte[] alarmDataArraySize = new byte[1];
  private byte[] reserved = new byte[2];
  private char[] alarmText = new char[64];
  private String alarmTextString;
  private char[] deviceText = new char[64];
  private String deviceTextString;
  private char[] dataText = new char[64];
  private String dataTextString;
  private char[] url = new char[128];
  private String urlString;
  public int getAlarmCode() { return alarmCode[0]; }
  public int getAlarmMask() { return alarmMask[0]; }
  public int getAlarmSystem() { return alarmSystem[0]; }
  public int getAlarmSeverity() { return alarmSeverity[0]; }
  public int getAlarmDataFormat() { return alarmDataFormat[0]; }
  public int getAlarmDataArraySize() { return alarmDataArraySize[0]; }
  public String getAlarmTag()
  {
    if (alarmTagString == null) alarmTagString = new String(alarmTag).trim();
    return alarmTagString;
  }
  public String getAlarmText()
  {
    if (alarmTextString == null) alarmTextString = new String(alarmText).trim();
    return alarmTextString;
  }
  public String getDeviceText()
  {
    if (deviceTextString == null) deviceTextString = new String(deviceText).trim();
    return deviceTextString;
  }
  public String getDataText()
  {
    if (alarmDataArraySize[0] == 0) dataTextString = new String("no data associated with alarm");
    if (dataTextString == null) dataTextString = new String(dataText).trim();
    return dataTextString;
  }
  public String getUrl()
  {
    if (urlString == null) urlString = new String(url).trim();
    return urlString;
  }
  public TAlarmDefinitionStructure()
  {
    super("ADSr4");
    addField(alarmTag,"alarmTag");
    addField(alarmCode,"alarmCode");
    addField(alarmMask,"alarmMask");
    addField(alarmSystem,"alarmSystem");
    addField(alarmSeverity,"alarmSeverity");
    addField(alarmDataFormat,"alarmDataFormat");
    addField(alarmDataArraySize,"alarmDataArraySize");
    addField(reserved,"reserved");
    addField(alarmText,"alarmText");
    addField(deviceText,"deviceText");
    addField(dataText,"dataText");
    addField(url,"url");
    initDone();  
  }}
