/*
 * File TAlarmWatchThreshold.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

/*
 * Created on Aug 31, 2005
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package de.desy.tine.server.alarms;

public class TAlarmWatchThreshold
{
  private int cnt;
  private float hi;
  private float hiwarn;
  private float lo;
  private float lowarn;
  private int mask;
  private int normal;
  private boolean normalIsAlarm = false;
  private int code;      /**< alarm code to apply in lieu of invalid_index */
  private int codeHigh;  /**< alarm code to apply in lieu of value_too_high */
  private int codeHighWarn;/**< alarm code to apply in lieu of warn_too_high */
  private int codeLow;   /**< alarm code to apply in lieu of value too low */
  private int codeLowWarn;/**< alarm code to apply in lieu of warn_too_low */
  private String tag;
  private String tagHigh;
  private String tagHighWarn;
  private String tagLow;
  private String tagLowWarn;
  public TAlarmWatchThreshold(int alarmCounter,float valueHigh,float warnHigh,float valueLow,float warnLow,int codeHigh,int codeHighWarn,int codeLow,int codeLowWarn,String tagHigh,String tagHighWarn,String tagLow, String tagLowWarn)
  {
    cnt = alarmCounter;
    hi = valueHigh;
    hiwarn = warnHigh;
    lo = valueLow;
    lowarn = warnLow;
    mask = 0;
    normal = 0;
    this.codeHigh = codeHigh;
    this.codeHighWarn = codeHighWarn;
    this.codeLow = codeLow;
    this.codeLowWarn = codeLowWarn;
    this.tagHigh = tagHigh;
    this.tagHighWarn = tagHighWarn;
    this.tagLow = tagLow;
    this.tagLowWarn = tagLowWarn;
  }
  public TAlarmWatchThreshold(int alarmCounter,float valueHigh,float warnHigh,float valueLow,float warnLow,int codeHigh,int codeHighWarn,int codeLow,int codeLowWarn)
  {
    this(alarmCounter,valueHigh,warnHigh,valueLow,warnLow,codeHigh,codeHighWarn,codeLow,codeLowWarn,null,null,null,null);
  }
  public TAlarmWatchThreshold(int alarmCounter,float valueHigh,float warnHigh,float valueLow,float warnLow)
  {
    cnt = alarmCounter;
    hi = valueHigh;
    hiwarn = warnHigh;
    lo = valueLow;
    lowarn = warnLow;
    mask = 0;
    normal = 0;
  }
  public TAlarmWatchThreshold(int alarmCounter,int valueMask,int valueNormal,boolean normalIsAlarm,int code,String tag)
  {
    cnt = alarmCounter;
    mask = valueMask;
    normal = valueNormal;
    this.normalIsAlarm = normalIsAlarm;
    this.code = code;
    this.tag = tag;
  }
  public TAlarmWatchThreshold(int alarmCounter,int valueMask,int valueNormal,boolean normalIsAlarm)
  {
    cnt = alarmCounter;
    mask = valueMask;
    normal = valueNormal;
    this.normalIsAlarm = normalIsAlarm;
  }
  public TAlarmWatchThreshold(int alarmCounter,int valueMask,int valueNormal)
  {
    cnt = alarmCounter;
    mask = valueMask;
    normal = valueNormal;
  }
  public int getCode() { return code; }
  public void setCode(int value) { code = value; }
  public int getCodeHigh() { return codeHigh; }
  public void setCodeHigh(int value) { codeHigh = value; }
  public int getCodeHighWarn() { return codeHighWarn; }
  public int getCodeLow() { return codeLow; }
  public void setCodeLow(int value) { codeLow = value; }
  public int getCodeLowWarn() { return codeLowWarn; }
  
  public int getValueMask() { return mask; }
  public int getValueNormal() { return normal; }
  public int getCnt() { return cnt; }
  public float getHi() { return hi; }
  public float getHiwarn() { return hiwarn; }
  public float getLo() { return lo; }
  public float getLowarn() { return lowarn; }
  public String getTag() { return tag;}
  public String getTagHigh() { return tagHigh; }
  public String getTagHighWarn() { return tagHighWarn; }
  public String getTagLow() { return tagLow; }
  public String getTagLowWarn() { return tagLowWarn; }
  public boolean isNormalAlarm()
  {
    return normalIsAlarm;
  }
}
