/*
 * File TClientStatus.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

/*
 * Created on Nov 9, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package de.desy.tine.server.connections;

import de.desy.tine.definitions.TFormat;
import de.desy.tine.definitions.TTransport;
import de.desy.tine.headers.*;
import de.desy.tine.server.equipment.TEquipmentModuleFactory;

/**
 * \internal
 * 
 * @author duval
 *
 */
public class TClientStatus
{
  public short id;          // contract ID as seen by consumer
  public short mode;        // SINGLE,REFRESH,POLL,CANCEL
  public int PollingRate;   // requested polling rate
  public long lasttime;     // time of most recent update
  public long starttime;    // start of initial subscription
  public short counter;     // counts remaining in subscription(s)
  public short Stale;       // 1 = send new data to consumer
  public short numblks;     // total number of message blocks
  public short blknum;      // block number of this message
  public short statusCode;  // interim completion
  public int mtu;           // maximal transport unit (carried as unsigned short)
  public int nextDataSize;  // next outgoing data size in bytes
  public int nextDataPtr;   // next outgoing data array pointer
  public int BytesRemaining;// number of bytes data remaining
  public int BytesTotal;    // total number of bytes data
  public int misses;        // data return after expiration
  public int reconnects;    // client unexpectedly re-establishes link
  public boolean isRetry;   // the contract is being resubmitted by the client
  public int transport;     // overlying transport mechanism
  public int protocol;      // tine protocol level
  public TClientStatus()
  {
    transport = TTransport.UDP;
  }
  public TClientStatus(int transport,int protocol)
  {
    this.transport = transport;
    this.protocol = protocol;
  }
  public void reset(TSubReqHdr sub)
  {
    int datasize = sub.getOutputDataSize();
    int fmtsize = TFormat.formatSizeOf(sub.contract.dataFormatOut);
    // reset certain fields according to the subscription ...
    PollingRate = sub.pollingInterval;
    id = sub.id;
    starttime = 1000 * (long)sub.starttime;
    blknum = 1;
    nextDataPtr = 0;
    BytesRemaining = datasize;
    BytesTotal = datasize;
    int hdrSize = TSubRspHdr.getHeaderSizeInBytes(protocol);
    int pmtu = TEquipmentModuleFactory.getServerPacketMTU();
    if (transport == TTransport.TCP && pmtu > TTransport.TCP_BUFFER_SIZE)
    { // don't allow larger MTUs for TCP !
      pmtu = TTransport.TCP_BUFFER_SIZE;
    }
    int datamtu = pmtu - hdrSize - 2;
    if (fmtsize > 0) datamtu -= datamtu%fmtsize;
    mtu = datamtu;
    nextDataSize = BytesRemaining;
    if (BytesRemaining > mtu) nextDataSize = mtu;
    numblks = (short)(datasize/mtu + (datasize%mtu > 0 ? 1 : 0));
    if (datasize == 0) numblks = 1;
  }
}
