/*
 * File TConnectionStruct.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.server.connections;

import de.desy.tine.definitions.TStrings;
import de.desy.tine.structUtils.TTaggedStructure;

public class TConnectionStruct extends TTaggedStructure
{
  char[] expName = new char[TStrings.EXPORT_NAME_SIZE];
  char[] prpName = new char[TStrings.PROPERTY_NAME_SIZE];
  char[] devName = new char[TStrings.DEVICE_NAME_SIZE];
  char[] context = new char[TStrings.CONTEXT_NAME_SIZE];
  char[] eqm = new char[TStrings.EQM_NAME_SIZE];
  short[] mode = new short[1];
  short[] access = new short[1];
  short[] formatIn = new short[1];
  short[] formatOut = new short[1];
  int[] sizeIn = new int[1];
  int[] sizeOut = new int[1];
  char[] strTagIn = new char[TStrings.TAG_NAME_SIZE];
  char[] strTagOut = new char[TStrings.TAG_NAME_SIZE];
  int[] pollRate = new int[1];
  int[] heartbeat = new int[1];
  int[] starttime = new int[1];
  int[] dtimestamp = new int[1];
  int[] dtimestampMSEC = new int[1];
  int[] counter = new int[1];
  short[] linkStatus = new short[1];
  short[] reserved = new short[1];

  private String charsToString(char[] c)
  {
    boolean done = false;
    for (int i=0; i<c.length; i++)
    {
      if (c[i] != 0 && !done) continue;
      done = true;
      c[i] = 0;
    }
    return new String(c).trim();
  }
  private void pushStringToChars(String s, char[] c)
  {
    int len = s.length();
    if (len > c.length) len = c.length;
    s.getChars(0, len, c, 0);
    for (int i=len; i<c.length; i++)
    {
      c[i] = 0;
    }
  }
  public TConnectionStruct()
  {
    super("CONTBLr4");
    addField(expName,"expName");
    addField(prpName,"prpName");
    addField(devName,"devName");
    addField(context,"context");
    addField(eqm,"eqm");
    addField(mode,"mode");
    addField(access,"access");
    addField(formatIn,"formatIn");
    addField(formatOut,"formatOut");
    addField(sizeIn,"sizeIn");
    addField(sizeOut,"sizeOut");
    addField(strTagIn,"strTagIn");
    addField(strTagOut,"strTagOut");
    addField(pollRate,"pollRate");
    addField(heartbeat,"heartbeat");
    addField(starttime,"starttime");
    addField(dtimestamp,"dtimestamp");
    addField(dtimestampMSEC,"dtimestampMSEC");
    addField(counter,"counter");
    addField(linkStatus,"linkStatus");
    addField(reserved,"reserved");
    initDone();
  }
  public String getExpName()
  {
    return charsToString(expName);
  }
  public void setExpName(String expName)
  {
    pushStringToChars(expName,this.expName);
  }
  public String getPrpName()
  {
    return charsToString(prpName);
  }
  public void setPrpName(String prpName)
  {
    pushStringToChars(prpName,this.prpName);
  }
  public String getDevName()
  {
    return charsToString(devName);
  }
  public void setDevName(String devName)
  {
    pushStringToChars(devName,this.devName);
  }
  public String getContext()
  {
    return charsToString(context);
  }
  public void setContext(String context)
  {
    pushStringToChars(context,this.context);
  }
  public String getEqm()
  {
    return charsToString(eqm);
  }
  public void setEqm(String eqm)
  {
    pushStringToChars(eqm,this.eqm);
  }
  public short getMode()
  {
    return mode[0];
  }
  public void setMode(short mode)
  {
    this.mode[0] = mode;
  }
  public short getAccess()
  {
    return access[0];
  }
  public void setAccess(short access)
  {
    this.access[0] = access;
  }
  public short getFormatIn()
  {
    return formatIn[0];
  }
  public void setFormatIn(short formatIn)
  {
    this.formatIn[0] = formatIn;
  }
  public short getFormatOut()
  {
    return formatOut[0];
  }
  public void setFormatOut(short formatOut)
  {
    this.formatOut[0] = formatOut;
  }
  public int getSizeIn()
  {
    return sizeIn[0];
  }
  public void setSizeIn(int sizeIn)
  {
    this.sizeIn[0] = sizeIn;
  }
  public int getSizeOut()
  {
    return sizeOut[0];
  }
  public void setSizeOut(int sizeOut)
  {
    this.sizeOut[0] = sizeOut;
  }

  public String getStrTagIn()
  {
    return charsToString(strTagIn);
  }
  public void setStrTagIn(String strTagIn)
  {
    pushStringToChars(strTagIn,this.strTagIn);
  }
  public String getStrTagOut()
  {
    return charsToString(strTagOut);
  }
  public void setStrTagOut(String strTagOut)
  {
    pushStringToChars(strTagOut,this.strTagOut);
  }
  public int getPollRate()
  {
    return pollRate[0];
  }
  public void setPollRate(int interval)
  {
    this.pollRate[0] = interval;
  }
  public int getHeartbeat()
  {
    return heartbeat[0];
  }
  public void setHeartbeat(int heartbeat)
  {
    this.heartbeat[0] = heartbeat;
  }
  public int getStarttime()
  {
    return starttime[0];
  }
  public void setStarttime(int starttime)
  {
    this.starttime[0] = starttime;
  }
  public int getDtimestamp()
  {
    return dtimestamp[0];
  }
  public void setDtimestamp(int secs)
  {
    this.dtimestamp[0] = secs;
  }
  public int getDtimestampMSEC()
  {
    return dtimestampMSEC[0];
  }
  public void setDtimestampMSEC(int msecs)
  {
    this.dtimestampMSEC[0] = msecs;
  }
  public int getCounter()
  {
    return counter[0];
  }
  public void setCounter(int counter)
  {
    this.counter[0] = counter;
  }
  public short getLinkStatus()
  {
    return linkStatus[0];
  }
  public void setLinkStatus(short status)
  {
    linkStatus[0] = status;
  }
}
