/*
 * File TContractEntry.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

/*
 * Created on Mar 11, 2007
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package de.desy.tine.server.connections;

import de.desy.tine.definitions.TStrings;
import de.desy.tine.structUtils.TTaggedStructure;

public class TContractEntry extends TTaggedStructure
{
  char[] EqpProperty = new char[TStrings.PROPERTY_NAME_SIZE];
  char[] EqpDeviceName = new char[TStrings.DEVICE_NAME_SIZE];
  char[] EqpName = new char[TStrings.EQM_NAME_SIZE];
  int[] EqpSizeIn = new int[1];
  int[] EqpSizeOut = new int[1];
  byte[] hEqpName = new byte[1];
  byte[] EqpAccess = new byte[1];
  byte[] EqpFormatIn = new byte[1];
  byte[] EqpFormatOut = new byte[1];
  char[] strTagIn = new char[TStrings.TAG_NAME_SIZE];
  char[] strTagOut = new char[TStrings.TAG_NAME_SIZE];
  int[] pollrate = new int[1];
  int[] nconsumers = new int[1];

  public TContractEntry()
  {
    super("CTQSr4");
    addField(EqpProperty,"property");
    addField(EqpDeviceName,"device");
    addField(EqpName,"eqm");
    addField(EqpSizeIn,"sizeIn");
    addField(EqpSizeOut,"sizeOut");
    addField(hEqpName,"eqmhandle");
    addField(EqpAccess,"access");
    addField(EqpFormatIn,"formatIn");
    addField(EqpFormatOut,"formatOut");
    addField(strTagIn,"tagIn");
    addField(strTagOut,"tagOut");
    addField(pollrate,"pollrate");
    addField(nconsumers,"nconsumers");
    initDone();
  }

  public byte[] getEqpAccess()
  {
    return EqpAccess;
  }

  public void setEqpAccess(byte[] eqpAccess)
  {
    EqpAccess = eqpAccess;
  }

  public char[] getEqpDeviceName()
  {
    return EqpDeviceName;
  }

  public void setEqpDeviceName(char[] eqpDeviceName)
  {
    EqpDeviceName = eqpDeviceName;
  }

  public byte[] getEqpFormatIn()
  {
    return EqpFormatIn;
  }

  public void setEqpFormatIn(byte[] eqpFormatIn)
  {
    EqpFormatIn = eqpFormatIn;
  }

  public byte[] getEqpFormatOut()
  {
    return EqpFormatOut;
  }

  public void setEqpFormatOut(byte[] eqpFormatOut)
  {
    EqpFormatOut = eqpFormatOut;
  }

  public char[] getEqpName()
  {
    return EqpName;
  }

  public void setEqpName(char[] eqpName)
  {
    EqpName = eqpName;
  }

  public char[] getEqpProperty()
  {
    return EqpProperty;
  }

  public void setEqpProperty(char[] eqpProperty)
  {
    EqpProperty = eqpProperty;
  }

  public int[] getEqpSizeIn()
  {
    return EqpSizeIn;
  }

  public void setEqpSizeIn(int[] eqpSizeIn)
  {
    EqpSizeIn = eqpSizeIn;
  }

  public int[] getEqpSizeOut()
  {
    return EqpSizeOut;
  }

  public void setEqpSizeOut(int[] eqpSizeOut)
  {
    EqpSizeOut = eqpSizeOut;
  }

  public byte[] getHEqpName()
  {
    return hEqpName;
  }

  public void setHEqpName(byte[] eqpName)
  {
    hEqpName = eqpName;
  }

  public int[] getNconsumers()
  {
    return nconsumers;
  }

  public void setNconsumers(int[] nconsumers)
  {
    this.nconsumers = nconsumers;
  }

  public int[] getPollrate()
  {
    return pollrate;
  }

  public void setPollrate(int[] pollrate)
  {
    this.pollrate = pollrate;
  }

  public char[] getStrTagIn()
  {
    return strTagIn;
  }

  public void setStrTagIn(char[] strTagIn)
  {
    this.strTagIn = strTagIn;
  }

  public char[] getStrTagOut()
  {
    return strTagOut;
  }

  public void setStrTagOut(char[] strTagOut)
  {
    this.strTagOut = strTagOut;
  }
}
