/*
 * File TNetAcl.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.server.connections;

import java.net.InetAddress;
import de.desy.tine.definitions.*;
import de.desy.tine.server.logger.*;

public class TNetAcl
{
  @Override
  public boolean equals(Object obj)
  {
    if (obj instanceof TNetAcl)
    {
      TNetAcl a = (TNetAcl)obj;
      return (a.getStrAddress().compareToIgnoreCase(strAdr) == 0);
    }
    if (obj instanceof String)
    {
      String s = (String)obj;
      return (s.compareToIgnoreCase(strAdr) == 0);
    }
    return false;
  }
  private String strAdr = null;
  private byte[] rawAdr = null;
  private int mask = 0;
  public TNetAcl(String netAdr)
  {
    strAdr = netAdr;
    if (netAdr == null || netAdr.length() == 0) return;
    String adr;
    int idx;
    try
    {
      if ((idx=netAdr.indexOf('/')) >= 0)
      {
        adr = strAdr.substring(0, idx);
        mask = Integer.parseInt(strAdr.substring(idx+1));
      }
      else
      {
        mask = 32; // start off pessimistic 
        adr = strAdr;
        String[] ip4segs = netAdr.split("\\.");
        if (ip4segs.length == 4 && ip4segs[3].compareTo("255") == 0) mask = 24;
        if (ip4segs.length == 4 && ip4segs[3].compareTo("0") == 0) mask = 24;
      }
      rawAdr = InetAddress.getByName(adr).getAddress();
    }
    catch (Exception e)
    {
      MsgLog.log("TNetAcl", "could not instantiate",TErrorList.code_failure,e,0);
    }
  }
  public String getStrAddress()
  {
    return strAdr;
  }
  public byte[] getRawAddress()
  {
    return rawAdr;
  }
  public int getMask()
  {
    return mask;
  }
}
