/*
 * File TCycleTrigger.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.server.equipment;


/**
 * Server side cycle trigger methods
 * A cycle trigger interface.
 * 
 * The update() method will be called upon incoming 
 * cycle triggers (or system error).  The current cycle number will be passed 
 * along with the status (usually 0). 
 * 
 * @author duval
 *
 */
public interface TCycleTrigger
{
  /**
   * Is processed following an incoming cycle number event.
   * 
   * If a CYCLER exists and a TCycleTrigger is in use, then
   * its update routine will be called following the trigger
   * event associated with the incoming cycle number or a link
   * timeout notification.
   * 
   * By default a CYCLER is a server called "CYCLER" in the same context as the
   * initializing device server and produces a keyword (property) called
   * "CycleNumber".  This gives rise to the Cycle Key: 
   * "/<context>/CYCLER/CycleNumber". If it is known a priori, that the 
   * CYCLER exists in another context or uses another server name or property
   * name, the cycle key can be set via API (setCyclerNumberKey() in
   * TEquipmentModule) or via environment (set TINE_CYCLE_KEY=\<key\>).
   * 
   * @param cycleNumber is the last arrived cycle number
   * @param status is the cycle link status
   */
  void update(int cycleNumber,int status);
}
