/*
 * File TCycleTriggerCallback.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.server.equipment;

import de.desy.tine.client.TLink;
import de.desy.tine.client.TLinkCallback;
import de.desy.tine.server.logger.MsgLog;

public class TCycleTriggerCallback implements TLinkCallback
{
  private int last_cc = -1;
  private int last_cycleNumber = -1;
  public void callback(TLink link)
  {
    int cc = link.getLinkStatus();
    if (cc != last_cc)
    {
      last_cc = cc;
      MsgLog.log("TCycleTriggerCallback","cycle trigger : " + link.getLastError(),cc,null,1);
    }
    if (cc != 0) return;
    int cycleNumber = TEquipmentModuleFactory.getCycleNumber();
    if (cycleNumber == last_cycleNumber) return;
    last_cycleNumber = cycleNumber;
    TCycleTrigger[] trgs = TEquipmentModuleFactory.getCycleTriggerList(); 
    for (int i=0; i<trgs.length; i++) trgs[i].update(cycleNumber, cc);
    TEquipmentModuleFactory.setSystemStamp(cycleNumber);
    // schedule properties 
  }    
}
