/*
 * File TEquipmentBackgroundTask.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

/*
 * Created on Nov 5, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package de.desy.tine.server.equipment;

/**
 * Servers wishing to register a background IO task should extend 
 * TEquipmentBackgroundTask
 * 
 * @author duval
 *
 */
public class TEquipmentBackgroundTask
{
  private long rateBkg = 1000;        // background polling rate msec
  private boolean isIdle = false;
  private TEquipmentModule eqm = null;
  public TEquipmentBackgroundTask() {}
  /**
   * Create a background task with the specified inteval and assign
   * it to the specified equipment module
   * 
   * @param eqm is the reference to an equipment module to be associated with the task
   * @param interval is the timer interval in milliseconds of the task 
   */
  public TEquipmentBackgroundTask(TEquipmentModule eqm,long interval)
  {
    this.setEquipmentModule(eqm);
    rateBkg = interval;
  }
  /** Returns the current idle state */
  public boolean isIdleState() { return isIdle; }
  /** Sets the idle state to the value given */
  public void setIdleState(boolean value) { isIdle = value; }
  /** Is the background dispatch routine */
  public void call() 
  { // the task
    return; 
  }
  /** Is the initialization dispatch routine */
  public void startup() 
  { // initialization prior to calls to task
    return;
  }
  /** Sets the background interval to the value given */
  public void setBackgroundTaskInterval(long msec)
  {
    if (msec > 0) rateBkg = msec;
  }
  /** Returns the background interval */
  public long getBackgroundTaskInterval() { return rateBkg; }
  public void setEquipmentModule(TEquipmentModule eqm)
  {
    this.eqm = eqm;
  }
  public TEquipmentModule getEquipmentModule()
  {
    return eqm;
  }
}
