/*
 * File TPropertySignal.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.server.equipment;

public class TPropertySignal
{
  public final static int PS_NONE = 0x00; // NULL signal
  public final static int PS_ACCESS = 0x01; // property is being accessed by a caller
  public final static int PS_RETRY = 0x02; // property is being retried
  public final static int PS_LATE = 0x04; // property is being returned late
  public final static int PS_PENDING = 0x08; // property is being called while last transmission still pending
  public final static int PS_SENT = 0x10; // property has been sent to caller
  public final static int PS_CALLED = 0x20; // property has been called and result returned
  public final static int PS_PROCESSED = 0x40; // property's returned data has been processed
  public final static int PS_SCHEDULED = 0x80; // property has been explicitly scheduled
  public final static int PS_ALL = 0xff; // signal on any event
  public static boolean isSet(int signal, int sigMask)
  {
    if (sigMask == PS_NONE || sigMask == PS_ALL) return true;
    return ((signal & sigMask) == signal);
  }
  public static String toString(int signal)
  {
    String s = "";
    String add = "";
    if ((signal & PS_ACCESS) != 0) s = "access";
    if (s.length() > 0 && add.length() == 0) add = " + ";
    if ((signal & PS_RETRY) != 0) s += add + "retry";
    if (s.length() > 0 && add.length() == 0) add = " + ";
    if ((signal & PS_LATE) != 0) s += add + "late";
    if (s.length() > 0 && add.length() == 0) add = " + ";
    if ((signal & PS_PENDING) != 0) s += add + "pending";
    if (s.length() > 0 && add.length() == 0) add = " + ";
    if ((signal & PS_SENT) != 0) s += add + "sent";
    if (s.length() > 0 && add.length() == 0) add = " + ";
    if ((signal & PS_CALLED) != 0) s += add + "called";
    if (s.length() > 0 && add.length() == 0) add = " + ";
    if ((signal & PS_PROCESSED) != 0) s += add + "processed";
    if (s.length() > 0 && add.length() == 0) add = " + ";
    if ((signal & PS_SCHEDULED) != 0) s += add + "scheduled";
    return s;
  }
}
