/*
 * File TWriteAccessInfo.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.server.equipment;

import de.desy.tine.definitions.TStrings;
import de.desy.tine.structUtils.TTaggedStructure;

public class TWriteAccessInfo extends TTaggedStructure
{
  char[] user = new char[TStrings.USERNAME_SIZE];
  char[] addr = new char[TStrings.ADDR_SIZE];
  char[] prp = new char[TStrings.PROPERTY_NAME_SIZE];
  char[] dev = new char[TStrings.DEVICE_NAME_SIZE];
  double[] timestamp = new double[1];
  int port = 0;
  String eqm = null;
  public String getUser() { return charsToString(user); }
  public String getAddress() { return charsToString(addr); }
  public String getProperty() { return charsToString(prp); }
  public String getDevice() { return charsToString(dev); }
  public double getTime() { return timestamp[0]; }
  public String getEqm() { return eqm; } 
  private String charsToString(char[] c)
  {
    boolean done = false;
    for (int i=0; i<c.length; i++)
    {
      if (c[i] != 0 && !done) continue;
      done = true;
      c[i] = 0;
    }
    return new String(c).trim();
  }
  private void pushStringToChars(String s, char[] c)
  {
    int len = s == null ? 0 : s.length();
    if (len > c.length) len = c.length;
    if (len > 0) s.getChars(0, len, c, 0);
    for (int i=len; i<c.length; i++)
    {
      c[i] = 0;
    }
  }
  public TWriteAccessInfo()
  {
    super("WRACCTBL");
    addField(this.user,"user");
    addField(this.addr,"addr");
    addField(this.prp,"prp");
    addField(this.dev,"dev");
    addField(timestamp,"timestamp");
    initDone();
  }  
  public TWriteAccessInfo(String user,String addr,String eqm,String prp,String dev,double time)
  {
    this(user,addr,0,eqm,prp,dev,time);
  }  
  public TWriteAccessInfo(String user,String addr,int port,String eqm,String prp,String dev,double time)
  {
    super("WRACCTBL");
    addField(this.user,"user");
    addField(this.addr,"addr");
    addField(this.prp,"prp");
    addField(this.dev,"dev");
    addField(timestamp,"timestamp");
    pushStringToChars(user, this.user);
    pushStringToChars(addr, this.addr);
    pushStringToChars(prp, this.prp);
    pushStringToChars(dev, this.dev);
    timestamp[0] = time;
    initDone();
    this.port = port;
    this.eqm = new String(eqm);
  }  
}
