/*
 * File DSUMMARY.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.server.histories;

import de.desy.tine.structUtils.TTaggedStructure;

public class DSUMMARY extends TTaggedStructure
{
  public static final int sizeInBytes = 48; // 4 doubles, 4 integers
  double[] ave = new double[1]; /**< data average over range */
  double[] dev = new double[1]; /**< standard deviation */
  double[] max = new double[1]; /**< data maximum */
  double[] min = new double[1]; /**< data minimum */
  int[] ts = new int[1];  /**< centered timestamp */
  int[] tMax = new int[1];/**< timestamp maximum */
  int[] tMin = new int[1];/**< timestamp minimum */
  int[] cnt = new int[1]; /**< number of data points in interval */
  
  private void initStructure()
  {
    addField(ave,"average");
    addField(dev,"deviation");
    addField(max,"maximum");
    addField(min,"minimum");
    addField(ts,"timeAve");
    addField(tMax,"timeMax");
    addField(tMin,"timeMin");
    addField(cnt,"counts");
    initDone();      
  }
  public DSUMMARY()
  {
    super("DSUMMARY");
    initStructure();
  }
  public DSUMMARY(double ave,double dev,double max,double min,
      long ts, long tsMax, long tsMin, int cnt)
  {
    super("DSUMMARY");
    initStructure();
    this.ave[0] = ave;
    this.dev[0] = dev;
    this.max[0] = max;
    this.min[0] = min;
    this.ts[0] = (int)(ts/1000);
    this.tMax[0] = (int)(tsMax/1000);
    this.tMin[0] = (int)(tsMin/1000);
    this.cnt[0] = cnt;
  }
  public double getAverage() { return ave[0]; }
  public double getDeviation() { return dev[0]; }
  public double getMax() { return max[0]; }
  public double getMin() { return min[0]; }
  public double[] getSummary()
  {
    double[] ds = new double[4];
    ds[0] = ave[0];
    ds[1] = dev[0];
    ds[2] = max[0];
    ds[3] = min[0];
    return ds;
  }
}
