/*
 * File THistorySpecification.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

/*
 * Created on Sep 1, 2005
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package de.desy.tine.server.histories;

public class THistorySpecification
{
  private int  pRate;              /* polling rate in msec */
  private int  aRate;               /* archive rate in msec */
  private int  depthSht;             /* for short term storage */
  private int  depthLng;               /* for long term storage */
  private int  hbt;            /* archive heartbeat in sec */
  private float pTol;                 /* percent tolerance */
  private float aTol;                /* absolute tolerance */
  private String rhsSrv;             /* Remote Server Name */
  private static THistorySpecification defaultInstance = null;
  public static THistorySpecification getDefaultInstance()
  {
    if (defaultInstance == null)
    {
      defaultInstance = new THistorySpecification(1000,1000,600,1,1800,(float)0.1,0,"");
    }
    return defaultInstance;
  }
  public THistorySpecification(int pollingRate,int archiveRate,int depthShort,int depthLong,int heartbeat,
                        float pcntTolerance,float absTolerance,String remoteServerSpecification)
  {
    pRate = pollingRate;
    aRate = archiveRate;
    depthSht = depthShort;
    depthLng = depthLong;
    hbt = heartbeat;
    if (pcntTolerance >= 1) pcntTolerance /= 100;
    pTol = pcntTolerance;
    aTol = absTolerance;
    rhsSrv = remoteServerSpecification;
  }
  public int getArchiveRate()
  {
    return aRate;
  }
  public void setArchiveRate(int rate)
  {
    aRate = rate;
  }
  public float getAbsoluteTolerance()
  {
    return aTol;
  }
  public void setAbsoluteTolerance(float tolerance)
  {
    aTol = tolerance;
  }
  public int getDepthLong()
  {
    return depthLng;
  }
  public int getDepthMonths()
  {
    return depthLng & 0xffff;
  }
  public int getDepthDays()
  {
    int ldd = depthLng >> 16;
    return ldd & 0x1f;            
  }
  public void setDepthLong(int depthLong)
  {
    this.depthLng = depthLong;
  }
  public void setDepthMonths(int depthMonths)
  {
    depthLng |= depthMonths;
  }
  public void setDepthDays(int depthDays)
  {
    int m = depthDays/31;
    depthLng += m;
    depthDays = depthDays%31;
    depthLng |= (depthDays << 16);
  }
  public int getDepthShort()
  {
    return depthSht;
  }
  public void setDepthShort(int depthShort)
  {
    this.depthSht = depthShort;
  }
  public int getDepthShortInSeconds()
  {
    return (int)(depthSht * (pRate/1000.0));
  }
  public int getHeartbeat()
  {
    return hbt;
  }
  public void setHeartbeat(int heartbeat)
  {
    this.hbt = heartbeat;
  }
  public int getPollngRate()
  {
    return pRate;
  }
  public void setPollngRate(int rate)
  {
    pRate = rate;
  }
  public float getPercentTolerance()
  {
    return pTol;
  }
  public void setPercentTolerance(float tolerance)
  {
    pTol = tolerance;
  }
  public String getRemoteServerSpecification()
  {
    return rhsSrv;
  }
  public void setRemoteServerSpecification(String serverSpecification)
  {
    this.rhsSrv = serverSpecification;
  }
}
