/*
 * File TPropertyEGU.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

/*
 * Created on Nov 5, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 * 
 * CHANGES 
 * 
 * 2005-02-10 JW  * min,maxValue renamed to minValue, maxValue
 * 				  * attributes made private and "javaized"
 * 				  * new Constructors
 * 				  
 */
package de.desy.tine.server.properties;

import de.desy.tine.definitions.TGraphType;

/**
 * \internal
 * 
 * @author duval 
 */
public class TPropertyEGU
{
  private String units;
  private float minValue;
  private float maxValue;
  private short graphType;
  private void initialize(String newUnits, short newGraphType, float newMinValue, float newMaxValue)
  {
    this.graphType = newGraphType;
    this.units = newUnits;
    this.minValue = newMinValue;
    this.maxValue = newMaxValue;
  }
  /**
   * Default Constructor. Creates a TPropertyEgu with units="none",
   * graphType=linear, minValue=0, maxValue=0.
   */
  public TPropertyEGU()
  {
    initialize("none", TGraphType.GT_LIN, 0, 0);
  }
  public TPropertyEGU(TPropertyEGU reference)
  {
  	if (reference == null) initialize("none", TGraphType.GT_LIN, 0, 0);
  	else initialize(reference.units, reference.graphType, reference.minValue, reference.maxValue);
  }
  /**
   * Constructs a TPropertyEGU with parameters given
   * 
   * @param newUnits
   * @param newGraphType
   * @param newMinValue
   * @param newMaxValue
   */
  public TPropertyEGU(String newUnits, short newGraphType, float newMinValue, float newMaxValue)
  {
    initialize(newUnits, newGraphType, newMinValue, newMaxValue);
  }
  /**
   * Constructs a TPropertyEGU with graph type GT_LIN
   * 
   * @param newUnits
   * @param newMinValue
   * @param newMaxValue
   */
  public TPropertyEGU(String newUnits, float newMinValue, float newMaxValue)
  {
    initialize(newUnits, TGraphType.GT_LIN, newMinValue, newMaxValue);
  }
  /**
   * @return Returns the graphType.
   */
  public short getGraphType()
  {
    return graphType;
  }
  /**
   * @param graphType
   *          The graphType to set.
   */
  public void setGraphType(short graphType)
  {
    this.graphType = graphType;
  }
  public void setGraphType(String graphTypeString)
  {
    graphType = TGraphType.getType(graphTypeString);
  }
  /**
   * @return Returns the maxValue.
   */
  public float getMaxValue()
  {
    return maxValue;
  }
  /**
   * @param maxValue
   *          The maxValue to set.
   */
  public void setMaxValue(float maxValue)
  {
    this.maxValue = maxValue;
  }
  /**
   * @return Returns the minValue.
   */
  public float getMinValue()
  {
    return minValue;
  }
  /**
   * @param minValue
   *          The minValue to set.
   */
  public void setMinValue(float minValue)
  {
    this.minValue = minValue;
  }
  /**
   * @return Returns the units.
   */
  public String getUnits()
  {
    return units;
  }
  /**
   * @param units
   *          The units to set.
   */
  public void setUnits(String units)
  {
    this.units = units;
  }
}