/*
 * File TInitializer.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.startup;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.util.logging.FileHandler;

public interface TInitializer
{
  public String getENSAddress();
  public void setENSAddress(String ip);
  public int getSrvPort();
  public int getClnPort();
  public int getTCPPort();
  public int getStreamPort();
  public int getPipePort();
  public int getGCastPort();
  public String getGCastAddress();
  public String getGCastLegacyAddress();
  public int getMCastPort();
  public String getNetCastAddress();
  public int getNetCastPort();
  public String getMCastAddress();
  public String getMCastLegacyAddress();
  public String getMyIpAddr();
  public byte[] getMacAddr();
  public String getMacAddrString();
  public String getMyIpv4Addr();
  public String getMyIpv6Addr();
  public String getMyHostName();  
  public String getHistoryHome();
  public String getFecHome();
  public String getTineHome();
  public String getLogHome();
  public String getTineCache();
  public String getSystemStartup();
  public String getSystemCwd();
  public BufferedReader getAddressResource();
  public BufferedReader getEquipmentResource();
  public BufferedReader getHostsResource();
  public BufferedReader getFecIdResource();
  public BufferedReader getExportsResource(String eqmName); 
  public BufferedReader getUsersResource(String eqmName,String tag);
  public BufferedReader getFollowersResource(String eqmName);
  public BufferedReader getGroupsResource(String eqmName,String grpName);
  public BufferedReader getNetsResource(String eqmName,String tag);
  public BufferedReader getAliasResource(String eqmName); 
  public BufferedReader getHistoryResource(String eqmName,String fileName); 
  public BufferedReader getAlmWatchResource(String eqmName);
  public BufferedReader getAlmDefinitionResource(String eqmName);
  public BufferedReader getDevicesResource(String eqmName);
  public FileHandler getFecLogResource();
  public BufferedWriter getFecLogWriter();
  public FileHandler getErrorLogResource();
  public BufferedWriter getErrorLogWriter();
  public FileHandler getCmdLogResource();
  public BufferedWriter getCmdLogWriter();
  public String getNetsFileLocation(String eqmName);
  public String getUsersFileLocation(String eqmName);
  public int getClnRcvBufferSize();
  public void setClnRcvBufferSize(int clnRcvBufferSize);
  public int getClnSndBufferSize();
  public void setClnSndBufferSize(int clnRcvBufferSize);
  public int getSckTimeToLive();
  public void setSckTimeToLive(int sckTimeToLive);
  public int getSrvPacketMtu();
  public void setSrvPacketMtu(int srvPacketMtu);
  public int getSrvRcvBufferSize();
  public void setSrvRcvBufferSize(int srvRcvBufferSize);
  public int getSrvSndBufferSize();
  public void setSrvSndBufferSize(int srvRcvBufferSize);
  public String getVersion(); 
  public int getMajorVersion();
  public int getMinorVersion();
  public int getRevision();
  public int getBuildId();
  public String getOs();
  public int getPid();
  public int getRevisionId();
  public int getVersionId();
  public boolean isRichClient();
  public void setRichClient(boolean value);
}
