/*
 * File TInitializerFactory.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.startup;

public abstract class TInitializerFactory
{
  private static TInitializerFactory factory = null;
  protected TInitializerFactory()
  {
  }
  public synchronized static TInitializerFactory getInstance()
  {
    if (factory != null) return factory;
    String className = System.getProperty("de.desy.tine.client.TInitializerFactory");
    if (className == null)
      factory = new DefaultTInitializerFactory();
    else
    {
      try
      {
        factory = (TInitializerFactory) Class.forName(className).getDeclaredConstructor().newInstance();
      }
      catch (Exception e)
      {
        throw new IllegalStateException(
            "Could not instantiate de.desy.tine.client Initializer Factory, reason: " + e);
      }
    }
    return factory;
  }
  public abstract TInitializer getInitializer();
}
