/*
 * File WildcardMatch.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

/*
 * Created on Feb 16, 2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package de.desy.tine.stringUtils;

/**
 * \internal
 * 
 * @author MCS TODO To change the template for this generated type comment go
 *         to Window - Preferences - Java - Code Style - Code Templates
 */
public class WildcardMatch
{
  private String[] parts; // Things we don't show around in public.
  private String[] partsLc; // Things we don't show around in public.
  private boolean startFixed = true;
  private boolean endFixed = true;
  private boolean matchAll;
  private boolean matchSingle;
  private boolean valid = true;
  private boolean caseSensitive = false;
  public WildcardMatch(String pattern)
  {
    valid = ((pattern != null) && (pattern.length() > 0));
    if (valid)
    {
      matchAll = pattern.equals("*");
      matchSingle = pattern.indexOf("*") < 0;
      if (matchAll)
      {
        parts = null;
      }
      else if (matchSingle)
      {
        parts = new String[1];
        parts[0] = pattern;
      }
      else
      {
        startFixed = !pattern.startsWith("*");
        endFixed = !pattern.endsWith("*");
        parts = pattern.split("\\*");
        valid = (parts != null);
      }
      if (parts != null)
      {
        partsLc = new String[parts.length];
        for (int i=0; i<parts.length; i++)
          partsLc[i] = parts[i].toLowerCase();
      }
    }
  }
  public boolean matches(String comparedText)
  {
    boolean rv = valid; // if valid, set rv true optimistically
    int foundIndex; // where a substring was found;
    int startIndex; // where the next substring search starts;
    int restLength;
    String text = comparedText;
    if (parts == null) return true;
    String portion = this.caseSensitive ? parts[0] : partsLc[0];
    if (!this.caseSensitive)
    {
      text = text.toLowerCase();
    }
    restLength = text.length();
    if (!valid)
    {
      rv = false;
    }
    else if (matchAll)
    {
      rv = true;
    }
    else if (matchSingle)
    {
      rv = text.equals(portion);
    }
    else
    { // Preliminary condition
      if (startFixed)
      {
        rv = rv && text.startsWith(portion);
        if (parts.length == 1) return rv; // we're done
      }
      // partial match
      startIndex = 0;
      for (int i = 0; ((i < parts.length) && rv); i++)
      {
        if (parts[i].length() > 0)
        { // tolerate multiple "**" (not necessary)
          portion = this.caseSensitive ? parts[i] : partsLc[i];
          foundIndex = text.indexOf(portion, startIndex);
          if (foundIndex >= 0)
          {
            startIndex = foundIndex + portion.length();
            restLength = text.length() - startIndex;
            rv = (restLength >= 0);
          }
          else
          { // not found
            rv = false; // breaks loop
          }
        }
      }
      if (rv && endFixed)
      { // rest length must be 0 if end fixed
        rv = (restLength == 0);
      }
    }
    return rv;
  }
  /**
   * @return Returns true if the case sensitivity is set.
   */
  public boolean isCaseSensitive()
  {
    return caseSensitive;
  }
  /**
   * @param value
   *          Case sensitivity to set .
   */
  public void setCaseSensitive(boolean value)
  {
    this.caseSensitive = value;
  }
}
