/*
 * File TString.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package de.desy.tine.structUtils;

import de.desy.tine.stringUtils.Str;

/**
 * @author karol
 */
public class TString
{
  private byte[] buffer;
  private int length;
  private int capacity;
  private String string;
  public int getCapacity()
  {
    return capacity;
  }
  public int getLength()
  {
    return length;
  }
  public TString()
  {
    capacity = 256;
    buffer = new byte[capacity];
    set("");
  }
  public TString(int size)
  {
    capacity = size;
    buffer = new byte[capacity];
    set("");
  }
  public TString(String string)
  {
    capacity = string.length();
    buffer = new byte[capacity];
    set(string);
  }
  public TString(int size, String initialString)
  {
    capacity = size;
    buffer = new byte[capacity];
    initialString = Str.toAscii(initialString);
    set(initialString);
  }
  public boolean set(String s)
  {
    boolean truncated = false;
    s = Str.toAscii(s);
    length = s.length();
    if (length > capacity)
    {
      length = capacity;
      truncated = true;
      string = new String(s.substring(0, length));
      for (int i = 0; i < capacity; i++)
        buffer[i] = s.getBytes()[i];
    }
    else
    {
      string = new String(s);
      for (int i = 0; i < length; i++)
        buffer[i] = s.getBytes()[i];
      for (int i = length; i < capacity; i++)
        buffer[i] = '\0';
    }
    return !truncated;
  }
  public boolean set(byte[] b)
  {
    boolean truncated = false;
    length = b.length;
    if (length > capacity)
    {
      length = capacity;
      truncated = true;
      for (int i = 0; i < capacity; i++)
        buffer[i] = b[i];
      string = Str.toAscii(new String(buffer));
    }
    else
    {
      int cStrLen = 0;
      for (int i = 0; i < length; i++)
      {
        if ((buffer[i] = b[i]) != '\0') cStrLen = i + 1;
      }
      for (int i = length; i < capacity; i++)
        buffer[i] = '\0';
      String s = Str.toAscii(new String(buffer));
      string = new String(s.substring(0, cStrLen));
    }
    return !truncated;
  }
  public String get()
  {
    return string;
  }
  public String toString()
  {
    return string;
  }
  public byte[] getBytes()
  {
    return buffer;
  }
}
