/*
 * File TStructBase.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

/*
 * Created on Feb 23, 2005
 *
 */
package de.desy.tine.structUtils;

import java.io.IOException;

import de.desy.tine.definitions.TErrorList;
import de.desy.tine.server.logger.MsgLog;

/**
 * \internal
 * 
 * A base class for Classes which import or export structures. This class is
 * derived from TStructIfc and provides standard methods for reading and writing
 * from/to byte arrays. TODO Maybe drop TStructIfc and TStructIo and
 * incorparate them into this class.
 * 
 * @author MCS
 */
public abstract class TStructBase implements TStructIfc
{
  /**
   * 
   */
  public TStructBase()
  {
    super();
    // TODO Auto-generated constructor stub
  }
  public final void readBytes(byte[] b)
  {
    try
    {
      TStructIo.bytesToStruct(this, b);
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("TStructBase.readBytes",e.toString(),TErrorList.code_failure,e,0);        
    }
  }
  public final void readBytes(byte[] b, int offset, int len)
  {
    try
    {
      TStructIo.bytesToStruct(this, b, offset, len);
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("TStructBase.readBytes",e.toString(),TErrorList.code_failure,e,0);        
    }
  }
  /**
   * Returns a byte array or null on error.
   * 
   * @return TODO Should propably throw IOException?
   */
  public final byte[] toByteArray()
  {
    byte[] rv;
    try
    {
      rv = TStructIo.structToBytes(this);
    }
    catch (IOException e)
    {
      rv = null;
    }
    return rv;
  }
}
