/*
 * File TStructIo.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

/*
 * Created on Feb 23, 2005
 *
 */
package de.desy.tine.structUtils;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import de.desy.tine.io.TDataInputStream;
import de.desy.tine.io.TDataOutputStream;

/**
 * \internal
 * 
 * @author MCS
 */
public class TStructIo
{
  /**
   * Disable constructor
   */
  private TStructIo()
  {
  }
  /**
   * Lets a structure read from a byte array.
   * 
   * @param struct
   * @param b
   * @throws IOException
   */
  public static void bytesToStruct(TStructIfc struct, byte[] b) throws IOException
  {
    ByteArrayInputStream bytesIn = new ByteArrayInputStream(b);
    TDataInputStream istream = new TDataInputStream(bytesIn);
    struct.readData(istream);
    istream.close();
  }
  /**
   * Lets a structure read from a byte array.
   * 
   * @param struct
   *          An object which implements TStructIfc.
   * @param b
   *          A byte array.
   * @param offset
   *          Offset within byte array.
   * @param len
   *          Max. lenth of bytes.
   * @throws IOException
   */
  public static void bytesToStruct(TStructIfc struct, byte[] b, int offset, int len) throws IOException
  {
    ByteArrayInputStream bytesIn = new ByteArrayInputStream(b, offset, len);
    TDataInputStream istream = new TDataInputStream(bytesIn);
    struct.readData(istream);
    istream.close();
  }
  /**
   * Lets a structure write its contents to a byte array.
   * 
   * @param struct
   * @return
   * @throws IOException
   */
  public static byte[] structToBytes(TStructIfc struct) throws IOException
  {
    ByteArrayOutputStream bytesOut = new ByteArrayOutputStream();
    TDataOutputStream ostream = new TDataOutputStream(bytesOut);
    struct.writeData(ostream);
    ostream.close();
    return bytesOut.toByteArray();
  }
}
