/*
 * File DBLDBL.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

/*
 * Created on May 7, 2003
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code Template
 */
package de.desy.tine.types;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import de.desy.tine.definitions.TErrorList;
import de.desy.tine.definitions.TFormat;
import de.desy.tine.endianUtils.Swap;
import de.desy.tine.server.logger.MsgLog;
public class DBLDBL implements TCompoundDataObject
{
  public double d1val;
  public double d2val;
  public double getDoubleValue() { return (double)d1val; }
  public int getIntegerValue() { return (int)d2val; }
  public static final int sizeInBytes = 16;
  //TODO: can make this final again by letting both DBLDBL and DBLTIME extend something else
  private static short dFormat = TFormat.CF_DBLDBL;
  protected String separator = ", ";
  public void setFieldSeparator(String value)
  {
    if (value != null) separator = value;
  }
  public void setValues(double newD1Value, double newD2Value)
  {
    d1val = newD1Value; d2val = newD2Value; 
  }
  public String getCsvString()
  {
    return d1val+" "+d2val;
  }  
  public int getSizeInBytes() { return sizeInBytes; }
  public short getFormat() { return dFormat; }    
  ByteArrayOutputStream dBuffer = new ByteArrayOutputStream(sizeInBytes);
  public byte[] hByteArray;
  public void setByteArray(byte[] byteArray) { hByteArray = byteArray; }
  public byte[] getByteArray() 
  { 
    return hByteArray == null ? toByteArray() : hByteArray; 
  }
  public DBLDBL() {d1val = 0; d2val = 0; }  
  public DBLDBL(double d) { d1val = d; d2val = d; }  
  public DBLDBL(double d1, double d2) { d1val = d1; d2val = d2;}  
  public DBLDBL(DBLDBL instance) { d1val = instance.d1val; d2val = instance.d2val; }
  @Override
  public void copy(TCompoundDataObject from)
  {
    if (from instanceof DBLDBL)
    {
      DBLDBL instance = (DBLDBL)from;
      d1val = instance.d1val; d2val = instance.d2val;
    }
  }
  public String getTag() { return null; }
  public String toString()
  {
      return "["+ d1val + separator + d2val + "]";
  }
  public DBLDBL newInstance()
  {
    return new DBLDBL();
  }
  public DBLDBL clone()
  {
    return new DBLDBL(this);
  }
  public boolean equals(DBLDBL instance)
  {
    if (instance.d1val != d1val) return false;
    if (instance.d2val != d2val) return false;
    return true;
  }
  public byte[] toByteArray()
  {
    try
    {
      dBuffer.reset();
      DataOutputStream ds = new DataOutputStream(dBuffer);
      byte[] b = Swap.Double(d1val);
      ds.write(b,0,8);
      b = Swap.Double(d2val);
      ds.write(b,0,8);
      ds.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log(this.getClass().getSimpleName(),e.getMessage(),TErrorList.code_failure,e,0);        
    }
    hByteArray = dBuffer.toByteArray();
    return hByteArray;
  }    
  public void toStruct()
  {
    try
    {
      ByteArrayInputStream dis = new ByteArrayInputStream(hByteArray);
      DataInputStream ds = new DataInputStream(dis);
      byte[] b = new byte[8];
      ds.read(b,0,8);
      d1val = Swap.Double(b);
      ds.read(b,0,8);
      d2val = Swap.Double(b);
      ds.close(); dis.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log(this.getClass().getSimpleName(),e.getMessage(),TErrorList.code_failure,e,0);        
    }  
  }
  public int hashCode()
  {
    final int PRIME = 31;
    int result = 1;
    long temp;
    temp = Double.doubleToLongBits(d1val);
    result = PRIME * result + (int) (temp ^ (temp >>> 32));
    temp = Double.doubleToLongBits(d2val);
    result = PRIME * result + (int) (temp ^ (temp >>> 32));
    return result;
  }
  public boolean equals(Object obj)
  {
    if (this == obj) return true;
    if (obj == null) return false;
    if (getClass() != obj.getClass()) return false;
    final DBLDBL other = (DBLDBL) obj;
    return equals(other);
  }
}