/*
 * File DBLINT.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.types;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import de.desy.tine.definitions.TFormat;
import de.desy.tine.endianUtils.Swap;
/**
 * @deprecated
 * @author duval
 *
 */
public final class DBLINT implements TCompoundDataObject
{
  public double dval;
  public int ival;
  public double getDoubleValue() { return (double)dval; }
  public int getIntegerValue() { return (int)ival; }
  public static int sizeInBytes = 12;
  public static short dFormat = TFormat.CF_DBLINT;
  public void setValues(double newDValue, int newIValue)
  {
    dval = newDValue; ival = newIValue; 
  }
  public void setByteArray(byte[] byteArray) { hByteArray = byteArray; }
  public int getSizeInBytes() { return sizeInBytes; }
  public short getFormat() { return dFormat; }    
  public String getTag() { return null; }
  ByteArrayOutputStream dBuffer = new ByteArrayOutputStream(sizeInBytes);
  public byte[] hByteArray;
  public byte[] getByteArray() { return hByteArray; }
  /**
   * @deprecated
   */
  public DBLINT() {dval = 0; ival = 0; }  
  /**
   * @deprecated
   */
  public DBLINT(float f) { dval = f; ival = 0; }  
  /**
   * @deprecated
   */
  public DBLINT(double d) { dval = d; ival = 0; }  
  /**
   * @deprecated
   */
  public DBLINT(int i) { ival = i; }  
  /**
   * @deprecated
   */
  public DBLINT(float f,int i) { dval = f; ival = i; }  
  /**
   * @deprecated
   */
  public DBLINT(double d,int i) { dval = d; ival = i; }  
  /**
   * @deprecated
   */
  public DBLINT(DBLINT instance) { dval = instance.dval; ival = instance.ival; }
  @Override
  public void copy(TCompoundDataObject from)
  {
    if (from instanceof DBLINT)
    {
      DBLINT instance = (DBLINT)from;
      dval = instance.dval; ival = instance.ival;
    }
  }
  public DBLINT newInstance()
  {
    return new DBLINT();
  }
  public DBLINT clone()
  {
    return new DBLINT(this);
  }
  private String separator = ", ";
  public void setFieldSeparator(String value)
  {
    if (value != null) separator = value;
  }  
  @Override
  public String toString()
  {
      return "["+ dval + separator + ival + "]";
  }
  public String getCsvString()
  {
    return dval+" "+ival;
  }  
  public boolean equals(DBLINT instance)
  {
    if (instance.dval != dval) return false;
    if (instance.ival != ival) return false;
    return true;
  }
  public byte[] toByteArray()
  {
	try
	{
		dBuffer.reset();
		DataOutputStream ds = new DataOutputStream(dBuffer);
		byte[] b = Swap.Double(dval);
		ds.write(b,0,8);
		ds.writeInt(Swap.Int(ival));
		ds.close();
	}
	catch (IOException e)
	{
	  e.printStackTrace();
	}
	hByteArray = dBuffer.toByteArray();
	return hByteArray;
  }  
  public void toStruct()
  {
	try
	{
		ByteArrayInputStream dis = new ByteArrayInputStream(hByteArray);
		DataInputStream ds = new DataInputStream(dis);
		byte[] b = new byte[8];
		ds.read(b,0,8);
		dval = Swap.Double(b);
		ival = Swap.Int(ds.readInt());
		ds.close(); dis.close();
	}
	catch (IOException e)
	{
	  e.printStackTrace();
	}	 
  }
  @Override
  public int hashCode()
  {
    final int PRIME = 31;
    int result = 1;
    long temp;
    temp = Double.doubleToLongBits(dval);
    result = PRIME * result + (int) (temp ^ (temp >>> 32));
    result = PRIME * result + ival;
    return result;
  }
  @Override
  public boolean equals(Object obj)
  {
    if (this == obj) return true;
    if (obj == null) return false;
    if (getClass() != obj.getClass()) return false;
    final DBLINT other = (DBLINT) obj;
    if (Double.doubleToLongBits(dval) != Double.doubleToLongBits(other.dval)) return false;
    if (ival != other.ival) return false;
    return true;
  }  
}