/*
 * File DBLTIME.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.types;

import de.desy.tine.dataUtils.TDataTime;
import de.desy.tine.definitions.TFormat;

public final class DBLTIME extends DBLDBL
{
  public DBLTIME() { this(0,0); }  
  public DBLTIME(double d) { this(d,d); }  
  private short dFormat = TFormat.CF_DBLTIME;
  public short getFormat() { return dFormat; } 
  public DBLTIME(double d1, double d2) { d1val = d1; d2val = d2; }  
  public DBLTIME(DBLTIME instance) { d1val = instance.d1val; d2val = instance.d2val; }
  public DBLTIME(DBLDBL instance) { d1val = instance.d1val; d2val = instance.d2val; }
  public String toString()
  {
      return "["+ d1val + separator + TDataTime.toString(d2val) + "]";
  }
  public boolean equals(DBLTIME instance)
  {
    if (instance.d1val != d1val) return false;
    if (instance.d2val != d2val) return false;
    return true;
  }
  public boolean equals(Object obj)
  {
    if (this == obj) return true;
    if (obj == null) return false;
    if (getClass() != obj.getClass()) return false;
    final DBLTIME other = (DBLTIME) obj;
    return equals(other);
  }
  public DBLTIME newInstance()
  {
    return new DBLTIME();
  }
  public DBLTIME clone()
  {
    return new DBLTIME(this);
  }  
}
