/*
 * File FWINDOW.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.types;

/**
 * \internal
 * 
 * Alias for INTINTFLTFLT
 *
 * Useful for providing data access ranges (windows) in the time domain
 * (the 2 integer fields as UTC values) and data domain (the 2 float fields
 * as min and maxValue values).
 */
public final class FWINDOW extends INTINTFLTFLT
{
  public FWINDOW() { f1val = 0; f2val = 0; i1val = 0; i2val = 0; }  
  public FWINDOW(float f) { f1val = f; f2val = 0; i1val = 0; i2val = 0; }  
  public FWINDOW(int i) { f1val = 0; f2val = 0; i1val = i; i2val = 0; }  
  public FWINDOW(float f1,float f2,int i1,int i2) { f1val = f1; f2val = f2; i1val = i1; i2val = i2; }  
  public FWINDOW(FWINDOW instance) 
  { 
    f1val = instance.f1val; f2val = instance.f2val; i1val = instance.i1val; i2val = instance.i2val; 
  }  
  public boolean equals(FWINDOW instance)
  {
    if (instance.i1val != i1val) return false;
    if (instance.i2val != i2val) return false;
    if (instance.f1val != f1val) return false;
    if (instance.f2val != f2val) return false;
    return true;
  }
  public boolean equals(Object obj)
  {
    if (this == obj) return true;
    if (obj == null) return false;
    if (getClass() != obj.getClass()) return false;
    final FWINDOW other = (FWINDOW) obj;
    return equals(other);
  }
}
