/*
 * File INTINT.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

/*
 * Created on Nov 7, 2005
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package de.desy.tine.types;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import de.desy.tine.definitions.TErrorList;
import de.desy.tine.definitions.TFormat;
import de.desy.tine.endianUtils.Swap;
import de.desy.tine.server.logger.MsgLog;

public class INTINT implements TCompoundDataObject
{
  public int i1val;
  public int i2val;
  public double getDoubleValue() { return (double)i1val; }
  public int getIntegerValue() { return (int)i2val; }
  public static final int sizeInBytes = 8;
  public static final short dFormat = TFormat.CF_INTINT;
  public void setValues(int newI1Value, int newI2Value)
  {
    i1val = newI1Value; i2val = newI2Value;
  }
  public int getSizeInBytes() { return sizeInBytes; }
  public short getFormat() { return dFormat; }    
  public String getTag() { return null; }
  ByteArrayOutputStream dBuffer = new ByteArrayOutputStream(sizeInBytes);
  public byte[] hByteArray;
  public void setByteArray(byte[] byteArray) { hByteArray = byteArray; }
  public byte[] getByteArray() 
  { 
    return hByteArray == null ? toByteArray() : hByteArray; 
  }
  public INTINT() {i1val = 0; i2val = 0; }  
  public INTINT(int l) { i1val = l; i2val = 0; }  
  public INTINT(int l,int i) { i1val = l; i2val = i; }  
  public INTINT(INTINT instance) { i1val = instance.i1val; i2val = instance.i2val; }
  @Override
  public void copy(TCompoundDataObject from)
  {
    if (from instanceof INTINT)
    {
      INTINT instance = (INTINT)from;
      i1val = instance.i1val; i2val = instance.i2val;
    }
  }
  
  protected String separator = ", ";
  public void setFieldSeparator(String value)
  {
    if (value != null) separator = value;
  }
  @Override
  public String toString()
  {
      return "["+ i1val + separator + i2val+"]";
  }
  public String getCsvString()
  {
    return i1val+" "+i2val;
  }  
  public INTINT newInstance()
  {
    return new INTINT();
  }
  public INTINT clone()
  {
    return new INTINT(this);
  }    
  public boolean equals(INTINT instance)
  {
    if (instance.i1val != i1val) return false;
    if (instance.i2val != i2val) return false;
    return true;
  }
  public byte[] toByteArray()
  {
    try
    {
        dBuffer.reset();
        DataOutputStream ds = new DataOutputStream(dBuffer);
        ds.writeInt(Swap.Int(i1val));
        ds.writeInt(Swap.Int(i2val));
        ds.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("INTINT",e.getMessage(),TErrorList.code_failure,e,0);        
    }
    hByteArray = dBuffer.toByteArray();
    return hByteArray;
  }  
  public void toStruct()
  {
    try
    {
        ByteArrayInputStream dis = new ByteArrayInputStream(hByteArray);
        DataInputStream ds = new DataInputStream(dis);
        i1val = Swap.Int(ds.readInt());
        i2val = Swap.Int(ds.readInt());
        ds.close(); dis.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("INTINT",e.getMessage(),TErrorList.code_failure,e,0);        
    }    
  }
  public int hashCode()
  {
    final int PRIME = 31;
    int result = 1;
    result = PRIME * result + i1val;
    result = PRIME * result + i2val;
    return result;
  }
  public boolean equals(Object obj)
  {
    if (this == obj) return true;
    if (obj == null) return false;
    if (getClass() != obj.getClass()) return false;
    final INTINT other = (INTINT) obj;
    return equals(other);
  }
 }
