/*
 * File INTINTFLTFLT.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.types;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import de.desy.tine.definitions.TErrorList;
import de.desy.tine.definitions.TFormat;
import de.desy.tine.endianUtils.Swap;
import de.desy.tine.server.logger.MsgLog;
public class INTINTFLTFLT implements TCompoundDataObject
{
  public float f1val;
  public float f2val;
  public int i1val;
  public int i2val;
  public double getDoubleValue() { return (double)f1val; }
  public int getIntegerValue() { return (int)i1val; }
  public static int sizeInBytes = 16;
  public static short dFormat = TFormat.CF_INTINTFLTFLT;
  public void setValues(float newF1Value, float newF2Value, int newI1Value, int newI2Value)
  {
    i1val = newI1Value; f1val = newF1Value; f2val = newF2Value; i2val = newI2Value; 
  }
  public int getSizeInBytes() { return sizeInBytes; }
  public String getTag() { return null; }
  public short getFormat() { return dFormat; }    
  ByteArrayOutputStream dBuffer = new ByteArrayOutputStream(sizeInBytes);
  public byte[] hByteArray;
  public void setByteArray(byte[] byteArray) { hByteArray = byteArray; }
  public byte[] getByteArray() 
  { 
    return hByteArray == null ? toByteArray() : hByteArray; 
  }
  public INTINTFLTFLT() { f1val = 0; f2val = 0; i1val = 0; i2val = 0; }  
  public INTINTFLTFLT(float f) { f1val = f; f2val = 0; i1val = 0; i2val = 0; }  
  public INTINTFLTFLT(int i) { f1val = 0; f2val = 0; i1val = i; i2val = 0; }  
  public INTINTFLTFLT(float f1,float f2,int i1,int i2) { f1val = f1; f2val = f2; i1val = i1; i2val = i2; }  
  public INTINTFLTFLT(INTINTFLTFLT instance) { f1val = instance.f1val; f2val = instance.f2val; i1val = instance.i1val; i2val = instance.i2val; }
  @Override
  public void copy(TCompoundDataObject from)
  {
    if (from instanceof INTINTFLTFLT)
    {
      INTINTFLTFLT instance = (INTINTFLTFLT)from;
      f1val = instance.f1val; f2val = instance.f2val; i1val = instance.i1val; i2val = instance.i2val;
    }
  }
  protected String separator = ", ";
  public void setFieldSeparator(String value)
  {
    if (value != null) separator = value;
  }
  @Override
  public String toString()
  {
      return "["+ i1val + separator + i2val + separator +f1val + separator + f2val+"]";
  }
  public String getCsvString()
  {
    return i1val+" "+i2val+" "+f1val+" "+f2val;
  }
  public INTINTFLTFLT(int ival1, int ival2, float fval1, float fval2)
  {
    i1val = ival1;
    i2val = ival2;
    f1val = fval1;
    f2val = fval2;
  }
  public INTINTFLTFLT newInstance()
  {
    return new INTINTFLTFLT();
  }
  public INTINTFLTFLT clone()
  {
    return new INTINTFLTFLT(this);
  }    
  public boolean equals(INTINTFLTFLT instance)
  {
    if (instance.i1val != i1val) return false;
    if (instance.i2val != i2val) return false;
    if (instance.f1val != f1val) return false;
    if (instance.f2val != f2val) return false;
    return true;
  }
  public byte[] toByteArray()
  {
    try
    {
      dBuffer.reset();
      DataOutputStream ds = new DataOutputStream(dBuffer);
      ds.writeInt(Swap.Int(i1val));
      ds.writeInt(Swap.Int(i2val));
      byte[] b = Swap.Float(f1val);
      ds.write(b,0,4);
      b = Swap.Float(f2val);
      ds.write(b,0,4);
      ds.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("INTINTFLTFLT",e.getMessage(),TErrorList.code_failure,e,0);        
    }
    hByteArray = dBuffer.toByteArray();
    return hByteArray;
  }    
  public void toStruct()
  {
    try
    {
      ByteArrayInputStream dis = new ByteArrayInputStream(hByteArray);
      DataInputStream ds = new DataInputStream(dis);
      i1val = Swap.Int(ds.readInt());
      i2val = Swap.Int(ds.readInt());
      byte[] b = new byte[4];
      ds.read(b,0,4);
      f1val = Swap.Float(b);
      ds.read(b,0,4);
      f2val = Swap.Float(b);
      ds.close(); dis.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("INTINTFLTFLT",e.getMessage(),TErrorList.code_failure,e,0);        
    }  
  }
  @Override
  public int hashCode()
  {
    final int PRIME = 31;
    int result = 1;
    result = PRIME * result + Float.floatToIntBits(f1val);
    result = PRIME * result + Float.floatToIntBits(f2val);
    result = PRIME * result + i1val;
    result = PRIME * result + i2val;
    return result;
  }
  @Override
  public boolean equals(Object obj)
  {
    if (this == obj) return true;
    if (obj == null) return false;
    if (getClass() != obj.getClass()) return false;
    final INTINTFLTFLT other = (INTINTFLTFLT) obj;
    return equals(other);
  }    
}