/*
 * File KEYVALUE.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.types;

public class KEYVALUE 
{
  private String keyvalue;
  private String separatorChar = "=";
  public KEYVALUE(String keyValue)
  {
    keyvalue = keyValue;
  }
  public KEYVALUE(String key,String value)
  {
    if (key == null || key.length() == 0) key = "unknown";
    if (value == null || value.length() == 0) value = "unknown";
    keyvalue = key+separatorChar+value;
  }
  public String getString()
  {
    return keyvalue == null ? "" : keyvalue;
  }
  public void putString(String newKeyValue)
  {
    if (newKeyValue==null) return;
    if (newKeyValue.indexOf('=') >= 0) separatorChar = "=";
    else if (newKeyValue.indexOf(':') >= 0) separatorChar = ":";
    keyvalue = newKeyValue;
  }
  public String getKey()
  {
    if (keyvalue == null) return null;
    int separator = keyvalue.indexOf(separatorChar);
    if (separator < 0) return keyvalue;
    if (separator == 0) return "";
    return keyvalue.substring(0, separator).trim();
  }
  public void setKey(String newKey)
  {
    if (newKey == null || newKey.length() == 0) newKey = "unknown";
    keyvalue = newKey+separatorChar+getValue();
  }
  public String getValue()
  {
    if (keyvalue == null) return null;
    int separator = keyvalue.indexOf(separatorChar);
    if (separator < 0) return "";
    return keyvalue.substring(separator+1).trim();    
  }
  public void setValue(String newValue)
  {
    if (newValue == null || newValue.length() == 0) newValue = "unknown";
    keyvalue = getKey()+separatorChar+newValue;
  }
}
