/*
 * File LNGINT.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

/*
 * Created on Nov 7, 2005
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package de.desy.tine.types;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import de.desy.tine.definitions.TErrorList;
import de.desy.tine.definitions.TFormat;
import de.desy.tine.endianUtils.Swap;
import de.desy.tine.server.logger.MsgLog;

/**
 * @deprecated
 * please use INTINT instead of this class!
 * @author duval
 *
 */
public class LNGINT implements TCompoundDataObject
{
  public int lval;
  public int ival;
  public double getDoubleValue() { return (double)lval; }
  public int getIntegerValue() { return (int)ival; }
  public static int sizeInBytes = 8;
  public static short dFormat = TFormat.CF_INTINT;
  public void setValues(int newI1Value, int newI2Value)
  {
    lval = newI1Value; ival = newI2Value;
  }
  public int getSizeInBytes() { return sizeInBytes; }
  public short getFormat() { return dFormat; }    
  public String getTag() { return null; }
  ByteArrayOutputStream dBuffer = new ByteArrayOutputStream(sizeInBytes);
  public byte[] hByteArray;
  public void setByteArray(byte[] byteArray) { hByteArray = byteArray; }
  public byte[] getByteArray() { return hByteArray; }
  public LNGINT() {lval = 0; ival = 0; }  
  public LNGINT(int l) { lval = l; ival = 0; }  
  public LNGINT(int l,int i) { lval = l; ival = i; }  
  public LNGINT(LNGINT instance) { ival = instance.ival; lval = instance.lval; }
  @Override
  public void copy(TCompoundDataObject from)
  {
    if (from instanceof LNGINT)
    {
      LNGINT instance = (LNGINT)from;
      ival = instance.ival; lval = instance.lval;
    }
  }
  
  protected String separator = ", ";
  public void setFieldSeparator(String value)
  {
    if (value != null) separator = value;
  }
  @Override
  public String toString()
  {
      return "["+ lval + separator + ival + "]";
  }
  public String getCsvString()
  {
    return lval+" "+ival;
  }  
  public LNGINT newInstance()
  {
    return new LNGINT();
  }
  public LNGINT clone()
  {
    return new LNGINT(this);
  }        
  public boolean equals(LNGINT instance)
  {
    if (instance.lval != lval) return false;
    if (instance.ival != ival) return false;
    return true;
  }
  public byte[] toByteArray()
  {
    try
    {
        dBuffer.reset();
        DataOutputStream ds = new DataOutputStream(dBuffer);
        ds.writeInt(Swap.Int(lval));
        ds.writeInt(Swap.Int(ival));
        ds.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("LNGINT",e.getMessage(),TErrorList.code_failure,e,0);        
    }
    hByteArray = dBuffer.toByteArray();
    return hByteArray;
  }  
  public void toStruct()
  {
    try
    {
        ByteArrayInputStream dis = new ByteArrayInputStream(hByteArray);
        DataInputStream ds = new DataInputStream(dis);
        lval = Swap.Int(ds.readInt());
        ival = Swap.Int(ds.readInt());
        ds.close(); dis.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("LNGINT",e.getMessage(),TErrorList.code_failure,e,0);        
    }    
  }
  @Override
  public int hashCode()
  {
    final int PRIME = 31;
    int result = 1;
    result = PRIME * result + ival;
    result = PRIME * result + lval;
    return result;
  }
  @Override
  public boolean equals(Object obj)
  {
    if (this == obj) return true;
    if (obj == null) return false;
    if (getClass() != obj.getClass()) return false;
    final LNGINT other = (LNGINT) obj;
    return equals(other);
  }
 }
