/*
 * File NAME16II.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

/*
 * Created on Nov 7, 2005
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package de.desy.tine.types;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import de.desy.tine.definitions.TErrorList;
import de.desy.tine.definitions.TFormat;
import de.desy.tine.endianUtils.Swap;
import de.desy.tine.server.logger.MsgLog;

public class NAME16II implements TCompoundDataObject
{
  public String name;
  public int i1val;
  public int i2val;
  public double getDoubleValue() { return (double)i1val; }
  public int getIntegerValue() { return (int)i2val; }
  public static final int sizeInBytes = 24;
  public static final short dFormat = TFormat.CF_NAME16II;
  public void setValues(String newName, int newI1Value, int newI2Value)
  {
    name = newName; i1val = newI1Value; i2val = newI2Value;
  }
  public void setName(String newName) { name = newName; }
  public String getName() { return name; }
  public int getSizeInBytes() { return sizeInBytes; }
  public short getFormat() { return dFormat; }    
  public String getTag() { return null; }
  ByteArrayOutputStream dBuffer = new ByteArrayOutputStream(sizeInBytes);
  public byte[] hByteArray;
  public void setByteArray(byte[] byteArray) { hByteArray = byteArray; }
  public byte[] getByteArray() 
  { 
    return hByteArray == null ? toByteArray() : hByteArray; 
  }
  public NAME16II() {name = new String(new StringBuffer(16)); i2val = 0; i1val =0;}  
  public NAME16II(String Name) { name = Name; i2val = 0; i1val = 0;}  
  public NAME16II(String Name,int i) { name = Name; i1val = i; i2val = 0;}  
  public NAME16II(String Name,int i1,int i2) { name = Name; i2val = i2; i1val = i1;}  
  public NAME16II(NAME16II instance) 
  { 
    name = instance.name == null ? new String(new StringBuffer(16)) : new String(instance.name); 
    i1val = instance.i1val; i2val = instance.i2val; 
  }
  public void copy(TCompoundDataObject from)
  {
    if (from instanceof NAME16II)
    {
      NAME16II instance = (NAME16II)from;
      name = instance.name;
      i1val = instance.i1val; i2val = instance.i2val; 
    }
  }
  protected String separator = ", ";
  public void setFieldSeparator(String value)
  {
    if (value != null) separator = value;
  }
  public String toString()
  {
      return "["+name+separator+i1val+separator+i2val+"]";
  }
  public String getCsvString()
  {
    String tname = name.trim();
    if (tname.contains(",") || tname.contains(" ") ||
        tname.contains("\t") || tname.contains("\n"))
    {
      return "\""+tname+" "+i1val+" "+i2val+"\"";
    }
    return tname + " "+i1val+" "+i2val;
  }
  public NAME16II newInstance()
  {
    return new NAME16II();
  }
  public NAME16II clone()
  {
    return new NAME16II(this);
  }            
  public boolean equals(NAME16II instance)
  {
    if (instance.name == null || name == null) return false;
    if (instance.name.compareTo(name) != 0) return false;
    if (instance.i1val != i1val) return false;
    if (instance.i2val != i2val) return false;
    return true;
  }
  public byte[] toByteArray()
  {
    if (name == null) name = new String(new StringBuffer(16));
    try
    {
      dBuffer.reset();
      DataOutputStream ds = new DataOutputStream(dBuffer);
      byte[] bname = name.getBytes();
      byte[] b = new byte[16];
      for (int i=0; i<16 && i<bname.length; i++) b[i] = bname[i];
      ds.write(b,0,b.length);
      ds.writeInt(Swap.Int(i1val));
      ds.writeInt(Swap.Int(i2val));
      ds.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("NAME16II","could not prepare byte stream",TErrorList.code_failure,e,0);
      return null;
    }
    hByteArray = dBuffer.toByteArray();
    return hByteArray;
  }  
  public void toStruct()
  {
    try
    {
      ByteArrayInputStream dis = new ByteArrayInputStream(hByteArray);
      DataInputStream ds = new DataInputStream(dis);
      byte[] b = new byte[sizeInBytes];
      ds.read(b,0,16);
      int len = 0; for (int i=0; i<16 && b[i] != 0; i++, len++);
      name = new String(b,0,len);
      i1val = Swap.Int(ds.readInt());
      i2val = Swap.Int(ds.readInt());
      ds.close(); dis.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("NAME16II","could not prepare byte stream",TErrorList.code_failure,e,0);
    }  
  }
  @Override
  public int hashCode()
  {
    final int PRIME = 31;
    int result = 1;
    result = PRIME * result + i1val;
    result = PRIME * result + i2val;
    result = PRIME * result + ((name == null) ? 0 : name.hashCode());
    return result;
  }
  @Override
  public boolean equals(Object obj)
  {
    if (this == obj) return true;
    if (obj == null) return false;
    if (getClass() != obj.getClass()) return false;
    final NAME16II other = (NAME16II) obj;
    return equals(other);
  }  
}
