/*
 * File NAMEINT.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

/*
 * Created on Aug 12, 2005
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package de.desy.tine.types;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import de.desy.tine.definitions.TErrorList;
import de.desy.tine.definitions.TFormat;
import de.desy.tine.endianUtils.Swap;
import de.desy.tine.server.logger.MsgLog;
public class NAMEINT
{
  public String name;
  public int ival;
  private int nameSizeInBytes = 16;
  private short dFormat = TFormat.CF_NAME16I;
  ByteArrayOutputStream dBuffer = new ByteArrayOutputStream(nameSizeInBytes+4);
  public byte[] hByteArray;
  public int getSizeInBytes() { return nameSizeInBytes + 4; }
  public short getFormat() { return dFormat; }    
  public String getTag() { return null; }
  public byte[] getByteArray() 
  { 
    return hByteArray == null ? toByteArray() : hByteArray; 
  }
  public void setByteArray(byte[] byteArray) { hByteArray = byteArray; }
  public void setValues(String newName, int newIValue)
  {
    name = newName; ival = newIValue;;
  }
  public void setName(String newName) { name = newName; }
  public String getName() { return name; }
  public NAMEINT() {name = new String(new StringBuffer(nameSizeInBytes)); ival = 0; }  
  public NAMEINT(String Name) { name = Name; ival = 0; }  
  public NAMEINT(String Name,int i) { name = Name; ival = i; }  
  public NAMEINT(int namelength) 
  {
    switch (namelength)
    {
      case 8: dFormat = TFormat.CF_NAME8I; break;
      case 16: dFormat = TFormat.CF_NAME16I; break;
      case 32: dFormat = TFormat.CF_NAME32I; break;
      case 48: dFormat = TFormat.CF_NAME48I; break;
      case 64: dFormat = TFormat.CF_NAME64I; break;
      default: return; // TODO: throw exception
    }
    nameSizeInBytes = namelength;
    name = new String(new StringBuffer(nameSizeInBytes)); 
    ival = 0; 
  }
  public NAMEINT(NAMEINT instance) 
  { 
    name = instance.name;
    ival = instance.ival;
    nameSizeInBytes = instance.nameSizeInBytes;
    if (name == null) name = new String(new StringBuffer(nameSizeInBytes));
    else name = new String(name);
    dBuffer = new ByteArrayOutputStream(nameSizeInBytes+4);
  }
  protected String separator = ", ";
  public void setFieldSeparator(String value)
  {
    if (value != null) separator = value;
  }
  public String toString()
  {
      return "["+ name + separator + ival + "]";
  }
  public String getCsvString()
  {
    String tname = name.trim();
    if (tname.contains(",") || tname.contains(" ") ||
        tname.contains("\t") || tname.contains("\n"))
    {
      return "\""+tname+" "+ival+"\"";
    }
    return tname + " " + ival;
  }
  public boolean equals(NAMEINT instance)
  {
    if (instance.nameSizeInBytes != nameSizeInBytes) return false;
    if (instance.name == null || name == null) return false;
    if (instance.name.compareTo(name)!= 0) return false;
    if (instance.ival != ival) return false;
    return true;
  }
  public int getNameSizeInBytes() { return nameSizeInBytes; }
  public byte[] toByteArray()
  {
    try
    {
        dBuffer.reset();
        DataOutputStream ds = new DataOutputStream(dBuffer);
        byte[] bname = name != null ? name.getBytes() : null;
        byte[] b = new byte[nameSizeInBytes];
        int nlen = bname != null ? bname.length : 0;
        for (int i=0; i<nameSizeInBytes && i<nlen; i++) b[i] = bname[i];
        ds.write(b,0,b.length);
        ds.writeInt(Swap.Int(ival));
        ds.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("NAMEINT","could not prepare byte stream",TErrorList.code_failure,e,0);
      return null;
    }
    hByteArray = dBuffer.toByteArray();
    return hByteArray;
  }  
  public void toStruct()
  {
    try
    {
        ByteArrayInputStream dis = new ByteArrayInputStream(hByteArray);
        DataInputStream ds = new DataInputStream(dis);
        byte[] b = new byte[nameSizeInBytes+4];
        ds.read(b,0,nameSizeInBytes);
        int len = 0; for (int i=0; i<nameSizeInBytes && b[i] != 0; i++, len++);
        name = new String(b,0,len);
        ival = Swap.Int(ds.readInt());
        ds.close(); dis.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("NAMEINT","could not prepare byte stream",TErrorList.code_failure,e,0);
    }    
  }
  public int hashCode()
  {
    final int PRIME = 31;
    int result = 1;
    result = PRIME * result + ival;
    result = PRIME * result + ((name == null) ? 0 : name.hashCode());
    result = PRIME * result + nameSizeInBytes;
    return result;
  }
  public boolean equals(Object obj)
  {
    if (this == obj) return true;
    if (obj == null) return false;
    if (getClass() != obj.getClass()) return false;
    final NAMEINT other = (NAMEINT) obj;
    return equals(other);
  }  
  public double getDoubleValue() { return (double)ival; }
  public int getIntegerValue() { return ival; }
}