/*
 * File TDS.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.types;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import de.desy.tine.definitions.TErrorList;
import de.desy.tine.definitions.TFormat;
import de.desy.tine.endianUtils.Swap;
import de.desy.tine.server.logger.MsgLog;
/**
 * @deprecated
 * please use INTFLTINT instead
 * @author duval
 *
 */
public final class TDS implements TCompoundDataObject
{
  public int tval;
  public float dval;
  public int sval;
  public double getDoubleValue() { return (double)dval; }
  public int getIntegerValue() { return (int)sval; }
  public static final int sizeInBytes = 12;
  public static final short dFormat = TFormat.CF_TDS;
  protected String separator = ", ";
  public void setFieldSeparator(String value)
  {
    if (value != null) separator = value;
  }
  public String toString()
  {
    return "["+ tval + separator + dval + separator + sval + "]";
  }
  public String getCsvString()
  {
    return tval+" "+dval+" "+sval;
  }  
  public void setValues(int newTValue, float newDValue, int newSValue)
  {
    tval = newTValue; dval = newDValue; sval = newSValue; 
  }
  public int getSizeInBytes() { return sizeInBytes; }
  public short getFormat() { return dFormat; }    
  public String getTag() { return null; }
  ByteArrayOutputStream dBuffer = new ByteArrayOutputStream(sizeInBytes);
  public byte[] hByteArray;
  public void setByteArray(byte[] byteArray) { hByteArray = byteArray; }
  public byte[] getByteArray() 
  { 
    return hByteArray == null ? toByteArray() : hByteArray; 
  }
  public TDS() { tval = 0; dval = 0; sval = 0; }  
  public TDS(float d) { tval = 0; dval = d; sval = 0; }  
  public TDS(int t) { tval = t; dval = 0; sval = 0; }  
  public TDS(float d,int t) { dval = d; tval = t; sval = 0; }  
  public TDS(float d,int t,int s) { dval = d; tval = t; sval = s; } 
  public TDS(TDS instance)
  {
    dval = instance.dval; tval = instance.tval; sval = instance.sval;
  }
  @Override
  public void copy(TCompoundDataObject from)
  {
    if (from instanceof TDS)
    {
      TDS instance = (TDS)from;
      dval = instance.dval; tval = instance.tval; sval = instance.sval;
    }
  }
  public TDS newInstance()
  {
    return new TDS();
  }
  public TDS clone()
  {
    return new TDS(this);
  }              
  public boolean equals(TDS instance)
  {
    if (instance.dval != dval) return false;
    if (instance.tval != tval) return false;
    if (instance.sval != sval) return false;
    return true;
  }
  public byte[] toByteArray()
  {
    try
    {
    	dBuffer.reset();
    	DataOutputStream ds = new DataOutputStream(dBuffer);
    	ds.writeInt(Swap.Int(tval));
    	byte[] b = Swap.Float(dval);
    	ds.write(b,0,4);
    	ds.writeInt(Swap.Int(sval));
    	ds.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("TDS.toByteArray",e.getMessage(),TErrorList.code_failure,e,0);        
    }
    hByteArray = dBuffer.toByteArray();
    return hByteArray;
  }    
  public void toStruct()
  {
    try
    {
    	ByteArrayInputStream dis = new ByteArrayInputStream(hByteArray);
    	DataInputStream ds = new DataInputStream(dis);
    	tval = Swap.Int(ds.readInt());
    	byte[] b = new byte[4];
    	ds.read(b,0,4);
    	dval = Swap.Float(b);
    	sval = Swap.Int(ds.readInt());
    	ds.close(); dis.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("UNAME.toStruct",e.getMessage(),TErrorList.code_failure,e,0);        
    }	 
  }
  public int hashCode()
  {
    final int PRIME = 31;
    int result = 1;
    result = PRIME * result + Float.floatToIntBits(dval);
    result = PRIME * result + sval;
    result = PRIME * result + tval;
    return result;
  }
  public boolean equals(Object obj)
  {
    if (this == obj) return true;
    if (obj == null) return false;
    if (getClass() != obj.getClass()) return false;
    final TDS other = (TDS) obj;
    return equals(other);
  }    
}