/*
 * File WINDOW.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

package de.desy.tine.types;

import de.desy.tine.definitions.TFormat;

/**
 * \internal
 * 
 * Alias for INTINTINTINT
 *
 * Useful for passing hardware address information (e.g. line, crate, module, subaddress)
 */
public final class WINDOW extends INTINTINTINT
{
  public WINDOW() { i1val = 0; i2val = 0; i3val = 0; i4val = 0; dFormat = TFormat.CF_WINDOW;}  
  public WINDOW(int i) { i1val = i; i2val = 0; i3val = 0; i4val = 0; dFormat = TFormat.CF_WINDOW;}  
  public WINDOW(int i1,int i2,int i3, int i4) { i1val = i1; i2val = i2; i3val = i3; i4val = i4; dFormat = TFormat.CF_WINDOW;}
  public int getTime1() { return i1val; }
  public int getTime2() { return i2val; }
  public WINDOW(WINDOW instance)
  {
    i1val = instance.i1val; i2val = instance.i2val; 
    i3val = instance.i3val; i4val = instance.i4val;
  }
  public boolean equals(WINDOW instance)
  {
    if (i1val != instance.i1val) return false; 
    if (i2val != instance.i2val) return false; 
    if (i3val != instance.i3val) return false; 
    if (i4val != instance.i4val) return false;
    return true;
  }
  public boolean equals(Object obj)
  {
    if (this == obj) return true;
    if (obj == null) return false;
    if (getClass() != obj.getClass()) return false;
    final WINDOW other = (WINDOW) obj;
    return equals(other);
  } 
}
