/*
 * File AliasCfg.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package de.desy.tine.xmlUtils;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import de.desy.tine.definitions.TErrorList;
import de.desy.tine.server.logger.MsgLog;

/**
 *
 * @author karol
 */
public class AliasCfg extends TineConfigItem
{
    private String name = new String("");
    private String target = new String("");

    public String getName()
    {
        return name;
    }

    public String getTarget()
    {
        return target;
    }

    public void setName(String name)
    {
        this.name = name;
    }

    public void setTarget(String target)
    {
        this.target = target;
    }
       
    public AliasCfg(String configItemName)
    {
        super(configItemName);
    }

    public AliasCfg(String configItemName, String name, String target)
    {
        super(configItemName);
        this.name = name;
        this.target = target;
    }

    @Override
    public String toString()    
    {
        return "[name = " + getName() + ", target = " + getTarget() +"]";
    }
    
    void parse(Element root)
    {
        NodeList children = root.getChildNodes();
        for (int i = 0; i < children.getLength(); i++)
        {
            Node child = children.item(i);
            if (child instanceof Element)
            {
                Element childElement = (Element) child;
                Text textNode = (Text) childElement.getFirstChild();
                if( textNode == null ) continue;
                String text = textNode.getData().trim();
                if(childElement.getTagName().equalsIgnoreCase("NAME"))
                {
                    setName(text);
                }
                else if(childElement.getTagName().equalsIgnoreCase("TARGET"))
                {
                    setTarget(text);
                }
                else MsgLog.log("AliasCfg","-WARNING: unsupported tag: " + childElement.getTagName(),TErrorList.invalid_field,null,0);
            }
        }
    }

    @Override
    void dump(int margin)
    {
        String m = new String("");
        for( int i = 0; i < margin; i++ ) m += " ";
       
        System.out.println( m + "Name = " + getName() );
        System.out.println( m + "Target = " + getTarget() );
    }
        
}
