/*
 * File DOMTreeModel.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package de.desy.tine.xmlUtils;

import javax.swing.event.TreeModelListener;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 *
 * @author karol
 */
class DOMTreeModel implements TreeModel
{

    private Document doc;
    
    public DOMTreeModel(Document doc)
    {
        this.doc = doc;
    }

    public Object getRoot()
    {
        return doc.getDocumentElement();
    }

    public Object getChild(Object parent, int index)
    {
       Node node = (Node)parent;
       NodeList list = node.getChildNodes();
       return list.item(index);
    }

    public int getChildCount(Object parent)
    {
       Node node = (Node)parent;
       NodeList list = node.getChildNodes();
       return list.getLength();
    }

    public boolean isLeaf(Object node)
    {
        return getChildCount(node) == 0;
    }

    public void valueForPathChanged(TreePath path, Object newValue)
    {        
    }

    public int getIndexOfChild(Object parent, Object child)
    {
        Node node = (Node)parent;
        NodeList list = node.getChildNodes();
        for( int i = 0; i < list.getLength(); i++ )
        {
            if( getChild(node,i) == child ) return i;
        }
        return -1;
    }

    public void addTreeModelListener(TreeModelListener l)
    {
    }

    public void removeTreeModelListener(TreeModelListener l)
    {
    }

}
