/*
 * File FecCfg.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package de.desy.tine.xmlUtils;

import java.util.LinkedList;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;

/**
 *
 * @author karol
 */
public class FecCfg extends TineConfigItem
{
    private String name = new String("");
    private String historyHome = new String("");
    private String context = new String("");
    private String subsystem = new String("");
    private String description = new String("");
    private String hardware = new String(""); 
    private String location = new String("");
    private String responsible = new String("");
    private ValidatedInt portOffset = new ValidatedInt();
    private LinkedList<AliasCfg> aliasList = new LinkedList<AliasCfg>();
    private LinkedList<EqmCfg> eqms =  new LinkedList<EqmCfg>();

    public LinkedList<AliasCfg> getAliasList()
    {
        return aliasList;
    }

    public FecCfg(String name)
    {
        super( name );
    }

    public void setDescription(String description)
    {
        this.description = description;
    }
    
    public String getDescription()
    {
        return description;
    }

    public void setLocation(String location)
    {
        this.location = location;
    }

    public String getLocation()
    {
        return location;
    }

    public void setHardware(String hardware)
    {
        this.hardware = hardware;
    }

    public String getHardware()
    {
        return hardware;
    }
    
    public LinkedList<EqmCfg> getEqmList()
    {
        return eqms;
    }

    public String getHistoryHome()
    {
        return historyHome;
    }

    public String getName()
    {
        return name;
    }

    public ValidatedInt getPortOffset()
    {
        return portOffset;
    }
    
    public String getContext()
    {
        return context;
    }

    public void setContext(String context)
    {
        this.context = context;
    }
    
    public String getSubsystem()
    {
      return subsystem;
    }

    public void setSubsystem(String subsystem)
    {
      this.subsystem = subsystem;
    }

    public String getResponsible()
    {
      return responsible;
    }

    public void setResponsible(String responsible)
    {
      this.responsible = responsible;
    }

    public void setAliasList(LinkedList<AliasCfg> aliasList)
    {
        this.aliasList = aliasList;
    }

    public void setEqmList(LinkedList<EqmCfg> eqms)
    {
        this.eqms = eqms;
    }

    public void setHistoryHome(String historyHome)
    {
        this.historyHome = historyHome;
    }

    public void setName(String name)
    {
        this.name = name;
    }

    public void setPortOffset(ValidatedInt portOffset)
    {
        this.portOffset = portOffset;
    }

    void dump(int margin)
    {
       String m = new String("");
       for( int i = 0; i < margin; i++ ) m += " ";
       
       System.out.println( m + "Name = " + getName() );
       System.out.println( m + "Alias Table :" );
       for( AliasCfg a : aliasList )
       {
           a.dump( margin + 3);
           System.out.println();
       }
       System.out.println( m + "Location = " + getLocation() );
       System.out.println( m + "Description = " + getDescription() );
       System.out.println( m + "Context = " + getContext() );
       System.out.println( m + "Subsystem = " + getSubsystem() );
       System.out.println( m + "Hardware = " + getHardware());       
       System.out.println( m + "Port Offset = " + getPortOffset().toString());
       System.out.println( m + "History home = " + getHistoryHome() );
           
       int idx = 0;
       for( EqmCfg e : eqms )
       { 
           System.out.println( m + "EQM[" + idx++ +"]" );  
           e.dump( margin + 3);
       }       
    }

    void parse(Element root)
    {
        NodeList children = root.getChildNodes();
        for (int i = 0; i < children.getLength(); i++)
        {
            Node child = children.item(i);
            if (child instanceof Element)
            {
                Element childElement = (Element) child;
                Text textNode = (Text) childElement.getFirstChild();
                if( textNode == null ) continue;
                String text = textNode.getData().trim();
                if (childElement.getTagName().equalsIgnoreCase("NAME"))
                {
                    setName(text);
                }
                else if( childElement.getTagName().equalsIgnoreCase("PORT_OFFSET"))
                {
                    setPortOffset( ValidatedInt.parse(text));
                }
                else if(childElement.getTagName().equalsIgnoreCase("HISTORY_HOME"))
                {
                    setHistoryHome(text);
                }
                else if(childElement.getTagName().equalsIgnoreCase("LOCATION"))
                {
                    setLocation(text);
                }
                else if(childElement.getTagName().equalsIgnoreCase("DESCRIPTION"))
                {
                    setDescription(text);
                }
                else if(childElement.getTagName().equalsIgnoreCase("HARDWARE"))
                {
                    setHardware(text);
                }
                else if(childElement.getTagName().equalsIgnoreCase("ALIAS"))
                {
                    AliasCfg ac = new AliasCfg("ALIAS");
                    aliasList.add(ac);
                    ac.parse( childElement );
                }
                else if(childElement.getTagName().equalsIgnoreCase("CONTEXT"))
                {
                    setContext(text);
                }
                else if(childElement.getTagName().equalsIgnoreCase("SUBSYSTEM"))
                {
                    setSubsystem(text);
                }
                else if(childElement.getTagName().equalsIgnoreCase("EQM"))
                {
                    EqmCfg ec = new EqmCfg("EQM");
                    eqms.add(ec);
                    ec.parse( childElement );
                }                
            }
        }
    }  
}
