/*
 * File TineCfgTreeModel.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package de.desy.tine.xmlUtils;

import java.lang.String;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.LinkedList;
import javax.swing.event.EventListenerList;
import javax.swing.event.TreeModelListener;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;
import de.desy.tine.definitions.TErrorList;
import de.desy.tine.server.logger.MsgLog;

/**
 *
 * @author karol
 */
class TineCfgTreeModel implements TreeModel
{
    @SuppressWarnings("unused")
    private FecCfg fecCfg;
    private EventListenerList eventListenerList = new EventListenerList();
    TineConfigTreeItem root;
            
            
    public TineCfgTreeModel(FecCfg fecCfg) 
    {
        this.fecCfg = fecCfg;
        root = new TineConfigTreeItem( getClass(), "fecCfg", fecCfg );
    }

    public Object getRoot()
    {
        return  root;
    }

    public Object getChild(Object parent, int index)
    {
      
       if( ((TineConfigTreeItem)parent).getValue().getClass().equals(LinkedList.class))
       {
            return new TineConfigTreeItem( String.class, "member", ((LinkedList<?>)((TineConfigTreeItem)parent).getValue()).get(index));
       }
       try
       {
            ArrayList<Field> fields = ((TineConfigTreeItem)parent).getFields();
            Field f = (Field)fields.get(index);
            Object parentValue = ((TineConfigTreeItem)parent).getValue();
            return new TineConfigTreeItem(f.getType(), f.getName(), f.get(parentValue));
       }
       catch(IllegalAccessException e)
       {
           MsgLog.log("TineCfgTreeModel","Illegal Access Exception handler",TErrorList.code_failure,e,0);
           return null;
       }
    }

    public int getChildCount(Object parent)
    {
      if( ((TineConfigTreeItem)parent).getValue().getClass().equals(LinkedList.class))  
          return ((LinkedList<?>)((TineConfigTreeItem)parent).getValue()).size();
      return ((TineConfigTreeItem)parent).getFields().size();
    }

    public boolean isLeaf(Object node)
    {
        return getChildCount(node) == 0;
    }

    public void valueForPathChanged(TreePath path, Object newValue)
    {        
    }

    public int getIndexOfChild(Object parent, Object child)
    {  
       int n = getChildCount(parent);
       for( int i = 0; i < n; i++ )
       {
           if( getChild(parent, i).equals(child)) return i;
       }
       return -1;
    }

    public void addTreeModelListener(TreeModelListener l)
    {
        eventListenerList.add(TreeModelListener.class, l);
    }

    public void removeTreeModelListener(TreeModelListener l)
    {
        eventListenerList.remove(TreeModelListener.class, l);
    }

}
