/*
 * File TineConfigItem.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package de.desy.tine.xmlUtils;

import java.lang.reflect.AccessibleObject;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import org.w3c.dom.Element;
import de.desy.tine.definitions.TErrorList;
import de.desy.tine.server.logger.MsgLog;

/**
 * @author karol
 */
abstract public class TineConfigItem
{
  abstract void parse(Element root);
  abstract void dump(int margin);
  String configItemName = new String("no name");
  public TineConfigItem(String name)
  {
    setConfigItemName(name);
  }
  public String getConfigItemName()
  {
    return configItemName;
  }
  public void setConfigItemName(String configItemName)
  {
    this.configItemName = configItemName;
  }
  private Object getFieldValue(Field f)
  {
    try
    {
      if (f.isAccessible())
      {
        return f.get(this);
      }
      String name = f.getName();
      if (name.length() > 1)
      {
        name = "get" + name.substring(0, 1).toUpperCase() + name.substring(1);
      }
      else
      {
        name = "get" + name.toUpperCase();
      }
      Method[] m = getClass().getMethods();
      for (int i = 0; i < m.length; i++)
      {
        if (m[i].getName().equals(name))
        {
          Class[] parTypes = m[i].getParameterTypes();
          if (parTypes.length == 0)
          {
            Object result;
            try
            {
              Object[] args = null;
              result = m[i].invoke(this, args);
            }
            catch (Exception e)
            {
              e.printStackTrace();
              MsgLog.log("TineConfigItem", "method execution:", TErrorList.code_failure, e, 0);
              return null;
            }
            return result;
          }
        }
      }
    }
    catch (Exception ex)
    {
      ex.printStackTrace();
      MsgLog.log("TineConfigItem", "method execution:", TErrorList.code_failure, ex, 0);
    }
    System.out.println("--WARNING: NO GETTER FOR: " + f.getName());
    return null;
  }
  @Override
  public String toString()
  {
    return getClass().getSimpleName();
  }
}
