/*
 * File ValidatedFormat.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package de.desy.tine.xmlUtils;

import de.desy.tine.definitions.TArrayType;
import de.desy.tine.definitions.TFormat;

/**
 * @author karol
 */
public class ValidatedFormat
{
  private boolean valid;
  private int format;
  private String tag;
  private int arrayType;
  public boolean isValid()
  {
    return valid;
  }
  public ValidatedInt getFormat()
  {
    return new ValidatedInt(format,valid);
  }
  public void setValid(boolean valid)
  {
    this.valid = valid;
  }
  public void setFormat(int value)
  {
    this.format = value;
  }
  public ValidatedFormat()
  {
    this.valid = false;
    this.format = 0;
  }
  public void set(int value, boolean valid)
  {
    this.valid = valid;
    this.format = value;
  }
  public ValidatedFormat(int format, String tag, int arrayType, boolean valid)
  {
    this.valid = valid;
    this.format = format;
    this.tag = tag;
    this.arrayType = arrayType;
  }
  @Override
  public String toString()
  {
    return "[" + Integer.toString(format) + "," + (valid ? " VALID]" : "NOT VALID]");
  }
  static ValidatedFormat parse(String s)
  {
    if (s == null || s.length() == 0)
      return new ValidatedFormat(TFormat.CF_NULL, "", TArrayType.AT_UNKNOWN, false);
    int fmt = TFormat.getFormatCode(s);
    String tag = TFormat.getFormatTag(s);
    int atyp = TFormat.getFormatArrayType(s);
    return new ValidatedFormat(fmt,tag,atyp,true);
  }
  public int getArrayType()
  {
    return arrayType;
  }
  public void setArrayType(int arrayType)
  {
    this.arrayType = arrayType;
  }
  public String getTag()
  {
    return tag;
  }
  public void setTag(String tag)
  {
    this.tag = tag;
  }
}
