/*
 * File ValidatedInt.java
 *  
 * Authors Philip Duval (DESY), Jaka Bobnar (CosyLab), 
 *    Jan Szczesny (DESY), Karol Bartkiewicz (DESY), 
 *    Marcus Walla (DESY), Tobias Tempel (DESY)  
 *  
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023  
 *  
 * The TINE .java codeset made available by open source license
 * inherits several conceptual ideas from the original ISOLDE 
 * control system written in the C language ((c)CERN, released under 
 * a permissive CERN licence akin to the later well known MIT licence) 
 * in use at CERN in 1992. We therefore owe a substantial debt to CERN 
 * for its inception, and would be remiss if we did not express our 
 * gratitude in the JAVA TINE open source file headers, although none 
 * of the current Java software is in anyway a redistribution of 
 * software from the ISOLDE system of that era.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * You may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *  
 * http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.  
 */

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package de.desy.tine.xmlUtils;

/**
 * @author karol
 */
public class ValidatedInt
{
  private boolean valid;
  private int value;
  public boolean isValid()
  {
    return valid;
  }
  public int getValue()
  {
    return value;
  }
  public void setValid(boolean valid)
  {
    this.valid = valid;
  }
  public void setValue(int value)
  {
    this.value = value;
  }
  public ValidatedInt()
  {
    this.valid = false;
    this.value = 0;
  }
  public void set(int value, boolean valid)
  {
    this.valid = valid;
    this.value = value;
  }
  public ValidatedInt(int value, boolean valid)
  {
    this.valid = valid;
    this.value = value;
  }
  @Override
  public String toString()
  {
    return "[" + Integer.toString(value) + "," + (valid ? " VALID]" : "NOT VALID]");
  }
  static ValidatedInt parse(String s)
  {
    if (s == null || s.length() == 0) return new ValidatedInt(0, false);
    for (int i = 0; i < s.length(); i++)
    {
      if (!Character.isDigit(s.charAt(i)))
      { // p.d. (24.10.2012): the first character can be a '-' !
        if (i != 0 || s.charAt(0) != '-') return new ValidatedInt(0, false);
      }
    }
    return new ValidatedInt(Integer.parseInt(s), true);
  }
}
