package de.desy.video.sw;

public class CColorLookupTable {

	int [] m_ColorLookupTable=null;
	int m_iColorMode = FALSEPITZ;
	int m_iCurMaxColors=0;
	int m_iEBitPP = 0;
		
	public static final int GRAYSCALE=1;
	public static final int FALSEPITZ=2;
	public static final int FALSEJET=3;		
	
	public CColorLookupTable()
	{
		AdjustTable( FALSEPITZ, 1, 8);
	}

	public int[] getColorLookupTable()
	{
		return m_ColorLookupTable;		
	}
	

	public boolean AdjustTablebdfj( int aByPP, int aEBitPP)
	{
		return AdjustTable( m_iColorMode, aByPP, aEBitPP);
	}
	
	public boolean AdjustTable( int aMode, int aByPP, int aEBitPP)
	{
		int numColors = aByPP << 8;
		//int eBitPP = aEBitPP;
		
		if ((m_iCurMaxColors != numColors) || (m_iEBitPP != aEBitPP) || (aMode != m_iColorMode))
		{
			m_ColorLookupTable = null;
			m_iCurMaxColors = numColors;
			m_iEBitPP = aEBitPP;
			m_iColorMode = aMode;
			m_ColorLookupTable = new int[1 << (aByPP*8)];
    			
    		fillColorLookupTable( aByPP );

    		// fill magenta to the rest
    		int maxval = 1 << aEBitPP;
    		int maxval_total = 1 << (aByPP*8);
    		int left = maxval_total-maxval;

    		if (left > 0)
    		{
    			try 
    			{
    			for (int i=0;i<left;i++)
    			{
    				// put magenta for bad colors that must not happen!
    				// 	alpha24 Red16 Green8 Blue
    				m_ColorLookupTable[i+maxval] = (255 << 24) | (255 << 16) | (255);
    			}
    			}
    			catch( ArrayIndexOutOfBoundsException ex) {}
    			
    		}
    		
    		return true;
		}
		return false;
	}
	
	private void fillColorLookupTable( int aByPP )
	{
		if (m_iColorMode == FALSEPITZ) 
			fillColorLookupTablePITZFalseColors( aByPP);
		else if (m_iColorMode == FALSEJET)
			fillColorLookupTableJETFalseColors( aByPP);
		else 
			fillColorLookupTableGrayscale( aByPP);
	}
	
	private void fillColorLookupTablePITZFalseColors( int aByPP)
	{
		//		 fill in PITZ false colors

		int dec = (int) (1 << m_iEBitPP);
		double decdbl = (double) dec;
		int i;

		// 	alpha24 Red16 Green8 Blue
		m_ColorLookupTable[0] = (255 << 24); // black 0-0-0

		// 	alpha24 Red16 Green8 Blue // white 255-255-255
		m_ColorLookupTable[dec-1] = (255 << 24) | (255 << 16) | (255 << 8) | (255); 
		
		int step1min = 1;
		int step1max = (int) (decdbl/3);
		
		int step2min = step1max + 1;
		int step2max = (int) ((decdbl*293)/500);
		
		int step3min = step2max + 1;
		int step3max = (int) ((decdbl*2)/3);
		
		int step4min = step3max + 1;
		int step4max = (int) decdbl-1;

		for (i=step1min;i<step2min;i++)
		{
			double m = -0.5608 / (dec/3-1.0);
			double n = 1.0 - m*1.0;

			m_ColorLookupTable[i] = (255 << 24) | ((int)((m*i+n)*255.0));
		}

		for (i=step2min;i<step3min;i++)
		{
			double m1= 0.561 / ( (dec*293/500)-(dec/3));
			double n1 = 0.263 - (m1 * (dec/3));
		
			double m2= -0.4314 / ( (dec*293/500)-(dec/3));
			double n2 = 0 - (m2 * (dec*293/500));

			//	alpha24 Red16 Green8 Blue
			m_ColorLookupTable[i] = (255 << 24) | (((int)((m1*i+n1)*255.0))<<8) | ((int)((m2*i+n2)*255.0));
		}

		for (i=step3min;i<step4min;i++)
		{
			double m1= 0.176 / ( (dec*2/3)-(dec*293/500));
			double n1 = 1 - (m1 * (dec*2/3));

			m_ColorLookupTable[i] = (255 << 24) | (((int)((m1*i+n1)*255.0))<<8);
		}
		
		for (i=step4min;i<step4max;i++)
		{
			double m1= 0.737 / (dec-(dec*2/3));
			double n1 = 1 - (m1 * dec);

			m_ColorLookupTable[i] = (255 << 24) | (((int)((m1*i+n1)*255.0))<<16);
		}
		
	}
	
	
	private void fillColorLookupTableJETFalseColors( int aByPP )
	{
		int dec = (int) (1 << m_iEBitPP);
		int i;
		
		int step1min = 0;
		int step1max = (int) ((dec/8)-1);
		
		int step2min = step1max + 1;
		int step2max = (int) (((dec*3)/8)-1);
		
		int step3min = step2max + 1;
		int step3max = (int) (((dec*5)/8)-1);
		
		int step4min = step3max + 1;
		int step4max = (int) (((dec*7)/8)-1);
		
		int step5min = step4max + 1;
		int step5max = (int) (dec-1);

		// 	alpha24 Red16 Green8 Blue // white 255-255-255
		m_ColorLookupTable[dec-1] = (255 << 24) | (255 << 16) | (255 << 8) | (255);

		double Dstep2min = (double) step2min;
		
		for (i=step1min;i<step2min;i++)
		{
			double Di= (double) i;
			double blue = (((Di/2.0)/Dstep2min)+0.5)*255.0;

			// 	alpha24 Red16 Green8 Blue			
			m_ColorLookupTable[i] = (255 << 24) | (int) blue;
		}

		for (i=step2min;i<step3min;i++)
		{
			double Di = (double) i;
			double green = ((Di/(Dstep2min*2.0))-0.5)*255.0;
			
			// alpha24 Red16 Green8 Blue
			m_ColorLookupTable[i] = (255 << 24) | (((int)green)<<8) | 255;
		}

		for (i=step3min;i<step4min;i++)
		{
			double Di = (double) i;
			double red = ((Di/(Dstep2min*2.0))-1.5) *255.0;
			double blue = ((1.0-((Di/(Dstep2min*2.0))-1.5))*255.0);
			
			// alpha24 Red16 Green8 Blue
			m_ColorLookupTable[i] = (255 << 24) | (((int)red)<<16) | (255<<8) | ((int) blue);
		}
		
		for (i=step4min;i<step5min;i++)
		{
			double Di = (double) i;
			double green = (((Di/(-(Dstep2min*2.0)))+3.5) *255.0);

			// alpha24 Red16 Green8 Blue
			m_ColorLookupTable[i] = (255 << 24) | (255<<16) | (((int)green)<<8);
		}

		for (i=step5min;i<step5max;i++)
		{
			double Di = (double) i;
			double red = (((Di/(-(Dstep2min*2.0)))+4.5) *255.0); 

			// alpha24 Red16 Green8 Blue			
			m_ColorLookupTable[i] = (255 << 24) | (((int)red)<<16);
		}
		
	}

	
	private void fillColorLookupTableGrayscale( int aByPP )
	{
		int maxval = (int) (1 << m_iEBitPP);
		
		try 
		{
		for (int i=0;i<maxval;i++)
		{
			int gray = (int) ((255.0*i)/maxval);
			// 	alpha24 Green16 Red8 Blue
			m_ColorLookupTable[i] = (255 << 24) | (gray << 16) | (gray << 8) | (gray);
		}
		}
		catch( ArrayIndexOutOfBoundsException ex) {}
		
	}	
}
