package de.desy.video.sw;

// big memory for testing:
// 	-Xms512m -Xmx512m -XX:PermSize=128m -XX:MaxPermSize=128m

import java.awt.AlphaComposite;
import java.awt.BorderLayout;
import java.awt.Canvas;
import java.awt.Color;
import java.awt.Composite;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.HeadlessException;
import java.awt.Image;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionAdapter;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelListener;
import java.awt.image.MemoryImageSource;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

public final class ImageDisplayer extends Container {

	public enum ColorMap {
		GRAYSCALE, PITZ, JET;
		
		@Override
		public String toString() {
			switch(this) {
				case GRAYSCALE : return "Grayscale";
				case PITZ : return "PITZ False Color";
				case JET : return "JET False Color";
			}
			return super.toString();
		}
	}

	private static final long serialVersionUID = 1L;
	private java.awt.image.BufferStrategy m_FlipStrategy = null;
	private Canvas m_VideoCanvas = null;

	private boolean m_bOverlayEnabled;
	private boolean m_bVideoCanvasMouseInOverlay;
	private boolean m_bVideoCanvasMouseInCanvas;
	private boolean m_bVideoCanvasMouseLeftButtonDown;
	private boolean m_bAOIZoom;
	private boolean m_bKeepAspectRatioEnabled;
	private boolean m_bFalsecolorEnabled = true;
	private ColorMap colorMap = ColorMap.PITZ;

	private final int minx = 592;
	private final int miny = 211; // 127 + 2x42 OSD;

	private int[] m_DisplayImageBuffer = null;// = new int[hdr.width*hdr.height];
	private int[] m_ColorLookupTableArray = null; // mapping from
	private CColorLookupTable m_CColorLookupTable = null;

	private long m_lastframenumber = -1;
	private long m_droppedframes = 0;
	private long m_receivedframes = 0;
	private double m_ratio = -0.001;

	private Image m_img = null; // @jve:decl-index=0:
	private MemoryImageSource m_mis = null; // @jve:decl-index=0:

	private int m_iColorMode = CColorLookupTable.FALSEPITZ;
	private boolean m_bDoUpdateColorLookupTable = true;
	private CVideoHeader3 m_Hdr = null;
	private byte[] m_dstBuf;

	/**
	 * This is the default constructor
	 */

	// SW! String aDefaultStartupServer
	public ImageDisplayer() {
		super();
		initialize();
	}
	
	@Override
	public void setVisible(boolean b) {
		super.setVisible(b);
		if (b) {
			m_VideoCanvas.createBufferStrategy(2);
			m_FlipStrategy = m_VideoCanvas.getBufferStrategy();

			m_CColorLookupTable = new CColorLookupTable();
		}
	}

	@Override
	public synchronized void addMouseListener(MouseListener l) {
		super.addMouseListener(l);
		m_VideoCanvas.addMouseListener(l);
	}
	
	@Override
	public synchronized void addMouseMotionListener(MouseMotionListener l) {
		super.addMouseMotionListener(l);
		m_VideoCanvas.addMouseMotionListener(l);
	}
	
	@Override
	public synchronized void addMouseWheelListener(MouseWheelListener l) {
		super.addMouseWheelListener(l);
		m_VideoCanvas.addMouseWheelListener(l);
	}
	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(559, 387);
		this.setMinimumSize(new Dimension(minx, miny));
		this.setLayout(new BorderLayout());
		
		m_VideoCanvas = new Canvas();
		m_VideoCanvas.setBackground(new Color(80, 80, 80));
		m_VideoCanvas.setBounds(5, 5, 200, 200);
		m_VideoCanvas.setIgnoreRepaint(true);
		m_VideoCanvas.addMouseListener(new MouseAdapter() {
			public void mouseExited(MouseEvent e) {
				boolean old = m_bVideoCanvasMouseInOverlay;
				m_bVideoCanvasMouseInOverlay = false;
				m_bVideoCanvasMouseInCanvas = false;
				// m_bVideoCanvasMouseLeftButtonDown = false;
				// System.out.println("Mouse exited.");
				if (old)
					repaintAnything();
			}

			public void mouseEntered(MouseEvent e) {
				boolean old = m_bVideoCanvasMouseInOverlay;
				// m_bVideoCanvasMouseInOverlay = true;
				m_bVideoCanvasMouseInCanvas = true;
				// m_bVideoCanvasMouseLeftButtonDown = false;
				// System.out.println("Mouse exited.");
				if (old)
					repaintAnything();
			}

			public void mousePressed(MouseEvent e) {
				// if (e.getModifiers() == MouseEvent.BUTTON1_DOWN_MASK)
				m_bVideoCanvasMouseLeftButtonDown = true;

			}

			public void mouseReleased(MouseEvent e) {
				// if (e.getModifiers() == MouseEvent.BUTTON1_DOWN_MASK)
				m_bVideoCanvasMouseLeftButtonDown = false;

			}

		});

		m_VideoCanvas.addMouseMotionListener(new MouseMotionAdapter() {
			public void mouseMoved(MouseEvent e) {
				// int xpos = e.getX();
				int ypos = e.getY();

				// boolean old = m_bVideoCanvasMouseInOverlay;

				if (((ypos > 0) && (ypos < 50))
						|| ((ypos > m_VideoCanvas.getHeight() - 50) && (ypos < m_VideoCanvas.getHeight())))
					m_bVideoCanvasMouseInOverlay = true;

				if (((ypos <= 0) || (ypos >= m_VideoCanvas.getHeight())))
					m_bVideoCanvasMouseInOverlay = false;

				m_bVideoCanvasMouseInCanvas = true;



				// if (old != m_bMouseInOverlay) repaintAnything();

			}
		});

		this.add(m_VideoCanvas, BorderLayout.CENTER);

		m_iColorMode = CColorLookupTable.FALSEPITZ;
		m_bDoUpdateColorLookupTable = true;

		m_bOverlayEnabled = false;
		m_bVideoCanvasMouseInOverlay = false;
		m_bVideoCanvasMouseInCanvas = false;
		m_bVideoCanvasMouseLeftButtonDown = false;

		m_bAOIZoom = false;

		this.addComponentListener(new ComponentAdapter() {
			public void componentResized(ComponentEvent arg0) {
				repaint();
			}

			public void componentMoved(ComponentEvent arg0) {
				repaint();
			}

			public void componentShown(ComponentEvent arg0) {
				repaint();
			}
		});
	}

	private AlphaComposite makeComposite(float alpha) {
		int type = AlphaComposite.SRC_OVER;
		return (AlphaComposite.getInstance(type, alpha));
	}

	// public synchronized void updateImage( Image img, long aFrameNumber )
	public void updateImage() {

		if (m_Hdr == null)
			return;

		Graphics g = null;
		do {
			// The following loop ensures that the contents of the drawing
			// buffer
			// are consistent in case the underlying surface was recreated
			do {
				try {
					g = m_FlipStrategy.getDrawGraphics();
					java.awt.Graphics2D g2d = (java.awt.Graphics2D) g;

					m_VideoCanvas.paint(g);

					if (m_img != null) {
						java.awt.Rectangle imgR = new java.awt.Rectangle();
						java.awt.Rectangle srcR = new java.awt.Rectangle();
						java.awt.Rectangle dstR = new java.awt.Rectangle();
						
						java.awt.Rectangle dstROrig = new java.awt.Rectangle();
						

						imgR.x = 0;
						imgR.y = 0;
						imgR.width = m_Hdr.frameHeader.sourceWidth;
						imgR.height = m_Hdr.frameHeader.sourceHeight;

						srcR.x = m_Hdr.frameHeader.xStart;
						srcR.y = m_Hdr.frameHeader.yStart;
						srcR.width = m_Hdr.frameHeader.aoiWidth;
						srcR.height = m_Hdr.frameHeader.aoiHeight;
						if (srcR.width == -1)
							srcR.width = imgR.width;
						if (srcR.height == -1)
							srcR.height = imgR.height;

						dstROrig.x = 2;
						dstROrig.y = 2;
						dstROrig.width = m_VideoCanvas.getWidth() - 4;
						dstROrig.height = m_VideoCanvas.getHeight() - 4;

						dstR.x = 2;
						dstR.y = 2;
						dstR.width = m_VideoCanvas.getWidth() - 4;
						dstR.height = m_VideoCanvas.getHeight() - 4;

						// dstR = dstROrig;

						if (m_bKeepAspectRatioEnabled == true) {
							double whratiovideo = 1.0;
							if (imgR.height > 0)
								whratiovideo = ((double) imgR.width) / ((double) imgR.height);

							for (;;) {
								dstR.height = (int) Math.round((dstR.width) / whratiovideo);
								if (dstR.height > dstROrig.height) {
									dstR.width--;
									continue;
								} else if (dstR.width <= dstROrig.width)
									break;
								else
									dstR.width = dstROrig.width - 1;
							}

							dstR.x = dstROrig.x + ((dstROrig.width - dstR.width) / 2);
							dstR.y = dstROrig.y + ((dstROrig.height - dstR.height) / 2);
						}

						// after: now width and height are adjusted properly to
						// canvas for whole big image
						if ((m_bAOIZoom == false) && ((srcR.width != imgR.width) || (srcR.height != imgR.height))) {
							g2d.setColor(new Color(0, 0, 0)); // DARK_
							g2d.fillRect(dstR.x, dstR.y, dstR.width, dstR.height);
						}

						if (m_bAOIZoom == true) {

							if (m_bKeepAspectRatioEnabled == true) {
								int widthaoi = srcR.width;
								int heightaoi = srcR.height;

								double whratioaoi = 1.0;
								if (heightaoi > 0)
									whratioaoi = ((double) widthaoi) / ((double) heightaoi);

								// heightaoi = height;
								widthaoi = dstROrig.width;

								for (;;) {
									heightaoi = (int) Math.round((widthaoi) / whratioaoi);
									if (heightaoi > dstROrig.height) {
										widthaoi--;
										continue;
									} else if (widthaoi <= dstROrig.width)
										break;
									else {
										widthaoi = dstROrig.width;
									}
								}

								dstR.width = widthaoi;
								dstR.height = heightaoi;

							}

							dstR.x = dstROrig.x + ((dstROrig.width - dstR.width) / 2);
							dstR.y = dstROrig.y + ((dstROrig.height - dstR.height) / 2);
						} else {
							double wscale_full = (dstR.width * 1.0 / imgR.width * 1.0);
							double hscale_full = (dstR.height * 1.0 / imgR.height * 1.0);

							dstR.width = (int) Math.round(srcR.width * wscale_full);
							dstR.height = (int) Math.round(srcR.height * hscale_full);

							dstR.x += ((int) (srcR.x * wscale_full));
							dstR.y += ((int) (srcR.y * hscale_full));
						}

						g2d.drawImage(m_img, dstR.x, dstR.y, dstR.width, dstR.height, null);

						String sAddPixelValue = "";

						if ((m_bOverlayEnabled == true) || (m_bVideoCanvasMouseInOverlay == true)) {
							if ((m_bVideoCanvasMouseInCanvas == true) && (m_bVideoCanvasMouseLeftButtonDown == true)) {
								sAddPixelValue = "";
								try {
									Point p = m_VideoCanvas.getMousePosition();
									if (p != null) {
										int pixelindex = 0;

										sAddPixelValue += "  (out of dimension)";

										if (((p.x >= dstR.x) && (p.x <= (dstR.x + dstR.width)))
												&& ((p.y >= dstR.y) && (p.y <= (dstR.y + dstR.height)))) {
											// do coordinate transform, src, dst

											double dst2srcWidthRelation = 1.0 * srcR.width / dstR.width;
											double dst2srcHeightRelation = 1.0 * srcR.height / dstR.height;

											int pixelx = (int) ((1.0 * (p.x - dstR.x)) * dst2srcWidthRelation);
											int pixely = (int) ((1.0 * (p.y - dstR.y)) * dst2srcHeightRelation);
											if (pixelx >= srcR.width)
												pixelx = srcR.width - 1;
											if (pixely >= srcR.height)
												pixely = srcR.height - 1;

											pixelindex = (srcR.width * pixely + pixelx)
													* m_Hdr.frameHeader.bytesPerPixel;

											sAddPixelValue = "  px(" + (pixelx + 1 + srcR.x) + ","
													+ (pixely + 1 + srcR.y) + ") = ";
											// debug:
											// sAddPixelValue = "
											// ("+p.x+"/"+p.y+")
											// px("+(pixelx+1+srcR.x)+","+(pixely+1+srcR.y)+")
											// = ";

											if (m_Hdr.frameHeader.imageFormat == CVideoHeader3.CF_IMAGE_FORMAT_GRAY) {
												long pxvalue = ((long) m_dstBuf[pixelindex]) & 0xff;
												if (m_Hdr.frameHeader.bytesPerPixel > 1)
													pxvalue |= (((long) m_dstBuf[pixelindex + 1]) & 0xff) << 8;
												if (m_Hdr.frameHeader.bytesPerPixel > 2)
													pxvalue |= (((long) m_dstBuf[pixelindex + 2]) & 0xff) << 16;
												if (m_Hdr.frameHeader.bytesPerPixel > 3)
													pxvalue |= (((long) m_dstBuf[pixelindex + 3]) & 0xff) << 24;

												int maxpxvalue = (1 << m_Hdr.frameHeader.effectiveBitsPerPixel) - 1;

												sAddPixelValue += pxvalue + " (out of " + maxpxvalue + ")";
											} else if (m_Hdr.frameHeader.imageFormat == CVideoHeader3.CF_IMAGE_FORMAT_RGB) {
												long redvalue = ((long) m_dstBuf[pixelindex]) & 0xff;
												long greenvalue = ((long) m_dstBuf[pixelindex + 1]) & 0xff;
												long bluevalue = ((long) m_dstBuf[pixelindex + 2]) & 0xff;

												sAddPixelValue += "(" + redvalue + "/" + greenvalue + "/" + bluevalue
														+ ")";
											} else {
												sAddPixelValue += "(unknown)";
											}
										}

									}
								} catch (HeadlessException ex1) {
								}
							}

						}

						if ((m_bOverlayEnabled == true) || (m_bVideoCanvasMouseInOverlay == true)) {
							Composite originalComposite = g2d.getComposite();
							g2d.setColor(new Color(190, 190, 190)); // DARK_
							g2d.setComposite(makeComposite(0.85F));
							g2d.fillRect(2, 2, m_VideoCanvas.getWidth() - 4, 40);
							g2d.fillRect(2, m_VideoCanvas.getHeight() - 42, m_VideoCanvas.getWidth() - 4, 40);
							g2d.setComposite(originalComposite);

							g2d.setColor(new Color(0, 0, 0));

							String s = "Source: " + m_Hdr.sourceHeader.cameraPortName + " ("
									+ CVideoHeader3.formatToString(m_Hdr.frameHeader.sourceFormat) + ")";
							g2d.drawString(s, 10, 16);

							s = "Size: " + m_Hdr.frameHeader.sourceWidth + " px * " + m_Hdr.frameHeader.sourceHeight
									+ " px";

							if ((m_Hdr.frameHeader.aoiWidth != -1) || (m_Hdr.frameHeader.aoiHeight != -1)) {
								s += "  (AOI: L " + m_Hdr.frameHeader.xStart + ", T " + m_Hdr.frameHeader.yStart
										+ ", W " + m_Hdr.frameHeader.aoiWidth + ", H " + m_Hdr.frameHeader.aoiHeight
										+ " )";
							} else
								s += "  (AOI: none)";
							g2d.drawString(s, 10, 34);

							DateFormat dfmt = new SimpleDateFormat("HH:mm:ss.SSS");

							s = "# " + m_Hdr.frameHeader.frameNumber + " - " + dfmt.format(m_Hdr.getTimestampAsDate());
							g2d.drawString(s, m_VideoCanvas.getWidth() - g2d.getFontMetrics().stringWidth(s) - 10, 16);

							s = "Drop: " + m_droppedframes + " ("
									+ String.format("%4.3f", new java.lang.Object[]{Double.valueOf(m_ratio * 100.0)})
									+ "%)";
							g2d.drawString(s, m_VideoCanvas.getWidth() - g2d.getFontMetrics().stringWidth(s) - 10, 34);

							s = "Format: " + CVideoHeader3.formatToString(m_Hdr.frameHeader.imageFormat);
							s += sAddPixelValue;
							g2d.drawString(s, 10, m_VideoCanvas.getHeight() - 26);

							s = "Flags: " + CVideoHeader3.flagsToString(m_Hdr.frameHeader.imageFlags);
							g2d.drawString(s, 10, m_VideoCanvas.getHeight() - 8);

							double rot = m_Hdr.frameHeader.imageRotation;

							double rotAbs = Math.abs(rot);

							s = "Rotation: " + String.format("%4.2f", new java.lang.Object[]{Double.valueOf(rotAbs)})
									+ "%";

							if (rot > 0.0)
								s += " cw";
							else if (rot < 0.0)
								s += " ccw";

							g2d.drawString(s, m_VideoCanvas.getWidth() - g2d.getFontMetrics().stringWidth(s) - 10,
									m_VideoCanvas.getHeight() - 8);

						}

					}

				} catch (Exception ex) {
					ex.printStackTrace();
				} finally {
					if (g != null) {
						g.dispose();
					}
				}
			} while (m_FlipStrategy.contentsRestored());

			// Display the buffer
			m_FlipStrategy.show();

			// Repeat the rendering if the drawing buffer was lost
		} while (m_FlipStrategy.contentsLost());
	}

	public void update(Graphics g) {
		paint(g);
	}

	public void repaintAnything() {
		updateImage();
		repaint();
	}


	/*
	 * private void RepaintCanvas() { Graphics g = null; try { if
	 * (m_FlipStrategy != null) { g = (java.awt.Graphics2D)
	 * m_FlipStrategy.getDrawGraphics(); m_VideoCanvas.paint(g); } } catch(
	 * Exception ex ) { ex.printStackTrace(); } finally { if (g != null)
	 * g.dispose(); } }
	 */

	public void setColorMap(ColorMap map) {
		if (this.colorMap == map)
			return;
		ColorMap oldValue = this.colorMap;
		this.colorMap = map;

		if (colorMap == ColorMap.GRAYSCALE) {
			m_iColorMode = CColorLookupTable.GRAYSCALE;
		} else if (colorMap == ColorMap.PITZ) {
			m_iColorMode = CColorLookupTable.FALSEPITZ;
		} else if (colorMap == ColorMap.JET) {
			m_iColorMode = CColorLookupTable.FALSEJET;
		}
		m_bDoUpdateColorLookupTable = true;
		firePropertyChange("colorMap", oldValue, this.colorMap);
		repaint();
	}
	
	public ColorMap getColorMap() {
		return colorMap;
	}

	public void setOverlayEnabled(boolean overlay) {
		if (this.m_bOverlayEnabled == overlay)
			return;
		this.m_bOverlayEnabled = overlay;
		firePropertyChange("overlayEnabled", !m_bOverlayEnabled, m_bOverlayEnabled);
		repaint();
	}

	public boolean isOverlayEnabled() {
		return m_bOverlayEnabled;
	}

	public void setAOIZoom(boolean zoom) {
		if (m_bAOIZoom == zoom)
			return;
		this.m_bAOIZoom = zoom;
		firePropertyChange("AOIZoom", !m_bAOIZoom, m_bAOIZoom);
		repaint();
	}

	public boolean isAOIZoom() {
		return m_bAOIZoom;
	}

	public void setKeepAspectRatioEnabled(boolean keep) {
		if (this.m_bKeepAspectRatioEnabled == keep)
			return;
		this.m_bKeepAspectRatioEnabled = keep;
		firePropertyChange("keepAspectRatioEnabled", !m_bKeepAspectRatioEnabled, m_bKeepAspectRatioEnabled);
		repaint();
	}
	
	public boolean isKeepAspectRatioEnabled() {
		return m_bKeepAspectRatioEnabled;
	}

	public boolean saveAsPNGImage(String fileNamePath) {
		return CBasicPNGWriterV3.write(fileNamePath, m_img, m_Hdr);
	}
	
	private void setFalseColorSwitchingEnabled(boolean enabled) {
		if (m_bFalsecolorEnabled == enabled) return;
		m_bFalsecolorEnabled = enabled;
		firePropertyChange("falseColorSwitchingEnabled", !m_bFalsecolorEnabled, m_bFalsecolorEnabled);
	}
	
	public boolean isFalseColorSwitchingEnabled() {
		return m_bFalsecolorEnabled;
	}

	public void updateValue(CVideoHeader3 hdr) {
		
		byte[] m_srcBuf = hdr.getImage().getImageFrameBuffer();
		
		// adjust into raw bits
		int pixellength = hdr.getAppendedWidth() * hdr.getAppendedHeight();
		int uncompressed_byte_length = pixellength * hdr.frameHeader.bytesPerPixel;

		if (m_DisplayImageBuffer == null) {
			m_DisplayImageBuffer = new int[pixellength];
		} else if (m_DisplayImageBuffer.length < pixellength) {
			m_DisplayImageBuffer = null;
			m_DisplayImageBuffer = new int[pixellength];
		}

		if (hdr.frameHeader.imageFormat == CVideoHeader3.CF_IMAGE_FORMAT_GRAY) {
			if ((m_dstBuf == null) || (m_dstBuf.length < uncompressed_byte_length))
				m_dstBuf = new byte[uncompressed_byte_length];

			if (convertImage(m_srcBuf, hdr, m_dstBuf) == false) {
				System.out.println("convertImage false");
				return;
			}

			if (m_bFalsecolorEnabled == false) {
				// enable false color switching in gui
				setFalseColorSwitchingEnabled(true);
			}

			// System.out.println("Image #"+hdr.framenumber+" received.");

			if ((m_ColorLookupTableArray == null)
					|| (m_ColorLookupTableArray.length < (hdr.frameHeader.bytesPerPixel << 8))) {
				m_bDoUpdateColorLookupTable = true;
			}

			if (m_bDoUpdateColorLookupTable == true) {
				boolean ret = m_CColorLookupTable.AdjustTable(m_iColorMode, hdr.frameHeader.bytesPerPixel,
						hdr.frameHeader.effectiveBitsPerPixel);
				if ((ret == true) || (m_ColorLookupTableArray == null))
					m_ColorLookupTableArray = m_CColorLookupTable.getColorLookupTable();
				m_bDoUpdateColorLookupTable = false;
			}

			int maxcol = (1 << hdr.frameHeader.effectiveBitsPerPixel) - 1;
			int index = 0;
			try {
				// not only 8bpp by now (but not "perfect")
				// todo make faster
				while (index < pixellength) {
					if (hdr.frameHeader.bytesPerPixel == 1)
						m_DisplayImageBuffer[index] = m_ColorLookupTableArray[(m_dstBuf[index]) & maxcol];

					if (hdr.frameHeader.bytesPerPixel == 2) {
						int tmp;
						int clutindex = 0;

						clutindex = ((m_dstBuf[2 * index]) & 0xff);
						tmp = (((m_dstBuf[2 * index + 1])) & 0xff) << 8;

						// &maxcol
						m_DisplayImageBuffer[index] = m_ColorLookupTableArray[tmp + clutindex];
					}

					index++;
				}
			} catch (ArrayIndexOutOfBoundsException ex) {
			}

		} else // can only be RGB now
		{
			if (m_bFalsecolorEnabled == true) {
				// disable false color switching in gui
				setFalseColorSwitchingEnabled(false);
			}

			if ((m_dstBuf == null) || (m_dstBuf.length < uncompressed_byte_length))
				m_dstBuf = new byte[uncompressed_byte_length];

			// rgb24
			if (convertImageColor(m_srcBuf, m_dstBuf, hdr, m_DisplayImageBuffer) == false) {
				System.out.println("convertImageColor false");
				return;
			}
		}

		if (m_mis == null) {
			m_mis = new MemoryImageSource(hdr.getAppendedWidth(), hdr.getAppendedHeight(), m_DisplayImageBuffer, 0, hdr
					.getAppendedWidth());
			m_mis.setAnimated(true);
			m_mis.setFullBufferUpdates(true);
		}

		if (m_img == null) {
			m_img = Toolkit.getDefaultToolkit().createImage(m_mis);
		}

		m_mis.newPixels();

		// calculate variables

		if (m_lastframenumber == -1) {
			m_lastframenumber = hdr.frameHeader.frameNumber;
			m_droppedframes = 0;
			m_receivedframes++;
		} else {
			if (hdr.frameHeader.frameNumber == m_lastframenumber) {
				System.out.println("Theyre same: " + hdr.frameHeader.frameNumber + "  " + m_lastframenumber + " !");
			} else if (hdr.frameHeader.frameNumber < m_lastframenumber) {
				m_receivedframes++;
			} else {
				m_droppedframes += (hdr.frameHeader.frameNumber - m_lastframenumber - 1);
				m_receivedframes++;
			}
			m_lastframenumber = hdr.frameHeader.frameNumber;

		}

		if (m_receivedframes != 0)
			m_ratio = ((1.0 * m_droppedframes) / (1.0 * (m_receivedframes)));

		if (m_img != null) {
			updateHeader(hdr);
			updateImage();
		}
	}
	
	private synchronized void updateHeader(CVideoHeader3 aHdr) {
		m_Hdr = aHdr;
	}

	// //////////////////////////////////////////////////////////////////////////////
	// frame decode handling
	// //////////////////////////////////////////////////////////////////////////////

	private boolean convertImage(byte[] aSrc, CVideoHeader3 aHdr, byte[] aDst) {
		int uncompressed_size = aHdr.getAppendedWidth() * aHdr.getAppendedHeight() * aHdr.frameHeader.bytesPerPixel;

		if ((aSrc == null) || (aDst == null) || (aDst.length < uncompressed_size))
			return false;

		if (aHdr.frameHeader.sourceFormat == CVideoHeader3.CF_IMAGE_FORMAT_GRAY) {
			if (aHdr.frameHeader.imageFormat != aHdr.frameHeader.sourceFormat) {
				// only HuffYUV is supported here as decompression
				if (aHdr.frameHeader.imageFormat != CVideoHeader3.CF_IMAGE_FORMAT_HUFFYUV)
					return false;

				CHuffmanDecompression.DecompressHuffYUV(aSrc, CVideoHeader3.HDRSIZE, aDst, 0, uncompressed_size);

				// adjust header after compression
				aHdr.frameHeader.imageFormat = aHdr.frameHeader.sourceFormat;
				aHdr.sourceHeader.totalLength = uncompressed_size + CVideoHeader3.HDRSIZE;
				aHdr.frameHeader.appendedFrameSize = uncompressed_size;
			} else {
				// System.arraycopy(aSrc, CVideoHeader3.HDRSIZE, aDst, 0, (int)
				// aHdr.frameHeader.appendedFrameSize);
				System.arraycopy(aSrc, 0, aDst, 0,  aHdr.frameHeader.appendedFrameSize);
			}
		} else {
			return false;
		}

		return true;
	}

	private boolean convertImageColor(byte[] aSrc, byte[] aDstRaw, CVideoHeader3 aHdr, int[] aDst) {
		int uncompressed_pixel_size = aHdr.getAppendedWidth() * aHdr.getAppendedHeight();

		if ((aSrc == null) || (aDst == null) || (aDst.length < uncompressed_pixel_size))
			return false;
		if ((aDstRaw == null) || (aDstRaw.length < uncompressed_pixel_size * aHdr.frameHeader.bytesPerPixel))
			return false;

		if (aHdr.frameHeader.imageFormat != CVideoHeader3.CF_IMAGE_FORMAT_RGB) {
			return false;
		}
		int index = 0;
		int index1 = 0;

		try {
			while (index < uncompressed_pixel_size) {
				int red16 = aSrc[index1++]& 0xFF;
				int green8 = aSrc[index1++] & 0xFF;
				int blue = aSrc[index1++] & 0xFF;

				aDst[index++] = 0xFF000000 | (red16 << 16) | (green8 << 8) | (blue);
				//  
			}
		} catch (ArrayIndexOutOfBoundsException ex) {
		}

		System.arraycopy(aSrc, 0, aDstRaw, 0, aHdr.frameHeader.appendedFrameSize);
		// System.arraycopy(aSrc, CVideoHeader3.HDRSIZE, aDstRaw, 0, (int)
		// aHdr.imH_length);

		return true;
	}

	void resetForReceiving() {
		m_img=null;
		m_mis=null;
		
		m_DisplayImageBuffer = null;
		m_ColorLookupTableArray = null;
		m_bDoUpdateColorLookupTable = true;
		m_lastframenumber=-1;
		m_droppedframes=0;
		m_receivedframes=0;
		m_ratio = -0.001;
	}
	
}
