package de.desy.video.sw;

import com.cosylab.gui.components.util.RunnerHelper;

import de.desy.acop.transport.AccessMode;
import de.desy.acop.transport.ConnectionFailed;
import de.desy.tine.client.TLink;
import de.desy.tine.client.TLinkCallback;
import de.desy.tine.dataUtils.TDataType;
import de.desy.tine.definitions.TAccess;
import de.desy.tine.definitions.TMode;
import de.desy.tine.types.IMAGE;

public class TineHandler implements TLinkCallback {

	private ImageDisplayer displayer;
	// buffers to work with
	private IMAGE m_srcImgHdr;
	private byte[] m_srcBuf;
	private TLink m_vidLink;
	private boolean m_bTransferRunning = false;

	public TineHandler(ImageDisplayer displayer) {
		this.displayer = displayer;
	}

	public void callback(TLink link) {
		
		if (!m_bTransferRunning) return;
		
		if (link.getLinkStatus() == 516)
			return;

		if (link.getLinkStatus() != 0) {
			System.out.println("Link Error : code=" + link.getLinkStatus() + " msg=" + link.getLastError());
			return;
		}

		displayer.updateValue(new CVideoHeader3(m_srcImgHdr));
	}

	public void openLink(String aCmdLine, AccessMode mode, int accessRate) throws Exception
	{
		String devName = aCmdLine;  // /context/server/device;
		String devProp = "Frame.Sched";
				
		if (devName.endsWith("/")) devName += "Output";
		
		if (!probeTine(devName, "Header")) {
			throw new ImageException("Error checking connection");
		}
		
		// buffer must be dividable by four
		// 192 = 188 headerbytes + (dividable by 8)
		if (m_srcBuf == null)
			m_srcBuf = new byte[1024*1024*6+192];
   
		m_srcImgHdr = new IMAGE();
	    m_srcImgHdr.setImageFrameBuffer(m_srcBuf);
	    
	    TDataType img_dout = new TDataType(m_srcImgHdr);
		
		m_vidLink = new TLink(devName,devProp,img_dout,null,TAccess.CA_READ);
		short tMode;;
		if (mode == AccessMode.POLL) {
			tMode = TMode.CM_POLL;
		} else if (mode == AccessMode.READ) {
			tMode = TMode.CM_SINGLE;
		} else {
			throw new ConnectionFailed("Cannot establish a connection. AccessMode should be POLL or READ.", null);
		}
	    if (m_vidLink.attach(tMode, this, accessRate) < 0)
	    {
	    	m_vidLink.cancel();
	    	m_vidLink = null;
    	  
	    	throw new ConnectionFailed("No permanent connection to server (\""+m_vidLink.getLastError()+"\".", null);
	    }
				
	    m_bTransferRunning = true;
	}

	private boolean probeTine(String aDevName, String aDevProp) throws Exception {
		byte[] framebits = new byte[CVideoHeader3.HDRSIZE];
		// 1 meg max frame size, e.g. 800x624 x 2 bypp + header
		// if bigger than server max work area size, then error 30
		// (invalid transport)

		TDataType dout = new TDataType(framebits);

		// get a reference array : synchronous call ...
		TLink ref = new TLink(aDevName, aDevProp, dout, null, TAccess.CA_READ);
		int cc = ref.execute();
		if (cc != 0) {
			// System.out.println("Error: get reference : " +
			// ref.getLinkStatus());
			// 517==RETCODE_NOFRAMEYET
			throw new ConnectionFailed("Probing Address did not succeed. Transfer was cancelled.", null);
		} else {
			ref.cancel();
			// System.out.println("Got "+dout.dCompletionLength+" bytes back.");
			if (dout.dCompletionLength != CVideoHeader3.HDRSIZE) {
				throw new ImageException("CF_IMAGE header is not 172 bytes in size. Transfer was cancelled.");
			}
		}

		framebits = null;
		return true;
	}

	public void closeLink() {
		if (m_vidLink != null) {
			m_vidLink.cancel();
			m_vidLink = null;
		}

		m_bTransferRunning = false;
	}
	
	public static void main(String[] args) throws Exception {
		ImageDisplayer d = new ImageDisplayer();
		TineHandler handler = new TineHandler(d);
		handler.openLink("/TEST/SGP_IMM1", AccessMode.POLL, 1000);
//		d.setVisible(true);
		RunnerHelper.runComponent(d, 800,600);
	}
}
