/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.chart;


import java.awt.AWTEvent;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GraphicsConfiguration;
import java.awt.GraphicsEnvironment;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.HierarchyBoundsListener;
import java.awt.event.HierarchyEvent;
import java.awt.event.MouseEvent;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;
import java.beans.Beans;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.imageio.ImageIO;
import javax.swing.AbstractAction;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.plaf.basic.BasicGraphicsUtils;

import com.cosylab.gui.components.customizer.AbstractCustomizerPanel;
import com.cosylab.gui.components.util.PopupManageable;
import com.cosylab.gui.components.util.PopupManager;

import de.desy.acop.displayers.AcopChartCustomizer;
import de.desy.acop.transport.AcopTransport;

/**
 * @brief Advanced Component Oriented Programming ACOP offers a powerful
 *        interface for both data acquisition and data rendition in a common
 *        bean.
 */
public class Acop extends JPanel implements Printable,Serializable, PopupManageable
{
  /**
   *
   */
  private static final long serialVersionUID = 351823797042270731L;
  protected AcopHisto[] histo = new AcopHisto[AcopConst.MAX_DISPLAY]; // maximum
                                                                      // 100
  protected int globalDisplayMode, enableDots, enableBarlineHisto, bitDisplayMask;
  protected double ganttBitDisplayBase, ganttBitDisplayHeight;
  // added by ike to demonstrate simple customizer
	private AbstractCustomizerPanel customizer;
	private PopupManager popupManager;
	private boolean popupEnabled;
  protected boolean frameBorderOn;
  boolean forceWidth = false;
  private int n_display;
  protected int m_lock;
  protected boolean drawTicks = true; // after new scalling
  protected boolean repainHisto = false;
  protected int rescaleFlag = 2;
  protected int histIndex = -1;
  private int loadXYindex = 0;
  private int m_filled = 0; // N_DISPLAY is filled
  private int m_keep = 0; // # of keeped histogram
  protected int lastDisp = 0;
  protected int lastTagHisto = 0;
  private int m_numberkeep = 0;
  private int m_keepstart = 0;
  protected AcopAxis[] axis = new AcopAxis[AcopConst.MAX_AXIS];
  protected AcopText[] ptext = new AcopText[AcopConst.MAX_TEXT]; // maximum 100
  private int n_text;
  private int loadTextIndex = 0;
  protected boolean textFlag = false; // Ondraw text?
  protected int rescaleTextFlag = 0;
  private double[] g_rdata = new double[512];
  private boolean RandomInitialized = false;
  private BasicStroke stroke = new BasicStroke(1.0f);
  // private Stroke penLine = null;
  // private int dxmin, dxmax, dymin, dymax, dxwidth, dywidth;
  private Rectangle titleRect = new Rectangle();
  // private Rectangle histRect = new Rectangle();
  protected Rectangle histRoundRect = new Rectangle();
  private Rectangle xRect = new Rectangle();
  private Rectangle yRect = new Rectangle();
  protected Rectangle yAxisRect = new Rectangle();
  protected Rectangle tagRect = new Rectangle();
  private Rectangle zoomRect = new Rectangle();
  private int changeScaleWidth;
  private int fLeft, fRight, fTop, fBottom;
  private int fLeftPixel, fRightPixel, fTopPixel, fBottomPixel;
  private boolean paintCursorMarker = false;
  protected boolean errWindowOn = false;
  protected boolean ScaleEnable = false; // yScaleArray on-off
  protected boolean ReferenceEnable = false;
  // protected int xleft, xright, dxwidth;
  protected double[][] bufferedMin = new double[AcopConst.lengthZoom][3];
  protected double[][] bufferedMax = new double[AcopConst.lengthZoom][3];
  protected int zoomIndex, zoomLogFlag;
  protected boolean isZoomed;
  protected boolean zoomFlag, movie; // zoom is on?
  protected Point moviepoint;
  protected int saveBestFlag;
  protected double moviePercent = 0.01;
  protected int movieLogCnt = 10;
  protected int xCnt = 0;
  protected int yCnt = 0;
  // protected double s_xmin, s_xmax, s_ymin, s_ymax; // user display rectangle
  protected int time_scale;
  protected double tick_xmin, tick_xmax; // ticks area
  protected int m_histstart = 0; // reference index of base histogram
  protected double wrapxmin, wrapxmax; // wrap x range
  private Point mStart, mEnd, mousepoint;
  private double mouseConvert[] = new double[2];
  private int mouseIndex;
  protected AcopScale[] sca = new AcopScale[3];
  // private AcopLine gridLine = new AcopLine();
  protected AcopFrame aFrame = new AcopFrame();
  //protected AcopTransport transport = new AcopTransport(this);
  // private BasicGraphicsUtils bGraphic = new BasicGraphicsUtils();
  private List<AcopListener> acopListeners = new ArrayList<>();
//  private java.awt.Graphics2D g2dima;
  private String yZoomRatio, xZoomRatio, bufferedZoomRatio;
  protected int yzero;
  protected java.awt.Color foreColor, backColor;
  private boolean captionChanged;
  private java.awt.Font captionFont;
  private double lastTextx, lastTexty;
  private int nBackRegions;
  private boolean backgroundRegionFixed;
  private int backkgroundRegionType[] = new int[AcopConst.MAX_BACKGROUND_REGION];
  private Rectangle2D.Double[] backRegionUserRect = new Rectangle2D.Double[AcopConst.MAX_BACKGROUND_REGION]; // userinput
                                                                                                              // rectangle
  private Rectangle[] backRegionRect = new Rectangle[AcopConst.MAX_BACKGROUND_REGION]; // converted
                                                                                        // device
                                                                                        // rect
  private Color[] backRegionColor = new Color[AcopConst.MAX_BACKGROUND_REGION]; // userinput
                                                                                // color
  private BufferedImage[] backRegionImage = new BufferedImage[AcopConst.MAX_BACKGROUND_REGION]; // userinput
                                                                                                // images
  protected Color pTagColorArray[];
  protected int nElement;
  protected Color TagBackColor; // init with forecolor
  private long mousePreviousClickTime, previousButton;
  private Point rButtonPoint;
  protected double yReferenceArray[];
  protected int yRASize, yRAdataSize;
  protected double yWindowArray[];
  protected int yWASize, yWAdataSize;
  protected double e_xmin, e_xmax, e_ymin, e_ymax; // error window display
                                                    // rectangle
  protected BasicStroke markerStroke;
  protected int m_marker;
  protected int saveMarkerWidth;
  protected double xlabelRotateGrad, xlabelcos, xlabelsin;
  protected int xlabelQudrant, xlabelAxis;
  protected int printScreenPosition;
  private int averagingIndex, averagingLength, NumberAveragingData;
  private double totalWeight;
  private double[] averageWeight = new double[AcopConst.N_DEF_DISPLAY];
  private double[][] averagingBuffer = new double[AcopConst.N_DEF_DISPLAY][];
  protected BasicStroke cursorMarkerStroke;
  private Point mMovePoint = new Point(); // for mouse
  private Point mOffsetMovePoint = new Point(); // for traslated coordinate
  private int cursorStay, cursorMarkerFlag, cursorix, cursoriy, cursorwx, cursorwy;
  private double cursorXPercent, cursorYPercent;
  private double cursorXposition, cursorYposition;
  private boolean firstText;
  protected boolean printTextLine = false;
  protected boolean appendLineIndexInTextMode = true;
  protected int zoomXmaxFlag = 0;
  protected AcopPesist[] persist = new AcopPesist[AcopConst.MAX_PERSIST];
  protected java.lang.String XLeftLabel = "";
  protected java.lang.String XRightLabel = "";
  protected java.lang.String[] ylab;
  protected int ylabON=0, ylabLength=0, yLabelDisplayOffset=0, yLabelStart=0, yLabelInterval=1;
  private boolean textAntiAliasing = false;
  protected boolean weightStatistics = true;
  private boolean snapZoomToNearestHour = false;

  /**
   * If true the zooming region is snapped to the nearest hour when zooming with a mouse.
   * @param snapZoomToNearestHour
   */
  public void setSnapZoomToNearestHour(boolean snapZoomToNearestHour) {
    if (this.snapZoomToNearestHour == snapZoomToNearestHour) return;
    this.snapZoomToNearestHour = snapZoomToNearestHour;
    firePropertyChange("snapZoomToNearestHour",!snapZoomToNearestHour,snapZoomToNearestHour);
  }

  public boolean isSnapZoomToNearestHour() {
    return snapZoomToNearestHour;
  }
  public int printScreen()
  {
    return printScreen(0);
  }
  public int printScreen(int position)
  {
    PrinterJob printerJob = PrinterJob.getPrinterJob();
    PageFormat pageFormat = printerJob.defaultPage();
    printScreenPosition = (position < 0 || position > 2) ? 0 : position;
    if (printerJob.pageDialog(pageFormat) != pageFormat)
    {
      if (printerJob.printDialog())
      {
        printerJob.setPrintable(this);
        try
        {
          printerJob.print();
        }
        catch (PrinterException e)
        {
          // Auto-generated catch block
          e.printStackTrace();
        }
      }
      else
        System.out.println("PrintDialog was cancelled");
    }
    return 0;
  }
  public int print(Graphics printG, PageFormat pageFormat, int pageIndex)
  {
    if (pageIndex == 0)
    {
      Graphics2D g = (Graphics2D) printG;
      double x1, x2, y1, y2, tmp, scale, offsetX, offsetY;
      x1 = pageFormat.getImageableWidth();
      x2 = this.getSize().width;
      y1 = pageFormat.getImageableHeight();
      y2 = this.getSize().height;
      tmp = x1 / x2;
      scale = y1 / y2;
      offsetX = 0;
      offsetY = 0;
      if (printScreenPosition != 0)
      {
        if (tmp < scale)
        {
          offsetY = y1 - y2 * tmp;
          if (printScreenPosition == 1) offsetY /= 2;
        }
        else
        {
          offsetX = x1 - x2 * scale;
          if (printScreenPosition == 1) offsetX /= 2;
        }
      }
      if (tmp < scale) scale = tmp;
      double xMargin = (pageFormat.getImageableWidth() - this.getSize().width * scale) / 2 + offsetX;
      double yMargin = (pageFormat.getImageableHeight() - this.getSize().height * scale) / 2 + offsetY;
      g.translate(pageFormat.getImageableX() + xMargin, pageFormat.getImageableY() + yMargin);
      g.scale(scale, scale);
      paint(g);
      return PAGE_EXISTS;
    }
    else
      return NO_SUCH_PAGE;
  }
  public int setBackgroundRegion(Rectangle2D.Double[] colorRegionRect, Color[] regionColor, boolean fixed)
  {
    return setBackgroundRegion(colorRegionRect, regionColor, null, null, fixed);
  }
  public int setBackgroundRegion(Rectangle2D.Double[] imageRegionRect, String[] imageFilename, boolean fixed)
  {
    return setBackgroundRegion(null, null, imageRegionRect, imageFilename, fixed);
  }
  public int setBackgroundRegion(Rectangle2D.Double[] colorRegionRect, Color[] regionColor,
      Rectangle2D.Double[] imageRegionRect, String[] imageFilename, boolean fixed)
  {
    int numColorRegion, numImageRegion, numRegion, colorLength, imageLength;
    if ((colorRegionRect == null && imageRegionRect == null) || // no rectangle
        (regionColor == null && imageFilename == null))
    {
      nBackRegions = 0;
      return 0;
    }
    numColorRegion = (colorRegionRect != null && regionColor != null) ? colorRegionRect.length : 0;
    numImageRegion = (imageRegionRect != null && imageFilename != null) ? imageRegionRect.length : 0;
    if (numColorRegion <= 0 && numImageRegion <= 0)
    {
      nBackRegions = 0;
      return 0;
    }
    colorLength = regionColor != null ? regionColor.length : 0;
    imageLength = imageFilename != null ? imageFilename.length : 0;
    numRegion = numColorRegion + numImageRegion;
    if (numRegion > AcopConst.MAX_BACKGROUND_REGION) numRegion = AcopConst.MAX_BACKGROUND_REGION;
    for (int i = 0; i < numRegion; i++) // load user region and color array or
                                        // images
    {
      backRegionUserRect[i] = i < numColorRegion ? colorRegionRect[i] : imageRegionRect[i - numColorRegion];
      if (backRegionUserRect[i] == null)
      {
        backkgroundRegionType[i] = 0;
        continue;
      }
      if (i < numColorRegion)
      {
        backRegionColor[i] = i < colorLength ? regionColor[i] : regionColor[i - 1];
        backkgroundRegionType[i] = 1;
      }
      else
      {
        backkgroundRegionType[i] = 2;
        if (i - numColorRegion >= imageLength)
          backRegionImage[i] = backRegionImage[i - 1];
        else {
        	try (InputStream in = new FileInputStream(imageFilename[i - numColorRegion]))
            {
              backRegionImage[i]  = ImageIO.read(in);
//              JPEGImageDecoder decoder = JPEGCodec.createJPEGDecoder(in);
//              backRegionImage[i] = decoder.decodeAsBufferedImage();
            }
            catch (IOException e)
            {
              e.printStackTrace();
              throw new RuntimeException("Could not load " + backRegionImage[i] + " with JPEGImageDecoder",e);
            }
        }

      }
    }
    backgroundRegionFixed = fixed;
    if (numRegion > 0)
    {
      nBackRegions = numRegion;
      convertBackgroundRegion(true);
    }
    return numRegion;
  }
  private void convertBackgroundRegion(boolean fourceConvert)
  {
    int xpos, xlength, ypos, ylength;
    if (nBackRegions == 0) return;
    if (!fourceConvert && backgroundRegionFixed) return;
    for (int i = 0; i < nBackRegions; i++) // convert
    {
      if (backkgroundRegionType[i] == 0) continue;
      ylength = (int) (aFrame.histRect.height * backRegionUserRect[i].height / sca[1].dispSize);
      ypos = (int) (aFrame.histRect.y - ylength + aFrame.histRect.height
          * (1. - (backRegionUserRect[i].y - sca[1].min) / sca[1].dispSize));
      xpos = (int) (aFrame.histRect.x + aFrame.histRect.width * (backRegionUserRect[i].x - sca[0].min)
          / sca[0].dispSize);
      xlength = (int) (aFrame.histRect.width * backRegionUserRect[i].width / sca[0].dispSize);
      backRegionRect[i] = new Rectangle(xpos, ypos, xlength, ylength);
    }
  }
  public int setDisplayGap(int pixel)
  {
    return setDisplayGap(pixel, pixel);
  }
  public int setDisplayGap(int xPixel, int yPixel)
  {
    return setDisplayGap(xPixel, xPixel, yPixel, yPixel);
  }
  public int setDisplayGap(int xLeftPixel, int xRightPixel, int yTopPixel, int yBottomPixel)
  {
    if (xLeftPixel < 0 || xLeftPixel >= aFrame.histRect.width / 2) xLeftPixel = 0;
    if (xRightPixel < 0 || xRightPixel >= aFrame.histRect.width / 2) xRightPixel = 0;
    if (yTopPixel < 0 || yTopPixel >= aFrame.histRect.height / 2) yTopPixel = 0;
    if (yBottomPixel < 0 || yBottomPixel >= aFrame.histRect.height / 2) yBottomPixel = 0;
    fLeftPixel = xLeftPixel;
    fRightPixel = xRightPixel;
    fTopPixel = yTopPixel;
    fBottomPixel = yBottomPixel;
    scaleFrameDraw();
    return xLeftPixel;
  }
  protected void averagingScaling(double df[], int start, int stop, int index)
  {
    int i, j, max;
    double maximum;
    if (averagingDepth > 0) // averaging
    {
      if (averagingLength == 0)
        NumberAveragingData = histo[index].npoints;
      else if (++averagingIndex == averagingDepth) averagingIndex = 0;
      if (averagingLength < averagingDepth)
      {
        if (averagingBuffer[averagingIndex] == null || averagingBuffer[averagingIndex].length < NumberAveragingData)
          averagingBuffer[averagingIndex] = new double[NumberAveragingData];
        averagingLength++;
        // calculate averaging exponential function
        totalWeight = 0;
        for (i = 0; i < averagingLength; i++)
          totalWeight += averageWeight[i];
      }
      max = histo[index].npoints < NumberAveragingData ? histo[index].npoints : NumberAveragingData;
      System.arraycopy(df, 0, averagingBuffer[averagingIndex], 0, max);
      if (averagingLength > 1) for (j = start; j < stop; j++)
      {
        if (j >= max) break;
        maximum = 0;
        for (i = 0; i < averagingLength; i++)
        {
          if (averagingMode == 0)
            maximum += averagingBuffer[i][j];
          else
            maximum += (averagingBuffer[i][j] * averageWeight[i]);
        }
        df[j] = maximum / (averagingMode == 0 ? averagingLength : totalWeight);
      }
    }
    if (yWAdataSize > 0 && histo[index].yScaleEnable)
    {
      max = histo[index].npoints < yWAdataSize ? histo[index].npoints : yWAdataSize;
      for (i = start; i < stop && i < max; i++)
        df[i] *= yWindowArray[i];
    }
    if (yRAdataSize > 0 && histo[index].yReferenceEnable)
    {
      max = histo[index].npoints < yRAdataSize ? histo[index].npoints : yRAdataSize;
      for (i = start; i < stop && i < max; i++)
        df[i] -= yReferenceArray[i];
    }
  }
  /**
   * Applies a weight function to the acop chart which can be
   * used by any or all plots.
   *
   * The weight function will be array-multiplied against the
   * y-axis data if the weighting criterion is turned on for a
   * given plot.
   *
   * @param weightArray is an array object containing the
   * weight values used in the array multiplication of the plotted
   * data.
   *
   * @return 0 upon success.
   */
  public int setWeightFunction(Object WeightArray)
  {
    return setWeightFunction(WeightArray, 0);
  }
  /**
   * @deprecated use setWeightFunction
   *
   * @param WeightArray
   * @return
   */
  @Deprecated
  public int weightFunction(Object WeightArray)
  {
    return setWeightFunction(WeightArray, 0);
  }
  /**
   * @deprecated use setWeightFunction
   *
   * @param WeightArray
   * @param ArraySize
   * @return
   */
  @Deprecated
  public int weightFunction(Object WeightArray, int ArraySize)
  {
    return setWeightFunction(WeightArray,ArraySize);
  }
  /**
   * Applies a weight function to the acop chart which can be
   * used by any or all plots.
   *
   * The weight function will be array-multiplied against the
   * y-axis data if the weighting criterion is turned on for a
   * given plot.
   *
   * @param weightArray is an array object containing the
   * weight values used in the array multiplication of the plotted
   * data.
   * @param length is the length of the weight array data to
   * use in the weighting multiplication.
   *
   * @return 0 upon success.
   */
  public int setWeightFunction(Object weightArray, int length)
  {
    int ndata;
    ndata = AcopHisto.getArrayLength(weightArray);
    if (ndata <= 0) return -1;
    if (length > 0 && length < ndata) ndata = length;
    if (yWindowArray == null || yWASize < ndata)
    {
      yWindowArray = new double[ndata];
      yWASize = ndata;
    }
    AcopHisto.copyArray(yWindowArray, weightArray, 0, ndata, ndata, 1);
    yWAdataSize = ndata;
    return 0;
  }
  /**
   * An array passed in the draw() method can also be plotted against
   * a reference array function supplied by the referenceFunction() method.
   *
   * @param referenceArray is the array to use as a reference for any
   * reference plots
   * @return 0 upon success
   *
   * @see setReferenced
   */
  public int setReferenceFunction(Object referenceArray)
  {
    return setReferenceFunction(referenceArray, 0);
  }
  /**
   * @deprecated use setReferenceFunction instead
   *
   * @param ReferenceArray
   * @return
   */
  @Deprecated
  public int referenceFunction(Object ReferenceArray)
  {
    return setReferenceFunction(ReferenceArray, 0);
  }
  /**
   * @deprecated use setReferenceFunction instead
   *
   * @param ReferenceArray
   * @param ArraySize
   * @return
   */
  @Deprecated
  public int referenceFunction(Object ReferenceArray, int ArraySize)
  {
    return setReferenceFunction(ReferenceArray, ArraySize);
  }
  /**
   * An array passed in the draw() method can also be plotted against
   * a reference array function supplied by the referenceFunction() method.
   *
   * @param referenceArray is the array to use as a reference for any
   * reference plots
   * @param length is the length of the reference array to apply.
   * @return
   */
  public int setReferenceFunction(Object referenceArray, int length)
  {
    int ndata;
    ndata = AcopHisto.getArrayLength(referenceArray);
    if (ndata <= 0) return -1;
    if (length > 0 && length < ndata) ndata = length;
    if (yReferenceArray == null || yRASize < ndata)
    {
      yReferenceArray = new double[ndata];
      yRASize = ndata;
    }
    AcopHisto.copyArray(yReferenceArray, referenceArray, 0, ndata, ndata, 1);
    yRAdataSize = ndata;
    return 0;
  }
  /**
   * Turns error color display on or off. This method applies to
   * all plots on the acop chart.
   *
   * @param value is the desired error window display state
   *
   * @see setErrorWindow, setErrorColor
   */
  public void applyErrorWindow(boolean value)
  {
    isErrorWindowOn(-1, value ? 1 : 0);
  }
  /**
   * Turns error color display on or off
   *
   * @param hDisplay is the display handle referring to the plot in question.
   * This is the return value of the original call
   * to the draw() method.
   * @param value is the desired error window display state
   *
   * @see setErrorWindow, setErrorColor
   */
  public void applyErrorWindow(int hDisplay,boolean value)
  {
    isErrorWindowOn(hDisplay, value ? 1 : 0);
  }
  /**
   * @return the current state of the error window display
   *
   * @see setErrorWindow, setErrorColor
   */
  public boolean isErrorWindowApplied()
  {
    return isErrorWindowOn(-1, -1);
  }
  /**
   * @param hDisplay is the display handle referring to the plot in question.
   * This is the return value of the original call
   * to the draw() method.
   * @return the current state of the error window display for the plot
   * specified.
   *
   * @see setErrorWindow, setErrorColor
   */
  public boolean isErrorWindowApplied(int hDisplay)
  {
    return isErrorWindowOn(hDisplay, -1);
  }
  public boolean isErrorWindowOn()
  {
    return isErrorWindowOn(-1, -1);
  }
  public boolean isErrorWindowOn(int hDisplay)
  {
    return isErrorWindowOn(hDisplay, -1);
  }
  public boolean isErrorWindowOn(boolean turnON)
  {
    return isErrorWindowOn(-1, turnON ? 1 : 0);
  }
  public boolean isErrorWindowOn(int hDisplay, boolean turnON)
  {
    return isErrorWindowOn(hDisplay, turnON ? 1 : 0);
  }
  private boolean isErrorWindowOn(int hDisplay, int value)
  {
    boolean arayon;
    if (hDisplay < -1 || hDisplay >= n_display) return false; // out of range,
                                                              // -1 to set for
                                                              // default value
    if (value < 0 || value > 1) // reading
      return (hDisplay == -1 || histo[hDisplay] == null) ? errWindowOn : histo[hDisplay].errWindowEnable;
    arayon = value != 0 ? true : false;
    if (hDisplay != -1)
    {
      if (histo[hDisplay] == null) return false;
      histo[hDisplay].errWindowEnable = arayon;
      return histo[hDisplay].errWindowEnable;
    }
    errWindowOn = arayon; // set default value and all the rest
    for (int i = 0; i < n_display; i++)
    {
      if (histo[i] == null) break;
      histo[i].errWindowEnable = arayon;
    }
    return arayon;
  }
  /**
   * Returns the current display criterion concerning whether or not
   * the display is plotted using a weight function or not.
   *
   * An array passed in the draw() method can also be plotted using a
   * a weight array function supplied by the weightFunction() method.
   * This will array multiply the contents of the plotted array with
   * the weight function array.
   *
   * @param hDisplay is the display handle referring to the plot
   * in question. This is the return value of the original call
   * to the draw() method.
   *
   * @return whether or not the designated plot is plotted against a
   * weight function.
   */
  public boolean isWeighted(int hDisplay)
  {
    return isWeighted(hDisplay, -1);
  }
  /**
   * Sets the weighted criterion.
   *
   * An array passed in the draw() method can also be plotted using
   * a weight array function supplied by the weightFunction() method.
   * This will array multiply the contents of the plotted array with
   * the weight function array.
   *
   * @param hDisplay is the display handle referring to the plot
   * in question. This is the return value of the original call
   * to the draw() method.
   * @param value is the desired weighted state
   *
   */
  public void setWeighted(int hDisplay,boolean value)
  {
    isWeighted(hDisplay,value ? 1 : 0);
  }
  public boolean isWeighted(int hDisplay, int value)
  {
    boolean arayon;
    if (hDisplay < -1 || hDisplay >= n_display) return false; // out of range,
                                                              // -1 to set for
                                                              // default value
    if (value < 0 || value > 1) // reading
      return (hDisplay == -1 || histo[hDisplay] == null) ? ScaleEnable : histo[hDisplay].yScaleEnable;
    arayon = value != 0 ? true : false;
    if (hDisplay != -1)
    {
      if (histo[hDisplay] == null) return false;
      histo[hDisplay].yScaleEnable = arayon;
      return histo[hDisplay].yScaleEnable;
    }
    ScaleEnable = arayon; // set default value and all the rest
    for (int i = 0; i < n_display; i++)
    {
      if (histo[hDisplay] == null) break;
      histo[hDisplay].yScaleEnable = arayon;
    }
    return arayon;
  }
  /**
   * Returns the current display criterion concerning whether or not
   * the display is plotted versus a reference function or not.
   *
   * An array passed in the draw() method can also be plotted against
   * a reference array function supplied by the referenceFunction() method.
   *
   * @param hDisplay is the display handle referring to the plot
   * in question. This is the return value of the original call
   * to the draw() method.
   *
   * @return whether or not the designated plot is plotted against a
   * reference function.
   */
  public boolean isReferenced(int hDisplay)
  {
    return isReferenced(hDisplay, -1);
  }
  /**
   * Sets the reference criterion.
   *
   * An array passed in the draw() method can also be plotted against
   * a reference array function supplied by the referenceFunction() method.
   *
   * @param hDisplay is the display handle referring to the plot
   * in question. This is the return value of the original call
   * to the draw() method.
   * @param value is the desired reference state
   *
   */
  public void setReferenced(int hDisplay,boolean value)
  {
    isReferenced(hDisplay,value ? 1 : 0);
  }
  public boolean isReferenced(int hDisplay, int value)
  {
    boolean arayon;
    if (hDisplay < -1 || hDisplay >= n_display) return false; // out of range,
                                                              // -1 to set for
                                                              // default value
    if (value < 0 || value > 1)
      // reading
      return (hDisplay == -1 || histo[hDisplay] == null) ? ReferenceEnable : histo[hDisplay].yReferenceEnable;
    arayon = value != 0 ? true : false;
    if (hDisplay != -1)
    {
      if (histo[hDisplay] == null) return false;
      histo[hDisplay].yReferenceEnable = arayon;
      return histo[hDisplay].yReferenceEnable;
    }
    ReferenceEnable = arayon; // set default value and all the rest
    for (int i = 0; i < n_display; i++)
    {
      if (histo[i] == null) break;
      histo[i].yReferenceEnable = arayon;
    }
    return arayon;
  }
  /**
   * Call this method to set the zooming state of an acop chart just as if a
   * user zoomed the chart from a GUI panel.
   *
   * @param zoomXMax is a boolean flag specifying whether the horizontal maximum
   * is to be zoomed or not.
   * @param zoomXMin is a boolean flag specifying whether the horizontal minimum
   * is to be zoomed or not.
   * @param zoomYMax is a boolean flag specifying whether the vertical maximum
   * is to be zoomed or not.
   * @param zoomYMin is a boolean flag specifying whether the vertical minimum
   * is to be zoomed or not.
   * @param zoomSecondaryYMax is a boolean flag specifying whether the secondary vertical maximum
   * is to be zoomed or not.
   * @param zoomSecondaryYMin is a boolean flag specifying whether the secondary vertical minimum
   * is to be zoomed or not.
   * @param newXMax is the new 'zoomed' horizontal maximum value.
   * @param newXMin is the new 'zoomed' horizontal minimum value.
   * @param newYMax is the new 'zoomed' vertical maximum value.
   * @param newYmin is the new 'zoomed' vertical minimum value.
   * @param newSecondaryYMax is the new 'zoomed' secondary axis vertical maximum value.
   * @param newSecondaryYMin is the new 'zoomed' secondary axis vertical minimum value.
   */
  public void SetZoom(boolean zoomXMax, boolean zoomXMin, boolean zoomYMax, boolean zoomYMin,
      boolean zoomSecondaryYMax, boolean zoomSecondaryYMin, double newXMax, double newXMin,
      double newYMax, double newYmin, double newSecondaryYMax, double newSecondaryYMin)
  {
    int dezoomFlag = 0, i;
    double[] storemin = new double[3];
    double[] storemax = new double[3];
    if (histo[m_histstart] == null || !aFrame.drawFlag) return;
    if (!zoomXMax) newXMax = sca[0].getMax();
    if (!zoomXMin) newXMin = sca[0].getMin();
    if (!zoomYMax) newYMax = sca[1].getMax();
    if (!zoomYMin) newYmin = sca[1].getMin();
    if (!zoomSecondaryYMax) newSecondaryYMax = sca[2].getMax();
    if (!zoomSecondaryYMin) newSecondaryYMin = sca[2].getMin();
    if (newXMin >= newXMax)
    {
      newXMin = sca[0].usermin;
      newXMax = sca[0].usermax;
    }
    if (newYmin >= newYMax)
    {
      newYmin = sca[1].usermin;
      newYMax = sca[1].usermax;
    }
    if (newSecondaryYMin >= newSecondaryYMax)
    {
      newSecondaryYMin = sca[2].usermin;
      newSecondaryYMax = sca[2].usermax;
    }
    if ((zooming & 0x4) != 0)
    {
      dezoomFlag = 0;
      if (zoomIndex > 0) // compare size
      {
        for (i = 0; i < 3; i++)
        {
          storemin[i] = sca[i].getMin();
          storemax[i] = sca[i].getMax();
        }
        if (Math.abs((newXMax - newXMin) - (storemax[0] - storemin[0])) < AcopConst.zero_check
            && Math.abs((newYMax - newYmin) - (storemax[1] - storemin[1])) < AcopConst.zero_check
            && Math.abs((newSecondaryYMax - newSecondaryYMin) - (storemax[2] - storemin[2])) < AcopConst.zero_check) return;
        if ((newXMax - newXMin) > (storemax[0] - storemin[0]) || (newYMax - newYmin) > (storemax[1] - storemin[1]) || (newSecondaryYMax - newSecondaryYMin) > (storemax[2] - storemin[2]))
        {
          dezoomFlag = 1;
          if (zoomIndex == 1) // finish zoom
          {
            newXMin = sca[0].usermin;
            newXMax = sca[0].usermax;
            newYmin = sca[1].usermin;
            newYMax = sca[1].usermax;
            newSecondaryYMin = sca[2].usermin;
            newSecondaryYMax = sca[2].usermax;
          }
        }
      }
      if (dezoomFlag == 0 && zoomIndex < AcopConst.lengthZoom)
      {
        for (i = 0; i < 3; i++)
        {
          storemin[i] = sca[i].getMin();
          storemax[i] = sca[i].getMax();
        }
      }
    }
    if ( zoomXmaxFlag != 2 ) zoomXmaxFlag = newXMax < sca[0].usermax ? 2 : 1;
    sca[0].setDispscale(newXMin, newXMax);
    sca[1].setDispscale(newYmin, newYMax);
    sca[2].setDispscale(newSecondaryYMin, newSecondaryYMax);
    if (sca[0].isOriginMaxMin() && sca[1].isOriginMaxMin())
    {
      isZoomed = false;
      zoomXmaxFlag = 0;
      if ((zooming & 0x4) != 0) zoomIndex = 0;
    }
    else
    {
      isZoomed = true;
      if ((zooming & 0x4) != 0)
      {
        if (dezoomFlag != 0)
        {
          zoomIndex--;
          for (i = 0; i < 3; i++)
          {
            sca[i].setDispscale(bufferedMin[zoomIndex][i], bufferedMax[zoomIndex][i]);
            if (sca[i].log != 0) sca[i].setDispscale(Math.pow(10.0, sca[i].min), Math.pow(10.0, sca[i].max));
          }
          bufferedZoomRatio = "Z" + zoomIndex;
        }
        else if (zoomIndex < AcopConst.lengthZoom)
        {
          for (i = 0; i < 3; i++)
          {
            bufferedMin[zoomIndex][i] = storemin[i];
            bufferedMax[zoomIndex][i] = storemax[i];
            if ( zoomIndex > 0 && sca[i].log != 0 )
            {
              if ( storemin[i] > 0 ) bufferedMin[zoomIndex][i] = Math.log(storemin[i]) / Math.log(10);
              if ( storemax[i] > 0 ) bufferedMax[zoomIndex][i] = Math.log(storemax[i]) / Math.log(10);
            }
          }
          zoomIndex++;
          bufferedZoomRatio = "Z" + zoomIndex;
        }
      }
    }
    zoomFlag = false;
    sca[0].scaleToGrid();
    if (m_marker == 2) setmarkerflag(2); // recalculate marker array
      // prepareDraw(TRUE, -1, FALSE, nScaleText); // rescale and not set max
      // limit
    rescaleFlag = 1;
    rescaleTextFlag = AcopConst.NEW_ZOOM_WINDOW;
    prepareDraw(); // rescale and not set max limit
    yZoomRatio = "V" + (int) (sca[1].userSize / sca[1].dispSize + 0.5) + ":1";
    xZoomRatio = "H" + (int) (sca[0].userSize / sca[0].dispSize + 0.5) + ":1";
  }
  /**
   * Call this method to set the zooming state of an acop chart just as if a
   * user zoomed the chart from a GUI panel.
   *
   * @param zoomXMax is a boolean flag specifying whether the horizontal maximum
   * is to be zoomed or not.
   * @param zoomXMin is a boolean flag specifying whether the horizontal minimum
   * is to be zoomed or not.
   * @param zoomYMax is a boolean flag specifying whether the vertical maximum
   * is to be zoomed or not.
   * @param zoomYMin is a boolean flag specifying whether the vertical minimum
   * is to be zoomed or not.
   * @param newXMax is the new 'zoomed' horizontal maximum value.
   * @param newXMin is the new 'zoomed' horizontal minimum value.
   * @param newYMax is the new 'zoomed' vertical maximum value.
   * @param newYMin is the new 'zoomed' vertical minimum value.
   */
  public void SetZoom(boolean zoomXMax, boolean zoomXMin, boolean zoomYMax, boolean zoomYMin, double newXMax, double newXMin,
      double newYMax, double newYMin)
  {
    SetZoom(zoomXMax, zoomXMin, zoomYMax, zoomYMin, false, false, newXMax, newXMin, newYMax, newYMin, 0, 0);
  }

  /**
   * Change the current vertical zoom boundaries without changing any zoom state.
   * @param primary true if the primary axis should be adjusted
   * @param secondary true if the secondary axis should be adjusted
   * @param min the new primary minimum
   * @param max the new primary maximum
   * @param secondaryMin the new secondary minimum
   * @param secondaryMax the new secondary maximum
   */
  public void adjustYZoom(boolean primary, boolean secondary, double min, double max, double secondaryMin, double secondaryMax) {
    if (zoomIndex == 0) return;
    if (!primary && !secondary) return;
    if (primary) {
      if (sca[1].log != 0) {
        sca[1].setDispscale(Math.pow(10.0, min), Math.pow(10.0, max));
      } else {
        sca[1].setDispscale(min,max);
      }
    }
    if (secondary) {
      if (sca[2].log != 0) {
        sca[2].setDispscale(Math.pow(10.0, secondaryMin), Math.pow(10.0, secondaryMax));
      } else {
        sca[2].setDispscale(secondaryMin,secondaryMax);
      }
    }
    rescaleFlag = 1;
    prepareDraw();

  }
  public int getScaledCoordinates(Object Coordinates)
  {
    int arySize, i; // return up to 6 values: Left, Right, Bottom, Top,
                    // SecondaryBottom, SecondaryTop
    if (Coordinates == null) return 0;
    arySize = AcopHisto.getArrayLength(Coordinates);
    if (arySize == 0) return 0;
    if (arySize > 6) arySize = 6;
    double[] tmpData = new double[arySize];
    for (i = 0; i < arySize; i++)
      tmpData[i] = (i % 2 == 0) ? sca[i / 2].getMin() : sca[i / 2].getMax();
    arySize = AcopHisto.copyArray(tmpData, Coordinates, 0, arySize, arySize, 0);
    return arySize;
  }
  /**
   * Retrieves the current data displayed on the chart and copies it into
   * the supplied array objects.
   *
   * The data drawn can be different from the arrays passed in the Draw()
   * method.  This is true for instance when an FFT or or Inversion or
   * fit has been applied, or a weightFunction() or referenceFunction() methods
   * has been applied.
   *
   * @param Yarray is an array object to hold the displayed vertical (y)
   * axis data.
   * @param Xarray is an array object to hold the displayed horizontal (x)
   * axis data.
   * @param length is the desired number of displayed data points to return.
   * @param offset is the offset in the displayed data array to begin retrieving
   * data.
   * @param hDisplay is the display handle referring to the plot to which the
   * data are to be retrieved.  This is the return value of the original call
   * to the draw() method.
   *
   * @return the number of points assigned to the y and x array objects passed.
   */
  public int getDrawnData(Object Yarray, Object Xarray, int length, int offset, int hDisplay)
  {
    if (hDisplay < 0 || hDisplay >= n_display || histo[hDisplay] == null) return -1;
    return histo[hDisplay].getDrawnData(Yarray, Xarray, length, offset);
  }
  /**
   * Retrieves the current data displayed on the chart and copies it into
   * the supplied array objects.
   *
   * @param Yarray is an array object to hold the displayed vertical (y)
   * axis data.
   * @param Xarray is an array object to hold the displayed horizontal (x)
   * axis data.
   * @param hDisplay is the display handle referring to the plot to which the
   * data are to be retrieved.  This is the return value of the original call
   * to the draw() method.
   * @param xpos is the horizontal (x) chart coordinates for which the data is
   * desired (the nearest data point will be select).  If the Yarray and Xarray
   * objects can hold more than 1 point then the returned data will begin at the
   * given x position.
   * @param ypos is the vertical (y) chart coordinates for which the data is
   * desired (the nearest data point will be select).  If the Yarray and Xarray
   * objects can hold more than 1 point then the returned data will begin at the
   * given y position.
   *
   * @return the number of points assigned to the y and x array objects passed.
   */
  public int getDrawnData(Object Yarray, Object Xarray, int hDisplay, double xpos, double ypos)
  {
    int dataIndex;
    if (hDisplay < 0 || hDisplay >= n_display || histo[hDisplay] == null) return -1;
    dataIndex = histo[hDisplay].getHistoIndex(xpos, ypos);
    return histo[hDisplay].getDrawnData(Yarray, Xarray, 1, dataIndex);
  }
  private void setlimit()
  {
    isZoomed = false; // new change!
    zoomXmaxFlag = 0;
    if ((zooming & 0x4) != 0) zoomIndex = 0;
    for (int i = 0; i < 3; i++)
      sca[i].setscale();
  }
  // setup HistoRect... ...
  private void scaleframe()
  {
    Rectangle r = getBounds();
    fLeft = frameWidth + frameLeftOffset; // init 4 offset
    fRight = frameWidth + frameRightOffset;
    fTop = frameWidth + frameTopOffset;
    fBottom = frameWidth + frameBottomOffset;
    if (fLeft < 0)
      fLeft = 0; // check x
    else if (fLeft >= r.width) fLeft = r.width - 1;
    if (fRight < 0)
      fRight = 0;
    else if ((fRight + fLeft) >= r.width) fRight = r.width - fLeft - 1;
    if (fTop < 0)
      fTop = 0; // check y
    else if (fTop >= r.height) fTop = r.height - 1;
    if (fBottom < 0)
      fBottom = 0;
    else if ((fBottom + fTop) >= r.height) fBottom = r.height - fTop - 1;
    // titleRect.setBounds(r.x+fLeft, r.y, r.width-fLeft-fRight, fTop);
    // titleRect.setBounds(fLeft, 0, r.width-fLeft-fRight, fTop);
    titleRect.setBounds(0, 0, r.width, fTop);
    // histRect.setBounds(r.x+fLeft, r.y+fTop, r.width-fLeft-fRight,
    // r.height-fTop-fBottom);
    // aFrame.histRect.setBounds(fLeft, fTop, r.width-fLeft-fRight,
    // r.height-fTop-fBottom);
    aFrame.histRect.setBounds(fLeft + fLeftPixel, fTop + fTopPixel, r.width - fLeft - fRight - fLeftPixel
        - fRightPixel, r.height - fTop - fBottom - fTopPixel - fBottomPixel);
    aFrame.zoomIcon.setBounds(aFrame.histRect.x + aFrame.histRect.width - aFrame.ZoomExtent,
        aFrame.histRect.y + 1, aFrame.ZoomExtent - 1, aFrame.ZoomExtent - 1);
    // histRoundRect.setBounds(aFrame.histRect.x - 1,aFrame.histRect.y -
    // 1,aFrame.histRect.width + 2,aFrame.histRect.height + 2);
    histRoundRect.setBounds(aFrame.histRect.x - fLeftPixel - 1, aFrame.histRect.y - fTopPixel - 1,
        aFrame.histRect.width + fLeftPixel + fRightPixel + 2, aFrame.histRect.height + fTopPixel
            + fBottomPixel + 2);
    // xRect.setBounds(r.x, r.y+r.height-fBottom, r.width, fBottom);
    xRect.setBounds(0, r.height - fBottom, r.width, fBottom);
    setTickRectangle(0);
    // yRect.setBounds(r.x, r.y, fLeft, r.height);
    yRect.setBounds(0, 0, fLeft, r.height);
    // yAxisRect.setBounds(r.x+r.width-fRight, r.y, fRight, r.height);
    yAxisRect.setBounds(r.width - fRight, 0, fRight, r.height);
    setTickRectangle(1);
    tagRect.setBounds(histRoundRect); // tagging

    listTextBox
        .setBounds(aFrame.histRect.x, aFrame.histRect.y, aFrame.histRect.width, aFrame.histRect.height);
  }
  // setup which rectangle used for ticks
  private void setTickRectangle(int i)
  {
    Rectangle tmpRect;
    if (sca[i].leftBottom) // take lower Rect for x and left Rect for y
    {
      sca[i].ticksRect = i == 0 ? xRect : yRect;
      tmpRect = i == 0 ? titleRect : yAxisRect;
    }
    else
    {
      sca[i].ticksRect = i == 0 ? titleRect : yAxisRect;
      tmpRect = i == 0 ? xRect : yRect;
    }
    if (i == 1) // secondasy Y ticks and label area
    {
      sca[2].ticksRect = tmpRect;
      sca[2].labelRect = sca[i].leftBottom != sca[i].leftBottLabel ? sca[i].ticksRect : tmpRect;
      sca[2].leftBottom = sca[i].leftBottom ? false : true;
      sca[2].leftBottLabel = sca[i].leftBottLabel ? false : true;
    }
    sca[i].labelRect = sca[i].leftBottom == sca[i].leftBottLabel ? sca[i].ticksRect : tmpRect;
  }
  private boolean drawOneHist(int num)
  {
    Graphics2D a = (Graphics2D) getGraphics();
    if (a == null) return false; // currently not displayable!!
    a.setClip(histRoundRect.x, histRoundRect.y, histRoundRect.width, histRoundRect.height);
    a.translate(aFrame.histRect.x, aFrame.histRect.y);
    histo[num].drawhist(a);
    a.translate(-aFrame.histRect.x, -aFrame.histRect.y);
    a.dispose();
    return true;
  }
  protected void prepareDraw()
  {
    double xtemp;
    int i, j, start, ende;
    if (randomData == true && sca[0].usermax > sca[0].usermin && sca[1].usermax > sca[1].usermin)
    {
      if (RandomInitialized == false)
      {
        for (i = 0; i < 512; i++)
          g_rdata[i] = Math.sin(i * 6.2832 / 512.);
        RandomInitialized = true;
      }
      xtemp = sca[0].userSize + 1; // one more point for the right edge!
      if (xtemp < 1.0) xtemp = 1.0;
      j = (int) xtemp;
      double[] ppvdata = new double[j];
      for (i = 0; i < j; i++)
        ppvdata[i] = sca[1].usermin + (1 + g_rdata[j >= 512 ? i % 512 : (int) (i * 512 / xtemp) % 512])
            * sca[1].userSize / 2;
      if (histo[0] == null)
      {
        histo[0] = new AcopHisto(this, 0);
      }
      if (histo[0].setHisto(ppvdata, null, null, null, 0, 0, foreColor) == 0)
      {
        if (wrapAround) histo[0].getMaxWindow();
        aFrame.drawFlag = true;
        histo[0].displayON = 1;
        histo[0].dismode = globalDisplayMode;
        histo[0].disDots = enableDots;
        histo[0].disBarHisto = enableBarlineHisto;
        histo[0].xext = XExtent;
        histo[0].yext = YExtent;
        histo[0].colorfilled = colorFilled;
        histo[0].barlineColorFilled = barlineColorFilled;
        loadXYindex = 0;
        lastDisp = 0;
      }
    }
    if (rescaleFlag == 2)
    {
      setlimit(); // load m_xmin... ...
      sca[0].scaleToGrid();
    }
    if (rescaleFlag > 0)
    {
      for (i = 0; i < 3; i++)
        sca[i].xyscale(rescaleFlag == 2 ? true : false); // set new min max(2007 hwu before was true)
//      convertBackgroundRegion(false);
    }
    // 2009-5-26
    if (rescaleFlag > 0 || drawTicks) convertBackgroundRegion(false);

    if (histIndex < 0 || histIndex >= n_display)
    {
      start = 0;
      ende = n_display;
    }
    else
    {
      start = histIndex;
      ende = histIndex + 1;
    }
    if (aFrame.drawFlag) for (i = start; i < ende; i++)
    {
      if (histo[i] == null) break;
      if (histo[i].displayON == 0 && i != m_histstart) continue;
      // if ( disp[n]->dismode >= AcopConst.mode_textbox ) continue;
      // scale screen: xleft, xright and dxwidth! getOxleftright(0);
      if (i == m_histstart) aFrame.getOxleftright(histo[i], sca[0]);
      // find wrapindex
      if (wrapAround && rescaleFlag != 0 && sca[0].log == 0) histo[i].setwrapindex();
      histo[i].setExtent();
      if (i == m_histstart)
      {
        if (histo[i].dismode == AcopConst.mode_textbox)
        {
          //listTextBox.removeAll();
          String allStmp = "";
          textArea.setText(allStmp);
        }
        else
          listTextBox.setVisible(false);
      }
      // map to screen coordinate
      histo[i].mapScreen(-1);
    }
    if (rescaleTextFlag > 0) for (i = 0; i < n_text; i++)
    {
      if (ptext[i] == null) break;
      ptext[i].rescaleText(rescaleTextFlag);
    }
    // if ( aFrame.drawFlag == false )
    // {
    // aFrame.xleft = 0;
    // aFrame.xright = aFrame.histRect.width;
    // aFrame.dxwidth =aFrame.xright - aFrame.xleft;
    // aFrame.absxleft = aFrame.xleft + aFrame.histRect.x;
    // return;
    // }
//    if (rescaleFlag > 0 && cursorStay > 0)
    if ((rescaleFlag > 0 || drawTicks) && cursorStay > 0)  // 2009-4-30
    {
      mMovePoint.x = sca[0].getScreenPosition(cursorXposition);
      mMovePoint.y = sca[1].getScreenPosition(cursorYposition);
    }
    if (sca[1].log != 0)
      yzero = aFrame.histRect.height;
    else if (sca[1].max >= 0 && sca[1].min <= 0)
      yzero = (int) (aFrame.histRect.height * (1 + sca[1].min / sca[1].dispSize));
    else
      yzero =  ((sca[1].max > 0 && sca[1].min > 0) ? aFrame.histRect.height : 0);
    if (repainHisto == true && drawTicks == false && rescaleFlag == 0 && rescaleTextFlag == 0
        && start + 1 == ende)
    {
      if (drawOneHist(start) == false) repaintHisto();
    }
    else if (rescaleFlag > 0 || drawTicks)
      repaint();
    // else if ( start == ende - 1 && repainHisto == false )
    // histo[start].drawhist(null);
    else
      repaintHisto();
    rescaleTextFlag = 0;
    rescaleFlag = 0;
    drawTicks = false;
    histIndex = -1;
    repainHisto = false;
  }
  private void setGlobalDisplayMode()
  {
    globalDisplayMode = displayMode;
    if ( displayMode > AcopConst.mode_textbox )
    {
      if ( displayMode == AcopConst.mode_polyline_dots ) globalDisplayMode = AcopConst.mode_polyline;
      if ( displayMode == AcopConst.mode_barline_dots ) globalDisplayMode = AcopConst.mode_barline;
      if ( displayMode == AcopConst.mode_histogram_dots ) globalDisplayMode = AcopConst.mode_histogram;
      if ( displayMode == AcopConst.mode_histogram_rastoring_dots ) globalDisplayMode = AcopConst.mode_histogram_rastoring;
      if ( displayMode != AcopConst.mode_barline_histo ) enableDots = 1;
      else enableDots = 0;
      if ( displayMode == AcopConst.mode_polyline_cross ) {
    	  globalDisplayMode = AcopConst.mode_polyline;
    	  enableDots = 2;
      }
      if ( displayMode == AcopConst.mode_barline_histo ||  displayMode == AcopConst.mode_barline_histo_dots )
      {
        globalDisplayMode = AcopConst.mode_barline;
        enableBarlineHisto = 1;
      } else if (displayMode == AcopConst.mode_annotation) {
    	  globalDisplayMode = AcopConst.mode_annotation;
    	  enableDots = 0;
      } else if (displayMode == AcopConst.mode_status) {
    	  globalDisplayMode = AcopConst.mode_status;
    	  enableDots = 0;
      }

      else enableBarlineHisto = 0;

      if ( displayMode == AcopConst.mode_barline_bit || displayMode == AcopConst.mode_barline_ganttplot )
      {
        globalDisplayMode = AcopConst.mode_barline;
        enableDots = 0;
      }
    }
    else
    {
      enableDots = 0;
      enableBarlineHisto = 0;
    }

  }
  
  public Acop() {
      this(true);
  }

  public Acop(boolean createAcopTransport)
  {
	  //setDoubleBuffered(false);
	// {{INIT_CONTROLS
    setLayout(null);
    setSize(502, 350);
    setPreferredSize(new Dimension(502, 350));
    setMinimumSize(new Dimension(100, 100));
    setPopupEnabled(true);
    addComponentListener(new ComponentAdapter() {
    	@Override
    	public void componentResized(ComponentEvent e) {
    		scaleFrameDraw();
    	}
    });
    addHierarchyBoundsListener(new HierarchyBoundsListener() {

		@Override
		public void ancestorResized(HierarchyEvent e) {
			scaleFrameDraw();
		}

		@Override
		public void ancestorMoved(HierarchyEvent e) {
			// Auto-generated method stub
		}
	});
    if (createAcopTransport) {
        setAcopTransport(new AcopTransport());
    }
    captionPosition = 1;
    captionChanged = true;
    keepColor = Color.gray;
    keepWidth = 1;
    keepStyle = AcopConst.PS_SOLID;
    m_tagColor = Color.cyan;
    m_tagging = false;
    pTagColorArray = null;
    nElement = 0;
    m_marker = 0;
    saveMarkerWidth = 0;
    m_markerColor = Color.black;
    m_markerMode = 0;
    m_markerStyle = AcopConst.PS_SOLID;
    m_markerWidth = 1;
    m_markerXExtent = -1;
    m_markerYExtent = -1;
    m_cursorMarkerColor = Color.black;
    rev_cursorMarkerColor = Color.white;
    m_cursorMarkerMode = 0;
    m_cursorMarkerStyle = AcopConst.PS_SOLID;
    m_cursorMarkerWidth = 1;
    cursorMarkerStroke = AcopConst.createStroke(m_cursorMarkerWidth, m_cursorMarkerStyle);
    m_cursorMarkerXExtent = -1;
    m_cursorMarkerYExtent = -1;
    mMovePoint.setLocation(-1, -1);
    mOffsetMovePoint.setLocation(-1, -1);
    cursorStay = 0;
    cursorXPercent = -1;
    cursorYPercent = -1;
    cursorMarkerFlag = 0;
    cursorix = 0;
    cursoriy = 0;
    cursorwx = 0;
    cursorwy = 0;
    m_errorColor = Color.red;
    colorFilled = true;
    averagingMode = 0;
    averagingDepth = 0;
    averagingIndex = 0;
    averagingLength = 0;
    NumberAveragingData = 0;
    for (int i = 0; i < AcopConst.N_DEF_DISPLAY; i++)
      averageWeight[i] = Math.exp(i);
    FFT = 0;
    printScreenPosition = 0;
    for (int i = 0; i < 3; i++)
    {
      if (i < 2) axis[i] = new AcopAxis(this, i);
      sca[i] = new AcopScale(this, i == 0 ? true : false, i == 2 ? true : false);
      // load pointer into AcopScale class
      sca[i].ticks = -1;
      sca[i].fineTicks = 1;
      sca[i].grid = false;
      sca[i].leftBottom = i < 2 ? true : false;
      sca[i].bestScale = true;  // 2007 hwu false;
      sca[i].ticktext = true;
      sca[i].log = 0;
      sca[i].xtime = 0;
      sca[i].tickLabelForeColor = Color.gray;
      sca[i].leftBottLabel = true;
      sca[i].OnFlag = i < 2 ? true : false;
      sca[i].setUserscale(i == 0 ? 0 : -100, 100);
      sca[i].labelOffsetMode = 0;
      sca[i].labelOffset = 0;
    }
    textArea = new JTextArea();
//  listTextBox = new java.awt.List();
    listTextBox = new JScrollPane(textArea);
    this.add(listTextBox);
    listTextBox.setVisible(false);
    nBackRegions = 0;
    changeScaleWidth = 0;
    invert = false;
    XShift = 0;
    XScale = 1;
    YShift = 0;
    YScale = 1;
    RandomInitialized = false;
    randomData = Beans.isDesignTime();
    randomText = Beans.isDesignTime();
    frameWidth = 30;
    frameLeftOffset = 10; // 2007 hwu
    frameRightOffset = 0;
    frameTopOffset = 0;
    frameBottomOffset = 0;
    graphStyle = 0;
    aFrame.leftTimeLabel = true;
    aFrame.absTimeDisplay = false;
    aFrame.histogramPosition = 1;
    aFrame.frameForeColor = Color.gray;
    aFrame.frameFont = new Font("MonoSpaced", Font.PLAIN, 12);
    caption = "test";
    displayMode = AcopConst.mode_polyline; // AcopConst.mode_histogram;
    setGlobalDisplayMode();
    bitDisplayMask = 0;
    ganttBitDisplayBase = 0;
    ganttBitDisplayHeight = 0.8;
    e_xmin = 0;
    e_xmax = 100;
    drawWidth = 1; // 1 pixel
    drawStyle = AcopConst.PS_SOLID;// DASH_DOT ;//AcopConst.PS_SOLID;
    e_ymin = -100;
    e_ymax = 100;
    // penLine = new BasicStroke(3);
    m_lock = 0;
    screenDepth = 20;
    n_display = screenDepth;
    textDepth = 20;
    n_text = textDepth; // copConst.MAX_TEXT;
    m_filled = 0;
    barlineColorFilled = 0;
    wrapxmin = -1.7e+308;
    wrapxmax = 1.7e+308; // wrap x range
    zoomIndex = 0;
    zoomLogFlag = 0;
    zooming = 7;  // 2007 hwo buffered zooming   3;
    isZoomed = false;
    zoomFlag = false;
    reuseScreen = false;
    reuseText = false;
    XExtent = 0;
    YExtent = 0;
    xlabelRotation = 0;
    foreColor = getForeground();
    backColor = getBackground();
    wrapAround = false;
    printTextStickyAttribute = 0;
    printTextLocation = 0;
    printTextRotation = 0;
    LeadingEdgeColor = getForeground();
    LeadingEdgeGapSize = 0;
    LeadingEdgeMotion = 0;
    LeadingEdgeMode = 0;
    LeadingEdgeWidth = 1;
    LeadingEdgeStyle = AcopConst.PS_SOLID;
    LeadingEdgeXExtent = -1;
    LeadingEdgeYExtent = -1;
    yRASize = yRAdataSize = 0;
    yWASize = yWAdataSize = 0;
    fLeft = 0;
    fRight = 0;
    fTop = 0;
    fBottom = 0;
    fLeftPixel = 0;
    fRightPixel = 0;
    fTopPixel = 0;
    fBottomPixel = 0;
    frameBorderOn = true;
    firstText = true;
    printTextLine = false;
    if ((zooming & 0x7) != 0) setZoomingWindow(zooming & 0x4);
    scaleFrameDraw();
    if (Beans.isDesignTime() && randomData == true)
    {
      prepareDraw();
      // System.out.println("DesignTime and RandomData");
    }
    // if ( randomData == true ) prepareDraw();
    enableEvents(AWTEvent.MOUSE_EVENT_MASK | AWTEvent.MOUSE_MOTION_EVENT_MASK | AWTEvent.COMPONENT_EVENT_MASK);
    // }}

  }
  // {{DECLARE_CONTROLS
  // }}

//  public void addPropertyChangeListener(String s, java.beans.PropertyChangeListener pcl)
//  {
//    System.out.println(" property change!!!" + s);
//    if ( s.equals("background") ) System.out.println("  background event!!!");
//    this.addPropertyChangeListener(
//        new PropertyChangeListener() {
//          public void propertyChange(PropertyChangeEvent e)
//            {
//                if (e.getPropertyName().equals("background")) {
//                  System.out.println("  background event!!!"); // Auto-generated Event stub AcopMouseClick()
//                }
//            }
//        }
//    );
//  }

  //  public void this. .addPropertyChangeListener("foreground", new PropertyChangeListener()
//      {
    //public void propertyChange(PropertyChangeEvent event)
    //{
    //}
//    });


  public int lockCursorMarker(boolean Enable)
  {
    return lockCursorMarker(Enable, false, 0, false, 0);
  }
  public int lockCursorMarker(boolean Enable, double Xposition, double Yposition)
  {
    return lockCursorMarker(Enable, true, Xposition, true, Yposition);
  }
  public int lockCursorMarker(boolean Enable, boolean xInput, double Xposition, boolean yInput,
      double Yposition)
  {
	  paintCursorMarker = true;
	  if (cursorLockedToCurrentScreenPosition) return cursorStay;
	  int on;
    if (m_cursorMarkerMode == 0) return -1; // has to turn mode on first!
    on = Enable ? 1 : 0;
    cursorStay = on;
    if (cursorStay > 0)
    {
      if (xInput)
        mMovePoint.x = sca[0].getScreenPosition(Xposition);
      else
        Xposition = sca[0].getUserPosition(mMovePoint.x);
      if (yInput)
        mMovePoint.y = sca[1].getScreenPosition(Yposition);
      else
        Yposition = sca[1].getUserPosition(mMovePoint.y);
      cursorXposition = Xposition;
      cursorYposition = Yposition;
    }

    if (aFrame.drawFlag) repaintHisto();
    return on;
  }
  private void drawCursorMarker(Graphics2D g2d, Point point)
  { int modeTmp;
  	if (!paintCursorMarker) return;
    Rectangle Cliprect = null;

    int initFlag = g2d == null ? 0 : 1;
    if (g2d == null) // called from mouse move, first map and clip, then remove
                      // previous line first
    {
      g2d = (Graphics2D) getGraphics();
      Cliprect = g2d.getClipBounds();
      if (Cliprect != null)
      {
        if (Cliprect.intersects(histRoundRect) == false) return;
      }
    }
    g2d.setXORMode(rev_cursorMarkerColor);
//    g2d.setXORMode(m_cursorMarkerColor);
    // g2d.setColor(m_cursorMarkerColor);
    g2d.setStroke(cursorMarkerStroke);
    if (initFlag == 0) // called from mouse move, first map and clip, then
                        // remove previous line first
    {
      g2d.clipRect(histRoundRect.x, histRoundRect.y, histRoundRect.width, histRoundRect.height);
      // g2d.translate(aFrame.histRect.x, aFrame.histRect.y);
      if (cursorMarkerFlag == 2) // already onscreen, repaint it without the cursor line
      {
        modeTmp = m_cursorMarkerMode;
        m_cursorMarkerMode = 0;
        paintImmediately(histRoundRect);
        m_cursorMarkerMode = modeTmp;
      }
      else if (cursorMarkerFlag != 0) // if already onscreen, remove previous line first
      {
        if ((m_cursorMarkerMode & 0x2) != 0)
          g2d.drawLine(cursorwy, cursorwx - cursoriy, cursorwy, cursorwx + cursoriy);
        if ((m_cursorMarkerMode & 0x1) != 0)
          g2d.drawLine(cursorwy - cursorix, cursorwx, cursorwy + cursorix, cursorwx);
      }
    }
    // calculate new position
    cursorwx = point.y;
    cursorwy = point.x;
    if (initFlag != 0)
    {
      cursorwx -= aFrame.histRect.y;  // 2009-1 bug fix
      cursorwy -= aFrame.histRect.x;
    }
    if ((m_cursorMarkerMode & 0x1) != 0)
    {
      if (initFlag != 0)
      {
        if (cursorXPercent < 0)
        {
          if (m_cursorMarkerXExtent < 0)
            cursorXPercent = 1;
          else
            cursorXPercent = ((sca[0].log != 0) ? Math.log(m_cursorMarkerXExtent) / Math.log(10)
                : m_cursorMarkerXExtent)
                / (sca[0].tick_max - sca[0].tick_min) / 2;
        }
        cursorix = (int) (aFrame.histRect.width * cursorXPercent);
      }
      g2d.drawLine(cursorwy - cursorix, cursorwx, cursorwy + cursorix, cursorwx);
    }
    if ((m_cursorMarkerMode & 0x2) != 0)
    {
      if (initFlag != 0)
      {
        if (cursorYPercent < 0)
        {
          if (m_cursorMarkerYExtent < 0)
            cursorYPercent = 1;
          else
            cursorYPercent = ((sca[1].log != 0) ? Math.log(m_cursorMarkerYExtent) / Math.log(10)
                : m_cursorMarkerYExtent)
                / sca[1].dispSize / 2;
        }
        cursoriy = (int) (aFrame.histRect.height * cursorYPercent);
      }
      g2d.drawLine(cursorwy, cursorwx - cursoriy, cursorwy, cursorwx + cursoriy);
    }
    g2d.setPaintMode();
    if (initFlag == 0)
    {
      // g2d.translate(-aFrame.histRect.x, -aFrame.histRect.y);
      g2d.setClip(Cliprect);
      g2d.dispose();
    }
    if (initFlag != 0)
    {
      cursorwx += aFrame.histRect.y;  // 2009-1 bug fix
      cursorwy += aFrame.histRect.x;
    }
    cursorMarkerFlag = initFlag == 1 ? 2 : 1; // cursor is on screen!
  }
  private void setZoomingWindow(int bufferedZoomFlag)
  {
    aFrame.ZoomExtent = bufferedZoomFlag != 0 ? 20 : 36;
    // zoom icon indicator
    aFrame.zoomIcon.setBounds(aFrame.histRect.x + aFrame.histRect.width - aFrame.ZoomExtent,
        aFrame.histRect.y + 1, aFrame.ZoomExtent - 1, aFrame.ZoomExtent - 1);
  }
  /**
   * Gets the text printed in the text field specified.
   *
   * The principal method for introducing printed text in an ACOP display is
   * through the printText() method. Text can also be altered through the
   * refreshText() method.  Precise text location is determined by
   * making use of the setPrintTextLocation and setPrintTextStickyAttribute
   * methods.
   *
   * @param leftBottom fills in the coordinates of the left-bottom origin
   * of the text field location on the chart.
   * Ignored if null.
   * @param size fills in the size of the text field location on the chart.
   * Ignored if null.
   * @param position fills in the assigned position of the text field.
   * Ignored if null.
   * @param hText is a handle to the text field for which the date is to be
   * retrieved.
   *
   * @return the text displayed in the text field given
   */
  public String getPrintedText(double[] leftBottom, double[] size, double[] position, int hText)
  {
    if (hText < 0 || hText >= n_text || ptext[hText] == null) return "Input Error!";
    return ptext[hText].getPrintedText(leftBottom, size, position);
  }
  public boolean isZoomed()
  {
    return this.isZoomed;
  }
  public int getHistogramOffsetLeft()
  {
    return aFrame.xleft;
  }
  public int getHistogramOffsetRight()
  {
    return aFrame.histRect.width - aFrame.xright;
  }
  public double getScaleLeft()
  {
    return sca[0].getMin();
  }
  public double getScaleRight()
  {
    return sca[0].getMax();
  }
  public double getScaleTop()
  {
    return sca[1].getMax();
  }
  public double getScaleBottom()
  {
    return sca[1].getMin();
  }
  /**
   * Returns the minimum data index visible in the currently displayed region. 
   * @param hDisplay the series/display for which to retrieve the minimum index 
   * @return the minimum index in the data array that is visible (inclusive)
   */
  public int getMinVisibleXIndex(int hDisplay)
  {
    if (hDisplay < 0 || hDisplay >= n_display) return -1;
    return histo[hDisplay].xMinIndex;
  }
  /**
   * Returns the maximum data index visible in the currently displayed region. 
   * @param hDisplay the series/display for which to retrieve the maximum index 
   * @return the maximum index in the data array that is visible (exclusive)
   */
  public int getMaxVisibleXIndex(int hDisplay)
  {
    if (hDisplay < 0 || hDisplay >= n_display) return -1;
    return histo[hDisplay].xMaxIndex;
  }
  private int loadAxis(int index, boolean AxisOn, Object PositionArray, int ArraySize, Color[] ColorArray,
      int[] StyleArray, int[] WidthArray)
  {
    int ret;
    ret = axis[index].axisON != AxisOn ? 1 : 0;
    axis[index].axisON = AxisOn;
    if (ArraySize < 1 && ArraySize != -1) return -1; // wrong input
    if (ArraySize != -1)
      ret += axis[index].loadData(ArraySize, PositionArray, ColorArray, StyleArray, WidthArray);
    if (ret != 0 && aFrame.drawFlag) repaintHisto();
    return 0;
  }
  public int xAxis(boolean AxisOn)
  {
    return loadAxis(0, AxisOn, null, -1, null, null, null);
  }
  public int yAxis(boolean AxisOn)
  {
    return loadAxis(1, AxisOn, null, -1, null, null, null);
  }
  public int xAxis(boolean AxisOn, Object PositionArray, int ArraySize, Color[] ColorArray, int[] StyleArray,
      int[] WidthArray)
  {
    return loadAxis(0, AxisOn, PositionArray, ArraySize, ColorArray, StyleArray, WidthArray);
  }
  public int yAxis(boolean AxisOn, Object PositionArray, int ArraySize, Color[] ColorArray, int[] StyleArray,
      int[] WidthArray)
  {
    return loadAxis(1, AxisOn, PositionArray, ArraySize, ColorArray, StyleArray, WidthArray);
  }
  /**
   * Replaces the displayed data with the contents of the input data
   * array for the display given.
   *
   * Using this method is an efficient way to update a displayed plot with
   * new data.  An alternative is to call one of the clearScreen()
   * methods followed by one of the draw(),
   * which would have the effect of replacing the entire
   * array of displayed data.
   *
   * @param Yarray is a data array object whose contents will be used to
   * update and refresh the same array portion of the displayed data for the
   * plot indicated.
   * @param Xarray is an array object containing the horizontal (x) axis
   * values associated with the elements in the Yarray object.
   * @param hDisplay is the display handle referring to the plot to which the
   * data are refreshed.  This is the return value of the original call
   * to the draw() method.
   * @param length is the length of the input Yarray and Xarray objects which
   * should be applied to the plot indicated
   * @param offset is the starting offset of the input Yarray and Xarray objects which
   * should be applied to the plot indicated
   *
   * @return the array index of the last updated array element.
   */
  public int refreshScreen(Object Yarray, Object Xarray, int hDisplay, int length, int offset)
  {
    if (hDisplay < 0 || hDisplay >= n_display || histo[hDisplay] == null) return -1;
    return histo[hDisplay].updateScreen(false, Yarray, length, offset, Xarray);
  }
  public int refreshScreen(Object Yarray, int hDisplay, int length, int offset, Object Xarray)
  {
    if (hDisplay < 0 || hDisplay >= n_display || histo[hDisplay] == null) return -1;
    return histo[hDisplay].updateScreen(false, Yarray, length, offset, Xarray);
  }
  /**
   * Replaces the displayed data with the contents of the input data
   * array for the display given.
   *
   * Using this method is an efficient way to update a displayed plot with
   * new data.  An alternative is to call one of the clearScreen()
   * methods followed by one of the draw(),
   * which would have the effect of replacing the entire
   * array of displayed data.
   *
   * @param Yarray is a data array object whose contents will be used to
   * update and refresh the same array portion of the displayed data for the
   * plot indicated.
   * @param Xarray is an array object containing the horizontal (x) axis
   * values associated with the elements in the Yarray object.
   * @param hDisplay is the display handle referring to the plot to which the
   * data are to be refreshed.  This is the return value of the original call
   * to the draw() method.
   *
   * @return the array index of the last updated array element.
   */
  public int refreshScreen(Object Yarray, Object Xarray, int hDisplay)
  {
    if (hDisplay < 0 || hDisplay >= n_display || histo[hDisplay] == null) return -1;
    return histo[hDisplay].updateScreen(false, Yarray, -1, -1, Xarray);
  }
  public int refreshScreen(Object Yarray, int hDisplay, Object Xarray)
  {
    if (hDisplay < 0 || hDisplay >= n_display || histo[hDisplay] == null) return -1;
    return histo[hDisplay].updateScreen(false, Yarray, -1, -1, Xarray);
  }
  /**
   * Replaces the displayed data with the contents of the input data
   * array for the display given.
   *
   * Using this method is an efficient way to update a displayed plot with
   * new data.  An alternative is to call one of the clearScreen()
   * methods followed by one of the draw(),
   * which would have the effect of replacing the entire
   * array of displayed data.
   *
   * @param Yarray is a data array object whose contents will be used to
   * update and refresh the same array portion of the displayed data for the
   * plot indicated.
   * @param hDisplay is the display handle referring to the plot to which the
   * data are to be refreshed.  This is the return value of the original call
   * to the draw() method.
   *
   * @return the array index of the last updated array element.
   */
  public int refreshScreen(Object Yarray, int hDisplay)
  {
    if (hDisplay < 0 || hDisplay >= n_display || histo[hDisplay] == null) return -1;
    if ( refreshPersistScreen(Yarray, hDisplay) == 0 ) return histo[hDisplay].npoints-1;
    return histo[hDisplay].updateScreen(false, Yarray, -1, -1, null);
  }
  /**
   * Appends the given elements to the 'end' of the plotted array.
   *
   * This method is useful for updating a trend chart.
   *
   * @param Yarray is an array containing the data point or points to append to
   * the end of the currently plotted data in the plot specified.
   * @param Xarray is an array containing the x-axis coordinate(s) paralleling the
   * YArray given.
   * @param hDisplay is the display handle referring to the plot to which the
   * data are to be appended.  This is the return value of the original call
   * to the draw() method.
   * @param length gives the length (i.e. number of array elements) to use from the
   * given Yarray and Xarray in appending the designated plot.
   * @param offset gives the starting index in the array elements to use from the
   * given Yarray and Xarray in appending the designated plot.
   *
   * @return if successful: the array element index of the last updated point.
   * On failure, a negative number is returned.
   */
  public int appendScreen(Object Yarray, Object Xarray, int hDisplay, int length, int offset)
  {
    if (hDisplay < 0 || hDisplay >= n_display || histo[hDisplay] == null) return -1;
    return histo[hDisplay].updateScreen(true, Yarray, length, offset, Xarray);
  }
  public int appendScreen(Object Yarray, int hDisplay, int Npoints, int StartIndex, Object Xarray)
  {
    if (hDisplay < 0 || hDisplay >= n_display || histo[hDisplay] == null) return -1;
    return histo[hDisplay].updateScreen(true, Yarray, Npoints, StartIndex, Xarray);
  }
  /**
   * Appends the given elements to the 'end' of the plotted array.
   *
   * This method is useful for updating a trend chart.
   *
   * @param Yarray is an array containing the data point or points to append to
   * the end of the currently plotted data in the plot specified.
   * @param Xarray is an array containing the x-axis coordinate(s) paralleling the
   * YArray given.
   * @param hDisplay is the display handle referring to the plot to which the
   * data are to be appended.  This is the return value of the original call
   * to the draw() method.
   *
   * @return if successful: the array element index of the last updated point.
   * On failure, a negative number is returned.
   */
  public int appendScreen(Object Yarray, Object Xarray, int hDisplay)
  {
    if (hDisplay < 0 || hDisplay >= n_display || histo[hDisplay] == null) return -1;
    return histo[hDisplay].updateScreen(true, Yarray, -1, -1, Xarray);
  }
  public int appendScreen(double Yvalue, double Xvalue, int hDisplay)
  {
    if (hDisplay < 0 || hDisplay >= n_display || histo[hDisplay] == null) return -1;
    return histo[hDisplay].updateScreen(true, new double[]{Yvalue}, -1, -1, new double[]{Xvalue});
  }
  public int appendScreen(Object Yarray, int hDisplay, Object Xarray)
  {
    if (hDisplay < 0 || hDisplay >= n_display || histo[hDisplay] == null) return -1;
    return histo[hDisplay].updateScreen(true, Yarray, -1, -1, Xarray);
  }
  /**
   * Appends the given elements to the 'end' of the plotted array.
   *
   * This method is useful for updating a trend chart.
   *
   * @param Yarray is an array containing the data point or points to append to
   * the end of the currently plotted data in the plot specified.
   * @param hDisplay is the display handle referring to the plot to which the
   * data are to be appended.  This is the return value of the original call
   * to the draw() method.
   *
   * @return if successful: the array element index of the last updated point.
   * On failure, a negative number is returned.
   */
  public int appendScreen(Object Yarray, int hDisplay)
  {
    if (hDisplay < 0 || hDisplay >= n_display || histo[hDisplay] == null) return -1;
    return histo[hDisplay].updateScreen(true, Yarray, -1, -1, null);
  }
  public int appendScreen(double value, int hDisplay)
  {
    if (hDisplay < 0 || hDisplay >= n_display || histo[hDisplay] == null) return -1;
    return histo[hDisplay].updateScreen(true, new double[]{value}, -1, -1, null);
  }
  /**
   * Draws a curve on the acop chart using the data provided and the chart display settings
   *
   * @param ydata is an array object containing the data points to be plotted on the vertical
   * axis. If ydata is not an array of a number type (e.g. a string array) then the draw method
   * while only succeed if the acop chart display mode is set to 'textbox'.
   * The horizontal (x) axis coordinates will be an array of monotonically
   * increasing integers.
   *
   * @return a positive display handle which refers to the plot shown on the chart.
   * This handle can be used in other acop API calls to for example refresh or clear
   * a particular plot.  Upon error, and negative value is returned.
   */
  public int draw(Object ydata)
  {
    return draw(ydata, null, null, null, 0, 0);
  }

  /**
   * Draws a curve on the acop chart using the data provided and the chart display settings
   *
   * @param ydata is an array object containing the data points to be plotted on the vertical
   * axis. If ydata is not an array of a number type (e.g. a string array) then the draw method
   * while only succeed if the acop chart display mode is set to 'textbox'.
   * @param xdata is an array object containing the horizontal (x) axis coordinates to associate with the
   * ydata data array.  In methods where this is optional, an array of monotonically
   * increasing integers is used as the horizontal axis (convenient for histograms).
   * @param bindToSecondaryAxis is a boolean flag which if 'true' fixes the display of the
   * ensuing plot to the settings of the secondary (y) axis parameters.
   *
   * @return a positive display handle which refers to the plot shown on the chart.
   * This handle can be used in other acop API calls to for example refresh or clear
   * a particular plot.  Upon error, and negative value is returned.
   */
  public int draw(Object ydata, Object xdata, boolean bindToSecondaryAxis) {
	  return draw(ydata, xdata, null, null, 0, 0,bindToSecondaryAxis);
  }
  /**
   * Draws a curve on the acop chart using the data provided and the chart display settings
   *
   * @param ydata is an array object containing the data points to be plotted on the vertical
   * axis. If ydata is not an array of a number type (e.g. a string array) then the draw method
   * while only succeed if the acop chart display mode is set to 'textbox'.
   * @param xdata is an array object containing the horizontal (x) axis coordinates to associate with the
   * ydata data array.  In methods where this is optional, an array of monotonically
   * increasing integers is used as the horizontal axis (convenient for histograms).
   *
   * @return a positive display handle which refers to the plot shown on the chart.
   * This handle can be used in other acop API calls to for example refresh or clear
   * a particular plot.  Upon error, and negative value is returned.
   */
  public int draw(Object ydata, Object xdata)
  {
    return draw(ydata, xdata, null, null, 0, 0);
  }
  /**
   * Draws a curve on the acop chart using the data provided and the chart display settings
   *
   * @param ydata is an array object containing the data points to be plotted on the vertical
   * axis. If ydata is not an array of a number type (e.g. a string array) then the draw method
   * while only succeed if the acop chart display mode is set to 'textbox'.
   * @param xdata is an array object containing the horizontal (x) axis coordinates to associate with the
   * ydata data array.  In methods where this is optional, an array of monotonically
   * increasing integers is used as the horizontal axis (convenient for histograms).
   * @param disable is array object, which if not null is essentially a boolean indication
   * as to whether the corresponding element in the ydata array is to be plotted or not.
   * If the disable array element is non-zero (i.e. true) then the data point is 'disabled'
   * and not shown.
   *
   * @return a positive display handle which refers to the plot shown on the chart.
   * This handle can be used in other acop API calls to for example refresh or clear
   * a particular plot.  Upon error, and negative value is returned.
   */
  public int draw(Object ydata, Object xdata, Object disable)
  {
    return draw(ydata, xdata, disable, null, 0, 0);
  }
  /**
   * Draws a curve on the acop chart using the data provided and the chart display settings
   *
   * @param ydata is an array object containing the data points to be plotted on the vertical
   * axis. If ydata is not an array of a number type (e.g. a string array) then the draw method
   * while only succeed if the acop chart display mode is set to 'textbox'.
   * @param xdata is an array object containing the horizontal (x) axis coordinates to associate with the
   * ydata data array.  In methods where this is optional, an array of monotonically
   * increasing integers is used as the horizontal axis (convenient for histograms).
   * @param disable is array object, which if not null is essentially a boolean indication
   * as to whether the corresponding element in the ydata array is to be plotted or not.
   * If the disable array element is non-zero (i.e. true) then the data point is 'disabled'
   * and not shown.
   * @param xlabel is string array containing horizontal (x) axis element labels, which are used as
   * tick mark labels.  If this array is null, then the corresponding x axis value is
   * used as a tick mark label.
   *
   * @return a positive display handle which refers to the plot shown on the chart.
   * This handle can be used in other acop API calls to for example refresh or clear
   * a particular plot.  Upon error, and negative value is returned.
   */
  public int draw(Object ydata, Object xdata, Object disable, java.lang.String[] xlabel)
  {
    return draw(ydata, xdata, disable, xlabel, 0, 0);
  }
  /**
   * Draws a curve on the acop chart using the data provided and the chart display settings
   *
   * @param ydata is an array object containing the data points to be plotted on the vertical
   * axis. If ydata is not an array of a number type (e.g. a string array) then the draw method
   * while only succeed if the acop chart display mode is set to 'textbox'.
   * @param xdata is an array object containing the horizontal (x) axis coordinates to associate with the
   * ydata data array.  In methods where this is optional, an array of monotonically
   * increasing integers is used as the horizontal axis (convenient for histograms).
   * @param disable is array object, which if not null is essentially a boolean indication
   * as to whether the corresponding element in the ydata array is to be plotted or not.
   * If the disable array element is non-zero (i.e. true) then the data point is 'disabled'
   * and not shown.
   * @param xlabel is string array containing horizontal (x) axis element labels, which are used as
   * tick mark labels.  If this array is null, then the corresponding x axis value is
   * used as a tick mark label.
   * @param ArraySize if non-zero gives the upper limit of the ydata array to be plotted.
   * @param MaxNumber if non-zero establishes the upper limit of the plot buffer to be used.
   * This is useful for establishing a plot 'ring' buffer which can be used in conjunction
   * the appendScreen() method.
   *
   * @return a positive display handle which refers to the plot shown on the chart.
   * This handle can be used in other acop API calls to for example refresh or clear
   * a particular plot.  Upon error a negative value is returned.
   */
  public int draw(Object ydata, Object xdata, Object disable, java.lang.String[] xlabel, int ArraySize,
	      int MaxNumber) {
	  return draw(ydata,xdata,disable,xlabel,ArraySize,MaxNumber,false);
  }

  
  protected static boolean assertPromptDisplay(long numPoints)
  {
    if (numPoints > 16000)
    {
      GraphicsConfiguration dsd = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().getDefaultConfiguration();
      AffineTransform dsdt = dsd.getDefaultTransform();
      if (dsdt.getScaleX() > 1 || (int)dsdt.getScaleY() > 1) return false;
    }
    return true;
  }
  /**
   * Draws a curve on the acop chart using the data provided and the chart display settings
   *
   * @param ydata is an array object containing the data points to be plotted on the vertical
   * axis. If ydata is not an array of a number type (e.g. a string array) then the draw method
   * while only succeed if the acop chart display mode is set to 'textbox'.
   * @param xdata is an array object containing the horizontal (x) axis coordinates to associate with the
   * ydata data array.  In methods where this is optional, an array of monotonically
   * increasing integers is used as the horizontal axis (convenient for histograms).
   * @param disable is array object, which if not null is essentially a boolean indication
   * as to whether the corresponding element in the ydata array is to be plotted or not.
   * If the disable array element is non-zero (i.e. true) then the data point is 'disabled'
   * and not shown.
   * @param xlabel is string array containing horizontal (x) axis element labels, which are used as
   * tick mark labels.  If this array is null, then the corresponding x axis value is
   * used as a tick mark label.
   * @param ArraySize if non-zero gives the upper limit of the ydata array to be plotted.
   * @param MaxNumber if non-zero establishes the upper limit of the plot buffer to be used.
   * This is useful for establishing a plot 'ring' buffer which can be used in conjunction
   * the appendScreen() method.
   * @param bindToSecondaryAxis is a boolean flag which if 'true' fixes the display of the
   * ensuing plot to the settings of the secondary (y) axis parameters.
   *
   * @return a positive display handle which refers to the plot shown on the chart.
   * This handle can be used in other acop API calls to for example refresh or clear
   * a particular plot.  Upon error a negative value is returned.
   */
  public int draw(Object ydata, Object xdata, Object disable, java.lang.String[] xlabel, int ArraySize,
      int MaxNumber, boolean bindToSecondaryAxis)
  {
    int i, tmp;
    if (m_lock == n_display) return -1; // no free slot to draw
    // this slot should not be overwrite
    if (aFrame.drawFlag == true && histo[loadXYindex] != null && histo[loadXYindex].displayON == 1)
    {
      // find next free slot
      for (i = loadXYindex + 1, tmp = 0; tmp < n_display; i++, tmp++)
      {
        if (i == n_display)
        {
          i = 0;
          m_filled = 1;
        }
        if (histo[i] == null || (histo[i].lockFlag == 0 && histo[i].displayON == 0))
          break;
        else if (reuseScreen && histo[i].lockFlag == 0) break;
      }
      if (tmp < n_display)
        loadXYindex = i;
      else
        return -2; // no free slot to draw
    }
    if (histo[loadXYindex] == null)
    {
      histo[loadXYindex] = new AcopHisto(this, loadXYindex);
      if (loadXYindex == 0 && aFrame.drawFlag == false && textFlag == true)
        rescaleTextFlag = AcopConst.NEW_PRINT_WINDOW;
      if (loadXYindex == 0 && textFlag == false) // first time;
      {
      }
    }
    if (histo[loadXYindex].setHisto(ydata, xdata, disable, xlabel, ArraySize, MaxNumber, foreColor) != 0)
      return -3;
    // if ( aFrame.drawFlag == false && textFlag == false ) drawTicks = true;
    if (aFrame.drawFlag == false)
      drawTicks = true;
    else
      drawTicks = false;
    // if ( histo[0].displayON != 0 ) repainHisto = true; // it was On
    if (aFrame.drawFlag == true && histo[loadXYindex].displayON == 0) repainHisto = true; // it was On
    aFrame.drawFlag = true;
    histo[loadXYindex].displayON = 1;
    histo[loadXYindex].dismode = globalDisplayMode;
    histo[loadXYindex].disDots = enableDots;
    histo[loadXYindex].disBarHisto = enableBarlineHisto;
    histo[loadXYindex].xext = XExtent;
    histo[loadXYindex].yext = YExtent;
    histo[loadXYindex].colorfilled = colorFilled;
    histo[loadXYindex].barlineColorFilled = barlineColorFilled;
    if (m_keep == 0)
    {
      if (m_filled != 0 && m_lock == 0) // no refrence!
      {
        m_histstart = loadXYindex + 1;
        if (m_histstart >= n_display) m_histstart = 0;
      }
    }
    else
    // move start pointer, change lasthist to keeppen
    {
      if (m_numberkeep == m_keep)
      {
        histo[m_keepstart].displayON = 0; // turn off one
        if (m_lock == 0) for (i = ++m_keepstart;; i++) // find histstart
          {
            if (i >= n_display) i = 0;
            if (i == loadXYindex) break;
            if (histo[i].displayON != 0)
            {
              m_histstart = i;
              break;
            }
          }
      }
    }
    if (wrapAround)
      histo[loadXYindex].getMaxWindow();
    else
      histo[loadXYindex].wrapindex = 0;
    histIndex = loadXYindex;
    lastDisp = loadXYindex;

      if (loadXYindex < 0 || loadXYindex >= n_display) return -1;
      if (histo[loadXYindex] != null && histo[loadXYindex].displayON != 0 && histo[loadXYindex].lockFlag == 0)
      {
        histo[loadXYindex].bindSecondaryY = bindToSecondaryAxis;
      }

    prepareDraw();
    m_keep = 0;
    return (loadXYindex);
  }
  /**
   * Clears the acop chart of all displays
   */
  public void clearScreen()
  {
    clearScreen(-1, 0);
  }
  /**
   * Removes the selected display from the chart
   *
   * @param hDisplay is the display handle referring to the plot to which the
   * data are to be appended.  This is the return value of the original call
   * to the draw() method.
   */
  public void clearScreen(int hDisplay)
  {
    clearScreen(hDisplay, 0);
  }
  /**
   * Removes the display from the chart and deletes the underlying data to free
   * resources.
   *
   * @param hDisplay is the display handle referring to the plot to which the
   * data are to be appended.  This is the return value of the original call
   * to the draw() method.
   */
  public void deleteScreen(int hDisplay)
  {
    clearScreen(hDisplay, 0, true);
  }
  /**
   * Removes the selected display from the chart, but retain a number
   * of displayed plots in the 'keep' color.
   *
   * @param hDisplay is the display handle referring to the plot to which the
   * data are to be appended.  This is the return value of the original call
   * to the draw() method.
   * @param KeepDepth gives the number of plots to retain (displayed in
   * the assigned 'keep' color (default = grey) on the display chart.
   */
  public void clearScreen(int hDisplay, int keepDepth)
  {
    clearScreen(hDisplay,keepDepth,false);
  }

  /**
   * Removes the display from the chart and on request also deletes the
   * underlying data arrays to free resources.
   *
   * @param hDisplay is the display handle referring to the plot to which the
   * data are to be appended.  This is the return value of the original call
   * to the draw() method.
   * @param KeepDepth gives the number of plots to retain (displayed in
   * the assigned 'keep' color (default = grey) on the display chart.
   * @param delete true if the data arrays should be deleted or false to retain
   * them for later recall
   */
  private void clearScreen(int hDisplay, int KeepDepth, boolean delete)
  {
    int i, j, num, tmp;
    boolean textbox = false;
    if (aFrame.drawFlag == false) return;

    persistClear(hDisplay);

    m_keep = KeepDepth; // number of kepted histogram
    if (m_keep < 0 || m_keep >= n_display) m_keep = 0;
    num = hDisplay; // histo number to be turned off
    if (num < 0 || num >= n_display) num = -1;
    // RedrawTextHist = 1;
    if (m_keep == 0 && num < 0) // turn all off, if not locked
    {
      if (m_lock == 0) // no locked histo, then reset
      {
        if (textFlag == false) drawTicks = true;
        aFrame.drawFlag = false;
        m_filled = 0;
        m_histstart = 0;
        loadXYindex = 0; // reset index, make displayflag to FALSE
        // if ( modeChange == TRUE )
        // {
        // drawTicks = TRUE;
        // modeChange = FALSE;
        // }
      }
      for (i = 0; i < n_display; i++)
      {
        if (histo[i] == null) continue;
        if (histo[i].displayON != 0 && histo[i].dismode == AcopConst.mode_textbox) textbox = true;
        if (histo[i] != null && histo[i].lockFlag == 0)
        {
          histo[i].displayON = 0;
          if (delete) {
            histo[i].deleteData();
          }
        }
      }
    }
    else
    {
      if (num >= 0) // clear hist num
      {
        if (histo[num] != null && histo[num].displayON != 0 && histo[num].lockFlag == 0)
        {
          if (histo[num].dismode == AcopConst.mode_textbox) textbox = true;
          histo[num].displayON = 0;
          if (delete) {
            histo[num].deleteData();
          }
          if (num == m_histstart) // change m_histstart to next visible histo
          {
            for (i = 0, j = m_histstart; i < n_display; i++, j++) // is all
                                                                  // histo off?
            {
              if (j >= n_display) j = 0;
              if (histo[j] != null && histo[j].displayON != 0)
              {
                m_histstart = num;
                break;
              }
            }
          }
        }
        else if (m_keep == 0) return; // already off -> nothing changed
      }
      if (m_keep > n_display - m_lock) m_keep = n_display - m_lock;
      if (m_keep != 0) // keep # of background histograms
      { // find keep start
        for (m_numberkeep = tmp = j = 0, i = loadXYindex + 1; j < m_keep;)
        {
          if (--i < 0) i += n_display;
          if (histo[i] == null || histo[i].lockFlag == 0) j++; // keep empty or
                                                                // notlocked
          if (histo[i] != null && histo[i].displayON != 0)
          {
            tmp++; // # of displayON
            if (histo[i].lockFlag == 0)
            {
              // turn keep to keep-pen, -color....
              histo[i].keephisto();
              m_numberkeep++;
              m_keepstart = i; // keephisto start #
              // hide tagging display for this keeped histogram!
              if (histo[i].dismode == AcopConst.mode_histogram
                  || histo[i].dismode == AcopConst.mode_histogram_rastoring
                  || histo[i].dismode == AcopConst.mode_dots || histo[i].dismode == AcopConst.mode_rectangle
                  || histo[i].dismode == AcopConst.mode_circle) histo[i].tagHist = 0;
            }
          }
        }
        if (m_lock == 0 && m_numberkeep != 0) m_histstart = m_keepstart; // first
                                                                          // visible
                                                                          // histo
        aFrame.drawFlag = (m_lock != 0 || tmp != 0) ? true : false;
        if (textFlag == false && aFrame.drawFlag == false) drawTicks = true;
        for (j = loadXYindex + 1;; j++) // turn rest off
        {
          if (j >= n_display) j = 0;
          if (j == i) break;
          if (histo[j] != null && histo[j].lockFlag == 0)
          {
            histo[j].displayON = 0;
            if (delete) {
              histo[j].deleteData();
            }
          }
        }
      }
      else
      { // is all histo off?
        for (i = 0, j = m_histstart; i < n_display; i++, j++)
        {
          if (j >= n_display) j = 0;
          if (histo[j] != null && histo[j].displayON != 0) break;
        }
        if (i == n_display)
        {
          aFrame.drawFlag = false;
          if (textFlag == false) drawTicks = true;
        }
      }
    }
    if ( textbox ) textArea.setText("");  //2012-1-9 clear textbox
    if (drawTicks || histo[m_histstart].dismode >= AcopConst.mode_textbox
        || (aFrame.drawFlag == false && textFlag == false))
      this.repaint();
    else
      repaintHisto();
  }
  /**
   * Recalls a display plot which has been cleared with one of the
   * clearScreen() methods to the acop chart.
   *
   * If the input display handle does not refer to a valid plot, then
   * nothing happens.
   *
   * @param hDisplay is the display handle referring to the plot to which the
   * data are to be appended.  This is the return value of the original call
   * to the draw() method.
   */
  public void recallScreen(int hDisplay)
  {
    int num;
    num = hDisplay; // number to be turned on
    if (num < 0 || num >= n_display) return;
    if (histo[num] == null) return;
    if (aFrame.drawFlag == false && m_lock == 0) m_histstart = num;
    if (histo[num].displayON == 0)
    {
      // RedrawTextHist = 1;
      histo[num].displayON = 1;
      aFrame.drawFlag = true;
      if (histo[num].dismode == AcopConst.mode_histogram
          || histo[num].dismode == AcopConst.mode_histogram_rastoring
          || histo[num].dismode == AcopConst.mode_dots || histo[num].dismode == AcopConst.mode_rectangle
          || histo[num].dismode == AcopConst.mode_circle)
      { // bring to top histogram!
        histo[num].tagHist = 1;
        lastTagHisto = num;
      }
      lastDisp = num;
      // find wrapindex
      if (wrapAround && sca[0].log == 0) histo[num].setwrapindex();
      // prepareDraw(FALSE,num,FALSE, 0);
      if (drawTicks)
      {
        this.repaint();
        drawTicks = false;
      }
      else
        drawOneHist(num);
    }
  }
  /**
   * It is occasionally desirable to fix one or more displays to the
   * acop chart and be able to clear all others with a simple call
   * to the clearScreen() method. Locking a screen enables you to do
   * this.
   *
   * @param hDisplay is the display handle referring to the plot
   * in question. This is the return value of the original call
   * to the draw() method.
   *
   * @return the current screen lock state
   */
  public boolean isScreenLocked(int hDisplay)
  {
    return lockScreen(-1,hDisplay) > 0;
  }
  /**
   * Sets the screen lock criterion.
   *
   * It is occasionally desirable to fix one or more displays to the
   * acop chart and be able to clear all others with a simple call
   * to the clearScreen() method. Locking a screen enables you to do
   * this.
   *
   * @param hDisplay is the display handle referring to the plot
   * in question. This is the return value of the original call
   * to the draw() method.
   * @param value is the desired screen lock state
   *
   * @param hDisplay
   * @param value
   */
  public void setScreenLocked(int hDisplay,boolean value)
  {
    lockScreen(value ? 1 : 0,hDisplay);
  }
  public long lockScreen(int LockOn, int hDisplay)
  {
    int num, ende, lock, i;
    lock = LockOn; // lock all the histo you see! (<0, 0, >0)
    num = hDisplay; // or unlock them as user requested
    if (num < 0 || num >= n_display)
    {
      num = 0;
      ende = n_display - 1;
    }
    else
    {
      ende = num;
    }
    if (lock < 0) // return current lock flag (ON or OFF) -> read only
    {
      if (num != ende)
        return m_lock; // number of locked histos
      else
        return histo[num] != null ? histo[num].lockFlag : -1;
    }
    else if (lock > 0 && aFrame.drawFlag == false) return -1; // nothing to lock
    for (i = num; num <= ende; num++)
    {
      if (histo[num] == null) continue;
      if (lock == 0) // unlock
      {
        if (histo[num].lockFlag == 1)
        {
          histo[num].lockFlag = 0;
          m_lock--;
        }
      }
      else if (histo[num].displayON != 0) // lock it if on the screen
      {
        if (histo[num].lockFlag == 0)
        {
          histo[num].lockFlag = 1;
          // first locked as histstart
          if (m_lock++ == 0 && num != m_histstart) m_histstart = num;
        }
      }
    }
    return (i != ende) ? m_lock : (histo[num] == null ? 0 : histo[ende].lockFlag);
  }
  /**
   * Prints a text field on the acop chart.
   *
   * The text is printed at the coordinates of the origin.
   *
   * @param text is the text to be displayed on the chart.
   *
   * @return a handle to the text field or -1 if an error has occured.
   */
  public int printText(java.lang.String text)
  {
    return callPrintText(text, 0, 0, false);
  }
  /**
   * Prints a text field on the acop chart.
   *
   * The text is printed at the coordinates given.
   *
   * @param text is the text to be displayed on the chart.
   * @param xpos is the x position (according to the assigned
   * chart's horizontal maximum and minimum) of the begining of the
   * text field.
   * @param ypos is the y position (according to the assigned
   * chart's vertical maximum and minimum) of the begining of the
   * text field.
   *
   * @return a handle to the text field or -1 if an error has occured.
   */
  public int printText(java.lang.String text, double xpos, double ypos)
  {
    return callPrintText(text, xpos, ypos, true);
  }
  protected int callPrintText(java.lang.String Text, double Xposition, double Yposition, boolean positionFlag)
  {
    int resca, i, firstDraw;
    if (Text.indexOf(13) == 0) return -1; // linefeed in the begining
    // the current visible slot should not be overwrite
    if (textFlag == true && ptext[loadTextIndex] != null && ptext[loadTextIndex].textON == true)
    {
      // find next free slot
      for (i = loadTextIndex + 1, resca = 0; resca < n_text; i++, resca++)
      {
        if (i == n_text) i = 0;
        if (ptext[i] == null)
        {
          ptext[i] = new AcopText(this);
          break;
        }
        else if (ptext[i].textON == false)
          break;
        else if (reuseText) break;
      }
      if (resca < n_text)
      {
        loadTextIndex = i;
      }
      else
      {
        return -1; // no free slot to prntText
      }
    }
    firstDraw = 0;
    if (ptext[loadTextIndex] == null)
    {
      ptext[loadTextIndex] = new AcopText(this);
      if (loadTextIndex == 0 && (aFrame.drawFlag == false || firstText == true))
      {
        firstDraw = 1;
        //removed by jbobnar
//        setlimit(); // load m_xmin... ...
        sca[0].scaleToGrid();
        for (i = 0; i < 3; i++)
          sca[i].xyscale(true); // set new min max
        scaleframe();
        aFrame.xleft = 0;
        aFrame.xright = aFrame.histRect.width;
        aFrame.dxwidth = aFrame.xright - aFrame.xleft;
        aFrame.absxleft = aFrame.xleft + aFrame.histRect.x;
      }
    }
    if (positionFlag == false) // load last positions
    {
      Xposition = lastTextx;
      Yposition = lastTexty;
    }
    if (ptext[loadTextIndex].printText(Text, Xposition, Yposition, foreColor, getFont()) == 0)
    {
      // i = loadTextIndex;
      // if ( ++loadTextIndex == n_text ) loadTextIndex = 0;
      // return i;
      // store last positions
      lastTextx = ptext[loadTextIndex].lastTextx;
      lastTexty = ptext[loadTextIndex].lastTexty;
      if (firstDraw != 0) repaint();
      // resca = loadTextIndex;
      // if ( ++loadTextIndex == n_text ) loadTextIndex = 0;
      // if ( disp[0] == 0 && scalecall < 2 ) {
      // InvalidateControl(NULL); // draw ticks and grid...if no histo at all
      // scalecall = 2;
      // }
      return (loadTextIndex);
    }
    firstText = false;
    return -1;
  }
  /**
   * Removes all displayed text within the acop chart.
   */
  public void clearText()
  {
    clearText(-1);
  }
  /**
   * Removes the displayed text associated with the given
   * text handle
   *
   * @param hText is the text handle returned from the original
   * call to printText().  A value of '-1' clears all displayed
   *  text.
   */
  public void clearText(int hText)
  {
    int start, ende, i, cnt;
    start = hText;
    if (start < 0 || start >= n_text)
    {
      start = 0;
      ende = n_text - 1;
    }
    else
      ende = start;
    for (cnt = 0, i = 0; i < n_text; i++)
    {
      if (ptext[i] == null) continue;
      if (ptext[i].textON == false) continue;
      if (i >= start && i <= ende)
      {
        ptext[i].textON = false;
        if (ptext[i].outside == false)
        	//jbobnar: just repaint everything because there might be lines printed next to the text
        	this.repaint();
//          this.repaint(ptext[i].textRect.x, ptext[i].textRect.y, ptext[i].textRect.width,
//              ptext[i].textRect.height);
      }
      else
        cnt++;
    }
    textFlag = cnt != 0 ? true : false;
    if (textFlag == false)
    {
      // scalecall = 0;
      if (aFrame.drawFlag == false)
      {
        this.repaint();
        drawTicks = true;
      }
    }
    if (start != ende) loadTextIndex = 0;
  }
  /**
   * Recalls displayed text which has been cleared with one of the
   * clearText() methods to the acop chart.
   *
   * If the input text handle does not refer to a valid text field, then
   * nothing happens.
   *
   * @param hText is the text handle referring to the text field to which the
   * text has been applied.  This is the return value of the original call
   * to the printText() method.
   */
  public void recallText(int hText)
  {
    int start;
    start = hText;
    if (start < 0 || start >= n_text) return;
    if (ptext[start] == null || ptext[start].textON == true) return;
    // if ( ptext[start].moveable && ptext[start].rescaled ) rescaleText(start,
    // ptext[start]->moveable, FALSE, ptext[start]->scalable);
    textFlag = true;
    ptext[start].textON = true;
    ptext[start].rescaleText(0); // use intern flag
    if (ptext[start].outside == true) return;
    if (drawTicks)
    {
      this.repaint();
      drawTicks = false;
    }
    else
      ptext[start].printUserText(null);
  }
  /**
   * Replaces the displayed text with the contents of the input text
   * array for the text field given.
   *
   * @param newText is the new text to use in the designated text field.
   * @param hText is the text field handle referring to the text which
   * should be refreshed.  This is the return value of the original call
   * to the printText() method.
   *
   * @return the handle of the text field is returned again.
   */
  public int refreshText(java.lang.String newText, int hText)
  {
    if (hText < 0 || hText >= n_text || ptext[hText] == null) return -1;
    ptext[hText].refreshText(newText);
    return hText;
  }
  public void setAveragingDepth(int averagingDepth)
  {
    if (this.averagingDepth == averagingDepth) return;
    if (averagingDepth < 2 || averagingDepth > AcopConst.N_DEF_DISPLAY) averagingDepth = 0;
    averagingIndex = 0;
    averagingLength = 0;
    NumberAveragingData = 0;
    int oldValue = this.averagingDepth;
    this.averagingDepth = averagingDepth;
    firePropertyChange("averagingDepth", oldValue, this.averagingDepth);
  }
  public int getAveragingDepth()
  {
    return this.averagingDepth;
  }
  public void setAveragingMode(int averagingMode)
  {
    if (this.averagingMode == averagingMode || averagingMode < 0 || averagingMode > 1) return;
    int oldValue = this.averagingMode;
    this.averagingMode = averagingMode;
    firePropertyChange("averagingMode", oldValue, this.averagingMode);
  }
  public int getAveragingMode()
  {
    return this.averagingMode;
  }
  /**
   * Assign a chart caption with this method call.
   *
   * @param caption is the new acop chart caption.  An
   * embedded '\n' will result in a new line in the caption.
   */
  public void setCaption(java.lang.String caption)
  {
//    if (this.caption != null && this.caption.equals(caption)) return;
    captionChanged = true;
    String oldValue = this.caption;
    this.caption = caption;
    this.repaint(titleRect.x, titleRect.y, titleRect.width, titleRect.height);
    firePropertyChange("caption", oldValue, this.caption);
  }
  /**
   * @return the assigned chart caption.
   */
  public java.lang.String getCaption()
  {
    return this.caption;
  }
  public void setCaptionLocation(int captionPosition)
  {
	if (this.captionPosition == captionPosition) return;
	int oldValue = this.captionPosition;
    this.captionPosition = captionPosition;
    this.repaint(titleRect.x, titleRect.y, titleRect.width, titleRect.height);
    firePropertyChange("captionPosition", oldValue, this.captionPosition);
  }
  public int getCaptionLocation()
  {
    return this.captionPosition;
  }
  public static enum displayModes {
    polyline,
    barline,
    histogram,
    dots,
    histogram_rastoring,
    rectangle,
    circle,
    textbox,
    polyline_dots,
    barline_dots,
    histogram_dots,
    histogram_rastoring_dots,
    barline_histo,
    barline_histo_dots,
    polyline_cross,
    annotation,
    status,
    barline_bit;
  }
  /**
   * Assigns a display mode to any plots using the draw() method.  Calling this
   * method after a plot is drawn will not affect its display, but will
   * only apply to those plots drawn afterwards.
   *
   * @param displayMode is the desired display mode.
   *
   */
  public void setDisplayMode(displayModes newMode)
  {
  	if (newMode == null) return;
    setDisplayMode(newMode.ordinal());
  }
  /**
   * Assigns a display mode to any plots using the draw() method.  Calling this
   * method after a plot is drawn will not affect its display, but will
   * only apply to those plots drawn afterwards.
   *
   * @param mode is the desired display mode. This should be one of
   *
   * polyline = 0,
     barline = 1,
     histogram = 2,
     dots = 3,
     histogram_rastoring = 4,
     rectangle = 5,
     circle = 6,
     textbox = 7,
     polyline_dots = 8,
     barline_dots = 9,
     histogram_dots = 10,
     histogram_rastoring_dots = 11,
     barline_histo = 12,
     barline_histo_dots = 13,
     polyline_cross = 14,
     annotation = 15,
     status = 16,
     barline_bit = 17;
   */
  public void setDisplayMode(int mode)
  // throws PropertyVetoException
  {
    if (this.displayMode == mode) return;
    /*
     * Integer oldValue = new Integer(this.displayMode); Integer newValue = new
     * Integer(displayMode); veto.fireVetoableChange("displayMode" ,oldValue,
     * newValue); this.displayMode = displayMode;
     * changes.firePropertyChange("displayMode" ,oldValue, newValue); if (
     * displayMode < AcopConst.mode_polyline ) displayMode =
     * AcopConst.mode_polyline; if ( displayMode > AcopConst.mode_textbox )
     * displayMode = AcopConst.mode_textbox;
     */
    if (mode < AcopConst.mode_polyline || mode > AcopConst.mode_barline_bit) return;
    int oldValue = this.displayMode;
    this.displayMode = mode;
    setGlobalDisplayMode();
    if (randomData == true)
    {
      rescaleFlag = 2;
      prepareDraw(); // rescale and reset limit
    }
    firePropertyChange("displayMode", oldValue, this.displayMode);
  }
  /**
   * @return the current display mode selection.
   */
  public int getDisplayMode()
  {
    return this.displayMode;
  }
  /**
   * Sets the desired draw (pen) width for the display plots.
   * Calling this method after a plot is drawn will not affect its display,
   * but will only apply to those plots drawn afterwards.
   *
   * @param drawWidth
   */
  public void setDrawWidth(int drawWidth)
  {
    if (this.drawWidth == drawWidth) return;
    if (drawWidth < 1) drawWidth = 1;
    int oldValue = this.drawWidth;
    this.drawWidth = drawWidth;
    if (randomData == true) repaintHisto();
    // prepareDraw(2, -1); // rescale and reset limit
    firePropertyChange("drawWidth", oldValue, this.drawWidth);
  }
  /**
   * @return the current display (pen) width.
   */
  public int getDrawWidth()
  {
    return this.drawWidth;
  }
  private void checkRange(int flag, double value)
  {
    if ( flag == 1 )  // xMin input
    {
      if (sca[0].usermax <= value + AcopConst.zero_check)
      {
        if (Math.abs(value) < AcopConst.zero_check)
          sca[0].usermax = value + 10;
        else
          sca[0].usermax = value + 0.5 * Math.abs(value);
      }
      sca[0].setUserscale(value, sca[0].usermax);
    }
    else if ( flag == 2 )  // xMax input
    {
      if (value <= sca[0].usermin + AcopConst.zero_check)
      {
        if (Math.abs(value) < AcopConst.zero_check)
          sca[0].usermin = value - 10;
        else
          sca[0].usermin = value - 0.5 * Math.abs(value);
      }
      sca[0].setUserscale(sca[0].usermin, value);
    }
    else if ( flag == 3 )
    {
      if (sca[1].usermax <= value + AcopConst.zero_check)
      {
        if (Math.abs(value) < AcopConst.zero_check)
          sca[1].usermax = value + 10;
        else
          sca[1].usermax = value + 0.5 * Math.abs(value);
      }
      sca[1].setUserscale(value, sca[1].usermax);
    }
    else if ( flag == 4 )
    {
      if (value <= sca[1].usermin + AcopConst.zero_check)
      {
        if (Math.abs(value) < AcopConst.zero_check)
          sca[1].usermin = value - 10;
        else
          sca[1].usermin = value - 0.5 * Math.abs(value);
      }
      sca[1].setUserscale(sca[1].usermin, value);
    }
  }
  public void setXRange(double xMin, double xMax)
  {
    if (Math.abs(sca[0].usermin - xMin) < AcopConst.zero_check) return;
    if (Math.abs(sca[0].usermax - xMax) < AcopConst.zero_check) return;
    double oldValue = sca[0].usermin;
    double oldValueMax = sca[0].usermax;
    drawTicks = true;
    checkRange(1, xMin);
    checkRange(2, xMax);
    rescaleFlag = 2;
    rescaleTextFlag = AcopConst.NEW_ZOOM_WINDOW;
    prepareDraw();
    firePropertyChange("xMin", oldValue, sca[0].usermin);
    firePropertyChange("xMax", oldValueMax, sca[0].usermax);
  }
  public void setYRange(double yMin, double yMax)
  {
    if (Math.abs(sca[1].usermin - yMin) < AcopConst.zero_check) return;
    if (Math.abs(sca[1].usermax - yMax) < AcopConst.zero_check) return;
    double oldValue = sca[1].usermin;
    double oldValueMax = sca[1].usermax;
    drawTicks = true;
    checkRange(3, yMin);
    checkRange(4, yMax);
    rescaleFlag = 2;
    rescaleTextFlag = AcopConst.NEW_ZOOM_WINDOW;
    prepareDraw();
    firePropertyChange("yMin", oldValue, sca[1].usermin);
    firePropertyChange("yMax", oldValueMax, sca[1].usermax);
  }
  public void setXYRange(double xMin, double xMax, double yMin, double yMax)
  {
    if (Math.abs(sca[0].usermin - xMin) < AcopConst.zero_check) return;
    if (Math.abs(sca[0].usermax - xMax) < AcopConst.zero_check) return;
    if (Math.abs(sca[1].usermin - yMin) < AcopConst.zero_check) return;
    if (Math.abs(sca[1].usermax - yMax) < AcopConst.zero_check) return;
    double oldValue = sca[0].usermin;
    double oldValueMax = sca[0].usermax;
    double oldValueY = sca[1].usermin;
    double oldValueYMax = sca[1].usermax;
    drawTicks = true;
    checkRange(1, xMin);
    checkRange(2, xMax);
    checkRange(3, yMin);
    checkRange(4, yMax);
    rescaleFlag = 2;
    rescaleTextFlag = AcopConst.NEW_ZOOM_WINDOW;
    prepareDraw();
    firePropertyChange("xMin", oldValue, sca[0].usermin);
    firePropertyChange("xMax", oldValueMax, sca[0].usermax);
    firePropertyChange("yMin", oldValueY, sca[1].usermin);
    firePropertyChange("yMax", oldValueYMax, sca[1].usermax);
  }
  /**
   * Sets the acop chart horizontal (x) axis minimum
   *
   * @param minValue is the desired display minimum value
   */
  public void setXMin(double minValue)
  {
    if (Math.abs(sca[0].usermin - minValue) < AcopConst.zero_check) return;
    double oldValue = sca[0].usermin;
    drawTicks = true;
    if (sca[0].usermax <= minValue + AcopConst.zero_check)
    {
      if (Math.abs(minValue) < AcopConst.zero_check)
        sca[0].usermax = minValue + 10;
      else
        sca[0].usermax = minValue + 0.5 * Math.abs(minValue);
    }

    sca[0].setUserscale(minValue, sca[0].usermax);
    rescaleFlag = 2;
    rescaleTextFlag = AcopConst.NEW_ZOOM_WINDOW;
    prepareDraw();
    firePropertyChange("xMin", oldValue, sca[0].usermin);
    // prepareDraw(TRUE, -1, TRUE, 0);
  }
  /**
   * @return the current setting for the x axis display minimum
   */
  public double getXMin()
  {
    return sca[0].usermin;
  }
  /**
   * Sets the acop chart horizontal (x) axis maximum
   *
   * @param maxValue is the desired display maximum value
   */
  public void setXMax(double maxValue)
  {
    if (Math.abs(sca[0].usermax - maxValue) < AcopConst.zero_check) return;
    double oldValue = sca[0].usermax;
    drawTicks = true;
    if (maxValue <= sca[0].usermin + AcopConst.zero_check)
    {
      if (Math.abs(maxValue) < AcopConst.zero_check)
        sca[0].usermin = maxValue - 10;
      else
        sca[0].usermin = maxValue - 0.5 * Math.abs(maxValue);
    }

    sca[0].setUserscale(sca[0].usermin, maxValue);
    rescaleFlag = 2;
    rescaleTextFlag = AcopConst.NEW_ZOOM_WINDOW;
    prepareDraw();
    firePropertyChange("xMax", oldValue, sca[0].usermax);
  }
  /**
   * @return the current setting for the x axis display maximum
   */
  public double getXMax()
  {
    return sca[0].usermax;
  }

  private double setYMax = Double.NaN;
  private double setYMin = Double.NaN;
  /**
   * Sets the acop chart primary vertical (y) axis minimum
   *
   * @param minValue is the desired display minimum value
   */
  public void setYMin(double minValue)
  {
    if (Math.abs(sca[1].usermin - minValue) < AcopConst.zero_check) return;
    double oldValue = sca[1].usermin;
    setYMin = minValue;
    if (m_marker == 2) setmarkerflag(2); // store origin value
    drawTicks = true;
    if (sca[1].usermax <= minValue + AcopConst.zero_check)
    {
      if (Math.abs(minValue) < AcopConst.zero_check)
        sca[1].usermax = minValue + 10;
      else
        sca[1].usermax = minValue + 0.5 * Math.abs(minValue);
    }
    if (Double.isNaN(setYMax)) {
        sca[1].setUserscale(minValue, sca[1].usermax);
    } else {
        sca[1].setUserscale(minValue, setYMax);
    }

    rescaleFlag = 2;
    rescaleTextFlag = AcopConst.NEW_ZOOM_WINDOW;
    prepareDraw();
    firePropertyChange("yMin", oldValue, sca[1].usermin);
  }
  /**
   * @return the current setting for the y axis display minimum
   */
  public double getYMin()
  {
    return sca[1].usermin;
  }
  /**
   * Sets the acop chart primary vertical (y) axis maximum
   *
   * @param maxValue is the desired display maximum value
   */
  public void setYMax(double maxValue)
  {
    if (Math.abs(sca[1].usermax - maxValue) < AcopConst.zero_check) return;
    double oldValue = sca[1].usermax;
    this.setYMax = maxValue;
    if (m_marker == 2) setmarkerflag(2); // store origin value
    drawTicks = true;
    if (maxValue <= sca[1].usermin + AcopConst.zero_check)
    {
      if (Math.abs(maxValue) < AcopConst.zero_check)
        sca[1].usermin = maxValue - 10;
      else
        sca[1].usermin = maxValue - 0.5 * Math.abs(maxValue);
    }
    if (Double.isNaN(setYMin)) {
        sca[1].setUserscale(sca[1].usermin, maxValue);
    } else {
        sca[1].setUserscale(setYMin, maxValue);
    }
    rescaleFlag = 2;
    rescaleTextFlag = AcopConst.NEW_ZOOM_WINDOW;
    prepareDraw();
    firePropertyChange("yMax", oldValue, sca[1].usermax);
  }
  /**
   * @return the current setting for the y axis display maximum
   */
  public double getYMax()
  {
    return sca[1].usermax;
  }
  /**
   * Sets the acop chart secondary vertical (y) axis minimum
   *
   * @param minValue is the desired display minimum value
   */
  public void setSecondaryYMin(double minValue)
  {
    if (Math.abs(sca[2].usermin - minValue) < AcopConst.zero_check) return;
    double oldValue = sca[2].usermin;
    if (sca[2].usermax <= minValue + AcopConst.zero_check)
    {
      if (Math.abs(minValue) < AcopConst.zero_check)
        sca[2].usermax = minValue + 10;
      else
        sca[2].usermax = minValue + 0.5 * Math.abs(minValue);
    }

    sca[2].setUserscale(minValue, sca[2].usermax);
    rescaleFlag = 2;
    rescaleTextFlag = AcopConst.NEW_ZOOM_WINDOW;
    prepareDraw();
    firePropertyChange("secondaryYMin", oldValue, minValue);
  }
  /**
   * @return the current setting for the secondary y axis display minimum
   */
  public double getSecondaryYMin()
  {
    return sca[2].usermin;
  }
  /**
   * Sets the acop chart secondary vertical (y) axis maximum
   *
   * @param maxValue is the desired display maximum value
   */
  public void setSecondaryYMax(double maxValue)
  {
    if (Math.abs(sca[2].usermax - maxValue) < AcopConst.zero_check) return;
    double oldValue = sca[2].usermax;
    if (maxValue <= sca[2].usermin + AcopConst.zero_check)
    {
      if (Math.abs(maxValue) < AcopConst.zero_check)
        sca[2].usermin = maxValue - 10;
      else
        sca[2].usermin = maxValue - 0.5 * Math.abs(maxValue);
    }

    sca[2].setUserscale(sca[2].usermin, maxValue);
    rescaleFlag = 2;
    rescaleTextFlag = AcopConst.NEW_ZOOM_WINDOW;
    prepareDraw();
    firePropertyChange("secondaryYMax", oldValue, maxValue);
  }
  /**
   * @return the current setting for the secondary y axis display maximum
   */
  public double getSecondaryYMax()
  {
    return sca[2].usermax;
  }
  private void scaleFrameDraw()
  {
    scaleframe();
    //    rescaleFlag = 1; //2005-Feb
    drawTicks = true;     // 2009-4 fix problem of ""The AcopChart is not repainted or validated when its size changes."
    rescaleTextFlag = AcopConst.NEW_TEXT_WINDOW;
    if (aFrame.drawFlag)
      prepareDraw();
    else
      changeScaleWidth = 0;
  }
  public void setFrameWidth(int frameWidth)
  {
    if (this.frameWidth == frameWidth) return;
    int oldValue = this.frameWidth;
    this.frameWidth = frameWidth;
    scaleFrameDraw();
    firePropertyChange("frameWidth", oldValue, this.frameWidth);
  }
  public int getFrameWidth()
  {
    return this.frameWidth;
  }
  /**
   * Sets the acop chart left frame offset to the desired value
   *
   * @param newOffset is the desired 'left' offset of the
   * acop frame with respect to the chart.  By default all border
   * frame offsets are set to 20.
   */
  public void setFrameLeftOffset(int newOffset)
  {
    if (this.frameLeftOffset == newOffset) return;
    int oldValue = this.frameLeftOffset;
    this.frameLeftOffset = newOffset;
    scaleFrameDraw();
    firePropertyChange("frameLeftOffset", oldValue, this.frameLeftOffset);
  }
  /**
   * @return the current acop chart border frame left offset
   */
  public int getFrameLeftOffset()
  {
    return this.frameLeftOffset;
  }
  /**
   * Sets the acop chart right frame offset to the desired value
   *
   * @param newOffset is the desired 'right' offset of the
   * acop frame with respect to the chart.  By default all border
   * frame offsets are set to 20.
   */
  public void setFrameRightOffset(int newOffset)
  {
    if (this.frameRightOffset == newOffset) return;
    int oldValue = this.frameRightOffset;
    this.frameRightOffset = newOffset;
    scaleFrameDraw();
    firePropertyChange("frameRightOffset", oldValue, this.frameRightOffset);
  }
  /**
   * @return the current acop chart border frame right offset
   */
  public int getFrameRightOffset()
  {
    return this.frameRightOffset;
  }
  /**
   * Sets the acop chart top frame offset to the desired value
   *
   * @param newOffset is the desired 'top' offset of the
   * acop frame with respect to the chart.  By default all border
   * frame offsets are set to 20.
   */
  public void setFrameTopOffset(int newOffset)
  {
    if (this.frameTopOffset == newOffset) return;
    int oldValue = this.frameTopOffset;
    this.frameTopOffset = newOffset;
    scaleFrameDraw();
    firePropertyChange("frameTopOffset", oldValue, this.frameTopOffset);
  }
  /**
   * @return the current acop chart border frame top offset
   */
  public int getFrameTopOffset()
  {
    return this.frameTopOffset;
  }
  /**
   * Sets the acop chart bottom frame offset to the desired value
   *
   * @param newOffset is the desired 'bottom' offset of the
   * acop frame with respect to the chart.  By default all border
   * frame offsets are set to 20.
   */
  public void setFrameBottomOffset(int newOffset)
  {
    if (this.frameBottomOffset == newOffset) return;
    int oldValue = this.frameBottomOffset;
    this.frameBottomOffset = newOffset;
    scaleFrameDraw();
    firePropertyChange("frameBottmOffset", oldValue, this.frameBottomOffset);
  }
  /**
   * @return the current acop chart border frame bottom offset
   */
  public int getFrameBottomOffset()
  {
    return this.frameBottomOffset;
  }
  public void setFrameForeColor(java.awt.Color frameForeColor)
  {
	if (aFrame.frameForeColor != null && aFrame.frameForeColor.equals(frameForeColor)) return;
	Color oldValue = aFrame.frameForeColor;
    aFrame.frameForeColor = frameForeColor;
    this.repaint();
    firePropertyChange("frameForColor", oldValue, aFrame.frameForeColor);
  }
  public java.awt.Color getFrameForeColor()
  {
    return aFrame.frameForeColor;
  }
  public void setXTickLabelColor(java.awt.Color xTickLabelColor)
  {
	  if (sca[0].tickLabelForeColor != null && sca[0].tickLabelForeColor.equals(xTickLabelColor)) return;
	  Color oldValue = sca[0].tickLabelForeColor;
    sca[0].tickLabelForeColor = xTickLabelColor;
    this.repaint(sca[0].ticksRect.x, sca[0].ticksRect.y, sca[0].ticksRect.width, sca[0].ticksRect.height);
    if (sca[0].leftBottom != sca[0].leftBottLabel)
      this.repaint(sca[0].labelRect.x, sca[0].labelRect.y, sca[0].labelRect.width, sca[0].labelRect.height);
    firePropertyChange("xTickLabelColor", oldValue, sca[0].tickLabelForeColor);
  }
  public java.awt.Color getXTickLabelColor()
  {
    return sca[0].tickLabelForeColor;
  }
  public void setYTickLabelColor(java.awt.Color yTickLabelColor)
  {
	  if (sca[1].tickLabelForeColor != null && sca[1].tickLabelForeColor.equals(yTickLabelColor)) return;
	  Color oldValue = sca[1].tickLabelForeColor;
    sca[1].tickLabelForeColor = yTickLabelColor;
    this.repaint(sca[1].ticksRect.x, sca[1].ticksRect.y, sca[1].ticksRect.width, sca[1].ticksRect.height);
    if (sca[1].leftBottom != sca[1].leftBottLabel)
      this.repaint(sca[1].labelRect.x, sca[1].labelRect.y, sca[1].labelRect.width, sca[1].labelRect.height);
    firePropertyChange("yTickLabelColor", oldValue, sca[1].tickLabelForeColor);
  }
  public java.awt.Color getYTickLabelColor()
  {
    return sca[1].tickLabelForeColor;
  }
  public void setSecondaryYTickLabelColor(java.awt.Color secondaryYTickLabelColor)
  {
	  if (sca[2].tickLabelForeColor != null && sca[2].tickLabelForeColor.equals(secondaryYTickLabelColor)) return;
	  Color oldValue = sca[2].tickLabelForeColor;
    sca[2].tickLabelForeColor = secondaryYTickLabelColor;
    firePropertyChange("secondaryYTickLabelColor", oldValue, sca[2].tickLabelForeColor);
    if (sca[2].OnFlag == false) return;
    this.repaint(sca[2].ticksRect.x, sca[2].ticksRect.y, sca[2].ticksRect.width, sca[2].ticksRect.height);
    if (sca[2].leftBottom != sca[2].leftBottLabel)
      this.repaint(sca[2].labelRect.x, sca[2].labelRect.y, sca[2].labelRect.width, sca[2].labelRect.height);

  }
  public java.awt.Color getSecondaryYTickLabelColor()
  {
    return sca[2].tickLabelForeColor;
  }
  public void setSecondaryYAxisOn(boolean secondaryYAxisOn)
  {
	  if (sca[2].OnFlag == secondaryYAxisOn) return;
    sca[2].OnFlag = secondaryYAxisOn;
    rescaleFlag = 2;
    rescaleTextFlag = AcopConst.NEW_ZOOM_WINDOW;
    if (aFrame.drawFlag == false) return;
    prepareDraw(); // rescale and reset limit
    firePropertyChange("secondaryYAxisOn", !secondaryYAxisOn, secondaryYAxisOn);
  }
  public boolean isSecondaryYAxisOn()
  {
    return sca[2].OnFlag;
  }
  public void setSecondaryYLogOn(boolean secondaryYLogOn)
  {
	  int temp = secondaryYLogOn ? 1 : 0 ;
	  if (temp == sca[2].log) return;
	  int oldValue = sca[2].log;
    sca[2].log = temp;
    rescaleFlag = 2;
    rescaleTextFlag = AcopConst.NEW_ZOOM_WINDOW;
    if (aFrame.drawFlag == false) return;
    prepareDraw(); // rescale and reset limit
    firePropertyChange("secondaryYLogOn", oldValue, sca[2].log);
  }
  public boolean isSecondaryYLogOn()
  {
    return sca[2].log != 0 ? true : false ;
  }
  public void setFrameFont(java.awt.Font frameFont)
  {
	if (aFrame.frameFont != null && aFrame.frameFont.equals(frameFont)) return;
	Font oldValue = aFrame.frameFont;
    aFrame.frameFont = frameFont;
    this.repaint();
    firePropertyChange("frameFont", oldValue, aFrame.frameFont);
  }
  public java.awt.Font getFrameFont()
  {
    return aFrame.frameFont;
  }
  public void setDrawStyle(drawStyles style)
  {
  	if (style == null) return;
  	setDrawStyle(style.ordinal());
  }
  /**
   * Set the line type used in drawing curves on an acop chart.
   *
   * Note: histograms are always solid.
   *
   * @param drawStyle is the desired type of drawn line.  One of
   * <p>PS_SOLID = 0,
   * <p>PS_DASH = 1,
   * <p>PS_DOT = 2,
   * <p>PS_DASH_DOT = 3;
   */
  public void setDrawStyle(int drawStyle)
  // throws PropertyVetoException
  {
    if (this.drawStyle != drawStyle)
    {
      /*
       * Integer oldValue = new Integer(this.drawStyle); Integer newValue = new
       * Integer(drawStyle); veto.fireVetoableChange("drawStyle" ,oldValue,
       * newValue); this.drawStyle = drawStyle;
       * changes.firePropertyChange("drawStyle" ,oldValue, newValue);
       */
      if (drawStyle < AcopConst.PS_SOLID || drawStyle > AcopConst.PS_DASH_DOT) return;
      int oldValue = this.drawStyle;
      this.drawStyle = drawStyle;
      if (randomData == true)
      {
        rescaleFlag = 2;
        prepareDraw(); // rescale and reset limit
      }
      firePropertyChange("drawStyle", oldValue, this.drawStyle);
    }
  }
  public int getDrawStyle()
  {
    return this.drawStyle;
  }
  public void setFFT(int fFT)
  {
//    if (fFT < 0 || fFT > AcopConst.FFT_RI_INVERSE) fFT = 0;
    if (fFT < 0 || fFT > AcopConst.FFT_POLYNORM_QUARTIC_FIT) fFT = 0;
    if (this.FFT == fFT) return;
    int oldValue = this.FFT;
    this.FFT = fFT;
    if (randomData == true)
    {
      rescaleFlag = 2;
      prepareDraw(); // rescale and reset limit
    }
    firePropertyChange("fFT", oldValue, this.FFT);
  }
  public int getFFT()
  {
    return this.FFT;
  }
  public static enum graphStyles {
    LIN_LIN,
    LIN_LOG,
    LOG_LIN,
    LOG_LOG,
    TIME_LIN,
    TIME_LOG,
  }
  /**
   * Set the acop chart graph style (x-axis/y-axis).
   *
   * @param style is the desired (enumerated) graph style.
   */
  public void setGraphStyle(graphStyles style)
  {
  	if (style == null) return;
  	setGraphStyle(style.ordinal());
  }
  /**
   * Set the acop chart graph style (x-axis/y-axis).
   *
   * @param graphStyle is the desired graph style. This should be
   * one of
   * <p>0 - LIN - LIN (x linear, y linear)
   * <p>1 - LIN - LOG (x linear, y log)
   * <p>2 - LOG - LIN (x log, y linear)
   * <p>3 - LOG - LOG (x log, y log)
   * <p>4 - TIME - LIN (x time in UTC, y linear)
   * <p>5 - TIME - LOG (x time in UTC, y log)
   */
  public void setGraphStyle(int graphStyle)
  {
    int changeYlog = 0;
    if (this.graphStyle == graphStyle) return;
    int oldValue = this.graphStyle;
    if (graphStyle < 0 || graphStyle > 5) graphStyle = 0;
    switch (graphStyle)
    {
      default:
      case 0: // lin-lin
        if (sca[1].log != 0) changeYlog = 1;
        sca[0].log = 0;
        sca[1].log = 0;
        sca[0].xtime = 0;
        break;
      case 1: // lin-log
        if (sca[1].log == 0) changeYlog = 1;
        sca[0].log = 0;
        sca[1].log = 1;
        sca[0].xtime = 0;
        break;
      case 2: // log-lin
        if (sca[1].log != 0) changeYlog = 1;
        sca[0].log = 1;
        sca[1].log = 0;
        sca[0].xtime = 0;
        break;
      case 3: // log-log
        if (sca[1].log == 0) changeYlog = 1;
        sca[0].log = 1;
        sca[1].log = 1;
        sca[0].xtime = 0;
        break;
      case 4: // time-lin
        if (sca[1].log != 0) changeYlog = 1;
        sca[0].log = 0;
        sca[1].log = 0;
        sca[0].xtime = 1;
        break;
      case 5: // time-log
        if (sca[1].log == 0) changeYlog = 1;
        sca[0].log = 0;
        sca[1].log = 1;
        sca[0].xtime = 1;
        break;
    }
    if (changeYlog != 0 && m_marker == 2) setmarkerflag(2); // recalculate
                                                            // marker array
    this.graphStyle = graphStyle;
    rescaleFlag = 2;
    rescaleTextFlag = AcopConst.NEW_ZOOM_WINDOW;
    prepareDraw(); // rescale and reset limit
    firePropertyChange("graphStyle", oldValue, this.graphStyle);
  }
  public int getGraphStyle()
  {
    return this.graphStyle;
  }
  public static enum drawStyles {
  	PS_SOLID,
  	PS_DASH,
  	PS_DOT,
  	PS_DASH_DOT,
  }
  public void setGridStyle(drawStyles style)
  {
  	if (style == null) return;
  	setGridStyle(style.ordinal());
  }
  /**
   * Sets the chart grid style
   *
   * @param gridStyle is the desired grid style.  This should be
   * one of
   * <p>PS_SOLID = 0
   * <p>PS_DASH = 1
   * <p>PS_DOT = 2
   * <p>PS_DASH_DOT = 3
   *
   */
  public void setGridStyle(int gridStyle)
  {
    if (sca[0].gridLineMode == gridStyle) return;
    int oldValue = sca[0].gridLineMode;
    for (int i=0; i<2; i++)
    {
      sca[i].gridLineMode = gridStyle;
      sca[i].gridStroke = AcopConst.createStroke(sca[i].gridLineWidth, sca[i].gridLineMode);
    }
    if (aFrame.drawFlag) repaintHisto();
    firePropertyChange("gridStyle", oldValue, sca[0].gridLineMode);
  }
  public int getGridStyle()
  {
    return sca[0].gridLineMode;
  }
  public void setGridColor(java.awt.Color gridColor)
  {
    if (sca[0].gridLineColor == gridColor) return;
    Color oldValue = sca[0].gridLineColor;
    for (int i=0; i<2; i++)
    {
      sca[i].gridLineColor = gridColor;
      sca[i].gridStroke = AcopConst.createStroke(sca[i].gridLineWidth, sca[i].gridLineMode);
    }
    if (aFrame.drawFlag) repaintHisto();
    firePropertyChange("gridColor", oldValue, sca[0].gridLineColor);
  }
  public java.awt.Color getGridColor()
  {
    return sca[0].gridLineColor;
  }
  public void setGridWidth(int gridWidth)
  {
    if (sca[0].gridLineWidth == gridWidth) return;
    if (gridWidth < 1) gridWidth = 1;
    int oldValue = sca[0].gridLineWidth;
    for (int i=0; i<2; i++)
    {
      sca[i].gridLineWidth = gridWidth;
      sca[i].gridStroke = AcopConst.createStroke(sca[i].gridLineWidth, sca[i].gridLineMode);
    }
    if (aFrame.drawFlag) repaintHisto();
    firePropertyChange("gridWidth", oldValue, sca[0].gridLineWidth);
  }
  public int getGridWidth()
  {
    return sca[0].gridLineWidth;
  }
  public void setKeepStyle(int keepStyle)
  {
	if (this.keepStyle == keepStyle) return;
	int oldValue = this.keepStyle;
    this.keepStyle = keepStyle;
    if (aFrame.drawFlag && m_keep != 0) repaintHisto();
    firePropertyChange("keepStyle", oldValue, this.keepStyle);
  }
  public int getKeepStyle()
  {
    return this.keepStyle;
  }
  /**
   * Sets the pen color for 'kept' display curves.
   *
   * When a display curve is cleared with the clearScreen method
   * which applies a 'keepDepth' > 0 then the last several cleared
   * displays (up to the keep depth) are displayed in on the chart in
   * an alternate color.  By default, this is 'grey'.  You can
   * determine this color by making use of this method call.
   *
   * @param newColor is the desired color of 'kept' display curves.
   */
  public void setKeepColor(java.awt.Color newColor)
  {
	if (this.keepColor != null && this.keepColor.equals(newColor)) return;
	Color oldValue = this.keepColor;
    this.keepColor = newColor;
    if (aFrame.drawFlag && m_keep != 0) repaintHisto();
    firePropertyChange("keepColor", oldValue, this.keepColor);
  }
  /**
   * When a display curve is cleared with the clearScreen method
   * which applies a 'keepDepth' > 0 then the last several cleared
   * displays (up to the keep depth) are displayed in on the chart in
   * an alternate color.  By default, this is 'grey'.  You can
   * determine this color by making use of setKeepColor().
   *
   * @return returns the current 'keep' color.
   */
  public java.awt.Color getKeepColor()
  {
    return this.keepColor;
  }
  public void setKeepWidth(int keepWidth)
  {
	if (keepWidth == this.keepWidth) return;
    if (keepWidth < 1) keepWidth = 1;
    int oldValue = this.keepWidth;
    this.keepWidth = keepWidth;
    if (aFrame.drawFlag && m_keep != 0) repaintHisto();
    firePropertyChange("keepWidth", oldValue, this.keepWidth);
  }
  public int getKeepWidth()
  {
    return this.keepWidth;
  }
  public void setHistogramPosition(int histogramPosition)
  {
	if (aFrame.histogramPosition == histogramPosition) return;
	int oldValue = aFrame.histogramPosition;
    aFrame.histogramPosition = histogramPosition;
    if (randomData == true)
    {
      rescaleFlag = 2;
      prepareDraw(); // rescale and reset limit
    }
    firePropertyChange("histogramPosition", oldValue, aFrame.histogramPosition);
  }
  public int getHistogramPosition()
  {
    return aFrame.histogramPosition;
  }
  public void setInvert(boolean invert)
  {
	if (this.invert == invert) return;
    this.invert = invert;
    if (randomData == true)
    {
      rescaleFlag = 2;
      prepareDraw(); // rescale and reset limit
    }
    firePropertyChange("invert", !this.invert, this.invert);
  }
  public boolean isInvert()
  {
    return this.invert;
  }

  public void setFrameBorderOn(boolean frameBorderOn)
  {
  if (this.frameBorderOn == frameBorderOn) return;
    this.frameBorderOn = frameBorderOn;
    if (randomData == true)
    {
      rescaleFlag = 2;
      prepareDraw(); // rescale and reset limit
    }
    firePropertyChange("frameBorderOn", !this.frameBorderOn, this.frameBorderOn);
  }
  public boolean isFrameBorderOn()
  {
    return this.frameBorderOn;
  }

  public void setRandomData(boolean randomData)
  {
	if (this.randomData == randomData) return;
    this.randomData = randomData;
    if (randomData)
    {
      rescaleFlag = 2;
      prepareDraw(); // rescale and reset limit
    }
    else
      clearScreen(-1, 0);

    firePropertyChange("randomData", !randomData, randomData);
  }
  public boolean isRandomData()
  {
    return this.randomData;
  }
  public void setRandomText(boolean randomText)
  {
	  if (this.randomText == randomText) return;
    this.randomText = randomText;
    if (randomData)
    {
      rescaleFlag = 2;
      prepareDraw(); // rescale and reset limit
    }
    else
      clearScreen(-1, 0);
    firePropertyChange("randomText", !randomText, randomText);
  }
  public boolean isRandomText()
  {
    return this.randomText;
  }
  public void setReuseScreen(boolean reuseScreen)
  {
	if (this.reuseScreen == reuseScreen) return;
    this.reuseScreen = reuseScreen;
    firePropertyChange("reuseScreen", !reuseScreen, reuseScreen);
  }
  public boolean isReuseScreen()
  {
    return this.reuseScreen;
  }
  public void setReuseText(boolean reuseText)
  {
	if (this.reuseText == reuseText) return;
    this.reuseText = reuseText;
    firePropertyChange("reuseText", !reuseText, reuseText);
  }
  public boolean isReuseText()
  {
    return this.reuseText;
  }
  /**
   * Set the capacity for the number of allowed display buffers
   * on the acop chart.
   *
   * @param screenDepth is the desired number of buffers.
   */
  public void setScreenDepth(int screenDepth)
  {
	if (this.screenDepth == screenDepth) return;

    int itmp = screenDepth;
    // if ( Beans.isDesignTime() == false ) return;
    // design mode
    if (itmp > AcopConst.MAX_DISPLAY)
      itmp = AcopConst.MAX_DISPLAY;
    else if (itmp < 1) itmp = 1;
    if (histo[itmp - 1] != null) return;
    screenDepth = itmp;
    int oldValue = this.screenDepth;
    n_display = screenDepth;
    this.screenDepth = screenDepth;
    firePropertyChange("screenDepth", oldValue, this.screenDepth);
  }
  public int getScreenDepth()
  {
    return this.screenDepth;
  }
  public void setTextDepth(int textDepth)
  {

    int itmp = textDepth;
    // if ( Beans.isDesignTime() == false ) return;
    // design mode
    if (itmp > AcopConst.MAX_TEXT)
      itmp = AcopConst.MAX_TEXT;
    else if (itmp < 1) itmp = 1;
    if (ptext[itmp - 1] != null) return;
    textDepth = itmp;
    if (this.textDepth == textDepth) return;
    n_text = textDepth;
    int oldValue = this.textDepth;
    this.textDepth = textDepth;
    firePropertyChange("textDepth", oldValue, this.textDepth);
  }
  public int getTextDepth()
  {
    return this.textDepth;
  }
  public void setWrapAround(boolean wrapAround)
  {
    int i;
    if (this.wrapAround == wrapAround) return;
    this.wrapAround = wrapAround;
    if (wrapAround) for (i = 0; i < n_display; i++)
      if (histo[i] != null && histo[i].displayON != 0)
        histo[i].getMaxWindow();
      else
        for (i = 0; i < n_display; i++)
          if (histo[i] != null && histo[i].displayON != 0) histo[i].wrapindex = 0;
    if (aFrame.drawFlag || textFlag)
    {
      // if ( wrapAround == false ) rescaleFlag = 1;
      // rescaleTextFlag = AcopConst.NEW_ZOOM_WINDOW;
      rescaleFlag = wrapAround == false ? 2 : 0;
      rescaleTextFlag = wrapAround == false ? AcopConst.NEW_ZOOM_WINDOW : 0;
      prepareDraw();
    }
    else if (wrapAround == false) drawTicks = true;
    firePropertyChange("wrapAround", !this.wrapAround, this.wrapAround);
  }
  public boolean isWrapAround()
  {
    return this.wrapAround;
  }
  /**
   * Sets the zooming criteria for the entire chart.
   *
   * Zooming within an acop chart is achieved by (intuitively) a left mouse-down
   * somewhere on the chart, dragging across to the opposite end of the desired
   * zoom rectangle and then releasing the mouse button.  The zoomed state will be
   * indicated by a small rectangle in the upper right hand corner of the chart's
   * display area.  The rectangle will either display the zoom-ratio or the
   * current zoomed buffer if buffered zooming is ON.
   *
   * Clicking on the small zoom rectangle will reverse zoom one-step (if buffered
   * zooming is on).  Double-Clicking on the small zoom rectangle will always
   * return completely from the zoomed state.
   *
   * The recognized zoomed region is given by the bits set within the zoom flag.
   *
   * @param zoomFlag is a flag containing the desired zooming critieria within
   * the lowest nibble.  These are
   * <p>0 - no zooming allowed
   * <p>1 - x-axis can be zoomed
   * <p>2 - y-axis can be zoomed
   * <p>3 - buffered zooming is ON
   * <p>The default zoomFlag is 0x07 (all bits ON), which allows both horizontal and
   * vertical zooming and turns buffered zooming ON.
   */
  public void setZooming(int zoomFlag)
  {
    if (this.zooming == zoomFlag) return;
    int oldValue = this.zooming;
    if (zoomFlag <= 7 && zoomFlag >= 4)
      zoomIndex = 0;
    else if (zoomFlag < 0 || zoomFlag > 3) zoomFlag = 0;
    if ((zoomFlag & 0x7) != 0) setZoomingWindow(zoomFlag & 0x4);
    if ((zoomFlag & 0x7) == 0 && isZoomed == true)
    {
      if (m_marker == 2) setmarkerflag(2); // restore marker array
        // prepareDraw(TRUE, -1, TRUE, nScaleText); // rescale and set max limit
      rescaleFlag = 2;
      rescaleTextFlag = AcopConst.NEW_ZOOM_WINDOW;
      prepareDraw();
      isZoomed = false;
      zoomXmaxFlag = 0;
      zoomIndex = 0;
    }
    this.zooming = zoomFlag;
    firePropertyChange("zooming", oldValue, this.zooming);
  }
  /**
   * Return the current zoom flag setting for the chart.
   *
   * @return the current zoom flag.
   *
   * @see setZooming()
   */
  public int getZooming()
  {
    return this.zooming;
  }
  /**
   * Sets the horizontal (x) axis label
   *
   * @param text is the text to apply to the x axis.
   */
  public void setXAxisLabel(java.lang.String text)
  {
	if (sca[0].AxisLabel != null && sca[0].AxisLabel.equals(text)) return;
	String oldValue = sca[0].AxisLabel;
    sca[0].AxisLabel = text;
    this.repaint(sca[0].labelRect.x, sca[0].labelRect.y, sca[0].labelRect.width, sca[0].labelRect.height);
    firePropertyChange("xAxisLabel", oldValue, sca[0].AxisLabel);
  }
  public java.lang.String getXAxisLabel()
  {
    return sca[0].AxisLabel;
  }
  /**
   * Sets the vertical (y) axis label
   *
   * @param text is the text to apply to the y axis.
   */
  public void setYAxisLabel(java.lang.String text)
  {
	  if (sca[1].AxisLabel != null && sca[1].AxisLabel.equals(text)) return;
	  String oldValue = sca[1].AxisLabel;
    sca[1].AxisLabel = text;
    this.repaint(sca[1].labelRect.x, sca[1].labelRect.y, sca[1].labelRect.width, sca[1].labelRect.height);
    firePropertyChange("yAxisLabel", oldValue, sca[1].AxisLabel);
  }
  public java.lang.String getYAxisLabel()
  {
    return sca[1].AxisLabel;
  }
  /**
   * Sets the secondary vertical (y) axis label
   *
   * @param text
   */
  public void setSecondaryYAxisLabel(java.lang.String text)
  {
	  if (sca[2].AxisLabel != null && sca[2].AxisLabel.equals(text)) return;
	  String oldValue = sca[2].AxisLabel;
	sca[2].AxisLabel = text;
    this.repaint(sca[2].labelRect.x,sca[2].labelRect.y,sca[2].labelRect.width,sca[2].labelRect.height);
    firePropertyChange("secondaryYAxisLabel", oldValue, sca[2].AxisLabel);
  }

  public java.lang.String getSecondaryYAxisLabel()
  {
	return sca[2].AxisLabel;
  }
  /**
   * Turns the 'best scale' logic on or off for the horizontal
   * (x) axis.
   *
   * If 'true' acop will find the nearest decimal settings for the
   * horizontal display axis, based on the current x-max and x-min
   * settings.
   *
   * @param useBestScale is the desired setting.
   */
  public void setXBestScale(boolean useBestScale)
  {
    if (sca[0].bestScale == useBestScale) return;
    sca[0].bestScale = useBestScale;
    firePropertyChange("xBestScale", !useBestScale, useBestScale);
    if (sca[0].log != 0) return;
    rescaleFlag = 2;
    rescaleTextFlag = AcopConst.NEW_ZOOM_WINDOW;
    prepareDraw(); // rescale and reset limit
  }
  /**
   * If 'true' acop will find the nearest decimal settings for the
   * horizontal display axis, based on the current x-max and x-min
   * settings.
   *
   * @return the current setting for this value
   */
  public boolean isXBestScale()
  {
    return sca[0].bestScale;
  }
  /**
   * Turns the 'best scale' logic on or off for the vertical
   * (y) axis.
   *
   * If 'true' acop will find the nearest decimal settings for the
   * vertical display axis, based on the current y-max and y-min
   * settings.
   *
   * @param useBestScale is the desired setting.
   */
  public void setYBestScale(boolean useBestScale)
  {
	if (sca[1].bestScale == useBestScale) return;
    sca[1].bestScale = useBestScale;
    firePropertyChange("yBestScale", !sca[1].bestScale, sca[1].bestScale);
    if (sca[1].log != 0) return;
    rescaleFlag = 2;
    rescaleTextFlag = AcopConst.NEW_ZOOM_WINDOW;
    prepareDraw(); // rescale and reset limit
  }
  /**
   * If 'true' acop will find the nearest decimal settings for the
   * vertical display axis, based on the current y-max and y-min
   * settings.
   *
   * @return the current setting for this value
   */
  public boolean isYBestScale()
  {
    return sca[1].bestScale;
  }
  /**
   * Turns the 'best scale' logic on or off for the secondary vertical
   * (y) axis.
   *
   * If 'true' acop will find the nearest decimal settings for the
   * secondary vertical display axis, based on the current secondary y-max and y-min
   * settings.
   *
   * @param useBestScale is the desired setting.
   */
  public void setSecondaryYBestScale(boolean useBestScale)
  {
	if (sca[2].bestScale == useBestScale) return;
    sca[2].bestScale = useBestScale;
    if (sca[2].log != 0) return;
    rescaleFlag = 2;
    rescaleTextFlag = AcopConst.NEW_ZOOM_WINDOW;
    prepareDraw(); // rescale and reset limit
    firePropertyChange("secondaryYBestScale", !useBestScale, useBestScale);
  }
  /**
   * If 'true' acop will find the nearest decimal settings for the
   * vertical display axis, based on the current secondary y-max and y-min
   * settings.
   *
   * @return the current setting for this value
   */
  public boolean isSecondaryYBestScale()
  {
    return sca[2].bestScale;
  }
  public void setXExtent(double XExtent)
  {
    if (this.XExtent == XExtent) return;
	double oldValue = this.XExtent;
    this.XExtent = XExtent;
    if (randomData == true)
    {
      rescaleFlag = 2;
      prepareDraw(); // rescale and reset limit
    }
    firePropertyChange("xExtent", oldValue, XExtent);
  }
  public double getXExtent()
  {
    return this.XExtent;
  }
  public void setYExtent(double YExtent)
  {
	  if (this.YExtent == YExtent) return;
	  double oldValue = this.YExtent;
    this.YExtent = YExtent;
    if (randomData == true)
    {
      rescaleFlag = 2;
      prepareDraw(); // rescale and reset limit
    }
    firePropertyChange("yExtent", oldValue, this.YExtent);
  }
  public double getYExtent()
  {
    return this.YExtent;
  }
  public void setXFineTicks(int XFineTicks)
  {
	  if (sca[0].fineTicks == XFineTicks) return;
	  int oldValue = sca[0].fineTicks;
    sca[0].fineTicks = XFineTicks;
    if (randomData == true || sca[0].log == 0)
    {
      rescaleFlag = 2;
      prepareDraw(); // rescale and reset limit
    }
    firePropertyChange("xFineTicks", oldValue, sca[0].fineTicks);
  }
  public int getXFineTicks()
  {
    return sca[0].fineTicks;
  }
  public void setYFineTicks(int YFineTicks)
  {
	if (sca[1].fineTicks == YFineTicks) return;
	int oldValue = sca[1].fineTicks;
    sca[1].fineTicks = YFineTicks;
    if (randomData == true || sca[1].log == 0)
    {
      rescaleFlag = 2;
      prepareDraw(); // rescale and reset limit
    }
    firePropertyChange("yFineTicks", oldValue, sca[1].fineTicks);
  }
  public int getYFineTicks()
  {
    return sca[1].fineTicks;
  }
  public void setSecondaryYFineTicks(int SecondaryYFineTicks)
  {
	  if (sca[2].fineTicks == SecondaryYFineTicks) return;
	  int oldValue = sca[2].fineTicks;
    sca[2].fineTicks = SecondaryYFineTicks;
    if (randomData == true || sca[2].log == 0)
    {
      rescaleFlag = 2;
      prepareDraw(); // rescale and reset limit
    }
    firePropertyChange("secondaryYFineTicks", oldValue, sca[2].fineTicks);
  }
  public int getSecondaryYFineTicks()
  {
    return sca[2].fineTicks;
  }
  public void setXGrid(boolean XGrid)
  {
	  if (sca[0].grid == XGrid) return;
    sca[0].grid = XGrid;
    repaintHisto();
    firePropertyChange("xGrid", !XGrid, XGrid);
  }
  public boolean isXGrid()
  {
    return sca[0].grid;
  }
  public void setYGrid(boolean YGrid)
  {
	  if (sca[1].grid == YGrid) return;
    sca[1].grid = YGrid;
    repaintHisto();
    firePropertyChange("yGrid", !sca[1].grid, sca[1].grid);
  }
  public boolean isYGrid()
  {
    return sca[1].grid;
  }
  public void setXScale(double XScale)
  {
	  if (this.XScale == XScale) return;
	  double oldValue = this.XScale;
    this.XScale = XScale;
    if (randomData == true)
    {
      rescaleFlag = 2;
      prepareDraw(); // rescale and reset limit
    }
    firePropertyChange("xScale", oldValue, this.XScale);
  }
  public double getXScale()
  {
    return this.XScale;
  }
  public void setYScale(double YScale)
  {
	  if (this.YScale == YScale) return;
	  double oldValue = this.YScale;
    this.YScale = YScale;
    if (randomData == true)
    {
      rescaleFlag = 2;
      prepareDraw(); // rescale and reset limit
    }
    firePropertyChange("yScale", oldValue, this.YScale);
  }
  public double getYScale()
  {
    return this.YScale;
  }
  public void setXShift(double XShift)
  {
	  if (this.XShift == XShift) return;
	  double oldValue = this.XShift;
    this.XShift = XShift;
    if (randomData == true)
    {
      rescaleFlag = 2;
      prepareDraw(); // rescale and reset limit
    }
    firePropertyChange("xShift", oldValue, this.XShift);
  }
  public double getXShift()
  {
    return this.XShift;
  }
  public void setYShift(double YShift)
  {
	  if (this.YShift == YShift) return;
	  double oldValue = this.YShift;
    this.YShift = YShift;
    if (randomData == true)
    {
      rescaleFlag = 2;
      prepareDraw(); // rescale and reset limit
    }
    firePropertyChange("yShift", oldValue, this.YShift);
  }
  public double getYShift()
  {
    return this.YShift;
  }
  public void setXTicks(int XTicks)
  {
	  if (sca[0].ticks == XTicks) return;
	  int oldValue = sca[0].ticks;
    sca[0].ticks = XTicks;
    if (randomData == true || sca[0].log == 0)
    {
      rescaleFlag = 2;
      prepareDraw(); // rescale and reset limit
    }
    firePropertyChange("xTicks", oldValue, sca[0].ticks);
  }
  public int getXTicks()
  {
    return sca[0].ticks;
  }
  public void setYTicks(int YTicks)
  {
	  if (sca[1].ticks == YTicks) return;
	  int oldValue = sca[1].ticks;
    sca[1].ticks = YTicks;
    if (randomData == true || sca[1].log == 0)
    {
      rescaleFlag = 2;
      prepareDraw(); // rescale and reset limit
    }
    firePropertyChange("yTicks", oldValue, sca[1].ticks);
  }
  public int getYTicks()
  {
    return sca[1].ticks;
  }
  public void setSecondaryYTicks(int SecondaryYTicks)
  {
	  if (sca[2].ticks == SecondaryYTicks) return;
	  int oldValue = sca[2].ticks;
    sca[2].ticks = SecondaryYTicks;
    if (randomData == true || sca[2].log == 0)
    {
      rescaleFlag = 2;
      prepareDraw(); // rescale and reset limit
    }
    firePropertyChange("secondaryYTicks", oldValue, sca[2].ticks);
  }
  public int getSecondaryYTicks()
  {
    return sca[2].ticks;
  }
  public void setXTicksBottom(boolean XTicksBottom)
  {
    if (sca[0].leftBottom == XTicksBottom) return;
    sca[0].leftBottom = XTicksBottom;
    setTickRectangle(0);
    this.repaint();
    firePropertyChange("xTicksBottom", !sca[0].leftBottom, sca[0].leftBottom);
  }
  public boolean getXTicksBottom()
  {
    return sca[0].leftBottom;
  }
  public void setYTicksLeft(boolean YTicksLeft)
  {
    if (sca[1].leftBottom == YTicksLeft) return;
    sca[1].leftBottom = YTicksLeft;
    setTickRectangle(1);
    this.repaint();
    firePropertyChange("yTicksLeft", !sca[1].leftBottom, sca[1].leftBottom);
  }
  public boolean getYTicksLeft()
  {
    return sca[1].leftBottom;
  }
  public void setXTickText(boolean XTickText)
  {
	if (sca[0].ticktext == XTickText) return;
    sca[0].ticktext = XTickText;
    this.repaint(sca[0].ticksRect.x, sca[0].ticksRect.y, sca[0].ticksRect.width, sca[0].ticksRect.height);
    firePropertyChange("xTickText", !sca[0].ticktext, sca[0].ticktext);
  }
  public boolean isXTickText()
  {
    return sca[0].ticktext;
  }
  public void setYTickText(boolean YTickText)
  {
	  if (sca[1].ticktext == YTickText) return;
    sca[1].ticktext = YTickText;
    this.repaint(sca[1].ticksRect.x, sca[1].ticksRect.y, sca[1].ticksRect.width, sca[1].ticksRect.height);
    firePropertyChange("yTickText", !sca[1].ticktext, sca[1].ticktext);
  }
  public boolean isYTickText()
  {
    return sca[1].ticktext;
  }
  public void setSecondaryYTickText(boolean SecondaryYTickText)
  {
	  if (sca[2].ticktext == SecondaryYTickText) return;
	sca[2].ticktext = SecondaryYTickText;
    this.repaint(sca[2].ticksRect.x,sca[2].ticksRect.y,sca[2].ticksRect.width,sca[2].ticksRect.height);
    firePropertyChange("secondaryYTickText", !SecondaryYTickText, SecondaryYTickText);
  }
  public boolean isSecondaryYTickText()
  {
	return sca[2].ticktext;
  }
  /**
   * Binds or un-binds the given scale to the secondary vertical (y) axis coordinates.
   *
   * @param hDisplay is the handle to the acop plot to which the auto-scale
   * should be applied (the value returned from the original call to the
   * draw() method).
   * @param bind determines whether the display is bound to the secondary
   * vertical (y) axis or not.
   *
   * @return 0 upon success, -1 upon failure.
   */
  public int bindToSecondaryYScale(int hDisplay, boolean bind)
  { int ret = 0;
    if (hDisplay < 0 || hDisplay >= n_display) return -1;
    if (histo[hDisplay] != null && histo[hDisplay].displayON != 0 && histo[hDisplay].lockFlag == 0)
    {
      histo[hDisplay].bindSecondaryY = bind;
      ret = bind ? 1 : 0;
      prepareDraw();
    }
    else ret = -1;
    return ret;
  }
  /**
   * Binds the given scale to the secondary vertical (y) axis coordinates.
   *
   * @param hDisplay is the handle to the acop plot to which the auto-scale
   * should be applied (the value returned from the original call to the
   * draw() method).
   * @return 0 upon success, -1 upon failure.
   */
  public int bindToSecondaryYScale(int hDisplay)
  {
    return bindToSecondaryYScale(hDisplay, true);
  }
  public void setColorFilled(boolean colorFilled)
  {
	if (this.colorFilled == colorFilled) return;
    this.colorFilled = colorFilled;
    if (randomData == true)
    {
      rescaleFlag = 2;
      prepareDraw(); // rescale and reset limit
    }
    firePropertyChange("colorFilled", !this.colorFilled, colorFilled);
  }
  public boolean isColorFilled()
  {
    return this.colorFilled;
  }
  /**
   * If the graph mode is 'LIN-TIME' or 'LOG-TIME' (i.e. the
   * horizontal axis refers to time, with values as UTC numbers
   * representing seconds since Jan. 1 1970, this method can
   * control the placement of the x-axis time lable.  By default
   * the time label is left hand side of the chart.  Call this
   * method passing 'false' to place the time label on the
   * right hand side of the chart.
   *
   * @param leftTimeLabel is the desired setting for the
   * left hand time label.  A value of 'false' will place
   * the time label on the right hand side of the plot.
   */
  public void setLeftTimeLabel(boolean leftTimeLabel)
  {
	  if (aFrame.leftTimeLabel == leftTimeLabel) return;
    aFrame.leftTimeLabel = leftTimeLabel;
    if (sca[0].xtime != 0)
    {
      sca[0].scaleToGrid();
      // if ( fBottom > 0 ) Invalidate(&xRect);
      repaint(xRect.x, xRect.y, xRect.width, xRect.height);
    }
    firePropertyChange("leftTimeLabel", !leftTimeLabel, leftTimeLabel);
  }
  public boolean isLeftTimeLabel()
  {
    return aFrame.leftTimeLabel;
  }
  public void setAbsTimeDisplay(boolean absTimeDisplay)
  {
    if (aFrame.absTimeDisplay == absTimeDisplay) return;
    aFrame.absTimeDisplay = absTimeDisplay;
    if (sca[0].xtime != 0)
    {
      sca[0].scaleToGrid();
    }
    firePropertyChange("absTimeDisplay", !absTimeDisplay, absTimeDisplay);
  }
  public boolean isAbsTimeDisplay()
  {
    return aFrame.absTimeDisplay;
  }
  public void setLeftYLabel(boolean leftYLabel)
  {
    if (sca[1].leftBottLabel == leftYLabel) return;
    sca[1].leftBottLabel = leftYLabel;
    setTickRectangle(1);
    if (aFrame.drawFlag == false || randomData == false) return;
    this.repaint();
    firePropertyChange("leftYLabel", !leftYLabel, leftYLabel);
  }
  public boolean isLeftYLabel()
  {
    return sca[1].leftBottLabel;
  }
  public void setBottomXLabel(boolean bottomXLabel)
  {
    if (sca[0].leftBottLabel == bottomXLabel) return;
    sca[0].leftBottLabel = bottomXLabel;
    setTickRectangle(0);
    firePropertyChange("bottomXLabel", !sca[0].leftBottLabel, sca[0].leftBottLabel);
    if (aFrame.drawFlag == false || randomData == false) return;
    this.repaint();
  }
  public boolean isBottomXLabel()
  {
    return sca[0].leftBottLabel;
  }
  public void setBarlineColorFilled(int barlineColorFilled)
  {
    if (this.barlineColorFilled == barlineColorFilled || this.barlineColorFilled * barlineColorFilled > 0 ) return;
    int oldValue = this.barlineColorFilled;
    this.barlineColorFilled = barlineColorFilled;
    if (aFrame.drawFlag) repaintHisto();
    firePropertyChange("barlineColorFilled", oldValue, this.barlineColorFilled);
  }
  public int getBarlineColorFilled()
  {
    return this.barlineColorFilled;
  }
  public void setPrintTextStickyAttribute(int printTextStickyAttribute)
  {
    if (this.printTextStickyAttribute == printTextStickyAttribute) return;
    int oldValue = this.printTextStickyAttribute;
    this.printTextStickyAttribute = printTextStickyAttribute;
    firePropertyChange("printTextStickyAttribute", oldValue, this.printTextStickyAttribute);
  }
  public int getPrintTextStickyAttribute()
  {
    return this.printTextStickyAttribute;
  }
  public void setPrintTextLocation(int printTextLocation)
  {
    if (this.printTextLocation == printTextLocation) return;
    int oldValue = this.printTextLocation;
    this.printTextLocation = printTextLocation;
    firePropertyChange("printTextLocation", oldValue, this.printTextLocation);
  }
  public int getPrintTextLocation()
  {
    return this.printTextLocation;
  }
  public void setPrintTextRotation(int printTextRotation)
  {
    if (this.printTextRotation == printTextRotation) return;
    int oldValue = this.printTextRotation;
    this.printTextRotation = printTextRotation;
    firePropertyChange("printTextRotation", oldValue, this.printTextRotation);
  }
  public int getPrintTextRotation()
  {
    return this.printTextRotation;
  }
  public void setXlabelRotation(int xlabelRotation)
  {
    xlabelRotation = xlabelRotation % 360;
    if (this.xlabelRotation == xlabelRotation) return;
    int oldValue = this.xlabelRotation;
    this.xlabelRotation = xlabelRotation;
    if (xlabelRotation == 90 || xlabelRotation == -270)
      xlabelAxis = 1;
    else if (xlabelRotation == 180 || xlabelRotation == -180)
      xlabelAxis = 2;
    else if (xlabelRotation == 270 || xlabelRotation == -90)
      xlabelAxis = 3;
    else
      xlabelAxis = 0;
    xlabelRotateGrad = xlabelRotation * Math.PI / 180;
    xlabelcos = Math.cos(xlabelRotateGrad);
    xlabelsin = Math.sin(xlabelRotateGrad);
    if (xlabelsin > 0)
      xlabelQudrant = xlabelcos < 0 ? 2 : 1;
    else
      xlabelQudrant = xlabelcos < 0 ? 3 : 4;
    firePropertyChange("xLabelRotation", oldValue, this.xlabelRotation);
  }
  public int getXlabelRotation()
  {
    return this.xlabelRotation;
  }
  public int leadingEdge(int LeadingEdgeMode, int hDisplay)
  {
    int num, ende;
    if (LeadingEdgeMode < 0 || LeadingEdgeMode > 3) LeadingEdgeMode = 0;
    if (hDisplay < 0 || hDisplay >= n_display)
    {
      num = 0;
      ende = n_display - 1;
    }
    else
      ende = num = n_display;
    for (; num <= ende; num++)
    {
      if (histo[hDisplay] == null) continue;
      histo[num].edgeMode = LeadingEdgeMode;
    }
    return LeadingEdgeMode;
  }
  public int leadingEdge(int hDisplay)
  {
    if (hDisplay < 0 || hDisplay >= n_display)
      return LeadingEdgeMode;
    else if (histo[hDisplay] == null)
      return 0;
    else
      return histo[hDisplay].edgeMode;
  }
  public void setLeadingEdgeColor(java.awt.Color LeadingEdgeColor)
  {
    if (this.LeadingEdgeColor == LeadingEdgeColor) return;
    Color oldValue = this.LeadingEdgeColor;
    this.LeadingEdgeColor = LeadingEdgeColor;
    firePropertyChange("leadingEdgeColor", oldValue, this.LeadingEdgeColor);
  }
  public java.awt.Color getLeadingEdgeColor()
  {
    return this.LeadingEdgeColor;
  }
  public void setLeadingEdgeGapSize(int LeadingEdgeGapSize)
  {
    if (this.LeadingEdgeGapSize == LeadingEdgeGapSize) return;
    int oldValue = this.LeadingEdgeGapSize;
    this.LeadingEdgeGapSize = LeadingEdgeGapSize;
    firePropertyChange("leadingEdgeGapSize", oldValue, this.LeadingEdgeGapSize);
  }
  public int getLeadingEdgeGapSize()
  {
    return this.LeadingEdgeGapSize;
  }
  public void setLeadingEdgeMotion(int LeadingEdgeMotion)
  {
    if (this.LeadingEdgeMotion == LeadingEdgeMotion) return;
    int oldValue = this.LeadingEdgeMotion;
    this.LeadingEdgeMotion = LeadingEdgeMotion;
    if (aFrame.drawFlag) repaintHisto();
    firePropertyChange("leadingEdgeMotion", oldValue, this.LeadingEdgeMotion);
  }
  public int getLeadingEdgeMotion()
  {
    return this.LeadingEdgeMotion;
  }
  public void setLeadingEdgeMode(int LeadingEdgeMode)
  {
    if (LeadingEdgeMode < 0 || LeadingEdgeMode > 3) LeadingEdgeMode = 0;
    if (this.LeadingEdgeMode == LeadingEdgeMode) return;
    int oldValue = this.LeadingEdgeMode;
    this.LeadingEdgeMode = LeadingEdgeMode;
    if (aFrame.drawFlag) repaintHisto();
    firePropertyChange("leadingEdgeMode", oldValue, this.LeadingEdgeMode);
  }
  public int getLeadingEdgeMode()
  {
    return this.LeadingEdgeMode;
  }
  public void setLeadingEdgeWidth(int LeadingEdgeWidth)
  {
    if (LeadingEdgeWidth < 0) LeadingEdgeWidth = 1;
    if (this.LeadingEdgeWidth == LeadingEdgeWidth) return;
    int oldValue = this.LeadingEdgeWidth;
    this.LeadingEdgeWidth = LeadingEdgeWidth;
    firePropertyChange("leadingEdgeWidth", oldValue, this.LeadingEdgeWidth);
  }
  public int getLeadingEdgeWidth()
  {
    return this.LeadingEdgeWidth;
  }
  public void setLeadingEdgeStyle(int LeadingEdgeStyle)
  {
    if (LeadingEdgeStyle < AcopConst.PS_SOLID) LeadingEdgeStyle = AcopConst.PS_SOLID;
    if (this.LeadingEdgeStyle == LeadingEdgeStyle) return;
    int oldValue = this.LeadingEdgeStyle;
    this.LeadingEdgeStyle = LeadingEdgeStyle;
    firePropertyChange("leadingEdgeStyle", oldValue, this.LeadingEdgeStyle);
  }
  public int getLeadingEdgeStyle()
  {
    return this.LeadingEdgeStyle;
  }
  public void setLeadingEdgeXExtent(double LeadingEdgeXExtent)
  {
    if (Math.abs(this.LeadingEdgeXExtent - LeadingEdgeXExtent) < AcopConst.zero_check) return;
    double oldValue = this.LeadingEdgeXExtent;
    this.LeadingEdgeXExtent = LeadingEdgeXExtent;
    firePropertyChange("leadingEdgeXExtent", oldValue, this.LeadingEdgeXExtent);
  }
  public double getLeadingEdgeXExtent()
  {
    return this.LeadingEdgeXExtent;
  }
  public void setLeadingEdgeYExtent(double LeadingEdgeYExtent)
  {
    if (Math.abs(this.LeadingEdgeYExtent - LeadingEdgeYExtent) < AcopConst.zero_check) return;
    double oldValue = this.LeadingEdgeYExtent;
    this.LeadingEdgeYExtent = LeadingEdgeYExtent;
    firePropertyChange("leadingEdgeYExtent", oldValue, this.LeadingEdgeYExtent);
  }
  public double getLeadingEdgeYExtent()
  {
    return this.LeadingEdgeYExtent;
  }
  /**
   * Applies an auto-scale to the entire acop chart according to the given
   * axis specifications
   *
   * @param xmx if 'true' will set the horizontal (x) axis
   * maximum value to the maximum value of any and all display plots
   * @param xmn if 'true' will set the horizontal (x) axis
   * minimum value to the minimum value of any and all display plots
   * @param ymx if 'true' will set the vertical (y) axis
   * maximum value to the maximum value of any and all display plots
   * @param ymn if 'true' will set the vertical (y) axis
   * minimum value to the minimum value of any and all display plots
   */
  public void autoScale(boolean xmx, boolean xmn, boolean ymx, boolean ymn)
  {
    autoScale(xmx, xmn, ymx, ymn, -1);
  }
  /**
   * Applies an auto-scale to the designated acop plot according to the given
   * axis specifications
   *
   * @param xmx if 'true' will set the horizontal (x) axis
   * maximum value to the maximum value of any and all display plots
   * @param xmn if 'true' will set the horizontal (x) axis
   * minimum value to the minimum value of any and all display plots
   * @param ymx if 'true' will set the vertical (y) axis
   * maximum value to the maximum value of any and all display plots
   * @param ymn if 'true' will set the vertical (y) axis
   * minimum value to the minimum value of any and all display plots
   * @param hDisplay is the handle to the acop plot to which the auto-scale
   * should be applied (the value returned from the original call to the
   * draw() method). Passing a negative value will auto-scale all
   * plots managed by the chart.
   */
  public void autoScale(boolean xmx, boolean xmn, boolean ymx, boolean ymn, int hDisplay)
  {
    double autotmp, checkXmin, checkXmax, xMax, xMin;
    double[] yMin = new double[2];
    double[] yMax = new double[2];
    double[] autotmpY = new double[2];
    int [] yFlags = new int[2];
    int [] autoflgY = new int[2];
    int i, j, autoflg, start, stop, yIndex;
    if (!xmx && !xmn && !ymx && !ymn) return;
    if (histo[m_histstart] == null) return;
    if (hDisplay < -1 || hDisplay >= n_display) return;
    if (hDisplay != -1 && histo[hDisplay] == null) return;
    if (hDisplay != -1)
    {
      start = hDisplay;
      stop = hDisplay + 1;
//      yIndex = histo[hIndex].bindSecondaryY ? 2 : 1;
    }
    else
    {
      start = 0;
      stop = n_display;
    }
    xMax = sca[0].usermax;
    if (xmx) for (i = start, autoflg = 0; i < stop && histo[i] != null; i++)
    {
//      if (i != m_histstart && histo[i].displayON == 0) continue;
    	if (histo[i].displayON == 0) continue;
      autotmp = histo[i].maxOf(histo[i].x);
      if (autoflg == 0)
      {
        xMax = autotmp;
        autoflg = 1;
      }
      else if (xMax < autotmp) xMax = autotmp;
    }
    xMin = sca[0].usermin;
    if (xmn) for (i = start, autoflg = 0; i < stop && histo[i] != null; i++)
    {
//      if (i != m_histstart && histo[i].displayON == 0) continue;
    	if (histo[i].displayON == 0) continue;
      autotmp = histo[i].minOf(histo[i].x);
      if (autoflg == 0)
      {
        xMin = autotmp;
        autoflg = 1;
      }
      else if (xMin > autotmp) xMin = autotmp;
    }
    for (i=0;i<2;i++)
    {
      yFlags[i] = autoflgY[i] = 0;
      yMax[i] = sca[i+1].usermax;
    }
//    yMax = sca[yIndex].usermax;
    if (ymx) for (i = start; i < stop && histo[i] != null; i++)
    {
//      if (i != m_histstart && histo[i].displayON == 0) continue;
    	if (histo[i].displayON == 0) continue;
      yIndex = histo[i].bindSecondaryY ? 1 : 0;
      yFlags[yIndex]++;
      if (xmx && xmn)
        autotmpY[yIndex] = histo[i].maxOf(histo[i].y);
      else
      {
        checkXmin = sca[0].getMin();
        checkXmax = sca[0].getMax();
        autotmpY[yIndex] = -1.7E308;
        int mode = histo[i].dismode;
        if (mode == AcopDisplayMode.Status.ordinal() || mode == AcopDisplayMode.Annotation.ordinal()) {
            continue;
        }
        for (j = 0; j < histo[i].npoints; j++)
        {
          if (histo[i].disable[j] != 0) continue;
          if (xmx == false && histo[i].x[j] > checkXmax) continue;
          if (xmn == false && histo[i].x[j] < checkXmin) continue;
          if (histo[i].y[j] > autotmpY[yIndex]) autotmpY[yIndex] = histo[i].y[j];
        }
      }
      if (autoflgY[yIndex] == 0)
      {
        yMax[yIndex] = autotmpY[yIndex];
        autoflgY[yIndex] = 1;
      }
      else if (yMax[yIndex] < autotmpY[yIndex]) yMax[yIndex] = autotmpY[yIndex];
    }
    for (i=0;i<2;i++)
    {
      autoflgY[i] = 0;
      yMin[i] = sca[i+1].usermin;
    }
//    yMin = sca[yIndex].usermin;
    if (ymn)
      for (i = start; i < stop && histo[i] != null; i++)
      {
//        if (i != m_histstart && histo[i].displayON == 0) continue;
    	  if (histo[i].displayON == 0) continue;
        yIndex = histo[i].bindSecondaryY ? 1 : 0;
        yFlags[yIndex]++;
        if (xmx && xmn)
          autotmpY[yIndex] = histo[i].minOf(histo[i].y);
        else
        {
          checkXmin = sca[0].getMin();
          checkXmax = sca[0].getMax();
          autotmpY[yIndex] = 1.7E308;
          int mode = histo[i].dismode;
          if (mode == AcopDisplayMode.Status.ordinal() || mode == AcopDisplayMode.Annotation.ordinal()) {
              continue;
          }
          for (j = 0; j < histo[i].npoints; j++)
          {
            if (histo[i].disable[j] != 0) continue;
            if (xmx == false && histo[i].x[j] > checkXmax) continue;
            if (xmn == false && histo[i].x[j] < checkXmin) continue;
            if (histo[i].y[j] < autotmpY[yIndex])
              if (sca[yIndex+1].log == 0 || histo[i].y[j] > 1.0e-30) autotmpY[yIndex] = histo[i].y[j];
          }
        }
        if (autoflgY[yIndex] == 0)
        {
          yMin[yIndex] = autotmpY[yIndex];
          autoflgY[yIndex] = 1;
        }
        else if (yMin[yIndex] > autotmpY[yIndex]) yMin[yIndex] = autotmpY[yIndex];
      }
    if (xMax <= xMin + AcopConst.zero_check)
    {
      autotmp = Math.abs(xMin);
      if (autotmp < AcopConst.zero_check)
      {
        xMin -= 5;
        xMax = xMin + 10;
      }
      else
      {
        xMin -= (0.5 * autotmp);
        xMax = xMin + autotmp;
      }
    }
    for (i = 0; i < 2; i++)
    {
      if ( yFlags[i] == 0 ) continue;
      if (yMax[i] <= yMin[i] + AcopConst.zero_check)
      {
        if ( !ymn && ymx )
        {
          autotmp = Math.abs(yMin[i]);
          if (autotmp < AcopConst.zero_check) yMax[i] = yMin[i] + 10;
          else yMax[i] = yMin[i] + Math.abs(yMin[i]);
        }
        else if ( ymn && !ymx )
        {
          autotmp = Math.abs(yMax[i]);
          if (autotmp < AcopConst.zero_check) yMin[i] = yMax[i] - 10;
          else yMin[i] = yMax[i] - Math.abs(yMax[i]);
        }
        else
        {
          autotmp = Math.abs(yMin[i]);
          if (autotmp < AcopConst.zero_check)
          {
            yMin[i] -= 5;
            yMax[i] = yMin[i] + 10;
          }
          else
          {
            yMin[i] -= (0.5 * autotmp);
            yMax[i] = yMin[i] + autotmp;
          }
        }
      }
    }
    if (xmx || xmn) sca[0].setUserscale(xMin, xMax);
    if (ymx || ymn) for (i=0; i<2; i++) if (yFlags[i] != 0) sca[i+1].setUserscale(yMin[i], yMax[i]);
    // prepareDraw(TRUE, -1, TRUE, 0);
    rescaleFlag = 2;
    rescaleTextFlag = AcopConst.NEW_ZOOM_WINDOW;
    prepareDraw();
  }
  protected void onRButtonDown(Point point, MouseEvent trigger)
  {
    if (aFrame.drawFlag == false && textFlag == false) return;
    if (aFrame.drawFlag && histo[m_histstart].dismode == AcopConst.mode_textbox) return;
    if ((isZoomed || (wrapAround && sca[0].log == 0)) && histRoundRect.contains(point))
    {
      movie = true;
      moviepoint = point;
      saveBestFlag = (sca[0].bestScale ? 1 : 0) + (sca[1].bestScale ? 2 : 0);
      sca[0].bestScale = false;
      sca[1].bestScale = false;
    }
    if (isZoomed) rButtonPoint = point;
  }
  private void doBufferedZoom(MouseEvent trigger)
  {
    int i;
    if ((zooming & 0x4) != 0 && zoomIndex > 0 )
    {
      zoomIndex--;
      for (i = 0; i < 3; i++)
      {
        sca[i].setDispscale(bufferedMin[zoomIndex][i], bufferedMax[zoomIndex][i]);
        if (sca[i].log != 0) sca[i].setDispscale(Math.pow(10.0, sca[i].min), Math.pow(10.0, sca[i].max));
      }
      bufferedZoomRatio = "Z" + zoomIndex;
      if (zoomIndex != 0) sca[0].scaleToGrid();
    }
    if (m_marker == 2) setmarkerflag(2); // recalculate marker array
    rescaleFlag = zoomIndex == 0 ? 2 : 1;
    rescaleTextFlag = AcopConst.NEW_ZOOM_WINDOW;
    prepareDraw(); // rescale and not set max limit
    if (zoomIndex == 0) isZoomed = false;
    notifyAcop(AcopEvent.ACOP_MOUSE_ZOOM_EVENT,trigger);
  }
  protected void onRButtonUp(Point point,MouseEvent trigger)
  {
    if (movie == false)
    {
      if (!isZoomed || (zooming & 0x4) == 0 || rButtonPoint != point) return;
      doBufferedZoom(trigger);
      return;
    }
    movie = false;
    if (saveBestFlag != 0)
    {
      if ((saveBestFlag & 0x1) != 0) sca[0].bestScale = true;
      if ((saveBestFlag & 0x2) != 0) sca[1].bestScale = true;
    }
  }
  protected void onMouseMove(Point point, MouseEvent trigger)
  {
    double x, y, xcheckmin, xcheckmax;
    boolean reScale;
    int i;
    double[] low = new double[3];
    double[] high = new double[3];
    reScale = true;
    // from log back to nomal for re-scaling
    for (i = 0; i < 3; i++)
    {
      if (i == 2 && sca[i].OnFlag == false) continue;
      low[i] = sca[i].getMin();
      high[i] = sca[i].getMax();
      // x = sca[0].getMin();
      // tx= sca[0].getMax();
      // y = sca[1].getMin();
      // ty= sca[1].getMax();
    }
    if (wrapAround == false || sca[0].log != 0)
    {
      xcheckmin = sca[0].usermin;
      xcheckmax = sca[0].usermax;
    }
    else
    {
      xcheckmin = wrapxmin - sca[0].dispSize - AcopConst.zero_check;
      xcheckmax = wrapxmax + sca[0].dispSize + AcopConst.zero_check;
    }
    if ((point.x < moviepoint.x && low[0] > xcheckmin) || (point.x > moviepoint.x && high[0] < xcheckmax)
        || (point.y > moviepoint.y && low[1] > sca[1].usermin)
        || (point.y < moviepoint.y && high[1] < sca[1].usermax))
    {
      for (i = 0; i < 3; i++)
      {
        if (i == 2 && sca[i].OnFlag == false) continue;
        if (sca[i].log != 0) sca[i].setDispscale(low[i], high[i]);
      }
      x = sca[0].dispSize * moviePercent; // shift 10 percent
      if (point.x < moviepoint.x)
      {
        if (sca[0].log != 0)
        {
          if ((++xCnt < movieLogCnt) || (sca[0].usermin < AcopConst.zero_check && sca[0].min < 0.1)
              || sca[0].min / 10 < sca[0].usermin)
            reScale = false;
          else
          {
            sca[0].setDispscale(sca[0].min / 10, sca[0].max / 10);
            xCnt = 0;
          }
        }
        else
        {
          if (sca[0].min - x < xcheckmin) x = sca[0].min - xcheckmin;
          sca[0].setDispscale(sca[0].min - x, sca[0].max - x);
          if (wrapAround && sca[0].max <= wrapxmin)
            sca[0].setDispscale(wrapxmax - sca[0].dispSize, wrapxmax);
        }
      }
      else if (point.x > moviepoint.x)
      {
        if (sca[0].log != 0)
        {
          if ((--xCnt <= -movieLogCnt) && sca[0].max * 10 <= sca[0].usermax)
          {
            sca[0].setDispscale(sca[0].min * 10, sca[0].max * 10);
            xCnt = 0;
          }
          else
            reScale = false;
        }
        else
        {
          if (sca[0].max + x > xcheckmax) x = xcheckmax - sca[0].max;
          sca[0].setDispscale(sca[0].min + x, sca[0].max + x);
          if (wrapAround && sca[0].min >= wrapxmax)
            sca[0].setDispscale(wrapxmin, wrapxmin + sca[0].dispSize);
        }
      }
      for (i = 1; i < 3; i++) // loop y and y2
      {
        if (i == 2 && sca[i].OnFlag == false) continue;
        y = sca[i].dispSize * moviePercent; // shift 10 percent
        if (point.y > moviepoint.y)
        {
          if (sca[i].log != 0)
          {
            if ((++yCnt < movieLogCnt) || (sca[i].usermin < AcopConst.zero_check && sca[i].min < 0.1)
                || sca[i].min / 10 < sca[i].usermin)
              reScale = false;
            else
            {
              sca[i].setDispscale(sca[i].min / 10, sca[i].max / 10);
              yCnt = 0;
            }
          }
          else
          {
            if (sca[i].min - y < sca[i].usermin) y = sca[i].min - sca[i].usermin;
            sca[i].setDispscale(sca[i].min - y, sca[i].max - y);
          }
        }
        else if (point.y < moviepoint.y)
        {
          if (sca[i].log != 0)
          {
            if ((--yCnt <= -movieLogCnt) && sca[i].max * 10 <= sca[i].usermax)
            {
              sca[i].setDispscale(sca[i].min * 10, sca[i].max * 10);
              yCnt = 0;
            }
            else
              reScale = false;
          }
          else
          {
            if (sca[i].max + y > sca[i].usermax) y = sca[i].usermax - sca[i].max;
            sca[i].setDispscale(sca[i].min + y, sca[i].max + y);
          }
        }
      }
      if (reScale == true)
      {
        sca[0].scaleToGrid();
        if (m_marker == 2) setmarkerflag(2); // restore marker array
        rescaleFlag = 1;
        rescaleTextFlag = AcopConst.NEW_MOVE_WINDOW;
        prepareDraw(); // rescale but keep present limit
        // prepareDraw(TRUE, -1, FALSE, 0); // rescale but keep present limit
        if (isZoomed && zooming != 0x4) notifyAcop(AcopEvent.ACOP_MOUSE_ZOOM_EVENT,trigger);
        // if ( isZoomed ) FireCursorZoom(s_xmax, s_xmin, s_ymax, s_ymin);
      }
      else
        for (i = 0; i < 3; i++)
        {
          if (i == 2 && sca[i].OnFlag == false) continue;
          if (sca[i].log != 0)
            sca[i].setDispscale(Math.log(sca[i].min) / Math.log(10), Math.log(sca[i].max) / Math.log(10));
        }
    }
    moviepoint = point;
  }
  protected void onLButtonDown(Point point)
  {
    if (histo[m_histstart] == null || aFrame.drawFlag == false) return;
    if (histo[m_histstart].dismode == AcopConst.mode_textbox) return;
    if ((zooming & 0x7) != 0)
    {
      zoomFlag = true;
      zoomRect.setBounds(point.x, point.y, 0, 0);
      mStart = point;
      mEnd = point;
      Graphics2D a = (Graphics2D) getGraphics();
      a.setXORMode(Color.darkGray);
      BasicGraphicsUtils.drawDashedRect(a, point.x, point.y, 0, 0);
      a.setPaintMode();
      a.dispose();
      // mEnd = mStart = point;
      // zoomRect2 = zoomRect;
      // CClientDC dc(this); // draw zoom rectangle
      // dc.DrawDragRect( &zoomRect2, size, NULL, size, NULL, NULL );
    }
    else
    {
      // mClickPoint = point;
      // mDoubleClickPoint = mClickPoint;
      // mDoubleClickPoint.x++;
    }
  }
  protected void onLButtonUp(Point point, MouseEvent trigger)
  {
    double ftmp, curPos;
    double[] storemin = new double[3];
    double[] storemax = new double[3];
    int i;
    if (zoomFlag == false) return;
    zoomFlag = false;
    if (mEnd.equals(point) == false || mStart == point) return;
    Graphics2D a = (Graphics2D) getGraphics(); // get rid of zoom drag
                                                // rectangle
    a.setXORMode(Color.darkGray);
    BasicGraphicsUtils.drawDashedRect(a, zoomRect.x, zoomRect.y, zoomRect.width, zoomRect.height);
    a.setPaintMode();
    a.dispose();
    // zoom outside the histo rectangle
    if (zoomRect.x + zoomRect.width < aFrame.histRect.x
        || zoomRect.x > aFrame.histRect.x + aFrame.histRect.width
        || zoomRect.y + zoomRect.height < aFrame.histRect.y
        || zoomRect.y > aFrame.histRect.y + aFrame.histRect.height) return;
    // zoom inside the histo rectangle and 2 pixel at least
    if (zoomRect.width > 2 && zoomRect.height > 2
        && (zoomRect.width < aFrame.histRect.width || zoomRect.height < aFrame.histRect.height))
    {
      if ((zooming & 0x4) != 0 && zoomIndex < AcopConst.lengthZoom) for (i = 0; i < 3; i++)
      {
        storemin[i] = sca[i].min;
        storemax[i] = sca[i].max;
      }
      if ((zooming & 0x1) != 0 || zooming == 0x4) // x zooming
      {
        ftmp = (zoomRect.x - aFrame.absxleft) * sca[0].dispSize / aFrame.dxwidth + sca[0].min;
        curPos = (zoomRect.x + zoomRect.width - aFrame.absxleft) * sca[0].dispSize / aFrame.dxwidth
            + sca[0].min;
        if ( zoomXmaxFlag != 2 ) zoomXmaxFlag = curPos < sca[0].max ? 2 : 1;
        if (snapZoomToNearestHour) {
            if (curPos % 3600 > 1800) {
                curPos = (long)(curPos / 3600L)*3600 + 3600;
            } else {
                curPos = (long)(curPos / 3600L)*3600;
            }
            if (ftmp % 3600 > 1800) {
                ftmp = (long)(ftmp / 3600L)*3600 + 3600;
            } else {
                ftmp = (long)(ftmp / 3600L)*3600;
            }
        }
        if (curPos < sca[0].max) sca[0].setDispscale(sca[0].min, curPos);
        if (ftmp > sca[0].min) sca[0].setDispscale(ftmp, sca[0].max);
      }
      if ((zooming & 0x2) != 0 || zooming == 0x4)
        // y and y2 zooming
        for (i = 1; i < 3; i++)
        {
          if (i == 2 && sca[i].OnFlag == false) continue;
          ftmp = (1. - (double) (zoomRect.y - aFrame.histRect.y) / aFrame.histRect.height) * sca[i].dispSize
              + sca[i].min;
          curPos = (1. - (double) (zoomRect.y + zoomRect.height - aFrame.histRect.y) / aFrame.histRect.height)
              * sca[i].dispSize + sca[i].min;
          if (curPos > sca[i].min) sca[i].setDispscale(curPos, sca[i].max);
          if (ftmp < sca[i].max) sca[i].setDispscale(sca[i].min, ftmp);
        }
      for (i = 0; i < 3; i++) // from log back to nomal for re-scaling
      {
        if (sca[i].log != 0)
          sca[i].setDispscale(Math.pow(10.0, sca[i].min), Math.pow(10.0, sca[i].max));
        else if (sca[i].max < sca[i].min + AcopConst.zero_check)
          sca[i].setDispscale(sca[i].min, sca[i].min + AcopConst.zero_check);
      }
      if (wrapAround && sca[0].log == 0)
      {
        if (sca[0].min > wrapxmax)
        {
          ftmp = wrapxmax - wrapxmin;
          sca[0].max -= ftmp;
          sca[0].min -= ftmp;
        }
        else if (sca[0].max < wrapxmin)
        {
          ftmp = wrapxmax - wrapxmin;
          sca[0].max += ftmp;
          sca[0].min += ftmp;
        }
      }
      isZoomed = true;
      sca[0].scaleToGrid();
      if (m_marker == 2) setmarkerflag(2); // recalculate marker array
        // prepareDraw(TRUE, -1, FALSE, nScaleText); // rescale and not set max
        // limit
      rescaleFlag = 1;
      rescaleTextFlag = AcopConst.NEW_ZOOM_WINDOW;
      prepareDraw(); // rescale and not set max limit
      yZoomRatio = "V" + (int) (sca[1].userSize / sca[1].dispSize + 0.5) + ":1";
      xZoomRatio = "H" + (int) (sca[0].userSize / sca[0].dispSize + 0.5) + ":1";
      if ((zooming & 0x4) != 0 && zoomIndex < AcopConst.lengthZoom)
      {
        for (i = 0; i < 3; i++)
        {
          bufferedMin[zoomIndex][i] = storemin[i];
          bufferedMax[zoomIndex][i] = storemax[i];
        }
        zoomIndex++;
        bufferedZoomRatio = "Z" + zoomIndex;
      }
      notifyAcop(AcopEvent.ACOP_MOUSE_ZOOM_EVENT,trigger);
      // FireCursorZoom(s_xmax, s_xmin, s_ymax, s_ymin);
    }
    // else if ( isZoomed && aFrame.zoomIcon.contains(point) ) doBufferedZoom();
    else if (zoomRect.width >= aFrame.histRect.width && zoomRect.height >= aFrame.histRect.height)
      doBufferedZoom(trigger); // return back to original setting
      // { // return back to original setting
      // if ( isZoomed && (sca[0].isOriginMaxMin() == false ||
      // sca[1].isOriginMaxMin() == false) )
      // {
      // rescaleFlag = 2;
      // if ( m_marker == 2 ) setmarkerflag(2); // restore marker array
      // rescaleTextFlag = AcopConst.NEW_ZOOM_WINDOW;
      // prepareDraw();
      // notifyAcop(AcopEvent.ACOP_MOUSE_ZOOM_EVENT);
      // }
      // isZoomed = false;
    // }
    // else
    // {
    // InvalidateControl(&zoomRect);
    // if ( getMousePosition(point, x, y, TRUE) ) FireCursorClick(XarrayElement,
    // x, y);
    // }
    // }
  }
  // public void resize(int width, int height)
  // {
  // if ( changeScaleWidth == 0 ) return;
  // scaleframe();
  // this.repaint();
  // }
  @Override
  protected void processComponentEvent(ComponentEvent componentEvent)
  {
    // make sue listeners are notified
    super.processComponentEvent(componentEvent);
    switch (componentEvent.getID())
    {
      case ComponentEvent.COMPONENT_RESIZED: // resized
        scaleFrameDraw();
        break;
      case ComponentEvent.COMPONENT_MOVED: // moved
        break;
      default:
        break;
    }
  }
  @Override
  protected void processMouseEvent(MouseEvent mouseEvent)
  {
    // make sue listeners are notified
    super.processMouseEvent(mouseEvent);
    mousepoint = mouseEvent.getPoint();
    switch (mouseEvent.getID())
    {
      case MouseEvent.MOUSE_ENTERED: // enter to de.desy.acop.chart area
        mouseIndex = -1;
        notifyAcop(AcopEvent.ACOP_MOUSE_ENTER_EVENT,mouseEvent);
        break;
      case MouseEvent.MOUSE_EXITED: // go out of de.desy.acop.chart area
        mouseIndex = -1;
        notifyAcop(AcopEvent.ACOP_MOUSE_EXIT_EVENT,mouseEvent);
        break;
      case MouseEvent.MOUSE_PRESSED: // mouse down
        if ((mouseEvent.getModifiers() & MouseEvent.BUTTON1_MASK) != 0)
          if (!mouseEvent.isControlDown()) onLButtonDown(mousepoint); // left button
        else if ((mouseEvent.getModifiers() & MouseEvent.BUTTON3_MASK) != 0) // right
                                                                              // button
          onRButtonDown(mousepoint,mouseEvent); // left button
          notifyAcop(AcopEvent.ACOP_MOUSE_PRESSED_EVENT,mouseEvent);
        break;
      case MouseEvent.MOUSE_RELEASED: // mouse up
        if ((mouseEvent.getModifiers() & MouseEvent.BUTTON1_MASK) != 0)
          onLButtonUp(mousepoint,mouseEvent); // left button
        else if ((mouseEvent.getModifiers() & MouseEvent.BUTTON3_MASK) != 0) // right
                                                                              // button
          onRButtonUp(mousepoint,mouseEvent); // left button
        break;
      case MouseEvent.MOUSE_CLICKED: // mouse click
        long mouseEventTime = mouseEvent.getWhen();
        long mouseButton = mouseEvent.getButton();
        boolean doubleClick;
        if (mouseButton == previousButton && (mouseEventTime - mousePreviousClickTime < 300))
          doubleClick = true;
        else
          doubleClick = false;
        mousePreviousClickTime = mouseEventTime;
        previousButton = mouseEvent.getButton();
        if (doubleClick) // implement double click events
        {
          if (histRoundRect.contains(mousepoint) && aFrame.drawFlag)
            mouseIndex = histo[m_histstart].getMousePosition(mousepoint, mouseConvert);
          else
            mouseIndex = -1;
          mousepoint.x += aFrame.histRect.x;  // convert it back for detect zoomIcon
          mousepoint.y += aFrame.histRect.y;
          if (isZoomed && aFrame.zoomIcon.contains(mousepoint)) // return to original size
          {
            rescaleFlag = 2;
            if (m_marker == 2) setmarkerflag(2); // restore marker array
            rescaleTextFlag = AcopConst.NEW_ZOOM_WINDOW;
            prepareDraw();
            isZoomed = false;
            zoomXmaxFlag = 0;
            notifyAcop(AcopEvent.ACOP_MOUSE_ZOOM_EVENT,mouseEvent);
          }
          notifyAcop(AcopEvent.ACOP_MOUSE_DCLICK_EVENT,mouseEvent);
        }
        // FireCursorClick(XarrayElement, x, y);
        // mouseEvent.notifyAll();
        else if (isZoomed && aFrame.zoomIcon.contains(mousepoint))
          doBufferedZoom(mouseEvent);
        // / {
        // / rescaleFlag = 2;
        // /// if ( m_marker == 2 ) setmarkerflag(2); // restore marker array
        // / rescaleTextFlag = AcopConst.NEW_ZOOM_WINDOW;
        // / prepareDraw();
        // / isZoomed = false;
        // / notifyAcop(AcopEvent.ACOP_MOUSE_ZOOM_EVENT);
        // / }
        else
        {
          if (histRoundRect.contains(mousepoint) && aFrame.drawFlag)
            mouseIndex = histo[m_histstart].getMousePosition(mousepoint, mouseConvert);
          else
            mouseIndex = -1;
          notifyAcop(AcopEvent.ACOP_MOUSE_CLICK_EVENT,mouseEvent);
        }
		if (mouseEvent.getButton()==MouseEvent.BUTTON3) getPopupManager().showPopup(this,mouseEvent.getX(),mouseEvent.getY());
        break;
      default:
        break;
    }
  }
  @Override
protected void processMouseMotionEvent(MouseEvent mouseMotionEvent)
  {
    int rWidth, rHeight, recX, recY;
    // make sue listeners are notified
    super.processMouseMotionEvent(mouseMotionEvent);
    mousepoint = mouseMotionEvent.getPoint();
    switch (mouseMotionEvent.getID())
    {
      case MouseEvent.MOUSE_MOVED: // mouse move
        if (aFrame.drawFlag && histRoundRect.contains(mousepoint))
        {
          if (m_cursorMarkerMode != 0 && cursorStay == 0) drawCursorMarker(null, mousepoint);
          mMovePoint.x = mousepoint.x;
          mMovePoint.y = mousepoint.y;
          mouseIndex = histo[m_histstart].getMousePosition(mousepoint, mouseConvert);
          if (!paintCursorMarker) {
        	  paintCursorMarker = true;
        	  repaintHisto();
          }
          notifyAcop(AcopEvent.ACOP_MOUSE_MOVE_EVENT,mouseMotionEvent);
        }
        // if ( getMousePosition(point, mx, my, TRUE) )
        // FireCursorMove(XarrayElement, mx, my);
        break;
      case MouseEvent.MOUSE_DRAGGED: // mouse dragging
    	  paintCursorMarker = true;
        if (zoomFlag)
        {
          Graphics2D a = (Graphics2D) getGraphics();
          a.setXORMode(Color.darkGray);
          BasicGraphicsUtils.drawDashedRect(a, zoomRect.x, zoomRect.y, zoomRect.width, zoomRect.height);
          mEnd = mousepoint;
          rWidth = Math.abs(mEnd.x - mStart.x);
          rHeight = Math.abs(mEnd.y - mStart.y);
          recX = Math.min(mEnd.x, mStart.x);
          recY = Math.min(mEnd.y, mStart.y);
          zoomRect.setBounds(recX, recY, rWidth, rHeight);
          BasicGraphicsUtils.drawDashedRect(a, recX, recY, rWidth, rHeight);
          a.setPaintMode();
          a.dispose();
        }
        else if (movie)
          onMouseMove(mousepoint,mouseMotionEvent);
        else if (aFrame.drawFlag && m_cursorMarkerMode != 0 && cursorStay == 0
            && histRoundRect.contains(mousepoint))
        {
          drawCursorMarker(null, mousepoint);
          mMovePoint.x = mousepoint.x;
          mMovePoint.y = mousepoint.y;
          mouseIndex = histo[m_histstart].getMousePosition(mousepoint, mouseConvert);
        }
        notifyAcop(AcopEvent.ACOP_MOUSE_DRAG_EVENT,mouseMotionEvent);
        break;
      default:
        break;
    }
  }
  private void drawtext(java.awt.Graphics2D g2d)
  {
    if (textFlag == false) return;
    Rectangle Cliprect = g2d.getClipBounds();
    if (Cliprect != null)
    {
      if (Cliprect.intersects(histRoundRect) == false) return;
    }
    g2d.clipRect(histRoundRect.x, histRoundRect.y, histRoundRect.width, histRoundRect.height);
    g2d.translate(aFrame.histRect.x, aFrame.histRect.y);
    for (int i = 0; i < n_text; i++)
    {
      if (ptext[i] == null) break;
      ptext[i].printUserText(g2d);
    }
    g2d.translate(-aFrame.histRect.x, -aFrame.histRect.y);
    g2d.setClip(Cliprect);
  }
  private void drawhist(java.awt.Graphics2D g2d)
  {
    if (aFrame.drawFlag == false && textFlag == false) return;
    Rectangle Cliprect = g2d.getClipBounds();
    if (Cliprect != null)
    {
      if (Cliprect.intersects(histRoundRect) == false) return;
    }
    g2d.clipRect(histRoundRect.x, histRoundRect.y, histRoundRect.width, histRoundRect.height);
    g2d.translate(aFrame.histRect.x, aFrame.histRect.y);
    for (int i = 0; i < 2; i++)
      sca[i].drawGrid(g2d);
    for (int i = 0; i < 2; i++)
      axis[i].drawAxis(g2d);
    if (aFrame.drawFlag) for (int i = 0; i < n_display; i++)
    {
      if (histo[i] == null) break;
      histo[i].drawhist(g2d);
    }
    if (aFrame.drawFlag && m_cursorMarkerMode != 0 && histRoundRect.contains(mMovePoint))
    {
      /* set pencolor to black (for first histo?) to avoid the color change of cursorMarker pen  */
      g2d.setColor(Color.black);
      if ( cursorStay > 0 )  // 2009-5-4 tootip may change the mouse position, reload the original value
      {
        mMovePoint.x = sca[0].getScreenPosition(cursorXposition);
        mMovePoint.y = sca[1].getScreenPosition(cursorYposition);
      }
      drawCursorMarker(g2d, mMovePoint);
    }
    else
      cursorMarkerFlag = 0; // no cursor drawn on screen!
    g2d.translate(-aFrame.histRect.x, -aFrame.histRect.y);
    g2d.setClip(Cliprect);
  }
  private void captionPaint(java.awt.Graphics2D g2d)
  {
    FontMetrics fMetrix;
    int xPosition, len, hei;
    Rectangle Cliprect = g2d.getClipBounds();
    if ( caption == null ) return;  /* 2008-2 hwu*/
    if (Cliprect != null)
    {
      if (Cliprect.intersects(titleRect) == false) return;
    }
    if (captionFont == null || captionChanged == true || randomData == true)
    {
      captionFont = new Font(g2d.getFont().getAttributes());
      captionChanged = false;
    }
    String[] newCaption = caption.split("\n");
    Font tmpFont = g2d.getFont();
    AffineTransform transform = g2d.getTransform();
    g2d.clipRect(titleRect.x, titleRect.y, titleRect.width + 1, titleRect.height + 1);
    g2d.setFont(captionFont);
    fMetrix = g2d.getFontMetrics();
    for ( int i=0; i<newCaption.length; i++ )
    {
    len = fMetrix.stringWidth(newCaption[i]);
    hei = (fMetrix.getDescent() + fMetrix.getAscent()) / 2;
    if (captionPosition == 1)
      xPosition = (titleRect.width - len) / 2;
    else if (captionPosition < 1)
      xPosition = 2;
    else
      xPosition = titleRect.width - len - 2;
    g2d.setColor(aFrame.frameForeColor);
    if ( newCaption.length == 1 )
      g2d.drawString(newCaption[i], xPosition, sca[0].leftBottom ? (fTop - 4) : (fTop - 4 - hei * 2) );
    else
      g2d.drawString(newCaption[i], xPosition, sca[0].leftBottom ? (fTop - 4) - (newCaption.length - i -1) * hei * 2 : (fTop - 4 - hei * 2) - (newCaption.length - i -1)* hei * 2 );
    }
    g2d.setTransform(transform);
    g2d.setFont(tmpFont);
    g2d.setClip(Cliprect);
  }
  @SuppressWarnings("unused")
private void backgroundPaint(java.awt.Graphics2D g2d, Rectangle Cliprect, Rectangle acopRect)
  {
    int i;
    if (nBackRegions == 0)
    {
      g2d.setColor(getBackground());
//      g2dima.setBackground(g2d.getBackground());
      if (Cliprect == null)
        g2d.fillRect(0,0,acopRect.width, acopRect.height);
//        g2dima.clearRect(0, 0, acopRect.width, acopRect.height);
      else
        g2d.fillRect(Cliprect.x, Cliprect.y, Cliprect.width, Cliprect.height);
//        g2dima.clearRect(Cliprect.x, Cliprect.y, Cliprect.width, Cliprect.height);
      return;
    }
    g2d.setColor(getBackground());
//    g2dima.setBackground(g2d.getBackground());
    if (false) // what to do for the display gap?????
    {
      if (Cliprect == null || Cliprect.intersects(titleRect) == true)
        g2d.fillRect(titleRect.x, titleRect.y, titleRect.width, titleRect.height);
//        g2dima.clearRect(titleRect.x, titleRect.y, titleRect.width, titleRect.height);
      if (Cliprect == null || Cliprect.intersects(xRect) == true)
        g2d.fillRect(xRect.x, xRect.y, xRect.width, xRect.height);
//        g2dima.clearRect(xRect.x, xRect.y, xRect.width, xRect.height);
      if (Cliprect == null || Cliprect.intersects(yRect) == true)
        g2d.fillRect(yRect.x, yRect.y, yRect.width, yRect.height);
//        g2dima.clearRect(yRect.x, yRect.y, yRect.width, yRect.height);
      if (Cliprect == null || Cliprect.intersects(yAxisRect) == true)
        g2d.fillRect(yAxisRect.x, yAxisRect.y, yAxisRect.width, yAxisRect.height);
//        g2dima.clearRect(yAxisRect.x, yAxisRect.y, yAxisRect.width, yAxisRect.height);
    }
    else
    {
      if (Cliprect == null)
        g2d.fillRect(0,0,acopRect.width, acopRect.height);
//        g2dima.clearRect(0, 0, acopRect.width, acopRect.height);
      else
        g2d.fillRect(Cliprect.x, Cliprect.y, Cliprect.width, Cliprect.height);
//        g2dima.clearRect(Cliprect.x, Cliprect.y, Cliprect.width, Cliprect.height);
    }
    g2d.clipRect(histRoundRect.x, histRoundRect.y, histRoundRect.width, histRoundRect.height);
    Rectangle tmpCliprect = g2d.getClipBounds();
    for (i = 0; i < nBackRegions; i++)
    {
      if (backkgroundRegionType[i] == 0) continue;
      if (tmpCliprect != null && tmpCliprect.intersects(backRegionRect[i]) == false) continue;
      if (backkgroundRegionType[i] == 1)
      {
        g2d.setColor(backRegionColor[i]);
//        g2dima.setBackground(backRegionColor[i]);
        g2d.fillRect(backRegionRect[i].x, backRegionRect[i].y, backRegionRect[i].width,
          backRegionRect[i].height);
//        g2dima.clearRect(backRegionRect[i].x, backRegionRect[i].y, backRegionRect[i].width,backRegionRect[i].height);
      }
      else if (backkgroundRegionType[i] == 2)
        g2d.drawImage(backRegionImage[i], backRegionRect[i].x, backRegionRect[i].y,
            backRegionRect[i].width, backRegionRect[i].height, this);
    }
    g2d.setClip(Cliprect);
  }
  @Override
  public void update(java.awt.Graphics a)
  {
    this.paint(a);
  }
  /* (non-Javadoc)
   * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
   */
  @Override
  protected void paintComponent(Graphics g) {
	//super.paintComponent(g);
//    java.awt.Graphics2D g2d = (Graphics2D) g;
    java.awt.Graphics2D g2dima = (Graphics2D) g.create();
    Rectangle r = getBounds();
    if (textAntiAliasing) {
    	g2dima.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
    } else {
    	g2dima.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,RenderingHints.VALUE_TEXT_ANTIALIAS_DEFAULT);
    }
    int i;
/* 2007 do not use double buffer any more hwu
    if (ima == null || r.width != ima.getWidth() || r.height != ima.getHeight())
    {
      // ima = new java.awt.image.BufferedImage(r.width, r.height,
      // BufferedImage.TYPE_INT_RGB);
      ima = new java.awt.image.BufferedImage(r.width, r.height, BufferedImage.TYPE_INT_ARGB);
      g2dima = ima.createGraphics();
    }
*/
//    g2dima = g2d; //2007
    Rectangle Cliprect = g2dima.getClipBounds(); // get original clip area

    // Ike: this method is not really painting background, but just clears it and in swint this
    // leaves white background
    backgroundPaint(g2dima, Cliprect, r);

    // Ike: in swing you have to paint yourself your gackground
//    g2dima.setColor(getBackground());
//    g2dima.fillRect(0,0,ima.getWidth(),ima.getHeight());

    g2dima.setColor(aFrame.frameForeColor); // draw border
    if ( frameBorderOn )
    g2dima.drawRect(0, 0, r.width - 1, r.height - 1);

    if (changeScaleWidth == 0) // prepare random data only once
    {
      scaleframe();
      /*
       * if ( Beans.isDesignTime() && randomData == true ) { prepareDraw();
       * System.out.println("DesignTime and RandomData"); }
       */
      changeScaleWidth = 1;
    }
    captionPaint(g2dima); // paint caption
    if (Cliprect == null || Cliprect.intersects(histRoundRect) == true)
    {
      g2dima.setColor(aFrame.frameForeColor);
      g2dima.drawRect(histRoundRect.x, histRoundRect.y, histRoundRect.width, histRoundRect.height);
    }
    drawhist(g2dima);
//    drawtext(g2dima);
    g2dima.setStroke(stroke);
    if (isZoomed && (Cliprect == null || Cliprect.intersects(aFrame.zoomIcon) == true))
    {
      g2dima.setColor(aFrame.frameForeColor);
      g2dima.setFont(aFrame.frameFont);
      g2dima.drawRect(aFrame.zoomIcon.x, aFrame.zoomIcon.y, aFrame.zoomIcon.width, aFrame.zoomIcon.height);
      g2dima.clipRect(aFrame.zoomIcon.x, aFrame.zoomIcon.y, aFrame.zoomIcon.width, aFrame.zoomIcon.height);
      if ((zooming & 0x4) != 0)
        g2dima.drawString(bufferedZoomRatio, aFrame.zoomIcon.x + 1, aFrame.zoomIcon.y
            + aFrame.zoomIcon.height - 1);
      else
      {
        g2dima.drawString(yZoomRatio, aFrame.zoomIcon.x + 1, aFrame.zoomIcon.y + aFrame.zoomIcon.width / 2);
        g2dima.drawString(xZoomRatio, aFrame.zoomIcon.x + 1, aFrame.zoomIcon.y + aFrame.zoomIcon.width - 1);
      }
      g2dima.setClip(Cliprect);
    }
    if ((textFlag == true || (aFrame.drawFlag == true && histo[m_histstart].dismode != AcopConst.mode_textbox))
        && (fLeft > 0 || fRight > 0 || fBottom > 0 || fTop > 0))
    {
      for (i = 0; i < 3; i++)
      {
        if (i == 2 && sca[i].OnFlag == false) continue;
        sca[i].drawTicks(g2dima, histo[m_histstart]);
      }
    }
    drawtext(g2dima);
    if (zoomFlag && (Cliprect == null || Cliprect.intersects(zoomRect) == true))
    {
      g2dima.setXORMode(Color.darkGray);
      BasicGraphicsUtils.drawDashedRect(g2dima, zoomRect.x, zoomRect.y, zoomRect.width, zoomRect.height);
      g2dima.setPaintMode();
    }

//    paintAnnotations(g2dima);
    // Ike: not necessary to call inside paintComponent method, because children are painted inside
    // paintChildren method in parent JPanel
    //super.paint(g2dima); // paint all children
/* 2007 do not use double buffer any more hwu
    if (Cliprect == null)
      g2d.drawImage(ima, 0, 0, this);
    else
      g2d.drawImage(ima, Cliprect.x, Cliprect.y, Cliprect.x + Cliprect.width, Cliprect.y + Cliprect.height,
          Cliprect.x, Cliprect.y, Cliprect.x + Cliprect.width, Cliprect.y + Cliprect.height, this);
*/
    // MotifGraphicsUtils.drawStringInRect(a,caption,
    // frameWidth+frameLeftOffset,
    // 0,
    // titleRect.width,
    // frameWidth,
    // MotifGraphicsUtils.CENTER);
    // todo. Add overriding code here.
  }
/*  public void paintAll(java.awt.Graphics a)
  {
    // todo. Add overriding code here.
  }*/
//  private void paintAnnotations(Graphics2D g) {
//	  g.setFont(AcopHisto.ANNOTATION_FONT);
//	  int xOffset = histRoundRect.x;
//	  int yOffset = histRoundRect.y;
//	  for (AcopHisto h : histo) {
//		  if (h == null) continue;
//		  int[] annotations = h.getAnnotationPoints();
//		  if (annotations != null) {
//    		  for (int i = 0; i < annotations.length; i++) {
//		          if (h.dispOn[i] == 0) continue;
//		          g.setColor(AcopHisto.ANNOTATION_COLOR);
//		          g.fillRect(annotations[i]-3 + xOffset,yOffset-1-15,15,15);
//		          g.setColor(h.penColor);
//		          g.drawString(AcopHisto.ANNOTATION_TEXT,annotations[i]+xOffset,yOffset-3);
//    		  }
//		  }
//	  }
//  }

  public static void main(String argv[])
  {
    class DriverFrame extends java.awt.Frame
    {
    private static final long serialVersionUID = 1L;

    public DriverFrame()
      {
        addWindowListener(new java.awt.event.WindowAdapter()
        {
          @Override
          public void windowClosing(java.awt.event.WindowEvent event)
          {
            dispose(); // free the system resources
            System.exit(0); // close the application
          }
          @Override
          public void windowOpened(java.awt.event.WindowEvent event)
          {
          }
        });
        setLayout(null);
        setSize(400, 300);
        add(new Acop());
      }
    }
    new DriverFrame().setVisible(true);
  }

  /*
   * public void addPropertyChangeListener(java.beans.PropertyChangeListener
   * pcl) { if (changes == null) changes = new
   * symantec.itools.beans.PropertyChangeSupport(this);
   * changes.addPropertyChangeListener("",pcl); } public void
   * removePropertyChangeListener(java.beans.PropertyChangeListener pcl) { if
   * (changes != null) changes.removePropertyChangeListener("",pcl); } public
   * void addVetoableChangeListener(java.beans.VetoableChangeListener vcl) { if
   * (veto == null) veto = new
   * symantec.itools.beans.VetoableChangeSupport(this);
   * veto.addVetoableChangeListener("",vcl); } public void
   * removeVetoableChangeListener(java.beans.VetoableChangeListener vcl) { if
   * (veto != null) veto.removeVetoableChangeListener("",vcl); }
   */
  public void addAcopListener(AcopListener l)
  {
    synchronized(acopListeners)
    {
      if (!acopListeners.contains(l)) acopListeners.add(l);
    }
  }
  public void removeAcopListener(AcopListener l)
  {
    synchronized(acopListeners)
    {
      if (acopListeners.contains(l)) acopListeners.remove(l);
    }
  }

  protected void notifyAcop(int id) {
	  notifyAcop(id,null);
  }
  // notify listening objects of Acop
  protected void notifyAcop(int id, MouseEvent trigger)
  {
    // create the event object
    if (id == AcopEvent.ACOP_MOUSE_CLICK_EVENT || id == AcopEvent.ACOP_MOUSE_MOVE_EVENT
        || id == AcopEvent.ACOP_MOUSE_ENTER_EVENT || id == AcopEvent.ACOP_MOUSE_EXIT_EVENT
        || id == AcopEvent.ACOP_MOUSE_ZOOM_EVENT /*|| id == AcopEvent.ACOP_RECEIVE_EVENT*/
        || id == AcopEvent.ACOP_MOUSE_DCLICK_EVENT || id == AcopEvent.ACOP_MOUSE_PRESSED_EVENT 
        || id == AcopEvent.ACOP_MOUSE_DRAG_EVENT)
    {
      AcopEvent evt;
      if (id == AcopEvent.ACOP_MOUSE_ZOOM_EVENT)
    	  if (sca[2] != null) {
    		  evt = new AcopEvent(this, trigger,id, sca[0].min, sca[1].min, sca[0].max, sca[1].max, sca[2].min,sca[2].max);
    	  } else {
    		  evt = new AcopEvent(this, trigger, id, sca[0].min, sca[1].min, sca[0].max, sca[1].max,0,0);
    	  }
      else /*if (id != AcopEvent.ACOP_RECEIVE_EVENT)*/
        evt = new AcopEvent(this, trigger, id, mouseIndex, mousepoint.x, mousepoint.y, mouseConvert[0],
            mouseConvert[1]);
/*      else
        evt = new AcopEvent(this, id, 1, 1); // link id ans error
*/
      AcopListener[] v;
      synchronized (acopListeners)
      {
        v = acopListeners.toArray(new AcopListener[acopListeners.size()]);
      }
      // fire events
      int cnt = v.length;
      for (int i = 0; i < cnt; i++)
      {
        AcopListener client = v[i];
        if (id == AcopEvent.ACOP_MOUSE_CLICK_EVENT)
          client.acopMouseClick(evt);
        else if (id == AcopEvent.ACOP_MOUSE_DCLICK_EVENT)
          client.acopMouseDoubleClick(evt);
        else if (id == AcopEvent.ACOP_MOUSE_MOVE_EVENT)
          client.acopMouseMove(evt);
        else if (id == AcopEvent.ACOP_MOUSE_ENTER_EVENT)
          client.acopMouseEnter(evt);
        else if (id == AcopEvent.ACOP_MOUSE_EXIT_EVENT)
          client.acopMouseExit(evt);
        else if (id == AcopEvent.ACOP_MOUSE_ZOOM_EVENT)
          client.acopMouseZoom(evt);
        else if (id == AcopEvent.ACOP_MOUSE_PRESSED_EVENT)
          client.acopMousePressed(evt);
        else if (id == AcopEvent.ACOP_MOUSE_DRAG_EVENT)
            client.acopMouseDragged(evt);
        //else if (id == AcopEvent.ACOP_RECEIVE_EVENT) client.AcopReceive(evt);
      }
    }
  }
  public void notifyAcop(int id, int link, int code)
  {
    // create the event object
    if (id == AcopEvent.ACOP_MOUSE_CLICK_EVENT || id == AcopEvent.ACOP_MOUSE_MOVE_EVENT
        || id == AcopEvent.ACOP_MOUSE_ENTER_EVENT || id == AcopEvent.ACOP_MOUSE_EXIT_EVENT
        || id == AcopEvent.ACOP_MOUSE_ZOOM_EVENT || /*id == AcopEvent.ACOP_RECEIVE_EVENT
        ||*/ id == AcopEvent.ACOP_MOUSE_DCLICK_EVENT || id == AcopEvent.ACOP_MOUSE_PRESSED_EVENT)
    {
      AcopEvent evt;
      if (id == AcopEvent.ACOP_MOUSE_ZOOM_EVENT) {
    	  if (sca[2] != null) {
    		  evt = new AcopEvent(this, null,id, sca[0].getMin(), sca[1].getMin(), sca[0].getMax(), sca[1].getMax(), sca[2].getMin(),sca[2].getMax());
    	  } else {
    		  evt = new AcopEvent(this, null,id, sca[0].getMin(), sca[1].getMin(), sca[0].getMax(), sca[1].getMax(), 0,0);
    	  }
      } else {
        evt = new AcopEvent(this, null,id, mouseIndex, mousepoint.x, mousepoint.y, mouseConvert[0],
            mouseConvert[1]);
      }
      AcopListener[] v;
      synchronized (acopListeners)
      {
        v =  acopListeners.toArray(new AcopListener[acopListeners.size()]);
      }
      // fire events
      int cnt = v.length;
      for (int i = 0; i < cnt; i++)
      {
        AcopListener client = v[i];
        if (id == AcopEvent.ACOP_MOUSE_CLICK_EVENT)
          client.acopMouseClick(evt);
        else if (id == AcopEvent.ACOP_MOUSE_DCLICK_EVENT)
          client.acopMouseDoubleClick(evt);
        else if (id == AcopEvent.ACOP_MOUSE_MOVE_EVENT)
          client.acopMouseMove(evt);
        else if (id == AcopEvent.ACOP_MOUSE_ENTER_EVENT)
          client.acopMouseEnter(evt);
        else if (id == AcopEvent.ACOP_MOUSE_EXIT_EVENT)
          client.acopMouseExit(evt);
        else if (id == AcopEvent.ACOP_MOUSE_ZOOM_EVENT)
          client.acopMouseZoom(evt);
        else if (id == AcopEvent.ACOP_MOUSE_PRESSED_EVENT)
            client.acopMousePressed(evt);
        /*else if (id == AcopEvent.ACOP_RECEIVE_EVENT) client.AcopReceive(evt);*/
      }
    }
  }
  public void setCursorMarkerColor(java.awt.Color cursorMarkerColor)
  {
    // cursorColor = ~(TranslateColor(m_cursorMarkerColor) ^
    // TranslateColor(GetBackColor())) & 0xffffff;
    if (m_cursorMarkerColor == cursorMarkerColor) return;
    Color oldValue = m_cursorMarkerColor;
    m_cursorMarkerColor = cursorMarkerColor;
//    rev_cursorMarkerColor = new Color(~m_cursorMarkerColor.getRGB());
    rev_cursorMarkerColor = new Color(~m_cursorMarkerColor.getRGB() ^ ~backColor.getRGB());
    if (m_cursorMarkerMode != 0 && aFrame.drawFlag) repaintHisto();
    firePropertyChange("cursorMarkerColor", oldValue, m_cursorMarkerColor);
  }
  public java.awt.Color getCursorMarkerColor()
  {
    return m_cursorMarkerColor;
  }
  public void setCursorMarkerMode(int cursorMarkerMode)
  {
    cursorMarkerMode = (cursorMarkerMode < 0 || cursorMarkerMode > 3) ? 0 : cursorMarkerMode;
    if (m_cursorMarkerMode == cursorMarkerMode) return;
    int oldValue = m_cursorMarkerMode;
    m_cursorMarkerMode = cursorMarkerMode;
    if (m_cursorMarkerMode != 0 && histRoundRect.contains(mMovePoint) == false)
    {
      mMovePoint.x = histRoundRect.x + histRoundRect.width / 2;
      mMovePoint.y = histRoundRect.y + histRoundRect.height / 2;
    }
    if (aFrame.drawFlag) repaintHisto();
    firePropertyChange("cursorMarkerMode", oldValue, m_cursorMarkerMode);
  }
  public int getCursorMarkerMode()
  {
    return this.m_cursorMarkerMode;
  }
  public void setCursorMarkerStyle(int cursorMarkerStyle)
  {
    if (m_cursorMarkerStyle == cursorMarkerStyle) return;
    int oldValue = m_cursorMarkerStyle;
    m_cursorMarkerStyle = cursorMarkerStyle;
    cursorMarkerStroke = AcopConst.createStroke(m_cursorMarkerWidth, m_cursorMarkerStyle);
    if (m_cursorMarkerMode != 0 && aFrame.drawFlag) repaintHisto();
    firePropertyChange("cursorMarkerStyle", oldValue, m_cursorMarkerStyle);
  }
  public int getCursorMarkerStyle()
  {
    return m_cursorMarkerStyle;
  }
  public void setCursorMarkerWidth(int cursorMarkerWidth)
  {
    if (m_cursorMarkerWidth == cursorMarkerWidth) return;
    int oldValue = m_cursorMarkerWidth;
    m_cursorMarkerWidth = cursorMarkerWidth;
    cursorMarkerStroke = AcopConst.createStroke(m_cursorMarkerWidth, m_cursorMarkerStyle);
    if (m_cursorMarkerMode != 0 && aFrame.drawFlag) repaintHisto();
    firePropertyChange("cursorMarkerWidth", oldValue, m_cursorMarkerWidth);
  }
  public int getCursorMarkerWidth()
  {
    return m_cursorMarkerWidth;
  }
  /**
   * Set the 'extent' (in disply coordinates) of the current horizontal
   * display marker (if any).
   *
   * Data points can also be displayed with a 'marker' on any given
   * acop chart.  For instance, histograms can be 'marked' with a
   * flat bar to note some particular value.  The extent of the marker can
   * be set with this call.
   *
   * @param newExtent is the desired marker horizontal (x) extent.
   *
   * @see setMarkerColor
   */
  public void setCursorMarkerXExtent(double newExtent)
  {
    if (m_cursorMarkerXExtent == newExtent) return;
    double oldValue = m_cursorMarkerXExtent;
    m_cursorMarkerXExtent = newExtent;
    cursorXPercent = -1;
    if (m_cursorMarkerMode != 0 && aFrame.drawFlag) repaintHisto();
    firePropertyChange("cursorMarkerXExtent", oldValue, m_cursorMarkerXExtent);
  }
  public double getCursorMarkerXExtent()
  {
    return m_cursorMarkerXExtent;
  }
  /**
   * Set the 'extent' (in disply coordinates) of the current vertical
   * display marker (if any).
   *
   * Data points can also be displayed with a 'marker' on any given
   * acop chart.  For instance, histograms can be 'marked' with a
   * flat bar to note some particular value.  The extent of the marker can
   * be set with this call.
   *
   * @param newExtent is the desired marker vertical (y) extent.
   *
   * @see setMarkerColor
   */
  public void setCursorMarkerYExtent(double newExtent)
  {
    if (m_cursorMarkerYExtent == newExtent) return;
    double oldValue = m_cursorMarkerYExtent;
    m_cursorMarkerYExtent = newExtent;
    cursorYPercent = -1;
    if (m_cursorMarkerMode != 0 && aFrame.drawFlag) repaintHisto();
    firePropertyChange("cursorMarkerYExtent", oldValue, m_cursorMarkerYExtent);
  }
  public double getCursorMarkerYExtent()
  {
    return m_cursorMarkerYExtent;
  }
  /**
   * Sets the 'marker' color to the desired value.
   *
   * Data points can also be displayed with a 'marker' on any given
   * acop chart.  For instance, histograms can be 'marked' with a
   * flat bar to note some particular value.  The color of the marker can
   * be set with this call.
   *
   * @param newColor is the desired marker color.
   *
   * @see setCursorMarkerYExtent, setCursorMarkerXExtent
   */
  public void setMarkerColor(java.awt.Color newColor)
  {
    if (m_markerColor == newColor) return;
    Color oldValue = m_markerColor;
    m_markerColor = newColor;
    if (m_markerMode == 0 && aFrame.drawFlag) repaintHisto();
    firePropertyChange("markerColor", oldValue, m_markerColor);
  }
  /**
   * Data points can also be displayed with a 'marker' on any given
   * acop chart.  For instance, histograms can be 'marked' with a
   * flat bar to note some particular value.  The color of the marker can
   * be set with setMarkerColor.
   *
   * @return the current value of the marker color.
   */
  public java.awt.Color getMarkerColor()
  {
    return m_markerColor;
  }
  public void setMarkerMode(int markerMode)
  {
    markerMode = (markerMode < 0 || markerMode > 3) ? 0 : markerMode;
    if (this.m_markerMode == markerMode) return;
    int oldValue = m_markerMode;
    m_markerMode = markerMode;
    if (aFrame.drawFlag) repaintHisto();
    firePropertyChange("markerMode", oldValue, m_markerMode);
  }
  public int getMarkerMode()
  {
    return this.m_markerMode;
  }
  public void setMarkerStyle(int markerStyle)
  {
    if (this.m_markerStyle == markerStyle) return;
    int oldValue = m_markerStyle;
    m_markerStyle = markerStyle;
    firePropertyChange("markerStyle", oldValue, m_markerStyle);
  }
  public int getMarkerStyle()
  {
    return this.m_markerStyle;
  }
  public void setMarkerWidth(int markerWidth)
  {
    if (this.m_markerWidth == markerWidth) return;
    int oldValue = m_markerWidth;
    m_markerWidth = markerWidth;
    firePropertyChange("markerWidth", oldValue, m_markerWidth);
  }
  public int getMarkerWidth()
  {
    return this.m_markerWidth;
  }
  public void setMarkerXExtent(double markerXExtent)
  {
    if (this.m_markerXExtent == markerXExtent) return;
    double oldValue = m_markerXExtent;
    m_markerXExtent = markerXExtent;
    firePropertyChange("markerXExtent", oldValue, m_markerXExtent);
  }
  public double getMarkerXExtent()
  {
    return this.m_markerXExtent;
  }
  public void setMarkerYExtent(double markerYExtent)
  {
    if (this.m_markerYExtent == markerYExtent) return;
    double oldValue = m_markerYExtent;
    m_markerYExtent = markerYExtent;
    firePropertyChange("markerYExtent", oldValue, m_markerYExtent);
  }
  public double getMarkerYExtent()
  {
    return this.m_markerYExtent;
  }
  /**
   * out-of-bounds 'error' data can be displayed in a different
   * color in order to highlight them.  To establish what consitutes
   * 'out-of-bounds' you should make use of the setErrorWindow
   * method, which establishes the 'normal' range of the displayed
   * data.
   *
   * Using this call will apply the error window to all plots in the
   * acop chart.
   *
   * @param xMax gives the maximum X bounds for the error window in
   * user coordinates.  Data whose X values are greater than XMax
   * will be drawn using the associated 'error' color.
   * @param xMin gives the minimum X bounds for the error window in
   * user coordinates.  Data whose X values are less than XMin
   * will be drawn using the associated 'error' color.
   * @param yMax gives the maximum Y bounds for the error window in
   * user coordinates.  Data whose Y values are greater than YMax
   * will be drawn using the associated 'error' color.
   * @param yMin gives the minimum Y bounds for the error window in
   * user coordinates.  Data whose Y values are less than YMin
   * will be drawn using the associated 'error' color.
   *
   * @see setErrorColor
   *
   * @return 0 upon success.
   */
  public int setErrorWindow(double xMax, double xMin, double yMax, double yMin)
  {
    return setErrorWindow(-1, xMax, xMin, yMax, yMin);
  }
  /**
   * @deprecated use setErrorWindow instead
   *
   * @param XMax
   * @param XMin
   * @param YMax
   * @param YMin
   * @return
   */
  @Deprecated
  public int errorWindow(double XMax, double XMin, double YMax, double YMin)
  {
    return setErrorWindow(-1, XMax, XMin, YMax, YMin);
  }
  /**
   * @deprecated use setErrorWindow instead
   *
   * @param hDisplay
   * @param XMax
   * @param XMin
   * @param YMax
   * @param YMin
   * @return
   */
  @Deprecated
  public int errorWindow(int hDisplay, double XMax, double XMin, double YMax, double YMin)
  {
    return setErrorWindow(hDisplay, XMax, XMin, YMax, YMin);
  }
  /**
   * out-of-bounds 'error' data can be displayed in a different
   * color in order to highlight them.  To establish what consitutes
   * 'out-of-bounds' you should make use of the setErrorWindow
   * method, which establishes the 'normal' range of the displayed
   * data.
   *
   * @param hDisplay is the display handle referring to the plot
   * in question. This is the return value of the original call
   * to the draw() method.
   * @param xMax gives the maximum X bounds for the error window in
   * user coordinates.  Data whose X values are greater than XMax
   * will be drawn using the associated 'error' color.
   * @param xMin gives the minimum X bounds for the error window in
   * user coordinates.  Data whose X values are less than XMin
   * will be drawn using the associated 'error' color.
   * @param yMax gives the maximum Y bounds for the error window in
   * user coordinates.  Data whose Y values are greater than YMax
   * will be drawn using the associated 'error' color.
   * @param yMin gives the minimum Y bounds for the error window in
   * user coordinates.  Data whose Y values are less than YMin
   * will be drawn using the associated 'error' color.
   *
   * @see setErrorColor
   *
   * @return 0 upon success.
   */
  public int setErrorWindow(int hDisplay, double xMax, double xMin, double yMax, double yMin)
  {
    int i, start, stop, on;
    // out of range, -1 to set for default value
    if (hDisplay < -1 || hDisplay > n_display) return -1;
    on = 0;
    start = hDisplay;
    stop = hDisplay + 1;
    if (hDisplay == -1) // set all
    {
      start = 0;
      stop = n_display;
      e_xmax = xMax; // set default
      e_xmin = xMin;
      e_ymax = yMax;
      e_ymin = yMin;
    }
    for (i = start; i < stop; i++)
    {
      if (histo[i] == null) break;
      histo[i].err_xmax = xMax;
      histo[i].err_xmin = xMin;
      histo[i].err_ymax = yMax;
      histo[i].err_ymin = yMin;
      if (histo[i].errWindowEnable && histo[i].displayON != 0) on = 1;
    }
    if (aFrame.drawFlag && on != 0) repaintHisto();
    return 0;
  }
  /**
   * Sets the display color of any out-of-bounds 'error' data
   * (defined by the 'error window').
   *
   * @param color is the display color to use for out-of-bounds
   * error data.
   */
  public void setErrorColor(java.awt.Color color)
  {
    if (m_errorColor == color) return;
    Color oldValue = m_errorColor;
    m_errorColor = color;
    if (histo[lastDisp] != null) histo[lastDisp].errorColor = m_errorColor;
    firePropertyChange("errorColor", oldValue, m_errorColor);
  }
  /**
   * @return the current setting for the 'error' color
   */
  public java.awt.Color getErrorColor()
  {
    return m_errorColor;
  }
  /**
   * Sets the color of any applied display 'tags', which are
   * usually applied to histogram style displays.
   *
   * @param color is the desired tag color.
   */
  public void setTagColor(java.awt.Color color)
  {
    if (m_tagColor == color) return;
    Color oldValue = m_tagColor;
    m_tagColor = color;
    for (int i = 0; i < n_display; i++)
    {
      if (histo[i] == null) break;
      histo[i].tagColor = m_tagColor;
    }
    firePropertyChange("tagColor", oldValue, m_tagColor);
  }
  /**
   * @return the current setting for the 'tag' color
   */
  public java.awt.Color getTagColor()
  {
    return m_tagColor;
  }
  public void setTagging(boolean tagging)
  {
    if (this.m_tagging == tagging) return;
    this.m_tagging = tagging;
    if (aFrame.drawFlag) repaintHisto();
    firePropertyChange("tagging", !m_tagging, m_tagging);
  }
  public boolean isTagging()
  {
    return this.m_tagging;
  }
  /**
   * Removes assigned display 'tags' from the display in question.
   * Depending on the display type, displayed data points can be
   * 'tagged', with a tag color. e.g. histogram elements can be
   * tagged with their own color).
   * This method removed these tags.
   *
   * @param hDisplay is the display handle referring to the plot to which the
   * data are to be appended.  This is the return value of the original call
   * to the draw() method.
   */
  public void clearTags(int hDisplay)
  {
    int i, j, ok, start, stop;
    if (pTagColorArray == null) return; // no hoistogram at all!
    if (hDisplay >= 0 && hDisplay < n_display) // tag one histogram only!
    {
      start = n_display;
      stop = start + 1;
    }
    else
    {
      start = 0;
      stop = n_display;
    }
    ok = 0;
    for (j = start; j < stop; j++) // clear all histogram tagging array!!
    {
      if (histo[j] == null) break;
      if (histo[j].dismode == AcopConst.mode_histogram
          || histo[j].dismode == AcopConst.mode_histogram_rastoring
          || histo[j].dismode == AcopConst.mode_dots || histo[j].dismode == AcopConst.mode_rectangle
          || histo[j].dismode == AcopConst.mode_circle)
      {
        ok = 1;
        for (i = 0; i < histo[j].npoints; i++)
          histo[j].m_tagColorArray[i] = histo[j].vorColor;
      }
    }
    if (ok != 0 && m_tagging && aFrame.drawFlag) repaintHisto();
  }
  private void repaintHisto()
  {
    this.repaint(histRoundRect.x, histRoundRect.y, histRoundRect.width, histRoundRect.height);
  }
  // read global marker flag
  public boolean marker()
  {
    return m_marker != 0 ? true : false;
  }
  // read marker flag for hDisplay
  public boolean marker(int hDisplay)
  {
    if (hDisplay < 0 || hDisplay >= n_display)
      return marker();
    else if (histo[hDisplay] == null)
      return false;
    else
      return histo[hDisplay].markerFlag != 0 ? true : false;
  }
  // set marker flag for all displays shown
  public boolean marker(boolean MarkerOn)
  {
    return marker(MarkerOn, -1);
  }
  // set marker flag for hDisplay only
  public boolean marker(boolean MarkerOn, int hDisplay)
  {
    int num, ende;
    m_marker = MarkerOn ? 2 : 0; // mark what you see!
    num = hDisplay;
    if (num < 0 || num >= n_display)
    {
      num = 0;
      ende = n_display - 1;
    }
    else
      ende = num;
    for (; num <= ende; num++)
    {
      if (histo[num] == null) continue;
      if (m_marker == 0)
        histo[num].markerFlag = 0;
      else if (histo[num].displayON != 0) histo[num].markerFlag = 1;
    }
    if (m_marker == 2) setmarkerflag(1); // copy once only
    // prepareDraw(FALSE, -1, FALSE, 0);
    if (aFrame.drawFlag)
      prepareDraw(); // dont scale keep old limit
    else
      changeScaleWidth = 0;
    return m_marker != 0 ? true : false;
  }
  protected void setmarkerflag(int flag)
  {
    for (int n = 0; n < n_display; n++)
    {
      if (histo[n] == null) break;
      if (flag == 2 || histo[n].displayON != 0) histo[n].marker_ary_cpy = flag;
    }
  }
  public boolean isTagged(int index, int hDisplay)
  {
    return isTagged(index, -1, hDisplay);
  }
  public boolean isTagged(int index, int value, int hDisplay)
  {
    boolean tagon;
    int j, found, lasfDisp, lastTagON, start, stop;
    if (pTagColorArray == null) return false; // no hoistogram at all!
    if (index < -1) return false; // out of range, -1 to set colors for all
    found = 0;
    lastTagON = -1;
    if (hDisplay >= 0 && hDisplay < n_display) // tag one histogram only!
    {
      start = hDisplay;
      stop = start + 1;
    }
    else
    {
      start = 0;
      stop = n_display;
    }
    tagon = value == 1 ? true : false; // setting
    for (j = start; j < stop; j++) // loop all histograms to mark tagging color
    {
      if (histo[j] == null) break;
      if (histo[j].dismode == AcopConst.mode_histogram
          || histo[j].dismode == AcopConst.mode_histogram_rastoring
          || histo[j].dismode == AcopConst.mode_dots || histo[j].dismode == AcopConst.mode_rectangle
          || histo[j].dismode == AcopConst.mode_circle)
      {
        if (index >= histo[j].npoints) continue; // out of range
        if (found == 0) // first found
        {
          if (value < 0 || value > 1)
            // reading
            return index == -1 ? m_tagging : (histo[j].m_tagColorArray[index].equals(histo[j].vorColor) ? false
                : true);
          found = 1;
          lasfDisp = j;
        }
        if (index != -1) // set color for this index only
          histo[j].m_tagColorArray[index] = tagon ? histo[j].tagColor : histo[j].vorColor;
        else
          // set color for all indexes
          for (int i = 0; i < histo[j].npoints; i++)
            histo[j].m_tagColorArray[i] = tagon ? histo[j].tagColor : histo[j].vorColor;
        if (histo[j].tagHist != 0) lasfDisp = j;
        if (j == lastTagHisto && histo[j].displayON != 0) lastTagON = j;
      }
    }
    if (found == 0) return false;
    // last recalled, refreshed or drawed, and still be seen on screen
    if (lastTagON >= 0) lastDisp = lastTagON;
    if (m_tagging && aFrame.drawFlag)
    {
      if (index == -1)
        repaintHisto();
      else
      {
        int xpos, left, right, low = 0, high = 0;
        // loop all histograms to mark tagging color
        for (lasfDisp = start; lasfDisp < stop; lasfDisp++)
        {
          if (histo[lasfDisp] == null) break;
          xpos = histo[lasfDisp].ix[index];
          if (histo[lasfDisp].dismode == AcopConst.mode_histogram
              || histo[lasfDisp].dismode == AcopConst.mode_histogram_rastoring)
          {
            left = xpos - histo[lasfDisp].penWidth;
            right = xpos + histo[lasfDisp].penWidth;
          }
          else
          {
            left = xpos - histo[lasfDisp].xextent;
            right = xpos + histo[lasfDisp].xextent;
          }
          if (lasfDisp == start)
          {
            low = left;
            high = right;
            continue;
          }
          if (low > left) low = left;
          if (high < right) high = right;
        }
        tagRect.x = low + aFrame.histRect.x;
        tagRect.width = high - low;
        repaintTagging();
      }
    }
    return tagon;
  }
  protected void repaintTagging()
  {
    this.repaint(tagRect.x, tagRect.y, tagRect.width, tagRect.height);
  }
  public int getTaggedColor(Color[] ColorArray, int Npoints, int StartIndex, int hDisplay)
  {
    int num, start, ndata;
    int i, j, k;
    if (hDisplay < 0 || hDisplay >= n_display || histo[hDisplay] == null
        || histo[hDisplay].m_tagColorArray == null) return -1;
    num = Npoints; // number of points, at least 1 point
    if (num < 1) num = histo[hDisplay].npoints;
    start = StartIndex; // start point: 0 to (npoints-1)
    if (start < 0 || start >= histo[hDisplay].npoints) return -8;
    if (num + start > histo[hDisplay].npoints) num = histo[hDisplay].npoints - start;
    num += start;
    if (ColorArray == null) return -2;
    ndata = ColorArray.length;
    if (ndata < num - start) num = start + ndata;
    for (k = start; k < num; k++)
    {
      i = k - start;
      j = k < histo[hDisplay].npoints ? k : (k - histo[hDisplay].npoints);
      ColorArray[i] = histo[hDisplay].m_tagColorArray[j];
    }
    return num - start; // return number of valid data
  }

  /* (non-Javadoc)
   * Sets the display foreground color, which is applied to the drawn
   * plot or text called via one of the draw() or printText() methods.
   *
   * Existing plots and text fields are not effected by calling this method.
   *
   * @see javax.swing.JComponent#setForeground(java.awt.Color)
   */
  @Override
  public void setForeground(Color c)
  {
    //    super.setForeground(c);
    if (foreColor == c) return;
    //Color oldValue = foreColor;
    Color oldValue = getForeground();
    super.setForeground(c);
    foreColor = c;
    if (randomData == true)
    {
      rescaleFlag = 2;
      prepareDraw(); // rescale and reset limit
    }
//    firePropertyChange("foreColor", oldValue, foreColor);
    firePropertyChange("foreground", oldValue, foreColor);
  }

//  public java.awt.Color getForeground()
//  {
//    return foreColor;
//  }
  @Override
  public void setBackground(Color c)
  {
    if (backColor == c) return;
    Color oldValue = getBackground();
    super.setBackground(c);
    // cursorColor = ~(TranslateColor(m_cursorMarkerColor) ^ TranslateColor(GetBackColor())) & 0xffffff;
    if (m_cursorMarkerColor != null) rev_cursorMarkerColor = new Color(~m_cursorMarkerColor.getRGB() ^ ~c.getRGB());
    backColor = c;
    firePropertyChange("background", oldValue, backColor);
  }

  // PropertyDescriptor ReceiveQueueDepth = new
  // PropertyDescriptor("ReceiveQueueDepth",beanClass,
  // "getReceiveQueueDepth","setReceiveQueueDepth");
//  protected java.awt.List listTextBox;
  JScrollPane listTextBox;
  JTextArea textArea;
  protected java.lang.String caption = "";
  protected int captionPosition;
  protected int barlineColorFilled;
  protected int frameWidth;
  // protected java.awt.Color frameForeColor;
  protected int frameLeftOffset;
  protected int frameRightOffset;
  protected int frameTopOffset;
  protected int frameBottomOffset;
  protected int displayMode;
  // protected double xMin;
  // protected double xMax;
  // protected double yMin;
  // protected double yMax;
  protected int drawWidth;
  // protected java.awt.Font frameFont;
  // protected int XTicks;
  protected double XScale;
  protected double XShift;
  protected int graphStyle;
  // protected int YTicks;
  protected double YScale;
  protected boolean wrapAround;
  protected double YShift;
  protected double YExtent;
  protected int screenDepth;
  protected int keepStyle;
  // protected boolean YGrid;
  // protected boolean XGrid;
  protected int zooming;
  // protected java.lang.String YAxisLabel = "";
  protected int keepWidth;
  protected boolean randomData;
  protected boolean randomText;
  // protected int XFineTicks;
  protected boolean invert;
  protected int drawStyle;
  // protected boolean XTickText;
  protected boolean reuseText;
  protected java.awt.Color keepColor;
  // protected boolean xBestScale;
  protected int averagingMode, averagingDepth;
  protected int FFT;
  protected double XExtent;
  // protected boolean YTickText;
  // protected int histogramPosition;
  // protected int YFineTicks;
  protected boolean reuseScreen;
  protected int textDepth;
  // protected java.lang.String XAxisLabel = "";
  // protected boolean yBestScale;
  protected boolean colorFilled;
  // protected boolean leftTimeLabel;
  protected int xlabelRotation;
  protected int printTextStickyAttribute;
  protected int printTextLocation;
  protected int printTextRotation;
  protected java.awt.Color m_tagColor;
  protected boolean m_tagging;
  protected java.awt.Color m_errorColor;
  protected java.awt.Color LeadingEdgeColor;
  protected int LeadingEdgeGapSize;
  protected int LeadingEdgeMotion;
  protected int LeadingEdgeMode;
  protected int LeadingEdgeWidth;
  protected int LeadingEdgeStyle;
  protected double LeadingEdgeXExtent;
  protected double LeadingEdgeYExtent;
  protected java.awt.Color m_markerColor;
  protected int m_markerMode;
  protected int m_markerStyle;
  protected int m_markerWidth;
  protected double m_markerXExtent;
  protected double m_markerYExtent;
  protected java.awt.Color m_cursorMarkerColor;
  protected java.awt.Color rev_cursorMarkerColor;
  protected int m_cursorMarkerMode;
  protected int m_cursorMarkerStyle;
  protected int m_cursorMarkerWidth;
  protected double m_cursorMarkerXExtent;
  protected double m_cursorMarkerYExtent;
  private AcopTransport acopTransport = null;
  /**
   * @return Returns the acopTransport.
   */
  public AcopTransport getAcopTransport()
  {
    return acopTransport;
  }
  /**
   * @param acopTransport The acopTransport to set.
   */
  public void setAcopTransport(AcopTransport acopTransport)
  {
    this.acopTransport = acopTransport;
  }

  // private symantec.itools.beans.PropertyChangeSupport changes = null;
  // private symantec.itools.beans.VetoableChangeSupport veto = null;

	/**
	 * Returns popup manager for adding popup actions.
	 *
	 * @see com.cosylab.gui.components.util.PopupManageable#getPopupManager()
	 */
	public PopupManager getPopupManager()
	{
		if (popupManager == null) {
			popupManager = new PopupManager(this, false);
			popupManager.addAction(new AbstractAction("Preferences...") {
				private static final long serialVersionUID = 1L;

					public void actionPerformed(ActionEvent e)
					{
					  getCustomizer().showDialog();
					}
				});
		}

		return popupManager;
	}

	/**
	 * Return true if the popup menu is enabled or false otherwise.
	 *
	 * @return true if popup is enabled
	 */
	public boolean isPopupEnabled() {
		return popupEnabled;
	}

	/**
	 * Enables or disables the popup menu.
	 *
	 * @param enabled true if enable or false if disableds
	 */
	public void setPopupEnabled(boolean enabled) {
		if (popupEnabled == enabled) return;
		popupEnabled = enabled;
		if (enabled) {
			addMouseListener(getPopupManager().getMouseHook());
		} else {
			removeMouseListener(getPopupManager().getMouseHook());
		}
		firePropertyChange("popupEnabled",!popupEnabled,popupEnabled);
	}
	/**
	 * Returns the Customizer instance for this component.
	 *
	 * @return the Customizer instance for this component.
	 */
	public AbstractCustomizerPanel getCustomizer()
	{
		if (customizer == null) {
			customizer = AbstractCustomizerPanel.findCustomizer(this);
			customizer.setObject(this);
		}
		else if (customizer instanceof AcopChartCustomizer)
		{ // reset properties (this works ...) 
		  customizer.setObject(this);
		}
		return customizer;
	}

	public boolean isZoomFlag() {
		return zoomFlag;
	}

	/**
	 * Sets the force width parameter. If true the set width will be forced
	 * even if the amount of data is huge.
	 *
	 * @param force true if width is force or false otherwise
	 */
	public void setForceWidth(boolean force) {
		if (forceWidth == force) return;
		this.forceWidth = force;
		firePropertyChange("forceWidth",!forceWidth,forceWidth);
	}

	/**
	 * Returns true if forced width is turned on or false otherwise.
	 *
	 * @return true if on
	 */
	public boolean isForceWidth() {
		return forceWidth;
	}
  /* first Para: FFT Parameter, fitted results, the last: residual standard deviation of least Chi Square fit */
	public double[] getCurveFitResults(int hDisplay)
  {
	  return histo[hDisplay].curveFitResults;
  }
	public int[] getAnnotations(int handle) {
		return histo[handle].getAnnotationPoints();
	}

	public int[] getStatuses(int handle) {
		return histo[handle].getStatusPoints();
	}
  public double getYDataMax(int hDisplay)
  {
    return histo[hDisplay].getYDataMax(true);
  }
  public double getYDataMin(int hDisplay)
  {
    return histo[hDisplay].getYDataMin(true);
  }
  public double getYDataMean(int hDisplay)
  {
    return histo[hDisplay].getYDataMean(true);
  }
  public double getYDataRms(int hDisplay)
  {
    return histo[hDisplay].getYDataRms(true);
  }
  public double getYDataStdDiv(int hDisplay)
  {
    return histo[hDisplay].getYDataStdDiv(true);
  }
  public double getXDataMax(int hDisplay)
  {
    return histo[hDisplay].getYDataMax(false);
  }
  public double getXDataMin(int hDisplay)
  {
    return histo[hDisplay].getYDataMin(false);
  }
  public double getXDataMean(int hDisplay)
  {
    return histo[hDisplay].getYDataMean(false);
  }
  public double getXDataRms(int hDisplay)
  {
    return histo[hDisplay].getYDataRms(false);
  }
  public double getXDataStdDiv(int hDisplay)
  {
    return histo[hDisplay].getYDataStdDiv(false);
  }
  public double getCursorXValue(int hDisplay) {
	  histo[hDisplay].getMousePosition(new Point(cursorwy,cursorwx),mouseConvert);
	  return mouseConvert[0];
  }
  private boolean cursorLockedToCurrentScreenPosition = false;
  public void lockCursorToCurrentScreenPosition(boolean lock) {
	  if (cursorLockedToCurrentScreenPosition == lock) return;
	  cursorLockedToCurrentScreenPosition = lock;
	  cursorStay = lock ? 0 : 1;
	  if (!cursorLockedToCurrentScreenPosition) {
		  lockCursorMarker(false);
	  }

  }
  /**
   * Returns the cursor position in pixels.
   *
   * @return the cursor position
   */
  public int getCursorXPosition() {
	  return cursorwy;
  }

  public void setCursorXPosition(int cursor) {
	  mMovePoint.x = cursor;
      if (aFrame.drawFlag) repaintHisto();
	  drawCursorMarker(null,new Point(cursor,cursorwx));
  }

  /**
   * Sets the flag whether a vertical line should be printed at the text location.
   *
   * @param print true if the line is printed or false otherwise
   */
  public void setPrintTextLine(boolean print) {
	  if (this.printTextLine == print) return;
	  this.printTextLine = print;
	  repaint();
	  firePropertyChange("printTextLine",!printTextLine,printTextLine);
  }

  	/**
  	 * @return true if a vertical line is printed at the text location
  	 */
    public boolean isPrintTextLine() {
    	return printTextLine;
    }

    /**
     * Sets the flag whether acop will print the line and histo indexes in each line when using text mode.
     *
     * @param append true for appending the indexes or false otherwise
     */
    public void setAppendLineIndexInTextMode(boolean append) {
    	if (appendLineIndexInTextMode == append) return;
    	this.appendLineIndexInTextMode = append;
    	firePropertyChange("appendLineIndexInTextMode",!append,append);
    }


    /**
     * @return the flag whether the histo and line indexes are printed at the beginning of each
     * 			line in the text mode
     */
	public boolean isAppendLineIndexInTextMode() {
		return appendLineIndexInTextMode;
	}

	/**
	 *
	 * @return the index of the zooming area (how many zoom ins have been performed from the original data area)
	 */
	public int getZoomIndex() {
		return zoomIndex;
	}
  /**
   * Returns 0
   * user call draw, use the return displayIndex call this routine
   * @return 0
   */
  public int persistScreen(int hDisplay, int depth, boolean memCopy, boolean endlessFlag, Color persistColor)
  { int i;
    if ( depth <= 1 ) depth = 1;
    for (i=0; i<AcopConst.MAX_PERSIST; i++)
    {
      if ( persist[i] == null ) break;
      if ( persist[i].hisIndex == hDisplay ) break;
    }
    if ( i >= AcopConst.MAX_PERSIST || persist[i] == null )
      for (i=0; i<AcopConst.MAX_PERSIST; i++)
      {
        if ( persist[i] == null ) break;
        if ( persist[i].hisIndex < 0 ) break;
      }
    if ( i >= AcopConst.MAX_PERSIST ) return -3;
    else if ( persist[i] == null ) persist[i] = new AcopPesist(this, hDisplay);
    else persist[i].hisIndex = hDisplay;

    return persist[i].initPesist(depth, memCopy, endlessFlag, persistColor);
  }
  public int persistScreen(int hDisplay, int depth, boolean memCopy, Color persistColor)
  {
    return persistScreen(hDisplay, depth, memCopy, true, persistColor);
  }
  public int persistScreen(int hDisplay, int depth, Color persistColor)
  {
    return persistScreen(hDisplay, depth, true, persistColor);
  }
  /**
   * Returns 0 if refreshPersistent succedde otherwise -1
   * can be direct called from user, or called from refreshScreen
   * Returns 0 if refreshPersistent succedde otherwise -1
   */
  public int refreshPersistScreen(Object Yarray, int hDisplay)
  { int i;
    for (i=0; i<AcopConst.MAX_PERSIST; i++)
    {
      if ( persist[i] == null ) break;
      if ( persist[i].hisIndex == hDisplay ) break;
    }
    if ( i < AcopConst.MAX_PERSIST && persist[i] != null )
    {
      persist[i].refreshPesist(Yarray);
      return 0;
    }
    return -1;
  }

  /**
   * call this routine to terminate the persistent paiting
   */
  public void persistClear(int hDisplay)
  { int i;
    for (i=0; i<AcopConst.MAX_PERSIST; i++)
    {
      if ( persist[i] == null ) break;
      if ( persist[i].hisIndex == hDisplay || hDisplay == - 1 )
      {
        persist[i].clearPesist();
        if ( hDisplay >= 0 ) break;
      }
    }
  }
  /**
   * Sets the horizontal (x) axis label
   *
   * @param text is the text to apply to the x axis.
   */
  public void setXLeftLabel(java.lang.String text)
  {
    if (XLeftLabel != null && XLeftLabel.equals(text)) return;
    String oldValue = XLeftLabel;
    XLeftLabel = text;
    this.repaint(sca[0].labelRect.x, sca[0].labelRect.y, sca[0].labelRect.width, sca[0].labelRect.height);
    firePropertyChange("xLeftLabel", oldValue, XLeftLabel);
  }
  public java.lang.String getXLeftLabel()
  {
    return XLeftLabel;
  }
  public void setXRightLabel(java.lang.String text)
  {
    if (XRightLabel != null && XRightLabel.equals(text)) return;
    String oldValue = XRightLabel;
    XRightLabel = text;
    this.repaint(sca[0].labelRect.x, sca[0].labelRect.y, sca[0].labelRect.width, sca[0].labelRect.height);
    firePropertyChange("xRightLabel", oldValue, XRightLabel);
  }
  public java.lang.String getXRightLabel()
  {
    return XRightLabel;
  }
  public void setBitDisplayMask(int bitMask)
  {
    if (this.bitDisplayMask == bitMask) return;
    int oldValue = this.bitDisplayMask;
    this.bitDisplayMask = bitMask;
    firePropertyChange("bitDisplayMask", oldValue, bitMask);
  }
  public int getBitDisplayMask()
  {
    return this.bitDisplayMask;
  }
  public void setGanttBitDisplayBase(double ganttBitBase)
  {
    if (this.ganttBitDisplayBase == ganttBitBase) return;
    double oldValue = this.ganttBitDisplayBase;
    this.ganttBitDisplayBase = ganttBitBase;
    firePropertyChange("ganttBitDisplayBase", oldValue, ganttBitBase);
  }
  public double getGanttBitDisplayBase()
  {
    return this.ganttBitDisplayBase;
  }
  public void setGanttBitDisplayHeight(double ganttBitHeight)
  {
    if (this.ganttBitDisplayHeight == ganttBitHeight) return;
    double oldValue = this.ganttBitDisplayHeight;
    this.ganttBitDisplayHeight = ganttBitHeight;
    firePropertyChange("ganttBitDisplayHeight", oldValue, ganttBitHeight);
  }
  public double getGanttBitDisplayHeight()
  {
    return this.ganttBitDisplayHeight;
  }
  public void setYLabel(java.lang.String[] ylabel, int start, int interval)
  {
    if (ylabel != null)
    {
      yLabelStart = start;
      yLabelInterval = interval;
    }
    setYLabel(ylabel);
  }
  public void setYLabel(java.lang.String[] ylabel)
  {
    firePropertyChange("yLabel", ylab, ylabel);
    if (ylabel == null) ylabON = 0;
    else
    {
      ylabON = 1;
      ylabLength = ylabel.length;
      if ( ylab == null || ylab.length != ylabLength) ylab = new java.lang.String[ylabLength];
      for (int i = 0; i < ylabel.length; i++) ylab[i] = ylabel[i];
    }
  }
  public java.lang.String[] getYLabel()
  {
    return this.ylab;
  }
  public void setYLabelDisplayOffset(int yLabelDisplayOffset)
  {
    if (this.yLabelDisplayOffset == yLabelDisplayOffset) return;
    int oldValue = this.yLabelDisplayOffset;
    this.yLabelDisplayOffset = yLabelDisplayOffset;
    firePropertyChange("yLabelDisplayOffset", oldValue, yLabelDisplayOffset);
  }
  public int getYLabelDisplayOffset()
  {
    return this.yLabelDisplayOffset;
  }
  /**
   * Adjusts axis tick labels to account for large but insignifcant
   * common offsets. Does not adjust data!
   *
   * @param mode is the desired axis display offset mode
   * Set the acop chart X-axis display offset mode.
   * one of
   * <p>0 - Off (normal X-axis display)
   * <p>1 - Use X-min (X-axis origin = 0 in display)
   * <p>2 - Use Modular (x-axis origin = modulo most significant common offset)
   */
  public void setXAxisDisplayOffsetMode(int mode)
  {
    if ( mode < 0 ) mode = 0;
    else if ( mode > 2 ) mode = 2;
    if (this.sca[0].labelOffsetMode == mode) return;
    int oldValue = this.sca[0].labelOffsetMode;
    this.sca[0].labelOffsetMode = mode;
    if ( mode == 0 ) this.sca[0].labelOffset = 0;
    rescaleFlag = 2;
    rescaleTextFlag = AcopConst.NEW_ZOOM_WINDOW;
    prepareDraw();
    firePropertyChange("xLabelOffsetMode", oldValue, mode);
  }
  /**
   * @return
   */
  public int getXAxisDislayOffsetMode()
  {
    return sca[0].labelOffsetMode;
  }
  public double getXAxisDisplayOffset()
  {
    return this.sca[0].labelOffset;
  }
  /**
   * Adjusts axis tick labels to account for large but insignifcant
   * common offsets. Does not adjust data!
   *
   * @param mode is the Y-axis display offset mode.
   * one of
   * <p>0 - Off (normal Y-axis display)
   * <p>1 - Use Y-min (Y-axis origin = 0 in display)
   * <p>2 - Use Modular (Y-axis origin = modulo most significant common offset)
   */
  public void setYAxisDisplayOffsetMode(int mode)
  {
    if ( mode < 0 ) mode = 0;
    else if ( mode > 2 ) mode = 2;
    if (this.sca[1].labelOffsetMode == mode) return;
    int oldValue = this.sca[1].labelOffsetMode;
    this.sca[1].labelOffsetMode = mode;
    if ( mode == 0 ) this.sca[1].labelOffset = 0;
    rescaleFlag = 2;
    rescaleTextFlag = AcopConst.NEW_ZOOM_WINDOW;
    prepareDraw();
    firePropertyChange("yLabelOffsetMode", oldValue, mode);
  }
  public int getYAxisDislayOffsetMode()
  {
    return sca[1].labelOffsetMode;
  }
  public double getYAxisDisplayOffset()
  {
    return this.sca[1].labelOffset;
  }
  /**
   * Adjusts axis tick labels to account for large but insignifcant
   * common offsets. Does not adjust data!
   *
   * @param mode is the secondary Y-axis display offset mode.
   * one of
   * <p>0 - Off (normal secondary Y-axis display)
   * <p>1 - Use secondary Y-min (secondary Y-axis origin = 0 in display)
   * <p>2 - Use Modular (secondary Y-axis origin = modulo most significant common offset)
   */
  public void setSecondYAxisDisplayOffsetMode(int mode)
  {
    if ( mode < 0 ) mode = 0;
    else if ( mode > 2 ) mode = 2;
    if (this.sca[2].labelOffsetMode == mode) return;
    int oldValue = this.sca[2].labelOffsetMode;
    this.sca[2].labelOffsetMode = mode;
    if ( mode == 0 ) this.sca[2].labelOffset = 0;
    rescaleFlag = 2;
    rescaleTextFlag = AcopConst.NEW_ZOOM_WINDOW;
    prepareDraw();
    firePropertyChange("secondYLabelOffsetMode", oldValue, mode);
  }
  public int getSecondYAxisDislayOffsetMode()
  {
    return sca[2].labelOffsetMode;
  }
  public double getSecondYAxisDisplayOffset()
  {
    return this.sca[2].labelOffset;
  }

  public void setTextAntiAliasing(boolean antialias) {
	  if (this.textAntiAliasing == antialias) return;
	  this.textAntiAliasing = antialias;
	  firePropertyChange("textAntiAliasing",!textAntiAliasing,textAntiAliasing);
  }
  public boolean isTextAntiAliasing() {
	  return textAntiAliasing;
  }

  public void setWeightedStatistics(boolean weightStatitics) {
    if (this.weightStatistics == weightStatitics) return;
    this.weightStatistics = weightStatitics;
    firePropertyChange("textAntiAliasing",!textAntiAliasing,textAntiAliasing);
  }
  public boolean isWeightedStatistics() {
    return weightStatistics;
  }
  /**
   * In case X best scale is used, the max tick will extend more into the right than the
   * last data point and the min tick might extend to the left.
   * If limit X tick to boundaries is true, the max tick will be set to the set max value
   * and the min tick will be set to the set min value.
   *
   * @param limit
   */
  public void setLimitXTickToBoundaries(boolean limit) {
	  if (sca[0].limitTickToBoundaries == limit) return;
	  sca[0].limitTickToBoundaries = limit;
	  firePropertyChange("limitTickToBoundaries",!limit,limit);
	  rescaleFlag = 2;
	  rescaleTextFlag = AcopConst.NEW_ZOOM_WINDOW;
	  prepareDraw(); // rescale and reset limit
  }
  public boolean isLimitTickToBoundaries() {
	  return sca[0].limitTickToBoundaries;
  }
}
